@@ -1,4 +1,35 @@
 # -*- coding: utf-8 -*-
+#-----------------------------------------------------------
+#
+# fTools
+# Copyright (C) 2008-2011  Carson Farmer
+# EMAIL: carson.farmer (at) gmail.com
+# WEB  : http://www.ftools.ca/fTools.html
+#
+# A collection of data management and analysis tools for vector data
+#
+# Geoprocessing functions adapted from 'Geoprocessing Plugin',
+# (C) 2008 by Dr. Horst Duester, Stefan Ziegler
+#-----------------------------------------------------------
+#
+# licensed under the terms of GNU GPL 2
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+#---------------------------------------------------------------------
+
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from qgis.core import *
@@ -12,6 +43,7 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
     QDialog.__init__( self )
     self.iface = iface
     self.setupUi( self )
+    self.param.setValidator(QDoubleValidator(self.param))
     self.myFunction = function
     QObject.connect( self.btnBrowse, SIGNAL( "clicked()" ), self.outFile )
     QObject.connect( self.inShapeA, SIGNAL( "currentIndexChanged(QString)" ), self.checkA )
@@ -21,8 +53,9 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
     self.manageGui()
     self.success = False
     self.cancel_close = self.buttonBox_2.button( QDialogButtonBox.Close )
+    self.buttonOk = self.buttonBox_2.button( QDialogButtonBox.Ok )
     self.progressBar.setValue (0 )
-  
+
   def checkA( self ):
     inputLayer = unicode( self.inShapeA.currentText() )
     if inputLayer != "":
@@ -31,7 +64,7 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
         self.useSelectedA.setCheckState( Qt.Checked )
       else:
         self.useSelectedA.setCheckState( Qt.Unchecked )
-  
+
   def checkB( self ):
     inputLayer = unicode( self.inShapeB.currentText() )
     if inputLayer != "":
@@ -40,7 +73,7 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
         self.useSelectedB.setCheckState( Qt.Checked )
       else:
         self.useSelectedB.setCheckState( Qt.Unchecked )
-  
+
   def update( self ):
     self.attrib.clear()
     inputLayer = unicode( self.inShapeA.currentText() )
@@ -51,48 +84,48 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
         self.attrib.addItem( unicode( changedField[i].name() ) )
       if self.myFunction == 4:
         self.attrib.addItem( "--- " + self.tr( "Dissolve all" ) + " ---" )
-  
+
   def accept( self ):
     if self.inShapeA.currentText() == "":
-      QMessageBox.warning( self, "Geoprocessing", self.tr( "Please specify an input layer" ) )
+      QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Please specify an input layer" ) )
     elif self.inShapeB.isVisible() and self.inShapeB.currentText() == "":
-      QMessageBox.warning( self, "Geoprocessing", self.tr( "Please specify a difference/intersect/union layer" ) )
+      QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Please specify a difference/intersect/union layer" ) )
     elif self.param.isEnabled() and self.param.isVisible() and self.param.text() == "":
-      QMessageBox.warning( self, "Geoprocessing", self.tr( "Please specify valid buffer value" ) )
+      QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Please specify valid buffer value" ) )
     elif self.attrib.isEnabled() and self.attrib.isVisible() and self.attrib.currentText() == "":
-      QMessageBox.warning( self, "Geoprocessing", self.tr( "Please specify dissolve field" ) )
+      QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Please specify dissolve field" ) )
     elif self.outShape.text() == "":
-      QMessageBox.warning( self, "Geoprocessing", self.tr( "Please specify output shapefile" ) )
+      QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Please specify output shapefile" ) )
     else:
       changedLayerA = ftools_utils.getVectorLayerByName( self.inShapeA.currentText() )
       changedLayerB = ftools_utils.getVectorLayerByName( self.inShapeB.currentText() )
       # check for selection in layer A
       if self.useSelectedA.isChecked() and changedLayerA.selectedFeatureCount() == 0:
-        QMessageBox.warning( self, "Geoprocessing", self.tr( "No features selected, please uncheck 'Use selected' or make a selection" ) )
+        QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "No features selected, please uncheck 'Use selected' or make a selection" ) )
       # check for selection in layer B
       elif self.inShapeB.isVisible() and self.useSelectedB.isChecked() and changedLayerB.selectedFeatureCount() == 0:
-        QMessageBox.warning( self, "Geoprocessing", self.tr( "No features selected, please uncheck 'Use selected' or make a selection" ) )
+        QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "No features selected, please uncheck 'Use selected' or make a selection" ) )
       else:
         self.outShape.clear()
         if self.attrib.isEnabled():
-          self.geoprocessing( self.inShapeA.currentText(), self.inShapeB.currentText(), 
+          self.geoprocessing( self.inShapeA.currentText(), self.inShapeB.currentText(),
           unicode( self.attrib.currentText() ), self.mergeOutput.checkState(), self.useSelectedA.checkState(),
-          self.useSelectedB.checkState() )
+          self.useSelectedB.checkState(), self.spnSegments.value() )
         else:
           if self.param.isEnabled() and self.param.isVisible():
             parameter = float( self.param.text() )
           else:
             parameter = None
-          self.geoprocessing( self.inShapeA.currentText(), self.inShapeB.currentText(), 
-          parameter, self.mergeOutput.checkState(), self.useSelectedA.checkState(), self.useSelectedB.checkState() )
-  
+          self.geoprocessing( self.inShapeA.currentText(), self.inShapeB.currentText(),
+          parameter, self.mergeOutput.checkState(), self.useSelectedA.checkState(), self.useSelectedB.checkState(), self.spnSegments.value() )
+
   def outFile( self ):
     self.outShape.clear()
     ( self.shapefileName, self.encoding ) = ftools_utils.saveDialog( self )
     if self.shapefileName is None or self.encoding is None:
       return
     self.outShape.setText( QString( self.shapefileName ) )
-  
+
   def manageGui( self ):
     if self.myFunction == 1:  # Buffer
       self.label_2.hide()
@@ -108,6 +141,8 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
       self.rdoField.setText( self.tr( "Create convex hulls based on input field" ) )
       self.label_4.hide()
       self.param.hide()
+      self.lblSegments.hide()
+      self.spnSegments.hide()
       self.setWindowTitle( self.tr( "Convex hull(s)" ) )
       self.mergeOutput.hide()
     elif self.myFunction == 4: # Dissolve
@@ -119,6 +154,8 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
       self.param.hide()
       self.rdoField.hide()
       self.mergeOutput.hide()
+      self.lblSegments.hide()
+      self.spnSegments.hide()
       self.setWindowTitle( self.tr( "Dissolve" ) )
     else:
       self.rdoBuffer.hide()
@@ -127,6 +164,8 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
       self.rdoField.hide()
       self.attrib.hide()
       self.mergeOutput.hide()
+      self.lblSegments.hide()
+      self.spnSegments.hide()
       if self.myFunction == 3: # Difference
         self.label_2.setText( self.tr( "Difference layer" ) )
         self.setWindowTitle( self.tr( "Difference" ) )
@@ -136,25 +175,26 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
       elif self.myFunction == 7: # Symetrical difference
         self.label_2.setText( self.tr( "Difference layer" ) )
         self.setWindowTitle( self.tr( "Symetrical difference" ) )
+        self.useSelectedA.hide()
+        self.useSelectedB.hide()
       elif self.myFunction == 8: # Clip
         self.label_2.setText( self.tr( "Clip layer" ) )
         self.setWindowTitle( self.tr( "Clip" ) )
       else: # Union
         self.label_2.setText( self.tr( "Union layer" ) )
         self.setWindowTitle( self.tr( "Union" ) )
+        self.useSelectedA.hide()
+        self.useSelectedB.hide()
     self.resize(381, 100)
+    self.populateLayers()
+
+  def populateLayers( self ):
     myListA = []
     myListB = []
     self.inShapeA.clear()
     self.inShapeB.clear()
-    
-    if self.myFunction == 5 or self.myFunction == 8 or self.myFunction == 3:
-      myListA = ftools_utils.getLayerNames( [ QGis.Point, QGis.Line, QGis.Polygon ] )
-      myListB = ftools_utils.getLayerNames( [ QGis.Polygon ] )
-    elif self.myFunction == 7 or self.myFunction == 6:
-      myListA = ftools_utils.getLayerNames( [ QGis.Polygon ] )
-      myListB = ftools_utils.getLayerNames( [ QGis.Polygon ] )
-    elif self.myFunction == 4:
+
+    if self.myFunction == 4:
       myListA = ftools_utils.getLayerNames( [ QGis.Polygon ] )
       myListB = []
     else:
@@ -162,7 +202,6 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
       myListB = ftools_utils.getLayerNames( [ QGis.Point, QGis.Line, QGis.Polygon ] )
     self.inShapeA.addItems( myListA )
     self.inShapeB.addItems( myListB )
-    return
 
 #1: Buffer
 #2: Convex Hull
@@ -173,57 +212,68 @@ class GeoprocessingDialog( QDialog, Ui_Dialog ):
 #7: Symetrical Difference
 #8: Clip
 
-  def geoprocessing( self,  myLayerA,  myLayerB,  myParam,  myMerge, mySelectionA, mySelectionB ):
+  def geoprocessing( self,  myLayerA,  myLayerB,  myParam,  myMerge, mySelectionA, mySelectionB, mySegments ):
     check = QFile( self.shapefileName )
     if check.exists():
       if not QgsVectorFileWriter.deleteShapeFile( self.shapefileName ):
-        QMessageBox.warning( self, "Geoprocessing", self.tr( "Unable to delete existing shapefile." ) )
+        QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Unable to delete existing shapefile." ) )
         return
-    self.testThread = geoprocessingThread( self.iface.mainWindow(), self, self.myFunction, myLayerA, 
-    myLayerB, myParam, myMerge, mySelectionA, mySelectionB, self.shapefileName, self.encoding )
+    self.buttonOk.setEnabled( False )
+    self.testThread = geoprocessingThread( self.iface.mainWindow(), self, self.myFunction, myLayerA,
+    myLayerB, myParam, myMerge, mySelectionA, mySelectionB, mySegments, self.shapefileName, self.encoding )
     QObject.connect( self.testThread, SIGNAL( "runFinished(PyQt_PyObject)" ), self.runFinishedFromThread )
     QObject.connect( self.testThread, SIGNAL( "runStatus(PyQt_PyObject)" ), self.runStatusFromThread )
     QObject.connect( self.testThread, SIGNAL( "runRange(PyQt_PyObject)" ), self.runRangeFromThread )
-    self.cancel_close.setText( "Cancel" )
+    self.cancel_close.setText( self.tr("Cancel") )
     QObject.connect( self.cancel_close, SIGNAL( "clicked()" ), self.cancelThread )
     self.testThread.start()
     return True
-  
+
   def cancelThread( self ):
     self.testThread.stop()
-  
+    self.buttonOk.setEnabled( True )
+
   def runFinishedFromThread( self, results ):
     self.testThread.stop()
-    self.cancel_close.setText( "Close" )
+    self.buttonOk.setEnabled( True )
+    self.cancel_close.setText( self.tr("Close") )
     QObject.disconnect( self.cancel_close, SIGNAL( "clicked()" ), self.cancelThread )
-    if not results[2] or not results[1] or not results [0]:
+    out_text = ""
+    if results[3] is not None:
+      QMessageBox.warning( self, self.tr( "Geoprocessing" ),
+                  self.tr( "No output created. File creation error:\n%1" )
+                  .arg( results[3] ) )
+      return
+    if (not results[2] is None and not results[2]) or not results[1] or not results [0]:
       out_text = self.tr( "\nWarnings:" )
       end_text = self.tr( "\nSome output geometries may be missing or invalid.\n\nWould you like to add the new layer anyway?" )
     else:
       out_text = "\n"
       end_text = self.tr( "\n\nWould you like to add the new layer to the TOC?" )
-    if not results[2]:
-      out_text = out_text + self.tr( "\nInput CRS error: Different input coordinate reference systems detected, results may not be as expected.")
+    if not results[2] is None:
+      if not results[2]:
+        out_text = out_text + self.tr( "\nInput CRS error: Different input coordinate reference systems detected, results may not be as expected.")
+    else:
+      out_text = out_text + self.tr( "\nInput CRS error: One or more input layers missing coordinate reference information, results may not be as expected.")
     if not results[1]:
       out_text = out_text + self.tr( "\nFeature geometry error: One or more output features ignored due to invalid geometry.")
     if not results[0]:
       out_text = out_text + self.tr( "\nGEOS geoprocessing error: One or more input features have invalid geometry.")
-    else:
-      out_text = ""
-    addToTOC = QMessageBox.question( self, "Geoprocessing", self.tr( "Created output shapefile:" ) + "\n"
-    + unicode( self.shapefileName ) + out_text + end_text, QMessageBox.Yes, QMessageBox.No, QMessageBox.NoButton )
+    addToTOC = QMessageBox.question( self, self.tr("Geoprocessing"), self.tr( "Created output shapefile:\n%1\n%2%3" ).arg( unicode( self.shapefileName ) ).arg( out_text ).arg( end_text ), QMessageBox.Yes, QMessageBox.No, QMessageBox.NoButton )
     if addToTOC == QMessageBox.Yes:
-      ftools_utils.addShapeToCanvas( unicode( self.shapefileName ) )
-    
+      if not ftools_utils.addShapeToCanvas( unicode( self.shapefileName ) ):
+          QMessageBox.warning( self, self.tr("Geoprocessing"), self.tr( "Error loading output shapefile:\n%1" ).arg( unicode( self.shapefileName ) ))
+      self.populateLayers()
+
   def runStatusFromThread( self, status ):
     self.progressBar.setValue( status )
-  
+
   def runRangeFromThread( self, range_vals ):
     self.progressBar.setRange( range_vals[ 0 ], range_vals[ 1 ] )
-  
+
 class geoprocessingThread( QThread ):
-  def __init__( self, parentThread, parentObject, function, myLayerA, myLayerB, 
-  myParam, myMerge, mySelectionA, mySelectionB, myName, myEncoding ):
+  def __init__( self, parentThread, parentObject, function, myLayerA, myLayerB,
+  myParam, myMerge, mySelectionA, mySelectionB, mySegments, myName, myEncoding ):
     QThread.__init__( self, parentThread )
     self.parent = parentObject
     self.running = False
@@ -234,39 +284,41 @@ class geoprocessingThread( QThread ):
     self.myMerge = myMerge
     self.mySelectionA = mySelectionA
     self.mySelectionB = mySelectionB
+    self.mySegments = int( mySegments )
     self.myName = myName
     self.myEncoding = myEncoding
-  
+
   def run( self ):
     self.running = True
     self.vlayerA = ftools_utils.getVectorLayerByName( self.myLayerA )
+    error = None
     if self.myFunction == 1 or self.myFunction == 2 or self.myFunction == 4:
       ( self.myParam, useField ) = self.checkParameter( self.vlayerA, self.myParam )
       if not self.myParam is None:
         if self.myFunction == 1:
-          geos, feature, match = self.buffering( useField )
+          geos, feature, match, error = self.buffering( useField )
         elif self.myFunction == 2:
-          geos, feature, match = self.convex_hull( useField )
+          geos, feature, match, error = self.convex_hull( useField )
         elif self.myFunction == 4:
-          geos, feature, match = self.dissolve( useField )
+          geos, feature, match, error = self.dissolve( useField )
     else:
       self.vlayerB = ftools_utils.getVectorLayerByName( self.myLayerB )
       if self.myFunction == 3:
-        geos, feature, match = self.difference()
+        geos, feature, match, error = self.difference()
       elif self.myFunction == 5:
-        geos, feature, match = self.intersect()
+        geos, feature, match, error = self.intersect()
       elif self.myFunction == 6:
-        geos, feature, match = self.union()
+        geos, feature, match, error = self.union()
       elif self.myFunction == 7:
-        geos, feature, match = self.symetrical_difference()
+        geos, feature, match, error = self.symetrical_difference()
       elif self.myFunction == 8:
-        geos, feature, match = self.clip()
-    self.emit( SIGNAL( "runFinished(PyQt_PyObject)" ), (geos, feature, match) )
+        geos, feature, match, error = self.clip()
+    self.emit( SIGNAL( "runFinished(PyQt_PyObject)" ), (geos, feature, match, error) )
     self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
-  
+
   def stop(self):
     self.running = False
-  
+
   def buffering( self, useField ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
@@ -274,8 +326,11 @@ class geoprocessingThread( QThread ):
     allAttrs = vproviderA.attributeIndexes()
     vproviderA.select( allAttrs )
     fields = vproviderA.fields()
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, QGis.WKBPolygon, vproviderA.crs() )
+    # check if writer was created properly, if not, return with error
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, True, writer.errorMessage()
     outFeat = QgsFeature()
     inFeat = QgsFeature()
     inGeom = QgsGeometry()
@@ -298,7 +353,7 @@ class geoprocessingThread( QThread ):
             value = self.myParam
           inGeom = QgsGeometry( inFeat.geometry() )
           try:
-            outGeom = inGeom.buffer( float( value ), 5 )
+            outGeom = inGeom.buffer( float( value ), self.mySegments )
             if first:
               tempGeom = QgsGeometry( outGeom )
               first = False
@@ -328,7 +383,7 @@ class geoprocessingThread( QThread ):
             value = self.myParam
           inGeom = QgsGeometry( inFeat.geometry() )
           try:
-            outGeom = inGeom.buffer( float( value ), 5 )
+            outGeom = inGeom.buffer( float( value ), self.mySegments )
             try:
               outFeat.setGeometry( outGeom )
               outFeat.setAttributeMap( atMap )
@@ -357,7 +412,7 @@ class geoprocessingThread( QThread ):
             value = self.myParam
           inGeom = QgsGeometry( inFeat.geometry() )
           try:
-            outGeom = inGeom.buffer( float( value ), 5 )
+            outGeom = inGeom.buffer( float( value ), self.mySegments )
             if first:
               tempGeom = QgsGeometry( outGeom )
               first = False
@@ -388,7 +443,7 @@ class geoprocessingThread( QThread ):
             value = self.myParam
           inGeom = QgsGeometry( inFeat.geometry() )
           try:
-            outGeom = inGeom.buffer( float( value ), 5 )
+            outGeom = inGeom.buffer( float( value ), self.mySegments )
             try:
               outFeat.setGeometry( outGeom )
               outFeat.setAttributeMap( atMap )
@@ -402,21 +457,25 @@ class geoprocessingThread( QThread ):
           nElement += 1
           self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, True
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, True, None
+
   def convex_hull(self, useField ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
     vproviderA = self.vlayerA.dataProvider()
     allAttrsA = vproviderA.attributeIndexes()
+    vproviderA.select(allAttrsA)
     fields = vproviderA.fields()
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, QGis.WKBPolygon, vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, True, writer.errorMessage()
     inFeat = QgsFeature()
     outFeat = QgsFeature()
     inGeom = QgsGeometry()
     outGeom = QgsGeometry()
     nElement = 0
+
     # there is selection in input layer
     if self.mySelectionA:
       nFeat = self.vlayerA.selectedFeatureCount()
@@ -474,6 +533,7 @@ class geoprocessingThread( QThread ):
           GEOS_EXCEPT = False
     # there is no selection in input layer
     else:
+      rect = self.vlayerA.extent()
       nFeat = vproviderA.featureCount()
       if useField:
         unique = ftools_utils.getUniqueValues( vproviderA, self.myParam )
@@ -484,8 +544,8 @@ class geoprocessingThread( QThread ):
           hull = []
           first = True
           outID = 0
-          vproviderA.select( allAttrsA )
-          vproviderA.rewind()
+          vproviderA.select( allAttrsA )#, rect )
+          #vproviderA.rewind()
           while vproviderA.nextFeature( inFeat ):
             atMap = inFeat.attributeMap()
             idVar = atMap[ self.myParam ]
@@ -515,7 +575,8 @@ class geoprocessingThread( QThread ):
         self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0 )
         self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
         hull = []
-        vproviderA.rewind()
+        #vproviderA.rewind()
+        vproviderA.select(allAttrsA)
         while vproviderA.nextFeature( inFeat ):
           inGeom = QgsGeometry( inFeat.geometry() )
           points = ftools_utils.extractPoints( inGeom )
@@ -530,16 +591,18 @@ class geoprocessingThread( QThread ):
         except:
           GEOS_EXCEPT = False
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, True
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, True, None
+
   def dissolve( self, useField ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
     vproviderA = self.vlayerA.dataProvider()
     allAttrsA = vproviderA.attributeIndexes()
     fields = vproviderA.fields()
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, True, writer.errorMessage()
     inFeat = QgsFeature()
     outFeat = QgsFeature()
     vproviderA.rewind()
@@ -667,8 +730,8 @@ class geoprocessingThread( QThread ):
             outFeat.setAttributeMap( attrs )
             writer.addFeature( outFeat )
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, True
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, True, None
+
   def difference( self ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
@@ -679,10 +742,17 @@ class geoprocessingThread( QThread ):
     allAttrsB = vproviderB.attributeIndexes()
     vproviderB.select( allAttrsB )
     fields = vproviderA.fields()
-    if vproviderA.crs() == vproviderB.crs(): crs_match = True
-    else: crs_match = False
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    # check for crs compatibility
+    crsA = vproviderA.crs()
+    crsB = vproviderB.crs()
+    if not crsA.isValid() or not crsB.isValid():
+        crs_match = None
+    else:
+        crs_match = crsA == crsB
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
@@ -819,8 +889,8 @@ class geoprocessingThread( QThread ):
               FEATURE_EXCEPT = False
               continue
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
+
   def intersect( self ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
@@ -830,11 +900,22 @@ class geoprocessingThread( QThread ):
     vproviderB = self.vlayerB.dataProvider()
     allAttrsB = vproviderB.attributeIndexes()
     vproviderB.select( allAttrsB )
-    if vproviderA.crs() == vproviderB.crs(): crs_match = True
-    else: crs_match = False
+    # check for crs compatibility
+    crsA = vproviderA.crs()
+    crsB = vproviderB.crs()
+    if not crsA.isValid() or not crsB.isValid():
+        crs_match = None
+    else:
+        crs_match = crsA == crsB
     fields = ftools_utils.combineVectorFields( self.vlayerA, self.vlayerB )
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    longNames = ftools_utils.checkFieldNameLength( fields )
+    if not longNames.isEmpty():
+      message = QString( 'Following field names are longer than 10 characters:\n%1' ).arg( longNames.join( '\n' ) )
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, message
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
@@ -951,7 +1032,7 @@ class geoprocessingThread( QThread ):
           geom = QgsGeometry( inFeatA.geometry() )
           atMapA = inFeatA.attributeMap()
           intersects = index.intersects( geom.boundingBox() )
-          for id in intersects: 
+          for id in intersects:
             vproviderB.featureAtId( int( id ), inFeatB , True, allAttrsB )
             tmpGeom = QgsGeometry( inFeatB.geometry() )
             try:
@@ -973,8 +1054,8 @@ class geoprocessingThread( QThread ):
               GEOS_EXCEPT = False
               break
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
+
   def union( self ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
@@ -984,120 +1065,172 @@ class geoprocessingThread( QThread ):
     vproviderB = self.vlayerB.dataProvider()
     allAttrsB = vproviderB.attributeIndexes()
     vproviderB.select( allAttrsB )
-    if vproviderA.crs() == vproviderB.crs(): crs_match = True
-    else: crs_match = False
+
+    # check for crs compatibility
+    crsA = vproviderA.crs()
+    crsB = vproviderB.crs()
+    if not crsA.isValid() or not crsB.isValid():
+        crs_match = None
+    else:
+        crs_match = crsA == crsB
+
     fields = ftools_utils.combineVectorFields( self.vlayerA, self.vlayerB )
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
-    fields, vproviderA.geometryType(), vproviderA.crs() )
+    longNames = ftools_utils.checkFieldNameLength( fields )
+    if not longNames.isEmpty():
+      message = QString( 'Following field names are longer than 10 characters:\n%1' ).arg( longNames.join( '\n' ) )
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, message
+
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding, fields,
+                                  vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
+
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
     indexA = ftools_utils.createIndex( vproviderB )
     indexB = ftools_utils.createIndex( vproviderA )
+
     nFeat = vproviderA.featureCount() * vproviderB.featureCount()
-    nElement = 0
     self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), 0)
     self.emit( SIGNAL( "runRange(PyQt_PyObject)" ), ( 0, nFeat ) )
+
     vproviderA.rewind()
+    count = 0
+    nElement = 0
+
     while vproviderA.nextFeature( inFeatA ):
       self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ), nElement )
       nElement += 1
       found = False
       geom = QgsGeometry( inFeatA.geometry() )
-      diffGeom = QgsGeometry( inFeatA.geometry() )
+      diff_geom = QgsGeometry( geom )
       atMapA = inFeatA.attributeMap()
       intersects = indexA.intersects( geom.boundingBox() )
-      if len( intersects ) <= 0:
+      if len( intersects ) < 1:
         try:
           outFeat.setGeometry( geom )
           outFeat.setAttributeMap( atMapA )
           writer.addFeature( outFeat )
         except:
+          # this really shouldn't happen, as we
+          # haven't edited the input geom at all
           FEATURE_EXCEPT = False
-          continue
       else:
         for id in intersects:
+          count += 1
           vproviderB.featureAtId( int( id ), inFeatB , True, allAttrsB )
           atMapB = inFeatB.attributeMap()
           tmpGeom = QgsGeometry( inFeatB.geometry() )
           try:
             if geom.intersects( tmpGeom ):
               found = True
-              diff_geom = QgsGeometry( geom.difference( tmpGeom ) )
-              int_geom = QgsGeometry( geom.intersection( tmpGeom ) )
-              if int_geom.wkbType() == 7:
-                int_com = geom.combine( tmpGeom )
-                int_sym = geom.symDifference( tmpGeom )
-                int_geom = QgsGeometry( int_com.difference( int_sym ) )
+              int_geom = geom.intersection( tmpGeom )
+
+              if int_geom is None:
+                # There was a problem creating the intersection
+                GEOS_EXCEPT = False
+                int_geom = QgsGeometry()
+              else:
+                int_geom = QgsGeometry(int_geom)
+
+              if diff_geom.intersects( tmpGeom ):
+                diff_geom = diff_geom.difference( tmpGeom )
+                if diff_geom is None:
+                  # It's possible there was an error here?
+                  diff_geom = QgsGeometry()
+                else:
+                  diff_geom = QgsGeometry(diff_geom)
+
+              if int_geom.wkbType() == 0:
+                # intersection produced different geomety types
+                temp_list = int_geom.asGeometryCollection()
+                for i in temp_list:
+                  if i.type() == geom.type():
+                      int_geom = QgsGeometry( i )
               try:
                 outFeat.setGeometry( int_geom )
                 outFeat.setAttributeMap( ftools_utils.combineVectorAttributes( atMapA, atMapB ) )
                 writer.addFeature( outFeat )
-              except:
+              except Exception, err:
                 FEATURE_EXCEPT = False
-                continue
             else:
+              # this only happends if the bounding box
+              # intersects, but the geometry doesn't
               try:
                 outFeat.setGeometry( geom )
                 outFeat.setAttributeMap( atMapA )
                 writer.addFeature( outFeat )
               except:
+                # also shoudn't ever happen
                 FEATURE_EXCEPT = False
-                continue
-          except:
+          except Exception, err:
             GEOS_EXCEPT = False
             found = False
-            continue
+
         if found:
           try:
+            if diff_geom.wkbType() == 0:
+              temp_list = diff_geom.asGeometryCollection()
+              for i in temp_list:
+                if i.type() == geom.type():
+                    diff_geom = QgsGeometry( i )
             outFeat.setGeometry( diff_geom )
             outFeat.setAttributeMap( atMapA )
             writer.addFeature( outFeat )
-          except:
+          except Exception, err:
             FEATURE_EXCEPT = False
-            continue
+
     length = len( vproviderA.fields().values() )
     vproviderB.rewind()
+
     while vproviderB.nextFeature( inFeatA ):
-      add = True
+      add = False
       geom = QgsGeometry( inFeatA.geometry() )
       diff_geom = QgsGeometry( geom )
       atMap = inFeatA.attributeMap().values()
       atMap = dict( zip( range( length, length + len( atMap ) ), atMap ) )
       intersects = indexB.intersects( geom.boundingBox() )
-      if len(intersects) <= 0:
+
+      if len(intersects) < 1:
         try:
-          outFeat.setGeometry( diff_geom )
+          outFeat.setGeometry( geom )
           outFeat.setAttributeMap( atMap )
           writer.addFeature( outFeat )
-        except:
+        except Exception, err:
           FEATURE_EXCEPT = False
-          continue
       else:
         for id in intersects:
           vproviderA.featureAtId( int( id ), inFeatB , True, allAttrsA )
-          atMapB = inFeatB.attributeMap()
           tmpGeom = QgsGeometry( inFeatB.geometry() )
+
           try:
             if diff_geom.intersects( tmpGeom ):
+              add = True
               diff_geom = QgsGeometry( diff_geom.difference( tmpGeom ) )
-          except:
+            else:
+              # this only happends if the bounding box
+              # intersects, but the geometry doesn't
+              outFeat.setGeometry( diff_geom )
+              outFeat.setAttributeMap( atMap )
+              writer.addFeature( outFeat )
+          except Exception, err:
             add = False
             GEOS_EXCEPT = False
-            break
+
         if add:
           try:
             outFeat.setGeometry( diff_geom )
             outFeat.setAttributeMap( atMap )
             writer.addFeature( outFeat )
-          except:
+          except Exception, err:
             FEATURE_EXCEPT = False
-            continue
+
       self.emit( SIGNAL( "runStatus(PyQt_PyObject)" ),  nElement )
       nElement += 1
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
+
   def symetrical_difference( self ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
@@ -1107,11 +1240,22 @@ class geoprocessingThread( QThread ):
     vproviderB = self.vlayerB.dataProvider()
     allAttrsB = vproviderB.attributeIndexes()
     vproviderB.select( allAttrsB )
-    if vproviderA.crs() == vproviderB.crs(): crs_match = True
-    else: crs_match = False
+    # check for crs compatibility
+    crsA = vproviderA.crs()
+    crsB = vproviderB.crs()
+    if not crsA.isValid() or not crsB.isValid():
+        crs_match = None
+    else:
+        crs_match = crsA == crsB
     fields = ftools_utils.combineVectorFields( self.vlayerA, self.vlayerB )
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    longNames = ftools_utils.checkFieldNameLength( fields )
+    if not longNames.isEmpty():
+      message = QString( 'Following field names are longer than 10 characters:\n%1' ).arg( longNames.join( '\n' ) )
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, message
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
@@ -1178,8 +1322,8 @@ class geoprocessingThread( QThread ):
           FEATURE_EXCEPT = False
           continue
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
-  
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
+
   def clip( self ):
     GEOS_EXCEPT = True
     FEATURE_EXCEPT = True
@@ -1189,11 +1333,18 @@ class geoprocessingThread( QThread ):
     vproviderB = self.vlayerB.dataProvider()
     allAttrsB = vproviderB.attributeIndexes()
     vproviderB.select( allAttrsB )
-    if vproviderA.crs() == vproviderB.crs(): crs_match = True
-    else: crs_match = False
+    # check for crs compatibility
+    crsA = vproviderA.crs()
+    crsB = vproviderB.crs()
+    if not crsA.isValid() or not crsB.isValid():
+        crs_match = None
+    else:
+        crs_match = crsA == crsB
     fields = vproviderA.fields()
-    writer = QgsVectorFileWriter( self.myName, self.myEncoding, 
+    writer = QgsVectorFileWriter( self.myName, self.myEncoding,
     fields, vproviderA.geometryType(), vproviderA.crs() )
+    if writer.hasError():
+      return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, writer.errorMessage()
     inFeatA = QgsFeature()
     inFeatB = QgsFeature()
     outFeat = QgsFeature()
@@ -1347,7 +1498,7 @@ class geoprocessingThread( QThread ):
                 continue
             except:
               GEOS_EXCEPT = False
-              continue          
+              continue
       # we have no selection in overlay layer
       else:
         while vproviderA.nextFeature( inFeatA ):
@@ -1392,9 +1543,9 @@ class geoprocessingThread( QThread ):
                   continue
               except:
                 GEOS_EXCEPT = False
-                continue              
+                continue
     del writer
-    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match
+    return GEOS_EXCEPT, FEATURE_EXCEPT, crs_match, None
 
   def checkParameter( self, layer, param ):
     if self.myFunction == 1:
