@@ -34,6 +34,7 @@
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
 #include "qgsfieldcalculator.h"
+#include "qgsfeatureaction.h"
 
 class QgsAttributeTableDock : public QDockWidget
 {
@@ -71,18 +72,17 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   mColumnBox = columnBox;
   columnBoxInit();
 
-  QSettings mySettings;
-  bool myDockFlag = mySettings.value( "/qgis/dockAttributeTable", false ).toBool();
+  bool myDockFlag = settings.value( "/qgis/dockAttributeTable", false ).toBool();
   if ( myDockFlag )
   {
-    mDock = new QgsAttributeTableDock( tr( "Attribute table - %1" ).arg( mLayer->name() ), QgisApp::instance() );
+    mDock = new QgsAttributeTableDock( tr( "Attribute table - %1 (%n Feature(s))", "feature count", mModel->rowCount() ).arg( mLayer->name() ), QgisApp::instance() );
     mDock->setAllowedAreas( Qt::BottomDockWidgetArea | Qt::TopDockWidgetArea );
     mDock->setWidget( this );
     connect( this, SIGNAL( destroyed() ), mDock, SLOT( close() ) );
     QgisApp::instance()->addDockWidget( Qt::BottomDockWidgetArea, mDock );
   }
 
-  setWindowTitle( tr( "Attribute table - %1" ).arg( mLayer->name() ) );
+  updateTitle();
 
   mRemoveSelectionButton->setIcon( getThemeIcon( "/mActionUnselectAttributes.png" ) );
   mSelectedToTopButton->setIcon( getThemeIcon( "/mActionSelectedToTop.png" ) );
@@ -100,13 +100,16 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   bool canDeleteFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures;
   bool canAddAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddAttributes;
   bool canDeleteAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes;
+  bool canAddFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures;
   mToggleEditingButton->setCheckable( true );
   mToggleEditingButton->setChecked( mLayer->isEditable() );
-  mToggleEditingButton->setEnabled( canChangeAttributes );
+  mToggleEditingButton->setEnabled( canChangeAttributes && !mLayer->isReadOnly() );
   mOpenFieldCalculator->setEnabled( canChangeAttributes && mLayer->isEditable() );
   mDeleteSelectedButton->setEnabled( canDeleteFeatures && mLayer->isEditable() );
   mAddAttribute->setEnabled( canAddAttributes && mLayer->isEditable() );
   mRemoveAttribute->setEnabled( canDeleteAttributes && mLayer->isEditable() );
+  mAddFeature->setEnabled( canAddFeatures && mLayer->isEditable() && mLayer->geometryType() == QGis::NoGeometry );
+  mAddFeature->setHidden( !canAddFeatures || mLayer->geometryType() != QGis::NoGeometry );
 
   // info from table to application
   connect( this, SIGNAL( editingToggled( QgsMapLayer * ) ), QgisApp::instance(), SLOT( toggleEditing( QgsMapLayer * ) ) );
@@ -115,6 +118,7 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   connect( mLayer, SIGNAL( editingStopped() ), this, SLOT( editingToggled() ) );
 
   connect( searchButton, SIGNAL( clicked() ), this, SLOT( search() ) );
+  connect( mAddFeature, SIGNAL( clicked() ), this, SLOT( addFeature() ) );
 
   connect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
   connect( mLayer, SIGNAL( layerDeleted() ), this, SLOT( close() ) );
@@ -122,6 +126,11 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *theLayer, QWid
   connect( mView->verticalHeader(), SIGNAL( sectionPressed( int ) ), this, SLOT( updateRowPressed( int ) ) );
   connect( mModel, SIGNAL( modelChanged() ), this, SLOT( updateSelection() ) );
 
+  if ( settings.value( "/qgis/attributeTableBehaviour", 0 ).toInt() == 2 )
+  {
+    connect( QgisApp::instance()->mapCanvas(), SIGNAL( extentsChanged() ), mModel, SLOT( layerModified() ) );
+  }
+
   mLastClickedHeaderIndex = 0;
   mSelectionModel = new QItemSelectionModel( mFilterModel );
   updateSelectionFromLayer();
@@ -135,6 +144,17 @@ QgsAttributeTableDialog::~QgsAttributeTableDialog()
   delete mSelectionModel;
 }
 
+void QgsAttributeTableDialog::updateTitle()
+{
+  setWindowTitle( tr( "Attribute table - %1 :: %n / %2 feature(s) selected",
+                      "feature count",
+                      mView->selectionModel()->selectedRows().size()
+                    )
+                  .arg( mLayer->name() )
+                  .arg( mModel->rowCount() )
+                );
+}
+
 void QgsAttributeTableDialog::closeEvent( QCloseEvent* event )
 {
   QDialog::closeEvent( event );
@@ -221,7 +241,19 @@ void QgsAttributeTableDialog::on_mCopySelectedRowsButton_clicked()
 
 void QgsAttributeTableDialog::on_mZoomMapToSelectedRowsButton_clicked()
 {
+  QSettings settings;
+  bool canvasFeatures = settings.value( "/qgis/attributeTableBehaviour", 0 ).toInt() == 2;
+  if ( canvasFeatures )
+  {
+    disconnect( QgisApp::instance()->mapCanvas(), SIGNAL( extentsChanged() ), mModel, SLOT( layerModified() ) );
+  }
+
   QgisApp::instance()->mapCanvas()->zoomToSelected( mLayer );
+
+  if ( canvasFeatures )
+  {
+    connect( QgisApp::instance()->mapCanvas(), SIGNAL( extentsChanged() ), mModel, SLOT( layerModified() ) );
+  }
 }
 
 void QgsAttributeTableDialog::on_mInvertSelectionButton_clicked()
@@ -250,16 +282,19 @@ void QgsAttributeTableDialog::on_cbxShowSelectedOnly_toggled( bool theFlag )
 
 void QgsAttributeTableDialog::columnBoxInit()
 {
-  QgsFieldMap fieldMap = mLayer->dataProvider()->fields();
+  QgsFieldMap fieldMap = mLayer->pendingFields();
   QgsFieldMap::Iterator it = fieldMap.begin();
 
   for ( ; it != fieldMap.end(); ++it )
-    mColumnBox->addItem( it.value().name() );
+    if ( mLayer->editType( it.key() ) != QgsVectorLayer::Hidden )
+      mColumnBox->addItem( it.value().name() );
+
+  mColumnBox->setCurrentIndex( mColumnBox->findText( mLayer->displayField() ) );
 }
 
 int QgsAttributeTableDialog::columnBoxColumnId()
 {
-  QgsFieldMap fieldMap = mLayer->dataProvider()->fields();
+  QgsFieldMap fieldMap = mLayer->pendingFields();
   QgsFieldMap::Iterator it = fieldMap.begin();
 
   for ( ; it != fieldMap.end(); ++it )
@@ -285,14 +320,16 @@ void QgsAttributeTableDialog::updateSelection()
 
   mSelectionModel->select( selection, QItemSelectionModel::ClearAndSelect );// | QItemSelectionModel::Columns);
   mView->setSelectionModel( mSelectionModel );
+  updateTitle();
 
-  /*for (int i = 0; i < mModel->rowCount(); ++i)
+#if 0
+  for ( int i = 0; i < mModel->rowCount(); ++i )
   {
-  int id = mModel->rowToId(i);
-    QgsDebugMsg(id);
+    int id = mModel->rowToId( i );
+    QgsDebugMsg( id );
   }
-  QgsDebugMsg("--------------");
-  */
+  QgsDebugMsg( "--------------" );
+#endif
 }
 
 void QgsAttributeTableDialog::updateRowPressed( int index )
@@ -332,6 +369,8 @@ void QgsAttributeTableDialog::updateRowSelection( int index )
     }
 
     updateRowSelection( first, last, 3 );
+    updateTitle();
+
     mView->setSelectionMode( QAbstractItemView::NoSelection );
     return;
   }
@@ -382,6 +421,7 @@ void QgsAttributeTableDialog::updateRowSelection( int index )
     }
   }
   mView->setSelectionMode( QAbstractItemView::NoSelection );
+  updateTitle();
 }
 
 void QgsAttributeTableDialog::updateRowSelection( int first, int last, int clickType )
@@ -530,15 +570,19 @@ void QgsAttributeTableDialog::doSearch( QString searchString )
     return;
   }
 
+  // TODO: fetch only necessary columns
+  // QStringList columns = searchTree->referencedColumns();
+  bool fetchGeom = searchTree->needsGeometry();
+
   QApplication::setOverrideCursor( Qt::WaitCursor );
   mSelectedFeatures.clear();
 
   if ( cbxSearchSelectedOnly->isChecked() )
   {
     QgsFeatureList selectedFeatures = mLayer->selectedFeatures();
-    for ( QgsFeatureList::ConstIterator it = selectedFeatures.begin(); it != selectedFeatures.end(); ++it )
+    for ( QgsFeatureList::Iterator it = selectedFeatures.begin(); it != selectedFeatures.end(); ++it )
     {
-      if ( searchTree->checkAgainst( mLayer->pendingFields(), it->attributeMap() ) )
+      if ( searchTree->checkAgainst( mLayer->pendingFields(), *it ) )
         mSelectedFeatures << it->id();
 
       // check if there were errors during evaluating
@@ -548,12 +592,12 @@ void QgsAttributeTableDialog::doSearch( QString searchString )
   }
   else
   {
-    mLayer->select( mLayer->pendingAllAttributesList(), QgsRectangle(), false );
+    mLayer->select( mLayer->pendingAllAttributesList(), QgsRectangle(), fetchGeom );
     QgsFeature f;
 
     while ( mLayer->nextFeature( f ) )
     {
-      if ( searchTree->checkAgainst( mLayer->pendingFields(), f.attributeMap() ) )
+      if ( searchTree->checkAgainst( mLayer->pendingFields(), f ) )
         mSelectedFeatures << f.id();
 
       // check if there were errors during evaluating
@@ -578,31 +622,49 @@ void QgsAttributeTableDialog::doSearch( QString searchString )
   connect( mLayer, SIGNAL( selectionChanged() ), this, SLOT( updateSelectionFromLayer() ) );
 
   QString str;
+  QWidget *w = mDock ? qobject_cast<QWidget*>( mDock ) : qobject_cast<QWidget*>( this );
   if ( mSelectedFeatures.size() )
-    str.sprintf( tr( "Found %d matching features.", "", mSelectedFeatures.size() ).toUtf8(), mSelectedFeatures.size() );
+  {
+    w->setWindowTitle( tr( "Attribute table - %1 (%n matching features)", "matching features", mSelectedFeatures.size() ).arg( mLayer->name() ) );
+  }
   else
-    str = tr( "No matching features found." );
-
-  QgisApp::instance()->statusBar()->showMessage( str );
+  {
+    w->setWindowTitle( tr( "Attribute table - %1 (No matching features)" ).arg( mLayer->name() ) );
+  }
 }
 
 void QgsAttributeTableDialog::search()
 {
 
-  QString str = mColumnBox->currentText();
-
-  const QgsFieldMap& flds = mLayer->dataProvider()->fields();
-  int fldIndex = mLayer->dataProvider()->fieldNameIndex( str );
+  QString fieldName = mColumnBox->currentText();
+  const QgsFieldMap& flds = mLayer->pendingFields();
+  int fldIndex = mLayer->fieldNameIndex( fieldName );
   QVariant::Type fldType = flds[fldIndex].type();
   bool numeric = ( fldType == QVariant::Int || fldType == QVariant::Double );
+  QString sensString = "ILIKE";
+  if ( cbxCaseSensitive->isChecked() )
+  {
+    sensString = "LIKE";
+  }
 
-  if ( numeric )
-    str += " = '";
+  QSettings settings;
+  QString nullValue = settings.value( "qgis/nullValue", "NULL" ).toString();
+
+  QString str;
+  if ( mQuery->displayText() == nullValue )
+  {
+    str = QString( "%1 IS NULL" ).arg( QgsSearchTreeNode::quotedColumnRef( fieldName ) );
+  }
   else
-    str += " ~ '";
-
-  str += mQuery->displayText();
-  str += "'";
+  {
+    str = QString( "%1 %2 '%3'" )
+          .arg( QgsSearchTreeNode::quotedColumnRef( fieldName ) )
+          .arg( numeric ? "=" : sensString )
+          .arg( numeric
+                ? mQuery->displayText().replace( "'", "''" )
+                :
+                "%" + mQuery->displayText().replace( "'", "''" ) + "%" ); // escape quotes
+  }
 
   doSearch( str );
 }
@@ -612,7 +674,7 @@ void QgsAttributeTableDialog::on_mAdvancedSearchButton_clicked()
   QgsSearchQueryBuilder dlg( mLayer, this );
   dlg.setSearchString( mQuery->displayText() );
 
-  if ( dlg.exec() )
+  if ( dlg.exec() == QDialog::Accepted )
     doSearch( dlg.searchString() );
 }
 
@@ -631,10 +693,12 @@ void QgsAttributeTableDialog::editingToggled()
   bool canDeleteFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures;
   bool canAddAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddAttributes;
   bool canDeleteAttributes = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes;
+  bool canAddFeatures = mLayer->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures;
   mOpenFieldCalculator->setEnabled( canChangeAttributes && mLayer->isEditable() );
   mDeleteSelectedButton->setEnabled( canDeleteFeatures && mLayer->isEditable() );
   mAddAttribute->setEnabled( canAddAttributes && mLayer->isEditable() );
   mRemoveAttribute->setEnabled( canDeleteAttributes && mLayer->isEditable() );
+  mAddFeature->setEnabled( canAddFeatures && mLayer->isEditable() && mLayer->geometryType() == QGis::NoGeometry );
 
   // (probably reload data if user stopped editing - possible revert)
   mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
@@ -670,7 +734,7 @@ void QgsAttributeTableDialog::on_mAddAttribute_clicked()
     return;
   }
 
-  QgsAddAttrDialog dialog( mLayer->dataProvider(), this );
+  QgsAddAttrDialog dialog( mLayer, this );
   if ( dialog.exec() == QDialog::Accepted )
   {
     mLayer->beginEditCommand( tr( "Attribute added" ) );
@@ -734,7 +798,26 @@ void QgsAttributeTableDialog::on_mOpenFieldCalculator_clicked()
   QgsFieldCalculator calc( mLayer );
   if ( calc.exec() == QDialog::Accepted )
   {
-    // update model - a field has been added or updated
+    int col = mModel->fieldCol( calc.changedAttributeId() );
+
+    if ( col >= 0 )
+    {
+      mModel->reload( mModel->index( 0, col ),
+                      mModel->index( mModel->rowCount(), col ) );
+    }
+  }
+}
+
+
+void QgsAttributeTableDialog::addFeature()
+{
+  if ( !mLayer->isEditable() )
+    return;
+
+  QgsFeature f;
+  QgsFeatureAction action( tr( "Geometryless feature added" ), f, mLayer, -1, -1, this );
+  if ( action.addFeature() )
+  {
     mModel->reload( mModel->index( 0, 0 ), mModel->index( mModel->rowCount(), mModel->columnCount() ) );
   }
 }
