@@ -87,5 +87,15 @@ void QgsAttributeTableMemoryModel::attributeValueChanged( int fid, int idx, cons
 {
   QgsDebugMsg( "entered." );
   mFeatureMap[fid].changeAttribute( idx, value );
-  reload( index( 0, 0 ), index( rowCount(), columnCount() ) );
+  QgsAttributeTableModel::attributeValueChanged( fid, idx, value );
+}
+
+bool QgsAttributeTableMemoryModel::removeRows( int row, int count, const QModelIndex &parent )
+{
+  QgsDebugMsg( "entered." );
+  for ( int i = row; i < row + count; i++ )
+  {
+    mFeatureMap.remove( mRowIdMap[ i ] );
+  }
+  return QgsAttributeTableModel::removeRows( row, count, parent );
 }
