@@ -19,114 +19,137 @@
 #include "qgsfield.h"
 #include "qgsvectorlayer.h"
 #include "qgslogger.h"
+#include "qgisapp.h"
+#include "qgsattributeaction.h"
+#include "qgsmapcanvas.h"
+#include "qgsfeatureaction.h"
 
 #include <QtGui>
 #include <QVariant>
+#include <limits>
 
-////////////////////////////
-// QgsAttributeTableModel //
-////////////////////////////
 
 QgsAttributeTableModel::QgsAttributeTableModel( QgsVectorLayer *theLayer, QObject *parent )
     : QAbstractTableModel( parent )
 {
-  mLastRowId = -1;
-  mLastRow = NULL;
+  mFeat.setFeatureId( std::numeric_limits<int>::min() );
   mLayer = theLayer;
-  mFeatureCount = mLayer->pendingFeatureCount();
   loadAttributes();
 
-
-  connect( mLayer, SIGNAL( layerModified( bool ) ), this, SLOT( layerModified( bool ) ) );
-  //connect(mLayer, SIGNAL(attributeAdded(int)), this, SLOT( attributeAdded(int)));
-  //connect(mLayer, SIGNAL(attributeDeleted(int)), this, SLOT( attributeDeleted(int)));
-  //connect(mLayer, SIGNAL(attributeValueChanged(int, int, const QVariant&)), this, SLOT( attributeValueChanged(int, int, const QVariant&)));
-  //connect(mLayer, SIGNAL(featureDeleted(int)), this, SLOT( featureDeleted(int)));
-  //connect(mLayer, SIGNAL(featureAdded(int)), this, SLOT( featureAdded(int)));
+  connect( mLayer, SIGNAL( attributeValueChanged( int, int, const QVariant& ) ), this, SLOT( attributeValueChanged( int, int, const QVariant& ) ) );
+  connect( mLayer, SIGNAL( featureAdded( int ) ), this, SLOT( featureAdded( int ) ) );
+  connect( mLayer, SIGNAL( featureDeleted( int ) ), this, SLOT( featureDeleted( int ) ) );
+  connect( mLayer, SIGNAL( attributeAdded( int ) ), this, SLOT( attributeAdded( int ) ) );
+  connect( mLayer, SIGNAL( attributeDeleted( int ) ), this, SLOT( attributeDeleted( int ) ) );
 
   loadLayer();
 }
 
-bool QgsAttributeTableModel::featureAtId( int fid )
+bool QgsAttributeTableModel::featureAtId( int fid ) const
 {
-  return mLayer->featureAtId( fid, mFeat, false, true );
+  QgsDebugMsgLevel( QString( "loading feature %1" ).arg( fid ), 3 );
+
+  if ( fid == std::numeric_limits<int>::min() )
+    return false;
+  else
+    return mLayer->featureAtId( fid, mFeat, false, true );
 }
 
 void QgsAttributeTableModel::featureDeleted( int fid )
 {
-  QgsDebugMsg( "entered." );
+  QgsDebugMsgLevel( QString( "deleted fid=%1 => row=%2" ).arg( fid ).arg( idToRow( fid ) ), 3 );
 
-#ifdef QGISDEBUG
-  int idx = mIdRowMap[fid];
-  QgsDebugMsg( idx );
-  QgsDebugMsg( fid );
-#endif
-
-  QgsDebugMsg( "id->row" );
-  QHash<int, int>::iterator it;
-  for ( it = mIdRowMap.begin(); it != mIdRowMap.end(); ++it )
-    QgsDebugMsg( QString( "%1->%2" ).arg( it.key() ).arg( *it ) );
-
-  QgsDebugMsg( "row->id" );
-
-  for ( it = mRowIdMap.begin(); it != mRowIdMap.end(); ++it )
-    QgsDebugMsg( QString( "%1->%2" ).arg( it.key() ).arg( *it ) );
+  int row = idToRow( fid );
 
+  beginRemoveRows( QModelIndex(), row, row );
+  removeRow( row );
+  endRemoveRows();
 }
 
-void QgsAttributeTableModel::featureAdded( int fid )
+bool QgsAttributeTableModel::removeRows( int row, int count, const QModelIndex &parent )
 {
-  QgsDebugMsg( "BM feature added" );
-  ++mFeatureCount;
-  mIdRowMap.insert( fid, mFeatureCount - 1 );
-  mRowIdMap.insert( mFeatureCount - 1, fid );
-  QgsDebugMsg( QString( "map sizes:%1, %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ) );
-  reload( index( 0, 0 ), index( rowCount(), columnCount() ) );
+  QgsDebugMsgLevel( QString( "remove %2 rows at %1 (rows %3, ids %4)" ).arg( row ).arg( count ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 3 );
+
+  // clean old references
+  for ( int i = row; i < row + count; i++ )
+  {
+    mIdRowMap.remove( mRowIdMap[ i ] );
+    mRowIdMap.remove( i );
+  }
+
+  // update maps
+  int n = mRowIdMap.size() + count;
+  for ( int i = row + count; i < n; i++ )
+  {
+    int id = mRowIdMap[i];
+    mIdRowMap[ id ] -= count;
+    mRowIdMap[ i-count ] = id;
+    mRowIdMap.remove( i );
+  }
+
+#ifdef QGISDEBUG
+  QHash<int, int>::iterator it;
+
+  QgsDebugMsgLevel( QString( "after removal rows %1, ids %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 4 );
+  QgsDebugMsgLevel( "id->row", 4 );
+  for ( it = mIdRowMap.begin(); it != mIdRowMap.end(); ++it )
+    QgsDebugMsgLevel( QString( "%1->%2" ).arg( it.key() ).arg( *it ), 4 );
+
+  QgsDebugMsgLevel( "row->id", 4 );
+  for ( it = mRowIdMap.begin(); it != mRowIdMap.end(); ++it )
+    QgsDebugMsgLevel( QString( "%1->%2" ).arg( it.key() ).arg( *it ), 4 );
+#endif
+
+  Q_ASSERT( mRowIdMap.size() == mIdRowMap.size() );
+
+  return true;
+}
+
+void QgsAttributeTableModel::featureAdded( int fid, bool newOperation )
+{
+  QgsDebugMsgLevel( QString( "feature %1 added (%2, rows %3, ids %4)" ).arg( fid ).arg( newOperation ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ), 3 );
+
+  int n = mRowIdMap.size();
+  if ( newOperation )
+    beginInsertRows( QModelIndex(), n, n );
+
+  mIdRowMap.insert( fid, n );
+  mRowIdMap.insert( n, fid );
+
+  if ( newOperation )
+    endInsertRows();
+
+  reload( index( rowCount() - 1, 0 ), index( rowCount() - 1, columnCount() ) );
 }
 
 void QgsAttributeTableModel::attributeAdded( int idx )
 {
-  QgsDebugMsg( "BM attribute added" );
+  QgsDebugMsg( "entered." );
+  loadAttributes();
   loadLayer();
-  QgsDebugMsg( QString( "map sizes:%1, %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ) );
-  reload( index( 0, 0 ), index( rowCount(), columnCount() ) );
   emit modelChanged();
 }
 
 void QgsAttributeTableModel::attributeDeleted( int idx )
 {
-  QgsDebugMsg( "BM attribute deleted" );
+  QgsDebugMsg( "entered." );
+  loadAttributes();
   loadLayer();
-  QgsDebugMsg( QString( "map sizes:%1, %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ) );
-  reload( index( 0, 0 ), index( rowCount(), columnCount() ) );
   emit modelChanged();
 }
 
 void QgsAttributeTableModel::layerDeleted()
 {
   QgsDebugMsg( "entered." );
-  mIdRowMap.clear();
-  mRowIdMap.clear();
-  QgsDebugMsg( QString( "map sizes:%1, %2" ).arg( mRowIdMap.size() ).arg( mIdRowMap.size() ) );
-  reload( index( 0, 0 ), index( rowCount(), columnCount() ) );
+
+  beginRemoveRows( QModelIndex(), 0, rowCount() - 1 );
+  removeRows( 0, rowCount() );
+  endRemoveRows();
 }
 
-//TODO: check whether caching in data()/setData() doesn't cache old value
 void QgsAttributeTableModel::attributeValueChanged( int fid, int idx, const QVariant &value )
 {
-  QgsDebugMsg( "entered." );
-  reload( index( 0, 0 ), index( rowCount(), columnCount() ) );
-}
-
-void QgsAttributeTableModel::layerModified( bool onlyGeometry )
-{
-  if ( onlyGeometry )
-    return;
-
-  loadAttributes();
-  loadLayer();
-  emit modelChanged();
-  emit headerDataChanged( Qt::Horizontal, 0, columnCount() - 1 );
+  setData( index( idToRow( fid ), fieldCol( idx ) ), value, Qt::EditRole );
 }
 
 void QgsAttributeTableModel::loadAttributes()
@@ -137,23 +160,8 @@ void QgsAttributeTableModel::loadAttributes()
   }
 
   bool ins = false, rm = false;
-  int pendingFieldCount = mLayer->pendingFields().size();
-
-  if ( mFieldCount < pendingFieldCount )
-  {
-    ins = true;
-    beginInsertColumns( QModelIndex(), mFieldCount, pendingFieldCount - 1 );
-  }
-  else if ( pendingFieldCount < mFieldCount )
-  {
-    rm = true;
-    beginRemoveColumns( QModelIndex(), pendingFieldCount, mFieldCount - 1 );
-  }
-
-  mFieldCount = 0;
-  mAttributes.clear();
-  mValueMaps.clear();
 
+  QgsAttributeList attributes;
   for ( QgsFieldMap::const_iterator it = mLayer->pendingFields().constBegin(); it != mLayer->pendingFields().end(); it++ )
   {
     switch ( mLayer->editType( it.key() ) )
@@ -169,10 +177,24 @@ void QgsAttributeTableModel::loadAttributes()
         break;
     }
 
-    mFieldCount++;
-    mAttributes << it.key();
+    attributes << it.key();
   }
 
+  if ( mFieldCount < attributes.size() )
+  {
+    ins = true;
+    beginInsertColumns( QModelIndex(), mFieldCount, attributes.size() - 1 );
+  }
+  else if ( attributes.size() < mFieldCount )
+  {
+    rm = true;
+    beginRemoveColumns( QModelIndex(), attributes.size(), mFieldCount - 1 );
+  }
+
+  mFieldCount = attributes.size();
+  mAttributes = attributes;
+  mValueMaps.clear();
+
   if ( ins )
   {
     endInsertColumns();
@@ -187,54 +209,41 @@ void QgsAttributeTableModel::loadLayer()
 {
   QgsDebugMsg( "entered." );
 
-  QgsFeature f;
-  bool ins = false, rm = false;
+  beginRemoveRows( QModelIndex(), 0, rowCount() - 1 );
+  removeRows( 0, rowCount() );
+  endRemoveRows();
 
-  mRowIdMap.clear();
-  mIdRowMap.clear();
+  QSettings settings;
+  int behaviour = settings.value( "/qgis/attributeTableBehaviour", 0 ).toInt();
 
-  int pendingFeatureCount = mLayer->pendingFeatureCount();
-  if ( mFeatureCount < pendingFeatureCount )
-  {
-    QgsDebugMsg( "ins" );
-    ins = true;
-    beginInsertRows( QModelIndex(), mFeatureCount, pendingFeatureCount - 1 );
-// QgsDebugMsg(QString("%1, %2").arg(mFeatureCount).arg(mLayer->pendingFeatureCount() - 1));
-  }
-  else if ( mFeatureCount > pendingFeatureCount )
-  {
-    QgsDebugMsg( "rm" );
-    rm = true;
-    beginRemoveRows( QModelIndex(), pendingFeatureCount, mFeatureCount - 1 );
-// QgsDebugMsg(QString("%1, %2").arg(mFeatureCount).arg(mLayer->pendingFeatureCount() -1));
-  }
-
-  mLayer->select( mAttributes, QgsRectangle(), false );
-
-  // preallocate data before inserting
-  mRowIdMap.reserve( pendingFeatureCount + 50 );
-  mIdRowMap.reserve( pendingFeatureCount + 50 );
-
-  for ( int i = 0; mLayer->nextFeature( f ); ++i )
-  {
-    mRowIdMap.insert( i, f.id() );
-    mIdRowMap.insert( f.id(), i );
-  }
-
-  // not needed when we have featureAdded signal
-  mFeatureCount = pendingFeatureCount;
-  mFieldCount = mAttributes.size();
-
-  if ( ins )
+  if ( behaviour == 1 )
   {
+    beginInsertRows( QModelIndex(), 0, mLayer->selectedFeatureCount() );
+    foreach( int fid, mLayer->selectedFeaturesIds() )
+    {
+      featureAdded( fid, false );
+    }
     endInsertRows();
-    QgsDebugMsg( "end ins" );
   }
-  else if ( rm )
+  else
   {
-    endRemoveRows();
-    QgsDebugMsg( "end rm" );
+    QgsRectangle rect;
+    if ( behaviour == 2 )
+    {
+      // current canvas only
+      rect = QgisApp::instance()->mapCanvas()->extent();
+    }
+
+    mLayer->select( QgsAttributeList(), rect, false );
+
+    QgsFeature f;
+    for ( int i = 0; mLayer->nextFeature( f ); ++i )
+    {
+      featureAdded( f.id() );
+    }
   }
+
+  mFieldCount = mAttributes.size();
 }
 
 void QgsAttributeTableModel::swapRows( int a, int b )
@@ -277,7 +286,7 @@ int QgsAttributeTableModel::rowToId( const int id ) const
   {
     QgsDebugMsg( QString( "rowToId: row %1 not in the map" ).arg( id ) );
     // return negative infinite (to avoid collision with newly added features)
-    return -999999;
+    return std::numeric_limits<int>::min();
   }
 
   return mRowIdMap[id];
@@ -288,9 +297,14 @@ int QgsAttributeTableModel::fieldIdx( int col ) const
   return mAttributes[ col ];
 }
 
+int QgsAttributeTableModel::fieldCol( int idx ) const
+{
+  return mAttributes.indexOf( idx );
+}
+
 int QgsAttributeTableModel::rowCount( const QModelIndex &parent ) const
 {
-  return mFeatureCount;
+  return mRowIdMap.size();
 }
 
 int QgsAttributeTableModel::columnCount( const QModelIndex &parent ) const
@@ -317,7 +331,10 @@ QVariant QgsAttributeTableModel::headerData( int section, Qt::Orientation orient
       return QVariant( attributeName );
     }
   }
-  else return QVariant();
+  else
+  {
+    return QVariant();
+  }
 }
 
 void QgsAttributeTableModel::sort( int column, Qt::SortOrder order )
@@ -363,16 +380,13 @@ void QgsAttributeTableModel::sort( int column, Qt::SortOrder order )
 }
 
 QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role ) const
-{
-  return (( QgsAttributeTableModel * ) this )->data( index, role );
-}
-
-QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role )
 {
   if ( !index.isValid() || ( role != Qt::TextAlignmentRole && role != Qt::DisplayRole && role != Qt::EditRole ) )
     return QVariant();
 
-  QVariant::Type fldType = mLayer->pendingFields()[ mAttributes[index.column()] ].type();
+  int fieldId = mAttributes[ index.column()];
+
+  QVariant::Type fldType = mLayer->pendingFields()[ fieldId ].type();
   bool fldNumeric = ( fldType == QVariant::Int || fldType == QVariant::Double );
 
   if ( role == Qt::TextAlignmentRole )
@@ -384,21 +398,17 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role )
   }
 
   // if we don't have the row in current cache, load it from layer first
-  if ( mLastRowId != rowToId( index.row() ) )
+  int rowId = rowToId( index.row() );
+  if ( mFeat.id() != rowId )
   {
-    bool res = featureAtId( rowToId( index.row() ) );
-
-    if ( !res )
+    if ( !featureAtId( rowId ) )
       return QVariant( "ERROR" );
-
-    mLastRowId = rowToId( index.row() );
-    mLastRow = ( QgsAttributeMap * ) & mFeat.attributeMap();
   }
 
-  if ( !mLastRow )
+  if ( mFeat.id() != rowId )
     return QVariant( "ERROR" );
 
-  const QVariant &val = ( *mLastRow )[ mAttributes[index.column()] ];
+  const QVariant &val = mFeat.attributeMap()[ fieldId ];
 
   if ( val.isNull() )
   {
@@ -409,13 +419,14 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role )
     }
     else
     {
-      return QVariant( "NULL" );
+      QSettings settings;
+      return settings.value( "qgis/nullValue", "NULL" );
     }
   }
 
-  if ( role == Qt::DisplayRole && mValueMaps.contains( index.column() ) )
+  if ( role == Qt::DisplayRole && mValueMaps.contains( fieldId ) )
   {
-    return mValueMaps[ index.column()]->key( val.toString(), QString( "(%1)" ).arg( val.toString() ) );
+    return mValueMaps[ fieldId ]->key( val.toString(), QString( "(%1)" ).arg( val.toString() ) );
   }
 
   return val.toString();
@@ -423,34 +434,20 @@ QVariant QgsAttributeTableModel::data( const QModelIndex &index, int role )
 
 bool QgsAttributeTableModel::setData( const QModelIndex &index, const QVariant &value, int role )
 {
-  if ( !index.isValid() || role != Qt::EditRole )
+  if ( !index.isValid() || role != Qt::EditRole || !mLayer->isEditable() )
     return false;
 
-  if ( !mLayer->isEditable() )
-    return false;
-
-  bool res = featureAtId( rowToId( index.row() ) );
-
-  if ( res )
+  int rowId = rowToId( index.row() );
+  if ( mFeat.id() == rowId || featureAtId( rowId ) )
   {
-    mLastRowId = rowToId( index.row() );
-    mLastRow = ( QgsAttributeMap * ) & mFeat.attributeMap();
-
-    disconnect( mLayer, SIGNAL( layerModified( bool ) ), this, SLOT( layerModified( bool ) ) );
-
-    mLayer->beginEditCommand( tr( "Attribute changed" ) );
-    mLayer->changeAttributeValue( rowToId( index.row() ), mAttributes[ index.column()], value, true );
-    mLayer->endEditCommand();
-
-    ( *mLastRow )[ mAttributes[index.column()] ] = value;
-
-    connect( mLayer, SIGNAL( layerModified( bool ) ), this, SLOT( layerModified( bool ) ) );
+    mFeat.changeAttribute( mAttributes[ index.column()], value );
   }
 
   if ( !mLayer->isModified() )
     return false;
 
   emit dataChanged( index, index );
+
   return true;
 }
 
@@ -461,7 +458,8 @@ Qt::ItemFlags QgsAttributeTableModel::flags( const QModelIndex &index ) const
 
   Qt::ItemFlags flags = QAbstractItemModel::flags( index );
 
-  if ( mLayer->isEditable() )
+  if ( mLayer->isEditable() &&
+       mLayer->editType( mAttributes[ index.column()] ) != QgsVectorLayer::Immutable )
     flags |= Qt::ItemIsEditable;
 
   return flags;
@@ -469,6 +467,7 @@ Qt::ItemFlags QgsAttributeTableModel::flags( const QModelIndex &index ) const
 
 void QgsAttributeTableModel::reload( const QModelIndex &index1, const QModelIndex &index2 )
 {
+  mFeat.setFeatureId( std::numeric_limits<int>::min() );
   emit dataChanged( index1, index2 );
 }
 
@@ -487,3 +486,32 @@ void QgsAttributeTableModel::incomingChangeLayout()
   emit layoutAboutToBeChanged();
 }
 
+void QgsAttributeTableModel::executeAction( int action, const QModelIndex &idx ) const
+{
+  QgsAttributeMap attributes;
+
+  for ( int i = 0; i < mAttributes.size(); i++ )
+  {
+    attributes.insert( mAttributes[i], data( index( idx.row(), i ), Qt::EditRole ) );
+  }
+
+  mLayer->actions()->doAction( action, attributes, fieldIdx( idx.column() ) );
+}
+
+void QgsAttributeTableModel::featureForm( QModelIndex &idx )
+{
+  QgsFeature f;
+  QgsAttributeMap attributes;
+
+  f.setFeatureId( rowToId( idx.row() ) );
+  for ( int i = 0; i < mAttributes.size(); i++ )
+  {
+    f.changeAttribute( mAttributes[i], data( index( idx.row(), i ), Qt::EditRole ) );
+  }
+
+  QgsFeatureAction action( tr( "Attributes changed" ), f, mLayer, -1, -1, this );
+  if ( mLayer->isEditable() )
+    action.editFeature();
+  else
+    action.viewFeatureForm();
+}
