@@ -17,15 +17,19 @@
 #define QGSATTRIBUTETABLEVIEW_H
 
 #include <QTableView>
+#include <QAction>
 
 class QgsAttributeTableModel;
 class QgsAttributeTableFilterModel;
 
 class QgsVectorLayer;
+class QMenu;
 
 
-class QgsAttributeTableView: public QTableView
+class QgsAttributeTableView : public QTableView
 {
+    Q_OBJECT
+
   public:
     QgsAttributeTableView( QWidget* parent = NULL );
     virtual ~QgsAttributeTableView();
@@ -42,9 +46,32 @@ class QgsAttributeTableView: public QTableView
      */
     void closeEvent( QCloseEvent *event );
 
+    void contextMenuEvent( QContextMenuEvent* );
+
   private:
     QgsAttributeTableModel* mModel;
     QgsAttributeTableFilterModel* mFilterModel;
+    QMenu *mActionPopup;
 };
 
+class QgsAttributeTableAction : public QAction
+{
+    Q_OBJECT
+
+  public:
+    QgsAttributeTableAction( const QString &name, QgsAttributeTableView *view, QgsAttributeTableModel *model, int action, const QModelIndex &fieldIdx ) :
+        QAction( name, view ), mModel( model ), mAction( action ), mFieldIdx( fieldIdx )
+    {}
+
+  public slots:
+    void execute();
+    void featureForm();
+
+  private:
+    QgsAttributeTableModel *mModel;
+    int mAction;
+    QModelIndex mFieldIdx;
+};
+
+
 #endif
