@@ -35,7 +35,10 @@
 #include "qgscomposerscalebarwidget.h"
 #include "qgscomposershape.h"
 #include "qgscomposershapewidget.h"
+#include "qgscomposerattributetable.h"
+#include "qgscomposertablewidget.h"
 #include "qgsexception.h"
+#include "qgslogger.h"
 #include "qgsproject.h"
 #include "qgsmapcanvas.h"
 #include "qgsmaprenderer.h"
@@ -44,38 +47,39 @@
 #include "qgscursors.h"
 
 #include <QCloseEvent>
+#include <QCheckBox>
 #include <QDesktopWidget>
 #include <QFileDialog>
 #include <QFileInfo>
+#include <QIcon>
+#include <QImageWriter>
 #include <QMatrix>
 #include <QMenuBar>
 #include <QMessageBox>
+#include <QPageSetupDialog>
 #include <QPainter>
-
-#include <QPrinter>
+#include <QPixmap>
 #include <QPrintDialog>
 #include <QSettings>
-#include <QIcon>
-#include <QPixmap>
-#if QT_VERSION < 0x040300
-#include <Q3Picture>
-#else
+#include <QSizeGrip>
 #include <QSvgGenerator>
-#endif
 #include <QToolBar>
 #include <QToolButton>
-#include <QImageWriter>
-#include <QCheckBox>
-#include <QSizeGrip>
-#include "qgslogger.h"
+#include <QUndoView>
+#include <QPaintEngine>
 
-QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(), mTitle( title )
+
+QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(), mTitle( title ), mUndoView( 0 )
 {
   setupUi( this );
   setWindowTitle( mTitle );
   setupTheme();
   QObject::connect( mButtonBox, SIGNAL( rejected() ), this, SLOT( close() ) );
 
+  QSettings settings;
+  int size = settings.value( "/IconSize", 24 ).toInt();
+  setIconSize( QSize( size, size ) );
+
   QToolButton* orderingToolButton = new QToolButton( this );
   orderingToolButton->setPopupMode( QToolButton::InstantPopup );
   orderingToolButton->setAutoRaise( true );
@@ -112,6 +116,7 @@ QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(),
   toggleActionGroup->addAction( mActionSelectMoveItem );
   toggleActionGroup->addAction( mActionAddBasicShape );
   toggleActionGroup->addAction( mActionAddArrow );
+  toggleActionGroup->addAction( mActionAddTable );
   toggleActionGroup->setExclusive( true );
 
   mActionAddNewMap->setCheckable( true );
@@ -138,6 +143,7 @@ QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(),
   fileMenu->addAction( mActionExportAsPDF );
   fileMenu->addAction( mActionExportAsSVG );
   fileMenu->addSeparator();
+  fileMenu->addAction( mActionPageSetup );
   fileMenu->addAction( mActionPrint );
   fileMenu->addSeparator();
   fileMenu->addAction( mActionQuit );
@@ -151,6 +157,9 @@ QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(),
   viewMenu->addAction( mActionRefreshView );
 
   QMenu *layoutMenu = menuBar()->addMenu( tr( "Layout" ) );
+  layoutMenu->addAction( mActionUndo );
+  layoutMenu->addAction( mActionRedo );
+  layoutMenu->addSeparator();
   layoutMenu->addAction( mActionAddNewMap );
   layoutMenu->addAction( mActionAddNewLabel );
   layoutMenu->addAction( mActionAddNewScalebar );
@@ -160,6 +169,7 @@ QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(),
   layoutMenu->addAction( mActionMoveItemContent );
   layoutMenu->addAction( mActionAddBasicShape );
   layoutMenu->addAction( mActionAddArrow );
+  layoutMenu->addAction( mActionAddTable );
   layoutMenu->addSeparator();
   layoutMenu->addAction( mActionGroupItems );
   layoutMenu->addAction( mActionUngroupItems );
@@ -193,7 +203,17 @@ QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(),
   mView = new QgsComposerView( mViewFrame );
   connectSlots();
 
+  //init undo/redo buttons
   mComposition  = new QgsComposition( mQgis->mapCanvas()->mapRenderer() );
+  mActionUndo->setEnabled( false );
+  mActionRedo->setEnabled( false );
+  if ( mComposition->undoStack() )
+  {
+    connect( mComposition->undoStack(), SIGNAL( canUndoChanged( bool ) ), mActionUndo, SLOT( setEnabled( bool ) ) );
+    connect( mComposition->undoStack(), SIGNAL( canRedoChanged( bool ) ), mActionRedo, SLOT( setEnabled( bool ) ) );
+  }
+
+
   mComposition->setParent( mView );
   mView->setComposition( mComposition );
 
@@ -211,6 +231,10 @@ QgsComposer::QgsComposer( QgisApp *qgis, const QString& title ): QMainWindow(),
 
   mCompositionNameComboBox->insertItem( 0, tr( "Map 1" ) );
 
+  //undo widget
+  mUndoView = new QUndoView( mComposition->undoStack(), this );
+  mOptionsTabWidget->addTab( mUndoView, tr( "Command history" ) );
+
   // Create size grip (needed by Mac OS X for QMainWindow if QStatusBar is not visible)
   mSizeGrip = new QSizeGrip( this );
   mSizeGrip->resize( mSizeGrip->sizeHint() );
@@ -248,6 +272,8 @@ void QgsComposer::setupTheme()
   mActionZoomIn->setIcon( QgisApp::getThemeIcon( "/mActionZoomIn.png" ) );
   mActionZoomOut->setIcon( QgisApp::getThemeIcon( "/mActionZoomOut.png" ) );
   mActionRefreshView->setIcon( QgisApp::getThemeIcon( "/mActionDraw.png" ) );
+  mActionUndo->setIcon( QgisApp::getThemeIcon( "/mActionUndo.png" ) );
+  mActionRedo->setIcon( QgisApp::getThemeIcon( "/mActionRedo.png" ) );
   mActionAddImage->setIcon( QgisApp::getThemeIcon( "/mActionAddImage.png" ) );
   mActionAddNewMap->setIcon( QgisApp::getThemeIcon( "/mActionAddMap.png" ) );
   mActionAddNewLabel->setIcon( QgisApp::getThemeIcon( "/mActionLabel.png" ) );
@@ -255,6 +281,7 @@ void QgsComposer::setupTheme()
   mActionAddNewScalebar->setIcon( QgisApp::getThemeIcon( "/mActionScaleBar.png" ) );
   mActionAddBasicShape->setIcon( QgisApp::getThemeIcon( "/mActionAddBasicShape.png" ) );
   mActionAddArrow->setIcon( QgisApp::getThemeIcon( "/mActionAddArrow.png" ) );
+  mActionAddTable->setIcon( QgisApp::getThemeIcon( "/mActionOpenTable.png" ) );
   mActionSelectMoveItem->setIcon( QgisApp::getThemeIcon( "/mActionSelectPan.png" ) );
   mActionMoveItemContent->setIcon( QgisApp::getThemeIcon( "/mActionMoveItemContent.png" ) );
   mActionGroupItems->setIcon( QgisApp::getThemeIcon( "/mActionGroupItems.png" ) );
@@ -271,9 +298,22 @@ void QgsComposer::setupTheme()
   mActionAlignBottom->setIcon( QgisApp::getThemeIcon( "/mActionAlignBottom.png" ) );
 }
 
+void QgsComposer::setIconSizes( int size )
+{
+  //Set the icon size of for all the toolbars created in the future.
+  setIconSize( QSize( size, size ) );
+
+  //Change all current icon sizes.
+  QList<QToolBar *> toolbars = findChildren<QToolBar *>();
+  foreach( QToolBar * toolbar, toolbars )
+  {
+    toolbar->setIconSize( QSize( size, size ) );
+  }
+}
+
 void QgsComposer::connectSlots()
 {
-  connect( mView, SIGNAL( selectedItemChanged( const QgsComposerItem* ) ), this, SLOT( showItemOptions( const QgsComposerItem* ) ) );
+  connect( mView, SIGNAL( selectedItemChanged( QgsComposerItem* ) ), this, SLOT( showItemOptions( QgsComposerItem* ) ) );
   connect( mView, SIGNAL( composerLabelAdded( QgsComposerLabel* ) ), this, SLOT( addComposerLabel( QgsComposerLabel* ) ) );
   connect( mView, SIGNAL( composerMapAdded( QgsComposerMap* ) ), this, SLOT( addComposerMap( QgsComposerMap* ) ) );
   connect( mView, SIGNAL( itemRemoved( QgsComposerItem* ) ), this, SLOT( deleteItem( QgsComposerItem* ) ) );
@@ -282,6 +322,7 @@ void QgsComposer::connectSlots()
   connect( mView, SIGNAL( composerPictureAdded( QgsComposerPicture* ) ), this, SLOT( addComposerPicture( QgsComposerPicture* ) ) );
   connect( mView, SIGNAL( composerShapeAdded( QgsComposerShape* ) ), this, SLOT( addComposerShape( QgsComposerShape* ) ) );
   connect( mView, SIGNAL( composerArrowAdded( QgsComposerArrow* ) ), this, SLOT( addComposerArrow( QgsComposerArrow* ) ) );
+  connect( mView, SIGNAL( composerTableAdded( QgsComposerAttributeTable* ) ), this, SLOT( addComposerTable( QgsComposerAttributeTable* ) ) );
   connect( mView, SIGNAL( actionFinished() ), this, SLOT( setSelectionTool() ) );
 }
 
@@ -357,7 +398,7 @@ void QgsComposer::showCompositionOptions( QWidget *w )
   mItemStackedWidget->addWidget( w );
 }
 
-void QgsComposer::showItemOptions( const QgsComposerItem* item )
+void QgsComposer::showItemOptions( QgsComposerItem* item )
 {
   QWidget* currentWidget = mItemStackedWidget->currentWidget();
 
@@ -368,7 +409,7 @@ void QgsComposer::showItemOptions( const QgsComposerItem* item )
     return;
   }
 
-  QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.find( const_cast<QgsComposerItem*>( item ) );
+  QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.find( item );
   if ( it == mItemWidgetMap.constEnd() )
   {
     return;
@@ -453,62 +494,88 @@ void QgsComposer::on_mActionRefreshView_triggered()
   mComposition->update();
 }
 
+// Hack to workaround Qt #5114 by disabling PatternTransform
+class QgsPaintEngineHack : public QPaintEngine
+{
+  public:
+    void fixFlags()
+    {
+      gccaps = 0;
+      gccaps |= ( QPaintEngine::PrimitiveTransform
+                  // | QPaintEngine::PatternTransform
+                  | QPaintEngine::PixmapTransform
+                  | QPaintEngine::PatternBrush
+                  // | QPaintEngine::LinearGradientFill
+                  // | QPaintEngine::RadialGradientFill
+                  // | QPaintEngine::ConicalGradientFill
+                  | QPaintEngine::AlphaBlend
+                  // | QPaintEngine::PorterDuff
+                  | QPaintEngine::PainterPaths
+                  | QPaintEngine::Antialiasing
+                  | QPaintEngine::BrushStroke
+                  | QPaintEngine::ConstantOpacity
+                  | QPaintEngine::MaskedBrush
+                  // | QPaintEngine::PerspectiveTransform
+                  | QPaintEngine::BlendModes
+                  // | QPaintEngine::ObjectBoundingModeGradients
+#if QT_VERSION >= 0x040500
+                  | QPaintEngine::RasterOpModes
+#endif
+                  | QPaintEngine::PaintOutsidePaintEvent
+                );
+    }
+};
+
 void QgsComposer::on_mActionExportAsPDF_triggered()
 {
   QSettings myQSettings;  // where we keep last used filter in persistent state
-  QString myLastUsedFile = myQSettings.value( "/UI/lastSaveAsPdfFile", "qgis.pdf" ).toString();
-  QFileInfo file( myLastUsedFile );
-  QFileDialog *myQFileDialog = new QFileDialog( this, tr( "Choose a file name to save the map as" ),
-      file.path(), tr( "PDF Format" ) + " (*.pdf *PDF)" );
-  myQFileDialog->selectFile( file.fileName() );
-  myQFileDialog->setFileMode( QFileDialog::AnyFile );
-  myQFileDialog->setAcceptMode( QFileDialog::AcceptSave );
+  QString lastUsedFile = myQSettings.value( "/UI/lastSaveAsPdfFile", "qgis.pdf" ).toString();
+  QFileInfo file( lastUsedFile );
 
-  int result = myQFileDialog->exec();
-  raise();
-  if ( result != QDialog::Accepted ) return;
+  QString outputFileName = QFileDialog::getSaveFileName(
+                             this,
+                             tr( "Choose a file name to save the map as" ),
+                             file.path(),
+                             tr( "PDF Format" ) + " (*.pdf *.PDF)" );
 
-  QString myOutputFileNameQString = myQFileDialog->selectedFiles().first();
-  if ( myOutputFileNameQString == "" ) return;
+  if ( !outputFileName.endsWith( ".pdf", Qt::CaseInsensitive ) )
+  {
+    outputFileName += ".pdf";
+  }
 
-  myQSettings.setValue( "/UI/lastSaveAsPdfFile", myOutputFileNameQString );
+  myQSettings.setValue( "/UI/lastSaveAsPdfFile", outputFileName );
 
   QPrinter printer;
 
   printer.setOutputFormat( QPrinter::PdfFormat );
-  printer.setOutputFileName( myOutputFileNameQString );
+  printer.setOutputFileName( outputFileName );
   printer.setPaperSize( QSizeF( mComposition->paperWidth(), mComposition->paperHeight() ), QPrinter::Millimeter );
 
+  QPaintEngine *engine = printer.paintEngine();
+  if ( engine )
+  {
+    QgsPaintEngineHack *hack = static_cast<QgsPaintEngineHack*>( engine );
+    hack->fixFlags();
+  }
+
   print( printer );
 }
 
 void QgsComposer::on_mActionPrint_triggered()
 {
-  QPrinter printer;
-  if ( mComposition )
-  {
-    if ( mComposition->paperWidth() >= mComposition->paperHeight() )
-    {
-      printer.setOrientation( QPrinter::Landscape );
-    }
-    else
-    {
-      printer.setOrientation( QPrinter::Portrait );
-    }
-  }
-  printer.setPaperSize( QSizeF( mComposition->paperWidth(), mComposition->paperHeight() ), QPrinter::Millimeter );
-  QPrintDialog printDialog( &printer, 0 );
+  //orientation and page size are already set to QPrinter in the page setup dialog
+  QPrintDialog printDialog( &mPrinter, 0 );
   if ( printDialog.exec() != QDialog::Accepted )
   {
     return;
   }
 
-  print( printer );
+  print( mPrinter );
 }
 
 void QgsComposer::print( QPrinter &printer )
 {
-  if ( !mComposition )
+  if ( !mComposition || !mView )
     return;
 
   if ( containsWMSLayer() )
@@ -530,35 +597,59 @@ void QgsComposer::print( QPrinter &printer )
 
   QApplication::setOverrideCursor( Qt::BusyCursor );
 
-  if ( mComposition->printAsRaster() )
+  bool printAsRaster = mComposition->printAsRaster();
+  //mView->setScene( 0 );
+
+  if ( printAsRaster )
   {
     //print out via QImage, code copied from on_mActionExportAsImage_activated
     int width = ( int )( mComposition->printResolution() * mComposition->paperWidth() / 25.4 );
     int height = ( int )( mComposition-> printResolution() * mComposition->paperHeight() / 25.4 );
     QImage image( QSize( width, height ), QImage::Format_ARGB32 );
-    image.setDotsPerMeterX( mComposition->printResolution() / 25.4 * 1000 );
-    image.setDotsPerMeterY( mComposition->printResolution() / 25.4 * 1000 );
-    image.fill( 0 );
-    QPainter imagePainter( &image );
-    QRectF sourceArea( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
-    QRectF targetArea( 0, 0, width, height );
-    mComposition->render( &imagePainter, targetArea, sourceArea );
-    imagePainter.end();
-    p.drawImage( targetArea, image, targetArea );
+    if ( !image.isNull() )
+    {
+      image.setDotsPerMeterX( mComposition->printResolution() / 25.4 * 1000 );
+      image.setDotsPerMeterY( mComposition->printResolution() / 25.4 * 1000 );
+      image.fill( 0 );
+      QPainter imagePainter( &image );
+      QRectF sourceArea( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
+      QRectF targetArea( 0, 0, width, height );
+      mView->setPaintingEnabled( false );
+      mComposition->render( &imagePainter, targetArea, sourceArea );
+      mView->setPaintingEnabled( true );
+      p.drawImage( targetArea, image, targetArea );
+    }
+    else
+    {
+      QApplication::restoreOverrideCursor();
+      int answer = QMessageBox::warning( 0,
+                                         tr( "Image too large" ),
+                                         tr( "Creation of image with %1x%2 pixels failed.  Retry without 'Print As Raster'?" )
+                                         .arg( width ).arg( height ),
+                                         QMessageBox::Ok | QMessageBox::Cancel,
+                                         QMessageBox::Ok );
+      if ( answer == QMessageBox::Cancel )
+      {
+        mComposition->setPlotStyle( savedPlotStyle );
+        return;
+      }
+
+      QApplication::setOverrideCursor( Qt::BusyCursor );
+      printAsRaster = false;
+    }
   }
-  else
+
+  if ( !printAsRaster )
   {
-#if QT_VERSION < 0x040400
-    QRectF paperRect( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
-    QRect pageRect = printer.pageRect();
-    mComposition->render( &p, pageRect, paperRect );
-#else
     //better in case of custom page size, but only possible with Qt>=4.4.0
     QRectF paperRectMM = printer.pageRect( QPrinter::Millimeter );
     QRectF paperRectPixel = printer.pageRect( QPrinter::DevicePixel );
+
+    mView->setPaintingEnabled( false );
     mComposition->render( &p, paperRectPixel, paperRectMM );
-#endif
+    mView->setPaintingEnabled( true );
   }
+
   mComposition->setPlotStyle( savedPlotStyle );
   QApplication::restoreOverrideCursor();
 }
@@ -575,122 +666,55 @@ void QgsComposer::on_mActionExportAsImage_triggered()
   int height = ( int )( mComposition-> printResolution() * mComposition->paperHeight() / 25.4 );
 
   int memuse = width * height * 3 / 1000000;  // pixmap + image
-  QgsDebugMsg( QString( "Image %1 x %2" ).arg( width ).arg( height ) );
+  QgsDebugMsg( QString( "Image %1x%2" ).arg( width ).arg( height ) );
   QgsDebugMsg( QString( "memuse = %1" ).arg( memuse ) );
 
-  if ( memuse > 200 )   // cca 4500 x 4500
+  if ( memuse > 200 )   // about 4500x4500
   {
     int answer = QMessageBox::warning( 0, tr( "Big image" ),
-                                       tr( "To create image %1 x %2 requires circa %3 MB of memory" )
+                                       tr( "To create image %1x%2 requires about %3 MB of memory. Proceed?" )
                                        .arg( width ).arg( height ).arg( memuse ),
-                                       QMessageBox::Ok,  QMessageBox::Abort );
+                                       QMessageBox::Ok | QMessageBox::Cancel,  QMessageBox::Ok );
 
     raise();
-    if ( answer == QMessageBox::Abort ) return;
+    if ( answer == QMessageBox::Cancel )
+      return;
   }
 
-  // Get file and format (stolen from qgisapp.cpp but modified significantely)
+  QPair<QString, QString> fileNExt = QgisGui::getSaveAsImageName( this, tr( "Choose a file name to save the map image as" ) );
 
-  //create a map to hold the QImageIO names and the filter names
-  //the QImageIO name must be passed to the mapcanvas saveas image function
-  typedef QMap<QString, QString> FilterMap;
-  FilterMap myFilterMap;
+  QgsDebugMsg( QString( "Selected filter: %1" ).arg( fileNExt.first ) );
+  QgsDebugMsg( QString( "Image type: %1" ).arg( fileNExt.second ) );
 
-  //find out the last used filter
-  QSettings myQSettings;  // where we keep last used filter in persistent state
-  QString myLastUsedFormat = myQSettings.value( "/UI/lastSaveAsImageFormat", "png" ).toString();
-  QString myLastUsedFile = myQSettings.value( "/UI/lastSaveAsImageFile", "qgis.png" ).toString();
-  QFileInfo file( myLastUsedFile );
-
-  // get a list of supported output image types
-  int myCounterInt = 0;
-  QString myFilters;
-  QString myLastUsedFilter;
-  for ( ; myCounterInt < QImageWriter::supportedImageFormats().count(); myCounterInt++ )
-  {
-    QString myFormat = QString( QImageWriter::supportedImageFormats().at( myCounterInt ) );
-    QString myFilter = tr( "%1 format (*.%2 *.%3)" )
-                       .arg( myFormat ).arg( myFormat.toLower() ).arg( myFormat.toUpper() );
-
-    if ( myCounterInt > 0 ) myFilters += ";;";
-    myFilters += myFilter;
-    myFilterMap[myFilter] = myFormat;
-    if ( myFormat == myLastUsedFormat )
-    {
-      myLastUsedFilter = myFilter;
-    }
-  }
-#ifdef QGISDEBUG
-  QgsDebugMsg( "Available Filters Map: " );
-  FilterMap::Iterator myIterator;
-  for ( myIterator = myFilterMap.begin(); myIterator != myFilterMap.end(); ++myIterator )
-  {
-    QgsDebugMsg( QString( "%1  :  %2" ).arg( myIterator.key() ).arg( myIterator.value() ) );
-  }
-#endif
-
-  //create a file dialog using the the filter list generated above
-  std::auto_ptr < QFileDialog > myQFileDialog(
-    new QFileDialog(
-      this,
-      tr( "Choose a file name to save the map image as" ),
-      file.path(),
-      myFilters
-    )
-  );
-
-  myQFileDialog->setFileMode( QFileDialog::AnyFile );
-
-  // set the filter to the last one used
-  myQFileDialog->selectFilter( myLastUsedFilter );
-
-  // set the 'Open' button to something that makes more sense
-  myQFileDialog->setAcceptMode( QFileDialog::AcceptSave );
-
-  //prompt the user for a file name
-  QString myOutputFileNameQString;
-
-  int result = myQFileDialog->exec();
-  //raise();
-
-  if ( result != QDialog::Accepted )
+  if ( fileNExt.first.isEmpty() )
   {
     return;
   }
 
-  myOutputFileNameQString = myQFileDialog->selectedFiles().last();
-  QgsDebugMsg( myOutputFileNameQString );
-  QString myFilterString = myQFileDialog->selectedFilter();
-  QgsDebugMsg( QString( "Selected filter: %1" ).arg( myFilterString ) );
-  QgsDebugMsg( QString( "Image type: %1" ).arg( myFilterMap[myFilterString] ) );
-
-  // Add the file type suffix to the fileName if required
-  if ( !myOutputFileNameQString.endsWith( myFilterMap[myFilterString] ) )
+  QImage image( QSize( width, height ), QImage::Format_ARGB32 );
+  if ( image.isNull() )
   {
-    myOutputFileNameQString += "." + myFilterMap[myFilterString];
+    QMessageBox::warning( 0,
+                          tr( "Image too big" ),
+                          tr( "Creation of image with %1x%2 pixels failed.  Export aborted." )
+                          .arg( width ).arg( height ),
+                          QMessageBox::Ok );
+    return;
   }
 
-  myQSettings.setValue( "/UI/lastSaveAsImageFormat", myFilterMap[myFilterString] );
-  myQSettings.setValue( "/UI/lastSaveAsImageFile", myOutputFileNameQString );
-
-  if ( myOutputFileNameQString == "" ) return;
-
   mComposition->setPlotStyle( QgsComposition::Print );
-  mView->setScene( 0 );
-
-  QImage image( QSize( width, height ), QImage::Format_ARGB32 );
   image.setDotsPerMeterX( mComposition->printResolution() / 25.4 * 1000 );
   image.setDotsPerMeterY( mComposition->printResolution() / 25.4 * 1000 );
   image.fill( 0 );
   QPainter p( &image );
   QRectF sourceArea( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
   QRectF targetArea( 0, 0, width, height );
+  mView->setPaintingEnabled( false );
   mComposition->render( &p, targetArea, sourceArea );
   p.end();
-
   mComposition->setPlotStyle( QgsComposition::Preview );
-  image.save( myOutputFileNameQString, myFilterMap[myFilterString].toLocal8Bit().data() );
-  mView->setScene( mComposition );
+  mView->setPaintingEnabled( true );
+  image.save( fileNExt.first, fileNExt.second.toLocal8Bit().constData() );
 }
 
 
@@ -701,10 +725,10 @@ void QgsComposer::on_mActionExportAsSVG_triggered()
     showWMSPrintingWarning();
   }
 
-  QString myQSettingsLabel = "/UI/displaySVGWarning";
-  QSettings myQSettings;
+  QString settingsLabel = "/UI/displaySVGWarning";
+  QSettings settings;
 
-  bool displaySVGWarning = myQSettings.value( myQSettingsLabel, true ).toBool();
+  bool displaySVGWarning = settings.value( settingsLabel, true ).toBool();
 
   if ( displaySVGWarning )
   {
@@ -713,7 +737,7 @@ void QgsComposer::on_mActionExportAsSVG_triggered()
     m->setCheckBoxText( tr( "Don't show this message again" ) );
     m->setCheckBoxState( Qt::Unchecked );
     m->setCheckBoxVisible( true );
-    m->setCheckBoxQSettingsLabel( myQSettingsLabel );
+    m->setCheckBoxQSettingsLabel( settingsLabel );
     m->setMessageAsHtml( tr( "<p>The SVG export function in Qgis has several "
                              "problems due to bugs and deficiencies in the " )
                          + tr( "Qt4 svg code. In particular, there are problems "
@@ -726,48 +750,54 @@ void QgsComposer::on_mActionExportAsSVG_triggered()
                                "</p>" ) );
     m->exec();
   }
-  QString myLastUsedFile = myQSettings.value( "/UI/lastSaveAsSvgFile", "qgis.svg" ).toString();
-  QFileInfo file( myLastUsedFile );
-  QFileDialog *myQFileDialog = new QFileDialog( this, tr( "Choose a file name to save the map as" ),
-      file.path(), tr( "SVG Format" ) + " (*.svg *SVG)" );
-  myQFileDialog->selectFile( file.fileName() );
-  myQFileDialog->setFileMode( QFileDialog::AnyFile );
-  myQFileDialog->setAcceptMode( QFileDialog::AcceptSave );
 
-  int result = myQFileDialog->exec();
-  raise();
-  if ( result != QDialog::Accepted ) return;
+  QString lastUsedFile = settings.value( "/UI/lastSaveAsSvgFile", "qgis.svg" ).toString();
+  QFileInfo file( lastUsedFile );
 
-  QString myOutputFileNameQString = myQFileDialog->selectedFiles().first();
-  if ( myOutputFileNameQString == "" ) return;
+  QString outputFileName = QFileDialog::getSaveFileName(
+                             this,
+                             tr( "Choose a file name to save the map as" ),
+                             file.path(),
+                             tr( "SVG Format" ) + " (*.svg *.SVG)" );
 
-  myQSettings.setValue( "/UI/lastSaveAsSvgFile", myOutputFileNameQString );
+  if ( outputFileName.isEmpty() )
+  {
+    return;
+  }
 
-  //mView->setScene(0);//don't redraw the scene on the display while we render
+
+  if ( !outputFileName.endsWith( ".svg", Qt::CaseInsensitive ) )
+  {
+    outputFileName += ".svg" ;
+  }
+
+  settings.setValue( "/UI/lastSaveAsSvgFile", outputFileName );
   mComposition->setPlotStyle( QgsComposition::Print );
 
   QSvgGenerator generator;
 #if QT_VERSION >= 0x040500
   generator.setTitle( QgsProject::instance()->title() );
 #endif
-  generator.setFileName( myOutputFileNameQString );
+  generator.setFileName( outputFileName );
   //width in pixel
   int width = ( int )( mComposition->paperWidth() * mComposition->printResolution() / 25.4 );
   //height in pixel
   int height = ( int )( mComposition->paperHeight() * mComposition->printResolution() / 25.4 );
   generator.setSize( QSize( width, height ) );
+#if QT_VERSION >= 0x040500
+  generator.setViewBox( QRect( 0, 0, width, height ) );
+#endif
   generator.setResolution( mComposition->printResolution() ); //because the rendering is done in mm, convert the dpi
 
   QPainter p( &generator );
 
   QRectF sourceArea( 0, 0, mComposition->paperWidth(), mComposition->paperHeight() );
   QRectF targetArea( 0, 0, width, height );
+  mView->setPaintingEnabled( false );
   mComposition->render( &p, targetArea, sourceArea );
-
   p.end();
-
   mComposition->setPlotStyle( QgsComposition::Preview );
-  //mView->setScene(mComposition->canvas()); //now that we're done, set the view to show the scene again
+  mView->setPaintingEnabled( true );
 }
 
 void QgsComposer::on_mActionSelectMoveItem_triggered()
@@ -826,6 +856,14 @@ void QgsComposer::on_mActionAddBasicShape_triggered()
   }
 }
 
+void QgsComposer::on_mActionAddTable_triggered()
+{
+  if ( mView )
+  {
+    mView->setCurrentTool( QgsComposerView::AddTable );
+  }
+}
+
 void QgsComposer::on_mActionAddArrow_triggered()
 {
   if ( mView )
@@ -839,7 +877,12 @@ void QgsComposer::on_mActionSaveAsTemplate_triggered()
   //show file dialog
   QSettings settings;
   QString lastSaveDir = settings.value( "UI/lastComposerTemplateDir", "" ).toString();
-  QString saveFileName = QFileDialog::getSaveFileName( 0, tr( "save template" ), lastSaveDir, "*.qpt" );
+  QString saveFileName = QFileDialog::getSaveFileName(
+                           this,
+                           tr( "Save template" ),
+                           lastSaveDir,
+                           tr( "Composer templates" ) + " (*.qpt *.QPT)" );
+
   if ( saveFileName.isEmpty() )
   {
     return;
@@ -890,6 +933,8 @@ void QgsComposer::on_mActionLoadFromTemplate_triggered()
     return;
   }
 
+  emit composerWillBeRemoved( mView );
+
   QDomDocument templateDocument;
   if ( !templateDocument.setContent( &templateFile, false ) )
   {
@@ -899,9 +944,7 @@ void QgsComposer::on_mActionLoadFromTemplate_triggered()
 
   deleteItems();
   readXML( templateDocument );
-
-  //clean up after template read (e.g. legend and map extent)
-  cleanupAfterTemplateRead();
+  emit composerAdded( mView );
 }
 
 void QgsComposer::on_mActionMoveItemContent_triggered()
@@ -1008,6 +1051,22 @@ void QgsComposer::on_mActionAlignBottom_triggered()
   }
 }
 
+void QgsComposer::on_mActionUndo_triggered()
+{
+  if ( mComposition && mComposition->undoStack() )
+  {
+    mComposition->undoStack()->undo();
+  }
+}
+
+void QgsComposer::on_mActionRedo_triggered()
+{
+  if ( mComposition && mComposition->undoStack() )
+  {
+    mComposition->undoStack()->redo();
+  }
+}
+
 void QgsComposer::moveEvent( QMoveEvent *e ) { saveWindowState(); }
 
 void QgsComposer::resizeEvent( QResizeEvent *e )
@@ -1030,7 +1089,7 @@ void QgsComposer::restoreWindowState()
   QSettings settings;
   restoreGeometry( settings.value( "/Composer/geometry" ).toByteArray() );
   QVariant splitterState = settings.value( "/Composer/splitterState" );
-  if ( splitterState != QVariant::QVariant() )
+  if ( !splitterState.isNull() )
   {
     //mSplitter->restoreState(settings.value("/Composer/splitterState").toByteArray());
   }
@@ -1099,6 +1158,7 @@ void QgsComposer::readXML( const QDomDocument& doc )
     return;
   }
   readXML( composerNodeList.at( 0 ).toElement(), doc, true );
+  cleanupAfterTemplateRead();
 }
 
 void QgsComposer::readXML( const QDomElement& composerElem, const QDomDocument& doc, bool fromTemplate )
@@ -1260,9 +1320,39 @@ void QgsComposer::readXML( const QDomElement& composerElem, const QDomDocument&
     showItemOptions( newArrow );
   }
 
+  //composer tables
+  QDomNodeList composerTableList = composerElem.elementsByTagName( "ComposerAttributeTable" );
+  for ( int i = 0; i < composerTableList.size(); ++i )
+  {
+    QDomElement currentTableElem = composerTableList.at( i ).toElement();
+    QgsComposerAttributeTable* newTable = new QgsComposerAttributeTable( mComposition );
+    newTable->readXML( currentTableElem, doc );
+    addComposerTable( newTable );
+    mComposition->addItem( newTable );
+    mComposition->update();
+    mComposition->clearSelection();
+    newTable->setSelected( true );
+    showItemOptions( newTable );
+  }
+
   mComposition->sortZList();
   mView->setComposition( mComposition );
 
+  if ( mUndoView )
+  {
+    //init undo/redo buttons
+    mActionUndo->setEnabled( false );
+    mActionRedo->setEnabled( false );
+    if ( mComposition->undoStack() )
+    {
+      mUndoView->setStack( mComposition->undoStack() );
+      connect( mComposition->undoStack(), SIGNAL( canUndoChanged( bool ) ), mActionUndo, SLOT( setEnabled( bool ) ) );
+      connect( mComposition->undoStack(), SIGNAL( canRedoChanged( bool ) ), mActionRedo, SLOT( setEnabled( bool ) ) );
+    }
+  }
+
+
+
   setSelectionTool();
 }
 
@@ -1296,6 +1386,7 @@ void QgsComposer::addComposerMap( QgsComposerMap* map )
     return;
   }
 
+  map->setMapCanvas( mapCanvas() ); //set canvas to composer map to have the possibility to draw canvas items
   QgsComposerMapWidget* mapWidget = new QgsComposerMapWidget( map );
   connect( this, SIGNAL( zoomLevelChanged() ), map, SLOT( renderModeUpdateCachedImage() ) );
   mItemWidgetMap.insert( map, mapWidget );
@@ -1331,6 +1422,10 @@ void QgsComposer::addComposerLegend( QgsComposerLegend* legend )
   }
 
   QgsComposerLegendWidget* lWidget = new QgsComposerLegendWidget( legend );
+  if ( sender() ) //only update if created from GUI (not after XML read)
+  {
+    lWidget->updateLegend();
+  }
   mItemWidgetMap.insert( legend, lWidget );
 }
 
@@ -1355,6 +1450,16 @@ void QgsComposer::addComposerShape( QgsComposerShape* shape )
   mItemWidgetMap.insert( shape, sWidget );
 }
 
+void QgsComposer::addComposerTable( QgsComposerAttributeTable* table )
+{
+  if ( !table )
+  {
+    return;
+  }
+  QgsComposerTableWidget* tWidget = new QgsComposerTableWidget( table );
+  mItemWidgetMap.insert( table, tWidget );
+}
+
 void QgsComposer::deleteItem( QgsComposerItem* item )
 {
   QMap<QgsComposerItem*, QWidget*>::iterator it = mItemWidgetMap.find( item );
@@ -1364,7 +1469,7 @@ void QgsComposer::deleteItem( QgsComposerItem* item )
     return;
   }
 
-  delete( it.key() );
+  //the item itself is not deleted here (usually, this is done in the destructor of QgsAddRemoveItemCommand)
   delete( it.value() );
   mItemWidgetMap.remove( it.key() );
 }
@@ -1466,4 +1571,13 @@ void QgsComposer::cleanupAfterTemplateRead()
   }
 }
 
+void QgsComposer::on_mActionPageSetup_triggered()
+{
+  if ( !mComposition )
+  {
+    return;
+  }
 
+  QPageSetupDialog pageSetupDialog( &mPrinter, this );
+  pageSetupDialog.exec();
+}
