@@ -20,6 +20,7 @@
 #include "ui_qgscomposerbase.h"
 #include "qgscomposeritem.h"
 #include "qgscontexthelp.h"
+#include <QPrinter>
 
 class QgisApp;
 class QgsComposerArrow;
@@ -29,6 +30,7 @@ class QgsComposerMap;
 class QgsComposerPicture;
 class QgsComposerScaleBar;
 class QgsComposerShape;
+class QgsComposerAttributeTable;
 class QgsComposerView;
 class QgsComposition;
 class QgsMapCanvas;
@@ -40,6 +42,7 @@ class QMoveEvent;
 class QResizeEvent;
 class QFile;
 class QSizeGrip;
+class QUndoView;
 
 /** \ingroup MapComposer
  * \brief A gui for composing a printable map.
@@ -55,6 +58,8 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! Set the pixmap / icons on the toolbar buttons
     void setupTheme();
 
+    void setIconSizes( int size );
+
     //! Open and show, set defaults if first time
     void open();
 
@@ -100,6 +105,10 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! Is emitted every time the view zoom has changed
     void zoomLevelChanged();
 
+    void composerAdded( QgsComposerView* v );
+    //!Composer deletes the old composerview when loading a template
+    void composerWillBeRemoved( QgsComposerView* v );
+
 
   public slots:
     //! Zoom to full extent of the paper
@@ -117,6 +126,9 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! Print the composition
     void on_mActionPrint_triggered();
 
+    //! Page Setup for composition
+    void on_mActionPageSetup_triggered();
+
     //! Print as image
     void on_mActionExportAsImage_triggered();
 
@@ -150,6 +162,9 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //! Add ellipse shape item
     void on_mActionAddBasicShape_triggered();
 
+    //! Add attribute table
+    void on_mActionAddTable_triggered();
+
     //! Save composer as template
     void on_mActionSaveAsTemplate_triggered();
 
@@ -194,10 +209,16 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     //!Align selected composer items to bottom
     void on_mActionAlignBottom_triggered();
 
+    //!Undo last composer change
+    void on_mActionUndo_triggered();
+
+    //!Redo last composer change
+    void on_mActionRedo_triggered();
+
     //! Save window state
     void saveWindowState();
 
-    /**Add a composer arrow to the item/widget map and crete a configuration widget for it*/
+    /**Add a composer arrow to the item/widget map and creates a configuration widget for it*/
     void addComposerArrow( QgsComposerArrow* arrow );
 
     /**Add a composer map to the item/widget map and creates a configuration widget for it*/
@@ -218,11 +239,14 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
     /**Adds a composer shape to the item/widget map and creates a configuration widget*/
     void addComposerShape( QgsComposerShape* shape );
 
-    /**Removes item from the item/widget map and deletes the configuration widget*/
+    /**Adds a composer table to the item/widget map and creates a configuration widget*/
+    void addComposerTable( QgsComposerAttributeTable* table );
+
+    /**Removes item from the item/widget map and deletes the configuration widget. Does not delete the item itself*/
     void deleteItem( QgsComposerItem* item );
 
     /**Shows the configuration widget for a composer item*/
-    void showItemOptions( const QgsComposerItem* i );
+    void showItemOptions( QgsComposerItem* i );
 
     //XML, usually connected with QgsProject::readProject and QgsProject::writeProject
 
@@ -292,6 +316,11 @@ class QgsComposer: public QMainWindow, private Ui::QgsComposerBase
 
     //! Window menu action to select this window
     QAction *mWindowAction;
+
+    //! Page & Printer Setup
+    QPrinter mPrinter;
+
+    QUndoView* mUndoView;
 };
 
 #endif
