@@ -36,9 +36,21 @@ class QgsComposerLabelWidget: public QWidget, private Ui::QgsComposerLabelWidget
     void on_mFontButton_clicked();
     void on_mMarginDoubleSpinBox_valueChanged( double d );
     void on_mFontColorButton_clicked();
+    void on_mCenterRadioButton_clicked();
+    void on_mLeftRadioButton_clicked();
+    void on_mRightRadioButton_clicked();
+    void on_mTopRadioButton_clicked();
+    void on_mBottomRadioButton_clicked();
+    void on_mMiddleRadioButton_clicked();
+    void on_mLabelIdLineEdit_textChanged( const QString& text );
+
+  private slots:
+    void setGuiElementValues();
 
   private:
     QgsComposerLabel* mComposerLabel;
+
+    void blockAllSignals( bool block );
 };
 
 #endif //QGSCOMPOSERLABELWIDGET
