@@ -34,6 +34,9 @@ class QgsComposerLegendWidget: public QWidget, private Ui::QgsComposerLegendWidg
     QgsComposerLegendWidget( QgsComposerLegend* legend );
     ~QgsComposerLegendWidget();
 
+    /**Updates the legend layers and groups*/
+    void updateLegend();
+
   public slots:
 
     void on_mTitleLineEdit_textChanged( const QString& text );
@@ -43,22 +46,35 @@ class QgsComposerLegendWidget: public QWidget, private Ui::QgsComposerLegendWidg
     void on_mSymbolSpaceSpinBox_valueChanged( double d );
     void on_mIconLabelSpaceSpinBox_valueChanged( double d );
     void on_mTitleFontButton_clicked();
+    void on_mGroupFontButton_clicked();
     void on_mLayerFontButton_clicked();
     void on_mItemFontButton_clicked();
     void on_mBoxSpaceSpinBox_valueChanged( double d );
+    void on_mCheckBoxAutoUpdate_stateChanged( int state );
+    void on_mMapComboBox_currentIndexChanged( int index );
 
     //item manipulation
     void on_mMoveDownToolButton_clicked();
     void on_mMoveUpToolButton_clicked();
     void on_mRemoveToolButton_clicked();
+    void on_mAddToolButton_clicked();
     void on_mEditPushButton_clicked();
     void on_mUpdatePushButton_clicked();
     void on_mUpdateAllPushButton_clicked();
+    void on_mAddGroupButton_clicked();
+
+  protected:
+    void showEvent( QShowEvent * event );
+
+  private slots:
+    /**Sets GUI according to state of mLegend*/
+    void setGuiElements();
 
   private:
     QgsComposerLegendWidget();
-    /**Sets GUI according to state of mLegend*/
-    void setGuiElements();
+    void blockAllSignals( bool b );
+    void refreshMapComboBox();
+
 
     QgsComposerLegend* mLegend;
 };
