@@ -29,16 +29,15 @@ QgsComposerMapWidget::QgsComposerMapWidget( QgsComposerMap* composerMap ): QWidg
   //add widget for general composer item properties
   QgsComposerItemWidget* itemPropertiesWidget = new QgsComposerItemWidget( this, composerMap );
   toolBox->addItem( itemPropertiesWidget, tr( "General options" ) );
-  QDoubleValidator v( 0 );
 
-  mWidthLineEdit->setValidator( &v );
-  mHeightLineEdit->setValidator( &v );
-  mScaleLineEdit->setValidator( &v );
+  mWidthLineEdit->setValidator( new QDoubleValidator( mWidthLineEdit ) );
+  mHeightLineEdit->setValidator( new QDoubleValidator( mHeightLineEdit ) );
+  mScaleLineEdit->setValidator( new QDoubleValidator( mScaleLineEdit ) );
 
-  mXMinLineEdit->setValidator( &v );
-  mXMaxLineEdit->setValidator( &v );
-  mYMinLineEdit->setValidator( &v );
-  mYMaxLineEdit->setValidator( &v );
+  mXMinLineEdit->setValidator( new QDoubleValidator( mXMinLineEdit ) );
+  mXMaxLineEdit->setValidator( new QDoubleValidator( mXMaxLineEdit ) );
+  mYMinLineEdit->setValidator( new QDoubleValidator( mYMinLineEdit ) );
+  mYMaxLineEdit->setValidator( new QDoubleValidator( mYMaxLineEdit ) );
 
   blockAllSignals( true );
   mPreviewModeComboBox->insertItem( 0, tr( "Cache" ) );
@@ -57,7 +56,7 @@ QgsComposerMapWidget::QgsComposerMapWidget( QgsComposerMap* composerMap ): QWidg
   mAnnotationDirectionComboBox->insertItem( 2, tr( "Boundary direction" ) );
   if ( composerMap )
   {
-    connect( composerMap, SIGNAL( extentChanged() ), this, SLOT( updateSettingsNoSignals() ) );
+    connect( composerMap, SIGNAL( itemChanged() ), this, SLOT( setGuiElementValues() ) );
   }
 
   updateGuiElements();
@@ -82,7 +81,10 @@ void QgsComposerMapWidget::on_mWidthLineEdit_editingFinished()
     QTransform composerMapTransform = mComposerMap->transform();
 
     QRectF newRect( composerMapTransform.dx(), composerMapTransform.dy(), newWidth, composerMapRect.height() );
+
+    mComposerMap->beginCommand( tr( "Change item width" ) );
     mComposerMap->setSceneRect( newRect );
+    mComposerMap->endCommand();
   }
 }
 
@@ -100,7 +102,9 @@ void QgsComposerMapWidget::on_mHeightLineEdit_editingFinished()
     QTransform composerMapTransform = mComposerMap->transform();
 
     QRectF newRect( composerMapTransform.dx(), composerMapTransform.dy(), composerMapRect.width(), newHeight );
+    mComposerMap->beginCommand( tr( "Change item height" ) );
     mComposerMap->setSceneRect( newRect );
+    mComposerMap->endCommand();
   }
 }
 
@@ -152,17 +156,21 @@ void QgsComposerMapWidget::on_mScaleLineEdit_editingFinished()
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Map scale changed" ) );
   mComposerMap->setNewScale( scaleDenominator );
+  mComposerMap->endCommand();
 }
 
-void QgsComposerMapWidget::on_mRotationSpinBox_valueChanged( int value )
+void QgsComposerMapWidget::on_mRotationSpinBox_valueChanged( double value )
 {
   if ( !mComposerMap )
   {
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Map rotation changed" ), QgsComposerMergeCommand::ComposerMapRotation );
   mComposerMap->setMapRotation( value );
+  mComposerMap->endCommand();
   mComposerMap->cache();
   mComposerMap->update();
 }
@@ -201,7 +209,9 @@ void QgsComposerMapWidget::on_mSetToMapCanvasExtentButton_clicked()
       mYMinLineEdit->setText( QString::number( newExtent.yMinimum() ) );
       mYMaxLineEdit->setText( QString::number( newExtent.yMaximum() ) );
 
+      mComposerMap->beginCommand( tr( "Map extent changed" ) );
       mComposerMap->setNewExtent( newExtent );
+      mComposerMap->endCommand();
     }
   }
 }
@@ -226,7 +236,7 @@ void QgsComposerMapWidget::on_mYMaxLineEdit_editingFinished()
   updateComposerExtentFromGui();
 }
 
-void QgsComposerMapWidget::updateSettingsNoSignals()
+void QgsComposerMapWidget::setGuiElementValues()
 {
   mHeightLineEdit->blockSignals( true );
   mWidthLineEdit->blockSignals( true );
@@ -293,6 +303,16 @@ void QgsComposerMapWidget::updateGuiElements()
       mKeepLayerListCheckBox->setCheckState( Qt::Unchecked );
     }
 
+    //draw canvas items
+    if ( mComposerMap->drawCanvasItems() )
+    {
+      mDrawCanvasItemsCheckBox->setCheckState( Qt::Checked );
+    }
+    else
+    {
+      mDrawCanvasItemsCheckBox->setCheckState( Qt::Unchecked );
+    }
+
     //grid
     if ( mComposerMap->gridEnabled() )
     {
@@ -389,7 +409,9 @@ void QgsComposerMapWidget::updateComposerExtentFromGui()
   if ( !conversionSuccess ) {return;}
 
   QgsRectangle newExtent( xmin, ymin, xmax, ymax );
+  mComposerMap->beginCommand( tr( "Map extent changed" ) );
   mComposerMap->setNewExtent( newExtent );
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::blockAllSignals( bool b )
@@ -419,6 +441,7 @@ void QgsComposerMapWidget::blockAllSignals( bool b )
   mDistanceToMapFrameSpinBox->blockSignals( b );
   mAnnotationDirectionComboBox->blockSignals( b );
   mCoordinatePrecisionSpinBox->blockSignals( b );
+  mDrawCanvasItemsCheckBox->blockSignals( b );
 }
 
 void QgsComposerMapWidget::on_mUpdatePreviewButton_clicked()
@@ -462,6 +485,23 @@ void QgsComposerMapWidget::on_mKeepLayerListCheckBox_stateChanged( int state )
   }
 }
 
+void QgsComposerMapWidget::on_mDrawCanvasItemsCheckBox_stateChanged( int state )
+{
+  if ( !mComposerMap )
+  {
+    return;
+  }
+
+  mComposerMap->beginCommand( tr( "Canvas items toggled" ) );
+  mComposerMap->setDrawCanvasItems( state == Qt::Checked );
+  mUpdatePreviewButton->setEnabled( false ); //prevent crashes because of many button clicks
+  mComposerMap->setCacheUpdated( false );
+  mComposerMap->cache();
+  mComposerMap->update();
+  mUpdatePreviewButton->setEnabled( true );
+  mComposerMap->endCommand();
+}
+
 void QgsComposerMapWidget::on_mGridCheckBox_toggled( bool state )
 {
   if ( !mComposerMap )
@@ -469,6 +509,7 @@ void QgsComposerMapWidget::on_mGridCheckBox_toggled( bool state )
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Grid checkbox toggled" ) );
   if ( state )
   {
     mComposerMap->setGridEnabled( true );
@@ -479,6 +520,7 @@ void QgsComposerMapWidget::on_mGridCheckBox_toggled( bool state )
   }
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mIntervalXSpinBox_editingFinished()
@@ -487,9 +529,12 @@ void QgsComposerMapWidget::on_mIntervalXSpinBox_editingFinished()
   {
     return;
   }
+
+  mComposerMap->beginCommand( tr( "Grid interval changed" ) );
   mComposerMap->setGridIntervalX( mIntervalXSpinBox->value() );
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mIntervalYSpinBox_editingFinished()
@@ -498,9 +543,11 @@ void QgsComposerMapWidget::on_mIntervalYSpinBox_editingFinished()
   {
     return;
   }
+  mComposerMap->beginCommand( tr( "Grid interval changed" ) );
   mComposerMap->setGridIntervalY( mIntervalYSpinBox->value() );
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mOffsetXSpinBox_editingFinished()
@@ -509,9 +556,11 @@ void QgsComposerMapWidget::on_mOffsetXSpinBox_editingFinished()
   {
     return;
   }
+  mComposerMap->beginCommand( tr( "Grid offset changed" ) );
   mComposerMap->setGridOffsetX( mOffsetXSpinBox->value() );
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mOffsetYSpinBox_editingFinished()
@@ -520,9 +569,11 @@ void QgsComposerMapWidget::on_mOffsetYSpinBox_editingFinished()
   {
     return;
   }
+  mComposerMap->beginCommand( tr( "Grid offset changed" ) );
   mComposerMap->setGridOffsetY( mOffsetYSpinBox->value() );
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mLineWidthSpinBox_valueChanged( double d )
@@ -531,8 +582,10 @@ void QgsComposerMapWidget::on_mLineWidthSpinBox_valueChanged( double d )
   {
     return;
   }
+  mComposerMap->beginCommand( tr( "Grid pen changed" ) );
   mComposerMap->setGridPenWidth( d );
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mLineColorButton_clicked()
@@ -544,8 +597,10 @@ void QgsComposerMapWidget::on_mLineColorButton_clicked()
   QColor newColor = QColorDialog::getColor( mLineColorButton->color() );
   if ( newColor.isValid() )
   {
+    mComposerMap->beginCommand( tr( "Grid pen changed" ) );
     mLineColorButton->setColor( newColor );
     mComposerMap->setGridPenColor( newColor );
+    mComposerMap->endCommand();
   }
   mComposerMap->update();
 }
@@ -565,7 +620,9 @@ void QgsComposerMapWidget::on_mGridTypeComboBox_currentIndexChanged( const QStri
   {
     mComposerMap->setGridStyle( QgsComposerMap::Solid );
   }
+  mComposerMap->beginCommand( tr( "Grid type changed" ) );
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mCrossWidthSpinBox_valueChanged( double d )
@@ -575,8 +632,10 @@ void QgsComposerMapWidget::on_mCrossWidthSpinBox_valueChanged( double d )
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Grid cross width changed" ) );
   mComposerMap->setCrossLength( d );
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mAnnotationFontButton_clicked()
@@ -587,17 +646,19 @@ void QgsComposerMapWidget::on_mAnnotationFontButton_clicked()
   }
 
   bool ok;
-#if defined(Q_WS_MAC) && QT_VERSION >= 0x040500 && !defined(__LP64__)
-  // Native Mac dialog works only for 64 bit Cocoa (observed in Qt 4.5.2, probably a Qt bug)
+#if defined(Q_WS_MAC) && QT_VERSION >= 0x040500 && defined(QT_MAC_USE_COCOA)
+  // Native Mac dialog works only for Qt Carbon
   QFont newFont = QFontDialog::getFont( &ok, mComposerMap->gridAnnotationFont(), this, QString(), QFontDialog::DontUseNativeDialog );
 #else
   QFont newFont = QFontDialog::getFont( &ok, mComposerMap->gridAnnotationFont(), this );
 #endif
   if ( ok )
   {
+    mComposerMap->beginCommand( tr( "Annotation font changed" ) );
     mComposerMap->setGridAnnotationFont( newFont );
     mComposerMap->updateBoundingRect();
     mComposerMap->update();
+    mComposerMap->endCommand();
   }
 }
 
@@ -607,9 +668,11 @@ void QgsComposerMapWidget::on_mDistanceToMapFrameSpinBox_valueChanged( double d
   {
     return;
   }
+  mComposerMap->beginCommand( tr( "Annotation distance changed" ), QgsComposerMergeCommand::ComposerMapAnnotationDistance );
   mComposerMap->setAnnotationFrameDistance( d );
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mAnnotationPositionComboBox_currentIndexChanged( const QString& text )
@@ -619,6 +682,7 @@ void QgsComposerMapWidget::on_mAnnotationPositionComboBox_currentIndexChanged( c
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Annotation position changed" ) );
   if ( text == tr( "Inside frame" ) )
   {
     mComposerMap->setGridAnnotationPosition( QgsComposerMap::InsideMapFrame );
@@ -629,6 +693,7 @@ void QgsComposerMapWidget::on_mAnnotationPositionComboBox_currentIndexChanged( c
   }
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mDrawAnnotationCheckBox_stateChanged( int state )
@@ -638,6 +703,7 @@ void QgsComposerMapWidget::on_mDrawAnnotationCheckBox_stateChanged( int state )
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Annotation toggled" ) );
   if ( state == Qt::Checked )
   {
     mComposerMap->setShowGridAnnotation( true );
@@ -648,6 +714,7 @@ void QgsComposerMapWidget::on_mDrawAnnotationCheckBox_stateChanged( int state )
   }
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mAnnotationDirectionComboBox_currentIndexChanged( const QString& text )
@@ -657,6 +724,7 @@ void QgsComposerMapWidget::on_mAnnotationDirectionComboBox_currentIndexChanged(
     return;
   }
 
+  mComposerMap->beginCommand( tr( "Changed annotation direction" ) );
   if ( text == tr( "Horizontal" ) )
   {
     mComposerMap->setGridAnnotationDirection( QgsComposerMap::Horizontal );
@@ -675,6 +743,7 @@ void QgsComposerMapWidget::on_mAnnotationDirectionComboBox_currentIndexChanged(
   }
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
 
 void QgsComposerMapWidget::on_mCoordinatePrecisionSpinBox_valueChanged( int value )
@@ -683,7 +752,9 @@ void QgsComposerMapWidget::on_mCoordinatePrecisionSpinBox_valueChanged( int valu
   {
     return;
   }
+  mComposerMap->beginCommand( tr( "Changed annotation precision" ) );
   mComposerMap->setGridAnnotationPrecision( value );
   mComposerMap->updateBoundingRect();
   mComposerMap->update();
+  mComposerMap->endCommand();
 }
