@@ -27,6 +27,7 @@
 #include <QMessageBox>
 #include <QPainter>
 #include <QProgressDialog>
+#include <QSettings>
 #include <QSvgRenderer>
 
 QgsComposerPictureWidget::QgsComposerPictureWidget( QgsComposerPicture* picture ): QWidget(), mPicture( picture )
@@ -45,7 +46,7 @@ QgsComposerPictureWidget::QgsComposerPictureWidget( QgsComposerPicture* picture
 
   //add preview icons
   addStandardDirectoriesToPreview();
-  connect( mPicture, SIGNAL( settingsChanged() ), this, SLOT( setGuiElementValues() ) );
+  connect( mPicture, SIGNAL( itemChanged() ), this, SLOT( setGuiElementValues() ) );
   connect( mPicture, SIGNAL( rotationChanged( double ) ), this, SLOT( setGuiElementValues() ) );
 }
 
@@ -87,8 +88,10 @@ void QgsComposerPictureWidget::on_mPictureBrowseButton_clicked()
   //pass file path to QgsComposerPicture
   if ( mPicture )
   {
+    mPicture->beginCommand( tr( "Picture changed" ) );
     mPicture->setPictureFile( filePath );
     mPicture->update();
+    mPicture->endCommand();
   }
 }
 
@@ -107,8 +110,10 @@ void QgsComposerPictureWidget::on_mPictureLineEdit_editingFinished()
       return;
     }
 
+    mPicture->beginCommand( tr( "Picture changed" ) );
     mPicture->setPictureFile( filePath );
     mPicture->update();
+    mPicture->endCommand();
   }
 }
 
@@ -122,8 +127,10 @@ void QgsComposerPictureWidget::on_mWidthLineEdit_editingFinished()
     double newWidth = mWidthLineEdit->text().toDouble( &conversionOk );
     if ( conversionOk )
     {
+      mPicture->beginCommand( tr( "Picture width changed" ) );
       QRectF newSceneRect( mPicture->transform().dx(), mPicture->transform().dy(), newWidth, pictureRect.height() );
       mPicture->setSceneRect( newSceneRect );
+      mPicture->endCommand();
     }
   }
 }
@@ -138,8 +145,10 @@ void QgsComposerPictureWidget::on_mHeightLineEdit_editingFinished()
     double newHeight = mHeightLineEdit->text().toDouble( &conversionOk );
     if ( conversionOk )
     {
+      mPicture->beginCommand( tr( "Picture height changed" ) );
       QRectF newSceneRect( mPicture->transform().dx(), mPicture->transform().dy(), pictureRect.width(), newHeight );
       mPicture->setSceneRect( newSceneRect );
+      mPicture->endCommand();
     }
   }
 }
@@ -148,8 +157,10 @@ void QgsComposerPictureWidget::on_mRotationSpinBox_valueChanged( double d )
 {
   if ( mPicture )
   {
+    mPicture->beginCommand( tr( "Picture rotation changed" ), QgsComposerMergeCommand::ComposerPictureRotation );
     mPicture->setRotation( d );
     mPicture->update();
+    mPicture->endCommand();
   }
 }
 
@@ -161,9 +172,11 @@ void QgsComposerPictureWidget::on_mPreviewListWidget_currentItemChanged( QListWi
   }
 
   QString absoluteFilePath = current->data( Qt::UserRole ).toString();
+  mPicture->beginCommand( tr( "Picture changed" ) );
   mPicture->setPictureFile( absoluteFilePath );
   mPictureLineEdit->setText( absoluteFilePath );
   mPicture->update();
+  mPicture->endCommand();
 }
 
 void QgsComposerPictureWidget::on_mAddDirectoryButton_clicked()
@@ -180,11 +193,24 @@ void QgsComposerPictureWidget::on_mAddDirectoryButton_clicked()
 
   //and add icons to the preview
   addDirectoryToPreview( directory );
+
+  //update the image directory list in the settings
+  QSettings s;
+  QStringList userDirList = s.value( "/Composer/PictureWidgetDirectories" ).toStringList();
+  if ( !userDirList.contains( directory ) )
+  {
+    userDirList.append( directory );
+  }
+  s.setValue( "/Composer/PictureWidgetDirectories", userDirList );
 }
 
 void QgsComposerPictureWidget::on_mRemoveDirectoryButton_clicked()
 {
   QString directoryToRemove = mSearchDirectoriesComboBox->currentText();
+  if ( directoryToRemove.isEmpty() )
+  {
+    return;
+  }
   mSearchDirectoriesComboBox->removeItem( mSearchDirectoriesComboBox->currentIndex() );
 
   //remove entries from back to front (to have the indices of existing items constant)
@@ -196,6 +222,12 @@ void QgsComposerPictureWidget::on_mRemoveDirectoryButton_clicked()
       delete( mPreviewListWidget->takeItem( i ) );
     }
   }
+
+  //update the image directory list in the settings
+  QSettings s;
+  QStringList userDirList = s.value( "/Composer/PictureWidgetDirectories" ).toStringList();
+  userDirList.removeOne( directoryToRemove );
+  s.setValue( "/Composer/PictureWidgetDirectories", userDirList );
 }
 
 void QgsComposerPictureWidget::on_mRotationFromComposerMapCheckBox_stateChanged( int state )
@@ -205,6 +237,7 @@ void QgsComposerPictureWidget::on_mRotationFromComposerMapCheckBox_stateChanged(
     return;
   }
 
+  mPicture->beginCommand( tr( "Rotation synchronisation toggled" ) );
   if ( state == Qt::Unchecked )
   {
     mPicture->setRotationMap( -1 );
@@ -219,10 +252,12 @@ void QgsComposerPictureWidget::on_mRotationFromComposerMapCheckBox_stateChanged(
       return;
     }
     int composerId = mComposerMapComboBox->itemData( currentItemIndex, Qt::UserRole ).toInt();
+
     mPicture->setRotationMap( composerId );
     mRotationSpinBox->setEnabled( false );
     mComposerMapComboBox->setEnabled( true );
   }
+  mPicture->endCommand();
 }
 
 void QgsComposerPictureWidget::showEvent( QShowEvent * event )
@@ -267,8 +302,10 @@ void QgsComposerPictureWidget::on_mComposerMapComboBox_activated( const QString
   {
     return;
   }
+  mPicture->beginCommand( tr( "Rotation map changed" ) );
   mPicture->setRotationMap( id );
   mPicture->update();
+  mPicture->endCommand();
 }
 
 void QgsComposerPictureWidget::refreshMapComboBox()
@@ -445,9 +482,14 @@ void QgsComposerPictureWidget::addStandardDirectoriesToPreview()
     QDir svgDirectory( svgPaths[i] );
     if ( !svgDirectory.exists() || !svgDirectory.isReadable() )
     {
-      return; //error
+      continue;
     }
 
+    //add directory itself
+    mSearchDirectoriesComboBox->addItem( svgDirectory.absolutePath() );
+    addDirectoryToPreview( svgDirectory.absolutePath() );
+
+    //and also subdirectories
     QFileInfoList directoryList = svgDirectory.entryInfoList( QDir::Dirs | QDir::NoDotAndDotDot );
     QFileInfoList::const_iterator dirIt = directoryList.constBegin();
     for ( ; dirIt != directoryList.constEnd(); ++dirIt )
@@ -458,6 +500,16 @@ void QgsComposerPictureWidget::addStandardDirectoriesToPreview()
       }
     }
   }
+
+  //include additional user-defined directories for images
+  QSettings s;
+  QStringList userDirList = s.value( "/Composer/PictureWidgetDirectories" ).toStringList();
+  QStringList::const_iterator userDirIt = userDirList.constBegin();
+  for ( ; userDirIt != userDirList.constEnd(); ++userDirIt )
+  {
+    addDirectoryToPreview( *userDirIt );
+    mSearchDirectoriesComboBox->addItem( *userDirIt );
+  }
 }
 
 bool QgsComposerPictureWidget::testSvgFile( const QString& filename ) const
@@ -470,19 +522,11 @@ bool QgsComposerPictureWidget::testSvgFile( const QString& filename ) const
   }
 
   QSvgRenderer svgRenderer( filename );
-  if ( svgRenderer.isValid() )
-  {
-    return true;
-  }
-  return false;
+  return svgRenderer.isValid();
 }
 
 bool QgsComposerPictureWidget::testImageFile( const QString& filename ) const
 {
   QString formatName = QString( QImageReader::imageFormat( filename ) );
-  if ( !formatName.isEmpty() )
-  {
-    return true; //file is in a supported pixel format
-  }
-  return false;
+  return !formatName.isEmpty(); //file is in a supported pixel format
 }
