@@ -20,10 +20,9 @@
 #ifndef QGSLEGEND_H
 #define QGSLEGEND_H
 
-#include <deque>
-#include <map>
-#include <set>
 #include <QTreeWidget>
+#include <QPair>
+#include <set>
 
 class QgsLegendGroup;
 class QgsLegendLayer;
@@ -35,6 +34,12 @@ class QDomElement;
 class QDomNode;
 class QMouseEvent;
 class QTreeWidgetItem;
+class QgsCoordinateReferenceSystem;
+
+//Information about relationship between groups and layers
+//key: group name (or null strings for single layers without groups)
+//value: containter with layer ids contained in the group
+typedef QPair< QString, QList<QString> > GroupLayerInfo;
 
 /**
    \class QgsLegend
@@ -84,14 +89,13 @@ class QgsLegend : public QTreeWidget
     // declaration was public while the definition was private
     class QgsLegendPixmaps;
 
-
   public:
     /*! Constructor.
     * @param qgis_app link to qgisapp
     * @param theParent An optional parent widget
     * @param theName An optional name for the widget
     */
-    QgsLegend( QWidget * parent = 0, const char *name = 0 );
+    QgsLegend( QgsMapCanvas *canvas, QWidget * parent = 0, const char *name = 0 );
 
     //! Destructor
     ~QgsLegend();
@@ -104,6 +108,14 @@ class QgsLegend : public QTreeWidget
     Else, 0 is returned.*/
     QgsMapLayer* currentLayer();
 
+    /*!Returns the currently selected layer QgsLegendLayers.
+    Else, an empty list is returned.*/
+    QList<QgsMapLayer *> selectedLayers();
+
+    /*!Returns all layers loaded in QgsMapCanvas.
+    Else, an empty list is returned.*/
+    QList<QgsMapLayer *> layers();
+
     /*!set the current layer
     returns true if the layer exists, false otherwise*/
     bool setCurrentLayer( QgsMapLayer *layer );
@@ -123,6 +135,9 @@ class QgsLegend : public QTreeWidget
     /**Returns a string list of groups*/
     QStringList groups();
 
+    //! Return the relationship between groups and layers in the legend
+    QList< GroupLayerInfo > groupLayerRelationship();
+
     /**Returns the first item in the hierarchy*/
     QTreeWidgetItem* firstItem();
 
@@ -150,7 +165,7 @@ class QgsLegend : public QTreeWidget
     void removeItem( QTreeWidgetItem* item );
 
     /**Returns the ids of the layers contained in this legend. The order is bottom->top*/
-    std::deque<QString> layerIDs();
+    QStringList layerIDs();
 
     /**Updates layer set of map canvas*/
     void updateMapCanvasLayerSet();
@@ -173,21 +188,18 @@ class QgsLegend : public QTreeWidget
     /**Removes an entry from mPixmapHeightValues*/
     void removePixmapHeightValue( int height );
 
-    /**Sets the toggle editing action. Usually called from QgisApp*/
-    void setToggleEditingAction( QAction* editingAction ) {mToggleEditingAction = editingAction;}
-
     /**Returns structure with legend pixmaps*/
     QgsLegendPixmaps& pixmaps() { return mPixmaps; }
 
-
-    void updateCheckStates( QTreeWidgetItem* item, Qt::CheckState state ) { item->setData( 0, Qt::UserRole, state ); }
+    /**Returns a layers check state*/
+    Qt::CheckState layerCheckState( QgsMapLayer * layer );
 
   public slots:
 
     /*!Adds a new layer group with the maplayer to the canvas*/
     void addLayer( QgsMapLayer * layer );
 
-    void setMapCanvas( QgsMapCanvas * canvas ) {mMapCanvas = canvas;}
+    void setLayerVisible( QgsMapLayer * layer, bool visible );
 
     /**Updates symbology items for a layer*/
     void refreshLayerSymbology( QString key, bool expandItem = true );
@@ -208,12 +220,31 @@ class QgsLegend : public QTreeWidget
 
     /*!
      * Slot called when user wishes to add a new empty layer group to the legend.
+     * If a the legend has a currentItem() then the new group will be nested into it
      * The user will be prompted for the name of the newly added group.
      * @param name name of the new group
      * @param expand expand the group
-     * @return void
+     * @return index of inserted group
      */
-    int addGroup( QString name = QString(), bool expand = true );
+    int addGroupToCurrentItem( QString name = QString(), bool expand = true );
+
+    /*!
+     * Slot called when user wishes to add a new empty layer group to the legend.
+     * The user will be prompted for the name of the newly added group.
+     * @param name name of the new group
+     * @param expand expand the group
+     * @return index of inserted group
+     */
+    int addGroup( QString name = QString(), bool expand = true, QTreeWidgetItem* parent = 0 );
+
+    /*!
+     * Slot called when user wishes to add a new empty layer group to the legend.
+     * All parameter are mandatory to be used to programatically nest a new group
+     * @param name name of the new group
+     * @param expand expand the group
+     * @return index of inserted group
+     */
+    int addGroup( QString name, bool expand, int parentIndex );
 
     /*!
      * Removes all groups with the given name.
@@ -230,21 +261,11 @@ class QgsLegend : public QTreeWidget
     /** called to write legend settings to project */
     void writeProject( QDomDocument & );
 
-    /**Removes the current LegendLayer*/
-    void removeCurrentLayer();
-
-    /**Removes a layer. If the layer is editable, a dialog is shown where user can select 'save', 'discard' and optionally 'cancel'. Cancel
-      is useful if a single layer is removed whereas on closing of the whole project or application, the cancel option may not be possible
-      @param ml the maplayer to remove
-      @param askCancelOnEditable gibe cancel option in the dialog for editable (and changed) layers
-      @param return false if canceled or in case of error, true else*/
-    bool removeLayer( QgsMapLayer* ml, bool askCancelOnEditable );
-
     /*!
      * Moves a layer to a group.
      * @param ml the maplayer to move
      * @param groupIndex index of group
-     * @return false if the group does not exist, false otherwise
+     * @note keep in mind that the group's index changes, if the moved layer is above the group.
      */
     void moveLayer( QgsMapLayer* ml, int groupIndex );
 
@@ -255,15 +276,22 @@ class QgsLegend : public QTreeWidget
     legend layer files*/
     void legendLayerZoom();
 
-    /***Zooms so that the pixels of the raster layer occupies exactly one screen pixel.
+    /**Zooms so that the pixels of the raster layer occupies exactly one screen pixel.
         Only works on raster layers*/
     void legendLayerZoomNative();
 
-    /**Show attribute table*/
-    void legendLayerAttributeTable();
+    /**Stretches the raster layer, if stretching is active, based on the min and max of the current extent.
+        Only workds on raster layers*/
+    void legendLayerStretchUsingCurrentExtent();
 
-    /**Shows the property dialog of the first legend layer file in a legend layer*/
-    void legendLayerShowProperties();
+    /**Updates check states when the map canvas layer set is changed */
+    void refreshCheckStates();
+
+    /** Remove selected layers */
+    void removeSelectedLayers();
+
+    /** Set CRS for selected layers */
+    void setCRSForSelectedLayers( const QgsCoordinateReferenceSystem &crs );
 
   protected:
 
@@ -307,13 +335,6 @@ class QgsLegend : public QTreeWidget
     void mouseReleaseEvent( QMouseEvent * e );
     void mouseDoubleClickEvent( QMouseEvent* e );
 
-    /**Stores the necessary information about the position of an item in the hierarchy. Afterwards,
-    this item may be moved back to the original position with resetToInitialPosition()*/
-    void storeInitialPosition( QTreeWidgetItem* li );
-
-    /**Moves an item back to the position where storeInitialPosition has been called*/
-    void resetToInitialPosition( QTreeWidgetItem* li );
-
     /**Returns the legend layer to which a map layer belongs to*/
     QgsLegendLayer* findLegendLayer( const QString& layerKey );
 
@@ -329,18 +350,23 @@ class QgsLegend : public QTreeWidget
     /**This function compares the layer order before a drag with the current layer ordering and triggers a canvas repaint if it has changed*/
     bool checkLayerOrderUpdate();
 
-    /**The target that the mouse is over when dragging */
+    // mouse is pressed
+    bool mMousePressedFlag;
+
+    // position of mouse when it is pressed at the start of a drag event.
+    QPoint mLastPressPos;
+
+    // layer our prior to move
+    QStringList mLayersPriorToMove;
+
+    // keep track of the items being dragged
+    QList< QTreeWidgetItem * > mItemsBeingMoved;
+
+    // The target that the mouse is over when dragging
     QTreeWidgetItem *mDropTarget;
 
-    enum DROP_ACTION_TYPE
-    {
-      BEFORE,
-      AFTER,
-      INTO_GROUP,
-      NO_ACTION
-    };
-    /** Set when mouse is moved over different kind of items, depending opn what they accept() */
-    DROP_ACTION_TYPE mDropAction;
+    // The action when the mouse is released
+    enum { BEFORE, INSERT, AFTER } mDropAction;
 
     /** Hide the line that indicates insertion position */
     void hideLine();
@@ -363,13 +389,16 @@ class QgsLegend : public QTreeWidget
     void handleRightClickEvent( QTreeWidgetItem* item, const QPoint& position );
     /**Removes the current legend group*/
     void legendGroupRemove();
+    /**Set the CRS of the current legend group*/
+    void legendGroupSetCRS();
     /**Removes a legend group and its layers*/
     void removeGroup( QgsLegendGroup * lg );
+    /**Removes a legend group and its layers*/
+    void setGroupCRS( QgsLegendGroup * lg, const QgsCoordinateReferenceSystem &crs );
     /**Sets all listview items to open*/
     void expandAll();
     /**Sets all listview items to closed*/
     void collapseAll();
-    /**Just for a test*/
     void handleItemChange( QTreeWidgetItem* item, int row );
     /** delegates current layer to map canvas */
     void handleCurrentItemChanged( QTreeWidgetItem* current, QTreeWidgetItem* previous );
@@ -379,6 +408,8 @@ class QgsLegend : public QTreeWidget
     void makeToTopLevelItem();
 
   private:
+    bool readXML( QgsLegendGroup *parent, const QDomNode &node );
+    bool writeXML( QList<QTreeWidgetItem *> items, QDomNode &node, QDomDocument &document );
 
     /*! Prevent the copying of QgsLegends
     * @todo See if this is really required - we may want multiple map, canvas and
@@ -393,41 +424,6 @@ class QgsLegend : public QTreeWidget
      */
     QgsLegend & operator=( QgsLegend const & );
 
-    /*!
-     * Position of mouse when it is pressed at the start of a drag event.
-     */
-    QPoint mLastPressPos;
-
-    /**True if the mouse is pressed*/
-    bool mMousePressedFlag;
-
-    /// keep track of the Item being dragged
-    QTreeWidgetItem* mItemBeingMoved;
-
-    /*!
-     * Position in the list of the item being moved as it was at the start of a drag event.
-     * An item at the top of the list will be 0 and each successive item below it
-     * will be 1,2 3 etc... regardless of nesting level.
-     */
-    int mItemBeingMovedOrigPos;
-
-    /**Information needed by 'storeInitialPosition' and 'resetToInitialPosition'*/
-    enum HIERARCHY_POSITION_TYPE
-    {
-      FIRST_ITEM,
-      FIRST_CHILD,
-      YOUNGER_SIBLING
-    };
-    HIERARCHY_POSITION_TYPE mRestoreInformation;
-    QTreeWidgetItem* mRestoreItem;
-
-    /**Stores the layer ordering before a mouse Move. After the move, this is used to
-     decide if the mapcanvas really has to be refreshed*/
-    std::deque<QString> mLayersPriorToMove;
-
-    /**Action for the legendlayer right click menu*/
-    QAction* mToggleEditingAction;
-
     /*!
      * A function to determine how far down in the list an item is (starting with one for the first Item).
      *If the item is not in the legend, -1 is returned
@@ -449,6 +445,8 @@ class QgsLegend : public QTreeWidget
     /**QgsLegend does not set the icon with/height to values lower than the minimum icon size*/
     QSize mMinimumIconSize;
 
+    bool mChanging;
+
     /** structure which holds pixmap which are used in legend */
     class QgsLegendPixmaps
     {
@@ -470,10 +468,16 @@ class QgsLegend : public QTreeWidget
     //! Widget that holds the indicator line //
     QWidget *mInsertionLine;
 
+#ifdef QGISDEBUG
+    void showItem( QString msg, QTreeWidgetItem *item );
+#endif
+
+    void updateGroupCheckStates( QTreeWidgetItem *item );
+
   signals:
     void itemMoved( QModelIndex oldIndex, QModelIndex newIndex );
 
-    void zOrderChanged( QgsLegend * lv );
+    void zOrderChanged();
 
     //! Emited whenever current (selected) layer changes
     //  the pointer to layer can be null if no layer is selected
