@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Tim Sutton   *
- *   aps02ts@macbuntu   *
+ *   Copyright (C) 2005 by Tim Sutton                                      *
+ *   aps02ts@macbuntu                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -57,47 +57,10 @@ QgsLegendGroup::~QgsLegendGroup()
 {}
 
 
-bool QgsLegendGroup::isLeafNode()
-{
-  return mLeafNodeFlag;
-}
-
-QgsLegendItem::DRAG_ACTION QgsLegendGroup::accept( LEGEND_ITEM_TYPE type )
-{
-  if ( type == LEGEND_GROUP )
-  {
-    return REORDER;
-  }
-  if ( type == LEGEND_LAYER )
-  {
-    return INSERT;
-  }
-  else
-  {
-    return NO_ACTION;
-  }
-}
-
-QgsLegendItem::DRAG_ACTION QgsLegendGroup::accept( const QgsLegendItem* li ) const
-{
-  if ( li )
-  {
-    LEGEND_ITEM_TYPE type = li->type();
-    if ( type == LEGEND_GROUP )
-    {
-      return REORDER;
-    }
-    if ( type == LEGEND_LAYER )
-    {
-      return INSERT;
-    }
-  }
-  return NO_ACTION;
-}
-
 bool QgsLegendGroup::insert( QgsLegendItem* theItem )
 {
-  if ( theItem->type() == LEGEND_LAYER )
+  if ( theItem->type() == LEGEND_GROUP ||
+       theItem->type() == LEGEND_LAYER )
   {
     // Always insert at top of list
     insertChild( 0, theItem );
@@ -107,43 +70,58 @@ bool QgsLegendGroup::insert( QgsLegendItem* theItem )
   return true;
 }
 
-std::list<QgsLegendLayer*> QgsLegendGroup::legendLayers()
+QList<QgsLegendLayer*> QgsLegendGroup::legendLayers( bool recurse )
 {
-  std::list<QgsLegendLayer*> result;
+  QList<QgsLegendLayer*> result;
   for ( int i = 0; i < childCount(); ++i )
   {
-    QgsLegendLayer* childItem = dynamic_cast<QgsLegendLayer *>( child( i ) );
-    if ( childItem )
+    QgsLegendLayer *layer = dynamic_cast<QgsLegendLayer *>( child( i ) );
+    if ( layer )
     {
-      result.push_back( childItem );
+      result.push_back( layer );
+    }
+
+    if ( !recurse )
+      continue;
+
+    QgsLegendGroup *group = dynamic_cast<QgsLegendGroup *>( child( i ) );
+    if ( group )
+    {
+      result << group->legendLayers( true );
     }
   }
   return result;
 }
 
-void QgsLegendGroup::updateCheckState()
+Qt::CheckState QgsLegendGroup::pendingCheckState()
 {
-  std::list<QgsLegendLayer*> llayers = legendLayers();
-  if ( llayers.size() == 0 )
+  QList<QgsLegendItem *> elements;
+
+  for ( int i = 0; i < childCount(); i++ )
   {
-    return;
+    QgsLegendItem *li = dynamic_cast<QgsLegendItem *>( child( i ) );
+
+    if ( !li )
+      continue;
+
+    if ( dynamic_cast<QgsLegendGroup *>( li ) || dynamic_cast<QgsLegendLayer *>( li ) )
+    {
+      elements << li;
+    }
   }
 
-  std::list<QgsLegendLayer*>::iterator iter = llayers.begin();
-  Qt::CheckState theState = ( *iter )->checkState( 0 );
-  for ( ; iter != llayers.end(); ++iter )
+  if ( elements.isEmpty() )
+    return Qt::PartiallyChecked;
+
+  Qt::CheckState theState = elements[0]->checkState( 0 );
+  foreach( QgsLegendItem * li, elements )
   {
-    if ( theState != ( *iter )->checkState( 0 ) )
+    if ( theState != li->checkState( 0 ) )
     {
       theState = Qt::PartiallyChecked;
       break;
     }
   }
 
-  if ( theState != checkState( 0 ) )
-  {
-    treeWidget()->blockSignals( true );
-    setCheckState( 0, theState );
-    treeWidget()->blockSignals( false );
-  }
+  return theState;
 }
