@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Tim Sutton   *
- *   aps02ts@macbuntu   *
+ *   Copyright (C) 2005 by Tim Sutton                                      *
+ *   aps02ts@macbuntu                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -37,14 +37,11 @@ class QgsLegendGroup : public QgsLegendItem
     QgsLegendGroup( QString name );
     ~QgsLegendGroup();
 
-    QgsLegendItem::DRAG_ACTION accept( LEGEND_ITEM_TYPE type );
-    QgsLegendItem::DRAG_ACTION accept( const QgsLegendItem* li ) const;
-    bool isLeafNode();
     bool insert( QgsLegendItem* theItem );
-    /**Returns all legend layers under this group*/
-    std::list<QgsLegendLayer*> legendLayers();
-    /**Goes through all the legendlayers and sets check state to checked/partially checked/unchecked*/
-    void updateCheckState();
+    /**Returns all legend layers under this group (including those of subgroups by default)*/
+    QList<QgsLegendLayer*> legendLayers( bool recurse = true );
+
+    Qt::CheckState pendingCheckState();
 };
 
 #endif
