@@ -25,18 +25,14 @@
 #include "qgslegendsymbologyitem.h"
 #include "qgslogger.h"
 
-#include "qgsapplication.h"
 #include "qgsfield.h"
+#include "qgsmapcanvasmap.h"
 #include "qgsmaplayerregistry.h"
 #include "qgsrasterlayer.h"
 #include "qgsrenderer.h"
 #include "qgssymbol.h"
 #include "qgsvectorlayer.h"
 #include "qgsvectordataprovider.h"
-#include "qgsvectorfilewriter.h"
-#include "qgsgenericprojectionselector.h"
-#include "qgsattributetabledialog.h"
-#include "qgsencodingfiledialog.h"
 
 #include "qgsrendererv2.h"
 #include "qgssymbolv2.h"
@@ -53,15 +49,28 @@
 #include <QMessageBox>
 #include <QPainter>
 #include <QSettings>
+#include <QFileDialog>
+#include <QProgressDialog>
 
 
 QgsLegendLayer::QgsLegendLayer( QgsMapLayer* layer )
     : QgsLegendItem( ),
-    mLyr( layer )
+    mLyr( layer ), mShowFeatureCount( false )
 {
   mType = LEGEND_LAYER;
-  setFlags( Qt::ItemIsEditable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsSelectable );
+
+  Qt::ItemFlags flags = Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable;
+
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer );
+  if ( !vlayer || vlayer->hasGeometryType() )
+  {
+    flags |= Qt::ItemIsUserCheckable;
+  }
+
+  setFlags( flags );
+
   setCheckState( 0, Qt::Checked );
+
   setText( 0, layer->name() );
   setupFont();
 
@@ -75,7 +84,7 @@ QgsLegendLayer::QgsLegendLayer( QgsMapLayer* layer )
   mLyr.setVisible( visible );
 
   // not in overview by default
-  mLyr.setInOverview( FALSE );
+  mLyr.setInOverview( false );
 
   // setup connections that will update the layer icons
   if ( qobject_cast<QgsVectorLayer *>( layer ) )
@@ -83,6 +92,11 @@ QgsLegendLayer::QgsLegendLayer( QgsMapLayer* layer )
     QgsDebugMsg( "Connecting signals for updating icons, layer " + layer->name() );
     connect( layer, SIGNAL( editingStarted() ), this, SLOT( updateIcon() ) );
     connect( layer, SIGNAL( editingStopped() ), this, SLOT( updateIcon() ) );
+    connect( layer, SIGNAL( layerModified( bool ) ), this, SLOT( updateAfterLayerModification( bool ) ) );
+  }
+  if ( qobject_cast<QgsRasterLayer *>( layer ) )
+  {
+    connect( layer, SIGNAL( dataChanged() ), this, SLOT( updateAfterLayerModification() ) );
   }
   connect( layer, SIGNAL( layerNameChanged() ), this, SLOT( layerNameChanged() ) );
 
@@ -95,6 +109,14 @@ QgsLegendLayer::~QgsLegendLayer()
   mType = LEGEND_LAYER;
 }
 
+void QgsLegendLayer::setCheckState( int column, Qt::CheckState state )
+{
+  if ( flags() & Qt::ItemIsUserCheckable )
+  {
+    QTreeWidgetItem::setCheckState( column, state );
+  }
+}
+
 void QgsLegendLayer::setupFont() //private method
 {
   QFont myFont = font( 0 );
@@ -102,48 +124,6 @@ void QgsLegendLayer::setupFont() //private method
   setFont( 0, myFont );
 }
 
-bool QgsLegendLayer::isLeafNode()
-{
-  return false;
-}
-
-QgsLegendItem::DRAG_ACTION QgsLegendLayer::accept( LEGEND_ITEM_TYPE type )
-{
-  if ( type == LEGEND_LAYER || type == LEGEND_GROUP )
-  {
-    return REORDER;
-  }
-  else
-  {
-    return NO_ACTION;
-  }
-}
-
-QgsLegendItem::DRAG_ACTION QgsLegendLayer::accept( const QgsLegendItem* li ) const
-{
-  if ( li && li != this )
-  {
-    LEGEND_ITEM_TYPE type = li->type();
-    if ( type == LEGEND_LAYER )
-    {
-      //if(parent() == li->parent())
-      //{
-      return REORDER;
-      //}
-    }
-    else if ( type == LEGEND_GROUP )
-    {
-      //only parent legend layers can change positions with groups
-      if ( parent() == 0 )
-      {
-        return REORDER;
-      }
-    }
-  }
-  return NO_ACTION;
-}
-
-
 QgsMapLayer* QgsLegendLayer::layer()
 {
   return mLyr.layer();
@@ -165,11 +145,13 @@ void QgsLegendLayer::refreshSymbology( const QString& key, double widthScale )
     else
       vectorLayerSymbology( vlayer, widthScale ); // get and change symbology
   }
-  else // RASTER
+  else if ( theMapLayer->type() == QgsMapLayer::RasterLayer ) // RASTER
   {
     QgsRasterLayer* rlayer = qobject_cast<QgsRasterLayer *>( theMapLayer );
     rasterLayerSymbology( rlayer ); // get and change symbology
   }
+
+  updateIcon();
 }
 
 void QgsLegendLayer::changeSymbologySettings( const QgsMapLayer* theMapLayer,
@@ -208,74 +190,99 @@ void QgsLegendLayer::changeSymbologySettings( const QgsMapLayer* theMapLayer,
 
 
 
-void QgsLegendLayer::vectorLayerSymbology( const QgsVectorLayer* layer, double widthScale )
+void QgsLegendLayer::vectorLayerSymbology( QgsVectorLayer* layer, double widthScale )
 {
-  SymbologyList itemList;
-
-  //add the new items
-  QString lw, uv, label;
-  const QgsRenderer* renderer = layer->renderer();
-  const QList<QgsSymbol*> sym = renderer->symbols();
-
-  //create an item for each classification field (only one for most renderers)
-  QSettings settings;
-  if ( settings.value( "/qgis/showLegendClassifiers", false ).toBool() )
+  if ( !layer )
   {
-    if ( renderer->needsAttributes() )
+    return;
+  }
+
+  SymbologyList itemList;
+  if ( layer->hasGeometryType() )
+  {
+    //add the new items
+    QString lw, uv, label;
+    const QgsRenderer* renderer = layer->renderer();
+    const QList<QgsSymbol*> sym = renderer->symbols();
+
+    //create an item for each classification field (only one for most renderers)
+    QSettings settings;
+    if ( settings.value( "/qgis/showLegendClassifiers", false ).toBool() )
     {
-      QgsAttributeList classfieldlist = renderer->classificationAttributes();
-      const QgsFieldMap& fields = layer->pendingFields();
-      for ( QgsAttributeList::iterator it = classfieldlist.begin(); it != classfieldlist.end(); ++it )
+      if ( renderer->needsAttributes() )
       {
-        QString classfieldname = layer->attributeAlias( *it );
-        if ( classfieldname.isEmpty() )
+        QgsAttributeList classfieldlist = renderer->classificationAttributes();
+        const QgsFieldMap& fields = layer->pendingFields();
+        for ( QgsAttributeList::iterator it = classfieldlist.begin(); it != classfieldlist.end(); ++it )
         {
-          classfieldname = fields[*it].name();
+          QString classfieldname = layer->attributeAlias( *it );
+          if ( classfieldname.isEmpty() )
+          {
+            classfieldname = fields[*it].name();
+          }
+          itemList.append( qMakePair( classfieldname, QPixmap() ) );
         }
-        itemList.append( qMakePair( classfieldname, QPixmap() ) );
       }
     }
+
+    QMap< QgsSymbol*, int > featureCountMap;
+    if ( mShowFeatureCount )
+    {
+      updateItemListCount( layer, sym, featureCountMap );
+    }
+
+    for ( QList<QgsSymbol*>::const_iterator it = sym.begin(); it != sym.end(); ++it )
+    {
+      QImage img;
+      if (( *it )->type() == QGis::Point )
+      {
+        img = ( *it )->getPointSymbolAsImage( widthScale );
+      }
+      else if (( *it )->type() == QGis::Line )
+      {
+        img = ( *it )->getLineSymbolAsImage();
+      }
+      else  if (( *it )->type() == QGis::Polygon )
+      {
+        img = ( *it )->getPolygonSymbolAsImage();
+      }
+      else
+      {
+        // must be a layer without geometry then
+      }
+
+      QString values;
+      lw = ( *it )->lowerValue();
+      if ( !lw.isEmpty() )
+      {
+        values += lw;
+      }
+      uv = ( *it )->upperValue();
+      if ( !uv.isEmpty() && lw != uv )
+      {
+        values += " - ";
+        values += uv;
+      }
+      label = ( *it )->label();
+      if ( !label.isEmpty() )
+      {
+        values += " ";
+        values += label;
+      }
+
+      if ( mShowFeatureCount )
+      {
+        int fCount = featureCountMap[*it];
+        if ( fCount >= 0 )
+        {
+          values += ( " [" + QString::number( fCount ) + "]" );
+        }
+      }
+
+      QPixmap pix = QPixmap::fromImage( img ); // convert to pixmap
+      itemList.append( qMakePair( values, pix ) );
+    }
   }
-
-  for ( QList<QgsSymbol*>::const_iterator it = sym.begin(); it != sym.end(); ++it )
-  {
-    QImage img;
-    if (( *it )->type() == QGis::Point )
-    {
-      img = ( *it )->getPointSymbolAsImage( widthScale );
-    }
-    else if (( *it )->type() == QGis::Line )
-    {
-      img = ( *it )->getLineSymbolAsImage();
-    }
-    else //polygon
-    {
-      img = ( *it )->getPolygonSymbolAsImage();
-    }
-
-    QString values;
-    lw = ( *it )->lowerValue();
-    if ( !lw.isEmpty() )
-    {
-      values += lw;
-    }
-    uv = ( *it )->upperValue();
-    if ( !uv.isEmpty() && lw != uv )
-    {
-      values += " - ";
-      values += uv;
-    }
-    label = ( *it )->label();
-    if ( !label.isEmpty() )
-    {
-      values += " ";
-      values += label;
-    }
-
-    QPixmap pix = QPixmap::fromImage( img ); // convert to pixmap
-    itemList.append( qMakePair( values, pix ) );
-  }
-
   changeSymbologySettings( layer, itemList );
 }
 
@@ -284,14 +291,17 @@ void QgsLegendLayer::vectorLayerSymbologyV2( QgsVectorLayer* layer )
 {
   QSize iconSize( 16, 16 );
 
-#if 0 // unused
-  QSettings settings;
-  bool showClassifiers = settings.value( "/qgis/showLegendClassifiers", false ).toBool();
-#endif
+  QgsFeatureRendererV2* renderer = layer->rendererV2();
+  if ( renderer )
+  {
+    SymbologyList itemList = renderer->legendSymbologyItems( iconSize );
+    if ( mShowFeatureCount )
+    {
+      updateItemListCountV2( itemList, layer );
+    }
 
-  SymbologyList itemList = layer->rendererV2()->legendSymbologyItems( iconSize );
-
-  changeSymbologySettings( layer, itemList );
+    changeSymbologySettings( layer, itemList );
+  }
 }
 
 void QgsLegendLayer::rasterLayerSymbology( QgsRasterLayer* layer )
@@ -301,8 +311,6 @@ void QgsLegendLayer::rasterLayerSymbology( QgsRasterLayer* layer )
   itemList.append( qMakePair( QString(), legendpixmap ) );
 
   changeSymbologySettings( layer, itemList );
-
-  updateIcon();
 }
 
 void QgsLegendLayer::updateIcon()
@@ -339,7 +347,16 @@ void QgsLegendLayer::updateIcon()
   // TODO: projection error icon?
 
   QIcon theIcon( newIcon );
+  QgsLegend* l = legend();
+  if ( l )
+  {
+    l->blockSignals( true ); //prevent unnecessary canvas redraw
+  }
   setIcon( 0, theIcon );
+  if ( l )
+  {
+    l->blockSignals( false );
+  }
 }
 
 QPixmap QgsLegendLayer::getOriginalPixmap()
@@ -362,16 +379,26 @@ QPixmap QgsLegendLayer::getOriginalPixmap()
         case QGis::Polygon:
           return QgisApp::getThemePixmap( "/mIconPolygonLayer.png" );
           break;
+        case QGis::NoGeometry:
+          return QgisApp::getThemePixmap( "/mIconTableLayer.png" );
         default:
           return QgisApp::getThemePixmap( "/mIconLayer.png" );
       }
     }
     else if ( theLayer->type() == QgsMapLayer::RasterLayer )
     {
-      QgsRasterLayer* rlayer = qobject_cast<QgsRasterLayer *>( theLayer );
-      QPixmap myPixmap( 32, 32 );
-      rlayer->thumbnailAsPixmap( &myPixmap );
-      return myPixmap;
+      QSettings s;
+      if ( s.value( "/qgis/createRasterLegendIcons", true ).toBool() )
+      {
+        QgsRasterLayer* rlayer = qobject_cast<QgsRasterLayer *>( theLayer );
+        QPixmap myPixmap( 32, 32 );
+        rlayer->thumbnailAsPixmap( &myPixmap );
+        return myPixmap;
+      }
+      else
+      {
+        return QPixmap();
+      }
     }
   }
 
@@ -379,29 +406,40 @@ QPixmap QgsLegendLayer::getOriginalPixmap()
   return QgisApp::getThemePixmap( "/mIconLayer.png" );
 }
 
-void QgsLegendLayer::addToPopupMenu( QMenu& theMenu, QAction* toggleEditingAction )
+void QgsLegendLayer::addToPopupMenu( QMenu& theMenu )
 {
-
-  QgsMapLayer* lyr = layer();
+  QgsMapLayer *lyr = layer();
+  QAction *toggleEditingAction = QgisApp::instance()->actionToggleEditing();
 
   // zoom to layer extent
   theMenu.addAction( QgisApp::getThemeIcon( "/mActionZoomToLayer.png" ),
-                     tr( "&Zoom to layer extent" ), legend(), SLOT( legendLayerZoom() ) );
+                     tr( "&Zoom to Layer Extent" ), legend(), SLOT( legendLayerZoom() ) );
   if ( lyr->type() == QgsMapLayer::RasterLayer )
   {
-    theMenu.addAction( tr( "&Zoom to best scale (100%)" ), legend(), SLOT( legendLayerZoomNative() ) );
+    theMenu.addAction( tr( "&Zoom to Best Scale (100%)" ), legend(), SLOT( legendLayerZoomNative() ) );
+
+    QgsRasterLayer *rasterLayer =  qobject_cast<QgsRasterLayer *>( lyr );
+    if ( rasterLayer && rasterLayer->rasterType() != QgsRasterLayer::Palette )
+    {
+      theMenu.addAction( tr( "&Stretch Using Current Extent" ), legend(), SLOT( legendLayerStretchUsingCurrentExtent() ) );
+    }
   }
 
   // show in overview
-  QAction* showInOverviewAction = theMenu.addAction( tr( "&Show in overview" ), this, SLOT( showInOverview() ) );
+  QAction* showInOverviewAction = theMenu.addAction( tr( "&Show in Overview" ), this, SLOT( showInOverview() ) );
   showInOverviewAction->setCheckable( true );
   showInOverviewAction->blockSignals( true );
   showInOverviewAction->setChecked( mLyr.isInOverview() );
   showInOverviewAction->blockSignals( false );
 
   // remove from canvas
-  theMenu.addAction( QgisApp::getThemeIcon( "/mActionRemove.png" ),
-                     tr( "&Remove" ), legend(), SLOT( removeCurrentLayer() ) );
+  theMenu.addAction( QgisApp::getThemeIcon( "/mActionRemoveLayer.png" ), tr( "&Remove" ), QgisApp::instance(), SLOT( removeLayer() ) );
+
+  // set layer crs
+  theMenu.addAction( QgisApp::getThemeIcon( "/mActionSetCRS.png" ), tr( "&Set Layer CRS" ), QgisApp::instance(), SLOT( setLayerCRS() ) );
+
+  // assign layer crs to project
+  theMenu.addAction( QgisApp::getThemeIcon( "/mActionSetProjectCRS.png" ), tr( "Set &Project CRS from Layer" ), QgisApp::instance(), SLOT( setProjectCRSFromLayer() ) );
 
   theMenu.addSeparator();
 
@@ -410,7 +448,8 @@ void QgsLegendLayer::addToPopupMenu( QMenu& theMenu, QAction* toggleEditingActio
     QgsVectorLayer* vlayer = qobject_cast<QgsVectorLayer *>( lyr );
 
     // attribute table
-    theMenu.addAction( tr( "&Open attribute table" ), this, SLOT( table() ) );
+    theMenu.addAction( QgisApp::getThemeIcon( "/mActionOpenTable.png" ), tr( "&Open Attribute Table" ),
+                       QgisApp::instance(), SLOT( attributeTable() ) );
 
     // allow editing
     int cap = vlayer->dataProvider()->capabilities();
@@ -423,41 +462,31 @@ void QgsLegendLayer::addToPopupMenu( QMenu& theMenu, QAction* toggleEditingActio
       }
     }
 
-    // save as shapefile
-    theMenu.addAction( tr( "Save as shapefile..." ), this, SLOT( saveAsShapefile() ) );
+    // save as vector file
+    theMenu.addAction( tr( "Save As..." ), QgisApp::instance(), SLOT( saveAsVectorFile() ) );
 
-    // save selection as shapefile
-    QAction* saveSelectionAction = theMenu.addAction( tr( "Save selection as shapefile..." ), this, SLOT( saveSelectionAsShapefile() ) );
+    // save selection as vector file
+    QAction* saveSelectionAsAction = theMenu.addAction( tr( "Save Selection As..." ), QgisApp::instance(), SLOT( saveSelectionAsVectorFile() ) );
     if ( vlayer->selectedFeatureCount() == 0 )
     {
-      saveSelectionAction->setEnabled( false );
+      saveSelectionAsAction->setEnabled( false );
     }
 
+    if ( !vlayer->isEditable() && vlayer->dataProvider()->supportsSubsetString() && vlayer->vectorJoins().isEmpty() )
+      theMenu.addAction( tr( "&Query..." ), QgisApp::instance(), SLOT( layerSubsetString() ) );
+
+    //show number of features in legend if requested
+    QAction* showNFeaturesAction = new QAction( tr( "Show Feature Count" ), &theMenu );
+    showNFeaturesAction->setCheckable( true );
+    showNFeaturesAction->setChecked( mShowFeatureCount );
+    QObject::connect( showNFeaturesAction, SIGNAL( toggled( bool ) ), this, SLOT( setShowFeatureCount( bool ) ) );
+    theMenu.addAction( showNFeaturesAction );
     theMenu.addSeparator();
   }
 
   // properties goes on bottom of menu for consistency with normal ui standards
   // e.g. kde stuff
-  theMenu.addAction( tr( "&Properties" ), legend(), SLOT( legendLayerShowProperties() ) );
-
-}
-
-void QgsLegendLayer::table()
-{
-  QgsVectorLayer * myLayer = qobject_cast<QgsVectorLayer *>( mLyr.layer() );
-  QgsAttributeTableDialog *mDialog = new QgsAttributeTableDialog( myLayer );
-  mDialog->show();
-  // the dialog will be deleted by itself on close
-}
-
-void QgsLegendLayer::saveAsShapefile()
-{
-  saveAsShapefileGeneral( FALSE );
-}
-
-void QgsLegendLayer::saveSelectionAsShapefile()
-{
-  saveAsShapefileGeneral( TRUE );
+  theMenu.addAction( tr( "&Properties" ), QgisApp::instance(), SLOT( layerProperties() ) );
 }
 
 //////////
@@ -492,125 +521,6 @@ void QgsLegendLayer::showInOverview()
   legend()->updateOverview();
 }
 
-
-
-
-void QgsLegendLayer::saveAsShapefileGeneral( bool saveOnlySelection )
-{
-  QgsCoordinateReferenceSystem destCRS;
-
-  if ( mLyr.layer()->type() != QgsMapLayer::VectorLayer )
-    return;
-
-  QgsVectorLayer* vlayer = qobject_cast<QgsVectorLayer *>( mLyr.layer() );
-
-  // get a name for the shapefile
-  // Get a file to process, starting at the current directory
-  QSettings settings;
-  QString filter =  QString( "Shapefiles (*.shp)" );
-  QString dirName = settings.value( "/UI/lastShapefileDir", "." ).toString();
-
-  QgsEncodingFileDialog* openFileDialog = new QgsEncodingFileDialog( 0,
-      tr( "Save layer as..." ),
-      dirName,
-      filter,
-      QString( "" ) );
-  openFileDialog->setAcceptMode( QFileDialog::AcceptSave );
-
-  // allow for selection of more than one file
-  //openFileDialog->setMode(QFileDialog::AnyFile);
-
-  if ( openFileDialog->exec() != QDialog::Accepted )
-    return;
-
-
-  QString encoding = openFileDialog->encoding();
-  QString shapefileName = openFileDialog->selectedFiles().first();
-  settings.setValue( "/UI/lastShapefileDir", QFileInfo( shapefileName ).absolutePath() );
-
-
-  if ( shapefileName.isNull() )
-    return;
-
-  // add the extension if not present
-  if ( !shapefileName.endsWith( ".shp", Qt::CaseInsensitive ) )
-  {
-    shapefileName += ".shp";
-  }
-
-  destCRS = vlayer->srs();
-  // Find out if we have projections enabled or not
-  if ( QgisApp::instance()->mapCanvas()->mapRenderer()->hasCrsTransformEnabled() )
-  {
-    destCRS = QgisApp::instance()->mapCanvas()->mapRenderer()->destinationSrs();
-  }
-
-  QgsGenericProjectionSelector * mySelector = new QgsGenericProjectionSelector();
-  mySelector->setSelectedCrsId( destCRS.srsid() );
-  mySelector->setMessage( tr( "Select the coordinate reference system for the saved shapefile. "
-                              "The data points will be transformed from the layer coordinate reference system." ) );
-
-  if ( mySelector->exec() )
-  {
-    QgsCoordinateReferenceSystem srs( mySelector->selectedCrsId(), QgsCoordinateReferenceSystem::InternalCrsId );
-    destCRS = srs;
-    //   destCRS->createFromId(mySelector->selectedCrsId(), QgsCoordinateReferenceSystem::InternalCrsId)
-  }
-  else
-  {
-    // Aborted CS selection, don't save.
-    delete mySelector;
-    return;
-  }
-
-  delete mySelector;
-
-  // overwrite the file - user will already have been prompted
-  // to verify they want to overwrite by the file dialog above
-  // might not even exists in the given case.
-  QgsVectorFileWriter::deleteShapeFile( shapefileName );
-
-  // ok if the file existed it should be deleted now so we can continue...
-  QApplication::setOverrideCursor( Qt::WaitCursor );
-
-  QgsVectorFileWriter::WriterError error;
-  error = QgsVectorFileWriter::writeAsShapefile( vlayer, shapefileName, encoding, &destCRS, saveOnlySelection );
-
-  QApplication::restoreOverrideCursor();
-
-  switch ( error )
-  {
-    case QgsVectorFileWriter::NoError:
-      QMessageBox::information( 0, tr( "Saving done" ), tr( "Export to Shapefile has been completed" ) );
-      break;
-
-    case QgsVectorFileWriter::ErrDriverNotFound:
-      QMessageBox::warning( 0, tr( "Driver not found" ), tr( "ESRI Shapefile driver is not available" ) );
-      break;
-
-    case QgsVectorFileWriter::ErrCreateDataSource:
-      QMessageBox::warning( 0, tr( "Error creating shapefile" ),
-                            tr( "The shapefile could not be created (%1)" ).arg( shapefileName ) );
-      break;
-
-    case QgsVectorFileWriter::ErrCreateLayer:
-      QMessageBox::warning( 0, tr( "Error" ), tr( "Layer creation failed" ) );
-      break;
-
-    case QgsVectorFileWriter::ErrAttributeTypeUnsupported:
-      QMessageBox::warning( 0, tr( "Error" ),
-                            tr( "Layer attribute table contains unsupported datatype(s)" ) );
-      break;
-
-    case QgsVectorFileWriter::ErrAttributeCreationFailed:
-      QMessageBox::warning( 0, tr( "Error" ),
-                            tr( "Creation of an attribute failed" ) );
-      break;
-  }
-}
-
-
-
 QString QgsLegendLayer::nameFromLayer( QgsMapLayer* layer )
 {
   QString sourcename = layer->publicSource(); //todo: move this duplicated code into a new function
@@ -628,7 +538,6 @@ QString QgsLegendLayer::nameFromLayer( QgsMapLayer* layer )
   return sourcename;
 }
 
-
 QgsMapCanvasLayer& QgsLegendLayer::canvasLayer()
 {
   return mLyr;
@@ -640,3 +549,160 @@ void QgsLegendLayer::layerNameChanged()
   setText( 0, name );
 }
 
+void QgsLegendLayer::updateAfterLayerModification()
+{
+  updateAfterLayerModification( false );
+}
+void QgsLegendLayer::updateAfterLayerModification( bool onlyGeomChanged )
+{
+  if ( onlyGeomChanged )
+  {
+    return;
+  }
+
+  double widthScale = 1.0;
+  QgsMapCanvas* canvas = QgisApp::instance()->mapCanvas();
+  if ( canvas && canvas->map() )
+  {
+    widthScale = canvas->map()->paintDevice().logicalDpiX() / 25.4;
+  }
+  refreshSymbology( mLyr.layer()->id(), widthScale );
+}
+
+void QgsLegendLayer::updateItemListCountV2( SymbologyList& itemList, QgsVectorLayer* layer )
+{
+  if ( !layer )
+  {
+    return;
+  }
+
+  QgsFeatureRendererV2* renderer = layer->rendererV2();
+  if ( !renderer )
+  {
+    return;
+  }
+  QgsRenderContext dummyContext;
+  renderer->startRender( dummyContext, layer );
+
+  //create map holding the symbol count
+  QMap< QgsSymbolV2*, int > mSymbolCountMap;
+  QgsLegendSymbolList symbolList = renderer->legendSymbolItems();
+  QgsLegendSymbolList::const_iterator symbolIt = symbolList.constBegin();
+  for ( ; symbolIt != symbolList.constEnd(); ++symbolIt )
+  {
+    mSymbolCountMap.insert( symbolIt->second, 0 );
+  }
+
+  //go through all features and count the number of occurrences
+  int nFeatures = layer->pendingFeatureCount();
+  QProgressDialog p( tr( "Updating feature count for layer " ) + layer->name(), tr( "Abort" ), 0, nFeatures );
+  p.setWindowModality( Qt::WindowModal );
+  int featuresCounted = 0;
+
+
+  layer->select( layer->pendingAllAttributesList(), QgsRectangle(), false, false );
+  QgsFeature f;
+  QgsSymbolV2* currentSymbol = 0;
+
+  while ( layer->nextFeature( f ) )
+  {
+    currentSymbol = renderer->symbolForFeature( f );
+    mSymbolCountMap[currentSymbol] += 1;
+    ++featuresCounted;
+    if ( featuresCounted % 50 == 0 )
+    {
+      if ( featuresCounted > nFeatures ) //sometimes the feature count is not correct
+      {
+        p.setMaximum( 0 );
+      }
+      p.setValue( featuresCounted );
+      if ( p.wasCanceled() )
+      {
+        return;
+      }
+    }
+  }
+  p.setValue( nFeatures );
+
+  QMap<QString, QPixmap> itemMap;
+  SymbologyList::const_iterator symbologyIt = itemList.constBegin();
+  for ( ; symbologyIt != itemList.constEnd(); ++ symbologyIt )
+  {
+    itemMap.insert( symbologyIt->first, symbologyIt->second );
+  }
+  itemList.clear();
+
+  //
+  symbolIt = symbolList.constBegin();
+  for ( ; symbolIt != symbolList.constEnd(); ++symbolIt )
+  {
+    itemList.push_back( qMakePair( symbolIt->first + " [" + QString::number( mSymbolCountMap[symbolIt->second] ) + "]", itemMap[symbolIt->first] ) );
+  }
+}
+
+void QgsLegendLayer::updateItemListCount( QgsVectorLayer* layer, const QList<QgsSymbol*>& sym, QMap< QgsSymbol*, int >& featureCountMap )
+{
+  featureCountMap.clear();
+  QList<QgsSymbol*>::const_iterator symbolIt = sym.constBegin();
+  for ( ; symbolIt != sym.constEnd(); ++symbolIt )
+  {
+    featureCountMap.insert( *symbolIt, 0 );
+  }
+
+  QgsRenderer* renderer = const_cast<QgsRenderer*>( layer->renderer() );
+  if ( !renderer )
+  {
+    return;
+  }
+
+  //go through all features and count the number of occurrences
+  int nFeatures = layer->pendingFeatureCount();
+  QProgressDialog p( tr( "Updating feature count for layer " ) + layer->name(), tr( "Abort" ), 0, nFeatures );
+  p.setWindowModality( Qt::WindowModal );
+  int featuresCounted = 0;
+
+  layer->select( layer->pendingAllAttributesList(), QgsRectangle(), false, false );
+  QgsFeature f;
+  QgsSymbol* currentSymbol = 0;
+
+  while ( layer->nextFeature( f ) )
+  {
+    currentSymbol = renderer->symbolForFeature( &f );
+    if ( currentSymbol )
+    {
+      featureCountMap[currentSymbol] += 1;
+    }
+    ++featuresCounted;
+
+    if ( featuresCounted % 50 == 0 )
+    {
+      if ( featuresCounted > nFeatures ) //sometimes the feature count is not correct
+      {
+        p.setMaximum( 0 );
+      }
+      p.setValue( featuresCounted );
+      if ( p.wasCanceled() ) //set all entries to -1 (= invalid)
+      {
+        QMap< QgsSymbol*, int >::iterator cIt = featureCountMap.begin();
+        for ( ; cIt != featureCountMap.end(); ++cIt )
+        {
+          cIt.value() = -1;
+        }
+        return;
+      }
+    }
+  }
+  p.setValue( nFeatures );
+}
+
+void QgsLegendLayer::setShowFeatureCount( bool show, bool update )
+{
+  if ( show != mShowFeatureCount )
+  {
+    mShowFeatureCount = show;
+    if ( update )
+    {
+      updateAfterLayerModification( false );
+    }
+  }
+}
