@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Tim Sutton   *
- *   aps02ts@macbuntu   *
+ *   Copyright (C) 2005 by Tim Sutton                                      *
+ *   aps02ts@macbuntu                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -29,6 +29,7 @@ class QgsLegendLayer;
 class QgsLegendPropertyGroup;
 class QgsMapLayer;
 class QgsRasterLayer;
+class QgsSymbol;
 class QgsVectorLayer;
 
 class QTreeWidget;
@@ -48,9 +49,6 @@ class QgsLegendLayer : public QgsLegendItem
     QgsLegendLayer( QgsMapLayer* layer );
     ~QgsLegendLayer();
 
-    bool isLeafNode();
-    QgsLegendItem::DRAG_ACTION accept( LEGEND_ITEM_TYPE type );
-    QgsLegendItem::DRAG_ACTION accept( const QgsLegendItem* li ) const;
     /**Returns the map layer associated the item*/
     QgsMapLayer* layer();
     QgsMapCanvasLayer& canvasLayer();
@@ -61,14 +59,14 @@ class QgsLegendLayer : public QgsLegendItem
     void refreshSymbology( const QString& key, double widthScale = 1.0 );
 
     /** called to add appropriate menu items to legend's popup menu */
-    void addToPopupMenu( QMenu& theMenu, QAction* toggleEditingAction );
+    void addToPopupMenu( QMenu& theMenu );
 
     /** Set layer to be visible in canvas */
-    void setVisible( bool visible = TRUE );
+    void setVisible( bool visible = true );
     /** Find out whether the layer is visible */
     bool isVisible();
 
-    void setInOverview( bool isInOverview = TRUE );
+    void setInOverview( bool isInOverview = true );
     /**Determines whether there are layers in overview*/
     bool isInOverview();
 
@@ -76,27 +74,31 @@ class QgsLegendLayer : public QgsLegendItem
      the name can be passed to the constructor of QgsLegendLayer*/
     static QString nameFromLayer( QgsMapLayer* layer );
 
+    /** set check state, but only if user checkable */
+    void setCheckState( int column, Qt::CheckState state );
+
   public slots:
 
     /**Toggle show in overview*/
     void showInOverview();
 
-    /**Show layer attribute table*/
-    void table();
-
-    void saveAsShapefile();
-    void saveSelectionAsShapefile();
-
     /**update the layer's icon to show whether is in editing mode or in overview */
     void updateIcon();
 
     /**Layer name has changed - set it also in legend*/
     void layerNameChanged();
 
+    /**Update symbology (e.g. to update feature count in the legend after editing operations)*/
+    void updateAfterLayerModification( bool onlyGeomChanged );
+    void updateAfterLayerModification();
+
+    void setShowFeatureCount( bool show, bool update = true );
+    bool showFeatureCount() const { return mShowFeatureCount; }
+
   protected:
 
     /** Prepare and change symbology for vector layer */
-    void vectorLayerSymbology( const QgsVectorLayer* mapLayer, double widthScale = 1.0 );
+    void vectorLayerSymbology( QgsVectorLayer* mapLayer, double widthScale = 1.0 );
 
     void vectorLayerSymbologyV2( QgsVectorLayer* vlayer );
 
@@ -106,10 +108,12 @@ class QgsLegendLayer : public QgsLegendItem
     /** Removes the symbology items of a layer and adds new ones. */
     void changeSymbologySettings( const QgsMapLayer* mapLayer, const SymbologyList& newSymbologyItems );
 
-    QPixmap getOriginalPixmap();
+    /**Adds feature counts to the symbology items (for symbology v2)*/
+    void updateItemListCountV2( SymbologyList& itemList, QgsVectorLayer* layer );
+    /**Calculates feature count for the individual symbols (old symbology)*/
+    void updateItemListCount( QgsVectorLayer* layer, const QList<QgsSymbol*>& sym, QMap< QgsSymbol*, int >& featureCountMap );
 
-    /**Save as shapefile (called from saveAsShapefile and saveSelectionAsShapefile)*/
-    void saveAsShapefileGeneral( bool saveOnlySelection );
+    QPixmap getOriginalPixmap();
 
   private:
     /** Helper method to make the font bold from all ctors.
@@ -122,6 +126,9 @@ class QgsLegendLayer : public QgsLegendItem
 
     /** layer identified by its layer id */
     QgsMapCanvasLayer mLyr;
+
+    /**True if number of features per legend class should is shown in the legend items*/
+    bool mShowFeatureCount;
 };
 
 #endif
