@@ -14,7 +14,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id$ */
+
 
 //qt includes
 #include <QBitmap>
@@ -67,12 +67,18 @@ typedef SInt32 SRefCon;
 #include "qgsmapcanvas.h"
 #include "qgsapplication.h"
 #include <qgsconfig.h>
-#include <qgssvnversion.h>
+#include <qgsversion.h>
 #include "qgsexception.h"
 #include "qgsproject.h"
 #include "qgsrectangle.h"
 #include "qgslogger.h"
 
+// (if Windows/Mac, use icon from resource)
+#if ! defined(Q_WS_WIN) && ! defined(Q_WS_MAC)
+#include "../../images/themes/default/qgis.xpm" // Linux
+#include <QIcon>
+#endif
+
 static const char * const ident_ = "$Id$";
 
 /** print usage text
@@ -80,7 +86,7 @@ static const char * const ident_ = "$Id$";
 void usage( std::string const & appName )
 {
   std::cerr << "Quantum GIS - " << VERSION << " '" << RELEASE_NAME << "' ("
-            << QGSSVNVERSION << ")\n"
+            << QGis::QGIS_DEV_VERSION << ")\n"
             << "Quantum GIS (QGIS) is a viewer for spatial data sets, including\n"
             << "raster and vector data.\n"
             << "Usage: " << appName <<  " [options] [FILES]\n"
@@ -92,6 +98,9 @@ void usage( std::string const & appName )
             << "\t[--project projectfile]\tload the given QGIS project\n"
             << "\t[--extent xmin,ymin,xmax,ymax]\tset initial map extent\n"
             << "\t[--nologo]\thide splash screen\n"
+            << "\t[--noplugins]\tdon't restore plugins on startup\n"
+            << "\t[--optionspath path]\tuse the given QSettings path\n"
+            << "\t[--configpath path]\tuse the given path for all user configuration\n"
             << "\t[--help]\t\tthis text\n\n"
             << "  FILES:\n"
             << "    Files specified on the command line can include rasters,\n"
@@ -120,70 +129,6 @@ static QString myProjectFileName = "";
 static QStringList myFileList;
 
 
-#ifdef Q_OS_MACX
-/* Mac OS OpenDocuments AppleEvent handler called when files are double-clicked.
- * May be called at startup before application is initialized as well as
- * at any time while the application is running.
- */
-OSErr openDocumentsAEHandler( const AppleEvent *event, AppleEvent *reply, SRefCon refCon )
-{
-  AEDescList docs;
-  if ( AEGetParamDesc( event, keyDirectObject, typeAEList, &docs ) == noErr )
-  {
-    // Get count of files to open
-    long count = 0;
-    AECountItems( &docs, &count );
-
-    // Examine files and load first project file followed by all other non-project files
-    myProjectFileName.truncate( 0 );
-    myFileList.clear();
-    for ( int i = 0; i < count; i++ )
-    {
-      FSRef ref;
-      UInt8 strBuffer[256];
-      if ( AEGetNthPtr( &docs, i + 1, typeFSRef, 0, 0, &ref, sizeof( ref ), 0 ) == noErr &&
-           FSRefMakePath( &ref, strBuffer, 256 ) == noErr )
-      {
-        QString fileName( QString::fromUtf8( reinterpret_cast<char *>( strBuffer ) ) );
-        if ( fileName.endsWith( ".qgs" ) )
-        {
-          // Load first project file and ignore all other project files
-          if ( myProjectFileName.isEmpty() )
-          {
-            myProjectFileName = fileName;
-          }
-        }
-        else
-        {
-          // Load all non-project files
-          myFileList.append( fileName );
-        }
-      }
-    }
-
-    // Open files now if application has been fully initialized (has objectName).
-    // Otherwise (if this routine is called by processEvents inside the QgisApp constructor
-    // at startup) wait for the command line file loader to notice these files.
-    QgisApp *qgis = QgisApp::instance();
-    if ( qgis && qgis->objectName() == "QgisApp" )
-    {
-      if ( !myProjectFileName.isEmpty() )
-      {
-        qgis->openProject( myProjectFileName );
-      }
-      for ( QStringList::Iterator myIterator = myFileList.begin();
-            myIterator != myFileList.end(); ++myIterator )
-      {
-        QString fileName = *myIterator;
-        qgis->openLayer( fileName );
-      }
-    }
-  }
-  return noErr;
-}
-#endif
-
-
 /* Test to determine if this program was started on Mac OS X by double-clicking
  * the application bundle rather then from a command line. If clicked, argv[1]
  * contains a process serial number in the form -psn_0_1234567. Don't process
@@ -266,6 +211,7 @@ int main( int argc, char *argv[] )
   int mySnapshotHeight = 600;
 
   bool myHideSplash = false;
+  bool myRestorePlugins = true;
 
   // This behaviour will set initial extent of map canvas, but only if
   // there are no command line arguments. This gives a usable map
@@ -279,6 +225,10 @@ int main( int argc, char *argv[] )
   // which is useful for testing
   QString myTranslationCode;
 
+  // The user can specify a path which will override the default path of custom
+  // user settings (~/.qgis) and it will be used for QSettings INI file
+  QString configpath;
+
 #ifndef WIN32
   if ( !bundleclicked( argc, argv ) )
   {
@@ -295,6 +245,7 @@ int main( int argc, char *argv[] )
         /* These options set a flag. */
         {"help", no_argument, 0, '?'},
         {"nologo", no_argument, 0, 'n'},
+        {"noplugins", no_argument, 0, 'P'},
         /* These options don't set a flag.
          *  We distinguish them by their indices. */
         {"snapshot", required_argument, 0, 's'},
@@ -303,13 +254,15 @@ int main( int argc, char *argv[] )
         {"lang",     required_argument, 0, 'l'},
         {"project",  required_argument, 0, 'p'},
         {"extent",   required_argument, 0, 'e'},
+        {"optionspath", required_argument, 0, 'o'},
+        {"configpath", required_argument, 0, 'c'},
         {0, 0, 0, 0}
       };
 
       /* getopt_long stores the option index here. */
       int option_index = 0;
 
-      optionChar = getopt_long( argc, argv, "swhlpe",
+      optionChar = getopt_long( argc, argv, "swhlpeoc",
                                 long_options, &option_index );
 
       /* Detect the end of the options. */
@@ -352,10 +305,22 @@ int main( int argc, char *argv[] )
           myProjectFileName = QDir::convertSeparators( QFileInfo( QFile::decodeName( optarg ) ).absoluteFilePath() );
           break;
 
+        case 'P':
+          myRestorePlugins = false;
+          break;
+
         case 'e':
           myInitialExtent = optarg;
           break;
 
+        case 'o':
+          QSettings::setPath( QSettings::IniFormat, QSettings::UserScope, optarg );
+          break;
+
+        case 'c':
+          configpath = optarg;
+          break;
+
         case '?':
           usage( argv[0] );
           return 2;   // XXX need standard exit codes
@@ -396,6 +361,10 @@ int main( int argc, char *argv[] )
     {
       myHideSplash = true;
     }
+    else if ( arg == "--noplugins" || arg == "-P" )
+    {
+      myRestorePlugins = false;
+    }
     else if ( i + 1 < argc && ( arg == "--snapshot" || arg == "-s" ) )
     {
       mySnapshotFileName = QDir::convertSeparators( QFileInfo( QFile::decodeName( argv[++i] ) ).absoluteFilePath() );
@@ -420,6 +389,15 @@ int main( int argc, char *argv[] )
     {
       myInitialExtent = argv[++i];
     }
+    else if ( i + 1 < argc && ( arg == "--optionspath" || arg == "-o" ) )
+    {
+      QSettings::setPath( QSettings::IniFormat, QSettings::UserScope, argv[++i] );
+    }
+    else if ( i + 1 < argc && ( arg == "--configpath" || arg == "-c" ) )
+    {
+      configpath = argv[++i];
+      QSettings::setPath( QSettings::IniFormat, QSettings::UserScope, configpath );
+    }
     else
     {
       myFileList.append( QDir::convertSeparators( QFileInfo( QFile::decodeName( argv[i] ) ).absoluteFilePath() ) );
@@ -438,29 +416,38 @@ int main( int argc, char *argv[] )
 #ifdef Q_WS_X11
   bool myUseGuiFlag = getenv( "DISPLAY" ) != 0;
 #else
-  bool myUseGuiFlag = TRUE;
+  bool myUseGuiFlag = true;
 #endif
   if ( !myUseGuiFlag )
   {
-    QgsDebugMsg( "QGIS starting in non-interactive mode not supported.\n You "
-                 "are seeing this message most likely because you have no DISPLAY "
-                 "environment variable set." );
+    std::cerr << QObject::tr(
+                "QGIS starting in non-interactive mode not supported.\n"
+                "You are seeing this message most likely because you "
+                "have no DISPLAY environment variable set.\n"
+              ).toUtf8().constData();
     exit( 1 ); //exit for now until a version of qgis is capabable of running non interactive
   }
-  QgsApplication myApp( argc, argv, myUseGuiFlag );
+
+  if ( !configpath.isEmpty() )
+  {
+    // tell QSettings to use INI format and save the file in custom config path
+    QSettings::setPath( QSettings::IniFormat, QSettings::UserScope, configpath );
+  }
+
+  QgsApplication myApp( argc, argv, myUseGuiFlag, configpath );
+
+// (if Windows/Mac, use icon from resource)
+#if ! defined(Q_WS_WIN) && ! defined(Q_WS_MAC)
+  myApp.setWindowIcon( QPixmap( qgis_xpm ) );        // Linux
+#endif
+
   //
   // Set up the QSettings environment must be done after qapp is created
   QCoreApplication::setOrganizationName( "QuantumGIS" );
   QCoreApplication::setOrganizationDomain( "qgis.org" );
   QCoreApplication::setApplicationName( "QGIS" );
+  QCoreApplication::setAttribute( Qt::AA_DontShowIconsInMenus, false );
 #ifdef Q_OS_MACX
-  // Install OpenDocuments AppleEvent handler after application object is initialized
-  // but before any other event handling (including dialogs or splash screens) occurs.
-  // If an OpenDocuments event has been created before the application was launched,
-  // it must be handled before some other event handler runs and dismisses it as unknown.
-  // If run at startup, the handler will set either or both of myProjectFileName and myFileList.
-  AEInstallEventHandler( kCoreEventClass, kAEOpenDocuments, openDocumentsAEHandler, 0, false );
-
   // If the GDAL plugins are bundled with the application and GDAL_DRIVER_PATH
   // is not already defined, use the GDAL plugins in the application bundle.
   QString gdalPlugins( QCoreApplication::applicationDirPath().append( "/lib/gdalplugins" ) );
@@ -534,16 +521,25 @@ int main( int argc, char *argv[] )
   {
     myApp.installTranslator( &qgistor );
   }
+  else
+  {
+    qWarning( "loading of qgis translation failed [%s]", QString( "%1/qgis_%2" ).arg( i18nPath ).arg( myTranslationCode ).toLocal8Bit().constData() );
+  }
+
   /* Translation file for Qt.
    * The strings from the QMenuBar context section are used by Qt/Mac to shift
    * the About, Preferences and Quit items to the Mac Application menu.
    * These items must be translated identically in both qt_ and qgis_ files.
    */
   QTranslator qttor( 0 );
-  if ( qttor.load( QString( "qt_" ) + myTranslationCode, i18nPath ) )
+  if ( qttor.load( QString( "qt_" ) + myTranslationCode, QLibraryInfo::location( QLibraryInfo::TranslationsPath ) ) )
   {
     myApp.installTranslator( &qttor );
   }
+  else
+  {
+    qWarning( "loading of qt translation failed [%s]", QString( "%1/qt_%2" ).arg( QLibraryInfo::location( QLibraryInfo::TranslationsPath ) ).arg( myTranslationCode ).toLocal8Bit().constData() );
+  }
 
   //set up splash screen
   QString mySplashPath( QgsApplication::splashPath() );
@@ -555,15 +551,8 @@ int main( int argc, char *argv[] )
   }
   else
   {
-#if defined(Q_OS_MACX) && QT_VERSION < 0x040300
-    //on mac automasking as done below does not work (as of qt 4.2.1)
-    //so we do it the old way see bug #387
-    QPixmap myMaskPixmap( mySplashPath + QString( "splash_mask.png" ), 0, Qt::ThresholdDither | Qt::ThresholdAlphaDither | Qt::AvoidDither );
-    mypSplash->setMask( myMaskPixmap.createHeuristicMask() );
-#else
     //for win and linux we can just automask and png transparency areas will be used
     mypSplash->setMask( myPixmap.mask() );
-#endif
     mypSplash->show();
   }
 
@@ -612,7 +601,7 @@ int main( int argc, char *argv[] )
   }
 #endif
 
-  QgisApp *qgis = new QgisApp( mypSplash ); // "QgisApp" used to find canonical instance
+  QgisApp *qgis = new QgisApp( mypSplash, myRestorePlugins ); // "QgisApp" used to find canonical instance
   qgis->setObjectName( "QgisApp" );
 
   /////////////////////////////////////////////////////////////////////
@@ -746,5 +735,10 @@ int main( int argc, char *argv[] )
 
   mypSplash->finish( qgis );
   delete mypSplash;
-  return myApp.exec();
+
+  qgis->completeInitialization();
+
+  int retval = myApp.exec();
+  delete qgis;
+  return retval;
 }
