@@ -22,9 +22,11 @@
 #include <QMessageBox>
 #include <QInputDialog>
 #include <QTextCodec>
+
 #include "qgslogger.h"
-#include "qgsencodingfiledialog.h"
 #include "qgsopenvectorlayerdialog.h"
+#include "qgsvectordataprovider.h"
+
 #include <ogr_api.h>
 #include "qgsproviderregistry.h"
 #include "qgsnewogrconnection.h"
@@ -40,8 +42,10 @@ QgsOpenVectorLayerDialog::QgsOpenVectorLayerDialog( QWidget* parent, Qt::WFlags
   cmbConnections->blockSignals( true );
   radioSrcFile->setChecked( true );
   mDataSourceType = "file";
+
   //set encoding
-  // cmbEncodings->setItemText( cmbEncodings->currentIndex(), QString( QTextCodec::codecForLocale()->name() ) );
+  cmbEncodings->addItems( QgsVectorDataProvider::availableEncodings() );
+
   QSettings settings;
   QString enc = settings.value( "/UI/encoding", QString( "System" ) ).toString();
 
@@ -94,13 +98,11 @@ QgsOpenVectorLayerDialog::~QgsOpenVectorLayerDialog()
 
 QStringList QgsOpenVectorLayerDialog::openFile()
 {
-
   QStringList selectedFiles;
   QgsDebugMsg( "Vector file filters: " + mVectorFileFilter );
   QString enc = encoding();
   QString title = tr( "Open an OGR Supported Vector Layer" );
-  QgisGui::openFilesRememberingFilter( "lastVectorFileFilter", mVectorFileFilter, selectedFiles, enc,
-                                       title );
+  QgisGui::openFilesRememberingFilter( "lastVectorFileFilter", mVectorFileFilter, selectedFiles, enc, title );
 
   return selectedFiles;
 }
@@ -238,12 +240,10 @@ void QgsOpenVectorLayerDialog::setConnectionTypeListPosition()
   QSettings settings;
 
   QString toSelect = settings.value( "/ogr/connections/selectedtype" ).toString();
-  bool set = false;
   for ( int i = 0; i < cmbDatabaseTypes->count(); ++i )
     if ( cmbDatabaseTypes->itemText( i ) == toSelect )
     {
       cmbDatabaseTypes->setCurrentIndex( i );
-      set = true;
       break;
     }
 }
@@ -313,7 +313,10 @@ void QgsOpenVectorLayerDialog::accept()
     bool makeConnection = false;
     if ( pass.isEmpty() )
     {
-      pass = QInputDialog::getText( this,
+      if ( cmbDatabaseTypes->currentText() == "MSSQL" )
+        makeConnection = true;
+      else
+        pass = QInputDialog::getText( this,
                                     tr( "Password for " ) + user,
                                     tr( "Please enter your password:" ),
                                     QLineEdit::Password, QString::null,
