@@ -39,11 +39,11 @@ class QTcpSocket;
 class QValidator;
 
 class QgisAppInterface;
+class QgsAnnotationItem;
 class QgsClipboard;
 class QgsComposer;
 class QgsComposerView;
 class QgsGeometry;
-class QgsHelpViewer;
 class QgsFeature;
 
 class QgsLegend;
@@ -51,35 +51,46 @@ class QgsMapCanvas;
 class QgsMapLayer;
 class QgsMapTip;
 class QgsMapTool;
+class QgsPalLabeling;
 class QgsPoint;
 class QgsProviderRegistry;
-class QgsPythonDialog;
 class QgsPythonUtils;
 class QgsRasterLayer;
 class QgsRectangle;
 class QgsUndoWidget;
 class QgsVectorLayer;
+class QgsTileScaleWidget;
 
 class QDomDocument;
+class QNetworkReply;
+class QNetworkProxy;
+class QAuthenticator;
+
+class QgsSnappingDialog;
+class QgsGPSInformationWidget;
 
 #include <QMainWindow>
 #include <QToolBar>
 #include <QAbstractSocket>
 #include <QPointer>
+#include <QSslError>
 
 #include "qgsconfig.h"
 #include "qgsfeature.h"
 #include "qgspoint.h"
+#include "qgssnappingdialog.h"
+
+#include "ui_qgisapp.h"
 
 /*! \class QgisApp
  * \brief Main window for the Qgis application
  */
-class QgisApp : public QMainWindow
+class QgisApp : public QMainWindow, private Ui::MainWindow
 {
     Q_OBJECT
   public:
     //! Constructor
-    QgisApp( QSplashScreen *splash, QWidget * parent = 0, Qt::WFlags fl = Qt::Window );
+    QgisApp( QSplashScreen *splash, bool restorePlugins = true, QWidget * parent = 0, Qt::WFlags fl = Qt::Window );
     //! Destructor
     ~QgisApp();
     /**
@@ -129,49 +140,46 @@ class QgisApp : public QMainWindow
     //! Remove all layers from the map and legend - reimplements same method from qgisappbase
     void removeAllLayers();
     /** Open a raster or vector file; ignore other files.
-      Used to process a commandline argument or OpenDocument AppleEvent.
+      Used to process a commandline argument, FileOpen or Drop event.
+      Set interactive to true if it is ok to ask the user for information (mostly for
+      when a vector layer has sublayers and we want to ask which sublayers to use).
       @returns true if the file is successfully opened
       */
-    bool openLayer( const QString & fileName );
+    bool openLayer( const QString & fileName, bool allowInteractive = false );
     /** Open the specified project file; prompt to save previous project if necessary.
-      Used to process a commandline argument or OpenDocument AppleEvent.
+      Used to process a commandline argument, FileOpen or Drop event.
       */
     void openProject( const QString & fileName );
     /** opens a qgis project file
       @returns false if unable to open the project
       */
     bool addProject( QString projectFile );
+    /** Convenience function to open either a project or a layer file.
+      */
+    void openFile( const QString & fileName );
     //!Overloaded version of the private function with same name that takes the imagename as a parameter
     void saveMapAsImage( QString, QPixmap * );
     /** Get the mapcanvas object from the app */
-    QgsMapCanvas * mapCanvas() { return mMapCanvas; };
+    QgsMapCanvas * mapCanvas();
 
     //! Set theme (icons)
     void setTheme( QString themeName = "default" );
+
+    void setIconSizes( int size );
+
     //! Setup the toolbar popup menus for a given theme
     void setupToolbarPopups( QString themeName );
     //! Returns a pointer to the internal clipboard
     QgsClipboard * clipboard();
 
-    /** Setup the proxy settings from the QSettings environment.
-      * This is not called by default in the constructor. Rather,
-      * the application must explicitly call setupProx(). If
-      * you write your own application and wish to explicitly
-      * set up your own proxy rather, you should e.g.
-      *  QNetworkProxy proxy;
-      *  proxy.setType(QNetworkProxy::Socks5Proxy);
-      *  proxy.setHostName("proxy.example.com");
-      *  proxy.setPort(1080);
-      *  proxy.setUser("username");
-      *  proxy.setPassword("password");
-      *  QNetworkProxy::setApplicationProxy(proxy);
-      *
-      *  (as documented in Qt documentation.
-    */
-    void setupProxy();
-
     static QgisApp *instance() { return smInstance; }
 
+    //! initialize network manager
+    void namSetup();
+
+    //! update proxy settings
+    void namUpdate();
+
     //! Helper to get a theme icon. It will fall back to the
     //default theme if the active theme does not have the required
     //icon.
@@ -208,90 +216,83 @@ class QgisApp : public QMainWindow
     //! Actions to be inserted in menus and toolbars
     QAction *actionNewProject() { return mActionNewProject; }
     QAction *actionOpenProject() { return mActionOpenProject; }
-    QAction *actionFileSeparator1() { return mActionFileSeparator1; }
     QAction *actionSaveProject() { return mActionSaveProject; }
     QAction *actionSaveProjectAs() { return mActionSaveProjectAs; }
     QAction *actionSaveMapAsImage() { return mActionSaveMapAsImage; }
-    QAction *actionFileSeparator2() { return mActionFileSeparator2; }
     QAction *actionProjectProperties() { return mActionProjectProperties; }
-    QAction *actionFileSeparator3() { return mActionFileSeparator3; }
     QAction *actionNewPrintComposer() { return mActionNewPrintComposer; }
-    QAction *actionFileSeparator4() { return mActionFileSeparator4; }
     QAction *actionExit() { return mActionExit; }
 
     QAction *actionCutFeatures() { return mActionCutFeatures; }
     QAction *actionCopyFeatures() { return mActionCopyFeatures; }
     QAction *actionPasteFeatures() { return mActionPasteFeatures; }
-    QAction *actionEditSeparator1() { return mActionEditSeparator1; }
     QAction *actionCapturePoint() { return mActionCapturePoint; }
     QAction *actionCaptureLine() { return mActionCaptureLine; }
     QAction *actionCapturePolygon() { return mActionCapturePolygon; }
     QAction *actionDeleteSelected() { return mActionDeleteSelected; }
     QAction *actionMoveFeature() { return mActionMoveFeature; }
     QAction *actionSplitFeatures() { return mActionSplitFeatures; }
-    //These three tools are deprecated - use node tool rather...
-    //QAction *actionAddVertex() { return mActionAddVertex; }
-    //QAction *actionDeleteVertex() { return mActionDeleteVertex; }
-    //QAction *actionMoveVertex() { return mActionMoveVertex; }
     QAction *actionAddRing() { return mActionAddRing; }
     QAction *actionAddIsland() { return mActionAddIsland; }
     QAction *actionSimplifyFeature() { return mActionSimplifyFeature; }
     QAction *actionDeleteRing() { return mActionDeleteRing; }
     QAction *actionDeletePart() { return mActionDeletePart; }
     QAction *actionNodeTool() { return mActionNodeTool; }
-    QAction *actionEditSeparator2() { return mActionEditSeparator2; }
+    QAction *actionSnappingOptions() { return mActionSnappingOptions; }
 
     QAction *actionPan() { return mActionPan; }
     QAction *actionZoomIn() { return mActionZoomIn; }
     QAction *actionZoomOut() { return mActionZoomOut; }
     QAction *actionSelect() { return mActionSelect; }
+    QAction *actionSelectRectangle() { return mActionSelectRectangle; }
+    QAction *actionSelectPolygon() { return mActionSelectPolygon; }
+    QAction *actionSelectFreehand() { return mActionSelectFreehand; }
+    QAction *actionSelectRadius() { return mActionSelectRadius; }
     QAction *actionIdentify() { return mActionIdentify; }
     QAction *actionMeasure() { return mActionMeasure; }
     QAction *actionMeasureArea() { return mActionMeasureArea; }
-    QAction *actionViewSeparator1() { return mActionViewSeparator1; }
     QAction *actionZoomFullExtent() { return mActionZoomFullExtent; }
     QAction *actionZoomToLayer() { return mActionZoomToLayer; }
     QAction *actionZoomToSelected() { return mActionZoomToSelected; }
     QAction *actionZoomLast() { return mActionZoomLast; }
     QAction *actionZoomNext() { return mActionZoomNext; }
     QAction *actionZoomActualSize() { return mActionZoomActualSize; }
-    QAction *actionViewSeparator2() { return mActionViewSeparator2; }
     QAction *actionMapTips() { return mActionMapTips; }
     QAction *actionNewBookmark() { return mActionNewBookmark; }
     QAction *actionShowBookmarks() { return mActionShowBookmarks; }
     QAction *actionDraw() { return mActionDraw; }
-    QAction *actionViewSeparator3() { return mActionViewSeparator3; }
 
     QAction *actionNewVectorLayer() { return mActionNewVectorLayer; }
+    QAction *actionNewSpatialLiteLayer() { return mActionNewSpatialiteLayer; }
     QAction *actionAddOgrLayer() { return mActionAddOgrLayer; }
     QAction *actionAddRasterLayer() { return mActionAddRasterLayer; }
     QAction *actionAddPgLayer() { return mActionAddPgLayer; }
     QAction *actionAddSpatiaLiteLayer() { return mActionAddSpatiaLiteLayer; };
     QAction *actionAddWmsLayer() { return mActionAddWmsLayer; }
-    QAction *actionLayerSeparator1() { return mActionLayerSeparator1; }
     QAction *actionOpenTable() { return mActionOpenTable; }
     QAction *actionToggleEditing() { return mActionToggleEditing; }
+    QAction *actionSaveEdits() { return mActionSaveEdits; }
     QAction *actionLayerSaveAs() { return mActionLayerSaveAs; }
     QAction *actionLayerSelectionSaveAs() { return mActionLayerSelectionSaveAs; }
     QAction *actionRemoveLayer() { return mActionRemoveLayer; }
+    QAction *actionSetLayerCRS() { return mActionSetLayerCRS; }
+    QAction *actionSetProjectCRSFromLayer() { return mActionSetProjectCRSFromLayer; }
+    QAction *actionTileScale() { return mActionTileScale; }
+    QAction *actionGpsTool() { return mActionGpsTool; }
     QAction *actionLayerProperties() { return mActionLayerProperties; }
-    QAction *actionLayerSeparator2() { return mActionLayerSeparator2; }
+    QAction *actionLayerSubsetString() { return mActionLayerSubsetString; }
     QAction *actionAddToOverview() { return mActionAddToOverview; }
     QAction *actionAddAllToOverview() { return mActionAddAllToOverview; }
     QAction *actionRemoveAllFromOverview() { return mActionRemoveAllFromOverview; }
-    QAction *actionLayerSeparator3() { return mActionLayerSeparator3; }
     QAction *actionHideAllLayers() { return mActionHideAllLayers; }
     QAction *actionShowAllLayers() { return mActionShowAllLayers; }
 
     QAction *actionManagePlugins() { return mActionManagePlugins; }
-    QAction *actionPluginSeparator1() { return mActionPluginSeparator1; }
     QAction *actionPluginListSeparator() { return mActionPluginSeparator1; }
-    QAction *actionPluginSeparator2() { return mActionPluginSeparator2; }
     QAction *actionPluginPythonSeparator() { return mActionPluginSeparator2; }
     QAction *actionShowPythonDialog() { return mActionShowPythonDialog; }
 
     QAction *actionToggleFullScreen() { return mActionToggleFullScreen; }
-    QAction *actionSettingsSeparator1() { return mActionSettingsSeparator1; }
     QAction *actionOptions() { return mActionOptions; }
     QAction *actionCustomProjection() { return mActionCustomProjection; }
     QAction *actionConfigureShortcuts() { return mActionConfigureShortcuts; }
@@ -299,17 +300,15 @@ class QgisApp : public QMainWindow
 #ifdef Q_WS_MAC
     QAction *actionWindowMinimize() { return mActionWindowMinimize; }
     QAction *actionWindowZoom() { return mActionWindowZoom; }
-    QAction *actionWindowSeparator1() { return mActionWindowSeparator1; }
     QAction *actionWindowAllToFront() { return mActionWindowAllToFront; }
-    QAction *actionWindowSeparator2() { return mActionWindowSeparator2; }
 #endif
 
     QAction *actionHelpContents() { return mActionHelpContents; }
-    QAction *actionHelpSeparator1() { return mActionHelpSeparator1; }
+    QAction *actionHelpAPI() { return mActionHelpAPI; }
     QAction *actionQgisHomePage() { return mActionQgisHomePage; }
     QAction *actionCheckQgisVersion() { return mActionCheckQgisVersion; }
-    QAction *actionHelpSeparator2() { return mActionHelpSeparator2; }
     QAction *actionAbout() { return mActionAbout; }
+    QAction *actionSponsors() { return mActionSponsors; }
 
     //! Menus
     QMenu *fileMenu() { return mFileMenu; }
@@ -318,6 +317,8 @@ class QgisApp : public QMainWindow
     QMenu *layerMenu() { return mLayerMenu; }
     QMenu *settingsMenu() { return mSettingsMenu; }
     QMenu *pluginMenu() { return mPluginMenu; }
+    QMenu *databaseMenu() { return mDatabaseMenu; }
+    QMenu *rasterMenu() { return mRasterMenu; }
 #ifdef Q_WS_MAC
     QMenu *firstRightStandardMenu() { return mWindowMenu; }
     QMenu *windowMenu() { return mWindowMenu; }
@@ -342,10 +343,26 @@ class QgisApp : public QMainWindow
     QToolBar *attributesToolBar() { return mAttributesToolBar; }
     QToolBar *pluginToolBar() { return mPluginToolBar; }
     QToolBar *helpToolBar() { return mHelpToolBar; }
+    QToolBar *rasterToolBar() { return mRasterToolBar; }
 
     //! run python
     void runPythonString( const QString &expr );
 
+    //! show layer properties
+    void showLayerProperties( QgsMapLayer *ml );
+
+    //! returns pointer to map legend
+    QgsLegend *legend();
+
+#ifdef Q_OS_WIN
+    //! ugly hack
+    void skipNextContextMenuEvent();
+#endif
+
+    //! emit initializationCompleted signal
+    //! @note added in 1.6
+    void completeInitialization();
+
   public slots:
     //! Zoom to full extent
     void zoomFull();
@@ -356,6 +373,21 @@ class QgisApp : public QMainWindow
     //! Zoom to selected features
     void zoomToSelected();
 
+    //! open the properties dialog for the currently selected layer
+    void layerProperties();
+
+    //! show the attribute table for the currently selected layer
+    void attributeTable();
+
+    //! mark project dirty
+    void markDirty();
+
+    //! layer was added
+    void layerWasAdded( QgsMapLayer * );
+
+    //! layer will be removed
+    void removingLayer( QString );
+
     void updateUndoActions();
 
     //! cuts selected features on the active layer to the clipboard
@@ -382,6 +414,28 @@ class QgisApp : public QMainWindow
     /**Deletes the selected attributes for the currently selected vector layer*/
     void deleteSelected( QgsMapLayer *layer = 0 );
 
+    //! project was written
+    void writeProject( QDomDocument & );
+
+    //! project was read
+    void readProject( const QDomDocument & );
+
+    //! request credentials for network manager
+    void namAuthenticationRequired( QNetworkReply *reply, QAuthenticator *auth );
+    void namProxyAuthenticationRequired( const QNetworkProxy &proxy, QAuthenticator *auth );
+#ifndef QT_NO_OPENSSL
+    void namSslErrors( QNetworkReply *reply, const QList<QSslError> &errors );
+#endif
+
+    //! update default action of toolbutton
+    void toolButtonActionTriggered( QAction * );
+
+    //! layer selection changed
+    void legendLayerSelectionChanged( void );
+
+    //! Watch for QFileOpenEvent.
+    virtual bool event( QEvent * event );
+
   protected:
 
     //! Handle state changes (WindowTitleChange)
@@ -395,7 +449,14 @@ class QgisApp : public QMainWindow
     //! reimplements widget keyPress event so we can check if cancel was pressed
     virtual void keyPressEvent( QKeyEvent * event );
 
+#ifdef Q_OS_WIN
+    //! reimplements context menu event
+    virtual void contextMenuEvent( QContextMenuEvent *event );
+#endif
+
   private slots:
+    //! QGis Sponsors
+    void sponsors();
     //! About QGis
     void about();
     //! Add a raster layer to the map (will prompt user for file name using dlg )
@@ -420,20 +481,48 @@ class QgisApp : public QMainWindow
     void userCenter();
     //! Remove a layer from the map and legend
     void removeLayer();
+    //! Set CRS of a layer
+    void setLayerCRS();
+    //! Assign layer CRS to project
+    void setProjectCRSFromLayer();
+    //! Show GPS tool
+    void showGpsTool();
+    //! Show tile scale slider
+    void showTileScale();
     //! zoom to extent of layer
     void zoomToLayerExtent();
     //! zoom to actual size of raster layer
     void zoomActualSize();
+    /** Perform a local histogram stretch on the active raster layer
+     * (stretch based on pixel values in view extent).
+     * Valid for non wms raster layers only.
+     * @note Added in QGIS 1.7 */
+    void localHistogramStretch();
+    /** perform a full histogram stretch on the active raster layer
+     * (stretch based on pixels values in full dataset)
+     * Valid for non wms raster layers only.
+     * @note Added in QGIS 1.7 */
+    void fullHistogramStretch();
     //! plugin manager
     void showPluginManager();
     //! load python support if possible
     void loadPythonSupport();
-    //! Find the QMenu with the given name (ie the user visible text on the menu item)
+    //! Find the QMenu with the given name within plugin menu (ie the user visible text on the menu item)
     QMenu* getPluginMenu( QString menuName );
     //! Add the action to the submenu with the given name under the plugin menu
     void addPluginToMenu( QString name, QAction* action );
     //! Remove the action to the submenu with the given name under the plugin menu
     void removePluginMenu( QString name, QAction* action );
+    //! Find the QMenu with the given name within the Database menu (ie the user visible text on the menu item)
+    QMenu* getDatabaseMenu( QString menuName );
+    //! Add the action to the submenu with the given name under the Database menu
+    void addPluginToDatabaseMenu( QString name, QAction* action );
+    //! Remove the action to the submenu with the given name under the Database menu
+    void removePluginDatabaseMenu( QString name, QAction* action );
+    //! Add "add layer" action to layer menu
+    void insertAddLayerAction( QAction* action );
+    //! Remove "add layer" action to layer menu
+    void removeAddLayerAction( QAction* action );
     //! Add an icon to the plugin toolbar
     int addPluginToolBarIcon( QAction * qAction );
     //! Remove an icon from the plugin toolbar
@@ -457,8 +546,15 @@ class QgisApp : public QMainWindow
     void fileNew();
     //! As above but allows forcing without prompt
     void fileNew( bool thePromptToSaveFlag );
+    //! Calculate new rasters from existing ones
+    void showRasterCalculator();
+
     //! Create a new empty vector layer
     void newVectorLayer();
+#ifdef HAVE_SPATIALITE
+    //! Create a new empty spatialite layer
+    void newSpatialiteLayer();
+#endif
     //! Print the current map view frame
     void newPrintComposer();
     void showComposerManager();
@@ -482,6 +578,8 @@ class QgisApp : public QMainWindow
     bool setActiveLayer( QgsMapLayer * );
     //! Open the help contents in a browser
     void helpContents();
+    //! Open the API documentation in a browser
+    void apiDocumentation();
     //! Open the QGIS homepage in users browser
     void helpQgisHomePage();
     //! Open a url in the users configured browser
@@ -539,25 +637,45 @@ class QgisApp : public QMainWindow
     void deletePart();
     //! merges the selected features together
     void mergeSelectedFeatures();
+    //! merges the attributes of selected features
+    void mergeAttributesOfSelectedFeatures();
     //! provides operations with nodes
     void nodeTool();
     //! activates the rotate points tool
     void rotatePointSymbols();
+    //! shows the snapping Options
+    void snappingOptions();
 
     //! activates the selection tool
     void select();
 
+    //! activates the rectangle selection tool
+    void selectByRectangle();
+
+    //! activates the polygon selection tool
+    void selectByPolygon();
+
+    //! activates the freehand selection tool
+    void selectByFreehand();
+
+    //! activates the radius selection tool
+    void selectByRadius();
+
     //! deselect features from all layers
     void deselectAll();
 
     //! refresh map canvas
     void refreshMapCanvas();
-    //! returns pointer to map legend
-    QgsLegend *legend() { return mMapLegend; }
 
     //! starts/stops editing mode of the current layer
     void toggleEditing();
 
+    //! save current edits and start new transaction
+    void saveEdits();
+
+    //! change layer subset of current vector layer
+    void layerSubsetString();
+
     //! map tool changed
     void mapToolChanged( QgsMapTool *tool );
 
@@ -565,6 +683,8 @@ class QgisApp : public QMainWindow
     Is called from the legend when the current legend item has changed*/
     void activateDeactivateLayerRelatedActions( QgsMapLayer* layer );
 
+    void selectionChanged( QgsMapLayer *layer );
+
     void showProgress( int theProgress, int theTotalSteps );
     void extentsViewToggled( bool theFlag );
     void showExtents();
@@ -591,19 +711,23 @@ class QgisApp : public QMainWindow
     void measure();
     //! Measure area
     void measureArea();
+    //! Measure angle
+    void measureAngle();
 
-    //! show the attribute table for the currently selected layer
-    void attributeTable();
+    //annotations
+    void addFormAnnotation();
+    void addTextAnnotation();
+    void modifyAnnotation();
+
+    //! shows label settings dialog (for labeling-ng)
+    void labeling();
 
     //! starts/stops editing mode of a layer
-    void toggleEditing( QgsMapLayer *layer );
+    bool toggleEditing( QgsMapLayer *layer, bool allowCancel = true );
 
     //! save current vector layer
-    void saveAsShapefile();
-    void saveSelectionAsShapefile();
-
-    //! open the properties dialog for the currently selected layer
-    void layerProperties();
+    void saveAsVectorFile();
+    void saveSelectionAsVectorFile();
 
     //! show python console
     void showPythonDialog();
@@ -637,8 +761,19 @@ class QgisApp : public QMainWindow
 
     void showStyleManagerV2();
 
-    //! project changed
-    void projectChanged( const QDomDocument & );
+    void writeAnnotationItemsToProject( QDomDocument& doc );
+
+    /**Creates the composer instances in a project file and adds them to the menu*/
+    bool loadComposersFromProject( const QDomDocument& doc );
+
+    bool loadAnnotationItemsFromProject( const QDomDocument& doc );
+
+    //! Activates the move label tool
+    void moveLabel();
+    //! Activates rotate label tool
+    void rotateLabel();
+    //! Activates label property tool
+    void changeLabelProperties();
 
   signals:
     /** emitted when a key is pressed and we want non widget sublasses to be able
@@ -679,21 +814,20 @@ class QgisApp : public QMainWindow
       @note added in version 1.4*/
     void composerWillBeRemoved( QgsComposerView* v );
 
+    /**This signal is emitted when QGIS' initialization is complete
+     @note added in version 1.6*/
+    void initializationCompleted();
 
   private:
-    /** This method will open a dialog so the user can select the sublayers
-    * to load
+    /** This method will open a dialog so the user can select the sublayers to load
     */
-    void askUserForSublayers( QgsVectorLayer *layer );
+    void askUserForOGRSublayers( QgsVectorLayer *layer );
+    void askUserForGDALSublayers( QgsRasterLayer *layer );
     /** Add a raster layer to the map (passed in as a ptr).
      * It won't force a refresh.
      */
     bool addRasterLayer( QgsRasterLayer * theRasterLayer );
-    //@todo We should move these next two into vector layer class
-    /** This helper checks to see whether the file name appears to be a valid shape file name */
-    bool isValidShapeFileName( QString theFileNameQString );
-    /** Overloaded version of the above function provided for convenience that takes a qstring pointer */
-    bool isValidShapeFileName( QString * theFileNameQString );
+
     /** add this file to the recently opened/saved projects list
      *  pass settings by reference since creating more than one
      * instance simultaneously results in data loss.
@@ -708,13 +842,19 @@ class QgisApp : public QMainWindow
     //! check to see if file is dirty and if so, prompt the user th save it
     bool saveDirty();
     /** Helper function to union several geometries together (used in function mergeSelectedFeatures)
-      @return 0 in case of error*/
-    QgsGeometry* unionGeometries( const QgsVectorLayer* vl, QgsFeatureList& featureList );
+      @return 0 in case of error or if canceled */
+    QgsGeometry* unionGeometries( const QgsVectorLayer* vl, QgsFeatureList& featureList, bool &canceled );
 
     /**Deletes all the composer objects and clears mPrintComposers*/
     void deletePrintComposers();
-    /**Creates the composer instances in a project file and adds them to the menu*/
-    bool loadComposersFromProject( const QString& projectFilePath );
+
+
+    void saveAsVectorFileGeneral( bool saveOnlySelection );
+
+    /**Returns all annotation items in the canvas*/
+    QList<QgsAnnotationItem*> annotationItems();
+    /**Removes annotation items in the canvas*/
+    void removeAnnotationItems();
 
     /// QgisApp aren't copyable
     QgisApp( QgisApp const & );
@@ -729,122 +869,15 @@ class QgisApp : public QMainWindow
     void createToolBars();
     void createStatusBar();
     void setupConnections();
-    void createLegend();
+    void initLegend();
     void createOverview();
-    void createCanvas();
+    void createCanvasTools();
     bool createDB();
     void createMapTips();
-
-    // toolbars ---------------------------------------
-    QToolBar *mFileToolBar;
-    QToolBar *mLayerToolBar;
-    QToolBar *mMapNavToolBar;
-    QToolBar *mDigitizeToolBar;
-    QToolBar *mAdvancedDigitizeToolBar;
-    QToolBar *mAttributesToolBar;
-    QToolBar *mPluginToolBar;
-    QToolBar *mHelpToolBar;
+    void updateCRSStatusBar();
 
     // actions for menus and toolbars -----------------
 
-    QAction *mActionNewProject;
-    QAction *mActionOpenProject;
-    QAction *mActionFileSeparator1;
-    QAction *mActionSaveProject;
-    QAction *mActionSaveProjectAs;
-    QAction *mActionSaveMapAsImage;
-    QAction *mActionFileSeparator2;
-    QAction *mActionProjectProperties;
-    QAction *mActionFileSeparator3;
-    QAction *mActionNewPrintComposer;
-    QAction *mActionShowComposerManager;
-    QAction *mActionFileSeparator4;
-    QAction *mActionExit;
-
-    QAction *mActionUndo;
-    QAction *mActionRedo;
-    QAction *mActionEditSeparator0;
-    QAction *mActionCutFeatures;
-    QAction *mActionCopyFeatures;
-    QAction *mActionPasteFeatures;
-    QAction *mActionEditSeparator1;
-    QAction *mActionCapturePoint;
-    QAction *mActionCaptureLine;
-    QAction *mActionCapturePolygon;
-    QAction *mActionDeleteSelected;
-    QAction *mActionMoveFeature;
-    QAction *mActionReshapeFeatures;
-    QAction *mActionSplitFeatures;
-#if 0 // deprecated
-    QAction *mActionAddVertex;
-    QAction *mActionDeleteVertex;
-    QAction *mActionMoveVertex;
-#endif
-    QAction *mActionAddRing;
-    QAction *mActionAddIsland;
-    QAction *mActionEditSeparator2;
-    QAction *mActionSimplifyFeature;
-    QAction *mActionDeleteRing;
-    QAction *mActionDeletePart;
-    QAction *mActionMergeFeatures;
-    QAction *mActionNodeTool;
-    QAction *mActionRotatePointSymbols;
-    QAction *mActionEditSeparator3;
-
-    QAction *mActionPan;
-    QAction *mActionZoomIn;
-    QAction *mActionZoomOut;
-    QAction *mActionSelect;
-    QAction *mActionDeselectAll;
-    QAction *mActionIdentify;
-    QAction *mActionMeasure;
-    QAction *mActionMeasureArea;
-    QAction *mActionViewSeparator1;
-    QAction *mActionZoomFullExtent;
-    QAction *mActionZoomToLayer;
-    QAction *mActionZoomToSelected;
-    QAction *mActionZoomLast;
-    QAction *mActionZoomNext;
-    QAction *mActionZoomActualSize;
-    QAction *mActionViewSeparator2;
-    QAction *mActionMapTips;
-    QAction *mActionNewBookmark;
-    QAction *mActionShowBookmarks;
-    QAction *mActionDraw;
-    QAction *mActionViewSeparator3;
-
-    QAction *mActionNewVectorLayer;
-    QAction *mActionAddOgrLayer;
-    QAction *mActionAddRasterLayer;
-    QAction *mActionAddPgLayer;
-    QAction *mActionAddSpatiaLiteLayer;
-    QAction *mActionAddWmsLayer;
-    QAction *mActionLayerSeparator1;
-    QAction *mActionOpenTable;
-    QAction *mActionToggleEditing;
-    QAction *mActionLayerSaveAs;
-    QAction *mActionLayerSelectionSaveAs;
-    QAction *mActionRemoveLayer;
-    QAction *mActionLayerProperties;
-    QAction *mActionLayerSeparator2;
-    QAction *mActionAddToOverview;
-    QAction *mActionAddAllToOverview;
-    QAction *mActionRemoveAllFromOverview;
-    QAction *mActionLayerSeparator3;
-    QAction *mActionHideAllLayers;
-    QAction *mActionShowAllLayers;
-
-    QAction *mActionManagePlugins;
-    QAction *mActionPluginSeparator1;
-    QAction *mActionPluginSeparator2;
-    QAction *mActionShowPythonDialog;
-
-    QAction *mActionToggleFullScreen;
-    QAction *mActionSettingsSeparator1;
-    QAction *mActionOptions;
-    QAction *mActionCustomProjection;
-    QAction *mActionConfigureShortcuts;
-
 #ifdef Q_WS_MAC
     QAction *mActionWindowMinimize;
     QAction *mActionWindowZoom;
@@ -854,37 +887,26 @@ class QgisApp : public QMainWindow
     QActionGroup *mWindowActions;
 #endif
 
-    QAction *mActionHelpContents;
-    QAction *mActionHelpSeparator1;
-    QAction *mActionQgisHomePage;
-    QAction *mActionCheckQgisVersion;
-    QAction *mActionHelpSeparator2;
-    QAction *mActionAbout;
-
-    QAction *mActionUseRendererV2;
-    QAction *mActionStyleManagerV2;
+    QAction* mActionPluginSeparator1;
+    QAction* mActionPluginSeparator2;
 
     // action groups ----------------------------------
     QActionGroup *mMapToolGroup;
 
     // menus ------------------------------------------
-    QMenu *mFileMenu;
-    QMenu *mEditMenu;
-    QMenu *mRecentProjectsMenu;
-    QMenu *mViewMenu;
-    QMenu *mPanelMenu;
-    QMenu *mToolbarMenu;
-    QMenu *mLayerMenu;
-    QMenu *mSettingsMenu;
+
 #ifdef Q_WS_MAC
     QMenu *mWindowMenu;
 #endif
-    QMenu *mPrintComposersMenu;
-    QMenu *mHelpMenu;
+    QMenu *mPanelMenu;
+    QMenu *mToolbarMenu;
 
     // docks ------------------------------------------
     QDockWidget *mLegendDock;
     QDockWidget *mOverviewDock;
+    QDockWidget *mpTileScaleDock;
+    QDockWidget *mpGpsDock;
+
 
 #ifdef Q_WS_MAC
     //! Window menu action to select this window
@@ -900,6 +922,7 @@ class QgisApp : public QMainWindow
         QgsMapTool* mIdentify;
         QgsMapTool* mMeasureDist;
         QgsMapTool* mMeasureArea;
+        QgsMapTool* mMeasureAngle;
         QgsMapTool* mCapturePoint;
         QgsMapTool* mCaptureLine;
         QgsMapTool* mCapturePolygon;
@@ -907,6 +930,10 @@ class QgisApp : public QMainWindow
         QgsMapTool* mReshapeFeatures;
         QgsMapTool* mSplitFeatures;
         QgsMapTool* mSelect;
+        QgsMapTool* mSelectRectangle;
+        QgsMapTool* mSelectPolygon;
+        QgsMapTool* mSelectFreehand;
+        QgsMapTool* mSelectRadius;
         QgsMapTool* mVertexAdd;
         QgsMapTool* mVertexMove;
         QgsMapTool* mVertexDelete;
@@ -917,6 +944,12 @@ class QgisApp : public QMainWindow
         QgsMapTool* mDeletePart;
         QgsMapTool* mNodeTool;
         QgsMapTool* mRotatePointSymbolsTool;
+        QgsMapTool* mAnnotation;
+        QgsMapTool* mFormAnnotation;
+        QgsMapTool* mTextAnnotation;
+        QgsMapTool* mMoveLabel;
+        QgsMapTool* mRotateLabel;
+        QgsMapTool* mChangeLabelProperties;
     } mMapTools;
 
     QgsMapTool *mNonEditMapTool;
@@ -941,12 +974,14 @@ class QgisApp : public QMainWindow
     QToolButton * mToggleExtentsViewButton;
     //! Button used to stop rendering
     QToolButton* mStopRenderButton;
+    //! Widget in status bar used to show current project CRS
+    QLabel * mOnTheFlyProjectionStatusLabel;
     //! Widget in status bar used to show status of on the fly projection
     QToolButton * mOnTheFlyProjectionStatusButton;
     //! Popup menu
     QMenu * mPopupMenu;
-    //! Top level plugin menu
-    QMenu *mPluginMenu;
+    //! Top level database menu
+    QMenu *mDatabaseMenu;
     //! Popup menu for the map overview tools
     QMenu *toolPopupOverviews;
     //! Popup menu for the display tools
@@ -977,8 +1012,6 @@ class QgisApp : public QMainWindow
     QTcpSocket *mSocket;
     QString mVersionMessage;
     QSplashScreen *mSplash;
-    //! help viewer
-    QgsHelpViewer *mHelpViewer;
     //! list of recently opened/saved project files
     QStringList mRecentProjectPaths;
     //! Print composers of this project, accessible by id string
@@ -1022,14 +1055,32 @@ class QgisApp : public QMainWindow
     //!flag to indicate that the previous screen mode was 'maximised'
     bool mPrevScreenModeMaximized;
 
-    QgsPythonDialog* mPythonConsole;
     QgsPythonUtils* mPythonUtils;
 
     static QgisApp *smInstance;
 
     QgsUndoWidget* mUndoWidget;
 
+    QgsSnappingDialog* mSnappingDialog;
+
+    //! Persistent tile scale slider
+    QgsTileScaleWidget * mpTileScaleWidget;
+
     int mLastComposerId;
+
+#ifdef Q_OS_WIN
+    int mSkipNextContextMenuEvent; // ugly hack
+#endif
+
+    //! Persistent GPS toolbox
+    QgsGPSInformationWidget * mpGpsWidget;
+
+    QgsPalLabeling* mLBL;
+
+    //! project changed
+    void projectChanged( const QDomDocument & );
+
+    QString mOldScale;
 };
 
 #endif
