@@ -24,6 +24,11 @@
 
 class QgisApp;
 
+#ifdef _MSC_VER
+#  pragma warning( push )
+#  pragma warning( disable: 4996 )  // declared deprecated
+#endif
+
 /** \class QgisAppInterface
  * \brief Interface class to provide access to private methods in QgisApp
  * for use by plugins.
@@ -107,6 +112,16 @@ class QgisAppInterface : public QgisInterface
     /** Remove action from the plugins menu */
     void removePluginMenu( QString name, QAction* action );
 
+    /** Add action to the Database menu */
+    void addPluginToDatabaseMenu( QString name, QAction* action );
+    /** Remove action from the Database menu */
+    void removePluginDatabaseMenu( QString name, QAction* action );
+
+    /** Add "add layer" action to the layer menu */
+    void insertAddLayerAction( QAction *action );
+    /** remove "add layer" action from the layer menu */
+    void removeAddLayerAction( QAction *action );
+
     /** Add a dock widget to the main window */
     void addDockWidget( Qt::DockWidgetArea area, QDockWidget * dockwidget );
 
@@ -115,6 +130,18 @@ class QgisAppInterface : public QgisInterface
 
     virtual void refreshLegend( QgsMapLayer *l );
 
+    /** show layer properties dialog for layer
+     * @param l layer to show properties table for
+     * @note added in added in 1.5
+     */
+    virtual void showLayerProperties( QgsMapLayer *l );
+
+    /** show layer attribute dialog for layer
+     * @param l layer to show attribute table for
+     * @note added in added in 1.7
+     */
+    virtual void showAttributeTable( QgsVectorLayer *l );
+
     /** Add window to Window menu. The action title is the window title
      * and the action should raise, unminimize and activate the window. */
     virtual void addWindow( QAction *action );
@@ -141,6 +168,7 @@ class QgisAppInterface : public QgisInterface
     virtual QMenu *layerMenu();
     virtual QMenu *settingsMenu();
     virtual QMenu *pluginMenu();
+    virtual QMenu *databaseMenu();
     virtual QMenu *firstRightStandardMenu();
     virtual QMenu *windowMenu();
     virtual QMenu *helpMenu();
@@ -150,9 +178,11 @@ class QgisAppInterface : public QgisInterface
     virtual QToolBar *layerToolBar();
     virtual QToolBar *mapNavToolToolBar();
     virtual QToolBar *digitizeToolBar();
+    virtual QToolBar *advancedDigitizeToolBar();
     virtual QToolBar *attributesToolBar();
     virtual QToolBar *pluginToolBar();
     virtual QToolBar *helpToolBar();
+    virtual QToolBar *rasterToolBar();
 
     //! File menu actions
     virtual QAction *actionNewProject();
@@ -184,6 +214,10 @@ class QgisAppInterface : public QgisInterface
     virtual QAction *actionMoveVertex();
     virtual QAction *actionAddRing();
     virtual QAction *actionAddIsland();
+    virtual QAction *actionSimplifyFeature();
+    virtual QAction *actionDeleteRing();
+    virtual QAction *actionDeletePart();
+    virtual QAction *actionNodeTool();
     virtual QAction *actionEditSeparator2();
 
     //! View menu actions
@@ -191,6 +225,10 @@ class QgisAppInterface : public QgisInterface
     virtual QAction *actionZoomIn();
     virtual QAction *actionZoomOut();
     virtual QAction *actionSelect();
+    virtual QAction *actionSelectRectangle();
+    virtual QAction *actionSelectPolygon();
+    virtual QAction *actionSelectFreehand();
+    virtual QAction *actionSelectRadius();
     virtual QAction *actionIdentify();
     virtual QAction *actionMeasure();
     virtual QAction *actionMeasureArea();
@@ -251,6 +289,14 @@ class QgisAppInterface : public QgisInterface
     virtual QAction *actionHelpSeparator2();
     virtual QAction *actionAbout();
 
+    //! open feature form
+    // returns true when dialog was accepted
+    // @param l vector layer
+    // @param f feature to show/modify
+    // @param updateFeatureOnly only update the feature update (don't change any attributes of the layer)
+    // @added in 1.6
+    virtual bool openFeatureForm( QgsVectorLayer *l, QgsFeature &f, bool updateFeatureOnly = false );
+
   signals:
     void currentThemeChanged( QString );
 
@@ -269,5 +315,8 @@ class QgisAppInterface : public QgisInterface
     QgsAppLegendInterface legendIface;
 };
 
+#ifdef _MSC_VER
+#  pragma warning( pop )
+#endif
 
 #endif //#define QGISAPPINTERFACE_H
