@@ -28,7 +28,6 @@
 
 /* Uncomment this block to use preloaded images
 #include <map>
-#include "qgslogger.h"
 std::map<QString, QPixmap> mugs;
 */
 #ifdef Q_OS_MACX
@@ -116,54 +115,6 @@ void QgsAbout::init()
   }
 
 
-  // read the SPONSORS file and populate the text widget
-  QFile sponsorFile( QgsApplication::sponsorsFilePath() );
-#ifdef QGISDEBUG
-  printf( "Reading sponsors file %s.............................................\n",
-          sponsorFile.fileName().toLocal8Bit().constData() );
-#endif
-  if ( sponsorFile.open( QIODevice::ReadOnly ) )
-  {
-    QString sponsorHTML = ""
-                          + tr( "<p>QGIS sponsorship programme:"
-                                "contribute to QGIS development</p>" )
-                          + "<hr>"
-                          "<table width='100%'>"
-                          "<tr><th>" + tr( "Name" ) + "</th>"
-                          "<th>" + tr( "Website" ) + "</th></tr>";
-    QString website;
-    QTextStream sponsorStream( &sponsorFile );
-    // Always use UTF-8
-    sponsorStream.setCodec( "UTF-8" );
-    QString sline;
-    while ( !sponsorStream.atEnd() )
-    {
-      sline = sponsorStream.readLine(); // line of text excluding '\n'
-      //ignore the line if it starts with a hash....
-      if ( sline.left( 1 ) == "#" ) continue;
-      QStringList myTokens = sline.split( "|", QString::SkipEmptyParts );
-      if ( myTokens.size() > 1 )
-      {
-        website = "<a href=\"" + myTokens[1].remove( ' ' ) + "\">" + myTokens[1] + "</a>";
-      }
-      else
-      {
-        website = "&nbsp;";
-      }
-      sponsorHTML += "<tr>";
-      sponsorHTML += "<td>" + myTokens[0] + "</td><td>" + website + "</td>";
-      // close the row
-      sponsorHTML += "</tr>";
-    }
-    sponsorHTML += "</table>";
-
-    QString myStyle = QgsApplication::reportStyleSheet();
-    txtSponsors->clear();
-    txtSponsors->document()->setDefaultStyleSheet( myStyle );
-    txtSponsors->setHtml( sponsorHTML );
-    QgsDebugMsg( QString( "sponsorHTML:%1" ).arg( sponsorHTML.toAscii().constData() ) );
-    QgsDebugMsg( QString( "txtSponsors:%1" ).arg( txtSponsors->toHtml().toAscii().constData() ) );
-  }
 
   // read the DONORS file and populate the text widget
   QFile donorsFile( QgsApplication::donorsFilePath() );
@@ -174,13 +125,11 @@ void QgsAbout::init()
   if ( donorsFile.open( QIODevice::ReadOnly ) )
   {
     QString donorsHTML = ""
-                         + tr( "<p>The following individuals and institutions have contributed "
-                               "money to fund QGIS development and other project costs</p>" )
-                         + "<hr>"
-                         "<table width='100%'>"
-                         "<tr><th>" + tr( "Name" ) + "</th>"
-                         "<th>" + tr( "Website" ) + "</th></tr>";
-    QString website;
+                         + tr( "<p>For a list of individuals and institutions who have contributed "
+                               "money to fund QGIS development and other project costs see "
+                               "<a href=\"http://qgis.org/en/sponsorship/donors.html\">"
+                               "http://qgis.org/en/sponsorship/donors.html</a></p>" );
+    /*QString website;
     QTextStream donorsStream( &donorsFile );
     // Always use UTF-8
     donorsStream.setCodec( "UTF-8" );
@@ -204,7 +153,7 @@ void QgsAbout::init()
       // close the row
       donorsHTML += "</tr>";
     }
-    donorsHTML += "</table>";
+    donorsHTML += "</table>";*/
 
     QString myStyle = QgsApplication::reportStyleSheet();
     txtDonors->clear();
@@ -262,6 +211,7 @@ void QgsAbout::init()
     QgsDebugMsg( QString( "translatorHTML:%1" ).arg( translatorHTML.toAscii().constData() ) );
     QgsDebugMsg( QString( "txtTranslators:%1" ).arg( txtTranslators->toHtml().toAscii().constData() ) );
   }
+  setWhatsNew();
 }
 
 void QgsAbout::setVersion( QString v )
@@ -269,12 +219,12 @@ void QgsAbout::setVersion( QString v )
   lblVersion->setText( v );
 }
 
-void QgsAbout::setWhatsNew( QString txt )
+void QgsAbout::setWhatsNew( )
 {
   QString myStyle = QgsApplication::reportStyleSheet();
   txtWhatsNew->clear();
   txtWhatsNew->document()->setDefaultStyleSheet( myStyle );
-  txtWhatsNew->setHtml( txt );
+  txtWhatsNew->setSource( "file:///" + QgsApplication::pkgDataPath() + "/doc/changelog.html" );
 }
 
 void QgsAbout::setPluginInfo()
@@ -291,7 +241,7 @@ void QgsAbout::setPluginInfo()
   myString += "</li>\n</ol>\n";
   //qt image plugins
   myString += "<b>" + tr( "Available Qt Image Plugins" ) + "</b><br>";
-  myString += tr( "Qt Image Plugin Search Paths	<br>" );
+  myString += tr( "Qt Image Plugin Search Paths <br>" );
   myString += QApplication::libraryPaths().join( "<br>" );
   myString += "<ol>\n<li>\n";
   QList<QByteArray> myImageFormats = QImageReader::supportedImageFormats();
@@ -310,11 +260,6 @@ void QgsAbout::setPluginInfo()
   txtProviders->setText( myString );
 }
 
-void QgsAbout::on_buttonCancel_clicked()
-{
-  reject();
-}
-
 void QgsAbout::on_btnQgisUser_clicked()
 {
   // find a browser
