@@ -24,6 +24,9 @@
 #include "qgsuniquevaluerenderer.h"
 #include "qgssymbol.h"
 #include "qgsattributeeditor.h"
+#include "qgshighlight.h"
+#include "qgssearchstring.h"
+#include "qgssearchtreenode.h"
 
 #include "qgisapp.h"
 
@@ -39,21 +42,27 @@
 #include <QUiLoader>
 #include <QDialog>
 #include <QVBoxLayout>
+#include <QLineEdit>
 
-QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeature )
-    : mDialog( 0 ),
-    mSettingsPath( "/Windows/AttributeDialog/" ),
-    mLayer( vl ),
-    mpFeature( thepFeature )
+int QgsAttributeDialog::smFormCounter = 0;
+
+QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeature, bool featureOwner )
+    : mDialog( 0 )
+    , mSettingsPath( "/Windows/AttributeDialog/" )
+    , mLayer( vl )
+    , mFeature( thepFeature )
+    , mFeatureOwner( featureOwner )
+    , mHighlight( 0 )
+    , mFormNr( -1 )
 {
-  if ( mpFeature == NULL || vl->dataProvider() == NULL )
+  if ( !mFeature || !vl->dataProvider() )
     return;
 
   const QgsFieldMap &theFieldMap = vl->pendingFields();
   if ( theFieldMap.isEmpty() )
     return;
 
-  QgsAttributeMap myAttributes = mpFeature->attributeMap();
+  QgsAttributeMap myAttributes = mFeature->attributeMap();
 
   QDialogButtonBox *buttonBox = NULL;
 
@@ -67,7 +76,7 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
 
       QFileInfo fi( vl->editForm() );
       loader.setWorkingDirectory( fi.dir() );
-      QWidget *myWidget = loader.load( &file, NULL );
+      QWidget *myWidget = loader.load( &file, QgisApp::instance() );
       file.close();
 
       mDialog = qobject_cast<QDialog*>( myWidget );
@@ -77,7 +86,7 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
 
   if ( !mDialog )
   {
-    mDialog = new QDialog();
+    mDialog = new QDialog( QgisApp::instance() );
 
     QGridLayout *gridLayout;
     QFrame *mFrame;
@@ -101,12 +110,12 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
     //
     //Set up dynamic inside a scroll box
     //
-    QVBoxLayout * mypOuterLayout = new QVBoxLayout();
+    QVBoxLayout *mypOuterLayout = new QVBoxLayout();
     mypOuterLayout->setContentsMargins( 0, 0, 0, 0 );
     //transfers layout ownership so no need to call delete
 
     mFrame->setLayout( mypOuterLayout );
-    QScrollArea * mypScrollArea = new QScrollArea();
+    QScrollArea *mypScrollArea = new QScrollArea();
     //transfers scroll area ownership so no need to call delete
     mypOuterLayout->addWidget( mypScrollArea );
     QFrame *mypInnerFrame = new QFrame();
@@ -115,7 +124,7 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
     //transfers frame ownership so no need to call delete
     mypScrollArea->setWidget( mypInnerFrame );
     mypScrollArea->setWidgetResizable( true );
-    QGridLayout * mypInnerLayout = new QGridLayout( mypInnerFrame );
+    QGridLayout *mypInnerLayout = new QGridLayout( mypInnerFrame );
 
     int index = 0;
     for ( QgsAttributeMap::const_iterator it = myAttributes.begin(); it != myAttributes.end(); ++it )
@@ -134,19 +143,26 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
       mypInnerLayout->addWidget( mypLabel, index, 0 );
       if ( myFieldType == QVariant::Int )
       {
-        mypLabel->setText( myFieldName + tr( " (int)" ) );
+        mypLabel->setText( myFieldName );
       }
       else if ( myFieldType == QVariant::Double )
       {
-        mypLabel->setText( myFieldName + tr( " (dbl)" ) );
+        mypLabel->setText( myFieldName );
+      }
+      else if ( myFieldType == QVariant::LongLong )
+      {
+        mypLabel->setText( myFieldName );
       }
       else //string
       {
         //any special behaviour for string goes here
-        mypLabel->setText( myFieldName + tr( " (txt)" ) );
+        mypLabel->setText( myFieldName );
       }
 
-      myWidget->setEnabled( vl->isEditable() );
+      if ( vl->editType( it.key() ) != QgsVectorLayer::Immutable )
+      {
+        myWidget->setEnabled( vl->isEditable() );
+      }
 
       mypInnerLayout->addWidget( myWidget, index, 1 );
       mpIndizes << it.key();
@@ -171,11 +187,56 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
 
       QgsAttributeEditor::createAttributeEditor( mDialog, myWidget, vl, it.key(), it.value() );
 
-      myWidget->setEnabled( vl->isEditable() );
+      if ( vl->editType( it.key() ) != QgsVectorLayer::Immutable )
+      {
+        myWidget->setEnabled( vl->isEditable() );
+      }
 
       mpIndizes << it.key();
       mpWidgets << myWidget;
     }
+
+    foreach( QLineEdit *le, mDialog->findChildren<QLineEdit*>() )
+    {
+      if ( !le->objectName().startsWith( "expr_" ) )
+        continue;
+
+      le->setReadOnly( true );
+      QString expr = le->text();
+      le->setText( tr( "Error" ) );
+
+      QgsSearchString ss;
+      if ( !ss.setString( expr ) )
+        continue;
+
+      QgsSearchTreeNode *st = ss.tree();
+      if ( !st )
+        continue;
+
+      if ( !mFeature->geometry() && st->needsGeometry() )
+      {
+        QgsFeature f;
+        if ( vl->featureAtId( mFeature->id(), f, true, false ) && f.geometry() )
+        {
+          mFeature->setGeometry( *f.geometry() );
+        }
+      }
+
+      QgsSearchTreeValue value;
+      st->getValue( value, st, vl->pendingFields(), *mFeature );
+
+      if ( !value.isError() )
+      {
+        if ( value.isNumeric() )
+          le->setText( QString::number( value.number() ) );
+        else
+          le->setText( value.string() );
+      }
+      else
+      {
+        le->setText( tr( "Error: %1" ).arg( st->errorMsg() ) );
+      }
+    }
   }
 
   if ( mDialog )
@@ -224,9 +285,10 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
       QgisApp::instance()->runPythonString( QString( "import %1" ).arg( module.left( pos ) ) );
     }
 
-    QgisApp::instance()->runPythonString( QString( "_qgis_featureform_%1 = wrapinstance( %2, QtGui.QDialog )" ).arg( mLayer->getLayerID() ).arg(( unsigned long ) mDialog ) );
+    mFormNr = smFormCounter++;
+    QgisApp::instance()->runPythonString( QString( "_qgis_featureform_%1 = wrapinstance( %2, QtGui.QDialog )" ).arg( mFormNr ).arg(( unsigned long ) mDialog ) );
 
-    QString expr = QString( "%1(_qgis_featureform_%2,'%2',%3)" ).arg( vl->editFormInit() ).arg( vl->getLayerID() ).arg( mpFeature->id() );
+    QString expr = QString( "%1(_qgis_featureform_%2,'%3',%4)" ).arg( vl->editFormInit() ).arg( mFormNr ).arg( vl->id() ).arg( mFeature->id() );
     QgsDebugMsg( QString( "running featureForm init: %1" ).arg( expr ) );
     QgisApp::instance()->runPythonString( expr );
   }
@@ -237,16 +299,32 @@ QgsAttributeDialog::QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeat
 
 QgsAttributeDialog::~QgsAttributeDialog()
 {
+  if ( mHighlight )
+  {
+    mHighlight->hide();
+    delete mHighlight;
+  }
+
+  if ( mFeatureOwner )
+  {
+    delete mFeature;
+  }
+
   saveGeometry();
+
+  if ( mDialog )
+  {
+    delete mDialog;
+  }
 }
 
 void QgsAttributeDialog::accept()
 {
-  if ( !mLayer->isEditable() )
+  if ( !mLayer->isEditable() || !mFeature )
     return;
 
   //write the new values back to the feature
-  QgsAttributeMap myAttributes = mpFeature->attributeMap();
+  QgsAttributeMap myAttributes = mFeature->attributeMap();
   int myIndex = 0;
   for ( QgsAttributeMap::const_iterator it = myAttributes.begin(); it != myAttributes.end(); ++it )
   {
@@ -254,7 +332,7 @@ void QgsAttributeDialog::accept()
 
     int idx = mpIndizes.value( myIndex );
     if ( QgsAttributeEditor::retrieveValue( mpWidgets.value( myIndex ), mLayer, idx, value ) )
-      mpFeature->changeAttribute( idx, value );
+      mFeature->changeAttribute( idx, value );
 
     ++myIndex;
   }
@@ -273,6 +351,18 @@ int QgsAttributeDialog::exec()
   }
 }
 
+void QgsAttributeDialog::show()
+{
+  if ( mDialog )
+  {
+    mDialog->setAttribute( Qt::WA_DeleteOnClose );
+    mDialog->show();
+    mDialog->raise();
+    mDialog->activateWindow();
+    mDialog->installEventFilter( this );
+  }
+}
+
 void QgsAttributeDialog::saveGeometry()
 {
   if ( mDialog )
@@ -291,12 +381,49 @@ void QgsAttributeDialog::restoreGeometry()
   }
 }
 
+void QgsAttributeDialog::setHighlight( QgsHighlight *h )
+{
+  if ( mHighlight )
+  {
+    delete mHighlight;
+  }
+
+  mHighlight = h;
+}
+
+
 void QgsAttributeDialog::dialogDestroyed()
 {
 #if 0
   mLayer->setProperty( "featureForm.dialog", QVariant() );
   mLayer->setProperty( "featureForm.id", QVariant() );
 #endif
-  QgisApp::instance()->runPythonString( QString( "del _qgis_featureform_%1" ).arg( mLayer->getLayerID() ) );
+  if ( -1 < mFormNr )
+  {
+    QString expr = QString( "if locals().has_key('_qgis_featureform_%1'): del _qgis_featureform_%1\n" ).arg( mFormNr );
+    QgisApp::instance()->runPythonString( expr );
+  }
+
   mDialog = NULL;
+  deleteLater();
+}
+
+bool QgsAttributeDialog::eventFilter( QObject *obj, QEvent *e )
+{
+  if ( mHighlight && obj == mDialog )
+  {
+    switch ( e->type() )
+    {
+      case QEvent::WindowActivate:
+        mHighlight->show();
+        break;
+      case QEvent::WindowDeactivate:
+        mHighlight->hide();
+        break;
+      default:
+        break;
+    }
+  }
+
+  return false;
 }
