@@ -25,13 +25,14 @@ class QgsFeature;
 class QLayout;
 class QgsField;
 class QgsVectorLayer;
+class QgsHighlight;
 
 class QgsAttributeDialog : public QObject
 {
     Q_OBJECT
 
   public:
-    QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature * thepFeature );
+    QgsAttributeDialog( QgsVectorLayer *vl, QgsFeature *thepFeature, bool featureOwner );
     ~QgsAttributeDialog();
 
     /** Saves the size and position for the next time
@@ -45,26 +46,31 @@ class QgsAttributeDialog : public QObject
      */
     void restoreGeometry();
 
+    void setHighlight( QgsHighlight *h );
+
     QDialog *dialog() { return mDialog; }
 
   public slots:
-    /** Overloaded accept method which will write the feature field
-     * values, then delegate to QDialog::accept()
-     */
     void accept();
 
     int exec();
+    void show();
 
     void dialogDestroyed();
 
   private:
+    bool eventFilter( QObject *obj, QEvent *event );
 
     QDialog *mDialog;
     QString mSettingsPath;
     QList<QWidget *> mpWidgets;
     QList<int> mpIndizes;
     QgsVectorLayer *mLayer;
-    QgsFeature *mpFeature;
+    QgsFeature *mFeature;
+    bool mFeatureOwner;
+    QgsHighlight *mHighlight;
+    int mFormNr;
+    static int smFormCounter;
 };
 
 #endif
