@@ -17,10 +17,9 @@
 /* $Id$ */
 
 #include "qgsattributetypedialog.h"
-
 #include "qgsattributetypeloaddialog.h"
-
 #include "qgsvectordataprovider.h"
+
 #include "qgslogger.h"
 
 #include <QTableWidgetItem>
@@ -33,19 +32,18 @@
 #include <cfloat>
 
 QgsAttributeTypeDialog::QgsAttributeTypeDialog( QgsVectorLayer *vl )
-    : QDialog(),
-    mLayer( vl )
+    : QDialog()
+    , mLayer( vl )
 {
   setupUi( this );
   tableWidget->insertRow( 0 );
   connect( selectionComboBox, SIGNAL( currentIndexChanged( int ) ), this, SLOT( setStackPage( int ) ) );
-  connect( removeSelectedButton, SIGNAL( pressed( ) ), this, SLOT( removeSelectedButtonPushed( ) ) );
-  connect( loadFromLayerButton, SIGNAL( pressed( ) ), this, SLOT( loadFromLayerButtonPushed( ) ) );
-  connect( loadFromCSVButton, SIGNAL( pressed( ) ), this, SLOT( loadFromCSVButtonPushed( ) ) );
-  connect( tableWidget,  SIGNAL( cellChanged( int, int ) ), this, SLOT( vCellChanged( int, int ) ) );
+  connect( removeSelectedButton, SIGNAL( clicked() ), this, SLOT( removeSelectedButtonPushed() ) );
+  connect( loadFromLayerButton, SIGNAL( clicked() ), this, SLOT( loadFromLayerButtonPushed() ) );
+  connect( loadFromCSVButton, SIGNAL( clicked() ), this, SLOT( loadFromCSVButtonPushed() ) );
+  connect( tableWidget, SIGNAL( cellChanged( int, int ) ), this, SLOT( vCellChanged( int, int ) ) );
 }
 
-
 QgsAttributeTypeDialog::~QgsAttributeTypeDialog()
 {
 
@@ -88,7 +86,6 @@ void QgsAttributeTypeDialog::vCellChanged( int row, int column )
 void QgsAttributeTypeDialog::removeSelectedButtonPushed()
 {
   QList<QTableWidgetItem *> list = tableWidget->selectedItems();
-  QList<QTableWidgetItem *>::iterator it = list.begin();
   QSet<int> rowsToRemove;
   int removed = 0;
   int i = 0;
@@ -211,12 +208,17 @@ void QgsAttributeTypeDialog::setPageForEditType( QgsVectorLayer::EditType editTy
 {
   switch ( editType )
   {
+    case QgsVectorLayer::LineEdit:
+      setPage( 0 );
+      break;
+
     case QgsVectorLayer::Classification:
       setPage( 1 );
       break;
 
     case QgsVectorLayer::EditRange:
     case QgsVectorLayer::SliderRange:
+    case QgsVectorLayer::DialRange:
       setPage( 2 );
       break;
 
@@ -251,18 +253,14 @@ void QgsAttributeTypeDialog::setPageForEditType( QgsVectorLayer::EditType editTy
 
     case QgsVectorLayer::TextEdit:
       setPage( 10 );
+      break;
 
-    case QgsVectorLayer::LineEdit:
-      setPage( 0 );
+    case QgsVectorLayer::Calendar:
+      setPage( 11 );
       break;
   }
 }
 
-void QgsAttributeTypeDialog::setPageForIndex( int index )
-{
-  setPageForEditType( mLayer->editType( index ) );
-}
-
 void QgsAttributeTypeDialog::setValueMap( QMap<QString, QVariant> valueMap )
 {
   mValueMap = valueMap;
@@ -273,19 +271,10 @@ void QgsAttributeTypeDialog::setRange( QgsVectorLayer::RangeData range )
   mRangeData = range;
 }
 
-void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
+void QgsAttributeTypeDialog::setIndex( int index, QgsVectorLayer::EditType editType )
 {
   mIndex = index;
   //need to set index for combobox
-  QgsVectorLayer::EditType editType;
-  if ( editTypeInt > -1 )
-  {
-    editType = QgsVectorLayer::EditType( editTypeInt );
-  }
-  else
-  {
-    editType = mLayer->editType( index );
-  }
 
   setWindowTitle( defaultWindowTitle() + " \"" + mLayer->pendingFields()[index].name() + "\"" );
   QgsAttributeList attributeList = QgsAttributeList();
@@ -297,7 +286,8 @@ void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
   //calculate min and max for range for this field
   if ( mLayer->pendingFields()[index].type() == QVariant::Int )
   {
-    sliderRadioButton->setDisabled( false );
+    rangeWidget->clear();
+    rangeWidget->addItems( QStringList() << tr( "Editable" ) << tr( "Slider" ) << tr( "Dial" ) );
     int min = INT_MIN;
     int max = INT_MAX;
     while ( mLayer->nextFeature( f ) )
@@ -319,8 +309,8 @@ void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
     double dMin = -DBL_MAX;
     double dMax = DBL_MAX;
 
-    sliderRadioButton->setDisabled( true );
-    editableRadioButton->setChecked( true );
+    rangeWidget->clear();
+    rangeWidget->addItems( QStringList() << tr( "Editable" ) << tr( "Slider" ) );
     while ( mLayer->nextFeature( f ) )
     {
       QVariant val = f.attributeMap()[index];
@@ -341,33 +331,20 @@ void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
   }
   valuesLabel->setText( text );
 
-  //setPageForIndex( index );
   setPageForEditType( editType );
 
   switch ( editType )
   {
     case QgsVectorLayer::ValueMap:
     {
-
       tableWidget->clearContents();
       for ( int i = tableWidget->rowCount() - 1; i > 0; i-- )
       {
         tableWidget->removeRow( i );
       }
 
-      // if some value map already present use it
-      QMap<QString, QVariant> map;
-      if ( !mValueMap.empty() )
-      {
-        map = mValueMap;
-      }
-      else
-      {
-        map = mLayer->valueMap( index );
-      }
-
       int row = 0;
-      for ( QMap<QString, QVariant>::iterator mit = map.begin(); mit != map.end(); mit++, row++ )
+      for ( QMap<QString, QVariant>::iterator mit = mValueMap.begin(); mit != mValueMap.end(); mit++, row++ )
       {
         tableWidget->insertRow( row );
         if ( mit.value().isNull() )
@@ -386,23 +363,32 @@ void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
 
     case QgsVectorLayer::EditRange:
     case QgsVectorLayer::SliderRange:
+    case QgsVectorLayer::DialRange:
     {
       if ( mLayer->pendingFields()[mIndex].type() != QVariant::Int )
       {
-        minimumSpinBox->setValue( mLayer->range( index ).mMin.toInt() );
-        maximumSpinBox->setValue( mLayer->range( index ).mMax.toInt() );
-        stepSpinBox->setValue( mLayer->range( index ).mStep.toInt() );
+        minimumSpinBox->setValue( mRangeData.mMin.toInt() );
+        maximumSpinBox->setValue( mRangeData.mMax.toInt() );
+        stepSpinBox->setValue( mRangeData.mStep.toInt() );
       }
       else if ( mLayer->pendingFields()[mIndex].type() == QVariant::Double )
       {
-        minimumDoubleSpinBox->setValue( mLayer->range( index ).mMin.toDouble() );
-        maximumDoubleSpinBox->setValue( mLayer->range( index ).mMax.toDouble() );
-        stepDoubleSpinBox->setValue( mLayer->range( index ).mStep.toDouble() );
+        minimumDoubleSpinBox->setValue( mRangeData.mMin.toDouble() );
+        maximumDoubleSpinBox->setValue( mRangeData.mMax.toDouble() );
+        stepDoubleSpinBox->setValue( mRangeData.mStep.toDouble() );
       }
       if ( editType == QgsVectorLayer::EditRange )
-        editableRadioButton->setChecked( true );
-      else //slider range
-        sliderRadioButton->setChecked( true );
+      {
+        rangeWidget->setCurrentIndex( 0 );
+      }
+      else if ( editType == QgsVectorLayer::SliderRange )
+      {
+        rangeWidget->setCurrentIndex( 1 );
+      }
+      else
+      {
+        rangeWidget->setCurrentIndex( 2 );
+      }
     }
     break;
 
@@ -410,7 +396,16 @@ void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
       editableUniqueValues->setChecked( editType == QgsVectorLayer::UniqueValuesEditable );
       break;
 
-    default:
+    case QgsVectorLayer::LineEdit:
+    case QgsVectorLayer::UniqueValues:
+    case QgsVectorLayer::Classification:
+    case QgsVectorLayer::CheckBox:
+    case QgsVectorLayer::FileName:
+    case QgsVectorLayer::Enumeration:
+    case QgsVectorLayer::Immutable:
+    case QgsVectorLayer::Hidden:
+    case QgsVectorLayer::TextEdit:
+    case QgsVectorLayer::Calendar:
       break;
   }
 }
@@ -418,13 +413,13 @@ void QgsAttributeTypeDialog::setIndex( int index, int editTypeInt )
 
 void QgsAttributeTypeDialog::setPage( int index )
 {
-  this->selectionComboBox->setCurrentIndex( index );
+  selectionComboBox->setCurrentIndex( index );
   setStackPage( index );
 }
 
 void QgsAttributeTypeDialog::setStackPage( int index )
 {
-  this->stackedWidget->setCurrentIndex( index );
+  stackedWidget->setCurrentIndex( index );
 
   bool okDisabled = false;
   if ( index == 2 )
@@ -436,7 +431,7 @@ void QgsAttributeTypeDialog::setStackPage( int index )
     }
     else if ( mLayer->pendingFields()[mIndex].type() != QVariant::Double )
     {
-      this->rangeStackedWidget->setCurrentIndex( 0 );
+      rangeStackedWidget->setCurrentIndex( 0 );
       //load data
       minimumSpinBox->setValue( mRangeData.mMin.toInt() );
       maximumSpinBox->setValue( mRangeData.mMax.toInt() );
@@ -444,7 +439,7 @@ void QgsAttributeTypeDialog::setStackPage( int index )
     }
     else
     {
-      this->rangeStackedWidget->setCurrentIndex( 1 );
+      rangeStackedWidget->setCurrentIndex( 1 );
       //load data
       minimumDoubleSpinBox->setValue( mRangeData.mMin.toDouble() );
       maximumDoubleSpinBox->setValue( mRangeData.mMax.toDouble() );
@@ -497,13 +492,17 @@ void QgsAttributeTypeDialog::accept()
                                                 stepDoubleSpinBox->value() );
       }
       //select correct one
-      if ( editableRadioButton->isChecked() )
+      switch ( rangeWidget->currentIndex() )
       {
-        mEditType = QgsVectorLayer::EditRange;
-      }
-      else
-      {
-        mEditType = QgsVectorLayer::SliderRange;
+        case 0:
+          mEditType = QgsVectorLayer::EditRange;
+          break;
+        case 1:
+          mEditType = QgsVectorLayer::SliderRange;
+          break;
+        case 2:
+          mEditType = QgsVectorLayer::DialRange;
+          break;
       }
       break;
     case 3:
@@ -524,13 +523,19 @@ void QgsAttributeTypeDialog::accept()
       mValueMap.clear();
       for ( int i = 0; i < tableWidget->rowCount() - 1; i++ )
       {
-        if ( tableWidget->item( i, 1 )->text().isNull() )
+        QTableWidgetItem *ki = tableWidget->item( i, 0 );
+        QTableWidgetItem *vi = tableWidget->item( i, 1 );
+
+        if ( !ki )
+          continue;
+
+        if ( !vi || vi->text().isNull() )
         {
-          mValueMap.insert( tableWidget->item( i, 0 )->text(), tableWidget->item( i, 0 )->text() );
+          mValueMap.insert( ki->text(), ki->text() );
         }
         else
         {
-          mValueMap.insert( tableWidget->item( i, 1 )->text(), tableWidget->item( i, 0 )->text() );
+          mValueMap.insert( vi->text(), ki->text() );
         }
       }
       mEditType = QgsVectorLayer::ValueMap;
@@ -550,6 +555,9 @@ void QgsAttributeTypeDialog::accept()
     case 10:
       mEditType = QgsVectorLayer::TextEdit;
       break;
+    case 11:
+      mEditType = QgsVectorLayer::Calendar;
+      break;
   }
 
   QDialog::accept();
