@@ -71,10 +71,11 @@ void QgsAttributeTypeLoadDialog::previewButtonPushed()
 void QgsAttributeTypeLoadDialog::fillLayerList()
 {
   layerComboBox->clear();
-  QMap<QString, QgsMapLayer*>::iterator layer_it = QgsMapLayerRegistry::instance()->mapLayers().begin();
-  for ( ; layer_it != QgsMapLayerRegistry::instance()->mapLayers().end(); layer_it++ )
+  foreach( QgsMapLayer *l, QgsMapLayerRegistry::instance()->mapLayers() )
   {
-    layerComboBox->addItem( layer_it.key() );
+    QgsVectorLayer *vl = qobject_cast< QgsVectorLayer * >( l );
+    if ( vl )
+      layerComboBox->addItem( vl->name(), vl->id() );
   }
 }
 
@@ -89,7 +90,7 @@ void QgsAttributeTypeLoadDialog::fillComboBoxes( int layerIndex )
     return;
   }
 
-  QgsMapLayer* dataLayer = QgsMapLayerRegistry::instance()->mapLayer( layerComboBox->currentText() );
+  QgsMapLayer* dataLayer = QgsMapLayerRegistry::instance()->mapLayer( layerComboBox->itemData( layerComboBox->currentIndex() ).toString() );
   QgsVectorLayer* vLayer = qobject_cast<QgsVectorLayer *>( dataLayer );
   if ( vLayer == NULL )
   {
@@ -120,7 +121,7 @@ void QgsAttributeTypeLoadDialog::createPreview( int fieldIndex, bool full )
   }
   int idx = keyComboBox->itemData( keyComboBox->currentIndex() ).toInt();
   int idx2 = valueComboBox->itemData( valueComboBox->currentIndex() ).toInt();
-  QgsMapLayer* dataLayer = QgsMapLayerRegistry::instance()->mapLayer( layerComboBox->currentText() );
+  QgsMapLayer* dataLayer = QgsMapLayerRegistry::instance()->mapLayer( layerComboBox->itemData( layerComboBox->currentIndex() ).toString() );
   QgsVectorLayer* vLayer = qobject_cast<QgsVectorLayer *>( dataLayer );
   if ( vLayer == NULL )
   {
@@ -170,7 +171,7 @@ void QgsAttributeTypeLoadDialog::loadDataToValueMap()
   mValueMap.clear();
   int idx = keyComboBox->itemData( keyComboBox->currentIndex() ).toInt();
   int idx2 = valueComboBox->itemData( valueComboBox->currentIndex() ).toInt();
-  QgsMapLayer* dataLayer = QgsMapLayerRegistry::instance()->mapLayer( layerComboBox->currentText() );
+  QgsMapLayer* dataLayer = QgsMapLayerRegistry::instance()->mapLayer( layerComboBox->itemData( layerComboBox->currentIndex() ).toString() );
   QgsVectorLayer* vLayer = qobject_cast<QgsVectorLayer *>( dataLayer );
   if ( vLayer == NULL )
   {
