@@ -23,6 +23,7 @@
 #include <QString>
 #include <QStringList>
 #include <QClipboard>
+#include <QSettings>
 
 #include "qgsclipboard.h"
 #include "qgsfeature.h"
@@ -30,7 +31,6 @@
 #include "qgsgeometry.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgslogger.h"
-#include "qgslogger.h"
 
 
 QgsClipboard::QgsClipboard()
@@ -75,7 +75,10 @@ void QgsClipboard::replaceWithCopyOf( const QgsFieldMap& fields, QgsFeatureList&
     if ( it->geometry() )
       textFields += it->geometry()->exportToWkt();
     else
-      textFields += "NULL";
+    {
+      QSettings settings;
+      textFields += settings.value( "qgis/nullValue", "NULL" ).toString();
+    }
 
     // QgsDebugMsg("about to traverse fields.");
     //
