@@ -43,8 +43,33 @@ QgsConfigureShortcutsDialog::QgsConfigureShortcutsDialog( QWidget* parent )
            this, SLOT( actionChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ) );
 
   populateActions();
+
+  restoreState();
 }
 
+QgsConfigureShortcutsDialog::~QgsConfigureShortcutsDialog()
+{
+    saveState();
+}
+
+/*!
+ * Function to save dialog window state
+ */
+void QgsConfigureShortcutsDialog::saveState()
+{
+    QSettings settings;
+    settings.setValue( "/Windows/ShortcutsDialog/geometry", saveGeometry() );
+}
+
+/*!
+ * Function to restore dialog window state
+ */
+void QgsConfigureShortcutsDialog::restoreState()
+{
+    QSettings settings;
+    restoreGeometry( settings.value( "/Windows/ShortcutsDialog/geometry" ).toByteArray() );
+}       
+
 void QgsConfigureShortcutsDialog::populateActions()
 {
   QList<QAction*> actions = QgsShortcutsManager::instance()->listActions();
@@ -73,7 +98,7 @@ void QgsConfigureShortcutsDialog::populateActions()
 
 void QgsConfigureShortcutsDialog::saveShortcuts()
 {
-  QString fileName = QFileDialog::getSaveFileName( this, tr( "Save shortcuts" ), ".", tr( "XML file (*.xml);; All files (*.*)" ) );
+  QString fileName = QFileDialog::getSaveFileName( this, tr( "Save shortcuts" ), ".", tr( "XML file (*.xml);; All files (*)" ) );
 
   if ( fileName.isEmpty() )
     return;
@@ -125,7 +150,7 @@ void QgsConfigureShortcutsDialog::saveShortcuts()
 
 void QgsConfigureShortcutsDialog::loadShortcuts()
 {
-  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load shortcuts" ), ".", tr( "XML file (*.xml);; All files (*.*)" ) );
+  QString fileName = QFileDialog::getOpenFileName( this, tr( "Load shortcuts" ), ".", tr( "XML file (*.xml);; All files (*)" ) );
 
   if ( fileName.isEmpty() )
   {
