@@ -22,11 +22,13 @@
 #include "qgisapp.h"
 #include "qgsmaplayer.h"
 #include "qgsvectorlayer.h"
-#include "qgsrubberband.h"
+#include "qgsrasterlayer.h"
+#include "qgshighlight.h"
 #include "qgsgeometry.h"
 #include "qgsattributedialog.h"
 #include "qgsmapcanvas.h"
 #include "qgsattributeaction.h"
+#include "qgsfeatureaction.h"
 
 #include <QCloseEvent>
 #include <QLabel>
@@ -39,22 +41,10 @@
 #include <QDockWidget>
 #include <QMenuBar>
 #include <QPushButton>
+#include <QTextBrowser>
 
 #include "qgslogger.h"
 
-static void _runPythonString( const QString &expr )
-{
-  QgisApp::instance()->runPythonString( expr );
-}
-
-void QgsFeatureAction::execute()
-{
-  int idx;
-  QList< QPair<QString, QString> > attributes;
-  mResults->retrieveAttributes( mFeatItem, attributes, idx );
-  mLayer->actions()->doAction( mAction, attributes, idx, _runPythonString );
-}
-
 class QgsIdentifyResultsDock : public QDockWidget
 {
   public:
@@ -73,15 +63,15 @@ class QgsIdentifyResultsDock : public QDockWidget
 // Tree hierarchy
 //
 // layer [userrole: QgsMapLayer]
-//   feature: displayfield|displayvalue [userrole: fid]
+//   feature: displayfield|displayvalue [userrole: fid, index in feature list]
 //     derived attributes (if any) [userrole: "derived"]
 //       name value
 //     actions (if any) [userrole: "actions"]
 //       edit [userrole: "edit"]
 //       action [userrole: "action", idx]
-//     name value
-//     name value
-//     name value
+//     displayname [userroles: fieldIdx, original name] displayvalue [userrole: original value]
+//     displayname [userroles: fieldIdx, original name] displayvalue [userrole: original value]
+//     displayname [userroles: fieldIdx, original name] displayvalue [userrole: original value]
 //   feature
 //     derived attributes (if any)
 //       name value
@@ -97,6 +87,7 @@ QgsIdentifyResults::QgsIdentifyResults( QgsMapCanvas *canvas, QWidget *parent, Q
 {
   setupUi( this );
   QSettings mySettings;
+  restoreGeometry( mySettings.value( "/Windows/Identify/geometry" ).toByteArray() );
   bool myDockFlag = mySettings.value( "/qgis/dockIdentifyResults", false ).toBool();
   if ( myDockFlag )
   {
@@ -123,7 +114,7 @@ QgsIdentifyResults::QgsIdentifyResults( QgsMapCanvas *canvas, QWidget *parent, Q
 
 QgsIdentifyResults::~QgsIdentifyResults()
 {
-  clearRubberbands();
+  clearHighlights();
   if ( mActionPopup )
     delete mActionPopup;
 }
@@ -141,42 +132,76 @@ QTreeWidgetItem *QgsIdentifyResults::layerItem( QObject *layer )
   return 0;
 }
 
-void QgsIdentifyResults::addFeature( QgsMapLayer *layer, int fid,
-                                     QString displayField, QString displayValue,
-                                     const QMap<QString, QString> &attributes,
+void QgsIdentifyResults::addFeature( QgsVectorLayer *vlayer,
+                                     const QgsFeature &f,
                                      const QMap<QString, QString> &derivedAttributes )
 {
-  QTreeWidgetItem *layItem = layerItem( layer );
-  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer );
+  QTreeWidgetItem *layItem = layerItem( vlayer );
 
   if ( layItem == 0 )
   {
-    layItem = new QTreeWidgetItem( QStringList() << layer->name() << tr( "Layer" ) );
-    layItem->setData( 0, Qt::UserRole, QVariant::fromValue( qobject_cast<QObject *>( layer ) ) );
+    layItem = new QTreeWidgetItem( QStringList() << QString::number( lstResults->topLevelItemCount() ) << vlayer->name() );
+    layItem->setData( 0, Qt::UserRole, QVariant::fromValue( qobject_cast<QObject *>( vlayer ) ) );
     lstResults->addTopLevelItem( layItem );
 
-    QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( layer );
-    if ( vlayer )
-    {
-      connect( vlayer, SIGNAL( layerDeleted() ), this, SLOT( layerDestroyed() ) );
-      connect( vlayer, SIGNAL( layerCrsChanged() ), this, SLOT( layerDestroyed() ) );
-      connect( vlayer, SIGNAL( featureDeleted( int ) ), this, SLOT( featureDeleted( int ) ) );
-      connect( vlayer, SIGNAL( editingStarted() ), this, SLOT( editingToggled() ) );
-      connect( vlayer, SIGNAL( editingStopped() ), this, SLOT( editingToggled() ) );
-    }
-    else
-    {
-      connect( layer, SIGNAL( destroyed() ), this, SLOT( layerDestroyed() ) );
-      connect( layer, SIGNAL( layerCrsChanged() ), this, SLOT( layerDestroyed() ) );
-    }
+    connect( vlayer, SIGNAL( layerDeleted() ), this, SLOT( layerDestroyed() ) );
+    connect( vlayer, SIGNAL( layerCrsChanged() ), this, SLOT( layerDestroyed() ) );
+    connect( vlayer, SIGNAL( featureDeleted( int ) ), this, SLOT( featureDeleted( int ) ) );
+    connect( vlayer, SIGNAL( attributeValueChanged( int, int, const QVariant & ) ), this, SLOT( attributeValueChanged( int, int, const QVariant & ) ) );
+    connect( vlayer, SIGNAL( editingStarted() ), this, SLOT( editingToggled() ) );
+    connect( vlayer, SIGNAL( editingStopped() ), this, SLOT( editingToggled() ) );
   }
 
-  QTreeWidgetItem *featItem = new QTreeWidgetItem( QStringList() << displayField << displayValue );
-  featItem->setData( 0, Qt::UserRole, fid );
+  QTreeWidgetItem *featItem = new QTreeWidgetItem;
+  featItem->setData( 0, Qt::UserRole, f.id() );
+  featItem->setData( 0, Qt::UserRole + 1, mFeatures.size() );
+  mFeatures << f;
+  layItem->addChild( featItem );
 
-  for ( QMap<QString, QString>::const_iterator it = attributes.begin(); it != attributes.end(); it++ )
+  for ( QgsAttributeMap::const_iterator it = f.attributeMap().begin(); it != f.attributeMap().end(); it++ )
   {
-    featItem->addChild( new QTreeWidgetItem( QStringList() << it.key() << it.value() ) );
+    QTreeWidgetItem *attrItem = new QTreeWidgetItem( QStringList() << QString::number( it.key() ) << it.value().toString() );
+
+    const QgsFieldMap &fields = vlayer->pendingFields();
+
+    QgsFieldMap::const_iterator fit = fields.find( it.key() );
+    if ( fit == fields.constEnd() )
+    {
+      delete attrItem;
+      continue;
+    }
+
+    attrItem->setData( 0, Qt::DisplayRole, vlayer->attributeDisplayName( it.key() ) );
+    attrItem->setData( 0, Qt::UserRole, fit->name() );
+    attrItem->setData( 0, Qt::UserRole + 1, it.key() );
+
+    QVariant value = it.value();
+    attrItem->setData( 1, Qt::UserRole, value );
+
+    switch ( vlayer->editType( it.key() ) )
+    {
+      case QgsVectorLayer::Hidden:
+        // skip the item
+        delete attrItem;
+        continue;
+
+      case QgsVectorLayer::ValueMap:
+        value = vlayer->valueMap( it.key() ).key( it->toString(), QString( "(%1)" ).arg( it->toString() ) );
+        break;
+
+      default:
+        break;
+    }
+
+    attrItem->setData( 1, Qt::DisplayRole, value );
+
+    if ( fit->name() == vlayer->displayField() )
+    {
+      featItem->setText( 0, attrItem->text( 0 ) );
+      featItem->setText( 1, attrItem->text( 1 ) );
+    }
+
+    featItem->addChild( attrItem );
   }
 
   if ( derivedAttributes.size() >= 0 )
@@ -191,16 +216,19 @@ void QgsIdentifyResults::addFeature( QgsMapLayer *layer, int fid,
     }
   }
 
-  if ( vlayer )
+  if ( vlayer->pendingFields().size() > 0 || vlayer->actions()->size() )
   {
     QTreeWidgetItem *actionItem = new QTreeWidgetItem( QStringList() << tr( "(Actions)" ) );
     actionItem->setData( 0, Qt::UserRole, "actions" );
     featItem->addChild( actionItem );
 
-    QTreeWidgetItem *editItem = new QTreeWidgetItem( QStringList() << "" << ( vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ) ) );
-    editItem->setIcon( 0, QgisApp::getThemeIcon( vlayer->isEditable() ? "/mIconEditable.png" : "/mIconEditable.png" ) );
-    editItem->setData( 0, Qt::UserRole, "edit" );
-    actionItem->addChild( editItem );
+    if ( vlayer->pendingFields().size() > 0 )
+    {
+      QTreeWidgetItem *editItem = new QTreeWidgetItem( QStringList() << "" << ( vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ) ) );
+      editItem->setIcon( 0, QgisApp::getThemeIcon( vlayer->isEditable() ? "/mIconEditable.png" : "/mIconEditable.png" ) );
+      editItem->setData( 0, Qt::UserRole, "edit" );
+      actionItem->addChild( editItem );
+    }
 
     for ( int i = 0; i < vlayer->actions()->size(); i++ )
     {
@@ -217,9 +245,58 @@ void QgsIdentifyResults::addFeature( QgsMapLayer *layer, int fid,
     }
   }
 
+  highlightFeature( featItem );
+}
+
+void QgsIdentifyResults::addFeature( QgsRasterLayer *layer,
+                                     QString label,
+                                     const QMap<QString, QString> &attributes,
+                                     const QMap<QString, QString> &derivedAttributes )
+{
+  QTreeWidgetItem *layItem = layerItem( layer );
+
+  if ( layItem == 0 )
+  {
+    layItem = new QTreeWidgetItem( QStringList() << QString::number( lstResults->topLevelItemCount() ) << layer->name() );
+    layItem->setData( 0, Qt::UserRole, QVariant::fromValue( qobject_cast<QObject *>( layer ) ) );
+    lstResults->addTopLevelItem( layItem );
+
+    connect( layer, SIGNAL( destroyed() ), this, SLOT( layerDestroyed() ) );
+    connect( layer, SIGNAL( layerCrsChanged() ), this, SLOT( layerDestroyed() ) );
+  }
+
+  QTreeWidgetItem *featItem = new QTreeWidgetItem( QStringList() << label << "" );
+  featItem->setData( 0, Qt::UserRole, -1 );
   layItem->addChild( featItem );
 
-  highlightFeature( featItem );
+  if ( layer && layer->providerKey() == "wms" )
+  {
+    QTreeWidgetItem *attrItem = new QTreeWidgetItem( QStringList() << attributes.begin().key() << "" );
+    featItem->addChild( attrItem );
+
+    QTextBrowser *tb = new QTextBrowser( attrItem->treeWidget() );
+    tb->setHtml( attributes.begin().value() );
+    attrItem->treeWidget()->setItemWidget( attrItem, 1, tb );
+  }
+  else
+  {
+    for ( QMap<QString, QString>::const_iterator it = attributes.begin(); it != attributes.end(); it++ )
+    {
+      featItem->addChild( new QTreeWidgetItem( QStringList() << it.key() << it.value() ) );
+    }
+  }
+
+  if ( derivedAttributes.size() >= 0 )
+  {
+    QTreeWidgetItem *derivedItem = new QTreeWidgetItem( QStringList() << tr( "(Derived)" ) );
+    derivedItem->setData( 0, Qt::UserRole, "derived" );
+    featItem->addChild( derivedItem );
+
+    for ( QMap< QString, QString>::const_iterator it = derivedAttributes.begin(); it != derivedAttributes.end(); it++ )
+    {
+      derivedItem->addChild( new QTreeWidgetItem( QStringList() << it.key() << it.value() ) );
+    }
+  }
 }
 
 void QgsIdentifyResults::editingToggled()
@@ -259,7 +336,7 @@ void QgsIdentifyResults::editingToggled()
 // Call to show the dialog box.
 void QgsIdentifyResults::show()
 {
-  // Enfore a few things before showing the dialog box
+  // Enforce a few things before showing the dialog box
   lstResults->sortItems( 0, Qt::AscendingOrder );
   expandColumnsToFit();
 
@@ -268,7 +345,9 @@ void QgsIdentifyResults::show()
     QTreeWidgetItem *layItem = lstResults->topLevelItem( 0 );
     QTreeWidgetItem *featItem = layItem->child( 0 );
 
-    if ( lstResults->topLevelItemCount() == 1 && layItem->childCount() == 1 && QSettings().value( "/Map/identifyAutoFeatureForm", false ).toBool() )
+    if ( lstResults->topLevelItemCount() == 1 &&
+         layItem->childCount() == 1 &&
+         QSettings().value( "/Map/identifyAutoFeatureForm", false ).toBool() )
     {
       QgsVectorLayer *layer = qobject_cast<QgsVectorLayer *>( layItem->data( 0, Qt::UserRole ).value<QObject *>() );
       if ( layer )
@@ -277,6 +356,7 @@ void QgsIdentifyResults::show()
         // don't show the form dialog instead of the results window
         lstResults->setCurrentItem( featItem );
         featureForm();
+        clear();
         return;
       }
     }
@@ -287,13 +367,15 @@ void QgsIdentifyResults::show()
   }
 
   QDialog::show();
+  raise();
 }
 
 // Slot called when user clicks the Close button
 // (saves the current window size/position)
 void QgsIdentifyResults::close()
 {
-  clearRubberbands();
+  clear();
+
   delete mActionPopup;
   mActionPopup = 0;
 
@@ -336,6 +418,7 @@ void QgsIdentifyResults::contextMenuEvent( QContextMenuEvent* event )
   if ( !item )
     return;
 
+  int idx = 0;
   QgsVectorLayer *vlayer = vectorLayer( item );
   if ( vlayer == 0 )
     return;
@@ -345,30 +428,41 @@ void QgsIdentifyResults::contextMenuEvent( QContextMenuEvent* event )
 
   mActionPopup = new QMenu();
 
-  mActionPopup->addAction( vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ), this, SLOT( featureForm() ) );
-  mActionPopup->addAction( tr( "Zoom to feature" ), this, SLOT( zoomToFeature() ) );
-  mActionPopup->addAction( tr( "Copy attribute value" ), this, SLOT( copyAttributeValue() ) );
-  mActionPopup->addAction( tr( "Copy feature attributes" ), this, SLOT( copyFeatureAttributes() ) );
-  mActionPopup->addSeparator();
+  QTreeWidgetItem *featItem = featureItem( item );
+  if ( featItem )
+  {
+    mActionPopup->addAction(
+      QgisApp::getThemeIcon( vlayer->isEditable() ? "/mIconEditable.png" : "/mIconEditable.png" ),
+      vlayer->isEditable() ? tr( "Edit feature form" ) : tr( "View feature form" ),
+      this, SLOT( featureForm() ) );
+    mActionPopup->addAction( tr( "Zoom to feature" ), this, SLOT( zoomToFeature() ) );
+    mActionPopup->addAction( tr( "Copy attribute value" ), this, SLOT( copyAttributeValue() ) );
+    mActionPopup->addAction( tr( "Copy feature attributes" ), this, SLOT( copyFeatureAttributes() ) );
+    mActionPopup->addSeparator();
+    if ( item->parent() == featItem && item->childCount() == 0 )
+    {
+      idx = item->data( 0, Qt::UserRole + 1 ).toInt();
+    }
+  }
+
   mActionPopup->addAction( tr( "Clear results" ), this, SLOT( clear() ) );
-  mActionPopup->addAction( tr( "Clear highlights" ), this, SLOT( clearRubberbands() ) );
+  mActionPopup->addAction( tr( "Clear highlights" ), this, SLOT( clearHighlights() ) );
   mActionPopup->addAction( tr( "Highlight all" ), this, SLOT( highlightAll() ) );
   mActionPopup->addAction( tr( "Highlight layer" ), this, SLOT( highlightLayer() ) );
   mActionPopup->addSeparator();
   mActionPopup->addAction( tr( "Expand all" ), this, SLOT( expandAll() ) );
   mActionPopup->addAction( tr( "Collapse all" ), this, SLOT( collapseAll() ) );
 
-  if ( vlayer->actions()->size() > 0 )
+  if ( featItem && vlayer->actions()->size() > 0 )
   {
     mActionPopup->addSeparator();
 
+    int featIdx = featItem->data( 0, Qt::UserRole + 1 ).toInt();
+
     // The assumption is made that an instance of QgsIdentifyResults is
     // created for each new Identify Results dialog box, and that the
     // contents of the popup menu doesn't change during the time that
     // such a dialog box is around.
-    QAction *a = mActionPopup->addAction( tr( "Run action" ) );
-    a->setEnabled( false );
-
     for ( int i = 0; i < vlayer->actions()->size(); i++ )
     {
       const QgsAction &action = vlayer->actions()->at( i );
@@ -376,22 +470,14 @@ void QgsIdentifyResults::contextMenuEvent( QContextMenuEvent* event )
       if ( !action.runable() )
         continue;
 
-      QgsFeatureAction *a = new QgsFeatureAction( action.name(), this, vlayer, i, featureItem( item ) );
-      mActionPopup->addAction( action.name(), a, SLOT( execute() ) );
+      QgsFeatureAction *a = new QgsFeatureAction( action.name(), mFeatures[ featIdx ], vlayer, i, idx, this );
+      mActionPopup->addAction( QgisApp::getThemeIcon( "/mAction.png" ), action.name(), a, SLOT( execute() ) );
     }
   }
 
   mActionPopup->popup( event->globalPos() );
 }
 
-// Restore last window position/size and show the window
-void QgsIdentifyResults::restorePosition()
-{
-  QSettings settings;
-  restoreGeometry( settings.value( "/Windows/Identify/geometry" ).toByteArray() );
-  show();
-}
-
 // Save the current window location (store in ~/.qt/qgisrc)
 void QgsIdentifyResults::saveWindowLocation()
 {
@@ -411,14 +497,14 @@ void QgsIdentifyResults::expandColumnsToFit()
   lstResults->resizeColumnToContents( 1 );
 }
 
-void QgsIdentifyResults::clearRubberbands()
+void QgsIdentifyResults::clearHighlights()
 {
-  foreach( QgsRubberBand *rb, mRubberBands )
+  foreach( QgsHighlight *h, mHighlights )
   {
-    delete rb;
+    delete h;
   }
 
-  mRubberBands.clear();
+  mHighlights.clear();
 }
 
 void QgsIdentifyResults::clear()
@@ -429,7 +515,7 @@ void QgsIdentifyResults::clear()
   }
 
   lstResults->clear();
-  clearRubberbands();
+  clearHighlights();
 }
 
 void QgsIdentifyResults::activate()
@@ -461,7 +547,7 @@ void QgsIdentifyResults::deactivate()
 void QgsIdentifyResults::doAction( QTreeWidgetItem *item, int action )
 {
   int idx;
-  QList< QPair<QString, QString> > attributes;
+  QgsAttributeMap attributes;
   QTreeWidgetItem *featItem = retrieveAttributes( item, attributes, idx );
   if ( !featItem )
     return;
@@ -490,6 +576,9 @@ void QgsIdentifyResults::doAction( QTreeWidgetItem *item, int action )
 
 QTreeWidgetItem *QgsIdentifyResults::featureItem( QTreeWidgetItem *item )
 {
+  if ( !item )
+    return 0;
+
   QTreeWidgetItem *featItem;
   if ( item->parent() )
   {
@@ -544,9 +633,11 @@ QgsVectorLayer *QgsIdentifyResults::vectorLayer( QTreeWidgetItem *item )
 }
 
 
-QTreeWidgetItem *QgsIdentifyResults::retrieveAttributes( QTreeWidgetItem *item, QList< QPair<QString, QString> > &attributes, int &idx )
+QTreeWidgetItem *QgsIdentifyResults::retrieveAttributes( QTreeWidgetItem *item, QgsAttributeMap &attributes, int &idx )
 {
   QTreeWidgetItem *featItem = featureItem( item );
+  if ( !featItem )
+    return 0;
 
   idx = -1;
 
@@ -557,8 +648,8 @@ QTreeWidgetItem *QgsIdentifyResults::retrieveAttributes( QTreeWidgetItem *item,
     if ( item->childCount() > 0 )
       continue;
     if ( item == lstResults->currentItem() )
-      idx = attributes.size();
-    attributes << QPair<QString, QString>( item->data( 0, Qt::DisplayRole ).toString(), item->data( 1, Qt::DisplayRole ).toString() );
+      idx = item->data( 0, Qt::UserRole + 1 ).toInt();
+    attributes.insert( item->data( 0, Qt::UserRole + 1 ).toInt(), item->data( 1, Qt::DisplayRole ) );
   }
 
   return featItem;
@@ -585,7 +676,7 @@ void QgsIdentifyResults::handleCurrentItemChanged( QTreeWidgetItem *current, QTr
   }
   else
   {
-    clearRubberbands();
+    clearHighlights();
     highlightFeature( current );
   }
 }
@@ -602,7 +693,7 @@ void QgsIdentifyResults::layerDestroyed()
     {
       for ( int j = 0; j < layItem->childCount(); j++ )
       {
-        delete mRubberBands.take( layItem->child( i ) );
+        delete mHighlights.take( layItem->child( i ) );
       }
     }
   }
@@ -626,6 +717,7 @@ void QgsIdentifyResults::disconnectLayer( QObject *layer )
   {
     disconnect( vlayer, SIGNAL( layerDeleted() ), this, SLOT( layerDestroyed() ) );
     disconnect( vlayer, SIGNAL( featureDeleted( int ) ), this, SLOT( featureDeleted( int ) ) );
+    disconnect( vlayer, SIGNAL( attributeValueChanged( int, int, const QVariant & ) ), this, SLOT( attributeValueChanged( int, int, const QVariant & ) ) );
     disconnect( vlayer, SIGNAL( editingStarted() ), this, SLOT( editingToggled() ) );
     disconnect( vlayer, SIGNAL( editingStopped() ), this, SLOT( editingToggled() ) );
   }
@@ -648,7 +740,7 @@ void QgsIdentifyResults::featureDeleted( int fid )
 
     if ( featItem && featItem->data( 0, Qt::UserRole ).toInt() == fid )
     {
-      delete mRubberBands.take( featItem );
+      delete mHighlights.take( featItem );
       delete featItem;
       break;
     }
@@ -665,6 +757,39 @@ void QgsIdentifyResults::featureDeleted( int fid )
   }
 }
 
+void QgsIdentifyResults::attributeValueChanged( int fid, int idx, const QVariant &val )
+{
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( sender() );
+  QTreeWidgetItem *layItem = layerItem( sender() );
+
+  if ( !layItem )
+    return;
+
+  for ( int i = 0; i < layItem->childCount(); i++ )
+  {
+    QTreeWidgetItem *featItem = layItem->child( i );
+
+    if ( featItem && featItem->data( 0, Qt::UserRole ).toInt() == fid )
+    {
+      if ( featItem->data( 0, Qt::DisplayRole ).toString() == vlayer->displayField() )
+        featItem->setData( 1, Qt::DisplayRole, val );
+
+      for ( int j = 0; j < featItem->childCount(); j++ )
+      {
+        QTreeWidgetItem *item = featItem->child( j );
+        if ( item->childCount() > 0 )
+          continue;
+
+        if ( item->data( 0, Qt::UserRole + 1 ).toInt() == idx )
+        {
+          item->setData( 1, Qt::DisplayRole, val );
+          return;
+        }
+      }
+    }
+  }
+}
+
 void QgsIdentifyResults::highlightFeature( QTreeWidgetItem *item )
 {
   QgsVectorLayer *layer = vectorLayer( item );
@@ -675,13 +800,13 @@ void QgsIdentifyResults::highlightFeature( QTreeWidgetItem *item )
   if ( !featItem )
     return;
 
-  if ( mRubberBands.contains( featItem ) )
+  if ( mHighlights.contains( featItem ) )
     return;
 
   int fid = featItem->data( 0, Qt::UserRole ).toInt();
 
   QgsFeature feat;
-  if ( ! layer->featureAtId( fid, feat, true, false ) )
+  if ( !layer->featureAtId( fid, feat, true, false ) )
   {
     return;
   }
@@ -691,14 +816,13 @@ void QgsIdentifyResults::highlightFeature( QTreeWidgetItem *item )
     return;
   }
 
-  QgsRubberBand *rb = new QgsRubberBand( mCanvas, feat.geometry()->type() == QGis::Polygon );
-  if ( rb )
+  QgsHighlight *h = new QgsHighlight( mCanvas, feat.geometry(), layer );
+  if ( h )
   {
-    rb->setToGeometry( feat.geometry(), layer );
-    rb->setWidth( 2 );
-    rb->setColor( Qt::red );
-    rb->show();
-    mRubberBands.insert( featItem, rb );
+    h->setWidth( 2 );
+    h->setColor( Qt::red );
+    h->show();
+    mHighlights.insert( featItem, h );
   }
 }
 
@@ -753,60 +877,24 @@ void QgsIdentifyResults::featureForm()
     return;
 
   int fid = featItem->data( 0, Qt::UserRole ).toInt();
+  int idx = featItem->data( 0, Qt::UserRole + 1 ).toInt();
 
   QgsFeature f;
   if ( !vlayer->featureAtId( fid, f ) )
     return;
 
-  QgsAttributeMap src = f.attributeMap();
-
-  vlayer->beginEditCommand( tr( "Attribute changed" ) );
-  QgsAttributeDialog *ad = new QgsAttributeDialog( vlayer, &f );
-
-  if ( !vlayer->isEditable() && vlayer->actions()->size() > 0 )
+  QgsFeatureAction action( tr( "Attribute changes" ), f, vlayer, idx, -1, this );
+  if ( vlayer->isEditable() )
   {
-    ad->dialog()->setContextMenuPolicy( Qt::ActionsContextMenu );
-
-    QAction *a = new QAction( tr( "Run actions" ), ad->dialog() );
-    a->setEnabled( false );
-    ad->dialog()->addAction( a );
-
-    for ( int i = 0; i < vlayer->actions()->size(); i++ )
+    if ( action.editFeature() )
     {
-      const QgsAction &action = vlayer->actions()->at( i );
-
-      if ( !action.runable() )
-        continue;
-
-      QgsFeatureAction *a = new QgsFeatureAction( action.name(), this, vlayer, i, featItem );
-      ad->dialog()->addAction( a );
-      connect( a, SIGNAL( triggered() ), a, SLOT( execute() ) );
-
-      QPushButton *pb = ad->dialog()->findChild<QPushButton *>( action.name() );
-      if ( pb )
-        connect( pb, SIGNAL( clicked() ), a, SLOT( execute() ) );
+      mCanvas->refresh();
     }
   }
-
-  if ( ad->exec() )
-  {
-    const QgsAttributeMap &dst = f.attributeMap();
-    for ( QgsAttributeMap::const_iterator it = dst.begin(); it != dst.end(); it++ )
-    {
-      if ( !src.contains( it.key() ) || it.value() != src[it.key()] )
-      {
-        vlayer->changeAttributeValue( f.id(), it.key(), it.value() );
-      }
-    }
-    vlayer->endEditCommand();
-  }
   else
   {
-    vlayer->destroyEditCommand();
+    action.viewFeatureForm( mHighlights.take( featItem ) );
   }
-
-  delete ad;
-  mCanvas->refresh();
 }
 
 void QgsIdentifyResults::highlightAll()
@@ -833,7 +921,7 @@ void QgsIdentifyResults::highlightLayer( QTreeWidgetItem *item )
   if ( !layItem )
     return;
 
-  clearRubberbands();
+  clearHighlights();
 
   for ( int i = 0; i < layItem->childCount(); i++ )
   {
@@ -865,13 +953,23 @@ void QgsIdentifyResults::copyFeatureAttributes()
   QClipboard *clipboard = QApplication::clipboard();
   QString text;
 
+  QgsVectorLayer *vlayer = vectorLayer( lstResults->currentItem() );
+  if ( !vlayer )
+    return;
+
   int idx;
-  QList< QPair<QString, QString> > attributes;
+  QgsAttributeMap attributes;
   retrieveAttributes( lstResults->currentItem(), attributes, idx );
 
-  for ( QList< QPair<QString, QString> >::iterator it = attributes.begin(); it != attributes.end(); it++ )
+  const QgsFieldMap &fields = vlayer->pendingFields();
+
+  for ( QgsAttributeMap::const_iterator it = attributes.begin(); it != attributes.end(); it++ )
   {
-    text += QString( "%1: %2\n" ).arg( it->first ).arg( it->second );
+    QgsFieldMap::const_iterator fit = fields.find( it.key() );
+    if ( fit == fields.constEnd() )
+      continue;
+
+    text += QString( "%1: %2\n" ).arg( fit->name() ).arg( it.value().toString() );
   }
 
   QgsDebugMsg( QString( "set clipboard: %1" ).arg( text ) );
