@@ -22,6 +22,7 @@
 #include "ui_qgsidentifyresultsbase.h"
 #include "qgsattributeaction.h"
 #include "qgscontexthelp.h"
+#include "qgsfeature.h"
 
 #include <QWidget>
 #include <QList>
@@ -31,9 +32,9 @@ class QTreeWidgetItem;
 class QAction;
 class QMenu;
 
-class QgsMapLayer;
 class QgsVectorLayer;
-class QgsRubberBand;
+class QgsRasterLayer;
+class QgsHighlight;
 class QgsMapCanvas;
 class QDockWidget;
 
@@ -53,9 +54,13 @@ class QgsIdentifyResults: public QDialog, private Ui::QgsIdentifyResultsBase
 
     ~QgsIdentifyResults();
 
-    /** Add add feature */
-    void addFeature( QgsMapLayer *layer, int fid,
-                     QString displayField, QString displayValue,
+    /** Add add feature from vector layer */
+    void addFeature( QgsVectorLayer *layer,
+                     const QgsFeature &f,
+                     const QMap< QString, QString > &derivedAttributes );
+
+    /** Add add feature from other layer */
+    void addFeature( QgsRasterLayer *layer, QString label,
                      const QMap< QString, QString > &attributes,
                      const QMap< QString, QString > &derivedAttributes );
 
@@ -82,6 +87,7 @@ class QgsIdentifyResults: public QDialog, private Ui::QgsIdentifyResultsBase
     void layerDestroyed();
     void editingToggled();
     void featureDeleted( int fid );
+    void attributeValueChanged( int fid, int idx, const QVariant & );
 
     void featureForm();
     void zoomToFeature();
@@ -89,7 +95,7 @@ class QgsIdentifyResults: public QDialog, private Ui::QgsIdentifyResultsBase
     void copyFeatureAttributes();
     void highlightAll();
     void highlightLayer();
-    void clearRubberbands();
+    void clearHighlights();
     void expandAll();
     void collapseAll();
 
@@ -103,14 +109,15 @@ class QgsIdentifyResults: public QDialog, private Ui::QgsIdentifyResultsBase
     /* Item in tree was clicked */
     void itemClicked( QTreeWidgetItem *lvi, int column );
 
-    QTreeWidgetItem *retrieveAttributes( QTreeWidgetItem *item, QList< QPair<QString, QString> > &attributes, int &currentIdx );
+    QTreeWidgetItem *retrieveAttributes( QTreeWidgetItem *item, QgsAttributeMap &attributes, int &currentIdx );
 
     void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
 
   private:
     QMenu *mActionPopup;
-    QMap<QTreeWidgetItem *, QgsRubberBand * > mRubberBands;
+    QMap<QTreeWidgetItem *, QgsHighlight * > mHighlights;
     QgsMapCanvas *mCanvas;
+    QList<QgsFeature> mFeatures;
 
     QgsVectorLayer *vectorLayer( QTreeWidgetItem *item );
     QTreeWidgetItem *featureItem( QTreeWidgetItem *item );
@@ -123,7 +130,6 @@ class QgsIdentifyResults: public QDialog, private Ui::QgsIdentifyResultsBase
     void setColumnText( int column, const QString & label );
     void expandColumnsToFit();
     void saveWindowLocation();
-    void restorePosition();
 
     void highlightFeature( QTreeWidgetItem *item );
 
@@ -132,23 +138,4 @@ class QgsIdentifyResults: public QDialog, private Ui::QgsIdentifyResultsBase
     QDockWidget *mDock;
 };
 
-class QgsFeatureAction : public QAction
-{
-    Q_OBJECT
-
-  public:
-    QgsFeatureAction( const QString &name, QgsIdentifyResults *results, QgsVectorLayer *vl, int action, QTreeWidgetItem *featItem ) :
-        QAction( name, results ), mResults( results ), mLayer( vl ), mAction( action ), mFeatItem( featItem )
-    {}
-
-  public slots:
-    void execute();
-
-  private:
-    QgsIdentifyResults *mResults;
-    QgsVectorLayer *mLayer;
-    int mAction;
-    QTreeWidgetItem *mFeatItem;
-};
-
 #endif
