@@ -14,15 +14,19 @@
  *                                                                         *
  ***************************************************************************/
 /* $Id$ */
+
+#include <limits>
+
 #include "qgslabeldialog.h"
 #include "qgsfield.h"
 #include "qgslabel.h"
 #include "qgslabelattributes.h"
+#include "qgslogger.h"
 
 #include <QColorDialog>
 #include <QFontDialog>
 #include <QTabWidget>
-#include "qgslogger.h"
+#include <QDoubleValidator>
 
 
 const int PIXMAP_WIDTH = 200;
@@ -44,7 +48,7 @@ QgsLabelDialog::QgsLabelDialog( QgsLabel *label,  QWidget *parent )
 
   connect( btnDefaultFont, SIGNAL( clicked() ),
            this, SLOT( changeFont() ) );
-  connect( pbnDefaultBufferColor_2, SIGNAL( clicked() ),
+  connect( pbnDefaultBufferColor, SIGNAL( clicked() ),
            this, SLOT( changeBufferColor() ) );
   connect( pbnDefaultFontColor, SIGNAL( clicked() ),
            this, SLOT( changeFontColor() ) );
@@ -89,6 +93,10 @@ void QgsLabelDialog::init( )
   cboUnderlineField->addItems( myFieldStringList );
   cboUnderlineField->setCurrentIndex( itemNoForField( mLabel->labelField( QgsLabel::Underline ), myFieldStringList ) );
 
+  cboStrikeOutField->clear();
+  cboStrikeOutField->addItems( myFieldStringList );
+  cboStrikeOutField->setCurrentIndex( itemNoForField( mLabel->labelField( QgsLabel::StrikeOut ), myFieldStringList ) );
+
   cboFontSizeField->clear();
   cboFontSizeField->addItems( myFieldStringList );
   cboFontSizeField->setCurrentIndex( itemNoForField( mLabel->labelField( QgsLabel::Size ), myFieldStringList ) );
@@ -142,8 +150,10 @@ void QgsLabelDialog::init( )
 
   // set up the scale based layer visibility stuff....
   chkUseScaleDependentRendering->setChecked( mLabel->scaleBasedVisibility() );
-  spinMinimumScale->setValue(( int )mLabel->minScale() );
-  spinMaximumScale->setValue(( int )mLabel->maxScale() );
+  leMinimumScale->setText( QString::number( mLabel->minScale(), 'f' ) );
+  leMinimumScale->setValidator( new QDoubleValidator( 0, std::numeric_limits<float>::max(), 1000, this ) );
+  leMaximumScale->setText( QString::number( mLabel->maxScale(), 'f' ) );
+  leMaximumScale->setValidator( new QDoubleValidator( 0, std::numeric_limits<float>::max(), 1000, this ) );
 
   //
   //set the non-databound fields up now
@@ -155,19 +165,12 @@ void QgsLabelDialog::init( )
     mFont.setPointSizeF( myLabelAttributes->size() );
 
     int myTypeInt = myLabelAttributes->sizeType();
-    if ( myTypeInt == QgsLabelAttributes::PointUnits )
-    {
-      radioFontSizeUnitsPoints->setChecked( true );
-    }
-    else //assume map units is checked
-    {
-      radioFontSizeUnitsMap->setChecked( true );
-    }
+    cboFontSizeUnits->setCurrentIndex( myTypeInt == QgsLabelAttributes::PointUnits ? 0 : 1 );
   }
   else //defaults for when no size has been set
   {
     mFont.setPointSizeF( myLabelAttributes->size() );
-    radioFontSizeUnitsPoints->setChecked( true );
+    cboFontSizeUnits->setCurrentIndex( 0 );
   }
 
   spinFontSize->setValue( myLabelAttributes->size() );
@@ -196,25 +199,27 @@ void QgsLabelDialog::init( )
   {
     mFont.setUnderline( false );
   }
+  if ( myLabelAttributes->strikeOutIsSet() )
+  {
+    mFont.setStrikeOut( myLabelAttributes->strikeOut() );
+  }
+  else
+  {
+    mFont.setStrikeOut( false );
+  }
 
   mFontColor = myLabelAttributes->color();
 
   if ( myLabelAttributes->offsetIsSet() )
   {
     int myTypeInt = myLabelAttributes->offsetType();
-    if ( myTypeInt == QgsLabelAttributes::PointUnits )
-    {
-      radioOffsetUnitsPoints->setChecked( true );
-    }
-    else
-    {
-      radioOffsetUnitsMap->setChecked( true );
-    }
+    cboOffsetUnits->setCurrentIndex( myTypeInt == QgsLabelAttributes::PointUnits ? 0 : 1 );
     spinXOffset->setValue( myLabelAttributes->xOffset() );
     spinYOffset->setValue( myLabelAttributes->yOffset() );
   }
   else //defaults for when no offset is defined
   {
+    cboOffsetUnits->setCurrentIndex( 0 );
     spinXOffset->setValue( 0 );
     spinYOffset->setValue( 0 );
   }
@@ -249,29 +254,23 @@ void QgsLabelDialog::init( )
   if ( myLabelAttributes->bufferSizeIsSet() )
   {
     int myTypeInt = myLabelAttributes->bufferSizeType();
-    if ( myTypeInt == QgsLabelAttributes::PointUnits )
-    {
-      radioBufferUnitsPoints->setChecked( true );
-    }
-    else
-    {
-      radioBufferUnitsMap->setChecked( true );
-    }
+    cboBufferSizeUnits->setCurrentIndex( myTypeInt == QgsLabelAttributes::PointUnits ? 0 : 1 );
     spinBufferSize->setValue( myLabelAttributes->bufferSize() );
   }
   else //defaults for when no offset is defined
   {
+    cboBufferSizeUnits->setCurrentIndex( 0 );
     spinBufferSize->setValue( 1 );
   }
   //set the state of the multiline enabled checkbox
   chkUseMultiline->setChecked( myLabelAttributes->multilineEnabled() );
+  //set the state of the selected features only checkbox
+  chkSelectedOnly->setChecked( myLabelAttributes->selectedOnly() );
   //set the state of the buffer enabled checkbox
   chkUseBuffer->setChecked( myLabelAttributes->bufferEnabled() );
 
   //NOTE: do we need this line too? TS
   spinBufferSize->setValue( myLabelAttributes->bufferSize() );
-  //TODO - transparency attributes for buffers
-
 }
 
 
@@ -282,24 +281,19 @@ void QgsLabelDialog::changeFont( void )
 
   qreal fontSize = mFont.pointSizeF();
   bool resultFlag;
-#if defined(Q_WS_MAC) && QT_VERSION >= 0x040500 && !defined(__LP64__)
-  // Native Mac dialog works only for 64 bit Cocoa (observed in Qt 4.5.2, probably a Qt bug)
+#if defined(Q_WS_MAC) && QT_VERSION >= 0x040500 && defined(QT_MAC_USE_COCOA)
+  // Native Mac dialog works only for Qt Carbon
   mFont = QFontDialog::getFont( &resultFlag, mFont, this, QString(), QFontDialog::DontUseNativeDialog );
 #else
   mFont = QFontDialog::getFont( &resultFlag, mFont, this );
 #endif
-  if ( resultFlag )
+  if ( !resultFlag )
+    return;
+
+  if ( mFont.pointSizeF() != fontSize )
   {
-    if ( mFont.pointSizeF() != fontSize )
-    {
-      // font is set to the font the user selected
-      spinFontSize->setValue( mFont.pointSizeF() );
-    }
-  }
-  else
-  {
-    // the user cancelled the dialog; font is set to the initial
-    // value, in this case Helvetica [Cronyx], 10
+    // font is set to the font the user selected
+    spinFontSize->setValue( mFont.pointSizeF() );
   }
   lblSample->setFont( mFont );
 }
@@ -308,7 +302,11 @@ void QgsLabelDialog::changeFontColor( void )
 {
   QgsDebugMsg( "entering." );
 
-  mFontColor = QColorDialog::getColor( mFontColor );
+  QColor color = QColorDialog::getColor( mFontColor );
+  if ( !color.isValid() )
+    return;
+
+  mFontColor = color;
   QPalette palette = lblSample->palette();
   palette.setColor( lblSample->foregroundRole(), mFontColor );
   lblSample->setPalette( palette );
@@ -318,7 +316,11 @@ void QgsLabelDialog::changeBufferColor( void )
 {
   QgsDebugMsg( "entering." );
 
-  mBufferColor = QColorDialog::getColor( mBufferColor );
+  QColor color = QColorDialog::getColor( mBufferColor );
+  if ( !color.isValid() )
+    return;
+
+  mBufferColor = color;
   QPalette palette = lblSample->palette();
   palette.setColor( lblSample->backgroundRole(), mBufferColor );
   lblSample->setPalette( palette );
@@ -327,14 +329,8 @@ void QgsLabelDialog::changeBufferColor( void )
 
 int QgsLabelDialog::itemNoForField( QString theFieldName, QStringList theFieldList )
 {
-  int myItemInt = 0;
-  for ( QStringList::Iterator it = theFieldList.begin(); it != theFieldList.end(); ++it )
-  {
-    if ( theFieldName == *it ) return myItemInt;
-    ++myItemInt;
-  }
   //if no matches assume first item in list is blank and return that
-  return 0;
+  return qMax( 0, theFieldList.indexOf( theFieldName ) );
 }
 
 QgsLabelDialog::~QgsLabelDialog()
@@ -351,29 +347,14 @@ void QgsLabelDialog::apply()
   QgsLabelAttributes * myLabelAttributes = mLabel->labelAttributes();
   myLabelAttributes->setText( leDefaultLabel->text() );
   myLabelAttributes->setFamily( mFont.family() );
-  int myTypeInt = 0;
-  if ( radioFontSizeUnitsPoints->isChecked() )
-  {
-    myTypeInt = QgsLabelAttributes::PointUnits;
-  }
-  else //assume map units is checked
-  {
-    myTypeInt = QgsLabelAttributes::MapUnits;
-  }
+  int myTypeInt = cboFontSizeUnits->currentIndex() == 0 ? QgsLabelAttributes::PointUnits : QgsLabelAttributes::MapUnits;
   myLabelAttributes->setSize( mFont.pointSizeF(), myTypeInt );
   myLabelAttributes->setBold( mFont.bold() );
   myLabelAttributes->setItalic( mFont.italic() );
   myLabelAttributes->setUnderline( mFont.underline() );
+  myLabelAttributes->setStrikeOut( mFont.strikeOut() );
   myLabelAttributes->setColor( mFontColor );
-  myTypeInt = 0;
-  if ( radioOffsetUnitsPoints->isChecked() )
-  {
-    myTypeInt = QgsLabelAttributes::PointUnits;
-  }
-  else
-  {
-    myTypeInt = QgsLabelAttributes::MapUnits;
-  }
+  myTypeInt = cboOffsetUnits->currentIndex() == 0 ?  QgsLabelAttributes::PointUnits : QgsLabelAttributes::MapUnits;
   myLabelAttributes->setOffset( spinXOffset->value(), spinYOffset->value(), myTypeInt );
   myLabelAttributes->setAutoAngle( spinAngle->value() == -1 );
   myLabelAttributes->setAngle( spinAngle->value() );
@@ -392,17 +373,10 @@ void QgsLabelDialog::apply()
   if ( radioOver->isChecked() )        myLabelAttributes->setAlignment( Qt::AlignCenter );
 
   myLabelAttributes->setMultilineEnabled( chkUseMultiline->isChecked() );
+  myLabelAttributes->setSelectedOnly( chkSelectedOnly->isChecked() );
   myLabelAttributes->setBufferEnabled( chkUseBuffer->isChecked() );
   myLabelAttributes->setBufferColor( mBufferColor );
-  myTypeInt = 0;
-  if ( radioBufferUnitsPoints->isChecked() )
-  {
-    myTypeInt = QgsLabelAttributes::PointUnits;
-  }
-  else
-  {
-    myTypeInt = QgsLabelAttributes::MapUnits;
-  }
+  myTypeInt = cboBufferSizeUnits->currentIndex() == 0 ? QgsLabelAttributes::PointUnits : QgsLabelAttributes::MapUnits;
   myLabelAttributes->setBufferSize( spinBufferSize->value(), myTypeInt );
   //TODO - transparency attributes for buffers
 
@@ -412,6 +386,7 @@ void QgsLabelDialog::apply()
   mLabel->setLabelField( QgsLabel::Bold,  fieldIndexFromName( cboBoldField->currentText() ) );
   mLabel->setLabelField( QgsLabel::Italic,  fieldIndexFromName( cboItalicField->currentText() ) );
   mLabel->setLabelField( QgsLabel::Underline,  fieldIndexFromName( cboUnderlineField->currentText() ) );
+  mLabel->setLabelField( QgsLabel::StrikeOut,  fieldIndexFromName( cboStrikeOutField->currentText() ) );
   mLabel->setLabelField( QgsLabel::Size,  fieldIndexFromName( cboFontSizeField->currentText() ) );
   mLabel->setLabelField( QgsLabel::SizeType,  fieldIndexFromName( cboFontSizeTypeField->currentText() ) );
   mLabel->setLabelField( QgsLabel::Color,  fieldIndexFromName( cboFontColorField->currentText() ) );
@@ -426,8 +401,8 @@ void QgsLabelDialog::apply()
 
   // set up the scale based layer visibility stuff....
   mLabel->setScaleBasedVisibility( chkUseScaleDependentRendering->isChecked() );
-  mLabel->setMinScale( spinMinimumScale->value() );
-  mLabel->setMaxScale( spinMaximumScale->value() );
+  mLabel->setMinScale( leMinimumScale->text().toFloat() );
+  mLabel->setMaxScale( leMaximumScale->text().toFloat() );
 }
 
 int QgsLabelDialog::fieldIndexFromName( QString name )
