@@ -27,22 +27,30 @@
 #include "qgsvectordataprovider.h"
 #include "qgsvectorlayer.h"
 #include "qgslogger.h"
+#include "qgsfeatureaction.h"
+
 #include <QMessageBox>
 #include <QMouseEvent>
 #include <QSettings>
 
-QgsMapToolAddFeature::QgsMapToolAddFeature( QgsMapCanvas* canvas, enum CaptureTool tool ): QgsMapToolCapture( canvas, tool )
+QgsMapToolAddFeature::QgsMapToolAddFeature( QgsMapCanvas* canvas, CaptureMode tool ): QgsMapToolCapture( canvas, tool )
 {
-
 }
 
 QgsMapToolAddFeature::~QgsMapToolAddFeature()
 {
+}
 
+bool QgsMapToolAddFeature::addFeature( QgsVectorLayer *vlayer, QgsFeature *f )
+{
+  QgsFeatureAction action( tr( "add feature" ), *f, vlayer, -1, -1, this );
+  return action.addFeature();
 }
 
 void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
 {
+  QgsDebugMsg( "entered." );
+
   QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( mCanvas->currentLayer() );
 
   if ( !vlayer )
@@ -54,19 +62,6 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
 
   QGis::WkbType layerWKBType = vlayer->wkbType();
 
-  //no support for adding features to 2.5D types yet
-  if ( layerWKBType == QGis::WKBLineString25D || layerWKBType == QGis::WKBPolygon25D ||
-       layerWKBType == QGis::WKBMultiLineString25D || layerWKBType == QGis::WKBPoint25D || layerWKBType == QGis::WKBMultiPoint25D )
-  {
-    QMessageBox::critical( 0, tr( "2.5D shape type not supported" ), tr( "Adding features to 2.5D shapetypes is not supported yet" ) );
-    delete mRubberBand;
-    mRubberBand = NULL;
-    mCapturing = FALSE;
-    mCaptureList.clear();
-    mCanvas->refresh();
-    return;
-  }
-
   QgsVectorDataProvider* provider = vlayer->dataProvider();
 
   if ( !( provider->capabilities() & QgsVectorDataProvider::AddFeatures ) )
@@ -79,13 +74,13 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
   if ( !vlayer->isEditable() )
   {
     QMessageBox::information( 0, tr( "Layer not editable" ),
-                              tr( "Cannot edit the vector layer. To make it editable, go to the file item "
-                                  "of the layer, right click and check 'Allow Editing'." ) );
+                              tr( "Cannot edit the vector layer. Use 'Toggle Editing' to make it editable." )
+                            );
     return;
   }
 
   // POINT CAPTURING
-  if ( mTool == CapturePoint )
+  if ( mode() == CapturePoint )
   {
     //check we only use this tool for point/multipoint layers
     if ( vlayer->geometryType() != QGis::Point )
@@ -129,36 +124,36 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
       QgsFeature* f = new QgsFeature( 0, "WKBPoint" );
 
       int size = 0;
-      char end = QgsApplication::endian();
+      char endian = QgsApplication::endian();
       unsigned char *wkb = NULL;
       int wkbtype = 0;
       double x = savePoint.x();
       double y = savePoint.y();
 
-      if ( layerWKBType == QGis::WKBPoint )
+      if ( layerWKBType == QGis::WKBPoint || layerWKBType == QGis::WKBPoint25D )
       {
         size = 1 + sizeof( int ) + 2 * sizeof( double );
         wkb = new unsigned char[size];
         wkbtype = QGis::WKBPoint;
-        memcpy( &wkb[0], &end, 1 );
+        memcpy( &wkb[0], &endian, 1 );
         memcpy( &wkb[1], &wkbtype, sizeof( int ) );
         memcpy( &wkb[5], &x, sizeof( double ) );
         memcpy( &wkb[5] + sizeof( double ), &y, sizeof( double ) );
       }
-      else if ( layerWKBType == QGis::WKBMultiPoint )
+      else if ( layerWKBType == QGis::WKBMultiPoint || layerWKBType == QGis::WKBMultiPoint25D )
       {
         size = 2 + 3 * sizeof( int ) + 2 * sizeof( double );
         wkb = new unsigned char[size];
         wkbtype = QGis::WKBMultiPoint;
         int position = 0;
-        memcpy( &wkb[position], &end, 1 );
+        memcpy( &wkb[position], &endian, 1 );
         position += 1;
         memcpy( &wkb[position], &wkbtype, sizeof( int ) );
         position += sizeof( int );
         int npoint = 1;
         memcpy( &wkb[position], &npoint, sizeof( int ) );
         position += sizeof( int );
-        memcpy( &wkb[position], &end, 1 );
+        memcpy( &wkb[position], &endian, 1 );
         position += 1;
         int pointtype = QGis::WKBPoint;
         memcpy( &wkb[position], &pointtype, sizeof( int ) );
@@ -169,50 +164,26 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
       }
 
       f->setGeometryAndOwnership( &wkb[0], size );
-      // add the fields to the QgsFeature
-      const QgsFieldMap fields = vlayer->pendingFields();
-      for ( QgsFieldMap::const_iterator it = fields.constBegin(); it != fields.constEnd(); ++it )
-      {
-        f->addAttribute( it.key(), provider->defaultValue( it.key() ) );
-      }
 
       vlayer->beginEditCommand( tr( "Feature added" ) );
 
-      // show the dialog to enter attribute values
-      QSettings settings;
-      bool isDisabledAttributeValuesDlg = settings.value( "/qgis/digitizing/disable_enter_attribute_values_dialog", false ).toBool();
-      if ( isDisabledAttributeValuesDlg )
+      if ( addFeature( vlayer, f ) )
       {
-        QgsDebugMsg( "Adding feature to layer" );
-        vlayer->addFeature( *f );
         vlayer->endEditCommand();
       }
       else
       {
-        QgsAttributeDialog *mypDialog = new QgsAttributeDialog( vlayer, f );
-        if ( mypDialog->exec() )
-        {
-          QgsDebugMsg( "Adding feature to layer" );
-          vlayer->addFeature( *f );
-          vlayer->endEditCommand();
-        }
-        else
-        {
-          vlayer->destroyEditCommand();
-          QgsDebugMsg( "Adding feature to layer failed" );
-          delete f;
-        }
-        delete mypDialog;
+        delete f;
+        vlayer->destroyEditCommand();
       }
 
       mCanvas->refresh();
     }
-
   }
-  else if ( mTool == CaptureLine || mTool == CapturePolygon )
+  else if ( mode() == CaptureLine || mode() == CapturePolygon )
   {
     //check we only use the line tool for line/multiline layers
-    if ( mTool == CaptureLine && vlayer->geometryType() != QGis::Line )
+    if ( mode() == CaptureLine && vlayer->geometryType() != QGis::Line )
     {
       QMessageBox::information( 0, tr( "Wrong editing tool" ),
                                 tr( "Cannot apply the 'capture line' tool on this vector layer" ) );
@@ -220,7 +191,7 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
     }
 
     //check we only use the polygon tool for polygon/multipolygon layers
-    if ( mTool == CapturePolygon && vlayer->geometryType() != QGis::Polygon )
+    if ( mode() == CapturePolygon && vlayer->geometryType() != QGis::Polygon )
     {
       QMessageBox::information( 0, tr( "Wrong editing tool" ),
                                 tr( "Cannot apply the 'capture polygon' tool on this vector layer" ) );
@@ -244,52 +215,46 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
 
     if ( e->button() == Qt::LeftButton )
     {
-      mCapturing = TRUE;
+      startCapturing();
     }
     else if ( e->button() == Qt::RightButton )
     {
       // End of string
 
-      mCapturing = FALSE;
-
       //lines: bail out if there are not at least two vertices
-      if ( mTool == CaptureLine && mCaptureList.size() < 2 )
+      if ( mode() == CaptureLine && size() < 2 )
       {
-        delete mRubberBand;
-        mRubberBand = NULL;
-        mCaptureList.clear();
+        stopCapturing();
         return;
       }
 
       //polygons: bail out if there are not at least two vertices
-      if ( mTool == CapturePolygon && mCaptureList.size() < 3 )
+      if ( mode() == CapturePolygon && size() < 3 )
       {
-        delete mRubberBand;
-        mRubberBand = NULL;
-        mCaptureList.clear();
+        stopCapturing();
         return;
       }
 
       //create QgsFeature with wkb representation
       QgsFeature* f = new QgsFeature( 0, "WKBLineString" );
       unsigned char* wkb;
-      int size;
-      char end = QgsApplication::endian();
+      int wkbsize;
+      char endian = QgsApplication::endian();
 
-      if ( mTool == CaptureLine )
+      if ( mode() == CaptureLine )
       {
-        if ( layerWKBType == QGis::WKBLineString )
+        if ( layerWKBType == QGis::WKBLineString || layerWKBType == QGis::WKBLineString25D )
         {
-          size = 1 + 2 * sizeof( int ) + 2 * mCaptureList.size() * sizeof( double );
-          wkb = new unsigned char[size];
+          wkbsize = 1 + 2 * sizeof( int ) + 2 * size() * sizeof( double );
+          wkb = new unsigned char[wkbsize];
           int wkbtype = QGis::WKBLineString;
-          int length = mCaptureList.size();
-          memcpy( &wkb[0], &end, 1 );
+          int length = size();
+          memcpy( &wkb[0], &endian, 1 );
           memcpy( &wkb[1], &wkbtype, sizeof( int ) );
           memcpy( &wkb[1+sizeof( int )], &length, sizeof( int ) );
           int position = 1 + 2 * sizeof( int );
           double x, y;
-          for ( QList<QgsPoint>::iterator it = mCaptureList.begin(); it != mCaptureList.end(); ++it )
+          for ( QList<QgsPoint>::iterator it = begin(); it != end(); ++it )
           {
             QgsPoint savePoint = *it;
             x = savePoint.x();
@@ -302,29 +267,29 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
             position += sizeof( double );
           }
         }
-        else if ( layerWKBType == QGis::WKBMultiLineString )
+        else if ( layerWKBType == QGis::WKBMultiLineString || layerWKBType == QGis::WKBMultiLineString25D )
         {
-          size = 1 + 2 * sizeof( int ) + 1 + 2 * sizeof( int ) + 2 * mCaptureList.size() * sizeof( double );
-          wkb = new unsigned char[size];
+          wkbsize = 1 + 2 * sizeof( int ) + 1 + 2 * sizeof( int ) + 2 * size() * sizeof( double );
+          wkb = new unsigned char[wkbsize];
           int position = 0;
           int wkbtype = QGis::WKBMultiLineString;
-          memcpy( &wkb[position], &end, 1 );
+          memcpy( &wkb[position], &endian, 1 );
           position += 1;
           memcpy( &wkb[position], &wkbtype, sizeof( int ) );
           position += sizeof( int );
           int nlines = 1;
           memcpy( &wkb[position], &nlines, sizeof( int ) );
           position += sizeof( int );
-          memcpy( &wkb[position], &end, 1 );
+          memcpy( &wkb[position], &endian, 1 );
           position += 1;
           int linewkbtype = QGis::WKBLineString;
           memcpy( &wkb[position], &linewkbtype, sizeof( int ) );
           position += sizeof( int );
-          int length = mCaptureList.size();
+          int length = size();
           memcpy( &wkb[position], &length, sizeof( int ) );
           position += sizeof( int );
           double x, y;
-          for ( QList<QgsPoint>::iterator it = mCaptureList.begin(); it != mCaptureList.end(); ++it )
+          for ( QList<QgsPoint>::iterator it = begin(); it != end(); ++it )
           {
             QgsPoint savePoint = *it;
             x = savePoint.x();
@@ -340,30 +305,28 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
         else
         {
           QMessageBox::critical( 0, tr( "Error" ), tr( "Cannot add feature. Unknown WKB type" ) );
-          delete mRubberBand;
-          mRubberBand = NULL;
-          mCaptureList.clear();
+          stopCapturing();
           return; //unknown wkbtype
         }
-        f->setGeometryAndOwnership( &wkb[0], size );
+        f->setGeometryAndOwnership( &wkb[0], wkbsize );
       }
       else // polygon
       {
-        if ( layerWKBType == QGis::WKBPolygon )
+        if ( layerWKBType == QGis::WKBPolygon ||  layerWKBType == QGis::WKBPolygon25D )
         {
-          size = 1 + 3 * sizeof( int ) + 2 * ( mCaptureList.size() + 1 ) * sizeof( double );
-          wkb = new unsigned char[size];
+          wkbsize = 1 + 3 * sizeof( int ) + 2 * ( size() + 1 ) * sizeof( double );
+          wkb = new unsigned char[wkbsize];
           int wkbtype = QGis::WKBPolygon;
-          int length = mCaptureList.size() + 1;//+1 because the first point is needed twice
+          int length = size() + 1;//+1 because the first point is needed twice
           int numrings = 1;
-          memcpy( &wkb[0], &end, 1 );
+          memcpy( &wkb[0], &endian, 1 );
           memcpy( &wkb[1], &wkbtype, sizeof( int ) );
           memcpy( &wkb[1+sizeof( int )], &numrings, sizeof( int ) );
           memcpy( &wkb[1+2*sizeof( int )], &length, sizeof( int ) );
           int position = 1 + 3 * sizeof( int );
           double x, y;
           QList<QgsPoint>::iterator it;
-          for ( it = mCaptureList.begin(); it != mCaptureList.end(); ++it )
+          for ( it = begin(); it != end(); ++it )
           {
             QgsPoint savePoint = *it;
             x = savePoint.x();
@@ -376,7 +339,7 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
             position += sizeof( double );
           }
           // close the polygon
-          it = mCaptureList.begin();
+          it = begin();
           QgsPoint savePoint = *it;
           x = savePoint.x();
           y = savePoint.y();
@@ -386,23 +349,23 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
 
           memcpy( &wkb[position], &y, sizeof( double ) );
         }
-        else if ( layerWKBType == QGis::WKBMultiPolygon )
+        else if ( layerWKBType == QGis::WKBMultiPolygon ||  layerWKBType == QGis::WKBMultiPolygon25D )
         {
-          size = 2 + 5 * sizeof( int ) + 2 * ( mCaptureList.size() + 1 ) * sizeof( double );
-          wkb = new unsigned char[size];
+          wkbsize = 2 + 5 * sizeof( int ) + 2 * ( size() + 1 ) * sizeof( double );
+          wkb = new unsigned char[wkbsize];
           int wkbtype = QGis::WKBMultiPolygon;
           int polygontype = QGis::WKBPolygon;
-          int length = mCaptureList.size() + 1;//+1 because the first point is needed twice
+          int length = size() + 1;//+1 because the first point is needed twice
           int numrings = 1;
           int numpolygons = 1;
           int position = 0; //pointer position relative to &wkb[0]
-          memcpy( &wkb[position], &end, 1 );
+          memcpy( &wkb[position], &endian, 1 );
           position += 1;
           memcpy( &wkb[position], &wkbtype, sizeof( int ) );
           position += sizeof( int );
           memcpy( &wkb[position], &numpolygons, sizeof( int ) );
           position += sizeof( int );
-          memcpy( &wkb[position], &end, 1 );
+          memcpy( &wkb[position], &endian, 1 );
           position += 1;
           memcpy( &wkb[position], &polygontype, sizeof( int ) );
           position += sizeof( int );
@@ -412,7 +375,7 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
           position += sizeof( int );
           double x, y;
           QList<QgsPoint>::iterator it;
-          for ( it = mCaptureList.begin(); it != mCaptureList.end(); ++it )//add the captured points to the polygon
+          for ( it = begin(); it != end(); ++it ) //add the captured points to the polygon
           {
             QgsPoint savePoint = *it;
             x = savePoint.x();
@@ -425,7 +388,7 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
             position += sizeof( double );
           }
           // close the polygon
-          it = mCaptureList.begin();
+          it = begin();
           QgsPoint savePoint = *it;
           x = savePoint.x();
           y = savePoint.y();
@@ -436,130 +399,52 @@ void QgsMapToolAddFeature::canvasReleaseEvent( QMouseEvent * e )
         else
         {
           QMessageBox::critical( 0, tr( "Error" ), tr( "Cannot add feature. Unknown WKB type" ) );
-          delete mRubberBand;
-          mRubberBand = NULL;
-          mCaptureList.clear();
+          stopCapturing();
           return; //unknown wkbtype
         }
-        f->setGeometryAndOwnership( &wkb[0], size );
+        f->setGeometryAndOwnership( &wkb[0], wkbsize );
 
-        int avoidIntersectionsReturn = avoidIntersectons( f->geometry() );
+        int avoidIntersectionsReturn = f->geometry()->avoidIntersections();
         if ( avoidIntersectionsReturn == 1 )
         {
           //not a polygon type. Impossible to get there
         }
-        else if ( avoidIntersectionsReturn == 2 )
+#if 0
+        else if ( avoidIntersectionsReturn == 2 ) //MH120131: disable this error message until there is a better way to cope with the single type / multi type problem
         {
           //bail out...
           QMessageBox::critical( 0, tr( "Error" ), tr( "The feature could not be added because removing the polygon intersections would change the geometry type" ) );
           delete f;
-          delete mRubberBand;
-          mRubberBand = 0;
-          mCaptureList.clear();
+          stopCapturing();
           return;
         }
+#endif
         else if ( avoidIntersectionsReturn == 3 )
         {
           QMessageBox::critical( 0, tr( "Error" ), tr( "An error was reported during intersection removal" ) );
         }
-
-
       }
 
-      // add the fields to the QgsFeature
-      const QgsFieldMap fields = vlayer->pendingFields();
-      for ( QgsFieldMap::const_iterator it = fields.begin(); it != fields.end(); ++it )
-      {
-        f->addAttribute( it.key(), provider->defaultValue( it.key() ) );
-      }
+      vlayer->beginEditCommand( tr( "Feature added" ) );
 
-      QSettings settings;
-      bool isDisabledAttributeValuesDlg = settings.value( "/qgis/digitizing/disable_enter_attribute_values_dialog", false ).toBool();
-      if ( isDisabledAttributeValuesDlg )
+      if ( addFeature( vlayer, f ) )
       {
-        vlayer->beginEditCommand( tr( "Feature added" ) );
-        if ( vlayer->addFeature( *f ) )
+        //add points to other features to keep topology up-to-date
+        int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
+        if ( topologicalEditing )
         {
-          //add points to other features to keep topology up-to-date
-          int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
-          if ( topologicalEditing )
-          {
-            vlayer->addTopologicalPoints( f->geometry() );
-          }
+          vlayer->addTopologicalPoints( f->geometry() );
         }
+
         vlayer->endEditCommand();
       }
       else
       {
-        QgsAttributeDialog * mypDialog = new QgsAttributeDialog( vlayer, f );
-        if ( mypDialog->exec() )
-        {
-          vlayer->beginEditCommand( tr( "Feature added" ) );
-          if ( vlayer->addFeature( *f ) )
-          {
-            //add points to other features to keep topology up-to-date
-            int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
-            if ( topologicalEditing )
-            {
-              vlayer->addTopologicalPoints( f->geometry() );
-            }
-          }
-          vlayer->endEditCommand();
-        }
-        mypDialog->deleteLater();
+        delete f;
+        vlayer->destroyEditCommand();
       }
-      delete f;
 
-      delete mRubberBand;
-      mRubberBand = NULL;
-
-      // delete the elements of mCaptureList
-      mCaptureList.clear();
-      mCanvas->refresh();
+      stopCapturing();
     }
   }
 }
-
-int QgsMapToolAddFeature::avoidIntersectons( QgsGeometry* g )
-{
-  int returnValue = 0;
-
-  //check if g has polygon type
-  if ( !g || g->type() != QGis::Polygon )
-  {
-    return 1;
-  }
-
-  QGis::WkbType geomTypeBeforeModification = g->wkbType();
-
-  //read avoid intersections list from project properties
-  bool listReadOk;
-  QStringList avoidIntersectionsList = QgsProject::instance()->readListEntry( "Digitizing", "/AvoidIntersectionsList", &listReadOk );
-  if ( !listReadOk )
-  {
-    return true; //no intersections stored in project does not mean error
-  }
-
-  //go through list, convert each layer to vector layer and call QgsVectorLayer::removePolygonIntersections for each
-  QgsVectorLayer* currentLayer = 0;
-  QStringList::const_iterator aIt = avoidIntersectionsList.constBegin();
-  for ( ; aIt != avoidIntersectionsList.constEnd(); ++aIt )
-  {
-    currentLayer = dynamic_cast<QgsVectorLayer*>( QgsMapLayerRegistry::instance()->mapLayer( *aIt ) );
-    if ( currentLayer )
-    {
-      if ( currentLayer->removePolygonIntersections( g ) != 0 )
-      {
-        returnValue = 3;
-      }
-    }
-  }
-
-  //make sure the geometry still has the same type (e.g. no change from polygon to multipolygon)
-  if ( g->wkbType() != geomTypeBeforeModification )
-  {
-    return 2;
-  }
-
-  return returnValue;
-}
