@@ -15,18 +15,16 @@
 /* $Id$ */
 
 #include "qgsmaptoolcapture.h"
+#include "qgsfeature.h"
 
 /**This tool adds new point/line/polygon features to already existing vector layers*/
-class QgsMapToolAddFeature: public QgsMapToolCapture
+class QgsMapToolAddFeature : public QgsMapToolCapture
 {
     Q_OBJECT
   public:
-    QgsMapToolAddFeature( QgsMapCanvas* canvas, enum CaptureTool tool );
+    QgsMapToolAddFeature( QgsMapCanvas* canvas, CaptureMode mode );
     virtual ~QgsMapToolAddFeature();
     void canvasReleaseEvent( QMouseEvent * e );
-  private:
-    /**Modifies geometry to avoid intersections with the layers specified in project properties
-    @return 0 in case of success, 1 if geometry is not of polygon type, 2 if avoid intersection would change the geometry type, \
-    3 other error during intersection removal*/
-    int avoidIntersectons( QgsGeometry* g );
+
+    bool addFeature( QgsVectorLayer *vlayer, QgsFeature *f );
 };
