@@ -14,32 +14,27 @@
  ***************************************************************************/
 /* $Id$ */
 
-#include "qgsapplication.h"
-#include "qgsattributedialog.h"
-#include "qgscoordinatetransform.h"
-#include "qgsfield.h"
-#include "qgslogger.h"
-#include "qgsgeometry.h"
 #include "qgsmaptoolcapture.h"
+
+#include "qgisapp.h"
+#include "qgsvertexmarker.h"
+#include "qgscursors.h"
+#include "qgsrubberband.h"
 #include "qgsmapcanvas.h"
 #include "qgsmaprenderer.h"
-#include "qgsmaptopixel.h"
-#include "qgsfeature.h"
-#include "qgsproject.h"
-#include "qgsrubberband.h"
 #include "qgsvectorlayer.h"
-#include "qgsvectordataprovider.h"
-#include "qgscursors.h"
+
 #include <QCursor>
 #include <QPixmap>
 #include <QMessageBox>
 #include <QMouseEvent>
+#include <QStatusBar>
 
 
-QgsMapToolCapture::QgsMapToolCapture( QgsMapCanvas* canvas, enum CaptureTool tool )
-    : QgsMapToolEdit( canvas ), mTool( tool ), mRubberBand( 0 )
+QgsMapToolCapture::QgsMapToolCapture( QgsMapCanvas* canvas, enum CaptureMode tool )
+    : QgsMapToolEdit( canvas ), mCaptureMode( tool ), mRubberBand( 0 )
 {
-  mCapturing = FALSE;
+  mCapturing = false;
 
   QPixmap mySelectQPixmap = QPixmap(( const char ** ) capture_point_cursor );
   mCursor = QCursor( mySelectQPixmap, 8, 8 );
@@ -49,8 +44,7 @@ QgsMapToolCapture::QgsMapToolCapture( QgsMapCanvas* canvas, enum CaptureTool too
 
 QgsMapToolCapture::~QgsMapToolCapture()
 {
-  delete mRubberBand;
-  mRubberBand = 0;
+  stopCapturing();
 }
 
 void QgsMapToolCapture::canvasMoveEvent( QMouseEvent * e )
@@ -80,10 +74,8 @@ void QgsMapToolCapture::renderComplete()
 
 void QgsMapToolCapture::deactivate()
 {
-  delete mRubberBand;
-  mRubberBand = 0;
-  mCaptureList.clear();
-
+  // don't stopCapturing() here, because the user might just want to
+  // pan or zoom and then continue digitizing.
   QgsMapTool::deactivate();
 }
 
@@ -98,7 +90,7 @@ int QgsMapToolCapture::addVertex( const QPoint& p )
 
   if ( !mRubberBand )
   {
-    mRubberBand = createRubberBand( mTool == CapturePolygon );
+    mRubberBand = createRubberBand( mCaptureMode == CapturePolygon );
   }
 
   QgsPoint digitisedPoint;
@@ -129,7 +121,9 @@ int QgsMapToolCapture::addVertex( const QPoint& p )
       return 2;
     }
     mRubberBand->addPoint( mapPoint );
-    mCaptureList.push_back( layerPoint );
+    mCaptureList.append( layerPoint );
+
+    validateGeometry();
   }
 
   return 0;
@@ -148,14 +142,101 @@ void QgsMapToolCapture::undo()
     }
 
     mRubberBand->removeLastPoint();
-    mCaptureList.pop_back();
+    mCaptureList.removeLast();
+
+    validateGeometry();
   }
 }
 
 void QgsMapToolCapture::keyPressEvent( QKeyEvent* e )
 {
-  if ( e->key() == Qt::Key_Backspace )
+  if ( e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete )
   {
     undo();
   }
 }
+
+void QgsMapToolCapture::startCapturing()
+{
+  mCapturing = true;
+}
+
+void QgsMapToolCapture::stopCapturing()
+{
+  if ( mRubberBand )
+  {
+    delete mRubberBand;
+    mRubberBand = 0;
+  }
+
+  while ( !mGeomErrorMarkers.isEmpty() )
+  {
+    delete mGeomErrorMarkers.takeFirst();
+  }
+
+  mGeomErrors.clear();
+
+#ifdef Q_OS_WIN
+  // hope your wearing your peril sensitive sunglasses.
+  QgisApp::instance()->skipNextContextMenuEvent();
+#endif
+
+  mCapturing = false;
+  mCaptureList.clear();
+  mCanvas->refresh();
+}
+
+void QgsMapToolCapture::closePolygon()
+{
+  mCaptureList.append( mCaptureList[0] );
+}
+
+void QgsMapToolCapture::validateGeometry()
+{
+  QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( mCanvas->currentLayer() );
+
+  mGeomErrors.clear();
+
+  while ( !mGeomErrorMarkers.isEmpty() )
+  {
+    delete mGeomErrorMarkers.takeFirst();
+  }
+
+  switch ( mCaptureMode )
+  {
+    case CapturePoint:
+      return;
+    case CaptureLine:
+      if ( mCaptureList.size() < 2 )
+        return;
+      QgsGeometry::validatePolyline( mGeomErrors, 0, mCaptureList.toVector() );
+      break;
+    case CapturePolygon:
+      if ( mCaptureList.size() < 3 )
+        return;
+      QgsGeometry::validatePolyline( mGeomErrors, 0, mCaptureList.toVector() << mCaptureList[0], true );
+      break;
+  }
+
+  QString tip;
+  for ( int i = 0; i < mGeomErrors.size(); i++ )
+  {
+    tip += mGeomErrors[i].what() + "\n";
+    if ( !mGeomErrors[i].hasWhere() )
+      continue;
+
+    QgsVertexMarker *vm =  new QgsVertexMarker( mCanvas );
+    vm->setCenter( mCanvas->mapRenderer()->layerToMapCoordinates( vlayer, mGeomErrors[i].where() ) );
+    vm->setIconType( QgsVertexMarker::ICON_X );
+    vm->setPenWidth( 2 );
+    vm->setToolTip( mGeomErrors[i].what() );
+    vm->setColor( Qt::green );
+    vm->setZValue( vm->zValue() + 1 );
+    mGeomErrorMarkers << vm;
+  }
+
+  QStatusBar *sb = QgisApp::instance()->statusBar();
+  sb->showMessage( QObject::tr( "%n geometry error(s) found.", "number of geometry errors", mGeomErrors.size() ) );
+  if ( !tip.isEmpty() )
+    sb->setToolTip( tip );
+}
