@@ -120,7 +120,7 @@ void QgsMapToolIdentify::canvasReleaseEvent( QMouseEvent * e )
       emit identifyProgress( i, mCanvas->layerCount() );
       emit identifyMessage( tr( "Identifying on %1..." ).arg( layer->name() ) );
 
-      if ( noIdentifyLayerIdList.contains( layer->getLayerID() ) )
+      if ( noIdentifyLayerIdList.contains( layer->id() ) )
         continue;
 
       if ( identifyLayer( layer, e->x(), e->y() ) )
@@ -143,7 +143,6 @@ void QgsMapToolIdentify::canvasReleaseEvent( QMouseEvent * e )
   if ( res )
   {
     results()->show();
-    results()->raise();
   }
   else
   {
@@ -210,13 +209,6 @@ bool QgsMapToolIdentify::identifyVectorLayer( QgsVectorLayer *layer, int x, int
     identifyValue = QGis::DEFAULT_IDENTIFY_RADIUS;
 
   int featureCount = 0;
-  const QgsFieldMap& fields = layer->pendingFields();
-
-  // init distance/area calculator
-  QgsDistanceArea calc;
-  calc.setProjectionsEnabled( mCanvas->hasCrsTransformEnabled() ); // project?
-  calc.setEllipsoid( ellipsoid );
-  calc.setSourceCrs( layer->srs().srsid() );
 
   QgsFeatureList featureList;
 
@@ -248,6 +240,14 @@ bool QgsMapToolIdentify::identifyVectorLayer( QgsVectorLayer *layer, int x, int
     QgsDebugMsg( QString( "Caught CRS exception %1" ).arg( cse.what() ) );
   }
 
+  // init distance/area calculator
+  QgsDistanceArea calc;
+  if ( !featureList.count() == 0 )
+  {
+      calc.setProjectionsEnabled( mCanvas->hasCrsTransformEnabled() ); // project?
+      calc.setEllipsoid( ellipsoid );
+      calc.setSourceCrs( layer->crs().srsid() );
+  }
   QgsFeatureList::iterator f_it = featureList.begin();
 
   for ( ; f_it != featureList.end(); ++f_it )
@@ -255,37 +255,7 @@ bool QgsMapToolIdentify::identifyVectorLayer( QgsVectorLayer *layer, int x, int
     featureCount++;
 
     int fid = f_it->id();
-    QString displayField, displayValue;
-    QMap<QString, QString> attributes, derivedAttributes;
-
-    const QgsAttributeMap& attr = f_it->attributeMap();
-
-    for ( QgsAttributeMap::const_iterator it = attr.begin(); it != attr.end(); ++it )
-    {
-      QString attributeName  = layer->attributeDisplayName( it.key() );
-      QString attributeValue = it->isNull() ? "NULL" : it->toString();
-
-      switch ( layer->editType( it.key() ) )
-      {
-        case QgsVectorLayer::Hidden:
-          continue;
-
-        case QgsVectorLayer::ValueMap:
-          attributeValue = layer->valueMap( it.key() ).key( it->toString(), QString( "(%1)" ).arg( it->toString() ) );
-          break;
-
-        default:
-          break;
-      }
-
-      if ( fields[it.key()].name() == layer->displayField() )
-      {
-        displayField = attributeName;
-        displayValue = attributeValue;
-      }
-
-      attributes.insert( attributeName, attributeValue );
-    }
+    QMap<QString, QString> derivedAttributes;
 
     // Calculate derived attributes and insert:
     // measure distance or area depending on geometry type
@@ -296,7 +266,8 @@ bool QgsMapToolIdentify::identifyVectorLayer( QgsVectorLayer *layer, int x, int
       convertMeasurement( calc, dist, myDisplayUnits, false );
       QString str = calc.textUnit( dist, 3, myDisplayUnits, false );  // dist and myDisplayUnits are out params
       derivedAttributes.insert( tr( "Length" ), str );
-      if ( f_it->geometry()->wkbType() == QGis::WKBLineString )
+      if ( f_it->geometry()->wkbType() == QGis::WKBLineString ||
+           f_it->geometry()->wkbType() == QGis::WKBLineString25D )
       {
         // Add the start and end points in as derived attributes
         str = QLocale::system().toString( f_it->geometry()->asPolyline().first().x(), 'g', 10 );
@@ -317,7 +288,9 @@ bool QgsMapToolIdentify::identifyVectorLayer( QgsVectorLayer *layer, int x, int
       QString str = calc.textUnit( area, 3, myDisplayUnits, true );
       derivedAttributes.insert( tr( "Area" ), str );
     }
-    else if ( layer->geometryType() == QGis::Point )
+    else if ( layer->geometryType() == QGis::Point &&
+              ( f_it->geometry()->wkbType() == QGis::WKBPoint ||
+                f_it->geometry()->wkbType() == QGis::WKBPoint25D ) )
     {
       // Include the x and y coordinates of the point as a derived attribute
       QString str;
@@ -329,7 +302,7 @@ bool QgsMapToolIdentify::identifyVectorLayer( QgsVectorLayer *layer, int x, int
 
     derivedAttributes.insert( tr( "feature id" ), fid < 0 ? tr( "new feature" ) : QString::number( fid ) );
 
-    results()->addFeature( layer, fid, displayField, displayValue, attributes, derivedAttributes );
+    results()->addFeature( layer, *f_it, derivedAttributes );
   }
 
   QgsDebugMsg( "Feature count on identify: " + QString::number( featureCount ) );
@@ -350,6 +323,7 @@ bool QgsMapToolIdentify::identifyRasterLayer( QgsRasterLayer *layer, int x, int
 
   QMap< QString, QString > attributes, derivedAttributes;
   QgsPoint idPoint = mCanvas->getCoordinateTransform()->toMapCoordinates( x, y );
+  idPoint = toLayerCoordinates( layer, idPoint );
   QString type;
 
   if ( layer->providerKey() == "wms" )
@@ -359,7 +333,7 @@ bool QgsMapToolIdentify::identifyRasterLayer( QgsRasterLayer *layer, int x, int
     //if WMS layer does not cover the view origin,
     //we need to map the view pixel coordinates
     //to WMS layer pixel coordinates
-    QgsRectangle viewExtent = mCanvas->extent();
+    QgsRectangle viewExtent = toLayerCoordinates( layer, mCanvas->extent() );
     QgsRectangle layerExtent = layer->extent();
     double mapUnitsPerPixel = mCanvas->mapUnitsPerPixel();
     if ( mapUnitsPerPixel > 0 && viewExtent.intersects( layerExtent ) )
@@ -374,7 +348,7 @@ bool QgsMapToolIdentify::identifyRasterLayer( QgsRasterLayer *layer, int x, int
         yMaxView > yMaxLayer ? floor( y - ( yMaxView - yMaxLayer ) / mapUnitsPerPixel ) : y
       );
 
-      attributes.insert( tr( "Feature info" ), layer->identifyAsText( idPoint ) );
+      attributes.insert( tr( "Feature info" ), layer->identifyAsHtml( idPoint ) );
     }
     else
     {
@@ -390,7 +364,7 @@ bool QgsMapToolIdentify::identifyRasterLayer( QgsRasterLayer *layer, int x, int
   if ( res )
   {
     derivedAttributes.insert( tr( "(clicked coordinate)" ), idPoint.toString() );
-    results()->addFeature( layer, -1, type, "", attributes, derivedAttributes );
+    results()->addFeature( layer, type, attributes, derivedAttributes );
   }
 
   return res;
