@@ -21,9 +21,14 @@
 #include "qgsvectordataprovider.h"
 #include "qgstolerance.h"
 #include "qgsgeometry.h"
-#include <math.h>
+#include <cmath>
 #include <QMouseEvent>
 #include <QMessageBox>
+#include "qgslogger.h"
+#include "qgisapp.h"
+#include "qgslegend.h"
+
+#include <QStatusBar>
 
 
 QgsRubberBand* QgsMapToolNodeTool::createRubberBandMarker( QgsPoint center, QgsVectorLayer* vlayer )
@@ -49,8 +54,6 @@ QgsRubberBand* QgsMapToolNodeTool::createRubberBandMarker( QgsPoint center, QgsV
   return marker;
 }
 
-
-
 QgsMapToolNodeTool::QgsMapToolNodeTool( QgsMapCanvas* canvas ): QgsMapToolVertexEdit( canvas )
 {
   mSelectionFeature = NULL;
@@ -67,6 +70,9 @@ QgsMapToolNodeTool::QgsMapToolNodeTool( QgsMapCanvas* canvas ): QgsMapToolVertex
   connect( canvas->mapRenderer(), SIGNAL( destinationSrsChanged() ), this, SLOT( coordinatesChanged() ) );
   //signal changing of coordinate renderer changed to repaint markers
   connect( canvas->mapRenderer(), SIGNAL( hasCrsTransformEnabled( bool ) ), this, SLOT( coordinatesChanged( ) ) );
+  //signal changing of current layer
+  connect( QgisApp::instance()->legend(), SIGNAL( currentLayerChanged( QgsMapLayer* ) ),
+           this, SLOT( currentLayerChanged( QgsMapLayer* ) ) );
 }
 
 QgsMapToolNodeTool::~QgsMapToolNodeTool()
@@ -117,20 +123,10 @@ void QgsMapToolNodeTool::layerModified( bool onlyGeometry )
   QgsVectorLayer* vlayer = qobject_cast<QgsVectorLayer *>( mCanvas->currentLayer() );
   if ( mSelectionFeature != NULL && !mChangingGeometry && ( vlayer->featureAtId( mSelectionFeature->featureId(), feat, true, false ) ) )
   {
-    try
+    if ( !feat.geometry()->isGeosEqual( *mSelectionFeature->feature()->geometry() ) )
     {
-      if ( !GEOSEquals( feat.geometry()->asGeos(), mSelectionFeature->feature()->geometry()->asGeos() ) )
-      {
-        mSelectionFeature->updateFromFeature();
-        //throw error
-      }
-    }
-    catch ( ... )
-    {
-      //GEOS is throwing exception when polygon is not valid to be able to edit it
-      //Only possibility to fix this operation sice node tool doesn't allow to strore invalid geometry after editing
       mSelectionFeature->updateFromFeature();
-      //if it does update markers just to be sure of correctness
+      //throw error
     }
   }
 }
@@ -148,7 +144,7 @@ void QgsMapToolNodeTool::createMovingRubberBands()
   for ( int i = 0; i < vertexMap.size(); i++ )
   {
     //create rubber band
-    if ( vertexMap[i].selected && !( vertexMap[i].inRubberBand ) )
+    if ( vertexMap[i].selected && !vertexMap[i].inRubberBand )
     {
       geometry->adjacentVertices( i, beforeVertex, afterVertex );
       vertex = i;
@@ -175,7 +171,7 @@ void QgsMapToolNodeTool::createMovingRubberBands()
         rb->addPoint( toMapCoordinates( mCanvas->currentLayer(), vertexMap[beforeVertex].point ), false );
         mSelectionFeature->setRubberBandValues( beforeVertex, true, lastRubberBand, index );
         vertexMap[beforeVertex].inRubberBand = true;
-        index ++;
+        index++;
       }
       while ( vertex != -1 && vertexMap[vertex].selected && !vertexMap[vertex].inRubberBand )
       {
@@ -189,7 +185,7 @@ void QgsMapToolNodeTool::createMovingRubberBands()
         //setting values about added vertex
         mSelectionFeature->setRubberBandValues( vertex, true, lastRubberBand, index );
         vertexMap[vertex].inRubberBand = true;
-        index ++;
+        index++;
         geometry->adjacentVertices( vertex, beforeVertex, vertex );
       }
       if ( vertex != -1 && !vertexMap[vertex].selected ) //add last point not moving if exists
@@ -197,10 +193,10 @@ void QgsMapToolNodeTool::createMovingRubberBands()
         rb->addPoint( toMapCoordinates( mCanvas->currentLayer(), vertexMap[vertex].point ), true );
         mSelectionFeature->setRubberBandValues( vertex, true, lastRubberBand, index );
         vertexMap[vertex].inRubberBand = true;
-        index ++;
+        index++;
       }
       mQgsRubberBands.append( rb );
-      lastRubberBand ++;
+      lastRubberBand++;
     }
   }
 }
@@ -396,7 +392,7 @@ void QgsMapToolNodeTool::canvasMoveEvent( QMouseEvent * e )
             double x = mapCoords.x() + posMapCoord.x() - firstCoords.x();
             double y = mapCoords.y() + posMapCoord.y() - firstCoords.y();
 
-            mQgsRubberBands[vertexMap[i].rubberBandNr]->movePoint( vertexMap[i].index + 1, QgsPoint( x, y ) );
+            mQgsRubberBands[vertexMap[i].rubberBandNr]->movePoint( vertexMap[i].index, QgsPoint( x, y ) );
             if ( vertexMap[i].index == 0 )
             {
               mQgsRubberBands[vertexMap[i].rubberBandNr]->movePoint( 0, QgsPoint( x, y ) );
@@ -409,16 +405,16 @@ void QgsMapToolNodeTool::canvasMoveEvent( QMouseEvent * e )
         double offsetY = posMapCoord.y() - mPosMapCoordBackup.y();
         for ( int i = 0; i < mTopologyRubberBand.size(); i++ )
         {
-          for ( int pointIndex = 0; pointIndex < mTopologyRubberBand[i]->numberOfVertices() - 1; pointIndex ++ )
+          for ( int pointIndex = 0; pointIndex < mTopologyRubberBand[i]->numberOfVertices() - 1; pointIndex++ )
           {
             if ( mTopologyRubberBandVertexes[i]->contains( pointIndex ) )
             {
-              const QgsPoint* point = mTopologyRubberBand[i]->getPoint( 0, pointIndex + 1 );
+              const QgsPoint* point = mTopologyRubberBand[i]->getPoint( 0, pointIndex );
               if ( point == 0 )
               {
                 break;
               }
-              mTopologyRubberBand[i]->movePoint( pointIndex + 1, QgsPoint( point->x() + offsetX, point->y() + offsetY ) );
+              mTopologyRubberBand[i]->movePoint( pointIndex, QgsPoint( point->x() + offsetX, point->y() + offsetY ) );
               if ( pointIndex == 0 )
               {
                 mTopologyRubberBand[i]->movePoint( pointIndex , QgsPoint( point->x() + offsetX, point->y() + offsetY ) );
@@ -470,7 +466,7 @@ bool QgsMapToolNodeTool::checkCorrectnessOfFeature( QgsVectorLayer *vlayer )
   }
   try
   {
-    if ( !GEOSEquals( feat.geometry()->asGeos(), mSelectionFeature->feature()->geometry()->asGeos() ) )
+    if ( !feat.geometry()->isGeosEqual( *mSelectionFeature->feature()->geometry() ) )
     {
       //update markers when geometries are not valid
       mSelectionFeature->updateFromFeature();
@@ -552,7 +548,7 @@ void QgsMapToolNodeTool::canvasPressEvent( QMouseEvent * e )
         mSelectAnother = true;
         QList<QgsSnappingResult>::iterator it = snapResults.begin();
         QgsSnappingResult snapResult;
-        for ( ; it != snapResults.end() ; ++it )
+        for ( ; it != snapResults.end(); ++it )
         {
           if ( it->snappedAtGeometry == mSelectionFeature->featureId() )
           {
@@ -774,7 +770,7 @@ void QgsMapToolNodeTool::removeRubberBands()
   mTopologyMovingVertexes.clear();
   mTopologyRubberBandVertexes.clear();
 
-  //remove all data from selected feature (no change to rubber bands itself
+  //remove all data from selected feature (no change to rubber bands itself)
   if ( mSelectionFeature != NULL )
     mSelectionFeature->cleanRubberBandsData();
 }
@@ -872,7 +868,7 @@ void QgsMapToolNodeTool::keyReleaseEvent( QKeyEvent* e )
     mCtrl = false;
     return;
   }
-  if ( e->key() == Qt::Key_Delete )
+  if ( mSelectionFeature && e->key() == Qt::Key_Delete )
   {
     mChangingGeometry = true;
     mSelectionFeature->deleteSelectedVertexes();
@@ -895,6 +891,11 @@ SelectionFeature::SelectionFeature()
 SelectionFeature::~SelectionFeature()
 {
   deleteVertexMap();
+
+  while ( !mGeomErrorMarkers.isEmpty() )
+  {
+    delete mGeomErrorMarkers.takeFirst();
+  }
 }
 
 void SelectionFeature::updateFeature()
@@ -936,8 +937,46 @@ void SelectionFeature::setSelectedFeature( int featureId,  QgsVectorLayer* vlaye
   {
     mFeature = feature;
   }
+
   //createvertexmap
   createVertexMap();
+
+  validateGeometry();
+}
+
+void SelectionFeature::validateGeometry( QgsGeometry *g )
+{
+  QgsDebugMsg( "validating geometry" );
+
+  if ( g == NULL )
+    g = mFeature->geometry();
+
+  g->validateGeometry( mGeomErrors );
+
+  while ( !mGeomErrorMarkers.isEmpty() )
+  {
+    delete mGeomErrorMarkers.takeFirst();
+  }
+
+  QString tip;
+
+  for ( int i = 0; i < mGeomErrors.size(); i++ )
+  {
+    tip += mGeomErrors[i].what() + "\n";
+    if ( !mGeomErrors[i].hasWhere() )
+      continue;
+
+    QgsVertexMarker *vm = createVertexMarker( mGeomErrors[i].where(), QgsVertexMarker::ICON_X );
+    vm->setToolTip( mGeomErrors[i].what() );
+    vm->setColor( Qt::green );
+    vm->setZValue( vm->zValue() + 1 );
+    mGeomErrorMarkers << vm;
+  }
+
+  QStatusBar *sb = QgisApp::instance()->statusBar();
+  sb->showMessage( QObject::tr( "%n geometry error(s) found.", "number of geometry errors", mGeomErrors.size() ) );
+  if ( !tip.isEmpty() )
+    sb->setToolTip( tip );
 }
 
 void SelectionFeature::deleteSelectedVertexes()
@@ -946,7 +985,7 @@ void SelectionFeature::deleteSelectedVertexes()
   QMultiMap<double, QgsSnappingResult> currentResultList;
   mVlayer->beginEditCommand( QObject::tr( "Deleted vertices" ) );
   int count = 0;
-  for ( int i = mVertexMap.size() - 1; i > -1 ; i-- )
+  for ( int i = mVertexMap.size() - 1; i > -1; i-- )
   {
     if ( mVertexMap[i].selected )
     {
@@ -983,7 +1022,10 @@ void SelectionFeature::deleteSelectedVertexes()
   }
   QgsFeature f;
   mVlayer->featureAtId( mFeatureId, f, true, false );
-  if ( !GEOSisValid( f.geometry()->asGeos() ) )
+
+  bool wasValid = false; // mGeomErrors.isEmpty();
+  bool isValid = f.geometry()->isGeosValid();
+  if ( wasValid && !isValid )
   {
     QMessageBox::warning( NULL,
                           tr( "Node tool" ),
@@ -992,9 +1034,10 @@ void SelectionFeature::deleteSelectedVertexes()
                           QMessageBox::Ok );
   }
 
-  if ( count != 0 && GEOSisValid( f.geometry()->asGeos() ) )
+  if ( count != 0 && ( !wasValid || isValid ) )
   {
     mVlayer->endEditCommand();
+    validateGeometry( f.geometry() );
   }
   else
   {
@@ -1010,7 +1053,7 @@ void SelectionFeature::moveSelectedVertexes( double changeX, double changeY )
   mVlayer->beginEditCommand( QObject::tr( "Moved vertices" ) );
   int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
   QMultiMap<double, QgsSnappingResult> currentResultList;
-  for ( int i = mVertexMap.size() - 1; i > -1 ; i-- )
+  for ( int i = mVertexMap.size() - 1; i > -1; i-- )
   {
     if ( mVertexMap[i].selected )
     {
@@ -1054,7 +1097,9 @@ void SelectionFeature::moveSelectedVertexes( double changeX, double changeY )
   }
   QgsFeature f;
   mVlayer->featureAtId( mFeatureId, f, true, false );
-  if ( !GEOSisValid( f.geometry()->asGeos() ) )
+  bool wasValid = false; // mGeomErrors.isEmpty();
+  bool isValid = f.geometry()->isGeosValid();
+  if ( wasValid && !isValid )
   {
     QMessageBox::warning( NULL,
                           tr( "Node tool" ),
@@ -1068,17 +1113,18 @@ void SelectionFeature::moveSelectedVertexes( double changeX, double changeY )
   else
   {
     mVlayer->endEditCommand();
+    validateGeometry( f.geometry() );
   }
   updateFeature();
 }
 
-QgsVertexMarker* SelectionFeature::createVertexMarker( QgsPoint center )
+QgsVertexMarker *SelectionFeature::createVertexMarker( QgsPoint center, QgsVertexMarker::IconType type )
 {
-  QgsVertexMarker* marker = new QgsVertexMarker( mCanvas );
+  QgsVertexMarker *marker = new QgsVertexMarker( mCanvas );
   QgsPoint newCenter = mCanvas->mapRenderer()->layerToMapCoordinates( mVlayer, center );
   marker->setCenter( newCenter );
 
-  marker->setIconType( QgsVertexMarker::ICON_BOX );
+  marker->setIconType( type );
 
   marker->setColor( Qt::red );
 
@@ -1118,11 +1164,12 @@ QgsFeature* SelectionFeature::feature()
 void SelectionFeature::createVertexMapPolygon()
 {
   int y = 0;
-  if ( !mFeature->geometry()->asPolygon().empty() )
+  QgsPolygon polygon = mFeature->geometry()->asPolygon();
+  if ( !polygon.empty() )
   { //polygon
-    for ( int i2 = 0; i2 < mFeature->geometry()->asPolygon().size(); i2++ )
+    for ( int i2 = 0; i2 < polygon.size(); i2++ )
     {
-      QgsPolyline poly = mFeature->geometry()->asPolygon()[i2];
+      const QgsPolyline& poly = polygon[i2];
       int i;
       for ( i = 0; i < poly.size(); i++ )
       {
@@ -1134,6 +1181,7 @@ void SelectionFeature::createVertexMapPolygon()
         entry.originalIndex = i;
         entry.inRubberBand = false;
         QgsVertexMarker* marker = createVertexMarker( poly[i] );
+        marker->setToolTip( tr( "ring %1, vertex %2" ).arg( i2 ).arg( i ) );
         entry.vertexMarker = marker;
         mVertexMap.insert( y + i, entry );
       }
@@ -1144,12 +1192,13 @@ void SelectionFeature::createVertexMapPolygon()
   }
   else //multipolygon
   {
-    for ( int i2 = 0 ; i2 < mFeature->geometry()->asMultiPolygon().size(); i2++ )
+    QgsMultiPolygon multiPolygon = mFeature->geometry()->asMultiPolygon();
+    for ( int i2 = 0; i2 < multiPolygon.size(); i2++ )
     { //iterating through polygons
-      QgsPolygon poly2 = mFeature->geometry()->asMultiPolygon()[i2];
+      const QgsPolygon& poly2 = multiPolygon[i2];
       for ( int i3 = 0; i3 < poly2.size(); i3++ )
       { //iterating through polygon rings
-        QgsPolyline poly = poly2[i3];
+        const QgsPolyline& poly = poly2[i3];
         int i;
         for ( i = 0; i < poly.size(); i++ )
         {
@@ -1161,7 +1210,7 @@ void SelectionFeature::createVertexMapPolygon()
           entry.originalIndex = y + i - 1;
           entry.inRubberBand = false;
           QgsVertexMarker* marker = createVertexMarker( poly[i] );
-
+          marker->setToolTip( tr( "polygon %1, ring %2, vertex %3" ).arg( i2 ).arg( i3 ).arg( i ) );
           entry.vertexMarker = marker;
           mVertexMap.insert( y + i, entry );
         }
@@ -1194,6 +1243,7 @@ void SelectionFeature::createVertexMapLine()
         entry.originalIndex = i;
         entry.inRubberBand = false;
         QgsVertexMarker* marker = createVertexMarker( poly[i] );
+        marker->setToolTip( tr( "polyline %1, vertex %2" ).arg( i2 ).arg( i ) );
         entry.vertexMarker = marker;
         mVertexMap.insert( y + i, entry );
       }
@@ -1214,6 +1264,7 @@ void SelectionFeature::createVertexMapLine()
       entry.originalIndex = i;
       entry.inRubberBand = false;
       QgsVertexMarker* marker = createVertexMarker( poly[i] );
+      marker->setToolTip( tr( "vertex %1" ).arg( i ) );
       entry.vertexMarker = marker;
       mVertexMap.insert( i, entry );
     }
@@ -1236,6 +1287,7 @@ void SelectionFeature::createVertexMapPoint()
       entry.originalIndex = 1;
       entry.inRubberBand = false;
       QgsVertexMarker* marker = createVertexMarker( poly[i] );
+      marker->setToolTip( tr( "point %1" ).arg( i ) );
       entry.vertexMarker = marker;
       mVertexMap.insert( i, entry );
     }
@@ -1251,6 +1303,7 @@ void SelectionFeature::createVertexMapPoint()
     entry.originalIndex = 1;
     entry.inRubberBand = false;
     QgsVertexMarker* marker = createVertexMarker( poly );
+    marker->setToolTip( tr( "single point" ) );
     entry.vertexMarker = marker;
     mVertexMap.insert( 1, entry );
   }
@@ -1306,7 +1359,7 @@ void SelectionFeature::deselectVertex( int vertexNr )
 
 void SelectionFeature::deselectAllVertexes()
 {
-  for ( int i = 0; i < mVertexMap.size() ; i++ )
+  for ( int i = 0; i < mVertexMap.size(); i++ )
   {
     mVertexMap[i].selected = false;
     mVertexMap[i].vertexMarker->setColor( Qt::red );
@@ -1318,7 +1371,7 @@ void SelectionFeature::deselectAllVertexes()
 void SelectionFeature::invertVertexSelection( int vertexNr, bool invert )
 {
   //inverting of selection of vertex
-  if ( mVertexMap[vertexNr].selected == false )
+  if ( !mVertexMap[vertexNr].selected )
   {
     //case vertex is not selected
     mVertexMap[vertexNr].selected = true;
@@ -1349,7 +1402,7 @@ void SelectionFeature::invertVertexSelection( int vertexNr, bool invert )
 void SelectionFeature::updateVertexMarkersPosition( QgsMapCanvas* canvas )
 {
   //function for on-line updating vertex markers without refresh of canvas
-  for ( int i = 0; i < mVertexMap.size() ; i++ )
+  for ( int i = 0; i < mVertexMap.size(); i++ )
   {
     mVertexMap[i].vertexMarker->setCenter( mCanvas->mapRenderer()->layerToMapCoordinates( mVlayer, mVertexMap[i].point ) );
     mVertexMap[i].vertexMarker->update();
@@ -1372,4 +1425,3 @@ QgsVectorLayer* SelectionFeature::vlayer()
 {
   return mVlayer;
 }
-
