@@ -1,9 +1,9 @@
 /***************************************************************************
-    qgsmaptoolselect.h  -  map tool for selecting features
+    qgsmaptoolselect.h  -  map tool for selecting features by single click
     ---------------------
-    begin                : January 2006
-    copyright            : (C) 2006 by Martin Dobias
-    email                : wonder.sk at gmail dot com
+    begin                : May 2010
+    copyright            : (C) 2010 by Jeremy Palmer
+    email                : jpalmer at linz dot govt dot nz
  ***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,12 +18,9 @@
 #define QGSMAPTOOLSELECT_H
 
 #include "qgsmaptool.h"
-#include <QRect>
 
-
-class QRubberBand;
 class QgsMapCanvas;
-
+class QMouseEvent;
 
 class QgsMapToolSelect : public QgsMapTool
 {
@@ -31,26 +28,10 @@ class QgsMapToolSelect : public QgsMapTool
   public:
     QgsMapToolSelect( QgsMapCanvas* canvas );
 
-    //! Overridden mouse move event
-    virtual void canvasMoveEvent( QMouseEvent * e );
-
-    //! Overridden mouse press event
-    virtual void canvasPressEvent( QMouseEvent * e );
-
     //! Overridden mouse release event
     virtual void canvasReleaseEvent( QMouseEvent * e );
 
-
-  protected:
-
-    //! stores actual select rect
-    QRect mSelectRect;
-
-    //! Flag to indicate a map canvas drag operation is taking place
-    bool mDragging;
-
-    //! rubber band for select rect
-    QRubberBand* mRubberBand;
+  private:
 };
 
 #endif
