@@ -47,8 +47,8 @@ void QgsMapToolSplitFeatures::canvasReleaseEvent( QMouseEvent * e )
   if ( !vlayer->isEditable() )
   {
     QMessageBox::information( 0, tr( "Layer not editable" ),
-                              tr( "Cannot edit the vector layer. To make it editable, go to the file item "
-                                  "of the layer, right click and check 'Allow Editing'." ) );
+                              tr( "Cannot edit the vector layer. Use 'Toggle Editing' to make it editable." )
+                            );
     return;
   }
 
@@ -69,30 +69,33 @@ void QgsMapToolSplitFeatures::canvasReleaseEvent( QMouseEvent * e )
 
   if ( e->button() == Qt::LeftButton )
   {
-    mCapturing = TRUE;
+    startCapturing();
   }
   else if ( e->button() == Qt::RightButton )
   {
-    mCapturing = FALSE;
-    delete mRubberBand;
-    mRubberBand = 0;
-
     //bring up dialog if a split was not possible (polygon) or only done once (line)
     int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
     vlayer->beginEditCommand( tr( "Features split" ) );
-    int returnCode = vlayer->splitFeatures( mCaptureList, topologicalEditing );
+    int returnCode = vlayer->splitFeatures( points(), topologicalEditing );
     vlayer->endEditCommand();
     if ( returnCode == 4 )
     {
       QMessageBox::warning( 0, tr( "No feature split done" ), tr( "If there are selected features, the split tool only applies to the selected ones. If you like to split all features under the split line, clear the selection" ) );
     }
+    else if ( returnCode == 3 )
+    {
+      QMessageBox::warning( 0, tr( "No feature split done" ), tr( "Cut edges detected. Make sure the line splits features into multiple parts." ) );
+    }
+    else if ( returnCode == 7 )
+    {
+      QMessageBox::warning( 0, tr( "No feature split done" ), tr( "The geometry is invalid. Please repair before trying to split it." ) );
+    }
     else if ( returnCode != 0 )
     {
       //several intersections but only one split (most likely line)
       QMessageBox::warning( 0, tr( "Split error" ), tr( "An error occured during feature splitting" ) );
     }
 
-    mCaptureList.clear();
-    mCanvas->refresh();
+    stopCapturing();
   }
 }
