@@ -51,6 +51,16 @@ QgsMeasureDialog::QgsMeasureDialog( QgsMeasureTool* tool, Qt::WFlags f )
 
   //mTable->setHeaderLabels(QStringList() << tr("Segments (in meters)") << tr("Total") << tr("Azimuth") );
 
+  QSettings settings;
+  int s = settings.value( "/qgis/measure/projectionEnabled", "2" ).toInt();
+  if ( s == 2 )
+    mcbProjectionEnabled->setCheckState( Qt::Checked );
+  else
+    mcbProjectionEnabled->setCheckState( Qt::Unchecked );
+
+  connect( mcbProjectionEnabled, SIGNAL( stateChanged( int ) ),
+           this, SLOT( changeProjectionEnabledState() ) );
+
   updateUi();
 }
 
@@ -76,7 +86,7 @@ void QgsMeasureDialog::mousePress( QgsPoint &point )
   if ( mTool->points().size() == 0 )
   {
     addPoint( point );
-    this->show();
+    show();
   }
   raise();
 
@@ -85,6 +95,9 @@ void QgsMeasureDialog::mousePress( QgsPoint &point )
 
 void QgsMeasureDialog::mouseMove( QgsPoint &point )
 {
+  QSettings settings;
+  int decimalPlaces = settings.value( "/qgis/measure/decimalplaces", "3" ).toInt();
+
   // show current distance/area while moving the point
   // by creating a temporary copy of point array
   // and adding moving point at the end
@@ -92,30 +105,33 @@ void QgsMeasureDialog::mouseMove( QgsPoint &point )
   {
     QList<QgsPoint> tmpPoints = mTool->points();
     tmpPoints.append( point );
-    double area = mTool->canvas()->mapRenderer()->distanceArea()->measurePolygon( tmpPoints );
-    editTotal->setText( formatArea( area ) );
+    double area = mDa.measurePolygon( tmpPoints );
+    editTotal->setText( formatArea( area, decimalPlaces ) );
   }
   else if ( !mMeasureArea && mTool->points().size() > 0 )
   {
     QgsPoint p1( mTool->points().last() ), p2( point );
 
-    double d = mTool->canvas()->mapRenderer()->distanceArea()->measureLine( p1, p2 );
-    editTotal->setText( formatDistance( mTotal + d ) );
+    double d = mDa.measureLine( p1, p2 );
+    editTotal->setText( formatDistance( mTotal + d, decimalPlaces ) );
     QGis::UnitType myDisplayUnits;
     // Ignore units
     convertMeasurement( d, myDisplayUnits, false );
     QTreeWidgetItem *item = mTable->topLevelItem( mTable->topLevelItemCount() - 1 );
-    item->setText( 0, QLocale::system().toString( d, 'f', 2 ) );
+    item->setText( 0, QLocale::system().toString( d, 'f', decimalPlaces ) );
   }
 }
 
 void QgsMeasureDialog::addPoint( QgsPoint &point )
 {
+  QSettings settings;
+  int decimalPlaces = settings.value( "/qgis/measure/decimalplaces", "3" ).toInt();
+
   int numPoints = mTool->points().size();
   if ( mMeasureArea && numPoints > 2 )
   {
-    double area = mTool->canvas()->mapRenderer()->distanceArea()->measurePolygon( mTool->points() );
-    editTotal->setText( formatArea( area ) );
+    double area = mDa.measurePolygon( mTool->points() );
+    editTotal->setText( formatArea( area, decimalPlaces ) );
   }
   else if ( !mMeasureArea && numPoints > 1 )
   {
@@ -123,19 +139,19 @@ void QgsMeasureDialog::addPoint( QgsPoint &point )
 
     QgsPoint p1 = mTool->points()[last], p2 = mTool->points()[last+1];
 
-    double d = mTool->canvas()->mapRenderer()->distanceArea()->measureLine( p1, p2 );
+    double d = mDa.measureLine( p1, p2 );
 
     mTotal += d;
-    editTotal->setText( formatDistance( mTotal ) );
+    editTotal->setText( formatDistance( mTotal, decimalPlaces ) );
 
     QGis::UnitType myDisplayUnits;
     // Ignore units
     convertMeasurement( d, myDisplayUnits, false );
 
     QTreeWidgetItem *item = mTable->topLevelItem( mTable->topLevelItemCount() - 1 );
-    item->setText( 0, QLocale::system().toString( d, 'f', 2 ) );
+    item->setText( 0, QLocale::system().toString( d, 'f', decimalPlaces ) );
 
-    item = new QTreeWidgetItem( QStringList( QLocale::system().toString( 0.0, 'f', 2 ) ) );
+    item = new QTreeWidgetItem( QStringList( QLocale::system().toString( 0.0, 'f', decimalPlaces ) ) );
     item->setTextAlignment( 0, Qt::AlignRight );
     mTable->addTopLevelItem( item );
     mTable->scrollToItem( item );
@@ -175,22 +191,31 @@ void QgsMeasureDialog::saveWindowLocation()
   settings.setValue( key, height() );
 }
 
-QString QgsMeasureDialog::formatDistance( double distance )
+QString QgsMeasureDialog::formatDistance( double distance, int decimalPlaces )
 {
+  QSettings settings;
+  bool baseUnit = settings.value( "/qgis/measure/keepbaseunit", false ).toBool();
+
   QGis::UnitType myDisplayUnits;
   convertMeasurement( distance, myDisplayUnits, false );
-  return QgsDistanceArea::textUnit( distance, 2, myDisplayUnits, false );
+  return QgsDistanceArea::textUnit( distance, decimalPlaces, myDisplayUnits, false, baseUnit );
 }
 
-QString QgsMeasureDialog::formatArea( double area )
+QString QgsMeasureDialog::formatArea( double area, int decimalPlaces )
 {
+  QSettings settings;
+  bool baseUnit = settings.value( "/qgis/measure/keepbaseunit", false ).toBool();
+
   QGis::UnitType myDisplayUnits;
   convertMeasurement( area, myDisplayUnits, true );
-  return QgsDistanceArea::textUnit( area, 2, myDisplayUnits, true );
+  return QgsDistanceArea::textUnit( area, decimalPlaces, myDisplayUnits, true, baseUnit );
 }
 
 void QgsMeasureDialog::updateUi()
 {
+  QSettings settings;
+  int decimalPlaces = settings.value( "/qgis/measure/decimalplaces", "3" ).toInt();
+
   double dummy = 1.0;
   QGis::UnitType myDisplayUnits;
   // The dummy distance is ignored
@@ -211,19 +236,20 @@ void QgsMeasureDialog::updateUi()
       break;
     case QGis::UnknownUnit:
       mTable->setHeaderLabels( QStringList( tr( "Segments" ) ) );
-  };
+  }
 
   if ( mMeasureArea )
   {
     mTable->hide();
-    editTotal->setText( formatArea( 0 ) );
+    editTotal->setText( formatArea( 0, decimalPlaces ) );
   }
   else
   {
     mTable->show();
-    editTotal->setText( formatDistance( 0 ) );
+    editTotal->setText( formatDistance( 0, decimalPlaces ) );
   }
 
+  configureDistanceArea();
 }
 
 void QgsMeasureDialog::convertMeasurement( double &measure, QGis::UnitType &u, bool isArea )
@@ -232,9 +258,8 @@ void QgsMeasureDialog::convertMeasurement( double &measure, QGis::UnitType &u, b
   // The parameter &u is out only...
 
   QGis::UnitType myUnits = mTool->canvas()->mapUnits();
-  if (( myUnits == QGis::Degrees || myUnits == QGis::Feet ) &&
-      mTool->canvas()->mapRenderer()->distanceArea()->ellipsoid() != "NONE" &&
-      mTool->canvas()->mapRenderer()->distanceArea()->hasCrsTransformEnabled() )
+  if (( myUnits == QGis::Degrees || myUnits == QGis::Feet )  &&
+      mcbProjectionEnabled->isChecked() )
   {
     // Measuring on an ellipsoid returns meters, and so does using projections???
     myUnits = QGis::Meters;
@@ -271,3 +296,72 @@ void QgsMeasureDialog::convertMeasurement( double &measure, QGis::UnitType &u, b
 
   u = myUnits;
 }
+
+void QgsMeasureDialog::changeProjectionEnabledState()
+{
+  // store value
+  QSettings settings;
+  if ( mcbProjectionEnabled->isChecked() )
+    settings.setValue( "/qgis/measure/projectionEnabled", 2 );
+  else
+    settings.setValue( "/qgis/measure/projectionEnabled", 0 );
+
+  // clear interface
+  mTable->clear();
+  QTreeWidgetItem* item = new QTreeWidgetItem( QStringList( QString::number( 0, 'f', 1 ) ) );
+  item->setTextAlignment( 0, Qt::AlignRight );
+  mTable->addTopLevelItem( item );
+  mTotal = 0;
+  updateUi();
+
+  int decimalPlaces = settings.value( "/qgis/measure/decimalplaces", "3" ).toInt();
+
+  if ( mMeasureArea )
+  {
+    double area = 0.0;
+    if ( mTool->points().size() > 1 )
+    {
+      area = mDa.measurePolygon( mTool->points() );
+    }
+    editTotal->setText( formatArea( area, decimalPlaces ) );
+  }
+  else
+  {
+    QList<QgsPoint>::const_iterator it;
+    bool b = true; // first point
+
+    QgsPoint p1, p2;
+
+    for ( it = mTool->points().constBegin(); it != mTool->points().constEnd(); ++it )
+    {
+      p2 = *it;
+      if ( !b )
+      {
+        double d  = mDa.measureLine( p1, p2 );
+        mTotal += d;
+        editTotal->setText( formatDistance( mTotal, decimalPlaces ) );
+        QGis::UnitType myDisplayUnits;
+
+        convertMeasurement( d, myDisplayUnits, false );
+
+        QTreeWidgetItem *item = mTable->topLevelItem( mTable->topLevelItemCount() - 1 );
+        item->setText( 0, QLocale::system().toString( d, 'f', decimalPlaces ) );
+        item = new QTreeWidgetItem( QStringList( QLocale::system().toString( 0.0, 'f', decimalPlaces ) ) );
+        item->setTextAlignment( 0, Qt::AlignRight );
+        mTable->addTopLevelItem( item );
+        mTable->scrollToItem( item );
+      }
+      p1 = p2;
+      b = false;
+    }
+  }
+}
+
+void QgsMeasureDialog::configureDistanceArea()
+{
+  QSettings settings;
+  QString ellipsoidId = settings.value( "/qgis/measure/ellipsoid", "WGS84" ).toString();
+  mDa.setSourceCrs( mTool->canvas()->mapRenderer()->destinationCrs().srsid() );
+  mDa.setEllipsoid( ellipsoidId );
+  mDa.setProjectionsEnabled( mcbProjectionEnabled->isChecked() );
+}
