@@ -62,14 +62,17 @@ class QgsMeasureDialog : public QDialog, private Ui::QgsMeasureBase
 
     //! Show the help for the dialog
     void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
+  private slots:
+    //! on change state projection enable
+    void changeProjectionEnabledState();
 
   private:
 
     //! formats distance to most appropriate units
-    QString formatDistance( double distance );
+    QString formatDistance( double distance, int decimalPlaces );
 
     //! formats area to most appropriate units
-    QString formatArea( double area );
+    QString formatArea( double area, int decimalPlaces );
 
     //! shows/hides table, shows correct units
     void updateUi();
@@ -77,11 +80,16 @@ class QgsMeasureDialog : public QDialog, private Ui::QgsMeasureBase
     //! Converts the measurement, depending on settings in options and current transformation
     void convertMeasurement( double &measure, QGis::UnitType &u, bool isArea );
 
+    //! Configures distance area objects with ellipsoid / output crs
+    void configureDistanceArea();
+
     double mTotal;
 
     //! indicates whether we're measuring distances or areas
     bool mMeasureArea;
 
+    QgsDistanceArea mDa;
+
     //! pointer to measure tool which owns this dialog
     QgsMeasureTool* mTool;
 };
