@@ -63,14 +63,13 @@ void QgsMeasureTool::activate()
 {
   mDialog->restorePosition();
   QgsMapTool::activate();
-  mRightMouseClicked = false;
 
   // ensure that we have correct settings
   updateSettings();
 
   // If we suspect that they have data that is projected, yet the
   // map CRS is set to a geographic one, warn them.
-  if ( mCanvas->mapRenderer()->distanceArea()->geographic() &&
+  if ( mCanvas->mapRenderer()->destinationCrs().geographicFlag() &&
        ( mCanvas->extent().height() > 360 ||
          mCanvas->extent().width() > 720 ) )
   {
