@@ -22,10 +22,18 @@
 #include "qgsmapcanvas.h"
 #include "qgsrubberband.h"
 #include "qgsvectorlayer.h"
+#include "qgsattributeeditor.h"
+
 #include <limits>
 #include <QComboBox>
+#include <QSettings>
 
-QgsMergeAttributesDialog::QgsMergeAttributesDialog( const QgsFeatureList& features, QgsVectorLayer* vl, QgsMapCanvas* canvas, QWidget * parent, Qt::WindowFlags f ): QDialog( parent, f ), mFeatureList( features ), mVectorLayer( vl ), mMapCanvas( canvas ), mSelectionRubberBand( 0 )
+QgsMergeAttributesDialog::QgsMergeAttributesDialog( const QgsFeatureList &features, QgsVectorLayer *vl, QgsMapCanvas *canvas, QWidget *parent, Qt::WindowFlags f )
+    : QDialog( parent, f )
+    , mFeatureList( features )
+    , mVectorLayer( vl )
+    , mMapCanvas( canvas )
+    , mSelectionRubberBand( 0 )
 {
   setupUi( this );
   createTableWidgetContents();
@@ -40,15 +48,25 @@ QgsMergeAttributesDialog::QgsMergeAttributesDialog( const QgsFeatureList& featur
 
   mFromSelectedPushButton->setIcon( QgisApp::getThemeIcon( "mActionFromSelectedFeature.png" ) );
   mRemoveFeatureFromSelectionButton->setIcon( QgisApp::getThemeIcon( "mActionRemoveSelectedFeature.png" ) );
+
+  QSettings settings;
+  restoreGeometry( settings.value( "/Windows/MergeAttributes/geometry" ).toByteArray() );
 }
 
-QgsMergeAttributesDialog::QgsMergeAttributesDialog(): QDialog()
+QgsMergeAttributesDialog::QgsMergeAttributesDialog()
+    : QDialog()
 {
   setupUi( this );
+
+  QSettings settings;
+  restoreGeometry( settings.value( "/Windows/MergeAttributes/geometry" ).toByteArray() );
 }
 
 QgsMergeAttributesDialog::~QgsMergeAttributesDialog()
 {
+  QSettings settings;
+  settings.setValue( "/Windows/MergeAttributes/geometry", saveGeometry() );
+
   delete mSelectionRubberBand;
 }
 
@@ -59,56 +77,58 @@ void QgsMergeAttributesDialog::createTableWidgetContents()
   {
     return;
   }
-  const QgsFieldMap& fieldMap = mVectorLayer->pendingFields();
 
   //combo box row, attributes titles, feature values and current merge results
   mTableWidget->setRowCount( mFeatureList.size() + 2 );
-  mTableWidget->setColumnCount( fieldMap.size() );
 
-  //create combo boxes
-  for ( int i = 0; i < fieldMap.size(); ++i )
+  //create combo boxes and insert attribute names
+  const QgsFieldMap& fieldMap = mVectorLayer->pendingFields();
+
+  int col = 0;
+  for ( QgsFieldMap::const_iterator fieldIt = fieldMap.constBegin();
+        fieldIt != fieldMap.constEnd();
+        ++fieldIt )
   {
-    mTableWidget->setCellWidget( 0, i, createMergeComboBox( fieldMap[i].type() ) );
-  }
+    if ( mVectorLayer->editType( fieldIt.key() ) == QgsVectorLayer::Hidden ||
+         mVectorLayer->editType( fieldIt.key() ) == QgsVectorLayer::Immutable )
+      continue;
 
-  QgsFieldMap::const_iterator fieldIt = fieldMap.constBegin();
+    mTableWidget->setColumnCount( col + 1 );
 
-  //insert attribute names
-  QStringList horizontalHeaderLabels;
-  for ( ; fieldIt != fieldMap.constEnd(); ++fieldIt )
-  {
-    horizontalHeaderLabels << fieldIt.value().name();
+    mTableWidget->setCellWidget( 0, col, createMergeComboBox( fieldIt->type() ) );
+
+    QTableWidgetItem *item = new QTableWidgetItem( fieldIt.value().name() );
+    item->setData( Qt::UserRole, fieldIt.key() );
+    mTableWidget->setHorizontalHeaderItem( col++, item );
   }
-  mTableWidget->setHorizontalHeaderLabels( horizontalHeaderLabels );
 
   //insert the attribute values
-  int currentRow = 1;
   QStringList verticalHeaderLabels; //the id column is in the
   verticalHeaderLabels << tr( "Id" );
 
   for ( int i = 0; i < mFeatureList.size(); ++i )
   {
     verticalHeaderLabels << QString::number( mFeatureList[i].id() );
-    QgsAttributeMap currentAttributeMap = mFeatureList[i].attributeMap();
-    QgsAttributeMap::const_iterator currentMapIt = currentAttributeMap.constBegin();
-    int col = 0;
-    for ( ; currentMapIt != currentAttributeMap.constEnd(); ++currentMapIt )
+
+    const QgsAttributeMap &attrs = mFeatureList[i].attributeMap();
+
+    for ( int j = 0; j < mTableWidget->columnCount(); j++ )
     {
-      QTableWidgetItem* attributeValItem = new QTableWidgetItem( currentMapIt.value().toString() );
+      int idx = mTableWidget->horizontalHeaderItem( j )->data( Qt::UserRole ).toInt();
+
+      QTableWidgetItem* attributeValItem = new QTableWidgetItem( attrs[idx].toString() );
       attributeValItem->setFlags( Qt::ItemIsEnabled | Qt::ItemIsSelectable );
-      mTableWidget->setItem( currentRow, col, attributeValItem );
-      ++col;
+      mTableWidget->setItem( i + 1, j, attributeValItem );
+      mTableWidget->setCellWidget( i + 1, j, QgsAttributeEditor::createAttributeEditor( mTableWidget, NULL, mVectorLayer, idx, attrs[idx] ) );
     }
-    ++currentRow;
   }
 
   //merge
   verticalHeaderLabels << tr( "Merge" );
   mTableWidget->setVerticalHeaderLabels( verticalHeaderLabels );
 
-
   //insert currently merged values
-  for ( int i = 0; i < fieldMap.size(); ++i )
+  for ( int i = 0; i < mTableWidget->columnCount(); ++i )
   {
     refreshMergedValue( i );
   }
@@ -179,7 +199,7 @@ void QgsMergeAttributesDialog::selectedRowChanged()
 
   int row = selectionList[0]->row();
 
-  if ( !mTableWidget || !mMapCanvas || !mVectorLayer || row < 1 || row >= ( mTableWidget->rowCount() ) )
+  if ( !mTableWidget || !mMapCanvas || !mVectorLayer || row < 1 || row >= mTableWidget->rowCount() )
   {
     return;
   }
@@ -254,21 +274,22 @@ void QgsMergeAttributesDialog::refreshMergedValue( int col )
 
 QString QgsMergeAttributesDialog::featureAttributeString( int featureId, int col )
 {
-  QString resultText;
-  for ( int i = 0; i < mFeatureList.size(); ++i )
+  int idx = mTableWidget->horizontalHeaderItem( col )->data( Qt::UserRole ).toInt();
+
+  int i;
+  for ( i = 0; i < mFeatureList.size() && mFeatureList[i].id() != featureId; i++ )
+    ;
+
+  QVariant value;
+  if ( i < mFeatureList.size() &&
+       QgsAttributeEditor::retrieveValue( mTableWidget->cellWidget( i + 1, col ), mVectorLayer, idx, value ) )
   {
-    int currentFid = mFeatureList[i].id();
-    if ( currentFid == featureId )
-    {
-      QTableWidgetItem* currentItem = mTableWidget->item( i + 1, col );
-      if ( !currentItem )
-      {
-        continue;
-      }
-      resultText = currentItem->text();
-    }
+    return value.toString();
+  }
+  else
+  {
+    return "";
   }
-  return resultText;
 }
 
 QString QgsMergeAttributesDialog::minimumAttributeString( int col )
@@ -474,17 +495,18 @@ void QgsMergeAttributesDialog::on_mRemoveFeatureFromSelectionButton_clicked()
   for ( int i = 0; i < mTableWidget->columnCount(); ++i )
   {
     QComboBox* currentComboBox = qobject_cast<QComboBox *>( mTableWidget->cellWidget( 0, i ) );
-    if ( currentComboBox )
-    {
-      currentComboBox->blockSignals( true );
-      currentComboBox->removeItem( currentComboBox->findText( tr( "feature %1" ).arg( featureId ) ) );
-      currentComboBox->blockSignals( false );
-    }
+    if ( !currentComboBox )
+      continue;
+
+    currentComboBox->blockSignals( true );
+    currentComboBox->removeItem( currentComboBox->findText( tr( "feature %1" ).arg( featureId ) ) );
+    currentComboBox->blockSignals( false );
   }
 
   //finally remove the feature from mFeatureList
-  QgsFeatureList::iterator f_it = mFeatureList.begin();
-  for ( ; f_it != mFeatureList.end(); ++f_it )
+  for ( QgsFeatureList::iterator f_it = mFeatureList.begin();
+        f_it != mFeatureList.end();
+        ++f_it )
   {
     if ( f_it->id() == featureId )
     {
@@ -507,29 +529,22 @@ void QgsMergeAttributesDialog::createRubberBandForFeature( int featureId )
 
 QgsAttributeMap QgsMergeAttributesDialog::mergedAttributesMap() const
 {
-  QgsAttributeMap resultMap;
   if ( mFeatureList.size() < 1 )
   {
-    return resultMap; //return empty map
+    return QgsAttributeMap();
   }
 
-  resultMap = mFeatureList[0].attributeMap();
-  int index = 0;
-  QgsAttributeMap::iterator it = resultMap.begin();
-
-  for ( ; it != resultMap.end(); ++it )
+  QgsAttributeMap resultMap;
+  for ( int i = 0; i < mTableWidget->columnCount(); i++ )
   {
-    QTableWidgetItem* currentItem = mTableWidget->item( mFeatureList.size() + 1, index );
+    int idx = mTableWidget->horizontalHeaderItem( i )->data( Qt::UserRole ).toInt();
+
+    QTableWidgetItem* currentItem = mTableWidget->item( mFeatureList.size() + 1, i );
     if ( !currentItem )
-    {
       continue;
-    }
-    QString mergedString = currentItem->text();
-    QVariant newValue( mergedString );
-    resultMap.insert( it.key(), newValue );
-    ++index;
+
+    resultMap.insert( idx, currentItem->text() );
   }
 
   return resultMap;
 }
-
