@@ -19,6 +19,8 @@
 #include "qgscontexthelp.h"
 #include <QSettings>
 #include <QMessageBox>
+#include <QUrl>
+#include <QPushButton>
 
 QgsNewHttpConnection::QgsNewHttpConnection(
   QWidget *parent, const QString& baseKey, const QString& connName, Qt::WFlags fl ):
@@ -39,16 +41,34 @@ QgsNewHttpConnection::QgsNewHttpConnection(
     QString credentialsKey = "/Qgis/WMS/" + connName;
     txtName->setText( connName );
     txtUrl->setText( settings.value( key + "/url" ).toString() );
+
+    if ( mBaseKey == "/Qgis/connections-wms/" )
+    {
+      cbxIgnoreGetMapURI->setChecked( settings.value( key + "/ignoreGetMapURI", false ).toBool() );
+      cbxIgnoreGetFeatureInfoURI->setChecked( settings.value( key + "/ignoreGetFeatureInfoURI", false ).toBool() );
+    }
+    else
+    {
+      cbxIgnoreGetMapURI->setVisible( false );
+      cbxIgnoreGetFeatureInfoURI->setVisible( false );
+    }
+
     txtUserName->setText( settings.value( credentialsKey + "/username" ).toString() );
     txtPassword->setText( settings.value( credentialsKey + "/password" ).toString() );
-
   }
+
+  on_txtName_textChanged( connName );
 }
 
 QgsNewHttpConnection::~QgsNewHttpConnection()
 {
 }
 
+void QgsNewHttpConnection::on_txtName_textChanged( const QString &text )
+{
+  buttonBox->button( QDialogButtonBox::Ok )->setDisabled( text.isEmpty() );
+}
+
 void QgsNewHttpConnection::accept()
 {
   QSettings settings;
@@ -73,9 +93,31 @@ void QgsNewHttpConnection::accept()
     settings.remove( "/Qgis/WMS/" + mOriginalConnName );
   }
 
-  settings.setValue( key + "/url", txtUrl->text().trimmed() );
+  QUrl url( txtUrl->text().trimmed() );
+
+  QList< QPair<QByteArray, QByteArray> > params = url.encodedQueryItems();
+  for ( int i = 0; i < params.size(); i++ )
+  {
+    if ( params[i].first.toUpper() == "SERVICE" ||
+         params[i].first.toUpper() == "REQUEST" ||
+         params[i].first.toUpper() == "FORMAT" )
+    {
+      params.removeAt( i-- );
+    }
+  }
+  url.setEncodedQueryItems( params );
+
+  settings.setValue( key + "/url", url.toString() );
+  if ( mBaseKey == "/Qgis/connections-wms/" )
+  {
+    settings.setValue( key + "/ignoreGetMapURI", cbxIgnoreGetMapURI->isChecked() );
+    settings.setValue( key + "/ignoreGetFeatureInfoURI", cbxIgnoreGetFeatureInfoURI->isChecked() );
+  }
+
   settings.setValue( credentialsKey + "/username", txtUserName->text() );
   settings.setValue( credentialsKey + "/password", txtPassword->text() );
 
+  settings.setValue( mBaseKey + "/selected", txtName->text() );
+
   QDialog::accept();
 }
