@@ -23,13 +23,20 @@
 #include "qgsgenericprojectionselector.h"
 #include "qgscoordinatereferencesystem.h"
 #include "qgstolerance.h"
+#include "qgsnetworkaccessmanager.h"
 
 #include <QFileDialog>
 #include <QSettings>
 #include <QColorDialog>
 #include <QLocale>
+#include <QToolBar>
+#include <QSize>
 
-#include <cassert>
+#if QT_VERSION >= 0x40500
+#include <QNetworkDiskCache>
+#endif
+
+#include <limits>
 #include <sqlite3.h>
 #include "qgslogger.h"
 #define ELLIPS_FLAT "NONE"
@@ -46,8 +53,16 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   connect( cmbTheme, SIGNAL( activated( const QString& ) ), this, SLOT( themeChanged( const QString& ) ) );
   connect( cmbTheme, SIGNAL( highlighted( const QString& ) ), this, SLOT( themeChanged( const QString& ) ) );
   connect( cmbTheme, SIGNAL( textChanged( const QString& ) ), this, SLOT( themeChanged( const QString& ) ) );
+
+  connect( cmbSize, SIGNAL( activated( const QString& ) ), this, SLOT( iconSizeChanged( const QString& ) ) );
+  connect( cmbSize, SIGNAL( highlighted( const QString& ) ), this, SLOT( iconSizeChanged( const QString& ) ) );
+  connect( cmbSize, SIGNAL( textChanged( const QString& ) ), this, SLOT( iconSizeChanged( const QString& ) ) );
   connect( this, SIGNAL( accepted() ), this, SLOT( saveOptions() ) );
 
+  cmbSize->addItem( "16" );
+  cmbSize->addItem( "24" );
+  cmbSize->addItem( "32" );
+
   cmbIdentifyMode->addItem( tr( "Current layer" ), 0 );
   cmbIdentifyMode->addItem( tr( "Top down, stop at first" ), 1 );
   cmbIdentifyMode->addItem( tr( "Top down" ), 2 );
@@ -64,9 +79,23 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   spinBoxIdentifyValue->setMinimum( 0.01 );
   spinBoxIdentifyValue->setValue( identifyValue );
 
+  //local directories to search when loading c++ plugins
+  QString myPaths = settings.value( "plugins/searchPathsForPlugins", "" ).toString();
+  if ( !myPaths.isEmpty() )
+  {
+    QStringList myPathList = myPaths.split( "|" );
+    QStringList::const_iterator pathIt = myPathList.constBegin();
+    for ( ; pathIt != myPathList.constEnd(); ++pathIt )
+    {
+      QListWidgetItem* newItem = new QListWidgetItem( mListPluginPaths );
+      newItem->setText( *pathIt );
+      newItem->setFlags( Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable );
+      mListPluginPaths->addItem( newItem );
+    }
+  }
 
   //local directories to search when looking for an SVG with a given basename
-  QString myPaths = settings.value( "svg/searchPathsForSVG", "" ).toString();
+  myPaths = settings.value( "svg/searchPathsForSVG", "" ).toString();
   if ( !myPaths.isEmpty() )
   {
     QStringList myPathList = myPaths.split( "|" );
@@ -114,17 +143,44 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
     }
   }
 
+#if QT_VERSION >= 0x40500
+  // cache settings
+  QNetworkDiskCache *cache = qobject_cast<QNetworkDiskCache*>( QgsNetworkAccessManager::instance()->cache() );
+  if ( cache )
+  {
+    mCacheDirectory->setText( cache->cacheDirectory() );
+    mCacheSize->setMinimum( 0 );
+    mCacheSize->setMaximum( std::numeric_limits<int>::max() );
+    mCacheSize->setSingleStep( 1024 );
+    QgsDebugMsg( QString( "set cacheSize: %1" ).arg( cache->maximumCacheSize() ) );
+    mCacheSize->setValue( cache->maximumCacheSize() / 1024 );
+  }
+#else
+  grpUrlExclude->setHidden( true );
+  grpCache->setHidden( true );
+#endif
+
+  //wms search server
+  leWmsSearch->setText( settings.value( "/qgis/WMSSearchUrl", "http://geopole.org/wms/search?search=%1&type=rss" ).toString() );
+
   // set the current theme
   cmbTheme->setItemText( cmbTheme->currentIndex(), settings.value( "/Themes" ).toString() );
 
+  // set the attribute table behaviour
+  cmbAttrTableBehaviour->clear();
+  cmbAttrTableBehaviour->addItem( tr( "Show all features" ) );
+  cmbAttrTableBehaviour->addItem( tr( "Show selected features" ) );
+  cmbAttrTableBehaviour->addItem( tr( "Show features in current canvas" ) );
+  cmbAttrTableBehaviour->setCurrentIndex( settings.value( "/qgis/attributeTableBehaviour", 0 ).toInt() );
+
   // set the display update threshold
   spinBoxUpdateThreshold->setValue( settings.value( "/Map/updateThreshold" ).toInt() );
   //set the default projection behaviour radio buttongs
-  if ( settings.value( "/Projections/defaultBehaviour" ).toString() == "prompt" )
+  if ( settings.value( "/Projections/defaultBehaviour", "prompt" ).toString() == "prompt" )
   {
     radPromptForProjection->setChecked( true );
   }
-  else if ( settings.value( "/Projections/defaultBehaviour" ).toString() == "useProject" )
+  else if ( settings.value( "/Projections/defaultBehaviour", "prompt" ).toString() == "useProject" )
   {
     radUseProjectProjection->setChecked( true );
   }
@@ -132,8 +188,18 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   {
     radUseGlobalProjection->setChecked( true );
   }
+  QString myLayerDefaultCrs = settings.value( "/Projections/layerDefaultCrs", GEO_EPSG_CRS_AUTHID ).toString();
+  mLayerDefaultCrs.createFromOgcWmsCrs( myLayerDefaultCrs );
+  //display the crs as friendly text rather than in wkt
+  leLayerGlobalCrs->setText( mLayerDefaultCrs.authid() + " - " + mLayerDefaultCrs.description() );
 
-  txtGlobalWkt->setText( settings.value( "/Projections/defaultProjectionString", GEOPROJ4 ).toString() );
+  //on the fly CRS transformation settings
+  chkOtfTransform->setChecked( settings.value( "/Projections/otfTransformEnabled", 0 ).toBool() );
+
+  QString myDefaultCrs = settings.value( "/Projections/projectDefaultCrs", GEO_EPSG_CRS_AUTHID ).toString();
+  mDefaultCrs.createFromOgcWmsCrs( myDefaultCrs );
+  //display the crs as friendly text rather than in wkt
+  leProjectGlobalCrs->setText( mDefaultCrs.authid() + " - " + mDefaultCrs.description() );
 
   // populate combo box with ellipsoids
   getEllipsoidList();
@@ -151,6 +217,41 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
     radMeters->setChecked( true );
   }
 
+  QButtonGroup* angleButtonGroup = new QButtonGroup( this );
+  angleButtonGroup->addButton( mDegreesRadioButton );
+  angleButtonGroup->addButton( mRadiansRadioButton );
+  angleButtonGroup->addButton( mGonRadioButton );
+
+  QString myAngleUnitsTxt = settings.value( "/qgis/measure/angleunits", "degrees" ).toString();
+  if ( myAngleUnitsTxt == "gon" )
+  {
+    mGonRadioButton->setChecked( true );
+  }
+  else if ( myAngleUnitsTxt == "radians" )
+  {
+    mRadiansRadioButton->setChecked( true );
+  }
+  else //degrees
+  {
+    mDegreesRadioButton->setChecked( true );
+  }
+
+  // set decimal places of the measure tool
+  int decimalPlaces = settings.value( "/qgis/measure/decimalplaces", "3" ).toInt();
+  mDecimalPlacesSpinBox->setRange( 0, 12 );
+  mDecimalPlacesSpinBox->setValue( decimalPlaces );
+
+  // set if base unit of measure tool should be changed
+  bool baseUnit = settings.value( "qgis/measure/keepbaseunit", false ).toBool();
+  if ( baseUnit == true )
+  {
+    mKeepBaseUnitCheckBox->setChecked( true );
+  }
+  else
+  {
+    mKeepBaseUnitCheckBox->setChecked( false );
+  }
+
 
   // add the themes to the combo box on the option dialog
   QDir myThemeDir( ":/images/themes/" );
@@ -167,12 +268,14 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
 
   // set the theme combo
   cmbTheme->setCurrentIndex( cmbTheme->findText( settings.value( "/Themes", "default" ).toString() ) );
-
+  cmbSize->setCurrentIndex( cmbSize->findText( settings.value( "/IconSize", 24 ).toString() ) );
   //set the state of the checkboxes
-  chkAntiAliasing->setChecked( settings.value( "/qgis/enable_anti_aliasing", false ).toBool() );
+  //Changed to default to true as of QGIS 1.7
+  chkAntiAliasing->setChecked( settings.value( "/qgis/enable_anti_aliasing", true ).toBool() );
   chkUseRenderCaching->setChecked( settings.value( "/qgis/enable_render_caching", false ).toBool() );
 
-  chkUseSymbologyNG->setChecked( settings.value( "/qgis/use_symbology_ng", false ).toBool() );
+  //Changed to default to true as of QGIS 1.7
+  chkUseSymbologyNG->setChecked( settings.value( "/qgis/use_symbology_ng", true ).toBool() );
 
   // Slightly awkard here at the settings value is true to use QImage,
   // but the checkbox is true to use QPixmap
@@ -180,15 +283,23 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   chkAddedVisibility->setChecked( settings.value( "/qgis/new_layers_visible", true ).toBool() );
   cbxLegendClassifiers->setChecked( settings.value( "/qgis/showLegendClassifiers", false ).toBool() );
   cbxHideSplash->setChecked( settings.value( "/qgis/hideSplash", false ).toBool() );
+  cbxShowTips->setChecked( settings.value( "/qgis/showTips", true ).toBool() );
   cbxAttributeTableDocked->setChecked( settings.value( "/qgis/dockAttributeTable", false ).toBool() );
   cbxIdentifyResultsDocked->setChecked( settings.value( "/qgis/dockIdentifyResults", false ).toBool() );
+  cbxSnappingOptionsDocked->setChecked( settings.value( "/qgis/dockSnapping", false ).toBool() );
   cbxAddPostgisDC->setChecked( settings.value( "/qgis/addPostgisDC", false ).toBool() );
+  cbxAddNewLayersToCurrentGroup->setChecked( settings.value( "/qgis/addNewLayersToCurrentGroup", false ).toBool() );
+  cbxCreateRasterLegendIcons->setChecked( settings.value( "/qgis/createRasterLegendIcons", true ).toBool() );
+  leNullValue->setText( settings.value( "qgis/nullValue", "NULL" ).toString() );
+
+  cmbLegendDoubleClickAction->setCurrentIndex( settings.value( "/qgis/legendDoubleClickAction", 0 ).toInt() );
 
   //set the color for selections
   int myRed = settings.value( "/qgis/default_selection_color_red", 255 ).toInt();
   int myGreen = settings.value( "/qgis/default_selection_color_green", 255 ).toInt();
   int myBlue = settings.value( "/qgis/default_selection_color_blue", 0 ).toInt();
-  pbnSelectionColor->setColor( QColor( myRed, myGreen, myBlue ) );
+  int myAlpha = settings.value( "/qgis/default_selection_color_alpha", 255 ).toInt();
+  pbnSelectionColor->setColor( QColor( myRed, myGreen, myBlue, myAlpha ) );
 
   //set the default color for canvas background
   myRed = settings.value( "/qgis/default_canvas_color_red", 255 ).toInt();
@@ -207,7 +318,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   chbAskToSaveProjectChanges->setChecked( settings.value( "qgis/askToSaveProjectChanges", QVariant( true ) ).toBool() );
   chbWarnOldProjectVersion->setChecked( settings.value( "/qgis/warnOldProjectVersion", QVariant( true ) ).toBool() );
 
-  cmbWheelAction->setCurrentIndex( settings.value( "/qgis/wheel_action", 0 ).toInt() );
+  cmbWheelAction->setCurrentIndex( settings.value( "/qgis/wheel_action", 2 ).toInt() );
   spinZoomFactor->setValue( settings.value( "/qgis/zoom_factor", 2 ).toDouble() );
 
   //
@@ -283,9 +394,10 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   }
   mMarkerSizeSpinBox->setValue( settings.value( "/qgis/digitizing/marker_size", 3 ).toInt() );
 
+  chkReuseLastValues->setChecked( settings.value( "/qgis/digitizing/reuseLastValues", false ).toBool() );
   chkDisableAttributeValuesDlg->setChecked( settings.value( "/qgis/digitizing/disable_enter_attribute_values_dialog", false ).toBool() );
 
-#ifdef Q_WS_MAC //MH: disable incremental update on Mac for now to avoid problems with resizing 
+#ifdef Q_WS_MAC //MH: disable incremental update on Mac for now to avoid problems with resizing
   groupBox_5->setEnabled( false );
 #endif //Q_WS_MAC
 
@@ -319,7 +431,7 @@ QgsOptions::QgsOptions( QWidget *parent, Qt::WFlags fl ) :
   } //default is central point
 
   restoreGeometry( settings.value( "/Windows/Options/geometry" ).toByteArray() );
-  listWidget->setCurrentRow( settings.value( "/Windows/Options/row" ).toInt() );
+  tabWidget->setCurrentIndex( settings.value( "/Windows/Options/row" ).toInt() );
 }
 
 //! Destructor
@@ -327,12 +439,17 @@ QgsOptions::~QgsOptions()
 {
   QSettings settings;
   settings.setValue( "/Windows/Options/geometry", saveGeometry() );
-  settings.setValue( "/Windows/Options/row", listWidget->currentRow() );
+  settings.setValue( "/Windows/Options/row", tabWidget->currentIndex() );
 }
 
 void QgsOptions::on_pbnSelectionColor_clicked()
 {
-  QColor color = QColorDialog::getColor( pbnSelectionColor->color(), this );
+#if QT_VERSION >= 0x040500
+  QColor color = QColorDialog::getColor( pbnSelectionColor->color(), 0, tr( "Selection color" ), QColorDialog::ShowAlphaChannel );
+#else
+  QColor color = QColorDialog::getColor( pbnSelectionColor->color() );
+#endif
+
   if ( color.isValid() )
   {
     pbnSelectionColor->setColor( color );
@@ -373,6 +490,13 @@ void QgsOptions::themeChanged( const QString &newThemeName )
   QgisApp::instance()->setTheme( newt );
 }
 
+void QgsOptions::iconSizeChanged( const QString &iconSize )
+{
+  int icon = iconSize.toInt();
+  QgisApp::instance()->setIconSizes( icon );
+
+}
+
 QString QgsOptions::theme()
 {
   // returns the current theme (as selected in the cmbTheme combo box)
@@ -383,8 +507,20 @@ void QgsOptions::saveOptions()
 {
   QSettings settings;
 
-  //search directories for svgs
+  //search directories for user plugins
   QString myPaths;
+  for ( int i = 0; i < mListPluginPaths->count(); ++i )
+  {
+    if ( i != 0 )
+    {
+      myPaths += "|";
+    }
+    myPaths += mListPluginPaths->item( i )->text();
+  }
+  settings.setValue( "plugins/searchPathsForPlugins", myPaths );
+
+  //search directories for svgs
+  myPaths.clear();
   for ( int i = 0; i < mListSVGPaths->count(); ++i )
   {
     if ( i != 0 )
@@ -406,6 +542,8 @@ void QgsOptions::saveOptions()
   settings.setValue( "proxy/proxyPassword", leProxyPassword->text() );
   settings.setValue( "proxy/proxyType", mProxyTypeComboBox->currentText() );
 
+  settings.setValue( "cache/directory", mCacheDirectory->text() );
+  settings.setValue( "cache/size", QVariant::fromValue( mCacheSize->value()*1024L ) );
 
   //url to exclude from proxys
   QString proxyExcludeString;
@@ -419,45 +557,57 @@ void QgsOptions::saveOptions()
   }
   settings.setValue( "proxy/proxyExcludedUrls", proxyExcludeString );
 
+  QgisApp::instance()->namUpdate();
+
+  //wms search url
+  settings.setValue( "/qgis/WMSSearchUrl", leWmsSearch->text() );
+
   //general settings
   settings.setValue( "/Map/identifyMode", cmbIdentifyMode->itemData( cmbIdentifyMode->currentIndex() ).toInt() );
   settings.setValue( "/Map/identifyAutoFeatureForm", cbxAutoFeatureForm->isChecked() );
   settings.setValue( "/Map/identifyRadius", spinBoxIdentifyValue->value() );
   settings.setValue( "/qgis/showLegendClassifiers", cbxLegendClassifiers->isChecked() );
   settings.setValue( "/qgis/hideSplash", cbxHideSplash->isChecked() );
+  settings.setValue( "/qgis/showTips", cbxShowTips->isChecked() );
   settings.setValue( "/qgis/dockAttributeTable", cbxAttributeTableDocked->isChecked() );
+  settings.setValue( "/qgis/attributeTableBehaviour", cmbAttrTableBehaviour->currentIndex() );
   settings.setValue( "/qgis/dockIdentifyResults", cbxIdentifyResultsDocked->isChecked() );
+  settings.setValue( "/qgis/dockSnapping", cbxSnappingOptionsDocked->isChecked() );
   settings.setValue( "/qgis/addPostgisDC", cbxAddPostgisDC->isChecked() );
+  settings.setValue( "/qgis/addNewLayersToCurrentGroup", cbxAddNewLayersToCurrentGroup->isChecked() );
+  settings.setValue( "/qgis/createRasterLegendIcons", cbxCreateRasterLegendIcons->isChecked() );
   settings.setValue( "/qgis/new_layers_visible", chkAddedVisibility->isChecked() );
   settings.setValue( "/qgis/enable_anti_aliasing", chkAntiAliasing->isChecked() );
   settings.setValue( "/qgis/enable_render_caching", chkUseRenderCaching->isChecked() );
   settings.setValue( "/qgis/use_qimage_to_render", !( chkUseQPixmap->isChecked() ) );
   settings.setValue( "/qgis/use_symbology_ng", chkUseSymbologyNG->isChecked() );
-  settings.setValue( "qgis/capitaliseLayerName", capitaliseCheckBox->isChecked() );
-  settings.setValue( "qgis/askToSaveProjectChanges", chbAskToSaveProjectChanges->isChecked() );
-  settings.setValue( "qgis/warnOldProjectVersion", chbWarnOldProjectVersion->isChecked() );
+  settings.setValue( "/qgis/legendDoubleClickAction", cmbLegendDoubleClickAction->currentIndex() );
+  settings.setValue( "/qgis/capitaliseLayerName", capitaliseCheckBox->isChecked() );
+  settings.setValue( "/qgis/askToSaveProjectChanges", chbAskToSaveProjectChanges->isChecked() );
+  settings.setValue( "/qgis/warnOldProjectVersion", chbWarnOldProjectVersion->isChecked() );
+  settings.setValue( "/qgis/nullValue", leNullValue->text() );
 
   //overlay placement method
   int overlayIndex = mOverlayAlgorithmComboBox->currentIndex();
   if ( overlayIndex == 1 )
   {
-    settings.setValue( "qgis/overlayPlacementAlgorithm", "Chain" );
+    settings.setValue( "/qgis/overlayPlacementAlgorithm", "Chain" );
   }
   else if ( overlayIndex == 2 )
   {
-    settings.setValue( "qgis/overlayPlacementAlgorithm", "Popmusic tabu chain" );
+    settings.setValue( "/qgis/overlayPlacementAlgorithm", "Popmusic tabu chain" );
   }
   else if ( overlayIndex == 3 )
   {
-    settings.setValue( "qgis/overlayPlacementAlgorithm",  "Popmusic tabu" );
+    settings.setValue( "/qgis/overlayPlacementAlgorithm",  "Popmusic tabu" );
   }
   else if ( overlayIndex == 4 )
   {
-    settings.setValue( "qgis/overlayPlacementAlgorithm", "Popmusic chain" );
+    settings.setValue( "/qgis/overlayPlacementAlgorithm", "Popmusic chain" );
   }
   else
   {
-    settings.setValue( "qgis/overlayPlacementAlgorithm", "Central point" );
+    settings.setValue( "/qgis/overlayPlacementAlgorithm", "Central point" );
   }
 
   if ( cmbTheme->currentText().length() == 0 )
@@ -468,6 +618,9 @@ void QgsOptions::saveOptions()
   {
     settings.setValue( "/Themes", cmbTheme->currentText() );
   }
+
+  settings.setValue( "/IconSize", cmbSize->currentText() );
+
   settings.setValue( "/Map/updateThreshold", spinBoxUpdateThreshold->value() );
   //check behaviour so default projection when new layer is added with no
   //projection defined...
@@ -487,7 +640,11 @@ void QgsOptions::saveOptions()
     settings.setValue( "/Projections/defaultBehaviour", "useGlobal" );
   }
 
-  settings.setValue( "/Projections/defaultProjectionString", txtGlobalWkt->toPlainText() );
+  settings.setValue( "/Projections/layerDefaultCrs", mLayerDefaultCrs.authid() );
+
+  // save 'on the fly' CRS transformation settings
+  settings.setValue( "/Projections/otfTransformEnabled", chkOtfTransform->isChecked() );
+  settings.setValue( "/Projections/projectDefaultCrs", mDefaultCrs.authid() );
 
   settings.setValue( "/qgis/measure/ellipsoid", getEllipsoidAcronym( cmbEllipsoid->currentText() ) );
 
@@ -500,11 +657,37 @@ void QgsOptions::saveOptions()
     settings.setValue( "/qgis/measure/displayunits", "meters" );
   }
   settings.setValue( "/qgis/measure/ellipsoid", getEllipsoidAcronym( cmbEllipsoid->currentText() ) );
+
+  if ( mDegreesRadioButton->isChecked() )
+  {
+
+  }
+
+  QString angleUnitString = "degrees";
+  if ( mRadiansRadioButton->isChecked() )
+  {
+    angleUnitString = "radians";
+  }
+  else if ( mGonRadioButton->isChecked() )
+  {
+    angleUnitString = "gon";
+  }
+  settings.setValue( "/qgis/measure/angleunits", angleUnitString );
+
+
+  int decimalPlaces = mDecimalPlacesSpinBox->value();
+  settings.setValue( "/qgis/measure/decimalplaces", decimalPlaces );
+
+  bool baseUnit = mKeepBaseUnitCheckBox->isChecked();
+  settings.setValue( "/qgis/measure/keepbaseunit", baseUnit );
+
+
   //set the color for selections
   QColor myColor = pbnSelectionColor->color();
   settings.setValue( "/qgis/default_selection_color_red", myColor.red() );
   settings.setValue( "/qgis/default_selection_color_green", myColor.green() );
   settings.setValue( "/qgis/default_selection_color_blue", myColor.blue() );
+  settings.setValue( "/qgis/default_selection_color_alpha", myColor.alpha() );
 
   //set the default color for canvas background
   myColor = pbnCanvasColor->color();
@@ -555,6 +738,7 @@ void QgsOptions::saveOptions()
   }
   settings.setValue( "/qgis/digitizing/marker_size", ( mMarkerSizeSpinBox->value() ) );
 
+  settings.setValue( "/qgis/digitizing/reuseLastValues", chkReuseLastValues->isChecked() );
   settings.setValue( "/qgis/digitizing/disable_enter_attribute_values_dialog", chkDisableAttributeValuesDlg->isChecked() );
 
   //
@@ -570,49 +754,44 @@ void QgsOptions::on_pbnSelectProjection_clicked()
   QSettings settings;
   QgsGenericProjectionSelector * mySelector = new QgsGenericProjectionSelector( this );
 
-  //find out srs id of current proj4 string
-  QgsCoordinateReferenceSystem refSys;
-  if ( refSys.createFromProj4( txtGlobalWkt->toPlainText() ) )
-  {
-    mySelector->setSelectedCrsId( refSys.srsid() );
-  }
+  //find out crs id of current proj4 string
+  mySelector->setSelectedCrsId( mLayerDefaultCrs.srsid() );
 
   if ( mySelector->exec() )
   {
-    //! @todo changes this control name in gui to txtGlobalProjString
-    txtGlobalWkt->setText( mySelector->selectedProj4String() );
-    QgsDebugMsg( QString( "------ Global Default Projection Selection set to ----------\n%1" ).arg( txtGlobalWkt->toPlainText() ) );
+    mLayerDefaultCrs.createFromOgcWmsCrs( mySelector->selectedAuthId() );
+    QgsDebugMsg( QString( "Setting default project CRS to : %1" ).arg( mySelector->selectedAuthId() ) );
+    leLayerGlobalCrs->setText( mLayerDefaultCrs.authid() + " - " + mLayerDefaultCrs.description() );
+    QgsDebugMsg( QString( "------ Global Layer Default Projection Selection set to ----------\n%1" ).arg( leLayerGlobalCrs->text() ) );
   }
   else
   {
-    QgsDebugMsg( "------ Global Default Projection Selection change cancelled ----------" );
+    QgsDebugMsg( "------ Global Layer Default Projection Selection change cancelled ----------" );
     QApplication::restoreOverrideCursor();
   }
 
 }
 
-void QgsOptions::on_chkAntiAliasing_stateChanged()
+void QgsOptions::on_pbnSelectOtfProjection_clicked()
 {
-  // We can't have the anti-aliasing turned on when QPixmap is being
-  // used (we we can. but it then doesn't do anti-aliasing, and this
-  // will confuse people).
-  if ( chkAntiAliasing->isChecked() )
+  QSettings settings;
+  QgsGenericProjectionSelector * mySelector = new QgsGenericProjectionSelector( this );
+
+  //find out crs id of current proj4 string
+  mySelector->setSelectedCrsId( mDefaultCrs.srsid() );
+
+  if ( mySelector->exec() )
   {
-    chkUseQPixmap->setChecked( false );
+    mDefaultCrs.createFromOgcWmsCrs( mySelector->selectedAuthId() );
+    QgsDebugMsg( QString( "Setting default project CRS to : %1" ).arg( mySelector->selectedAuthId() ) );
+    leProjectGlobalCrs->setText( mDefaultCrs.authid() + " - " + mDefaultCrs.description() );
+    QgsDebugMsg( QString( "------ Global OTF Projection Selection set to ----------\n%1" ).arg( leProjectGlobalCrs->text() ) );
   }
-
-}
-
-void QgsOptions::on_chkUseQPixmap_stateChanged()
-{
-  // We can't have the anti-aliasing turned on when QPixmap is being
-  // used (we we can. but it then doesn't do anti-aliasing, and this
-  // will confuse people).
-  if ( chkUseQPixmap->isChecked() )
+  else
   {
-    chkAntiAliasing->setChecked( false );
+    QgsDebugMsg( "------ Global OTF Projection Selection change cancelled ----------" );
+    QApplication::restoreOverrideCursor();
   }
-
 }
 
 // Return state of the visibility flag for newly added layers. If
@@ -643,7 +822,7 @@ void QgsOptions::getEllipsoidList()
     QgsDebugMsg( QString( "Can't open database: %1" ).arg( sqlite3_errmsg( myDatabase ) ) );
     // XXX This will likely never happen since on open, sqlite creates the
     //     database if it does not exist.
-    assert( myResult == 0 );
+    Q_ASSERT( myResult == 0 );
   }
 
   // Set up the query to retrieve the projection information needed to populate the ELLIPSOID list
@@ -676,7 +855,7 @@ QString QgsOptions::getEllipsoidAcronym( QString theEllipsoidName )
     QgsDebugMsg( QString( "Can't open database: %1" ).arg( sqlite3_errmsg( myDatabase ) ) );
     // XXX This will likely never happen since on open, sqlite creates the
     //     database if it does not exist.
-    assert( myResult == 0 );
+    Q_ASSERT( myResult == 0 );
   }
   // Set up the query to retrieve the projection information needed to populate the ELLIPSOID list
   QString mySql = "select acronym from tbl_ellipsoid where name='" + theEllipsoidName + "'";
@@ -708,7 +887,7 @@ QString QgsOptions::getEllipsoidName( QString theEllipsoidAcronym )
     QgsDebugMsg( QString( "Can't open database: %1" ).arg( sqlite3_errmsg( myDatabase ) ) );
     // XXX This will likely never happen since on open, sqlite creates the
     //     database if it does not exist.
-    assert( myResult == 0 );
+    Q_ASSERT( myResult == 0 );
   }
   // Set up the query to retrieve the projection information needed to populate the ELLIPSOID list
   QString mySql = "select name from tbl_ellipsoid where acronym='" + theEllipsoidAcronym + "'";
@@ -742,6 +921,33 @@ QStringList QgsOptions::i18nList()
   return myList;
 }
 
+void QgsOptions::on_mBtnAddPluginPath_clicked()
+{
+  QString myDir = QFileDialog::getExistingDirectory(
+                    this,
+                    tr( "Choose a directory" ),
+                    QDir::toNativeSeparators( QDir::homePath() ),
+                    QFileDialog::ShowDirsOnly
+                  );
+
+  if ( ! myDir.isEmpty() )
+  {
+    QListWidgetItem* newItem = new QListWidgetItem( mListPluginPaths );
+    newItem->setText( myDir );
+    newItem->setFlags( Qt::ItemIsEditable | Qt::ItemIsEnabled | Qt::ItemIsSelectable );
+    mListPluginPaths->addItem( newItem );
+    mListPluginPaths->setCurrentItem( newItem );
+  }
+}
+
+void QgsOptions::on_mBtnRemovePluginPath_clicked()
+{
+  int currentRow = mListPluginPaths->currentRow();
+  QListWidgetItem* itemToRemove = mListPluginPaths->takeItem( currentRow );
+  delete itemToRemove;
+}
+
+
 void QgsOptions::on_mBtnAddSVGPath_clicked()
 {
   QString myDir = QFileDialog::getExistingDirectory(
@@ -783,3 +989,26 @@ void QgsOptions::on_mRemoveUrlPushButton_clicked()
   QListWidgetItem* itemToRemove = mExcludeUrlListWidget->takeItem( currentRow );
   delete itemToRemove;
 }
+
+void QgsOptions::on_mBrowseCacheDirectory_clicked()
+{
+  QString myDir = QFileDialog::getExistingDirectory(
+                    this,
+                    tr( "Choose a directory" ),
+                    QDir::toNativeSeparators( mCacheDirectory->text() ),
+                    QFileDialog::ShowDirsOnly
+                  );
+
+  if ( !myDir.isEmpty() )
+  {
+    mCacheDirectory->setText( QDir::toNativeSeparators( myDir ) );
+  }
+
+}
+
+void QgsOptions::on_mClearCache_clicked()
+{
+#if QT_VERSION >= 0x40500
+  QgsNetworkAccessManager::instance()->cache()->clear();
+#endif
+}
