@@ -23,6 +23,8 @@
 #include "qgisgui.h"
 #include "qgscontexthelp.h"
 
+#include <qgscoordinatereferencesystem.h>
+
 
 /**
  * \class QgsOptions
@@ -50,13 +52,14 @@ class QgsOptions : public QDialog, private Ui::QgsOptionsBase
   public slots:
     //! Slot called when user chooses to change the project wide projection.
     void on_pbnSelectProjection_clicked();
-    void on_chkAntiAliasing_stateChanged();
-    void on_chkUseQPixmap_stateChanged();
+    //! Slot called when user chooses to change the default 'on the fly' projection.
+    void on_pbnSelectOtfProjection_clicked();
     void saveOptions();
     //! Slot to change the theme this is handled when the user
     // activates or highlights a theme name in the drop-down list
     void themeChanged( const QString & );
 
+    void iconSizeChanged( const QString &iconSize );
     /**
      * Return the desired state of newly added layers. If a layer
      * is to be drawn when added to the map, this function returns
@@ -89,6 +92,18 @@ class QgsOptions : public QDialog, private Ui::QgsOptionsBase
     /**Remove an URL to exclude from Proxy*/
     void on_mRemoveUrlPushButton_clicked();
 
+    /* Let the user add a path to the list of search paths
+     * used for finding user Plugin libs.
+     * @note added in QGIS 1.7
+     */
+    void on_mBtnAddPluginPath_clicked();
+
+    /* Let the user remove a path to the list of search paths
+     * used for finding Plugin libs.
+     * @note added in QGIS 1.7
+     */
+    void on_mBtnRemovePluginPath_clicked();
+
     /* Let the user add a path to the list of search paths
      * used for finding SVG files.
      * @note added in QGIS 1.4
@@ -103,6 +118,9 @@ class QgsOptions : public QDialog, private Ui::QgsOptionsBase
 
     void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
 
+    void on_mBrowseCacheDirectory_clicked();
+    void on_mClearCache_clicked();
+
   protected:
     //! Populates combo box with ellipsoids
     void getEllipsoidList();
@@ -111,12 +129,9 @@ class QgsOptions : public QDialog, private Ui::QgsOptionsBase
     QString getEllipsoidName( QString theEllipsoidAcronym );
 
   private:
-    //
     QStringList i18nList();
-
-    //!Default proj4 string used for new layers added that have no projection
-    QString mGlobalProj4String;
-
+    QgsCoordinateReferenceSystem mDefaultCrs;
+    QgsCoordinateReferenceSystem mLayerDefaultCrs;
 };
 
 #endif // #ifndef QGSOPTIONS_H
