@@ -20,7 +20,6 @@
 #include "qgsprojectproperties.h"
 
 //qgis includes
-#include "qgsavoidintersectionsdialog.h"
 #include "qgscontexthelp.h"
 #include "qgscoordinatetransform.h"
 #include "qgslogger.h"
@@ -32,11 +31,12 @@
 #include "qgsrenderer.h"
 #include "qgssnappingdialog.h"
 #include "qgsrasterlayer.h"
+#include "qgsgenericprojectionselector.h"
 
 //qt includes
 #include <QColorDialog>
 #include <QHeaderView>  // Qt 4.4
-#include "qgslogger.h"
+#include <QMessageBox>
 
 //stdc++ includes
 
@@ -49,21 +49,22 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
   connect( buttonBox, SIGNAL( rejected() ), this, SLOT( reject() ) );
   connect( buttonBox->button( QDialogButtonBox::Apply ), SIGNAL( clicked() ), this, SLOT( apply() ) );
   connect( this, SIGNAL( accepted() ), this, SLOT( apply() ) );
+  connect( projectionSelector, SIGNAL( sridSelected( QString ) ), this, SLOT( setMapUnitsToCurrentProjection() ) );
 
   ///////////////////////////////////////////////////////////
   // Properties stored in map canvas's QgsMapRenderer
   // these ones are propagated to QgsProject by a signal
 
-  QgsMapRenderer* myRender = mMapCanvas->mapRenderer();
-  QGis::UnitType myUnit = myRender->mapUnits();
+  QgsMapRenderer* myRenderer = mMapCanvas->mapRenderer();
+  QGis::UnitType myUnit = myRenderer->mapUnits();
   setMapUnits( myUnit );
 
   //see if the user wants on the fly projection enabled
-  bool myProjectionEnabled = myRender->hasCrsTransformEnabled();
+  bool myProjectionEnabled = myRenderer->hasCrsTransformEnabled();
   cbxProjectionEnabled->setChecked( myProjectionEnabled );
   btnGrpMapUnits->setEnabled( !myProjectionEnabled );
 
-  long myCRSID = myRender->destinationSrs().srsid();
+  long myCRSID = myRenderer->destinationCrs().srsid();
   QgsDebugMsg( "Read project CRSID: " + QString::number( myCRSID ) );
   projectionSelector->setSelectedCrsId( myCRSID );
 
@@ -78,6 +79,8 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
   if ( automaticPrecision )
   {
     radAutomatic->setChecked( true );
+    spinBoxDP->setDisabled( true );
+    labelDP->setDisabled( true );
   }
   else
   {
@@ -93,7 +96,8 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
   int myRedInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorRedPart", 255 );
   int myGreenInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorGreenPart", 255 );
   int myBlueInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorBluePart", 0 );
-  QColor myColor = QColor( myRedInt, myGreenInt, myBlueInt );
+  int myAlphaInt = QgsProject::instance()->readNumEntry( "Gui", "/SelectionColorAlphaPart", 255 );
+  QColor myColor = QColor( myRedInt, myGreenInt, myBlueInt, myAlphaInt );
   pbnSelectionColor->setColor( myColor );
 
   //get the color for map canvas background and set button color accordingly (default white)
@@ -103,114 +107,8 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
   myColor = QColor( myRedInt, myGreenInt, myBlueInt );
   pbnCanvasColor->setColor( myColor );
 
-  //read the digitizing settings
-  int topologicalEditing = QgsProject::instance()->readNumEntry( "Digitizing", "/TopologicalEditing", 0 );
-  if ( topologicalEditing != 0 )
-  {
-    mEnableTopologicalEditingCheckBox->setCheckState( Qt::Checked );
-  }
-  else
-  {
-    mEnableTopologicalEditingCheckBox->setCheckState( Qt::Unchecked );
-  }
-
-  bool avoidIntersectionListOk;
-  mAvoidIntersectionsSettings.clear();
-  QStringList avoidIntersectionsList = QgsProject::instance()->readListEntry( "Digitizing", "/AvoidIntersectionsList", &avoidIntersectionListOk );
-  if ( avoidIntersectionListOk )
-  {
-    QStringList::const_iterator avoidIt = avoidIntersectionsList.constBegin();
-    for ( ; avoidIt != avoidIntersectionsList.constEnd(); ++avoidIt )
-    {
-      mAvoidIntersectionsSettings.insert( *avoidIt );
-    }
-  }
-
-  bool layerIdListOk, enabledListOk, toleranceListOk, toleranceUnitListOk, snapToListOk;
-  QStringList layerIdList = QgsProject::instance()->readListEntry( "Digitizing", "/LayerSnappingList", &layerIdListOk );
-  QStringList enabledList = QgsProject::instance()->readListEntry( "Digitizing", "/LayerSnappingEnabledList", &enabledListOk );
-  QStringList toleranceList = QgsProject::instance()->readListEntry( "Digitizing", "/LayerSnappingToleranceList", & toleranceListOk );
-  QStringList toleranceUnitList = QgsProject::instance()->readListEntry( "Digitizing", "/LayerSnappingToleranceUnitList", & toleranceUnitListOk );
-  QStringList snapToList = QgsProject::instance()->readListEntry( "Digitizing", "/LayerSnapToList", &snapToListOk );
-
-  QStringList::const_iterator idIter = layerIdList.constBegin();
-  QStringList::const_iterator enabledIter = enabledList.constBegin();
-  QStringList::const_iterator tolIter = toleranceList.constBegin();
-  QStringList::const_iterator tolUnitIter = toleranceUnitList.constBegin();
-  QStringList::const_iterator snapToIter = snapToList.constBegin();
-
   QgsMapLayer* currentLayer = 0;
 
-  //create the new layer entries
-  for ( ; idIter != layerIdList.constEnd(); ++idIter, ++enabledIter, ++tolIter, ++tolUnitIter, ++snapToIter )
-  {
-    if ( layerIdListOk )
-    {
-      currentLayer = QgsMapLayerRegistry::instance()->mapLayer( *idIter );
-    }
-    else
-    {
-      break;
-    }
-
-    if ( currentLayer )
-    {
-      LayerEntry newEntry;
-      newEntry.layerName = currentLayer->name();
-
-      newEntry.checked = false;
-      if ( enabledListOk && enabledIter != enabledList.constEnd() )
-      {
-        if (( *enabledIter ) == "enabled" )
-        {
-          newEntry.checked = true;
-        }
-      }
-
-      //snap to vertex / segment / vertex and segment
-      if ( snapToListOk && snapToIter != snapToList.constEnd() )
-      {
-        if (( *snapToIter ) == "to_vertex" )
-        {
-          newEntry.snapTo = 0;
-        }
-        else if (( *snapToIter ) == "to_segment" )
-        {
-          newEntry.snapTo = 1;
-        }
-        else //to vertex and segment
-        {
-          newEntry.snapTo = 2;
-        }
-      }
-      else
-      {
-        newEntry.snapTo = 0;
-      }
-
-      //snap tolerance
-      if ( toleranceListOk && tolIter != toleranceList.constEnd() )
-      {
-        newEntry.tolerance = tolIter->toDouble();
-      }
-      else
-      {
-        newEntry.tolerance = 0;
-      }
-
-      //snap tolerance unit
-      if ( toleranceUnitListOk && tolUnitIter != toleranceUnitList.constEnd() )
-      {
-        newEntry.toleranceUnit = tolUnitIter->toInt();
-      }
-      else
-      {
-        newEntry.toleranceUnit = 0;
-      }
-      mSnappingLayerSettings.insert( *idIter, newEntry );
-    }
-  }
-
   QStringList noIdentifyLayerIdList = QgsProject::instance()->readListEntry( "Identify", "/disabledLayers" );
 
   const QMap<QString, QgsMapLayer*> &mapLayers = QgsMapLayerRegistry::instance()->mapLayers();
@@ -229,10 +127,10 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
     currentLayer = it.value();
 
     QTableWidgetItem *twi = new QTableWidgetItem( QString::number( i ) );
-    twi->setData( Qt::UserRole, it.key() );
     twIdentifyLayers->setVerticalHeaderItem( i, twi );
 
     twi = new QTableWidgetItem( currentLayer->name() );
+    twi->setData( Qt::UserRole, it.key() );
     twi->setFlags( twi->flags() & ~Qt::ItemIsEditable );
     twIdentifyLayers->setItem( i, 0, twi );
 
@@ -260,10 +158,63 @@ QgsProjectProperties::QgsProjectProperties( QgsMapCanvas* mapCanvas, QWidget *pa
     twIdentifyLayers->setItem( i, 1, twi );
 
     QCheckBox *cb = new QCheckBox();
-    cb->setChecked( !noIdentifyLayerIdList.contains( currentLayer->getLayerID() ) );
+    cb->setChecked( !noIdentifyLayerIdList.contains( currentLayer->id() ) );
     twIdentifyLayers->setCellWidget( i, 2, cb );
   }
 
+  grpWMSServiceCapabilities->setChecked( QgsProject::instance()->readBoolEntry( "WMSServiceCapabilities", "/", false ) );
+  mWMSTitle->setText( QgsProject::instance()->readEntry( "WMSServiceTitle", "/" ) );
+  mWMSContactOrganization->setText( QgsProject::instance()->readEntry( "WMSContactOrganization", "/", "" ) );
+  mWMSContactPerson->setText( QgsProject::instance()->readEntry( "WMSContactPerson", "/", "" ) );
+  mWMSContactMail->setText( QgsProject::instance()->readEntry( "WMSContactMail", "/", "" ) );
+  mWMSContactPhone->setText( QgsProject::instance()->readEntry( "WMSContactPhone", "/", "" ) );
+  mWMSAbstract->setPlainText( QgsProject::instance()->readEntry( "WMSServiceAbstract", "/", "" ) );
+
+  bool ok;
+  QStringList values;
+
+  mWMSExtMinX->setValidator( new QDoubleValidator( mWMSExtMinX ) );
+  mWMSExtMinY->setValidator( new QDoubleValidator( mWMSExtMinY ) );
+  mWMSExtMaxX->setValidator( new QDoubleValidator( mWMSExtMaxX ) );
+  mWMSExtMaxY->setValidator( new QDoubleValidator( mWMSExtMaxY ) );
+
+  values = QgsProject::instance()->readListEntry( "WMSExtent", "/", &ok );
+  grpWMSExt->setChecked( ok && values.size() == 4 );
+  if ( grpWMSExt->isChecked() )
+  {
+    mWMSExtMinX->setText( values[0] );
+    mWMSExtMinY->setText( values[1] );
+    mWMSExtMaxX->setText( values[2] );
+    mWMSExtMaxY->setText( values[3] );
+  }
+
+  values = QgsProject::instance()->readListEntry( "WMSCrsList", "/", &ok );
+  grpWMSList->setChecked( ok && values.size() > 0 );
+  if ( grpWMSList->isChecked() )
+  {
+    mWMSList->addItems( values );
+  }
+  else
+  {
+    values = QgsProject::instance()->readListEntry( "WMSEpsgList", "/", &ok );
+    grpWMSList->setChecked( ok && values.size() > 0 );
+    if ( grpWMSList->isChecked() )
+    {
+      QStringList list;
+      foreach( QString value, values )
+      {
+        list << QString( "EPSG:%1" ).arg( value );
+      }
+
+      mWMSList->addItems( list );
+    }
+  }
+
+  grpWMSList->setChecked( mWMSList->count() > 0 );
+
+  bool addWktGeometry = QgsProject::instance()->readBoolEntry( "WMSAddWktGeometry", "/" );
+  mAddWktGeometryCheckBox->setChecked( addWktGeometry );
+
   restoreState();
 }
 
@@ -346,11 +297,9 @@ void QgsProjectProperties::apply()
     mapUnit = QGis::Degrees;
   }
 
-  QgsMapRenderer* myRender = mMapCanvas->mapRenderer();
-
-  myRender->setMapUnits( mapUnit );
-
-  myRender->setProjectionsEnabled( cbxProjectionEnabled->isChecked() );
+  QgsMapRenderer* myRenderer = mMapCanvas->mapRenderer();
+  myRenderer->setMapUnits( mapUnit );
+  myRenderer->setProjectionsEnabled( cbxProjectionEnabled->isChecked() );
 
   // Only change the projection if there is a node in the tree
   // selected that has an srid. This prevents error if the user
@@ -360,7 +309,7 @@ void QgsProjectProperties::apply()
   if ( myCRSID )
   {
     QgsCoordinateReferenceSystem srs( myCRSID, QgsCoordinateReferenceSystem::InternalCrsId );
-    myRender->setDestinationSrs( srs );
+    myRenderer->setDestinationCrs( srs );
     QgsDebugMsg( QString( "Selected CRS " ) + srs.description() );
     // write the currently selected projections _proj string_ to project settings
     QgsDebugMsg( QString( "SpatialRefSys/ProjectCRSProj4String: %1" ).arg( projectionSelector->selectedProj4String() ) );
@@ -372,7 +321,7 @@ void QgsProjectProperties::apply()
       // If we couldn't get the map units, default to the value in the
       // projectproperties dialog box (set above)
       if ( srs.mapUnits() != QGis::UnknownUnit )
-        myRender->setMapUnits( srs.mapUnits() );
+        myRenderer->setMapUnits( srs.mapUnits() );
     }
   }
 
@@ -395,6 +344,7 @@ void QgsProjectProperties::apply()
   QgsProject::instance()->writeEntry( "Gui", "/SelectionColorRedPart", myColor.red() );
   QgsProject::instance()->writeEntry( "Gui", "/SelectionColorGreenPart", myColor.green() );
   QgsProject::instance()->writeEntry( "Gui", "/SelectionColorBluePart", myColor.blue() );
+  QgsProject::instance()->writeEntry( "Gui", "/SelectionColorAlphaPart", myColor.alpha() );
   QgsRenderer::setSelectionColor( myColor );
 
   //set the color for canvas
@@ -403,78 +353,66 @@ void QgsProjectProperties::apply()
   QgsProject::instance()->writeEntry( "Gui", "/CanvasColorGreenPart", myColor.green() );
   QgsProject::instance()->writeEntry( "Gui", "/CanvasColorBluePart", myColor.blue() );
 
-  //write the digitizing settings
-  int topologicalEditingEnabled = ( mEnableTopologicalEditingCheckBox->checkState() == Qt::Checked ) ? 1 : 0;
-  QgsProject::instance()->writeEntry( "Digitizing", "/TopologicalEditing", topologicalEditingEnabled );
-
-  //store avoid intersection layers
-  QStringList avoidIntersectionList;
-  QSet<QString>::const_iterator avoidIt = mAvoidIntersectionsSettings.constBegin();
-  for ( ; avoidIt != mAvoidIntersectionsSettings.constEnd(); ++avoidIt )
-  {
-    avoidIntersectionList.append( *avoidIt );
-  }
-  QgsProject::instance()->writeEntry( "Digitizing", "/AvoidIntersectionsList", avoidIntersectionList );
-
-
-  QMap<QString, LayerEntry>::const_iterator layerEntryIt;
-
-  //store the layer snapping settings as string lists
-  QStringList layerIdList;
-  QStringList snapToList;
-  QStringList toleranceList;
-  QStringList enabledList;
-  QStringList toleranceUnitList;
-
-  for ( layerEntryIt = mSnappingLayerSettings.constBegin(); layerEntryIt != mSnappingLayerSettings.constEnd(); ++layerEntryIt )
-  {
-    layerIdList << layerEntryIt.key();
-    toleranceList << QString::number( layerEntryIt->tolerance, 'f' );
-    toleranceUnitList << QString::number(( int )layerEntryIt->toleranceUnit );
-    if ( layerEntryIt->checked )
-    {
-      enabledList << "enabled";
-    }
-    else
-    {
-      enabledList << "disabled";
-    }
-    if ( layerEntryIt->snapTo == 0 )
-    {
-      snapToList << "to_vertex";
-    }
-    else if ( layerEntryIt->snapTo == 1 )
-    {
-      snapToList << "to_segment";
-    }
-    else //to vertex and segment
-    {
-      snapToList << "to_vertex_and_segment";
-    }
-  }
-
-  if ( mSnappingLayerSettings.size() > 0 )
-  {
-    QgsProject::instance()->writeEntry( "Digitizing", "/LayerSnappingList", layerIdList );
-    QgsProject::instance()->writeEntry( "Digitizing", "/LayerSnapToList", snapToList );
-    QgsProject::instance()->writeEntry( "Digitizing", "/LayerSnappingToleranceList", toleranceList );
-    QgsProject::instance()->writeEntry( "Digitizing", "/LayerSnappingToleranceUnitList", toleranceUnitList );
-    QgsProject::instance()->writeEntry( "Digitizing", "/LayerSnappingEnabledList", enabledList );
-  }
-
   QStringList noIdentifyLayerList;
   for ( int i = 0; i < twIdentifyLayers->rowCount(); i++ )
   {
     QCheckBox *cb = qobject_cast<QCheckBox *>( twIdentifyLayers->cellWidget( i, 2 ) );
     if ( cb && !cb->isChecked() )
     {
-      QString id = twIdentifyLayers->verticalHeaderItem( i )->data( Qt::UserRole ).toString();
+      QString id = twIdentifyLayers->item( i, 0 )->data( Qt::UserRole ).toString();
       noIdentifyLayerList << id;
     }
   }
 
   QgsProject::instance()->writeEntry( "Identify", "/disabledLayers", noIdentifyLayerList );
 
+  QgsProject::instance()->writeEntry( "WMSServiceCapabilities", "/", grpWMSServiceCapabilities->isChecked() );
+  QgsProject::instance()->writeEntry( "WMSServiceTitle", "/", mWMSTitle->text() );
+  QgsProject::instance()->writeEntry( "WMSContactOrganization", "/", mWMSContactOrganization->text() );
+  QgsProject::instance()->writeEntry( "WMSContactPerson", "/", mWMSContactPerson->text() );
+  QgsProject::instance()->writeEntry( "WMSContactMail", "/", mWMSContactMail->text() );
+  QgsProject::instance()->writeEntry( "WMSContactPhone", "/", mWMSContactPhone->text() );
+  QgsProject::instance()->writeEntry( "WMSServiceAbstract", "/", mWMSAbstract->toPlainText() );
+
+  if ( grpWMSExt->isChecked() )
+  {
+    QgsProject::instance()->writeEntry( "WMSExtent", "/",
+                                        QStringList()
+                                        << mWMSExtMinX->text()
+                                        << mWMSExtMinY->text()
+                                        << mWMSExtMaxX->text()
+                                        << mWMSExtMaxY->text() );
+  }
+  else
+  {
+    QgsProject::instance()->removeEntry( "WMSExtent", "/" );
+  }
+
+  if ( grpWMSList->isChecked() && mWMSList->count() == 0 )
+  {
+    QMessageBox::information( this, tr( "Coordinate System Restriction" ), tr( "No coordinate systems selected. Disabling restriction." ) );
+    grpWMSList->setChecked( false );
+  }
+
+  QgsProject::instance()->removeEntry( "WMSEpsgList", "/" );
+
+  if ( grpWMSList->isChecked() )
+  {
+    QStringList crslist;
+    for ( int i = 0; i < mWMSList->count(); i++ )
+    {
+      crslist << mWMSList->item( i )->text();
+    }
+
+    QgsProject::instance()->writeEntry( "WMSCrsList", "/", crslist );
+  }
+  else
+  {
+    QgsProject::instance()->removeEntry( "WMSCrsList", "/" );
+  }
+
+  QgsProject::instance()->writeEntry( "WMSAddWktGeometry", "/", mAddWktGeometryCheckBox->isChecked() );
+
   //todo XXX set canvas color
   emit refresh();
 }
@@ -491,7 +429,12 @@ void QgsProjectProperties::showProjectionsTab()
 
 void QgsProjectProperties::on_pbnSelectionColor_clicked()
 {
-  QColor color = QColorDialog::getColor( pbnSelectionColor->color(), this );
+#if QT_VERSION >= 0x040500
+  QColor color = QColorDialog::getColor( pbnSelectionColor->color(), 0, tr( "Selection color" ), QColorDialog::ShowAlphaChannel );
+#else
+  QColor color = QColorDialog::getColor( pbnSelectionColor->color() );
+#endif
+
   if ( color.isValid() )
   {
     pbnSelectionColor->setColor( color );
@@ -507,30 +450,39 @@ void QgsProjectProperties::on_pbnCanvasColor_clicked()
   }
 }
 
-void QgsProjectProperties::on_mAvoidIntersectionsPushButton_clicked()
-{
-  QgsAvoidIntersectionsDialog d( mMapCanvas, mAvoidIntersectionsSettings );
-  if ( d.exec() == QDialog::Accepted )
-  {
-    d.enabledLayers( mAvoidIntersectionsSettings );
-  }
-}
-
-void QgsProjectProperties::on_mSnappingOptionsPushButton_clicked()
-{
-  QgsSnappingDialog d( mMapCanvas, mSnappingLayerSettings );
-  if ( d.exec() == QDialog::Accepted )
-  {
-    //retrieve the new layer snapping settings from the dialog
-    d.layerSettings( mSnappingLayerSettings );
-  }
-}
-
 void QgsProjectProperties::on_cbxProjectionEnabled_stateChanged( int state )
 {
   btnGrpMapUnits->setEnabled( state == Qt::Unchecked );
 }
 
+void QgsProjectProperties::setMapUnitsToCurrentProjection()
+{
+  long myCRSID = projectionSelector->selectedCrsId();
+  if ( myCRSID )
+  {
+    QgsCoordinateReferenceSystem srs( myCRSID, QgsCoordinateReferenceSystem::InternalCrsId );
+    //set radio button to crs map unit type
+    QGis::UnitType units = srs.mapUnits();
+    switch ( units )
+    {
+      case QGis::Meters:
+        radMeters->setChecked( true );
+        break;
+      case QGis::Feet:
+        radFeet->setChecked( true );
+        break;
+      case QGis::Degrees:
+        radDecimalDegrees->setChecked( true );
+        break;
+      case QGis::DegreesMinutesSeconds:
+        radDMS->setChecked( true );
+        break;
+      default:
+        break;
+    }
+  }
+}
+
 /*!
  * Function to save dialog window state
  */
@@ -550,3 +502,71 @@ void QgsProjectProperties::restoreState()
   restoreGeometry( settings.value( "/Windows/ProjectProperties/geometry" ).toByteArray() );
   tabWidget->setCurrentIndex( settings.value( "/Windows/ProjectProperties/tab" ).toInt() );
 }
+
+/*!
+ * Set WMS default extent to current canvas extent
+ */
+void QgsProjectProperties::on_pbnWMSExtCanvas_clicked()
+{
+  QgsRectangle ext = mMapCanvas->extent();
+  mWMSExtMinX->setText( QString::number( ext.xMinimum(), 'f', 8 ) );
+  mWMSExtMinY->setText( QString::number( ext.yMinimum(), 'f', 8 ) );
+  mWMSExtMaxX->setText( QString::number( ext.xMaximum(), 'f', 8 ) );
+  mWMSExtMaxY->setText( QString::number( ext.yMaximum(), 'f', 8 ) );
+}
+
+void QgsProjectProperties::on_pbnWMSAddSRS_clicked()
+{
+  QgsGenericProjectionSelector *mySelector = new QgsGenericProjectionSelector( this );
+  mySelector->setMessage();
+  if ( mySelector->exec() )
+  {
+    QString authid = mySelector->selectedAuthId();
+
+    QList<QListWidgetItem *> items = mWMSList->findItems( authid.mid( 5 ), Qt::MatchFixedString );
+    if ( items.size() == 0 )
+    {
+      mWMSList->addItem( authid );
+    }
+    else
+    {
+      QMessageBox::information( this, tr( "Coordinate System Restriction" ), tr( "CRS %1 was already selected" ).arg( authid ) );
+    }
+  }
+
+  delete mySelector;
+}
+
+void QgsProjectProperties::on_pbnWMSRemoveSRS_clicked()
+{
+  foreach( QListWidgetItem *item, mWMSList->selectedItems() )
+  delete item;
+}
+
+void QgsProjectProperties::on_pbnWMSSetUsedSRS_clicked()
+{
+  if ( mWMSList->count() > 1 )
+  {
+    if ( QMessageBox::question( this,
+                                tr( "Coordinate System Restrictions" ),
+                                tr( "The current selection of coordinate systems will be lost.\nProceed?" ) ) == QMessageBox::No )
+      return;
+  }
+
+  QSet<QString> crsList;
+
+  if ( cbxProjectionEnabled->isChecked() )
+  {
+    QgsCoordinateReferenceSystem srs( projectionSelector->selectedCrsId(), QgsCoordinateReferenceSystem::InternalCrsId );
+    crsList << srs.authid();
+  }
+
+  const QMap<QString, QgsMapLayer*> &mapLayers = QgsMapLayerRegistry::instance()->mapLayers();
+  for ( QMap<QString, QgsMapLayer*>::const_iterator it = mapLayers.constBegin(); it != mapLayers.constEnd(); it++ )
+  {
+    crsList << it.value()->crs().authid();
+  }
+
+  mWMSList->clear();
+  mWMSList->addItems( crsList.values() );
+}
