@@ -21,7 +21,6 @@
 #include "ui_qgsprojectpropertiesbase.h"
 #include "qgis.h"
 #include "qgisgui.h"
-#include "qgssnappingdialog.h"
 #include "qgscontexthelp.h"
 
 class QgsMapCanvas;
@@ -85,23 +84,26 @@ class QgsProjectProperties : public QDialog, private Ui::QgsProjectPropertiesBas
      */
     void on_pbnCanvasColor_clicked();
 
+    /*!
+     * Slots for WMS project settings
+     */
+    void on_pbnWMSExtCanvas_clicked();
+    void on_pbnWMSAddSRS_clicked();
+    void on_pbnWMSRemoveSRS_clicked();
+    void on_pbnWMSSetUsedSRS_clicked();
+
     /*!
      * Slot to show the context help for this dialog
      */
     void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
 
-    /*!
-      *
-      */
-    void on_mAvoidIntersectionsPushButton_clicked();
-
-    /*!
-     * Slot to show dialog for the layer snapping options
-     */
-    void on_mSnappingOptionsPushButton_clicked();
-
     void on_cbxProjectionEnabled_stateChanged( int state );
 
+    /*!
+      * If user changes the CRS, set the corresponding map units
+      */
+    void setMapUnitsToCurrentProjection();
+
   signals:
     //! Signal used to inform listeners that the mouse display precision may have changed
     void displayPrecisionChanged();
@@ -113,15 +115,6 @@ class QgsProjectProperties : public QDialog, private Ui::QgsProjectPropertiesBas
   private:
     QgsMapCanvas* mMapCanvas;
 
-    /**Snapping settings to pass/read from QgsSnappingDialog.
-     Key is the layer id, the pair consists of snap to vertex = 0/snap to segment = 1,
-    snapping tolerance*/
-    QMap<QString, LayerEntry> mSnappingLayerSettings;
-
-    /**Stores ids of layers where intersections of new polygons is considered. Is passed to / read from QgsAvoidIntersectionsDialog*/
-    QSet<QString> mAvoidIntersectionsSettings;
-
-
     /*!
      * Function to save dialog window state
      */
