@@ -16,18 +16,20 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id$ */
+
 #ifndef QGSRASTERLAYERPROPERTIES_H
 #define QGSRASTERLAYERPROPERTIES_H
 
 #include "ui_qgsrasterlayerpropertiesbase.h"
 #include "qgisgui.h"
+#include "qgsmaptool.h"
 #include "qgscolorrampshader.h"
 #include "qgscontexthelp.h"
 
 class QgsMapLayer;
+class QgsMapCanvas;
 class QgsRasterLayer;
-
+class QgsMapToolEmitPoint;
 
 /**Property sheet for a raster map layer
   *@author Tim Sutton
@@ -41,7 +43,7 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     /** \brief Constructor
      * @param ml Map layer for which properties will be displayed
      */
-    QgsRasterLayerProperties( QgsMapLayer *lyr, QWidget *parent = 0, Qt::WFlags = QgisGui::ModalDialogFlags );
+    QgsRasterLayerProperties( QgsMapLayer *lyr, QgsMapCanvas* theCanvas, QWidget *parent = 0, Qt::WFlags = QgisGui::ModalDialogFlags );
     /** \brief Destructor */
     ~QgsRasterLayerProperties();
 
@@ -64,8 +66,10 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     void on_pbnDefaultValues_clicked();
     /** \brief slot executed when user wishes to export transparency values */
     void on_pbnExportTransparentPixelValues_clicked();
+    /** \brief auto slot executed when the active page in the main widget stack is changed */
+    void on_tabBar_currentChanged( int theTab );
     /** \brief slot executed when user wishes to refresh raster histogram */
-    void on_pbnHistRefresh_clicked();
+    void refreshHistogram();
     /** \brief slow executed when user wishes to import transparency values */
     void on_pbnImportTransparentPixelValues_clicked();
     /** \brief slot executed when user presses "Remove Selected Row" button on the transparency page */
@@ -82,6 +86,10 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     void on_rbtnThreeBandMinMax_toggled( bool );
     /** \brief slot executed when the three band standard deviation radio button is pressed. */
     void on_rbtnThreeBandStdDev_toggled( bool );
+    /** \brief slot executed when the reset null value to file default icon is selected */
+    void on_btnResetNull_clicked( );
+
+    void pixelSelected( const QgsPoint& );
     /** \brief this slot clears min max values from gui */
     void sboxSingleBandStdDev_valueChanged( double );
     /** \brief this slot clears min max values from gui */
@@ -92,48 +100,54 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     void userDefinedMinMax_textEdited( QString );
 
   private slots:
-    /** This slow handles necessary interface modifiations (i.e., loading min max values) */
+    /** This slow handles necessary interface modifications (i.e. loading min max values) */
     void on_cboBlue_currentIndexChanged( const QString& );
-    /** This slow handles necessary interface modifiations (i.e., loading min max values) */
+    /** This slow handles necessary interface modifications (i.e. loading min max values) */
     void on_cboGray_currentIndexChanged( const QString& );
-    /** This slow handles necessary interface modifiations (i.e., loading min max values) */
+    /** This slow handles necessary interface modifications (i.e. loading min max values) */
     void on_cboGreen_currentIndexChanged( const QString& );
-    /** This slow handles necessary interface modifiations (i.e., loading min max values) */
+    /** This slow handles necessary interface modifications (i.e. loading min max values) */
     void on_cboRed_currentIndexChanged( const QString& );
-    /**The slot handles necessary interface modifications based when color map selected changes*/
+    /** This slot handles necessary interface modifications based when color map selected changes */
     void on_cboxColorMap_currentIndexChanged( const QString& );
-    /**This slot calculates classification values and colors for the tree widget on the colormap tab*/
+    /** This slot calculates classification values and colors for the tree widget on the colormap tab */
     void on_mClassifyButton_clicked();
-    /**This slot deletes the current class from the tree widget on the colormap tab*/
+    /** This slot deletes the current class from the tree widget on the colormap tab */
     void on_mDeleteEntryButton_clicked();
-    /**Callback for double clicks on the colormap entry widget*/
+    /** Callback for double clicks on the colormap entry widget */
     void handleColormapTreeWidgetDoubleClick( QTreeWidgetItem* item, int column );
-    /**This slot adds a new row to the color map table */
+    /** This slot adds a new row to the color map table */
     void on_pbtnAddColorMapEntry_clicked();
-    /**This slots saves the current color map to a file */
+    /** This slots saves the current color map to a file */
     void on_pbtnExportColorMapToFile_clicked();
-    /**This slots loads the current color map from a band */
+    /** This slots loads the current color map from a band */
     void on_pbtnLoadColorMapFromBand_clicked();
-    /**This slots loads the current color map from a file */
+    /** This slots loads the current color map from a file */
     void on_pbtnLoadColorMapFromFile_clicked();
-    /**This slot loads the minimum and maximum values from the raster band and updates the gui*/
+    /** This slot loads the minimum and maximum values from the raster band and updates the gui */
     void on_pbtnLoadMinMax_clicked();
-    /**This slot sets the default band combination varaible to current band combination */
+    /** This slot sets the default band combination variable to current band combination */
     void on_pbtnMakeBandCombinationDefault_clicked();
-    /**This slot sets the default contrast enhancement varaible  to current contrast enhancement algorithm */
+    /** This slot sets the default contrast enhancement variable  to current contrast enhancement algorithm */
     void on_pbtnMakeContrastEnhancementAlgorithmDefault_clicked();
-    /**This slot will sort the color map  in ascending order*/
+    /** This slot sets the standard deviation default */
+    void on_pbtnMakeStandardDeviationDefault_clicked();
+    /** This slot will sort the color map in ascending order */
     void on_pbtnSortColorMap_clicked();
-    /** Load the default style when appriate button is pressed. */
+    /** Load the default style when appropriate button is pressed. */
     void on_pbnLoadDefaultStyle_clicked();
-    /** Save the default style when appriate button is pressed. */
+    /** Save the default style when appropriate button is pressed. */
     void on_pbnSaveDefaultStyle_clicked();
-    /** Load a saved  style when appriate button is pressed. */
+    /** Load a saved style when appropriate button is pressed. */
     void on_pbnLoadStyle_clicked();
     /** Save a style when appriate button is pressed. */
     void on_pbnSaveStyleAs_clicked();
     /** Help button */
     void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
+    /** This slot lets you save the histogram as an image to disk */
+    void on_mSaveAsImageButton_clicked();
+    /**Enable or disable Build pyramids button depending on selection in pyramids list*/
+    void toggleBuildPyramidsButton();
 
   signals:
 
@@ -148,6 +162,9 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     /** \brief Default contrast enhancement algorithm */
     QString mDefaultContrastEnhancementAlgorithm;
 
+    /** \brief default standard deviation */
+    double mDefaultStandardDeviation;
+
     /** \brief Default band combination */
     int mDefaultRedBand;
     int mDefaultGreenBand;
@@ -165,19 +182,12 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     /** \brief Pointer to the raster layer that this property dilog changes the behaviour of. */
     QgsRasterLayer * mRasterLayer;
 
-    /** \brief If the underlying raster layer is of GDAL type (i.e. non-provider)
+    /** \brief If the underlying raster layer doesn't have a provider
 
         This variable is used to determine if various parts of the Properties UI are
         included or not
      */
-    bool mRasterLayerIsGdal;
-
-    /** \brief If the underlying raster layer is of WMS type (i.e. WMS data provider)
-
-        This variable is used to determine if various parts of the Properties UI are
-        included or not
-     */
-    bool mRasterLayerIsWms;
+    //bool mRasterLayerIsInternal;
 
     /** \brief Clear current color map table and population with values from new list */
     void populateColorMapTable( const QList<QgsColorRampShader::ColorRampItem>& );
@@ -203,6 +213,8 @@ class QgsRasterLayerProperties : public QDialog, private Ui::QgsRasterLayerPrope
     QLinearGradient highlightGradient();
     qreal mGradientHeight;
     qreal mGradientWidth;
-};
 
+    QgsMapCanvas* mMapCanvas;
+    QgsMapToolEmitPoint* mPixelSelectorTool;
+};
 #endif
