@@ -30,11 +30,57 @@
 #include <QPainter>
 #include <QImage>
 #include <QFileDialog>
-#include <QListWidgetItem>
 #include <QKeyEvent>
+#include <QAbstractListModel>
+#include <QPixmapCache>
 
 #define DO_NOT_USE_STR "<off>"
 
+
+class QgsMarkerListModel : public QAbstractListModel
+{
+  public:
+    QgsMarkerListModel( QObject* parent ) : QAbstractListModel( parent )
+    {
+      mMarkers = QgsMarkerCatalogue::instance()->list();
+    }
+
+    int rowCount( const QModelIndex & parent = QModelIndex() ) const
+    {
+      return mMarkers.size();
+    }
+
+    QVariant data( const QModelIndex & index, int role = Qt::DisplayRole ) const
+    {
+      QString marker = mMarkers.at( index.row() );
+
+      if ( role == Qt::DecorationRole ) // icon
+      {
+        QPixmap pixmap;
+
+        if ( !QPixmapCache::find( "ssd_" + marker, pixmap ) )
+        {
+          QPen pen( QColor( 0, 0, 255 ) );
+          QBrush brush( QColor( 220, 220, 220 ), Qt::SolidPattern );
+          pixmap = QPixmap::fromImage( QgsMarkerCatalogue::instance()->imageMarker( marker, 18, pen, brush ) );
+          QPixmapCache::insert( "ssd_" + marker, pixmap );
+        }
+
+        return pixmap;
+      }
+      else if ( role == Qt::UserRole || role == Qt::ToolTipRole )
+      {
+        return marker;
+      }
+
+      return QVariant();
+    }
+
+  protected:
+    QStringList mMarkers;
+};
+
+
 QgsSingleSymbolDialog::QgsSingleSymbolDialog(): QDialog(), mVectorLayer( 0 )
 {
   setupUi( this );
@@ -71,8 +117,6 @@ QgsSingleSymbolDialog::QgsSingleSymbolDialog( QgsVectorLayer * layer, bool disab
   connect( btnFillColor, SIGNAL( clicked() ), this, SLOT( selectFillColor() ) );
   connect( outlinewidthspinbox, SIGNAL( valueChanged( double ) ), this, SLOT( resendSettingsChanged() ) );
   connect( mLabelEdit, SIGNAL( textChanged( const QString& ) ), this, SLOT( resendSettingsChanged() ) );
-  connect( lstSymbols, SIGNAL( currentItemChanged( QListWidgetItem *, QListWidgetItem * ) ),
-           this, SLOT( symbolChanged( QListWidgetItem *, QListWidgetItem * ) ) );
   connect( mPointSizeSpinBox, SIGNAL( valueChanged( double ) ), this, SLOT( resendSettingsChanged() ) );
   connect( mPointSizeUnitsCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( resendSettingsChanged() ) );
   connect( mRotationClassificationComboBox, SIGNAL( currentIndexChanged( const QString & ) ),
@@ -95,32 +139,11 @@ QgsSingleSymbolDialog::QgsSingleSymbolDialog( QgsVectorLayer * layer, bool disab
 
 void QgsSingleSymbolDialog::refreshMarkers()
 {
-  lstSymbols->blockSignals( true );
-  lstSymbols->clear();
+  QgsMarkerListModel *m = new QgsMarkerListModel( lstSymbols );
+  lstSymbols->setModel( m );
 
-  QPen pen( QColor( 0, 0, 255 ) );
-  QBrush brush( QColor( 220, 220, 220 ), Qt::SolidPattern );
-  int size = 18;
-  int myCounter = 0;
-  QStringList ml = QgsMarkerCatalogue::instance()->list();
-  for ( QStringList::iterator it = ml.begin(); it != ml.end(); ++it )
-  {
-    QPixmap myPixmap = QPixmap::fromImage( QgsMarkerCatalogue::instance()->imageMarker( *it, size, pen, brush ) );
-    QListWidgetItem * mypItem = new QListWidgetItem( lstSymbols );
-    QIcon myIcon;
-    myIcon.addPixmap( myPixmap );
-    mypItem->setIcon( myIcon );
-    mypItem->setText( "" );
-    mypItem->setToolTip( *it );
-    //store the symbol offset in the UserData role for later retrieval
-    mypItem->setData( Qt::UserRole, *it );
-    mypItem->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
-    if ( mVectorLayer && mVectorLayer->geometryType() != QGis::Point )
-    {
-      break;
-    }
-    ++myCounter;
-  }
+  connect( lstSymbols->selectionModel(), SIGNAL( currentChanged( const QModelIndex &, const QModelIndex & ) ),
+           this, SLOT( symbolChanged( const QModelIndex &, const QModelIndex & ) ) );
 
   // Find out the numerical fields of mVectorLayer, and populate the ComboBoxes
   QgsVectorDataProvider *provider = mVectorLayer->dataProvider();
@@ -157,16 +180,17 @@ void QgsSingleSymbolDialog::refreshMarkers()
   //set outline / line style
   //
   cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "SolidLine" ) ), "", "SolidLine" );
+  cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "NoPen" ) ), tr( "None" ), "NoPen" );
   cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "DashLine" ) ), "", "DashLine" );
   cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "DotLine" ) ), "", "DotLine" );
   cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "DashDotLine" ) ), "" , "DashDotLine" );
   cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "DashDotDotLine" ) ), "", "DashDotDotLine" );
-  cboOutlineStyle->addItem( QIcon( QgsSymbologyUtils::char2LinePixmap( "NoPen" ) ), tr( "None" ), "NoPen" );
 
   //
   //set pattern icons and state
   //
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "SolidPattern" ) ), "", "SolidPattern" );
+  cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "NoBrush" ) ), tr( "None" ), "NoBrush" );
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "HorPattern" ) ), "", "HorPattern" );
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "VerPattern" ) ), "", "VerPattern" );
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "CrossPattern" ) ), "", "CrossPattern" );
@@ -180,7 +204,6 @@ void QgsSingleSymbolDialog::refreshMarkers()
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "Dense5Pattern" ) ), "", "Dense5Pattern" );
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "Dense6Pattern" ) ), "", "Dense6Pattern" );
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "Dense7Pattern" ) ), "", "Dense7Pattern" );
-  cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "NoBrush" ) ), tr( "None" ), "NoBrush" );
   cboFillStyle->addItem( QIcon( QgsSymbologyUtils::char2PatternPixmap( "TexturePattern" ) ), tr( "Texture" ), "TexturePattern" );
 
   if ( mVectorLayer && mVectorLayer->geometryType() != QGis::Point )
@@ -229,7 +252,12 @@ QgsSingleSymbolDialog::~QgsSingleSymbolDialog()
 
 void QgsSingleSymbolDialog::selectOutlineColor()
 {
+#if defined(Q_WS_MAC) && QT_VERSION >= 0x040500 && defined(QT_MAC_USE_COCOA)
+  // Native Mac dialog works only for Qt Carbon
+  QColor c = QColorDialog::getColor( btnOutlineColor->color(), this, "", QColorDialog::DontUseNativeDialog );
+#else
   QColor c = QColorDialog::getColor( btnOutlineColor->color(), this );
+#endif
 
   if ( c.isValid() )
   {
@@ -242,7 +270,12 @@ void QgsSingleSymbolDialog::selectOutlineColor()
 
 void QgsSingleSymbolDialog::selectFillColor()
 {
+#if defined(Q_WS_MAC) && QT_VERSION >= 0x040500 && defined(QT_MAC_USE_COCOA)
+  // Native Mac dialog works only for Qt Carbon
+  QColor c = QColorDialog::getColor( btnFillColor->color(), this, "", QColorDialog::DontUseNativeDialog );
+#else
   QColor c = QColorDialog::getColor( btnFillColor->color(), this );
+#endif
 
   if ( c.isValid() )
   {
@@ -256,11 +289,12 @@ void QgsSingleSymbolDialog::selectFillColor()
 //should this method have a different name?
 void QgsSingleSymbolDialog::selectTextureImage()
 {
-  QString fileName = QFileDialog::getOpenFileName( this, "Open File",
+  QString fileName = QFileDialog::getOpenFileName( this,
+                     tr( "Open File" ),
                      mTexturePath,
-                     "Images (*.png *.xpm *.jpg)" ); //should we allow other types of images?
+                     tr( "Images (*.png *.xpm *.jpg)" ) ); //should we allow other types of images?
 
-  if ( fileName.isNull() == false )
+  if ( !fileName.isNull() )
   { //only process the string if the user clicked OK
     mTexturePath = fileName;
     resendSettingsChanged();
@@ -282,9 +316,10 @@ void QgsSingleSymbolDialog::apply( QgsSymbol *sy )
   //
   // Apply point symbol
   //
-  if ( lstSymbols->isEnabled() && lstSymbols->currentItem() )
+  if ( lstSymbols->isEnabled() && lstSymbols->currentIndex().isValid() )
   {
-    sy->setNamedPointSymbol( lstSymbols->currentItem()->data( Qt::UserRole ).toString() ) ;
+    QAbstractItemModel *m = lstSymbols->model();
+    sy->setNamedPointSymbol( m->data( lstSymbols->currentIndex(), Qt::UserRole ).toString() );
   }
 
   if ( mPointSizeSpinBox->isEnabled() )
@@ -375,12 +410,15 @@ void QgsSingleSymbolDialog::set( const QgsSymbol *sy )
 
   // Set point symbol
   QString mySymbolName = sy->pointSymbolName();
-  for ( int i = 0; i < lstSymbols->count(); ++i )
+
+  QAbstractItemModel *m = lstSymbols->model();
+  for ( int i = 0; i < m->rowCount(); i++ )
   {
-    if ( lstSymbols->item( i )->data( Qt::UserRole ).toString() == ( mySymbolName ) )
+    QModelIndex idx( m->index( i, 0 ) );
+    if ( m->data( idx, Qt::UserRole ).toString() == mySymbolName )
     {
-      lstSymbols->setCurrentItem( lstSymbols->item( i ) );
-      lstSymbols->item( i )->setBackground( QBrush( Qt::cyan ) );
+      lstSymbols->setCurrentIndex( idx );
+      // m->setData( idx, Qt::UserRole+1, Qt::cyan );
       break;
     }
   }
@@ -477,8 +515,13 @@ void QgsSingleSymbolDialog::updateSet( const QgsSymbol *sy )
   if ( mLabelEdit->isEnabled() && mLabelEdit->text() != sy->label() )
     mLabelEdit->setEnabled( false );
 
-  if ( lstSymbols->isEnabled() && lstSymbols->currentItem()->data( Qt::UserRole ).toString() != sy->pointSymbolName() )
-    lstSymbols->setEnabled( false );
+  if ( lstSymbols->isEnabled() && lstSymbols->currentIndex().isValid() )
+  {
+    QAbstractItemModel *m = lstSymbols->model();
+
+    if ( m->data( lstSymbols->currentIndex(), Qt::UserRole ).toString() != sy->pointSymbolName() )
+      lstSymbols->setEnabled( false );
+  }
 
   if ( mPointSizeSpinBox->isEnabled() && !doubleNear( mPointSizeSpinBox->value(), sy->pointSize() ) )
     mPointSizeSpinBox->setEnabled( false );
@@ -611,14 +654,8 @@ void QgsSingleSymbolDialog::setLabel( QString label )
   mLabelEdit->setText( label );
 }
 
-void QgsSingleSymbolDialog::symbolChanged
-( QListWidgetItem * current, QListWidgetItem * previous )
+void QgsSingleSymbolDialog::symbolChanged( const QModelIndex &current, const QModelIndex &previous )
 {
-  current->setBackground( QBrush( Qt::cyan ) );
-  if ( previous )
-  {
-    previous->setBackground( QBrush( Qt::white ) );
-  }
   emit settingsChanged();
 }
 
