@@ -286,15 +286,14 @@ void QgsUniqueValueDialog::changeClassificationAttribute()
                               tr( "The classification field was changed from '%1' to '%2'.\n"
                                   "Should the existing classes be deleted before classification?" )
                               .arg( mOldClassificationAttribute ).arg( attributeName ),
-                              QMessageBox::Ok | QMessageBox::Cancel ) == QMessageBox::Ok )
+                              QMessageBox::Yes | QMessageBox::No ) == QMessageBox::Yes )
   {
     mClassListWidget->clearSelection();
     deleteSelectedClasses();
   }
   mOldClassificationAttribute = attributeName;
 
-  QgsVectorDataProvider *provider = dynamic_cast<QgsVectorDataProvider *>( mVectorLayer->dataProvider() );
-  if ( provider )
+  if ( mVectorLayer )
   {
     int nr = mClassificationComboBox->itemData( mClassificationComboBox->currentIndex() ).toInt();
     if ( nr == -1 )
@@ -303,7 +302,7 @@ void QgsUniqueValueDialog::changeClassificationAttribute()
     }
 
     QList<QVariant> values;
-    provider->uniqueValues( nr, values );
+    mVectorLayer->uniqueValues( nr, values );
 
     for ( int i = 0; i < values.size(); i++ )
     {
@@ -428,11 +427,10 @@ void QgsUniqueValueDialog::applySymbologyChanges()
     }
 
     QgsSymbol *symbol = mValues[ value ];
-    symbol->setLabel( sydialog.label() );
     symbol->setLowerValue( value );
     sydialog.apply( symbol );
 
-    item->setToolTip( sydialog.label() );
+    item->setToolTip( symbol->label() );
     item->setData( Qt::UserRole, value );
     updateEntryIcon( symbol, item );
   }
