@@ -104,8 +104,18 @@ class QgsVectorLayerProperties : public QDialog, private Ui::QgsVectorLayerPrope
     void on_tblAttributes_cellChanged( int row, int column );
     void on_mCalculateFieldButton_clicked();
     void on_pbnSelectEditForm_clicked();
+    void on_tabWidget_currentChanged( int idx );
     void on_buttonBox_helpRequested() { QgsContextHelp::run( metaObject()->className() ); }
+    void on_mAddCategoryPushButton_clicked();
+    void on_mRemoveCategoryPushButton_clicked();
+    void on_mDiagramFontButton_clicked();
+    void on_mFixedSizeCheckBox_stateChanged( int state );
+    void on_mScaleDependentDiagramVisibilityCheckBox_stateChanged( int state );
+    void on_mFindMaximumValueButton_clicked();
+    void on_mBackgroundColorButton_clicked();
+    void on_mDiagramPenColorButton_clicked();
 
+    void enableLabelOptions( bool theFlag );
     void addAttribute();
     void deleteAttribute();
 
@@ -115,6 +125,12 @@ class QgsVectorLayerProperties : public QDialog, private Ui::QgsVectorLayerPrope
     void useNewSymbology();
     void setUsingNewSymbology( bool useNewSymbology );
 
+    void on_mButtonAddJoin_clicked();
+    void on_mButtonRemoveJoin_clicked();
+
+    /**Set color for diagram category*/
+    void handleDiagramItemDoubleClick( QTreeWidgetItem * item, int column );
+
   signals:
 
     /** emitted when changes to layer were saved to update legend */
@@ -140,6 +156,9 @@ class QgsVectorLayerProperties : public QDialog, private Ui::QgsVectorLayerPrope
     };
 
     QgsVectorLayer *layer;
+
+    bool mMetadataFilled;
+
     /**Renderer dialog which is shown*/
     QDialog* mRendererDialog;
     /**Buffer renderer, which is assigned to the vector layer when apply is pressed*/
@@ -156,16 +175,23 @@ class QgsVectorLayerProperties : public QDialog, private Ui::QgsVectorLayerPrope
     QMap<int, QgsVectorLayer::RangeData> mRanges;
     QMap<int, QPair<QString, QString> > mCheckedStates;
 
+    QFont mDiagramFont;
+
     void updateButtons();
     void loadRows();
     void setRow( int row, int idx, const QgsField &field );
 
+    void initDiagramTab();
+
     /**Requests all overlay plugis from the plugin registry. Useful for inserting their dialogs as new tabs*/
     QList<QgsVectorOverlayPlugin*> overlayPlugins() const;
 
     /**Buffer pixmap which takes the picture of renderers before they are assigned to the vector layer*/
     //QPixmap bufferPixmap;
 
+    /**Adds a new join to mJoinTreeWidget*/
+    void addJoinToTreeWidget( const QgsVectorJoinInfo& join );
+
     static QMap< QgsVectorLayer::EditType, QString > editTypeMap;
     static void setupEditTypes();
     static QString editTypeButtonText( QgsVectorLayer::EditType type );
