@@ -31,16 +31,19 @@
 #include "qgsrectangle.h" //just for debugging
 #include "qgslogger.h"
 
-#ifndef Q_OS_MACX
 #include <cmath>
-#else
-#include <math.h>
-#endif
 
 #define FONT_WORKAROUND_SCALE 10 //scale factor for upscaling fontsize and downscaling painter
 
-QgsComposerItem::QgsComposerItem( QgsComposition* composition, bool manageZValue ): QObject( 0 ), QGraphicsRectItem( 0 ), mComposition( composition ), mBoundingResizeRectangle( 0 ), \
-    mFrame( true ), mItemPositionLocked( false ), mLastValidViewScaleFactor( -1 ), mRotation( 0 )
+QgsComposerItem::QgsComposerItem( QgsComposition* composition, bool manageZValue )
+    : QObject( 0 )
+    , QGraphicsRectItem( 0 )
+    , mComposition( composition )
+    , mBoundingResizeRectangle( 0 )
+    , mFrame( true )
+    , mItemPositionLocked( false )
+    , mLastValidViewScaleFactor( -1 )
+    , mRotation( 0 )
 {
   setFlag( QGraphicsItem::ItemIsSelectable, true );
   setAcceptsHoverEvents( true );
@@ -58,9 +61,15 @@ QgsComposerItem::QgsComposerItem( QgsComposition* composition, bool manageZValue
   }
 }
 
-QgsComposerItem::QgsComposerItem( qreal x, qreal y, qreal width, qreal height, QgsComposition* composition, bool manageZValue ): \
-    QObject( 0 ), QGraphicsRectItem( 0, 0, width, height, 0 ), mComposition( composition ), mBoundingResizeRectangle( 0 ), mFrame( true ), \
-    mItemPositionLocked( false ), mLastValidViewScaleFactor( -1 ), mRotation( 0 )
+QgsComposerItem::QgsComposerItem( qreal x, qreal y, qreal width, qreal height, QgsComposition* composition, bool manageZValue )
+    : QObject( 0 )
+    , QGraphicsRectItem( 0, 0, width, height, 0 )
+    , mComposition( composition )
+    , mBoundingResizeRectangle( 0 )
+    , mFrame( true )
+    , mItemPositionLocked( false )
+    , mLastValidViewScaleFactor( -1 )
+    , mRotation( 0 )
 {
   setFlag( QGraphicsItem::ItemIsSelectable, true );
   setAcceptsHoverEvents( true );
@@ -262,6 +271,30 @@ bool QgsComposerItem::_readXML( const QDomElement& itemElem, const QDomDocument&
   return true;
 }
 
+void QgsComposerItem::beginCommand( const QString& commandText, QgsComposerMergeCommand::Context c )
+{
+  if ( mComposition )
+  {
+    mComposition->beginCommand( this, commandText, c );
+  }
+}
+
+void QgsComposerItem::endCommand()
+{
+  if ( mComposition )
+  {
+    mComposition->endCommand();
+  }
+}
+
+void QgsComposerItem::cancelCommand()
+{
+  if ( mComposition )
+  {
+    mComposition->cancelCommand();
+  }
+}
+
 void QgsComposerItem::mouseMoveEvent( QGraphicsSceneMouseEvent * event )
 {
   if ( mItemPositionLocked )
@@ -333,12 +366,14 @@ void QgsComposerItem::mouseReleaseEvent( QGraphicsSceneMouseEvent * event )
   double diffY = mouseMoveStopPoint.y() - mMouseMoveStartPos.y();
 
   //it was only a click
-  if ( abs( diffX ) < std::numeric_limits<double>::min() && abs( diffY ) < std::numeric_limits<double>::min() )
+  if ( qAbs( diffX ) < std::numeric_limits<double>::min() && qAbs( diffY ) < std::numeric_limits<double>::min() )
   {
     return;
   }
 
+  beginCommand( tr( "Change item position" ) );
   changeItemRectangle( mouseMoveStopPoint, mMouseMoveStartPos, this, diffX, diffY, this );
+  endCommand();
 
   //reset default action
   mCurrentMouseMoveAction = QgsComposerItem::MoveItem;
@@ -530,8 +565,8 @@ void QgsComposerItem::changeItemRectangle( const QPointF& currentPosition, const
       }
       else  //for composer items, we prefer setSceneRect as subclasses can implement custom behaviour (e.g. item group)
       {
-        changeComposerItem->setSceneRect( QRectF( originalItem->transform().dx() + moveRectX, \
-                                          originalItem->transform().dy() + moveRectY, \
+        changeComposerItem->setSceneRect( QRectF( originalItem->transform().dx() + moveRectX,
+                                          originalItem->transform().dy() + moveRectY,
                                           originalItem->rect().width(), originalItem->rect().height() ) );
       }
     }
@@ -550,7 +585,7 @@ void QgsComposerItem::changeItemRectangle( const QPointF& currentPosition, const
   }
   else //for composer items, we prefer setSceneRect as subclasses can implement custom behaviour (e.g. item group)
   {
-    changeComposerItem->setSceneRect( QRectF( originalItem->transform().dx() + mx, originalItem->transform().dy() + my, \
+    changeComposerItem->setSceneRect( QRectF( originalItem->transform().dx() + mx, originalItem->transform().dy() + my,
                                       originalItem->rect().width() + rx, originalItem->rect().height() + ry ) );
   }
 }
@@ -617,7 +652,11 @@ void QgsComposerItem::setItemPosition( double x, double y, ItemPositionMode item
 {
   double width = rect().width();
   double height = rect().height();
+  setItemPosition( x, y, width, height, itemPoint );
+}
 
+void QgsComposerItem::setItemPosition( double x, double y, double width, double height, ItemPositionMode itemPoint )
+{
   double upperLeftX = x;
   double upperLeftY = y;
 
@@ -706,7 +745,7 @@ void QgsComposerItem::drawText( QPainter* p, double x, double y, const QString&
   p->restore();
 }
 
-void QgsComposerItem::drawText( QPainter* p, const QRectF& rect, const QString& text, const QFont& font ) const
+void QgsComposerItem::drawText( QPainter* p, const QRectF& rect, const QString& text, const QFont& font, Qt::AlignmentFlag halignement, Qt::AlignmentFlag valignment ) const
 {
   QFont textFont = scaledFontPixelSize( font );
 
@@ -717,7 +756,43 @@ void QgsComposerItem::drawText( QPainter* p, const QRectF& rect, const QString&
   p->setFont( textFont );
   double scaleFactor = 1.0 / FONT_WORKAROUND_SCALE;
   p->scale( scaleFactor, scaleFactor );
-  p->drawText( scaledRect, Qt::AlignLeft | Qt::AlignTop | Qt::TextWordWrap, text );
+  p->drawText( scaledRect, halignement | valignment | Qt::TextWordWrap, text );
+  p->restore();
+}
+void QgsComposerItem::drawArrowHead( QPainter* p, double x, double y, double angle, double arrowHeadWidth ) const
+{
+  if ( !p )
+  {
+    return;
+  }
+  double angleRad = angle / 180.0 * M_PI;
+  QPointF middlePoint( x, y );
+  //rotate both arrow points
+  QPointF p1 = QPointF( -arrowHeadWidth / 2.0, arrowHeadWidth );
+  QPointF p2 = QPointF( arrowHeadWidth / 2.0, arrowHeadWidth );
+
+  QPointF p1Rotated, p2Rotated;
+  p1Rotated.setX( p1.x() * cos( angleRad ) + p1.y() * -sin( angleRad ) );
+  p1Rotated.setY( p1.x() * sin( angleRad ) + p1.y() * cos( angleRad ) );
+  p2Rotated.setX( p2.x() * cos( angleRad ) + p2.y() * -sin( angleRad ) );
+  p2Rotated.setY( p2.x() * sin( angleRad ) + p2.y() * cos( angleRad ) );
+
+  QPolygonF arrowHeadPoly;
+  arrowHeadPoly << middlePoint;
+  arrowHeadPoly << QPointF( middlePoint.x() + p1Rotated.x(), middlePoint.y() + p1Rotated.y() );
+  arrowHeadPoly << QPointF( middlePoint.x() + p2Rotated.x(), middlePoint.y() + p2Rotated.y() );
+
+  p->save();
+
+  QPen arrowPen = p->pen();
+  arrowPen.setJoinStyle( Qt::RoundJoin );
+  QBrush arrowBrush = p->brush();
+  arrowBrush.setStyle( Qt::SolidPattern );
+  p->setPen( arrowPen );
+  p->setBrush( arrowBrush );
+  arrowBrush.setStyle( Qt::SolidPattern );
+  p->drawPolygon( arrowHeadPoly );
+
   p->restore();
 }
 
@@ -728,6 +803,13 @@ double QgsComposerItem::textWidthMillimeters( const QFont& font, const QString&
   return ( fontMetrics.width( text ) / FONT_WORKAROUND_SCALE );
 }
 
+double QgsComposerItem::fontHeightCharacterMM( const QFont& font, const QChar& c ) const
+{
+  QFont metricsFont = scaledFontPixelSize( font );
+  QFontMetricsF fontMetrics( metricsFont );
+  return ( fontMetrics.boundingRect( c ).height() / FONT_WORKAROUND_SCALE );
+}
+
 double QgsComposerItem::fontAscentMillimeters( const QFont& font ) const
 {
   QFont metricsFont = scaledFontPixelSize( font );
@@ -748,6 +830,24 @@ QFont QgsComposerItem::scaledFontPixelSize( const QFont& font ) const
   return scaledFont;
 }
 
+double QgsComposerItem::angle( const QPointF& p1, const QPointF& p2 ) const
+{
+  double xDiff = p2.x() - p1.x();
+  double yDiff = p2.y() - p1.y();
+  double length = sqrt( xDiff * xDiff + yDiff * yDiff );
+  if ( length <= 0 )
+  {
+    return 0;
+  }
+
+  double angle = acos(( -yDiff * length ) / ( length * length ) ) * 180 / M_PI;
+  if ( xDiff < 0 )
+  {
+    return ( 360 - angle );
+  }
+  return angle;
+}
+
 double QgsComposerItem::horizontalViewScaleFactor() const
 {
   double result = -1;
@@ -821,7 +921,7 @@ void QgsComposerItem::setRotation( double r )
 
 bool QgsComposerItem::imageSizeConsideringRotation( double& width, double& height ) const
 {
-  if ( abs( mRotation ) <= 0 ) //width and height stays the same if there is no rotation
+  if ( qAbs( mRotation ) <= 0.0 ) //width and height stays the same if there is no rotation
   {
     return true;
   }
@@ -858,7 +958,6 @@ bool QgsComposerItem::imageSizeConsideringRotation( double& width, double& heigh
   //assume points 1 and 3 are on the rectangle boundaries. Calculate 2 and 4.
   double distM1 = sqrt(( x1 - midX ) * ( x1 - midX ) + ( y1 - midY ) * ( y1 - midY ) );
   QPointF p2 = pointOnLineWithDistance( QPointF( midX, midY ), QPointF( x2, y2 ), distM1 );
-  QPointF p4 = pointOnLineWithDistance( QPointF( midX, midY ), QPointF( x4, y4 ), distM1 );
 
   if ( p2.x() < width && p2.x() > 0 && p2.y() < height && p2.y() > 0 )
   {
@@ -995,3 +1094,8 @@ void QgsComposerItem::rotate( double angle, double& x, double& y ) const
   x = xRot;
   y = yRot;
 }
+
+void QgsComposerItem::repaint()
+{
+  update();
+}
