@@ -20,7 +20,8 @@
 #include <QDomElement>
 #include <QPainter>
 
-QgsComposerLabel::QgsComposerLabel( QgsComposition *composition ): QgsComposerItem( composition ), mMargin( 1.0 ), mFontColor( QColor( 0, 0, 0 ) )
+QgsComposerLabel::QgsComposerLabel( QgsComposition *composition ): QgsComposerItem( composition ), mMargin( 1.0 ), mFontColor( QColor( 0, 0, 0 ) ), \
+    mHAlignment( Qt::AlignLeft ), mVAlignment( Qt::AlignTop )
 {
   //default font size is 10 point
   mFont.setPointSizeF( 10 );
@@ -49,7 +50,7 @@ void QgsComposerLabel::paint( QPainter* painter, const QStyleOptionGraphicsItem*
                       rect().height() - 2 * penWidth - 2 * mMargin );
 
 
-  drawText( painter, painterRect, displayText(), mFont );
+  drawText( painter, painterRect, displayText(), mFont, mHAlignment, mVAlignment );
 
   drawFrame( painter );
   if ( isSelected() )
@@ -112,6 +113,8 @@ QFont QgsComposerLabel::font() const
 
 bool QgsComposerLabel::writeXML( QDomElement& elem, QDomDocument & doc ) const
 {
+  QString alignment;
+
   if ( elem.isNull() )
   {
     return false;
@@ -122,6 +125,10 @@ bool QgsComposerLabel::writeXML( QDomElement& elem, QDomDocument & doc ) const
   composerLabelElem.setAttribute( "labelText", mText );
   composerLabelElem.setAttribute( "margin", QString::number( mMargin ) );
 
+  composerLabelElem.setAttribute( "halign", mHAlignment );
+  composerLabelElem.setAttribute( "valign", mVAlignment );
+  composerLabelElem.setAttribute( "id", mId );
+
 
   //font
   QDomElement labelFontElem = doc.createElement( "LabelFont" );
@@ -141,6 +148,8 @@ bool QgsComposerLabel::writeXML( QDomElement& elem, QDomDocument & doc ) const
 
 bool QgsComposerLabel::readXML( const QDomElement& itemElem, const QDomDocument& doc )
 {
+  QString alignment;
+
   if ( itemElem.isNull() )
   {
     return false;
@@ -154,6 +163,15 @@ bool QgsComposerLabel::readXML( const QDomElement& itemElem, const QDomDocument&
   //margin
   mMargin = itemElem.attribute( "margin" ).toDouble();
 
+  //Horizontal alignment
+  mHAlignment = ( Qt::AlignmentFlag )( itemElem.attribute( "halign" ).toInt() );
+
+  //Vertical alignment
+  mVAlignment = ( Qt::AlignmentFlag )( itemElem.attribute( "valign" ).toInt() );
+
+  //id
+  mId = itemElem.attribute( "id", "" );
+
   //font
   QDomNodeList labelFontList = itemElem.elementsByTagName( "LabelFont" );
   if ( labelFontList.size() > 0 )
@@ -184,5 +202,6 @@ bool QgsComposerLabel::readXML( const QDomElement& itemElem, const QDomDocument&
     QDomElement composerItemElem = composerItemList.at( 0 ).toElement();
     _readXML( composerItemElem, doc );
   }
+  emit itemChanged();
   return true;
 }
