@@ -28,6 +28,9 @@ class CORE_EXPORT QgsComposerLabel: public QgsComposerItem
     QgsComposerLabel( QgsComposition *composition );
     ~QgsComposerLabel();
 
+    /** return correct graphics item type. Added in v1.7 */
+    virtual int type() const { return ComposerLabel; }
+
     /** \brief Reimplementation of QCanvasItem::paint*/
     void paint( QPainter* painter, const QStyleOptionGraphicsItem* itemStyle, QWidget* pWidget );
 
@@ -43,6 +46,10 @@ class CORE_EXPORT QgsComposerLabel: public QgsComposerItem
 
     QFont font() const;
     void setFont( const QFont& f );
+    Qt::AlignmentFlag vAlign() const { return mVAlignment; }
+    Qt::AlignmentFlag hAlign() const { return mHAlignment; }
+    void setHAlign( Qt::AlignmentFlag a ) {mHAlignment = a;}
+    void setVAlign( Qt::AlignmentFlag a ) {mVAlignment = a;}
     double margin() {return mMargin;}
     void setMargin( double m ) {mMargin = m;}
 
@@ -53,17 +60,26 @@ class CORE_EXPORT QgsComposerLabel: public QgsComposerItem
         @note: this function was added in version 1.4*/
     QColor fontColor() const {return mFontColor;}
 
-    /** stores state in Dom node
-       * @param node is Dom node corresponding to 'Composer' tag
-       * @param temp write template file
+    /** stores state in Dom element
+       * @param elem is Dom element corresponding to 'Composer' tag
+       * @param doc document
        */
     bool writeXML( QDomElement& elem, QDomDocument & doc ) const;
 
     /** sets state from Dom document
-       * @param node is Dom node corresponding to 'ComposerLabel' tag
+       * @param itemElem is Dom element corresponding to 'ComposerLabel' tag
+       * @param doc document
        */
     bool readXML( const QDomElement& itemElem, const QDomDocument& doc );
 
+    /**Get label identification number
+      @note this method was added in version 1.7*/
+    QString id() const { return mId; }
+
+    /**Set label identification number
+      @note this method was added in version 1.7*/
+    void setId( const QString& id ) { mId = id; }
+
   private:
     // Text
     QString mText;
@@ -77,6 +93,15 @@ class CORE_EXPORT QgsComposerLabel: public QgsComposerItem
     // Font color
     QColor mFontColor;
 
+    // Horizontal Alignment
+    Qt::AlignmentFlag mHAlignment;
+
+    // Vertical Alignment
+    Qt::AlignmentFlag mVAlignment;
+
+    // Label id (unique within the same composition)
+    QString mId;
+
     /**Replaces replace '$CURRENT_DATE<(FORMAT)>' with the current date (e.g. $CURRENT_DATE(d 'June' yyyy)*/
     void replaceDateText( QString& text ) const;
 };
