@@ -16,21 +16,31 @@
  ***************************************************************************/
 
 #include "qgscomposerlegend.h"
+#include "qgscomposerlegenditem.h"
+#include "qgscomposermap.h"
 #include "qgsmaplayer.h"
 #include "qgsmaplayerregistry.h"
 #include "qgsmaprenderer.h"
 #include "qgsrenderer.h" //for brush scaling
 #include "qgssymbol.h"
+#include "qgssymbolv2.h"
 #include <QDomDocument>
 #include <QDomElement>
 #include <QPainter>
 
-QgsComposerLegend::QgsComposerLegend( QgsComposition* composition ): QgsComposerItem( composition ), mTitle( tr( "Legend" ) ), mBoxSpace( 2 ), mLayerSpace( 3 ), mSymbolSpace( 2 ), mIconLabelSpace( 2 )
+QgsComposerLegend::QgsComposerLegend( QgsComposition* composition )
+    : QgsComposerItem( composition )
+    , mTitle( tr( "Legend" ) )
+    , mBoxSpace( 2 )
+    , mLayerSpace( 2 )
+    , mSymbolSpace( 2 )
+    , mIconLabelSpace( 2 ), mComposerMap( 0 )
 {
-  QStringList idList = layerIdList();
-  mLegendModel.setLayerSet( idList );
+  //QStringList idList = layerIdList();
+  //mLegendModel.setLayerSet( idList );
 
-  mTitleFont.setPointSizeF( 14.0 );
+  mTitleFont.setPointSizeF( 16.0 );
+  mGroupFont.setPointSizeF( 14.0 );
   mLayerFont.setPointSizeF( 12.0 );
   mItemFont.setPointSizeF( 12.0 );
 
@@ -41,7 +51,7 @@ QgsComposerLegend::QgsComposerLegend( QgsComposition* composition ): QgsComposer
   connect( &mLegendModel, SIGNAL( layersChanged() ), this, SLOT( synchronizeWithModel() ) );
 }
 
-QgsComposerLegend::QgsComposerLegend(): QgsComposerItem( 0 )
+QgsComposerLegend::QgsComposerLegend(): QgsComposerItem( 0 ), mComposerMap( 0 )
 {
 
 }
@@ -94,54 +104,42 @@ QSizeF QgsComposerLegend::paintAndDetermineSize( QPainter* painter )
 
   maxXCoord = 2 * mBoxSpace + textWidthMillimeters( mTitleFont, mTitle );
 
-  //draw only visible layer items
-  QgsMapRenderer* theMapRenderer = mComposition->mapRenderer();
-  QStringList visibleLayerIds;
-  if ( theMapRenderer )
-  {
-    visibleLayerIds = theMapRenderer->layerSet();
-  }
-
-
+  double currentItemMaxX = 0; //maximum x-coordinate for current item
   for ( int i = 0; i < numLayerItems; ++i )
   {
     currentLayerItem = rootItem->child( i );
-    if ( currentLayerItem )
+    QgsComposerLegendItem* currentLegendItem = dynamic_cast<QgsComposerLegendItem*>( currentLayerItem );
+    if ( currentLegendItem )
     {
-      QString currentLayerId = currentLayerItem->data().toString();
-      int opacity = 255;
-      QgsMapLayer* currentLayer = QgsMapLayerRegistry::instance()->mapLayer( currentLayerId );
-      if ( currentLayer )
+      QgsComposerLegendItem::ItemType type = currentLegendItem->itemType();
+      if ( type == QgsComposerLegendItem::GroupItem )
       {
-        opacity = currentLayer->getTransparency();
+        drawGroupItem( painter, dynamic_cast<QgsComposerGroupItem*>( currentLegendItem ), currentYCoordinate, currentItemMaxX );
+        maxXCoord = qMax( maxXCoord, currentItemMaxX );
       }
-
-      if ( visibleLayerIds.contains( currentLayerId ) )
+      else if ( type == QgsComposerLegendItem::LayerItem )
       {
-        //Let the user omit the layer title item by having an empty layer title string
-        if ( !currentLayerItem->text().isEmpty() )
-        {
-          currentYCoordinate += mLayerSpace;
-          currentYCoordinate += fontAscentMillimeters( mLayerFont );
-
-          //draw layer Item
-          if ( painter )
-          {
-            painter->setPen( QColor( 0, 0, 0 ) );
-            drawText( painter, mBoxSpace, currentYCoordinate, currentLayerItem->text(), mLayerFont );
-          }
-        }
-
-        maxXCoord = std::max( maxXCoord, 2 * mBoxSpace + textWidthMillimeters( mLayerFont, currentLayerItem->text() ) );
-
-        //and child items
-        drawLayerChildItems( painter, currentLayerItem, currentYCoordinate, maxXCoord, opacity );
+        drawLayerItem( painter, dynamic_cast<QgsComposerLayerItem*>( currentLegendItem ), currentYCoordinate, currentItemMaxX );
+        maxXCoord = qMax( maxXCoord, currentItemMaxX );
       }
     }
   }
 
   currentYCoordinate += mBoxSpace;
 
+  size.setHeight( currentYCoordinate );
+  size.setWidth( maxXCoord );
+
+  //adjust box if width or height is to small
+  if ( painter && currentYCoordinate > rect().height() )
+  {
+    setSceneRect( QRectF( transform().dx(), transform().dy(), rect().width(), currentYCoordinate ) );
+  }
+  if ( painter && maxXCoord > rect().width() )
+  {
+    setSceneRect( QRectF( transform().dx(), transform().dy(), maxXCoord, rect().height() ) );
+  }
+
   if ( painter )
   {
     painter->restore();
@@ -154,22 +152,87 @@ QSizeF QgsComposerLegend::paintAndDetermineSize( QPainter* painter )
     }
   }
 
-  size.setHeight( currentYCoordinate );
-  size.setWidth( maxXCoord );
-
-  //adjust box if width or height is to small
-  if ( painter && currentYCoordinate > rect().width() )
-  {
-    setSceneRect( QRectF( transform().dx(), transform().dy(), rect().width(), currentYCoordinate ) );
-  }
-  if ( painter && maxXCoord > rect().height() )
-  {
-    setSceneRect( QRectF( transform().dx(), transform().dy(), maxXCoord, rect().height() ) );
-  }
-
   return size;
 }
 
+void QgsComposerLegend::drawGroupItem( QPainter* p, QgsComposerGroupItem* groupItem, double& currentYCoord, double& maxXCoord )
+{
+  if ( !p || !groupItem )
+  {
+    return;
+  }
+
+  currentYCoord += mLayerSpace;
+  currentYCoord += fontAscentMillimeters( mGroupFont );
+
+  p->setPen( QColor( 0, 0, 0 ) );
+  drawText( p, mBoxSpace, currentYCoord, groupItem->text(), mGroupFont );
+
+  //maximum x-coordinate of current item
+  double currentMaxXCoord = 2 * mBoxSpace + textWidthMillimeters( mGroupFont, groupItem->text() );
+  maxXCoord = qMax( currentMaxXCoord, maxXCoord );
+
+  //children can be other group items or layer items
+  int numChildItems = groupItem->rowCount();
+  QStandardItem* currentChildItem = 0;
+
+  for ( int i = 0; i < numChildItems; ++i )
+  {
+    currentChildItem = groupItem->child( i );
+    QgsComposerLegendItem* currentLegendItem = dynamic_cast<QgsComposerLegendItem*>( currentChildItem );
+    QgsComposerLegendItem::ItemType type = currentLegendItem->itemType();
+    if ( type == QgsComposerLegendItem::GroupItem )
+    {
+      drawGroupItem( p, dynamic_cast<QgsComposerGroupItem*>( currentLegendItem ), currentYCoord, currentMaxXCoord );
+      maxXCoord = qMax( currentMaxXCoord, maxXCoord );
+    }
+    else if ( type == QgsComposerLegendItem::LayerItem )
+    {
+      drawLayerItem( p, dynamic_cast<QgsComposerLayerItem*>( currentLegendItem ), currentYCoord, currentMaxXCoord );
+      maxXCoord = qMax( currentMaxXCoord, maxXCoord );
+    }
+  }
+}
+
+void QgsComposerLegend::drawLayerItem( QPainter* p, QgsComposerLayerItem* layerItem, double& currentYCoord, double& maxXCoord )
+{
+  if ( !layerItem )
+  {
+    return;
+  }
+
+  int opacity = 255;
+  QgsMapLayer* currentLayer = QgsMapLayerRegistry::instance()->mapLayer( layerItem->layerID() );
+  if ( currentLayer )
+  {
+    opacity = currentLayer->getTransparency();
+  }
+
+  //Let the user omit the layer title item by having an empty layer title string
+  if ( !layerItem->text().isEmpty() )
+  {
+    currentYCoord += mLayerSpace;
+    currentYCoord += fontAscentMillimeters( mLayerFont );
+
+    //draw layer Item
+    if ( p )
+    {
+      p->setPen( QColor( 0, 0, 0 ) );
+      drawText( p, mBoxSpace, currentYCoord, layerItem->text(), mLayerFont );
+    }
+
+    maxXCoord = qMax( maxXCoord, 2 * mBoxSpace + textWidthMillimeters( mLayerFont, layerItem->text() ) );
+  }
+  else //layer title omited
+  {
+    //symbol space will be added before the item later
+    currentYCoord += ( mLayerSpace - mSymbolSpace );
+  }
+
+  //and child items
+  drawLayerChildItems( p, layerItem, currentYCoord, maxXCoord, opacity );
+}
+
 void QgsComposerLegend::adjustBoxSize()
 {
   QSizeF size = paintAndDetermineSize( 0 );
@@ -186,8 +249,14 @@ void QgsComposerLegend::drawLayerChildItems( QPainter* p, QStandardItem* layerIt
     return;
   }
 
-  //standerd item height
-  double itemHeight = std::max( mSymbolHeight, fontAscentMillimeters( mItemFont ) );
+  //Draw all symbols first and the texts after (to find out the x coordinate to have the text aligned)
+  QList<double> childYCoords;
+  QList<double> realItemHeights;
+
+  double textHeight = fontHeightCharacterMM( mItemFont, QChar( '0' ) );
+  double itemHeight = qMax( mSymbolHeight, textHeight );
+
+  double textAlignCoord = 0; //alignment for legend text
 
   QStandardItem* currentItem;
 
@@ -209,20 +278,42 @@ void QgsComposerLegend::drawLayerChildItems( QPainter* p, QStandardItem* layerIt
       continue;
     }
 
-    //take QgsSymbol* from user data
-    QVariant symbolVariant = currentItem->data();
     QgsSymbol* symbol = 0;
-    if ( symbolVariant.canConvert<void*>() )
+    QgsComposerSymbolItem* symbolItem = dynamic_cast<QgsComposerSymbolItem*>( currentItem );
+    if ( symbolItem )
     {
-      void* symbolData = symbolVariant.value<void*>();
-      symbol = ( QgsSymbol* )( symbolData );
+      symbol = symbolItem->symbol();
     }
 
+    QgsSymbolV2* symbolNg = 0;
+    QgsComposerSymbolV2Item* symbolV2Item = dynamic_cast<QgsComposerSymbolV2Item*>( currentItem );
+    if ( symbolV2Item )
+    {
+      symbolNg = symbolV2Item->symbolV2();
+    }
+    QgsComposerRasterSymbolItem* rasterItem = dynamic_cast<QgsComposerRasterSymbolItem*>( currentItem );
+
     if ( symbol )  //item with symbol?
     {
       //draw symbol
       drawSymbol( p, symbol, currentYCoord + ( itemHeight - mSymbolHeight ) / 2, currentXCoord, realSymbolHeight, layerOpacity );
-      realItemHeight = std::max( realSymbolHeight, itemHeight );
+      realItemHeight = qMax( realSymbolHeight, itemHeight );
+      currentXCoord += mIconLabelSpace;
+    }
+    else if ( symbolNg ) //item with symbol NG?
+    {
+      drawSymbolV2( p, symbolNg, currentYCoord + ( itemHeight - mSymbolHeight ) / 2, currentXCoord, realSymbolHeight, layerOpacity );
+      realItemHeight = qMax( realSymbolHeight, itemHeight );
+      currentXCoord += mIconLabelSpace;
+    }
+    else if ( rasterItem )
+    {
+      if ( p )
+      {
+        p->setBrush( rasterItem->color() );
+        p->drawRect( QRectF( currentXCoord, currentYCoord + ( itemHeight - mSymbolHeight ) / 2, mSymbolWidth, mSymbolHeight ) );
+      }
+      currentXCoord += mSymbolWidth;
       currentXCoord += mIconLabelSpace;
     }
     else //item with icon?
@@ -230,22 +321,27 @@ void QgsComposerLegend::drawLayerChildItems( QPainter* p, QStandardItem* layerIt
       QIcon symbolIcon = currentItem->icon();
       if ( !symbolIcon.isNull() && p )
       {
-        symbolIcon.paint( p, currentXCoord, currentYCoord, mSymbolWidth, mSymbolHeight );
+        symbolIcon.paint( p, currentXCoord, currentYCoord + ( itemHeight - mSymbolHeight ) / 2, mSymbolWidth, mSymbolHeight );
         currentXCoord += mSymbolWidth;
         currentXCoord += mIconLabelSpace;
       }
     }
 
-    //finally draw text
+    childYCoords.push_back( currentYCoord );
+    realItemHeights.push_back( realItemHeight );
+    currentYCoord += realItemHeight;
+    textAlignCoord = qMax( currentXCoord, textAlignCoord );
+  }
+
+  maxXCoord = qMax( maxXCoord, textAlignCoord );
+  for ( int i = 0; i < numChildren; ++i )
+  {
     if ( p )
     {
       p->setPen( QColor( 0, 0, 0 ) );
-      drawText( p, currentXCoord, currentYCoord + fontAscentMillimeters( mItemFont ) + ( realItemHeight - fontAscentMillimeters( mItemFont ) ) / 2, currentItem->text(), mItemFont );
+      drawText( p, textAlignCoord, childYCoords.at( i ) + textHeight + ( realItemHeights.at( i ) - textHeight ) / 2, layerItem->child( i, 0 )->text(), mItemFont );
+      maxXCoord = qMax( maxXCoord, textAlignCoord + mBoxSpace + textWidthMillimeters( mItemFont,  layerItem->child( i, 0 )->text() ) );
     }
-
-    maxXCoord = std::max( maxXCoord, currentXCoord + textWidthMillimeters( mItemFont, currentItem->text() ) + mBoxSpace );
-
-    currentYCoord += realItemHeight;
   }
 }
 
@@ -271,11 +367,90 @@ void QgsComposerLegend::drawSymbol( QPainter* p, QgsSymbol* s, double currentYCo
       symbolHeight = mSymbolHeight;
       break;
     case QGis::UnknownGeometry:
+    case QGis::NoGeometry:
       // shouldn't occur
       break;
   }
 }
 
+void QgsComposerLegend::drawSymbolV2( QPainter* p, QgsSymbolV2* s, double currentYCoord, double& currentXPosition, double& symbolHeight, int layerOpacity ) const
+{
+  if ( !p || !s )
+  {
+    return;
+  }
+
+  double rasterScaleFactor = 1.0;
+  if ( p )
+  {
+    QPaintDevice* paintDevice = p->device();
+    if ( !paintDevice )
+    {
+      return;
+    }
+    rasterScaleFactor = ( paintDevice->logicalDpiX() + paintDevice->logicalDpiY() ) / 2.0 / 25.4;
+  }
+
+  //consider relation to composer map for symbol sizes in mm
+  bool sizeInMapUnits = s->outputUnit() == QgsSymbolV2::MapUnit;
+  double mmPerMapUnit = 1;
+  if ( mComposerMap )
+  {
+    mmPerMapUnit = mComposerMap->mapUnitsToMM();
+  }
+  QgsMarkerSymbolV2* markerSymbol = dynamic_cast<QgsMarkerSymbolV2*>( s );
+
+  //Consider symbol size for point markers
+  double height = mSymbolHeight;
+  double width = mSymbolWidth;
+  double size = 0;
+  //Center small marker symbols
+  double widthOffset = 0;
+  double heightOffset = 0;
+
+  if ( markerSymbol )
+  {
+    size = markerSymbol->size();
+    height = size;
+    width = size;
+    if ( mComposerMap && sizeInMapUnits )
+    {
+      height *= mmPerMapUnit;
+      width *= mmPerMapUnit;
+      markerSymbol->setSize( width );
+    }
+    if ( width < mSymbolWidth )
+    {
+      widthOffset = ( mSymbolWidth - width ) / 2.0;
+    }
+    if ( height < mSymbolHeight )
+    {
+      heightOffset = ( mSymbolHeight - height ) / 2.0;
+    }
+  }
+
+  p->save();
+  p->translate( currentXPosition + widthOffset, currentYCoord + heightOffset );
+  p->scale( 1.0 / rasterScaleFactor, 1.0 / rasterScaleFactor );
+
+  if ( markerSymbol && sizeInMapUnits )
+  {
+    s->setOutputUnit( QgsSymbolV2::MM );
+  }
+  s->drawPreviewIcon( p, QSize( width * rasterScaleFactor, height * rasterScaleFactor ) );
+
+  if ( markerSymbol && sizeInMapUnits )
+  {
+    s->setOutputUnit( QgsSymbolV2::MapUnit );
+    markerSymbol->setSize( size );
+  }
+
+  p->restore();
+  currentXPosition += width;
+  currentXPosition += 2 * widthOffset;
+  symbolHeight = height + 2 * heightOffset;
+}
+
 void QgsComposerLegend::drawPointSymbol( QPainter* p, QgsSymbol* s, double currentYCoord, double& currentXPosition, double& symbolHeight, int opacity ) const
 {
   if ( !s )
@@ -374,16 +549,16 @@ void QgsComposerLegend::drawPolygonSymbol( QPainter* p, QgsSymbol* s, double cur
 
 QStringList QgsComposerLegend::layerIdList() const
 {
-  QStringList layerIdList;
-  QMap<QString, QgsMapLayer*> layerMap =  QgsMapLayerRegistry::instance()->mapLayers();
-  QMap<QString, QgsMapLayer*>::const_iterator mapIt = layerMap.constBegin();
-
-  for ( ; mapIt != layerMap.constEnd(); ++mapIt )
+  //take layer list from map renderer (to have legend order)
+  if ( mComposition )
   {
-    layerIdList.push_back( mapIt.key() );
+    QgsMapRenderer* r = mComposition->mapRenderer();
+    if ( r )
+    {
+      return r->layerSet();
+    }
   }
-
-  return layerIdList;
+  return QStringList();
 }
 
 void QgsComposerLegend::synchronizeWithModel()
@@ -399,6 +574,13 @@ void QgsComposerLegend::setTitleFont( const QFont& f )
   update();
 }
 
+void QgsComposerLegend::setGroupFont( const QFont& f )
+{
+  mGroupFont = f;
+  adjustBoxSize();
+  update();
+}
+
 void QgsComposerLegend::setLayerFont( const QFont& f )
 {
   mLayerFont = f;
@@ -418,6 +600,11 @@ QFont QgsComposerLegend::titleFont() const
   return mTitleFont;
 }
 
+QFont QgsComposerLegend::groupFont() const
+{
+  return mGroupFont;
+}
+
 QFont QgsComposerLegend::layerFont() const
 {
   return mLayerFont;
@@ -447,6 +634,7 @@ bool QgsComposerLegend::writeXML( QDomElement& elem, QDomDocument & doc ) const
   //write general properties
   composerLegendElem.setAttribute( "title", mTitle );
   composerLegendElem.setAttribute( "titleFont", mTitleFont.toString() );
+  composerLegendElem.setAttribute( "groupFont", mGroupFont.toString() );
   composerLegendElem.setAttribute( "layerFont", mLayerFont.toString() );
   composerLegendElem.setAttribute( "itemFont", mItemFont.toString() );
   composerLegendElem.setAttribute( "boxSpace", QString::number( mBoxSpace ) );
@@ -456,6 +644,11 @@ bool QgsComposerLegend::writeXML( QDomElement& elem, QDomDocument & doc ) const
   composerLegendElem.setAttribute( "symbolWidth", mSymbolWidth );
   composerLegendElem.setAttribute( "symbolHeight", mSymbolHeight );
 
+  if ( mComposerMap )
+  {
+    composerLegendElem.setAttribute( "map", mComposerMap->id() );
+  }
+
   //write model properties
   mLegendModel.writeXML( composerLegendElem, doc );
 
@@ -478,6 +671,13 @@ bool QgsComposerLegend::readXML( const QDomElement& itemElem, const QDomDocument
   {
     mTitleFont.fromString( titleFontString );
   }
+  //group font
+  QString groupFontString = itemElem.attribute( "groupFont" );
+  if ( !groupFontString.isEmpty() )
+  {
+    mGroupFont.fromString( groupFontString );
+  }
+
   //layer font
   QString layerFontString = itemElem.attribute( "layerFont" );
   if ( !layerFontString.isEmpty() )
@@ -499,12 +699,17 @@ bool QgsComposerLegend::readXML( const QDomElement& itemElem, const QDomDocument
   mSymbolWidth = itemElem.attribute( "symbolWidth", "7.0" ).toDouble();
   mSymbolHeight = itemElem.attribute( "symbolHeight", "14.0" ).toDouble();
 
+  //composer map
+  if ( !itemElem.attribute( "map" ).isEmpty() )
+  {
+    mComposerMap = mComposition->getComposerMapById( itemElem.attribute( "map" ).toInt() );
+  }
+
   //read model properties
   QDomNodeList modelNodeList = itemElem.elementsByTagName( "Model" );
   if ( modelNodeList.size() > 0 )
   {
     QDomElement modelElem = modelNodeList.at( 0 ).toElement();
-    mLegendModel.clear();
     mLegendModel.readXML( modelElem, doc );
   }
 
@@ -516,5 +721,18 @@ bool QgsComposerLegend::readXML( const QDomElement& itemElem, const QDomDocument
     _readXML( composerItemElem, doc );
   }
 
+  emit itemChanged();
   return true;
 }
+
+void QgsComposerLegend::setComposerMap( const QgsComposerMap* map )
+{
+  mComposerMap = map;
+  QObject::connect( map, SIGNAL( destroyed( QObject* ) ), this, SLOT( invalidateCurrentMap() ) );
+}
+
+void QgsComposerLegend::invalidateCurrentMap()
+{
+  disconnect( mComposerMap, SIGNAL( destroyed( QObject* ) ), this, SLOT( invalidateCurrentMap() ) );
+  mComposerMap = 0;
+}
