@@ -24,7 +24,6 @@
 #include "qgsmaplayerregistry.h"
 #include "qgsmaptopixel.h"
 #include "qgsproject.h"
-#include "qgsmaprenderer.h"
 #include "qgsrasterlayer.h"
 #include "qgsrendercontext.h"
 #include "qgsscalecalculator.h"
@@ -34,6 +33,7 @@
 #include "qgslabelattributes.h"
 
 #include <QGraphicsScene>
+#include <QGraphicsView>
 #include <QPainter>
 #include <QSettings>
 #include <iostream>
@@ -42,10 +42,24 @@
 QgsComposerMap::QgsComposerMap( QgsComposition *composition, int x, int y, int width, int height )
     : QgsComposerItem( x, y, width, height, composition ), mKeepLayerSet( false ), mGridEnabled( false ), mGridStyle( Solid ), \
     mGridIntervalX( 0.0 ), mGridIntervalY( 0.0 ), mGridOffsetX( 0.0 ), mGridOffsetY( 0.0 ), mGridAnnotationPrecision( 3 ), mShowGridAnnotation( false ), \
-    mGridAnnotationPosition( OutsideMapFrame ), mAnnotationFrameDistance( 1.0 ), mGridAnnotationDirection( Horizontal ), mCrossLength( 3 )
+    mGridAnnotationPosition( OutsideMapFrame ), mAnnotationFrameDistance( 1.0 ), mGridAnnotationDirection( Horizontal ),
+    mCrossLength( 3 ), mMapCanvas( 0 ), mDrawCanvasItems( true )
 {
   mComposition = composition;
-  mId = mComposition->composerMapItems().size();
+
+  //mId = mComposition->composerMapItems().size();
+  int maxId = -1;
+  QList<const QgsComposerMap*> mapList = mComposition->composerMapItems();
+  QList<const QgsComposerMap*>::const_iterator mapIt = mapList.constBegin();
+  for ( ; mapIt != mapList.constEnd(); ++mapIt )
+  {
+    if (( *mapIt )->id() > maxId )
+    {
+      maxId = ( *mapIt )->id();
+    }
+  }
+  mId = maxId + 1;
+
   mMapRenderer = mComposition->mapRenderer();
   mPreviewMode = QgsComposerMap::Rectangle;
   mCurrentRectangle = rect();
@@ -73,7 +87,8 @@ QgsComposerMap::QgsComposerMap( QgsComposition *composition, int x, int y, int w
 QgsComposerMap::QgsComposerMap( QgsComposition *composition )
     : QgsComposerItem( 0, 0, 10, 10, composition ), mKeepLayerSet( false ), mGridEnabled( false ), mGridStyle( Solid ), \
     mGridIntervalX( 0.0 ), mGridIntervalY( 0.0 ), mGridOffsetX( 0.0 ), mGridOffsetY( 0.0 ), mGridAnnotationPrecision( 3 ), mShowGridAnnotation( false ), \
-    mGridAnnotationPosition( OutsideMapFrame ), mAnnotationFrameDistance( 1.0 ), mGridAnnotationDirection( Horizontal ), mCrossLength( 3 )
+    mGridAnnotationPosition( OutsideMapFrame ), mAnnotationFrameDistance( 1.0 ), mGridAnnotationDirection( Horizontal ), mCrossLength( 3 ),
+    mMapCanvas( 0 ), mDrawCanvasItems( true )
 {
   //Offset
   mXOffset = 0.0;
@@ -95,9 +110,14 @@ QgsComposerMap::~QgsComposerMap()
 {
 }
 
+void QgsComposerMap::draw( QPainter *painter, const QgsRectangle& extent, const QSize& size, int dpi )
+{
+  draw( painter, extent, QSizeF( size.width(), size.height() ), dpi );
+}
+
 /* This function is called by paint() and cache() to render the map.  It does not override any functions
 from QGraphicsItem. */
-void QgsComposerMap::draw( QPainter *painter, const QgsRectangle& extent, const QSize& size, int dpi )
+void QgsComposerMap::draw( QPainter *painter, const QgsRectangle& extent, const QSizeF& size, double dpi )
 {
   if ( !painter )
   {
@@ -112,6 +132,8 @@ void QgsComposerMap::draw( QPainter *painter, const QgsRectangle& extent, const
   QgsMapRenderer theMapRenderer;
   theMapRenderer.setExtent( extent );
   theMapRenderer.setOutputSize( size, dpi );
+  if ( mMapRenderer->labelingEngine() )
+    theMapRenderer.setLabelingEngine( mMapRenderer->labelingEngine()->clone() );
 
   //use stored layer set or read current set from main canvas
   if ( mKeepLayerSet )
@@ -123,11 +145,12 @@ void QgsComposerMap::draw( QPainter *painter, const QgsRectangle& extent, const
     theMapRenderer.setLayerSet( mMapRenderer->layerSet() );
   }
   theMapRenderer.setProjectionsEnabled( mMapRenderer->hasCrsTransformEnabled() );
-  theMapRenderer.setDestinationSrs( mMapRenderer->destinationSrs() );
+  theMapRenderer.setDestinationCrs( mMapRenderer->destinationCrs() );
 
   //set antialiasing if enabled in options
   QSettings settings;
-  if ( settings.value( "/qgis/enable_anti_aliasing", false ).toBool() )
+  // Changed to enable anti aliased rendering by default as of QGIS 1.7
+  if ( settings.value( "/qgis/enable_anti_aliasing", true ).toBool() )
   {
     painter->setRenderHint( QPainter::Antialiasing );
   }
@@ -139,6 +162,9 @@ void QgsComposerMap::draw( QPainter *painter, const QgsRectangle& extent, const
     theRendererContext->setRenderingStopped( false );
   }
 
+  // force vector output (no caching of marker images etc.)
+  theRendererContext->setForceVectorOutput( true );
+
   //force composer map scale for scale dependent visibility
   double bk_scale = theMapRenderer.scale();
   theMapRenderer.setScale( scale() );
@@ -200,7 +226,7 @@ void QgsComposerMap::cache( void )
 
   QPainter p( &mCacheImage );
 
-  draw( &p, requestExtent, QSize( w, h ), mCacheImage.logicalDpiX() );
+  draw( &p, requestExtent, QSizeF( w, h ), mCacheImage.logicalDpiX() );
   p.end();
   mCacheUpdated = true;
 
@@ -255,14 +281,19 @@ void QgsComposerMap::paint( QPainter* painter, const QStyleOptionGraphicsItem* i
     double yTopLeftShift = ( mExtent.yMaximum() - rotationPoint.y() ) * mapUnitsToMM();
 
     painter->save();
-    //painter->scale( scale, scale );
+
     painter->translate( mXOffset, mYOffset );
     painter->translate( xTopLeftShift, yTopLeftShift );
     painter->rotate( mRotation );
     painter->translate( xShiftMM, -yShiftMM );
     painter->scale( scale, scale );
     painter->drawImage( 0, 0, mCacheImage );
+
+    //restore rotation
     painter->restore();
+
+    //draw canvas items
+    drawCanvasItems( painter, itemStyle );
   }
   else if ( mComposition->plotStyle() == QgsComposition::Print ||
             mComposition->plotStyle() == QgsComposition::Postscript )
@@ -282,7 +313,7 @@ void QgsComposerMap::paint( QPainter* painter, const QStyleOptionGraphicsItem* i
     QgsRectangle requestRectangle;
     requestedExtent( requestRectangle );
 
-    QSize theSize( requestRectangle.width() * mapUnitsToMM(), requestRectangle.height() * mapUnitsToMM() );
+    QSizeF theSize( requestRectangle.width() * mapUnitsToMM(), requestRectangle.height() * mapUnitsToMM() );
     QgsPoint rotationPoint = QgsPoint(( mExtent.xMaximum() + mExtent.xMinimum() ) / 2.0, ( mExtent.yMaximum() + mExtent.yMinimum() ) / 2.0 );
 
     //shift such that rotation point is at 0/0 point in the coordinate system
@@ -298,8 +329,13 @@ void QgsComposerMap::paint( QPainter* painter, const QStyleOptionGraphicsItem* i
     painter->rotate( mRotation );
     painter->translate( xShiftMM, -yShiftMM );
     draw( painter, requestRectangle, theSize, 25.4 ); //scene coordinates seem to be in mm
+
+    //restore rotation
     painter->restore();
 
+    //draw canvas items
+    drawCanvasItems( painter, itemStyle );
+
     mDrawing = false;
   }
 
@@ -321,6 +357,11 @@ void QgsComposerMap::paint( QPainter* painter, const QStyleOptionGraphicsItem* i
 
 void QgsComposerMap::updateCachedImage( void )
 {
+  //If we have a locked layer set then don't reload the map canvas.
+  if ( mKeepLayerSet )
+  {
+    return;
+  }
   syncLayerSet(); //layer list may have changed
   mCacheUpdated = false;
   cache();
@@ -365,9 +406,10 @@ void QgsComposerMap::moveContent( double dx, double dy )
     mExtent.setXMaximum( mExtent.xMaximum() + dx );
     mExtent.setYMinimum( mExtent.yMinimum() + dy );
     mExtent.setYMaximum( mExtent.yMaximum() + dy );
-    emit extentChanged();
     cache();
     update();
+    emit itemChanged();
+    emit extentChanged();
   }
 }
 
@@ -382,7 +424,7 @@ void QgsComposerMap::zoomContent( int delta, double x, double y )
 
   //read zoom mode
   //0: zoom, 1: zoom and recenter, 2: zoom to cursor, 3: nothing
-  int zoomMode = settings.value( "/qgis/wheel_action", 0 ).toInt();
+  int zoomMode = settings.value( "/qgis/wheel_action", 2 ).toInt();
   if ( zoomMode == 3 ) //do nothing
   {
     return;
@@ -433,9 +475,10 @@ void QgsComposerMap::zoomContent( int delta, double x, double y )
   mExtent.setYMaximum( centerY + newIntervalY / 2 );
   mExtent.setYMinimum( centerY - newIntervalY / 2 );
 
-  emit extentChanged();
   cache();
   update();
+  emit itemChanged();
+  emit extentChanged();
 }
 
 void QgsComposerMap::setSceneRect( const QRectF& rectangle )
@@ -450,13 +493,15 @@ void QgsComposerMap::setSceneRect( const QRectF& rectangle )
   double newHeight = mExtent.width() * h / w ;
   mExtent = QgsRectangle( mExtent.xMinimum(), mExtent.yMinimum(), mExtent.xMaximum(), mExtent.yMinimum() + newHeight );
   mCacheUpdated = false;
-  emit extentChanged();
+
   if ( mPreviewMode != Rectangle )
   {
     cache();
   }
   updateBoundingRect();
   update();
+  emit itemChanged();
+  emit extentChanged();
 }
 
 void QgsComposerMap::setNewExtent( const QgsRectangle& extent )
@@ -487,9 +532,10 @@ void QgsComposerMap::setNewScale( double scaleDenominator )
   double scaleRatio = scaleDenominator / currentScaleDenominator;
   mExtent.scale( scaleRatio );
   mCacheUpdated = false;
-  emit extentChanged();
   cache();
   update();
+  emit itemChanged();
+  emit extentChanged();
 }
 
 void QgsComposerMap::setOffset( double xOffset, double yOffset )
@@ -582,12 +628,21 @@ bool QgsComposerMap::writeXML( QDomElement& elem, QDomDocument & doc ) const
     composerMapElem.setAttribute( "keepLayerSet", "false" );
   }
 
+  if ( mDrawCanvasItems )
+  {
+    composerMapElem.setAttribute( "drawCanvasItems", "true" );
+  }
+  else
+  {
+    composerMapElem.setAttribute( "drawCanvasItems", "false" );
+  }
+
   //extent
   QDomElement extentElem = doc.createElement( "Extent" );
-  extentElem.setAttribute( "xmin", QString::number( mExtent.xMinimum() ) );
-  extentElem.setAttribute( "xmax", QString::number( mExtent.xMaximum() ) );
-  extentElem.setAttribute( "ymin", QString::number( mExtent.yMinimum() ) );
-  extentElem.setAttribute( "ymax", QString::number( mExtent.yMaximum() ) );
+  extentElem.setAttribute( "xmin", mExtent.xMinimum() );
+  extentElem.setAttribute( "xmax", mExtent.xMaximum() );
+  extentElem.setAttribute( "ymin", mExtent.yMinimum() );
+  extentElem.setAttribute( "ymax", mExtent.yMaximum() );
   composerMapElem.appendChild( extentElem );
 
   //layer set
@@ -686,6 +741,16 @@ bool QgsComposerMap::readXML( const QDomElement& itemElem, const QDomDocument& d
     mKeepLayerSet = false;
   }
 
+  QString drawCanvasItemsFlag = itemElem.attribute( "drawCanvasItems" );
+  if ( drawCanvasItemsFlag.compare( "true", Qt::CaseInsensitive ) == 0 )
+  {
+    mDrawCanvasItems = true;
+  }
+  else
+  {
+    mDrawCanvasItems = false;
+  }
+
   //mLayerSet
   QDomNodeList layerSetNodeList = itemElem.elementsByTagName( "LayerSet" );
   QStringList layerSet;
@@ -743,6 +808,7 @@ bool QgsComposerMap::readXML( const QDomElement& itemElem, const QDomDocument& d
   }
 
   updateBoundingRect();
+  emit itemChanged();
   return true;
 }
 
@@ -761,7 +827,17 @@ void QgsComposerMap::syncLayerSet()
     return;
   }
 
-  QStringList currentLayerSet = mMapRenderer->layerSet();
+  //if layer set is fixed, do a lookup in the layer registry to also find the non-visible layers
+  QStringList currentLayerSet;
+  if ( mKeepLayerSet )
+  {
+    currentLayerSet = QgsMapLayerRegistry::instance()->mapLayers().uniqueKeys();
+  }
+  else //only consider layers visible in the map
+  {
+    currentLayerSet = mMapRenderer->layerSet();
+  }
+
   for ( int i = mLayerSet.size() - 1; i >= 0; --i )
   {
     if ( !currentLayerSet.contains( mLayerSet.at( i ) ) )
@@ -886,7 +962,8 @@ void QgsComposerMap::drawCoordinateAnnotation( QPainter* p, const QPointF& pos,
 {
   Border frameBorder = borderForLineCoord( pos );
   double textWidth = textWidthMillimeters( mGridAnnotationFont, annotationString );
-  double textHeight = fontAscentMillimeters( mGridAnnotationFont );
+  //relevant for annotations is the height of digits
+  double textHeight = fontHeightCharacterMM( mGridAnnotationFont, QChar( '0' ) );
   double xpos = pos.x();
   double ypos = pos.y();
   int rotation = 0;
@@ -898,7 +975,7 @@ void QgsComposerMap::drawCoordinateAnnotation( QPainter* p, const QPointF& pos,
     {
       if ( mGridAnnotationDirection == Vertical || mGridAnnotationDirection == BoundaryDirection )
       {
-        xpos -= textHeight + mAnnotationFrameDistance;
+        xpos += textHeight + mAnnotationFrameDistance;
         ypos += textWidth / 2.0;
         rotation = 270;
       }
@@ -1039,11 +1116,15 @@ int QgsComposerMap::xGridLines( QList< QPair< double, QLineF > >& lines ) const
     return 1;
   }
 
+
   QPolygonF mapPolygon = transformedMapPolygon();
   QRectF mapBoundingRect = mapPolygon.boundingRect();
-  double currentLevel = ( int )(( mapBoundingRect.top() - mGridOffsetY ) / mGridIntervalY + 1.0 ) * mGridIntervalY + mGridOffsetY;
 
-  if ( mRotation <= 0.0 )
+  //consider to round up to the next step in case the left boundary is > 0
+  double roundCorrection = mapBoundingRect.top() > 0 ? 1.0 : 0.0;
+  double currentLevel = ( int )(( mapBoundingRect.top() - mGridOffsetY ) / mGridIntervalY + roundCorrection ) * mGridIntervalY + mGridOffsetY;
+
+  if ( doubleNear( mRotation, 0.0 ) )
   {
     //no rotation. Do it 'the easy way'
 
@@ -1106,9 +1187,12 @@ int QgsComposerMap::yGridLines( QList< QPair< double, QLineF > >& lines ) const
 
   QPolygonF mapPolygon = transformedMapPolygon();
   QRectF mapBoundingRect = mapPolygon.boundingRect();
-  double currentLevel = ( int )(( mapBoundingRect.left() - mGridOffsetX ) / mGridIntervalX + 1.0 ) * mGridIntervalX + mGridOffsetX;
 
-  if ( mRotation <= 0.0 )
+  //consider to round up to the next step in case the left boundary is > 0
+  double roundCorrection = mapBoundingRect.left() > 0 ? 1.0 : 0.0;
+  double currentLevel = ( int )(( mapBoundingRect.left() - mGridOffsetX ) / mGridIntervalX + roundCorrection ) * mGridIntervalX + mGridOffsetX;
+
+  if ( doubleNear( mRotation, 0.0 ) )
   {
     //no rotation. Do it 'the easy way'
     double xCanvasCoord;
@@ -1242,16 +1326,16 @@ double QgsComposerMap::maxExtension() const
   for ( ; it != xLines.constEnd(); ++it )
   {
     currentAnnotationString = QString::number( it->first, 'f', mGridAnnotationPrecision );
-    currentExtension = std::max( textWidthMillimeters( mGridAnnotationFont, currentAnnotationString ), fontAscentMillimeters( mGridAnnotationFont ) );
-    maxExtension = std::max( maxExtension, currentExtension );
+    currentExtension = qMax( textWidthMillimeters( mGridAnnotationFont, currentAnnotationString ), fontAscentMillimeters( mGridAnnotationFont ) );
+    maxExtension = qMax( maxExtension, currentExtension );
   }
 
   it = yLines.constBegin();
   for ( ; it != yLines.constEnd(); ++it )
   {
     currentAnnotationString = QString::number( it->first, 'f', mGridAnnotationPrecision );
-    currentExtension = std::max( textWidthMillimeters( mGridAnnotationFont, currentAnnotationString ), fontAscentMillimeters( mGridAnnotationFont ) );
-    maxExtension = std::max( maxExtension, currentExtension );
+    currentExtension = qMax( textWidthMillimeters( mGridAnnotationFont, currentAnnotationString ), fontAscentMillimeters( mGridAnnotationFont ) );
+    maxExtension = qMax( maxExtension, currentExtension );
   }
 
   return maxExtension + mAnnotationFrameDistance;
@@ -1378,3 +1462,132 @@ QgsComposerMap::Border QgsComposerMap::borderForLineCoord( const QPointF& p ) co
     return Bottom;
   }
 }
+
+void QgsComposerMap::drawCanvasItems( QPainter* painter, const QStyleOptionGraphicsItem* itemStyle )
+{
+  if ( !mMapCanvas || !mDrawCanvasItems )
+  {
+    return;
+  }
+
+  QList<QGraphicsItem*> itemList = mMapCanvas->items();
+  if ( itemList.size() < 1 )
+  {
+    return;
+  }
+  QGraphicsItem* currentItem = 0;
+
+#if QT_VERSION >= 0x40600 //Qt 4.6 provides the items in visibility order
+  for ( int i = itemList.size() - 1; i >= 0; --i )
+  {
+    currentItem = itemList.at( i );
+    //don't draw mapcanvasmap (has z value -10)
+    if ( !currentItem || currentItem->zValue() == -10 )
+    {
+      continue;
+    }
+    drawCanvasItem( currentItem, painter, itemStyle );
+  }
+#else //Qt <4.6 provides the items in random order
+  QMultiMap<int, QGraphicsItem*> topLevelItems;
+  QMultiMap<QGraphicsItem*, QGraphicsItem*> childItems; //QMultiMap<parentItem, childItem>
+
+  for ( int i = 0; i < itemList.size(); ++i )
+  {
+    currentItem = itemList.at( i );
+    //don't draw mapcanvasmap (has z value -10)
+    if ( !currentItem || currentItem->zValue() == -10 )
+    {
+      continue;
+    }
+    if ( currentItem->parentItem() )
+    {
+      childItems.insert( currentItem->parentItem(), currentItem );
+    }
+    else
+    {
+      topLevelItems.insert( currentItem->zValue(), currentItem );
+    }
+  }
+
+  QMultiMap<int, QGraphicsItem*>::iterator topLevelIt = topLevelItems.begin();
+  for ( ; topLevelIt != topLevelItems.end(); ++topLevelIt )
+  {
+    drawCanvasItem( topLevelIt.value(), painter, itemStyle );
+    //Draw children. They probably should be sorted according to z-order, but we don't do it because this code is only
+    //there for backward compatibility. And currently, having several embedded children is not used in QGIS
+    QMap<QGraphicsItem*, QGraphicsItem*>::iterator childIt = childItems.find( topLevelIt.value() );
+    while ( childIt != childItems.end() && childIt.key() == topLevelIt.value() )
+    {
+      drawCanvasItem( childIt.value(), painter, itemStyle );
+      ++childIt;
+    }
+  }
+#endif
+}
+
+void QgsComposerMap::drawCanvasItem( QGraphicsItem* item, QPainter* painter, const QStyleOptionGraphicsItem* itemStyle )
+{
+  if ( !item || !mMapCanvas || !mMapRenderer  || !item->isVisible() )
+  {
+    return;
+  }
+
+  painter->save();
+
+  QgsRectangle rendererExtent = mMapRenderer->extent();
+  QgsRectangle composerMapExtent = mExtent;
+
+  //determine scale factor according to graphics view dpi
+  double scaleFactor = 1.0 / mMapCanvas->logicalDpiX() * 25.4;
+
+  double itemX, itemY;
+  QGraphicsItem* parent = item->parentItem();
+  if ( !parent )
+  {
+    QPointF mapPos = composerMapPosForItem( item );
+    itemX = mapPos.x();
+    itemY = mapPos.y();
+  }
+  else //place item relative to the parent item
+  {
+    QPointF itemScenePos = item->scenePos();
+    QPointF parentScenePos = parent->scenePos();
+
+    QPointF mapPos = composerMapPosForItem( parent );
+
+    itemX = mapPos.x() + ( itemScenePos.x() - parentScenePos.x() ) * scaleFactor;
+    itemY = mapPos.y() + ( itemScenePos.y() - parentScenePos.y() ) * scaleFactor;
+  }
+  painter->translate( itemX, itemY );
+
+
+  painter->scale( scaleFactor, scaleFactor );
+
+  //a little trick to let the item know that the paint request comes from the composer
+  item->setData( 0, "composer" );
+  item->paint( painter, itemStyle, 0 );
+  item->setData( 0, "" );
+  painter->restore();
+}
+
+QPointF QgsComposerMap::composerMapPosForItem( const QGraphicsItem* item ) const
+{
+  if ( !item || !mMapCanvas || !mMapRenderer )
+  {
+    return QPointF( 0, 0 );
+  }
+
+  if ( mExtent.height() <= 0 || mExtent.width() <= 0 || mMapCanvas->width() <= 0 || mMapCanvas->height() <= 0 )
+  {
+    return QPointF( 0, 0 );
+  }
+
+  QRectF graphicsSceneRect = mMapCanvas->sceneRect();
+  QPointF itemScenePos = item->scenePos();
+  QgsRectangle mapRendererExtent = mMapRenderer->extent();
+
+  double mapX = itemScenePos.x() / graphicsSceneRect.width() * mapRendererExtent.width() + mapRendererExtent.xMinimum();
+  double mapY = mapRendererExtent.yMaximum() - itemScenePos.y() / graphicsSceneRect.height() * mapRendererExtent.height();
+  return mapToItemCoords( QPointF( mapX, mapY ) );
+}
