@@ -31,6 +31,7 @@
 QgsComposerScaleBar::QgsComposerScaleBar( QgsComposition* composition ): QgsComposerItem( composition ), mComposerMap( 0 ), mStyle( 0 ), mSegmentMillimeters( 0.0 )
 {
   applyDefaultSettings();
+  applyDefaultSize();
 }
 
 QgsComposerScaleBar::~QgsComposerScaleBar()
@@ -130,11 +131,13 @@ void QgsComposerScaleBar::applyDefaultSettings()
 
   mLabelBarSpace = 3.0;
   mBoxContentSpace = 1.0;
+}
 
+void QgsComposerScaleBar::applyDefaultSize()
+{
   if ( mComposerMap )
   {
     //calculate mNumUnitsPerSegment
-    QRectF composerItemRect = mComposerMap->rect();
     QgsRectangle composerMapRect = mComposerMap->extent();
 
     double proposedScaleBarLength = composerMapRect.width() / 4;
