@@ -24,7 +24,9 @@
 #include <QGraphicsRectItem>
 #include <QSettings>
 
-QgsComposition::QgsComposition( QgsMapRenderer* mapRenderer ): QGraphicsScene( 0 ), mMapRenderer( mapRenderer ), mPlotStyle( QgsComposition::Preview ), mPaperItem( 0 ), mSnapToGrid( false ), mSnapGridResolution( 0.0 ), mSnapGridOffsetX( 0.0 ), mSnapGridOffsetY( 0.0 )
+QgsComposition::QgsComposition( QgsMapRenderer* mapRenderer ):
+    QGraphicsScene( 0 ), mMapRenderer( mapRenderer ), mPlotStyle( QgsComposition::Preview ), mPaperItem( 0 ), mPrintAsRaster( false ), mSnapToGrid( false ),
+    mSnapGridResolution( 0.0 ), mSnapGridOffsetX( 0.0 ), mSnapGridOffsetY( 0.0 ), mActiveCommand( 0 )
 {
   setBackgroundBrush( Qt::gray );
 
@@ -35,23 +37,23 @@ QgsComposition::QgsComposition( QgsMapRenderer* mapRenderer ): QGraphicsScene( 0
   mPaperItem->setZValue( 0 );
   mPrintResolution = 300; //hardcoded default
   loadGridAppearanceSettings();
-
-  //mPrintAsRaster
-  QSettings s;
-  mPrintAsRaster = s.value( "/qgis/composerPrintAsRaster", false ).toBool();
 }
 
-QgsComposition::QgsComposition(): QGraphicsScene( 0 ), mMapRenderer( 0 ), mPlotStyle( QgsComposition::Preview ), mPaperItem( 0 ), mPrintAsRaster( false ), mSnapToGrid( false ), mSnapGridResolution( 0.0 ), mSnapGridOffsetX( 0.0 ), mSnapGridOffsetY( 0.0 )
+QgsComposition::QgsComposition():
+    QGraphicsScene( 0 ), mMapRenderer( 0 ), mPlotStyle( QgsComposition::Preview ), mPaperItem( 0 ), mPrintAsRaster( false ),
+    mSnapToGrid( false ), mSnapGridResolution( 0.0 ), mSnapGridOffsetX( 0.0 ), mSnapGridOffsetY( 0.0 ), mActiveCommand( 0 )
 {
   loadGridAppearanceSettings();
-  //mPrintAsRaster
-  QSettings s;
-  mPrintAsRaster = s.value( "/qgis/composerPrintAsRaster", false ).toBool();
 }
 
 QgsComposition::~QgsComposition()
 {
   delete mPaperItem;
+
+  // make sure that all composer items are removed before
+  // this class is deconstructed - to avoid segfaults
+  // when composer items access in destructor composition that isn't valid anymore
+  clear();
 }
 
 void QgsComposition::setPaperSize( double width, double height )
@@ -188,6 +190,7 @@ bool QgsComposition::writeXML( QDomElement& composerElem, QDomDocument& doc )
   compositionElem.setAttribute( "snapGridOffsetY", mSnapGridOffsetY );
 
   compositionElem.setAttribute( "printResolution", mPrintResolution );
+  compositionElem.setAttribute( "printAsRaster", mPrintAsRaster );
 
   composerElem.appendChild( compositionElem );
 
@@ -227,6 +230,7 @@ bool QgsComposition::readXML( const QDomElement& compositionElem, const QDomDocu
   mSnapGridResolution = compositionElem.attribute( "snapGridResolution" ).toDouble();
   mSnapGridOffsetX = compositionElem.attribute( "snapGridOffsetX" ).toDouble();
   mSnapGridOffsetY = compositionElem.attribute( "snapGridOffsetY" ).toDouble();
+  mPrintAsRaster = compositionElem.attribute( "printAsRaster" ).toInt();
 
   mPrintResolution = compositionElem.attribute( "printResolution", "300" ).toInt();
 
@@ -321,6 +325,7 @@ void QgsComposition::moveSelectedItemsToTop()
 {
   QList<QgsComposerItem*> selectedItems = selectedComposerItems();
   QList<QgsComposerItem*>::iterator it = selectedItems.begin();
+
   for ( ; it != selectedItems.end(); ++it )
   {
     moveItemToTop( *it );
@@ -384,13 +389,18 @@ void QgsComposition::alignSelectedItemsLeft()
   double minXCoordinate = selectedItemBBox.left();
 
   //align items left to minimum x coordinate
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Aligned items left" ) );
   QList<QgsComposerItem*>::iterator align_it = selectedItems.begin();
   for ( ; align_it != selectedItems.end(); ++align_it )
   {
+    QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *align_it, "", parentCommand );
+    subcommand->savePreviousState();
     QTransform itemTransform = ( *align_it )->transform();
     itemTransform.translate( minXCoordinate - itemTransform.dx(), 0 );
     ( *align_it )->setTransform( itemTransform );
+    subcommand->saveAfterState();
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::alignSelectedItemsHCenter()
@@ -410,13 +420,18 @@ void QgsComposition::alignSelectedItemsHCenter()
   double averageXCoord = ( selectedItemBBox.left() + selectedItemBBox.right() ) / 2.0;
 
   //place items
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Aligned items hcenter" ) );
   QList<QgsComposerItem*>::iterator align_it = selectedItems.begin();
   for ( ; align_it != selectedItems.end(); ++align_it )
   {
+    QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *align_it, "", parentCommand );
+    subcommand->savePreviousState();
     QTransform itemTransform = ( *align_it )->transform();
     itemTransform.translate( averageXCoord - itemTransform.dx() - ( *align_it )->rect().width() / 2.0, 0 );
     ( *align_it )->setTransform( itemTransform );
+    subcommand->saveAfterState();
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::alignSelectedItemsRight()
@@ -436,13 +451,18 @@ void QgsComposition::alignSelectedItemsRight()
   double maxXCoordinate = selectedItemBBox.right();
 
   //align items right to maximum x coordinate
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Aligned items right" ) );
   QList<QgsComposerItem*>::iterator align_it = selectedItems.begin();
   for ( ; align_it != selectedItems.end(); ++align_it )
   {
+    QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *align_it, "", parentCommand );
+    subcommand->savePreviousState();
     QTransform itemTransform = ( *align_it )->transform();
     itemTransform.translate( maxXCoordinate - itemTransform.dx() - ( *align_it )->rect().width(), 0 );
     ( *align_it )->setTransform( itemTransform );
+    subcommand->saveAfterState();
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::alignSelectedItemsTop()
@@ -460,13 +480,19 @@ void QgsComposition::alignSelectedItemsTop()
   }
 
   double minYCoordinate = selectedItemBBox.top();
+
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Aligned items top" ) );
   QList<QgsComposerItem*>::iterator align_it = selectedItems.begin();
   for ( ; align_it != selectedItems.end(); ++align_it )
   {
+    QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *align_it, "", parentCommand );
+    subcommand->savePreviousState();
     QTransform itemTransform = ( *align_it )->transform();
     itemTransform.translate( 0, minYCoordinate - itemTransform.dy() );
     ( *align_it )->setTransform( itemTransform );
+    subcommand->saveAfterState();
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::alignSelectedItemsVCenter()
@@ -484,13 +510,18 @@ void QgsComposition::alignSelectedItemsVCenter()
   }
 
   double averageYCoord = ( selectedItemBBox.top() + selectedItemBBox.bottom() ) / 2.0;
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Aligned items vcenter" ) );
   QList<QgsComposerItem*>::iterator align_it = selectedItems.begin();
   for ( ; align_it != selectedItems.end(); ++align_it )
   {
+    QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *align_it, "", parentCommand );
+    subcommand->savePreviousState();
     QTransform itemTransform = ( *align_it )->transform();
     itemTransform.translate( 0, averageYCoord - itemTransform.dy() - ( *align_it )->rect().height() / 2 );
     ( *align_it )->setTransform( itemTransform );
+    subcommand->saveAfterState();
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::alignSelectedItemsBottom()
@@ -508,13 +539,18 @@ void QgsComposition::alignSelectedItemsBottom()
   }
 
   double maxYCoord = selectedItemBBox.bottom();
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Aligned items bottom" ) );
   QList<QgsComposerItem*>::iterator align_it = selectedItems.begin();
   for ( ; align_it != selectedItems.end(); ++align_it )
   {
+    QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *align_it, "", parentCommand );
+    subcommand->savePreviousState();
     QTransform itemTransform = ( *align_it )->transform();
     itemTransform.translate( 0, maxYCoord - itemTransform.dy() - ( *align_it )->rect().height() );
     ( *align_it )->setTransform( itemTransform );
+    subcommand->saveAfterState();
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::updateZValues()
@@ -523,85 +559,46 @@ void QgsComposition::updateZValues()
   QLinkedList<QgsComposerItem*>::iterator it = mItemZList.begin();
   QgsComposerItem* currentItem = 0;
 
+  QUndoCommand* parentCommand = new QUndoCommand( tr( "Item z-order changed" ) );
   for ( ; it != mItemZList.end(); ++it )
   {
     currentItem = *it;
     if ( currentItem )
     {
-      QgsDebugMsg( QString::number( counter ) );
+      QgsComposerItemCommand* subcommand = new QgsComposerItemCommand( *it, "", parentCommand );
+      subcommand->savePreviousState();
       currentItem->setZValue( counter );
+      subcommand->saveAfterState();
     }
     ++counter;
   }
+  mUndoStack.push( parentCommand );
 }
 
 void QgsComposition::sortZList()
 {
-#ifdef QGISDEBUG
-  //debug: list before sorting
-  QgsDebugMsg( "before sorting" );
-  QLinkedList<QgsComposerItem*>::iterator before_it = mItemZList.begin();
-  for ( ; before_it != mItemZList.end(); ++before_it )
+  if ( mItemZList.size() < 2 )
   {
-    QgsDebugMsg( QString( "%1" ).arg(( *before_it )->zValue() ) );
+    return;
   }
-#endif
 
-  QMutableLinkedListIterator<QgsComposerItem*> it( mItemZList );
-  int previousZ, afterZ; //z values of items before and after
-  QgsComposerItem* previousItem;
-  QgsComposerItem* afterItem;
+  QLinkedList<QgsComposerItem*>::const_iterator lIt = mItemZList.constBegin();
+  QLinkedList<QgsComposerItem*> sortedList;
 
-  while ( it.hasNext() )
+  for ( ; lIt != mItemZList.constEnd(); ++lIt )
   {
-    previousItem = it.next();
-    if ( previousItem )
+    QLinkedList<QgsComposerItem*>::iterator insertIt = sortedList.begin();
+    for ( ; insertIt != sortedList.end(); ++insertIt )
     {
-      previousZ = previousItem->zValue();
-    }
-    else
-    {
-      previousZ = -1;
-    }
-
-    if ( !it.hasNext() )
-    {
-      break; //this is the end...
-    }
-    afterItem = it.peekNext();
-
-    if ( afterItem )
-    {
-      afterZ = afterItem->zValue();
-    }
-    else
-    {
-      afterZ = -1;
-    }
-
-    if ( previousZ > afterZ )
-    {
-      //swap items
-      if ( previousItem && afterItem )
+      if (( *lIt )->zValue() < ( *insertIt )->zValue() )
       {
-        it.remove();
-        it.next();
-        it.insert( previousItem );
-        it.previous();
+        break;
       }
     }
+    sortedList.insert( insertIt, ( *lIt ) );
   }
 
-#ifdef QGISDEBUG
-  //debug: list after sorting
-  //debug: list before sorting
-  QgsDebugMsg( "after sorting" );
-  QLinkedList<QgsComposerItem*>::iterator after_it = mItemZList.begin();
-  for ( ; after_it != mItemZList.end(); ++after_it )
-  {
-    QgsDebugMsg( QString( "%1" ).arg(( *after_it )->zValue() ) );
-  }
-#endif
+  mItemZList = sortedList;
 }
 
 QPointF QgsComposition::snapPointToGrid( const QPointF& scenePoint ) const
@@ -714,13 +711,6 @@ void QgsComposition::setGridStyle( GridStyle s )
   saveGridAppearanceSettings();
 }
 
-void QgsComposition::setPrintAsRaster( bool enabled )
-{
-  mPrintAsRaster = enabled;
-  QSettings s;
-  s.setValue( "/qgis/composerPrintAsRaster", QVariant( mPrintAsRaster ) );
-}
-
 void QgsComposition::loadGridAppearanceSettings()
 {
   //read grid style, grid color and pen width from settings
@@ -775,3 +765,46 @@ void QgsComposition::saveGridAppearanceSettings()
     s.setValue( "/qgis/composerGridStyle", "Crosses" );
   }
 }
+
+void QgsComposition::beginCommand( QgsComposerItem* item, const QString& commandText, QgsComposerMergeCommand::Context c )
+{
+  delete mActiveCommand;
+  if ( !item )
+  {
+    mActiveCommand = 0;
+    return;
+  }
+
+  if ( c == QgsComposerMergeCommand::Unknown )
+  {
+    mActiveCommand = new QgsComposerItemCommand( item, commandText );
+  }
+  else
+  {
+    mActiveCommand = new QgsComposerMergeCommand( c, item, commandText );
+  }
+  mActiveCommand->savePreviousState();
+}
+
+void QgsComposition::endCommand()
+{
+  if ( mActiveCommand )
+  {
+    mActiveCommand->saveAfterState();
+    if ( mActiveCommand->containsChange() ) //protect against empty commands
+    {
+      mUndoStack.push( mActiveCommand );
+    }
+    else
+    {
+      delete mActiveCommand;
+    }
+    mActiveCommand = 0;
+  }
+}
+
+void QgsComposition::cancelCommand()
+{
+  delete mActiveCommand;
+  mActiveCommand = 0;
+}
