@@ -86,6 +86,11 @@ namespace pal
 
       void setLabelInfo( LabelInfo* info ) { labelInfo = info; }
       void setDistLabel( double dist ) { distlabel = dist; }
+      //Set label position of the feature to fixed x/y values
+      void setFixedPosition( double x, double y ) { fixedPos = true; fixedPosX = x; fixedPosY = y;}
+      bool fixedPosition() const { return fixedPos; }
+      //Set label rotation to fixed value
+      void setFixedAngle( double a ) { fixedRotation = true; fixedAngle = a; }
 
     protected:
       Layer *layer;
@@ -97,6 +102,13 @@ namespace pal
 
       char *uid;
 
+      bool fixedPos; //true in case of fixed position (only 1 candidate position with cost 0)
+      double fixedPosX;
+      double fixedPosY;
+      //Fixed (e.g. data defined) angle only makes sense together with fixed position
+      bool fixedRotation;
+      double fixedAngle; //fixed angle value (in rad)
+
       // array of parts - possibly not necessary
       //int nPart;
       //FeaturePart** parts;
