@@ -80,16 +80,16 @@ namespace pal
 
 
   inline int max( int a, int b )
-  {return( a > b ? a : b );}
+  { return a > b ? a : b; }
 
   inline double max( double a, double b )
-  {return( a > b ? a : b );}
+  { return a > b ? a : b; }
 
   inline int min( int a, int b )
-  {return( a < b ? a : b );}
+  { return a < b ? a : b; }
 
   inline double min( double a, double b )
-  {return( a < b ? a : b );}
+  { return a < b ? a : b; }
 
   inline double vabs( double x )
   { return x >= 0 ? x : -x; }
