@@ -17,6 +17,11 @@
 #define QGSAPPLICATION_H
 
 #include <QApplication>
+#include <QEvent>
+
+#include <qgis.h>
+
+#define ABISYM(x)  x ## VERSION_INT
 
 /** \ingroup core
  * Extends QApplication to provide access to QGIS specific resources such
@@ -26,18 +31,25 @@ class CORE_EXPORT QgsApplication: public QApplication
 {
     Q_OBJECT
   public:
-    QgsApplication( int & argc, char ** argv, bool GUIenabled );
+    //! @note customConfigDir parameter added in v1.6
+    QgsApplication( int & argc, char ** argv, bool GUIenabled, QString customConfigPath = QString() );
     virtual ~QgsApplication();
 
+    //! Watch for QFileOpenEvent.
+    virtual bool event( QEvent * event );
+
     //! Catch exceptions when sending event to receiver.
     virtual bool notify( QObject * receiver, QEvent * event );
 
+    //! Set the FileOpen event receiver
+    static void setFileOpenEventReceiver( QObject * receiver );
+
     /** Set the active theme to the specified theme.
      * The theme name should be a single word e.g. 'default','classic'.
      * The theme search path usually will be pkgDataPath + "/themes/" + themName + "/"
      * but plugin writers etc can use themeName() as a basis for searching
      * for resources in their own datastores e.g. a Qt4 resource bundle.
-     * @Note A basic test will be carried out to ensure the theme search path
+     * @note A basic test will be carried out to ensure the theme search path
      * based on the supplied theme name exists. If it does not the theme name will
      * be reverted to 'default'.
      */
@@ -80,9 +92,6 @@ class CORE_EXPORT QgsApplication: public QApplication
     //! Returns the path to the help application.
     static const QString helpAppPath();
 
-    //! Returns the path to the mapserver export application.
-    static const QString msexportAppPath();
-
     //! Returns the path to the translation directory.
     static const QString i18nPath();
 
@@ -109,8 +118,8 @@ class CORE_EXPORT QgsApplication: public QApplication
     static const QStringList svgPaths();
 
     //! Returns the paths to svg applications svg directory.
-    //! @note deprecated since 1.4
-    static const QString svgPath();
+    //! @deprecated since 1.4 - use svgPaths()
+    Q_DECL_DEPRECATED static const QString svgPath();
 
     //! Returns the path to the application prefix directory.
     static const QString prefixPath();
@@ -127,6 +136,11 @@ class CORE_EXPORT QgsApplication: public QApplication
     //! Returns the path to the default theme directory.
     static const QString defaultThemePath();
 
+    //! Returns path to the desired icon file.
+    //! First it tries to use the active theme path, then default theme path
+    //! @note Added in 1.5
+    static QString iconPath( QString iconFile );
+
     //! Returns the path to user's style. Added in QGIS 1.4
     static const QString userStyleV2Path();
 
@@ -134,7 +148,7 @@ class CORE_EXPORT QgsApplication: public QApplication
     static const QString defaultStyleV2Path();
 
     //! Alters prefix path - used by 3rd party apps
-    static void setPrefixPath( const QString thePrefixPath, bool useDefaultPaths = FALSE );
+    static void setPrefixPath( const QString thePrefixPath, bool useDefaultPaths = false );
 
     //! Alters plugin path - used by 3rd party apps
     static void setPluginPath( const QString thePluginPath );
@@ -142,6 +156,9 @@ class CORE_EXPORT QgsApplication: public QApplication
     //! Alters pkg data path - used by 3rd party apps
     static void setPkgDataPath( const QString thePkgDataPath );
 
+    //! Alters default svg paths - used by 3rd party apps. Added in QGIS 1.5
+    static void setDefaultSvgPaths( const QStringList& pathList );
+
     //! loads providers
     static void initQgis();
 
@@ -179,11 +196,24 @@ class CORE_EXPORT QgsApplication: public QApplication
      */
     static void registerOgrDrivers();
 
+    /**Converts absolute path to path relative to target
+      @note: this method was added in version 1.6*/
+    static QString absolutePathToRelativePath( QString apath, QString targetPath );
+    /**Converts path relative to target to an absolute path
+      @note: this method was added in version 1.6*/
+    static QString relativePathToAbsolutePath( QString rpath, QString targetPath );
+
   private:
-    static QString mPrefixPath;
-    static QString mPluginPath;
-    static QString mPkgDataPath;
-    static QString mThemeName;
+    static QObject* ABISYM( mFileOpenEventReceiver );
+    static QStringList ABISYM( mFileOpenEventList );
+
+    static QString ABISYM( mPrefixPath );
+    static QString ABISYM( mPluginPath );
+    static QString ABISYM( mPkgDataPath );
+    static QString ABISYM( mThemeName );
+    static QStringList ABISYM( mDefaultSvgPaths );
+
+    static QString ABISYM( mConfigPath );
 };
 
 #endif
