@@ -31,41 +31,39 @@
 #include "qgsmessageoutput.h"
 #include "qgis.h" //const vals declared here
 
-#include <cassert>
 #include <sqlite3.h>
 
 //gdal and ogr includes (needed for == operator)
 #include <ogr_srs_api.h>
 #include <cpl_error.h>
 #include <cpl_conv.h>
-#include "qgslogger.h"
 
 CUSTOM_CRS_VALIDATION QgsCoordinateReferenceSystem::mCustomSrsValidation = NULL;
 
 //--------------------------
 
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem()
-    : mMapUnits( QGis::UnknownUnit ),
-    mIsValidFlag( 0 ),
-    mValidationHint( 0 )
+    : mMapUnits( QGis::UnknownUnit )
+    , mIsValidFlag( 0 )
+    , mValidationHint( "" )
 {
   mCRS = OSRNewSpatialReference( NULL );
 }
 
-QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( QString theWkt )
-    : mMapUnits( QGis::UnknownUnit ),
-    mIsValidFlag( 0 ),
-    mValidationHint( 0 )
+QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( QString theDefinition )
+    : mMapUnits( QGis::UnknownUnit )
+    , mIsValidFlag( 0 )
+    , mValidationHint( "" )
 {
   mCRS = OSRNewSpatialReference( NULL );
-  createFromWkt( theWkt );
+  createFromString( theDefinition );
 }
 
 
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const long theId, CrsType theType )
-    : mMapUnits( QGis::UnknownUnit ),
-    mIsValidFlag( 0 ),
-    mValidationHint( 0 )
+    : mMapUnits( QGis::UnknownUnit )
+    , mIsValidFlag( 0 )
+    , mValidationHint( "" )
 {
   mCRS = OSRNewSpatialReference( NULL );
   createFromId( theId, theType );
@@ -76,50 +74,76 @@ QgsCoordinateReferenceSystem::~QgsCoordinateReferenceSystem()
   OSRDestroySpatialReference( mCRS );
 }
 
-void QgsCoordinateReferenceSystem::createFromId( const long theId, CrsType theType )
+bool QgsCoordinateReferenceSystem::createFromId( const long theId, CrsType theType )
 {
+  bool result = false;
   switch ( theType )
   {
     case InternalCrsId:
-      createFromSrsId( theId );
+      result = createFromSrsId( theId );
       break;
     case PostgisCrsId:
-      createFromSrid( theId );
+      result = createFromSrid( theId );
       break;
     case EpsgCrsId:
-      createFromEpsg( theId );
+      result = createFromOgcWmsCrs( QString( "EPSG:%1" ).arg( theId ) );
       break;
     default:
       //THIS IS BAD...THIS PART OF CODE SHOULD NEVER BE REACHED...
       QgsDebugMsg( "Unexpected case reached!" );
   };
+  return result;
+}
+
+bool QgsCoordinateReferenceSystem::createFromString( const QString theDefinition )
+{
+  bool result = false;
+  QRegExp reCrsId( "^(epsg|postgis|internal)\\:(\\d+)$", Qt::CaseInsensitive );
+  if ( reCrsId.indexIn( theDefinition ) == 0 )
+  {
+    QString authName = reCrsId.cap( 1 ).toLower();
+    CrsType type = InternalCrsId;
+    if ( authName == "epsg" ) type = EpsgCrsId;
+    if ( authName == "postgis" ) type = PostgisCrsId;
+    long id = reCrsId.cap( 2 ).toLong();
+    result = createFromId( id, type );
+  }
+  else
+  {
+    QRegExp reCrsStr( "^(?:(wkt|proj4)\\:)?(.+)$", Qt::CaseInsensitive );
+    if ( reCrsStr.indexIn( theDefinition ) == 0 )
+    {
+      if ( reCrsStr.cap( 1 ).toLower() == "proj4" )
+      {
+        result = createFromProj4( reCrsStr.cap( 2 ) );
+      }
+      else
+      {
+        result = createFromWkt( reCrsStr.cap( 2 ) );
+      }
+    }
+  }
+  return result;
 }
 
 bool QgsCoordinateReferenceSystem::createFromOgcWmsCrs( QString theCrs )
 {
-  QStringList parts = theCrs.split( ":" );
-
-  if ( parts.at( 0 ).compare( "EPSG", Qt::CaseInsensitive ) == 0 )
+  QRegExp re( "(user|custom|qgis):(\\d+)", Qt::CaseInsensitive );
+  if ( re.exactMatch( theCrs ) && createFromSrsId( re.cap( 2 ).toInt() ) )
   {
-    createFromEpsg( parts.at( 1 ).toLong() );
-  }
-  else if ( parts.at( 0 ).compare( "CRS", Qt::CaseInsensitive ) == 0 )
-  {
-    if ( parts.at( 1 ) == "84" )
-    {
-      //! \todo - CRS:84 is hardcoded to EPSG:4326 - see if this is appropriate
-      /**
-       *  See WMS 1.3 standard appendix B3 for details
-       */
-      createFromEpsg( 4326 );
-    }
-  }
-  else
-  {
-    return FALSE;
+    return true;
   }
 
-  return TRUE;
+  if ( loadFromDb( QgsApplication::srsDbFilePath(), "lower(auth_name||':'||auth_id)", theCrs.toLower() ) )
+    return true;
+
+  if ( theCrs.compare( "CRS:84", Qt::CaseInsensitive ) == 0 )
+  {
+    createFromSrsId( GEOCRS_ID );
+    return true;
+  }
+
+  return false;
 }
 
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QgsCoordinateReferenceSystem &srs )
@@ -140,7 +164,7 @@ QgsCoordinateReferenceSystem& QgsCoordinateReferenceSystem::operator=( const Qgs
     mGeoFlag = srs.mGeoFlag;
     mMapUnits = srs.mMapUnits;
     mSRID = srs.mSRID;
-    mEpsg = srs.mEpsg;
+    mAuthId = srs.mAuthId;
     mIsValidFlag = srs.mIsValidFlag;
     mValidationHint = srs.mValidationHint;
     if ( mIsValidFlag )
@@ -166,28 +190,28 @@ void QgsCoordinateReferenceSystem::validate()
 
   if ( !mIsValidFlag )
     // set the default
-    createFromProj4( GEOPROJ4 );
+    createFromOgcWmsCrs( GEO_EPSG_CRS_AUTHID );
 }
 
 bool QgsCoordinateReferenceSystem::createFromSrid( long id )
 {
-  return loadFromDb( QgsApplication::srsDbFilePath(), "srid", id );
+  return loadFromDb( QgsApplication::srsDbFilePath(), "srid", QString::number( id ) );
 }
 
 bool QgsCoordinateReferenceSystem::createFromEpsg( long id )
 {
-  return loadFromDb( QgsApplication::srsDbFilePath(), "epsg", id );
+  return createFromOgcWmsCrs( QString( "EPSG:%1" ).arg( id ) );
 }
 
 bool QgsCoordinateReferenceSystem::createFromSrsId( long id )
 {
-  return loadFromDb( id < 100000 ? QgsApplication::srsDbFilePath() :
-                     QgsApplication::qgisUserDbFilePath(), "srs_id", id );
+  return loadFromDb( id < USER_CRS_START_ID ? QgsApplication::srsDbFilePath() : QgsApplication::qgisUserDbFilePath(),
+                     "srs_id", QString::number( id ) );
 }
 
-bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString field, long id )
+bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString expression, QString value )
 {
-  QgsDebugMsg( "load CRS from " + db + " where " + field + " is " + QString::number( id ) );
+  QgsDebugMsgLevel( "load CRS from " + db + " where " + expression + " is " + value, 3 );
   mIsValidFlag = false;
 
   QFileInfo myInfo( db );
@@ -203,7 +227,7 @@ bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString field, long i
   int           myResult;
   //check the db is available
   myResult = openDb( db, &myDatabase );
-  if ( myResult )
+  if ( myResult != SQLITE_OK )
   {
     QgsDebugMsg( "failed : " + db + " could not be opened!" );
     return mIsValidFlag;
@@ -216,11 +240,12 @@ bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString field, long i
     ellipsoid_acronym NOT NULL,
     parameters text NOT NULL,
     srid integer NOT NULL,
-    epsg integer NOT NULL,
+    auth_name varchar NOT NULL,
+    auth_id integer NOT NULL,
     is_geo integer NOT NULL);
   */
 
-  QString mySql = "select srs_id,description,projection_acronym,ellipsoid_acronym,parameters,srid,epsg,is_geo from tbl_srs where " + field + "='" + QString::number( id ) + "'";
+  QString mySql = "select srs_id,description,projection_acronym,ellipsoid_acronym,parameters,srid,auth_name||':'||auth_id,is_geo from tbl_srs where " + expression + "=" + quotedValue( value );
   myResult = sqlite3_prepare( myDatabase, mySql.toUtf8(), mySql.toUtf8().length(), &myPreparedStatement, &myTail );
   // XXX Need to free memory from the error msg if one is set
   if ( myResult == SQLITE_OK && sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
@@ -231,11 +256,25 @@ bool QgsCoordinateReferenceSystem::loadFromDb( QString db, QString field, long i
     mEllipsoidAcronym = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 3 ) );
     QString toProj4 = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 4 ) );
     mSRID = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 5 ) ).toLong();
-    mEpsg = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 6 ) ).toLong();
-    int geo = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 7 ) ).toInt();
-    mGeoFlag = ( geo == 0 ? false : true );
-    setProj4String( toProj4 );
-    setMapUnits();
+    mAuthId = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 6 ) );
+    mGeoFlag = QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 7 ) ).toInt() != 0;
+
+    if ( mSrsId >= USER_CRS_START_ID && mAuthId.isEmpty() )
+    {
+      mAuthId = QString( "USER:%1" ).arg( mSrsId );
+    }
+    else if ( mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
+    {
+      OSRDestroySpatialReference( mCRS );
+      mCRS = OSRNewSpatialReference( NULL );
+      mIsValidFlag = OSRSetFromUserInput( mCRS, mAuthId.toLower().toAscii() ) == OGRERR_NONE;
+      setMapUnits();
+    }
+
+    if ( !mIsValidFlag )
+    {
+      setProj4String( toProj4 );
+    }
   }
   else
   {
@@ -255,9 +294,9 @@ bool QgsCoordinateReferenceSystem::createFromWkt( QString theWkt )
     QgsDebugMsg( "theWkt is uninitialised, operation failed" );
     return mIsValidFlag;
   }
-  QgsDebugMsg( "QgsCoordinateReferenceSystem::createFromWkt(QString theWkt) using: " + theWkt );
+  QgsDebugMsg( "wkt: " + theWkt );
   QByteArray ba = theWkt.toLatin1();
-  const char *pWkt = ba;
+  const char *pWkt = ba.data();
 
   OGRErr myInputResult = OSRImportFromWkt( mCRS, ( char ** ) & pWkt );
 
@@ -266,10 +305,20 @@ bool QgsCoordinateReferenceSystem::createFromWkt( QString theWkt )
     QgsDebugMsg( "\n---------------------------------------------------------------" );
     QgsDebugMsg( "This CRS could *** NOT *** be set from the supplied Wkt " );
     QgsDebugMsg( "INPUT: " + theWkt );
+    QgsDebugMsg( QString( "UNUSED WKT: %1" ).arg( pWkt ) );
     QgsDebugMsg( "---------------------------------------------------------------\n" );
     return mIsValidFlag;
   }
 
+  if ( OSRAutoIdentifyEPSG( mCRS ) == OGRERR_NONE )
+  {
+    QString authid = QString( "%1:%2" )
+                     .arg( OSRGetAuthorityName( mCRS, NULL ) )
+                     .arg( OSRGetAuthorityCode( mCRS, NULL ) );
+    QgsDebugMsg( "authid recognized as " + authid );
+    return createFromOgcWmsCrs( authid );
+  }
+
   // always morph from esri as it doesn't hurt anything
   // FW: Hey, that's not right!  It can screw stuff up! Disable
   //myOgrSpatialRef.morphFromESRI();
@@ -278,10 +327,21 @@ bool QgsCoordinateReferenceSystem::createFromWkt( QString theWkt )
   char *proj4src = NULL;
   OSRExportToProj4( mCRS, &proj4src );
 
-  //now that we have the proj4string, delegate to createFromProj4String so
+  //now that we have the proj4string, delegate to createFromProj4 so
   // that we can try to fill in the remaining class members...
   //create from Proj will set the isValidFlag
-  createFromProj4( QString( proj4src ) );
+  if ( !createFromProj4( proj4src ) )
+  {
+    CPLFree( proj4src );
+
+    // try fixed up version
+    OSRFixup( mCRS );
+
+    OSRExportToProj4( mCRS, &proj4src );
+
+    createFromProj4( proj4src );
+  }
+
   CPLFree( proj4src );
 
   return mIsValidFlag;
@@ -303,76 +363,55 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString theProj4String
   // +proj=lcc +lat_1=46.8 +lat_0=46.8 +lon_0=2.337229166666664 +k_0=0.99987742
   // +x_0=600000 +y_0=2200000 +a=6378249.2 +b=6356515.000000472 +units=m +no_defs
   //
+  QgsDebugMsg( "proj4: " + theProj4String );
   mIsValidFlag = false;
 
-  QRegExp myProjRegExp( "\\+proj=\\S+" );
-  int myStart = 0;
-  int myLength = 0;
-  myStart = myProjRegExp.indexIn( theProj4String, myStart );
+  QRegExp myProjRegExp( "\\+proj=(\\S+)" );
+  int myStart = myProjRegExp.indexIn( theProj4String );
   if ( myStart == -1 )
   {
-    QgsDebugMsg( "error proj string supplied has no +proj argument" );
+    QgsDebugMsg( "proj string supplied has no +proj argument" );
     return mIsValidFlag;
   }
+
+  mProjectionAcronym = myProjRegExp.cap( 1 );
+
+  QRegExp myEllipseRegExp( "\\+ellps=(\\S+)" );
+  myStart = myEllipseRegExp.indexIn( theProj4String );
+  if ( myStart == -1 )
+  {
+    QgsDebugMsg( "proj string supplied has no +ellps argument" );
+    mEllipsoidAcronym = "";
+  }
   else
   {
-    myLength = myProjRegExp.matchedLength();
+    mEllipsoidAcronym = myEllipseRegExp.cap( 1 );
   }
 
-  mProjectionAcronym = theProj4String.mid( myStart + PROJ_PREFIX_LEN, myLength - PROJ_PREFIX_LEN );
-
-  QRegExp myEllipseRegExp( "\\+ellps=\\S+" );
-  myStart = 0;
-  myLength = 0;
-  myStart = myEllipseRegExp.indexIn( theProj4String, myStart );
-  if ( myStart != -1 )
+  QRegExp myAxisRegExp( "\\+a=(\\S+)" );
+  myStart = myAxisRegExp.indexIn( theProj4String );
+  if ( myStart == -1 )
   {
-    myLength = myEllipseRegExp.matchedLength();
-    mEllipsoidAcronym = theProj4String.mid( myStart + ELLPS_PREFIX_LEN, myLength - ELLPS_PREFIX_LEN );
-  }
-
-  QRegExp myAxisRegExp( "\\+a=\\S+" );
-  myStart = 0;
-  myLength = 0;
-  myStart = myAxisRegExp.indexIn( theProj4String, myStart );
-  if ( myStart == -1 && mEllipsoidAcronym.isNull() )
-  {
-    QgsDebugMsg( "proj string supplied has no +ellps or +a argument" );
-    return mIsValidFlag;
+    QgsDebugMsg( "proj string supplied has no +a argument" );
   }
 
   /*
-  * We try to match the proj string to and srsid using the following logic:
-  *
-  * - perform a whole text search on srs name (if not null). The srs name will
-  *   have been set if this method has been delegated to from createFromWkt.
-  * Normally we wouldnt expect this to work, but its worth trying first
-  * as its quicker than methods below..
-  */
+   * We try to match the proj string to and srsid using the following logic:
+   *
+   * - perform a whole text search on srs name (if not null). The srs name will
+   *   have been set if this method has been delegated to from createFromWkt.
+   * Normally we wouldnt expect this to work, but its worth trying first
+   * as its quicker than methods below..
+   */
   long mySrsId = 0;
   QgsCoordinateReferenceSystem::RecordMap myRecord;
 
-  // *** Matching on descriptions feels iffy. Different projs can have same description. Homann ***
-  // if ( !mDescription.trimmed().isEmpty() )
-  //{
-  //  myRecord = getRecord( "select * from tbl_srs where description='" + mDescription.trimmed() + "'" );
-  //}
-
   /*
-  * - if the above does not match perform a whole text search on proj4 string (if not null)
-  */
+   * - if the above does not match perform a whole text search on proj4 string (if not null)
+   */
   // QgsDebugMsg( "wholetext match on name failed, trying proj4string match" );
-  myRecord = getRecord( "select * from tbl_srs where parameters='" + theProj4String.trimmed() + "'" );
-  if ( !myRecord.empty() )
-  {
-    mySrsId = myRecord["srs_id"].toLong();
-    QgsDebugMsg( "proj4string match search for srsid returned srsid: " + QString::number( mySrsId ) );
-    if ( mySrsId > 0 )
-    {
-      createFromSrsId( mySrsId );
-    }
-  }
-  else
+  myRecord = getRecord( "select * from tbl_srs where parameters=" + quotedValue( theProj4String.trimmed() ) );
+  if ( myRecord.empty() )
   {
     // Ticket #722 - aaronr
     // Check if we can swap the lat_1 and lat_2 params (if they exist) to see if we match...
@@ -387,7 +426,7 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString theProj4String
     QString lat2Str = "";
     myStart1 = myLat1RegExp.indexIn( theProj4String, myStart1 );
     myStart2 = myLat2RegExp.indexIn( theProj4String, myStart2 );
-    if (( myStart1 != -1 ) && ( myStart2 != -1 ) )
+    if ( myStart1 != -1 && myStart2 != -1 )
     {
       myLength1 = myLat1RegExp.matchedLength();
       myLength2 = myLat2RegExp.matchedLength();
@@ -395,7 +434,7 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString theProj4String
       lat2Str = theProj4String.mid( myStart2 + LAT_PREFIX_LEN, myLength2 - LAT_PREFIX_LEN );
     }
     // If we found the lat_1 and lat_2 we need to swap and check to see if we can find it...
-    if (( lat1Str != "" ) && ( lat2Str != "" ) )
+    if ( lat1Str != "" && lat2Str != "" )
     {
       // Make our new string to check...
       QString theProj4StringModified = theProj4String;
@@ -406,35 +445,70 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString theProj4String
       myStart2 = myLat2RegExp.indexIn( theProj4String, myStart2 );
       theProj4StringModified.replace( myStart2 + LAT_PREFIX_LEN, myLength2 - LAT_PREFIX_LEN, lat1Str );
       QgsDebugMsg( "trying proj4string match with swapped lat_1,lat_2" );
-      myRecord = getRecord( "select * from tbl_srs where parameters='" + theProj4StringModified.trimmed() + "'" );
-      if ( !myRecord.empty() )
-      {
-        // Success!  We have found the proj string by swapping the lat_1 and lat_2
-        setProj4String( theProj4StringModified );
-        mySrsId = myRecord["srs_id"].toLong();
-        QgsDebugMsg( "proj4string match search for srsid returned srsid: " + QString::number( mySrsId ) );
-        if ( mySrsId > 0 )
-        {
-          createFromSrsId( mySrsId );
-        }
-      }
+      myRecord = getRecord( "select * from tbl_srs where parameters=" + quotedValue( theProj4StringModified.trimmed() ) );
     }
-    else
+  }
+
+  if ( myRecord.empty() )
+  {
+    // match all arameters individually:
+    // - order of parameters doesn't matter
+    // - found definition may have more parameters (like +towgs84 in GDAL)
+    // - retry without datum, if no match is found (looks like +datum<>WGS84 was dropped in GDAL)
+
+    QString sql = "SELECT * FROM tbl_srs WHERE ";
+    QString delim = "";
+    QString datum;
+    foreach( QString param, theProj4String.split( " ", QString::SkipEmptyParts ) )
     {
-      // Last ditch attempt to piece together what we know of the projection to find a match...
-      QgsDebugMsg( "globbing search for srsid from this proj string" );
-      setProj4String( theProj4String );
-      mySrsId = findMatchingProj();
-      QgsDebugMsg( "globbing search for srsid returned srsid: " + QString::number( mySrsId ) );
-      if ( mySrsId > 0 )
+      QString arg = QString( "' '||parameters||' ' LIKE %1" ).arg( quotedValue( QString( "% %1 %" ).arg( param ) ) );
+      if ( param.startsWith( "+datum=" ) )
       {
-        createFromSrsId( mySrsId );
+        datum = arg;
       }
       else
       {
-        mIsValidFlag = false;
+        sql += delim + arg;
+        delim = " AND ";
       }
     }
+
+    if ( !datum.isEmpty() )
+    {
+      myRecord = getRecord( sql + delim + datum );
+    }
+
+    if ( myRecord.empty() )
+    {
+      // datum might have disappeared in definition - retry without it
+      myRecord = getRecord( sql );
+    }
+  }
+
+  if ( !myRecord.empty() )
+  {
+    mySrsId = myRecord["srs_id"].toLong();
+    QgsDebugMsg( "proj4string param match search for srsid returned srsid: " + QString::number( mySrsId ) );
+    if ( mySrsId > 0 )
+    {
+      createFromSrsId( mySrsId );
+    }
+  }
+  else
+  {
+    // Last ditch attempt to piece together what we know of the projection to find a match...
+    QgsDebugMsg( "globbing search for srsid from this proj string" );
+    setProj4String( theProj4String );
+    mySrsId = findMatchingProj();
+    QgsDebugMsg( "globbing search for srsid returned srsid: " + QString::number( mySrsId ) );
+    if ( mySrsId > 0 )
+    {
+      createFromSrsId( mySrsId );
+    }
+    else
+    {
+      mIsValidFlag = false;
+    }
   }
 
   // if we failed to look up the projection in database, don't worry. we can still use it :)
@@ -442,31 +516,45 @@ bool QgsCoordinateReferenceSystem::createFromProj4( const QString theProj4String
   {
     QgsDebugMsg( "Projection is not found in databases." );
     setProj4String( theProj4String );
+
     // Is the SRS is valid now, we know it's a decent +proj string that can be entered into the srs.db
     if ( mIsValidFlag )
     {
-      // Try to save. If not possible, set to invalid. Problems on read only systems?
-      QgsDebugMsg( "Projection appears to be valid. Save to database!" );
-      mIsValidFlag = saveAsUserCRS();
-      // The srsid is not set, we should do that now.
-      if ( mIsValidFlag )
+      // but the proj.4 parsed string might already be in our database
+      myRecord = getRecord( "select * from tbl_srs where parameters=" + quotedValue( toProj4() ) );
+      if ( myRecord.empty() )
       {
-        myRecord = getRecord( "select * from tbl_srs where parameters='" + theProj4String.trimmed() + "'" );
-        if ( !myRecord.empty() )
+        // It's not, so try to add it
+        QgsDebugMsg( "Projection appears to be valid. Save to database!" );
+        mIsValidFlag = saveAsUserCRS();
+
+        if ( mIsValidFlag )
         {
-          mySrsId = myRecord["srs_id"].toLong();
-          QgsDebugMsg( "proj4string match search for srsid returned srsid: " + QString::number( mySrsId ) );
-          if ( mySrsId > 0 )
-          {
-            createFromSrsId( mySrsId );
-          }
-          else
-          {
-            QgsDebugMsg( "Couldn't find newly added proj string?" );
-            mIsValidFlag = false;
-          }
+          // but validate that it's there afterwards
+          myRecord = getRecord( "select * from tbl_srs where parameters=" + quotedValue( toProj4() ) );
         }
       }
+
+      if ( !myRecord.empty() )
+      {
+        // take the srid from the record
+        mySrsId = myRecord["srs_id"].toLong();
+        QgsDebugMsg( "proj4string match search for srsid returned srsid: " + QString::number( mySrsId ) );
+        if ( mySrsId > 0 )
+        {
+          createFromSrsId( mySrsId );
+        }
+        else
+        {
+          QgsDebugMsg( QString( "invalid srid %1 found" ).arg( mySrsId ) );
+          mIsValidFlag = false;
+        }
+      }
+      else
+      {
+        QgsDebugMsg( "Couldn't find newly added proj string?" );
+        mIsValidFlag = false;
+      }
     }
   }
 
@@ -499,7 +587,7 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
 
   //check the db is available
   myResult = openDb( myDatabaseFileName, &myDatabase );
-  if ( myResult )
+  if ( myResult != SQLITE_OK )
   {
     return myMap;
   }
@@ -510,7 +598,7 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
   {
     QgsDebugMsg( "trying system srs.db" );
     int myColumnCount = sqlite3_column_count( myPreparedStatement );
-    //loop through each column in the record adding its field name and vvalue to the map
+    //loop through each column in the record adding its expression name and value to the map
     for ( int myColNo = 0; myColNo < myColumnCount; myColNo++ )
     {
       myFieldName = QString::fromUtf8(( char * )sqlite3_column_name( myPreparedStatement, myColNo ) );
@@ -520,7 +608,7 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
   }
   else
   {
-    QgsDebugMsg( "trying system qgis.db" );
+    QgsDebugMsg( "trying user qgis.db" );
     sqlite3_finalize( myPreparedStatement );
     sqlite3_close( myDatabase );
 
@@ -529,13 +617,13 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
     myFileInfo.setFile( myDatabaseFileName );
     if ( !myFileInfo.exists( ) )
     {
-      QgsDebugMsg( "users qgis.db not found" );
+      QgsDebugMsg( "user qgis.db not found" );
       return myMap;
     }
 
     //check the db is available
     myResult = openDb( myDatabaseFileName, &myDatabase );
-    if ( myResult )
+    if ( myResult != SQLITE_OK )
     {
       return myMap;
     }
@@ -545,7 +633,7 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
     if ( myResult == SQLITE_OK && sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
     {
       int myColumnCount = sqlite3_column_count( myPreparedStatement );
-      //loop through each column in the record adding its field name and vvalue to the map
+      //loop through each column in the record adding its field name and value to the map
       for ( int myColNo = 0; myColNo < myColumnCount; myColNo++ )
       {
         myFieldName = QString::fromUtf8(( char * )sqlite3_column_name( myPreparedStatement, myColNo ) );
@@ -572,9 +660,6 @@ QgsCoordinateReferenceSystem::RecordMap QgsCoordinateReferenceSystem::getRecord(
 #endif
 
   return myMap;
-
-
-
 }
 
 // Accessors -----------------------------------
@@ -593,7 +678,15 @@ long QgsCoordinateReferenceSystem::postgisSrid() const
 
 long QgsCoordinateReferenceSystem::epsg() const
 {
-  return mEpsg;
+  if ( mAuthId.startsWith( "EPSG:", Qt::CaseInsensitive ) )
+    return mAuthId.mid( 5 ).toLong();
+  else
+    return 0;
+}
+
+QString QgsCoordinateReferenceSystem::authid() const
+{
+  return mAuthId;
 }
 
 QString QgsCoordinateReferenceSystem::description() const
@@ -665,6 +758,10 @@ void QgsCoordinateReferenceSystem::setInternalId( long theSrsId )
 {
   mSrsId = theSrsId;
 }
+void QgsCoordinateReferenceSystem::setAuthId( QString authId )
+{
+  mAuthId = authId;
+}
 void QgsCoordinateReferenceSystem::setSrid( long theSrid )
 {
   mSRID = theSrid;
@@ -695,7 +792,7 @@ void QgsCoordinateReferenceSystem::setGeographicFlag( bool theGeoFlag )
 }
 void QgsCoordinateReferenceSystem::setEpsg( long theEpsg )
 {
-  mEpsg = theEpsg;
+  mAuthId = QString( "EPSG:%1" ).arg( theEpsg );
 }
 void  QgsCoordinateReferenceSystem::setProjectionAcronym( QString theProjectionAcronym )
 {
@@ -733,12 +830,12 @@ void QgsCoordinateReferenceSystem::setMapUnits()
     static const double feetToMeter = 0.3048;
     static const double smallNum = 1e-3;
 
-    if ( std::abs( toMeter - feetToMeter ) < smallNum )
+    if ( qAbs( toMeter - feetToMeter ) < smallNum )
       unit = "Foot";
 
     QgsDebugMsg( "Projection has linear units of " + unit );
 
-    if ( unit == "Meter" )
+    if ( doubleNear( toMeter, 1.0 ) ) //Unit name for meters would be "metre"
       mMapUnits = QGis::Meters;
     else if ( unit == "Foot" )
       mMapUnits = QGis::Feet;
@@ -788,20 +885,21 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
   int           myResult;
 
   // Set up the query to retrieve the projection information needed to populate the list
-  QString mySql = QString( "select srs_id,parameters from tbl_srs where projection_acronym='" +
-                           mProjectionAcronym + "' and ellipsoid_acronym='" + mEllipsoidAcronym + "'" );
+  QString mySql = QString( "select srs_id,parameters from tbl_srs where projection_acronym=%1 and ellipsoid_acronym=%2" )
+                  .arg( quotedValue( mProjectionAcronym ) )
+                  .arg( quotedValue( mEllipsoidAcronym ) );
   // Get the full path name to the sqlite3 spatial reference database.
   QString myDatabaseFileName = QgsApplication::srsDbFilePath();
 
   //check the db is available
   myResult = openDb( myDatabaseFileName, &myDatabase );
-  if ( myResult )
+  if ( myResult != SQLITE_OK )
   {
     return 0;
   }
 
   myResult = sqlite3_prepare( myDatabase, mySql.toUtf8(), mySql.toUtf8().length(), &myPreparedStatement, &myTail );
-  // XXX Need to free memory from the error msg if one is set
+// XXX Need to free memory from the error msg if one is set
   if ( myResult == SQLITE_OK )
   {
 
@@ -834,13 +932,13 @@ long QgsCoordinateReferenceSystem::findMatchingProj()
   myDatabaseFileName = QgsApplication::qgisUserDbFilePath();
   //check the db is available
   myResult = openDb( myDatabaseFileName, &myDatabase );
-  if ( myResult )
+  if ( myResult != SQLITE_OK )
   {
     return 0;
   }
 
   myResult = sqlite3_prepare( myDatabase, mySql.toUtf8(), mySql.toUtf8().length(), &myPreparedStatement, &myTail );
-  // XXX Need to free memory from the error msg if one is set
+// XXX Need to free memory from the error msg if one is set
   if ( myResult == SQLITE_OK )
   {
 
@@ -933,20 +1031,30 @@ bool QgsCoordinateReferenceSystem::readXML( QDomNode & theNode )
 
   if ( ! srsNode.isNull() )
   {
-    QDomNode myNode = srsNode.namedItem( "epsg" );
-    QDomElement myElement = myNode.toElement();
+    bool initialized = false;
 
-    if ( createFromEpsg( myElement.text().toLong() ) )
+    QDomNode myNode = srsNode.namedItem( "authid" );
+    if ( !myNode.isNull() )
     {
-      // createFromEpsg() sets everything, including map units
-      QgsDebugMsg( "Setting from EPSG id" );
+      initialized = createFromOgcWmsCrs( myNode.toElement().text() );
+    }
+
+    if ( !initialized )
+    {
+      myNode = srsNode.namedItem( "epsg" );
+      if ( !myNode.isNull() )
+        initialized = createFromOgcWmsCrs( QString( "EPSG:%1" ).arg( myNode.toElement().text().toLong() ) );
+    }
+
+    if ( initialized )
+    {
+      QgsDebugMsg( "Set from auth id" );
     }
     else
     {
       myNode = srsNode.namedItem( "proj4" );
-      myElement = myNode.toElement();
 
-      if ( createFromProj4( myElement.text() ) )
+      if ( createFromProj4( myNode.toElement().text() ) )
       {
         // createFromProj4() sets everything, including map units
         QgsDebugMsg( "Setting from proj4 string" );
@@ -956,36 +1064,28 @@ bool QgsCoordinateReferenceSystem::readXML( QDomNode & theNode )
         QgsDebugMsg( "Setting from elements one by one" );
 
         myNode = srsNode.namedItem( "proj4" );
-        myElement = myNode.toElement();
-        setProj4String( myElement.text() );
+        setProj4String( myNode.toElement().text() );
 
         myNode = srsNode.namedItem( "srsid" );
-        myElement = myNode.toElement();
-        setInternalId( myElement.text().toLong() );
+        setInternalId( myNode.toElement().text().toLong() );
 
         myNode = srsNode.namedItem( "srid" );
-        myElement = myNode.toElement();
-        setSrid( myElement.text().toLong() );
+        setSrid( myNode.toElement().text().toLong() );
 
-        myNode = srsNode.namedItem( "epsg" );
-        myElement = myNode.toElement();
-        setEpsg( myElement.text().toLong() );
+        myNode = srsNode.namedItem( "authid" );
+        setAuthId( myNode.toElement().text() );
 
         myNode = srsNode.namedItem( "description" );
-        myElement = myNode.toElement();
-        setDescription( myElement.text() );
+        setDescription( myNode.toElement().text() );
 
         myNode = srsNode.namedItem( "projectionacronym" );
-        myElement = myNode.toElement();
-        setProjectionAcronym( myElement.text() );
+        setProjectionAcronym( myNode.toElement().text() );
 
         myNode = srsNode.namedItem( "ellipsoidacronym" );
-        myElement = myNode.toElement();
-        setEllipsoidAcronym( myElement.text() );
+        setEllipsoidAcronym( myNode.toElement().text() );
 
         myNode = srsNode.namedItem( "geographicflag" );
-        myElement = myNode.toElement();
-        if ( myElement.text().compare( "true" ) )
+        if ( myNode.toElement().text().compare( "true" ) )
         {
           setGeographicFlag( true );
         }
@@ -1005,7 +1105,7 @@ bool QgsCoordinateReferenceSystem::readXML( QDomNode & theNode )
   else
   {
     // Return default CRS if none was found in the XML.
-    createFromEpsg( GEO_EPSG_CRS_ID );
+    createFromId( GEOCRS_ID, InternalCrsId );
   }
   return true;
 }
@@ -1028,8 +1128,8 @@ bool QgsCoordinateReferenceSystem::writeXML( QDomNode & theNode, QDomDocument &
   mySridElement.appendChild( theDoc.createTextNode( QString::number( postgisSrid() ) ) );
   mySrsElement.appendChild( mySridElement );
 
-  QDomElement myEpsgElement  = theDoc.createElement( "epsg" );
-  myEpsgElement.appendChild( theDoc.createTextNode( QString::number( epsg() ) ) );
+  QDomElement myEpsgElement  = theDoc.createElement( "authid" );
+  myEpsgElement.appendChild( theDoc.createTextNode( authid() ) );
   mySrsElement.appendChild( myEpsgElement );
 
   QDomElement myDescriptionElement  = theDoc.createElement( "description" );
@@ -1127,16 +1227,16 @@ QString QgsCoordinateReferenceSystem::proj4FromSrsId( const int theSrsId )
   // close the database
   sqlite3_close( db );
 
-  //assert(myProjString.length() > 0);
+  //Q_ASSERT(myProjString.length() > 0);
   return myProjString;
 }
 
 int QgsCoordinateReferenceSystem::openDb( QString path, sqlite3 **db )
 {
-  QgsDebugMsg( "path = " + path );
+  QgsDebugMsgLevel( "path = " + path, 3 );
   int myResult = sqlite3_open( path.toUtf8().data(), db );
 
-  if ( myResult )
+  if ( myResult != SQLITE_OK )
   {
     QgsDebugMsg( "Can't open database: " + QString( sqlite3_errmsg( *db ) ) );
     // XXX This will likely never happen since on open, sqlite creates the
@@ -1144,9 +1244,10 @@ int QgsCoordinateReferenceSystem::openDb( QString path, sqlite3 **db )
     // ... unfortunately it happens on Windows
     QgsMessageOutput* output = QgsMessageOutput::createMessageOutput();
     output->setTitle( "Error" );
-    output->setMessage( "Could not open CRS database " + path +
-                        "<br>Error(" + QString::number( myResult ) + "): " +
-                        QString( sqlite3_errmsg( *db ) ), QgsMessageOutput::MessageText );
+    output->setMessage( QObject::tr( "Could not open CRS database %1<br>Error(%2): %3" )
+                        .arg( path )
+                        .arg( myResult )
+                        .arg( sqlite3_errmsg( *db ) ), QgsMessageOutput::MessageText );
     output->showMessage();
   }
   return myResult;
@@ -1199,7 +1300,6 @@ QString QgsCoordinateReferenceSystem::validationHint()
 
 bool QgsCoordinateReferenceSystem::saveAsUserCRS()
 {
-
   if ( ! mIsValidFlag )
   {
     QgsDebugMsg( "Can't save an invalid CRS!" );
@@ -1217,18 +1317,22 @@ bool QgsCoordinateReferenceSystem::saveAsUserCRS()
   //the autoinc related system tables.
   if ( getRecordCount() == 0 )
   {
-    mySql = QString( "insert into tbl_srs (srs_id,description,projection_acronym,ellipsoid_acronym,parameters,is_geo) " )
-            + " values (" + QString::number( USER_CRS_START_ID ) + ",'"
-            + sqlSafeString( myName ) + "','" + projectionAcronym()
-            + "','" + ellipsoidAcronym()  + "','" + sqlSafeString( toProj4() )
-            + "',0)"; // <-- is_geo shamelessly hard coded for now
+    mySql = "insert into tbl_srs (srs_id,description,projection_acronym,ellipsoid_acronym,parameters,is_geo) values ("
+            + QString::number( USER_CRS_START_ID )
+            + "," + quotedValue( myName )
+            + "," + quotedValue( projectionAcronym() )
+            + "," + quotedValue( ellipsoidAcronym() )
+            + "," + quotedValue( toProj4() )
+            + ",0)"; // <-- is_geo shamelessly hard coded for now
   }
   else
   {
-    mySql = "insert into tbl_srs (description,projection_acronym,ellipsoid_acronym,parameters,is_geo) values ('"
-            + sqlSafeString( myName ) + "','" + projectionAcronym()
-            + "','" + ellipsoidAcronym()  + "','" + sqlSafeString( toProj4() )
-            + "',0)"; // <-- is_geo shamelessly hard coded for now
+    mySql = "insert into tbl_srs (description,projection_acronym,ellipsoid_acronym,parameters,is_geo) values ("
+            + quotedValue( myName )
+            + "," + quotedValue( projectionAcronym() )
+            + "," + quotedValue( ellipsoidAcronym() )
+            + "," + quotedValue( toProj4() )
+            + ",0)"; // <-- is_geo shamelessly hard coded for now
   }
   sqlite3      *myDatabase;
   const char   *myTail;
@@ -1238,17 +1342,16 @@ bool QgsCoordinateReferenceSystem::saveAsUserCRS()
   myResult = sqlite3_open( QgsApplication::qgisUserDbFilePath().toUtf8().data(), &myDatabase );
   if ( myResult != SQLITE_OK )
   {
-    QgsDebugMsg( QString( "Can't open database: %1 \n please notify  QGIS developers of this error \n %2 (file name) " ).arg( sqlite3_errmsg( myDatabase ) ).arg( QgsApplication::qgisUserDbFilePath() ) );
-    // XXX This will likely never happen since on open, sqlite creates the
-    //     database if it does not exist.
-    assert( myResult == SQLITE_OK );
+    QgsDebugMsg( QString( "Can't open or create database %1: %2" )
+                 .arg( QgsApplication::qgisUserDbFilePath() )
+                 .arg( sqlite3_errmsg( myDatabase ) ) );
+    return false;
   }
   QgsDebugMsg( QString( "Update or insert sql \n%1" ).arg( mySql ) );
   myResult = sqlite3_prepare( myDatabase, mySql.toUtf8(), mySql.toUtf8().length(), &myPreparedStatement, &myTail );
   sqlite3_step( myPreparedStatement );
   // XXX Need to free memory from the error msg if one is set
   return myResult == SQLITE_OK;
-
 }
 
 long QgsCoordinateReferenceSystem::getRecordCount()
@@ -1263,9 +1366,7 @@ long QgsCoordinateReferenceSystem::getRecordCount()
   if ( myResult != SQLITE_OK )
   {
     QgsDebugMsg( QString( "Can't open database: %1" ).arg( sqlite3_errmsg( myDatabase ) ) );
-    // XXX This will likely never happen since on open, sqlite creates the
-    //     database if it does not exist.
-    assert( myResult == SQLITE_OK );
+    return 0;
   }
   // Set up the query to retrieve the projection information needed to populate the ELLIPSOID list
   QString mySql = "select count(*) from tbl_srs";
@@ -1283,37 +1384,10 @@ long QgsCoordinateReferenceSystem::getRecordCount()
   sqlite3_finalize( myPreparedStatement );
   sqlite3_close( myDatabase );
   return myRecordCount;
-
 }
 
-const QString QgsCoordinateReferenceSystem::sqlSafeString( const QString theSQL )
+QString QgsCoordinateReferenceSystem::quotedValue( QString value )
 {
-
-  QString myRetval;
-  QChar *it = ( QChar * )theSQL.unicode();
-  for ( int i = 0; i < theSQL.length(); i++ )
-  {
-    if ( *it == '\"' )
-    {
-      myRetval += "\\\"";
-    }
-    else if ( *it == '\'' )
-    {
-      myRetval += "\\'";
-    }
-    else if ( *it == '\\' )
-    {
-      myRetval += "\\\\";
-    }
-    else if ( *it == '%' )
-    {
-      myRetval += "\\%";
-    }
-    else
-    {
-      myRetval += *it;
-    }
-    it++;
-  }
-  return myRetval;
+  value.replace( "'", "''" );
+  return value.prepend( "'" ).append( "'" );
 }
