@@ -15,16 +15,14 @@
  *                                                                         *
  ***************************************************************************/
 /* $Id$ */
-#include <cassert>
 #include "qgscoordinatetransform.h"
+#include "qgsmessageoutput.h"
 #include "qgslogger.h"
 
 //qt includes
 #include <QDomNode>
 #include <QDomElement>
-#include <QTextStream>
 #include <QApplication>
-#include "qgslogger.h"
 
 extern "C"
 {
@@ -34,16 +32,20 @@ extern "C"
 // if defined shows all information about transform to stdout
 // #define COORDINATE_TRANSFORM_VERBOSE
 
-
-
-QgsCoordinateTransform::QgsCoordinateTransform( ) : QObject(), mSourceCRS(), mDestCRS()
-
+QgsCoordinateTransform::QgsCoordinateTransform()
+    : QObject()
+    , mInitialisedFlag( false )
+    , mSourceProjection( 0 )
+    , mDestinationProjection( 0 )
 {
   setFinder();
 }
 
-QgsCoordinateTransform::QgsCoordinateTransform( const QgsCoordinateReferenceSystem& source,
-    const QgsCoordinateReferenceSystem& dest )
+QgsCoordinateTransform::QgsCoordinateTransform( const QgsCoordinateReferenceSystem& source, const QgsCoordinateReferenceSystem& dest )
+    : QObject()
+    , mInitialisedFlag( false )
+    , mSourceProjection( 0 )
+    , mDestinationProjection( 0 )
 {
   setFinder();
   mSourceCRS = source;
@@ -52,14 +54,21 @@ QgsCoordinateTransform::QgsCoordinateTransform( const QgsCoordinateReferenceSyst
 }
 
 QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrsId, long theDestSrsId )
-    : mSourceCRS( theSourceSrsId, QgsCoordinateReferenceSystem::InternalCrsId ),
-    mDestCRS( theDestSrsId, QgsCoordinateReferenceSystem::InternalCrsId )
+    : QObject()
+    , mInitialisedFlag( false )
+    , mSourceCRS( theSourceSrsId, QgsCoordinateReferenceSystem::InternalCrsId )
+    , mDestCRS( theDestSrsId, QgsCoordinateReferenceSystem::InternalCrsId )
+    , mSourceProjection( 0 )
+    , mDestinationProjection( 0 )
 {
   initialise();
 }
 
-QgsCoordinateTransform::QgsCoordinateTransform( QString theSourceCRS, QString theDestCRS ) : QObject()
-
+QgsCoordinateTransform::QgsCoordinateTransform( QString theSourceCRS, QString theDestCRS )
+    : QObject()
+    , mInitialisedFlag( false )
+    , mSourceProjection( 0 )
+    , mDestinationProjection( 0 )
 {
   setFinder();
   mSourceCRS.createFromWkt( theSourceCRS );
@@ -73,7 +82,11 @@ QgsCoordinateTransform::QgsCoordinateTransform( QString theSourceCRS, QString th
 
 QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrid,
     QString theDestWkt,
-    QgsCoordinateReferenceSystem::CrsType theSourceCRSType ): QObject()
+    QgsCoordinateReferenceSystem::CrsType theSourceCRSType )
+    : QObject()
+    , mInitialisedFlag( false )
+    , mSourceProjection( 0 )
+    , mDestinationProjection( 0 )
 {
   setFinder();
 
@@ -89,11 +102,11 @@ QgsCoordinateTransform::QgsCoordinateTransform( long theSourceSrid,
 QgsCoordinateTransform::~QgsCoordinateTransform()
 {
   // free the proj objects
-  if ( mSourceProjection != 0 )
+  if ( mSourceProjection )
   {
     pj_free( mSourceProjection );
   }
-  if ( mDestinationProjection != 0 )
+  if ( mDestinationProjection )
   {
     pj_free( mDestinationProjection );
   }
@@ -110,7 +123,6 @@ void QgsCoordinateTransform::setDestCRS( const QgsCoordinateReferenceSystem& the
   initialise();
 }
 
-
 void QgsCoordinateTransform::setDestCRSID( long theCRSID )
 {
   //!todo Add some logic here to determine if the srsid is a system or user one
@@ -122,11 +134,6 @@ void QgsCoordinateTransform::setDestCRSID( long theCRSID )
 // And probably shouldn't be a void
 void QgsCoordinateTransform::initialise()
 {
-
-  mInitialisedFlag = false; //guilty until proven innocent...
-  mSourceProjection = NULL;
-  mDestinationProjection = NULL;
-
   // XXX Warning - multiple return paths in this block!!
   if ( !mSourceCRS.isValid() )
   {
@@ -143,7 +150,7 @@ void QgsCoordinateTransform::initialise()
     //No destination projection is set so we set the default output projection to
     //be the same as input proj. This only happens on the first layer loaded
     //whatever that may be...
-    mDestCRS.createFromProj4( mSourceCRS.toProj4() );
+    mDestCRS.createFromOgcWmsCrs( mSourceCRS.authid() );
   }
 
   // init the projections (destination and source)
@@ -156,11 +163,11 @@ void QgsCoordinateTransform::initialise()
 #endif
 
   mInitialisedFlag = true;
-  if ( mDestinationProjection == NULL )
+  if ( !mDestinationProjection )
   {
     mInitialisedFlag = false;
   }
-  if ( mSourceProjection == NULL )
+  if ( !mSourceProjection )
   {
     mInitialisedFlag = false;
   }
@@ -194,13 +201,13 @@ void QgsCoordinateTransform::initialise()
     // If the source and destination projection are the same, set the short
     // circuit flag (no transform takes place)
     mShortCircuit = true;
-    QgsDebugMsg( "Source/Dest CRS equal, shortcircuit is set." );
+    QgsDebugMsgLevel( "Source/Dest CRS equal, shortcircuit is set.", 3 );
   }
   else
   {
     // Transform must take place
     mShortCircuit = false;
-    QgsDebugMsg( "Source/Dest CRS UNequal, shortcircuit is NOt set." );
+    QgsDebugMsgLevel( "Source/Dest CRS UNequal, shortcircuit is NOt set.", 3 );
   }
 
 }
@@ -227,7 +234,7 @@ QgsPoint QgsCoordinateTransform::transform( const QgsPoint thePoint, TransformDi
   catch ( QgsCsException &cse )
   {
     // rethrow the exception
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
+    QgsDebugMsg( "rethrowing exception" );
     throw cse;
   }
 
@@ -244,7 +251,7 @@ QgsPoint QgsCoordinateTransform::transform( const double theX, const double theY
   catch ( QgsCsException &cse )
   {
     // rethrow the exception
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
+    QgsDebugMsg( "rethrowing exception" );
     throw cse;
   }
 }
@@ -270,7 +277,7 @@ QgsRectangle QgsCoordinateTransform::transform( const QgsRectangle theRect, Tran
   catch ( QgsCsException &cse )
   {
     // rethrow the exception
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
+    QgsDebugMsg( "rethrowing exception" );
     throw cse;
   }
 
@@ -304,7 +311,7 @@ void QgsCoordinateTransform::transformInPlace( double& x, double& y, double& z,
   catch ( QgsCsException &cse )
   {
     // rethrow the exception
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
+    QgsDebugMsg( "rethrowing exception" );
     throw cse;
   }
 }
@@ -316,7 +323,7 @@ void QgsCoordinateTransform::transformInPlace( std::vector<double>& x,
   if ( mShortCircuit || !mInitialisedFlag )
     return;
 
-  assert( x.size() == y.size() );
+  Q_ASSERT( x.size() == y.size() );
 
   // Apparently, if one has a std::vector, it is valid to use the
   // address of the first element in the vector as a pointer to an
@@ -330,7 +337,7 @@ void QgsCoordinateTransform::transformInPlace( std::vector<double>& x,
   catch ( QgsCsException &cse )
   {
     // rethrow the exception
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
+    QgsDebugMsg( "rethrowing exception" );
     throw cse;
   }
 }
@@ -346,6 +353,12 @@ QgsRectangle QgsCoordinateTransform::transformBoundingBox( const QgsRectangle re
   if ( mShortCircuit || !mInitialisedFlag )
     return rect;
 
+  if ( rect.isEmpty() )
+  {
+    QgsPoint p = transform( rect.xMinimum(), rect.yMinimum(), direction );
+    return QgsRectangle( p, p );
+  }
+
   static const int numP = 8;
 
   QgsRectangle bb_rect;
@@ -394,7 +407,7 @@ QgsRectangle QgsCoordinateTransform::transformBoundingBox( const QgsRectangle re
   catch ( QgsCsException &cse )
   {
     // rethrow the exception
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
+    QgsDebugMsg( "rethrowing exception" );
     throw cse;
   }
 
@@ -402,7 +415,8 @@ QgsRectangle QgsCoordinateTransform::transformBoundingBox( const QgsRectangle re
 
   for ( int i = 0; i < numP * numP; i++ )
   {
-    bb_rect.combineExtentWith( x[i], y[i] );
+    if ( qIsFinite( x[i] ) && qIsFinite( y[i] ) )
+      bb_rect.combineExtentWith( x[i], y[i] );
   }
 
   QgsDebugMsg( "Projected extent: " + QString(( bb_rect.toString() ).toLocal8Bit().data() ) );
@@ -454,43 +468,45 @@ void QgsCoordinateTransform::transformCoords( const int& numPoints, double *x, d
   if ( direction == ReverseTransform )
   {
     projResult = pj_transform( mDestinationProjection, mSourceProjection, numPoints, 0, x, y, z );
-    dir = "inverse";
+    dir = tr( "inverse transform" );
   }
   else
   {
-    assert( mSourceProjection != 0 );
-    assert( mDestinationProjection != 0 );
+    Q_ASSERT( mSourceProjection != 0 );
+    Q_ASSERT( mDestinationProjection != 0 );
     projResult = pj_transform( mSourceProjection, mDestinationProjection, numPoints, 0, x, y, z );
-    dir = "forward";
+    dir = tr( "forward transform" );
   }
 
   if ( projResult != 0 )
   {
     //something bad happened....
-    QString msg;
-    QTextStream pjErr( &msg );
+    QString points;
 
-    pjErr << tr( "Failed" ) << " " << dir << " " << tr( "transform of" ) << '\n';
     for ( int i = 0; i < numPoints; ++i )
     {
       if ( direction == ForwardTransform )
       {
-        pjErr << "(" << x[i] << ", " << y[i] << ")\n";
+        points += QString( "(%1, %2)\n" ).arg( x[i] ).arg( y[i] );
       }
       else
       {
-        pjErr << "(" << x[i] * RAD_TO_DEG << ", " << y[i] * RAD_TO_DEG << ")\n";
+        points += QString( "(%1, %2)\n" ).arg( x[i] * RAD_TO_DEG ).arg( y[i] * RAD_TO_DEG );
       }
     }
 
-    pjErr << tr( "with error: " ) << QString::fromUtf8( pj_strerrno( projResult ) ) << '\n';
+    QString msg = tr( "%1 of\n%2\nfailed with error: %3\n" )
+                  .arg( dir )
+                  .arg( points )
+                  .arg( QString::fromUtf8( pj_strerrno( projResult ) ) );
 
-    QgsDebugMsg( "Projection failed emitting invalid transform signal: " + QString( msg.toLocal8Bit().data() ) );
+    QgsDebugMsg( "Projection failed emitting invalid transform signal: " + msg );
 
     emit invalidTransformInput();
 
-    QgsLogger::warning( "Throwing exception " + QString( __FILE__ ) + QString::number( __LINE__ ) );
-    throw  QgsCsException( msg );
+    QgsDebugMsg( "throwing exception" );
+
+    throw QgsCsException( msg );
   }
 
   // if the result is lat/long, convert the results from radians back
@@ -530,15 +546,14 @@ bool QgsCoordinateTransform::readXML( QDomNode & theNode )
 
 bool QgsCoordinateTransform::writeXML( QDomNode & theNode, QDomDocument & theDoc )
 {
-
   QDomElement myNodeElement = theNode.toElement();
-  QDomElement myTransformElement  = theDoc.createElement( "coordinatetransform" );
+  QDomElement myTransformElement = theDoc.createElement( "coordinatetransform" );
 
-  QDomElement mySourceElement  = theDoc.createElement( "sourcesrs" );
+  QDomElement mySourceElement = theDoc.createElement( "sourcesrs" );
   mSourceCRS.writeXML( mySourceElement, theDoc );
   myTransformElement.appendChild( mySourceElement );
 
-  QDomElement myDestElement  = theDoc.createElement( "destinationsrs" );
+  QDomElement myDestElement = theDoc.createElement( "destinationsrs" );
   mDestCRS.writeXML( myDestElement, theDoc );
   myTransformElement.appendChild( myDestElement );
 
@@ -559,7 +574,7 @@ const char *finder( const char *name )
 
 void QgsCoordinateTransform::setFinder()
 {
-#ifdef WIN32
+#if 0
   // Attention! It should be possible to set PROJ_LIB
   // but it can happen that it was previously set by installer
   // (version 0.7) and the old installation was deleted
@@ -572,4 +587,3 @@ void QgsCoordinateTransform::setFinder()
   pj_set_finder( finder );
 #endif
 }
-
