@@ -23,12 +23,12 @@
 #include <QStringList>
 #include <QRegExp>
 
-QgsDataSourceURI::QgsDataSourceURI() : mSSLmode( SSLprefer ), mKeyColumn( "" )
+QgsDataSourceURI::QgsDataSourceURI() : mSSLmode( SSLprefer ), mKeyColumn( "" ), mUseEstimatedMetadata( false )
 {
   // do nothing
 }
 
-QgsDataSourceURI::QgsDataSourceURI( QString uri ) : mSSLmode( SSLprefer ), mKeyColumn( "" )
+QgsDataSourceURI::QgsDataSourceURI( QString uri ) : mSSLmode( SSLprefer ), mKeyColumn( "" ), mUseEstimatedMetadata( false )
 {
   int i = 0;
   while ( i < uri.length() )
@@ -103,14 +103,22 @@ QgsDataSourceURI::QgsDataSourceURI( QString uri ) : mSSLmode( SSLprefer ), mKeyC
 
           i++;
         }
+        else
+        {
+          mGeometryColumn = QString::null;
+        }
       }
       else if ( pname == "key" )
       {
         mKeyColumn = pval;
       }
+      else if ( pname == "estimatedmetadata" )
+      {
+        mUseEstimatedMetadata = pval == "true";
+      }
       else if ( pname == "service" )
       {
-        QgsDebugMsg( "service keyword ignored" );
+        mService = pval;
       }
       else if ( pname == "user" )
       {
@@ -192,6 +200,11 @@ QString QgsDataSourceURI::removePassword( const QString& aUri )
     regexp.setPattern( " password=.* " );
     safeName.replace( regexp, " " );
   }
+  else if ( aUri.contains( ",password=" ) )
+  {
+    regexp.setPattern( ",password=.*," );
+    safeName.replace( regexp, "," );
+  }
   else if ( aUri.contains( "IDB:" ) )
   {
     regexp.setPattern( " pass=.* " );
@@ -216,6 +229,16 @@ QString QgsDataSourceURI::username() const
   return mUsername;
 }
 
+void QgsDataSourceURI::setUsername( QString username )
+{
+  mUsername = username;
+}
+
+QString QgsDataSourceURI::service() const
+{
+  return mService;
+}
+
 QString QgsDataSourceURI::host() const
 {
   return mHost;
@@ -231,6 +254,11 @@ QString QgsDataSourceURI::password() const
   return mPassword;
 }
 
+void QgsDataSourceURI::setPassword( QString password )
+{
+  mPassword = password;
+}
+
 QString QgsDataSourceURI::port() const
 {
   return mPort;
@@ -271,6 +299,17 @@ void QgsDataSourceURI::setKeyColumn( QString column )
   mKeyColumn = column;
 }
 
+
+void QgsDataSourceURI::setUseEstimatedMetadata( bool theFlag )
+{
+  mUseEstimatedMetadata = theFlag;
+}
+
+bool QgsDataSourceURI::useEstimatedMetadata() const
+{
+  return mUseEstimatedMetadata;
+}
+
 void QgsDataSourceURI::setSql( QString sql )
 {
   mSql = sql;
@@ -281,12 +320,12 @@ void QgsDataSourceURI::clearSchema()
   mSchema = "";
 }
 
-QString QgsDataSourceURI::escape( const QString &theVal ) const
+QString QgsDataSourceURI::escape( const QString &theVal, QChar delim = '\'' ) const
 {
   QString val = theVal;
 
   val.replace( "\\", "\\\\" );
-  val.replace( "\'", "\\'" );
+  val.replace( delim, QString( "\\%1" ).arg( delim ) );
 
   return val;
 }
@@ -324,7 +363,7 @@ QString QgsDataSourceURI::getValue( const QString &uri, int &i )
         i++;
         if ( i == uri.length() )
           continue;
-        if ( uri[i] != '\'' && uri[i] != '\\' )
+        if ( uri[i] != delim && uri[i] != '\\' )
           i--;
       }
       else if ( uri[i] == delim )
@@ -367,37 +406,46 @@ QString QgsDataSourceURI::getValue( const QString &uri, int &i )
 
 QString QgsDataSourceURI::connectionInfo() const
 {
-  QString connectionInfo = "dbname='" + escape( mDatabase ) + "'";
+  QStringList connectionItems;
 
-  if ( mHost != "" )
+  if ( mDatabase != "" )
   {
-    connectionInfo += " host=" + mHost;
+    connectionItems << "dbname='" + escape( mDatabase ) + "'";
+  }
+
+  if ( mService != "" )
+  {
+    connectionItems << "service='" + escape( mService ) + "'";
+  }
+  else if ( mHost != "" )
+  {
+    connectionItems << "host=" + mHost;
     if ( mPort != "" )
-      connectionInfo += " port=" + mPort;
+      connectionItems << "port=" + mPort;
   }
 
   if ( mUsername != "" )
   {
-    connectionInfo += " user='" + escape( mUsername ) + "'";
+    connectionItems << "user='" + escape( mUsername ) + "'";
 
     if ( mPassword != "" )
     {
-      connectionInfo += " password='" + escape( mPassword ) + "'";
+      connectionItems << "password='" + escape( mPassword ) + "'";
     }
   }
 
   if ( mSSLmode == SSLdisable )
-    connectionInfo += " sslmode=disable";
+    connectionItems << "sslmode=disable";
   else if ( mSSLmode == SSLallow )
-    connectionInfo += " sslmode=allow";
+    connectionItems << "sslmode=allow";
   else if ( mSSLmode == SSLrequire )
-    connectionInfo += " sslmode=require";
+    connectionItems << "sslmode=require";
 #if 0
   else if ( mSSLmode == SSLprefer )
-    connectionInfo += " sslmode=prefer";
+    connectionItems << "sslmode=prefer";
 #endif
 
-  return connectionInfo;
+  return connectionItems.join( " " );
 }
 
 QString QgsDataSourceURI::uri() const
@@ -409,9 +457,14 @@ QString QgsDataSourceURI::uri() const
     theUri += QString( " key='%1'" ).arg( escape( mKeyColumn ) );
   }
 
-  theUri += QString( " table=%1 (%2) sql=%3" )
+  if ( mUseEstimatedMetadata )
+  {
+    theUri += QString( " estimatedmetadata=true" );
+  }
+
+  theUri += QString( " table=%1%2 sql=%3" )
             .arg( quotedTablename() )
-            .arg( mGeometryColumn )
+            .arg( mGeometryColumn.isNull() ? QString() : QString( " (%1)" ).arg( mGeometryColumn ) )
             .arg( mSql );
 
   return theUri;
@@ -419,10 +472,13 @@ QString QgsDataSourceURI::uri() const
 
 QString QgsDataSourceURI::quotedTablename() const
 {
-  if ( mSchema != "" )
-    return QString( "\"%1\".\"%2\"" ).arg( mSchema ).arg( mTable );
+  if ( !mSchema.isEmpty() )
+    return QString( "\"%1\".\"%2\"" )
+           .arg( escape( mSchema, '"' ) )
+           .arg( escape( mTable, '"' ) );
   else
-    return QString( "\"%1\"" ).arg( mTable );
+    return QString( "\"%1\"" )
+           .arg( escape( mTable, '"' ) );
 }
 
 void QgsDataSourceURI::setConnection( const QString &host,
@@ -440,6 +496,19 @@ void QgsDataSourceURI::setConnection( const QString &host,
   mSSLmode = sslmode;
 }
 
+void QgsDataSourceURI::setConnection( const QString &service,
+                                      const QString &database,
+                                      const QString &username,
+                                      const QString &password,
+                                      SSLmode sslmode )
+{
+  mService = service;
+  mDatabase = database;
+  mUsername = username;
+  mPassword = password;
+  mSSLmode = sslmode;
+}
+
 void QgsDataSourceURI::setDataSource( const QString &schema,
                                       const QString &table,
                                       const QString &geometryColumn,
