@@ -41,9 +41,9 @@
 QgsDistanceArea::QgsDistanceArea()
 {
   // init with default settings
-  mProjectionsEnabled = FALSE;
+  mProjectionsEnabled = false;
   mCoordTransform = new QgsCoordinateTransform;
-  setSourceEpsgCrsId( GEO_EPSG_CRS_ID ); // WGS 84
+  setSourceCrs( GEOCRS_ID ); // WGS 84
   setEllipsoid( "WGS84" );
 }
 
@@ -69,10 +69,16 @@ void QgsDistanceArea::setSourceCrs( long srsid )
 void QgsDistanceArea::setSourceEpsgCrsId( long epsgId )
 {
   QgsCoordinateReferenceSystem srcCRS;
-  srcCRS.createFromEpsg( epsgId );
+  srcCRS.createFromOgcWmsCrs( QString( "EPSG:%1" ).arg( epsgId ) );
   mCoordTransform->setSourceCrs( srcCRS );
 }
 
+void QgsDistanceArea::setSourceAuthId( QString authId )
+{
+  QgsCoordinateReferenceSystem srcCRS;
+  srcCRS.createFromOgcWmsCrs( authId );
+  mCoordTransform->setSourceCrs( srcCRS );
+}
 
 bool QgsDistanceArea::setEllipsoid( const QString& ellipsoid )
 {
@@ -120,7 +126,7 @@ bool QgsDistanceArea::setEllipsoid( const QString& ellipsoid )
   // row for this ellipsoid wasn't found?
   if ( radius.isEmpty() || parameter2.isEmpty() )
   {
-    QgsDebugMsg( QString( "setEllipsoid: no row in tbl_ellipsoid for acronym '" ) + ellipsoid.toLocal8Bit().data() + "'" );
+    QgsDebugMsg( QString( "setEllipsoid: no row in tbl_ellipsoid for acronym '%1'" ).arg( ellipsoid ) );
     return false;
   }
 
@@ -129,7 +135,7 @@ bool QgsDistanceArea::setEllipsoid( const QString& ellipsoid )
     mSemiMajor = radius.mid( 2 ).toDouble();
   else
   {
-    QgsDebugMsg( QString( "setEllipsoid: wrong format of radius field: '" ) + radius.toLocal8Bit().data() + "'" );
+    QgsDebugMsg( QString( "setEllipsoid: wrong format of radius field: '%1'" ).arg( radius ) );
     return false;
   }
 
@@ -148,17 +154,15 @@ bool QgsDistanceArea::setEllipsoid( const QString& ellipsoid )
   }
   else
   {
-    QgsDebugMsg( QString( "setEllipsoid: wrong format of parameter2 field: '" ) + parameter2.toLocal8Bit().data() + "'" );
+    QgsDebugMsg( QString( "setEllipsoid: wrong format of parameter2 field: '%1'" ).arg( parameter2 ) );
     return false;
   }
 
-  QgsDebugMsg( QString( "setEllipsoid: a=" ) + mSemiMajor + ", b=" + mSemiMinor + ", 1/f=" + mInvFlattening );
+  QgsDebugMsg( QString( "setEllipsoid: a=%1, b=%2, 1/f=%3" ).arg( mSemiMajor ).arg( mSemiMinor ).arg( mInvFlattening ) );
 
 
   // get spatial ref system for ellipsoid
-  QString proj4 = "+proj=longlat +ellps=";
-  proj4 += ellipsoid;
-  proj4 += " +no_defs";
+  QString proj4 = "+proj=longlat +ellps=" + ellipsoid + " +no_defs";
   QgsCoordinateReferenceSystem destCRS;
   destCRS.createFromProj4( proj4 );
 
@@ -179,6 +183,9 @@ double QgsDistanceArea::measure( QgsGeometry* geometry )
     return 0.0;
 
   unsigned char* wkb = geometry->asWkb();
+  if ( !wkb )
+    return 0.0;
+
   unsigned char* ptr;
   unsigned int wkbType;
   double res, resTotal = 0;
@@ -214,7 +221,7 @@ double QgsDistanceArea::measure( QgsGeometry* geometry )
     case QGis::WKBPolygon25D:
       hasZptr = true;
     case QGis::WKBPolygon:
-      measurePolygon( wkb, &res, hasZptr );
+      measurePolygon( wkb, &res, 0, hasZptr );
       QgsDebugMsg( "returning " + QString::number( res ) );
       return res;
 
@@ -225,7 +232,60 @@ double QgsDistanceArea::measure( QgsGeometry* geometry )
       ptr = wkb + 9;
       for ( i = 0; i < count; i++ )
       {
-        ptr = measurePolygon( ptr, &res, hasZptr );
+        ptr = measurePolygon( ptr, &res, 0, hasZptr );
+        resTotal += res;
+      }
+      QgsDebugMsg( "returning " + QString::number( resTotal ) );
+      return resTotal;
+
+    default:
+      QgsDebugMsg( QString( "measure: unexpected geometry type: %1" ).arg( wkbType ) );
+      return 0;
+  }
+}
+
+double QgsDistanceArea::measurePerimeter( QgsGeometry* geometry )
+{
+  if ( !geometry )
+    return 0.0;
+
+  unsigned char* wkb = geometry->asWkb();
+  if ( !wkb )
+    return 0.0;
+
+  unsigned char* ptr;
+  unsigned int wkbType;
+  double res, resTotal = 0;
+  int count, i;
+
+  memcpy( &wkbType, ( wkb + 1 ), sizeof( wkbType ) );
+
+  // measure distance or area based on what is the type of geometry
+  bool hasZptr = false;
+
+  switch ( wkbType )
+  {
+    case QGis::WKBLineString25D:
+    case QGis::WKBLineString:
+    case QGis::WKBMultiLineString25D:
+    case QGis::WKBMultiLineString:
+      return 0.0;
+
+    case QGis::WKBPolygon25D:
+      hasZptr = true;
+    case QGis::WKBPolygon:
+      measurePolygon( wkb, 0, &res, hasZptr );
+      QgsDebugMsg( "returning " + QString::number( res ) );
+      return res;
+
+    case QGis::WKBMultiPolygon25D:
+      hasZptr = true;
+    case QGis::WKBMultiPolygon:
+      count = *(( int* )( wkb + 5 ) );
+      ptr = wkb + 9;
+      for ( i = 0; i < count; i++ )
+      {
+        ptr = measurePolygon( ptr, 0, &res, hasZptr );
         resTotal += res;
       }
       QgsDebugMsg( "returning " + QString::number( resTotal ) );
@@ -335,7 +395,7 @@ double QgsDistanceArea::measureLine( const QgsPoint& p1, const QgsPoint& p2 )
 }
 
 
-unsigned char* QgsDistanceArea::measurePolygon( unsigned char* feature, double* area, bool hasZptr )
+unsigned char* QgsDistanceArea::measurePolygon( unsigned char* feature, double* area, double* perimeter, bool hasZptr )
 {
   // get number of rings in the polygon
   unsigned int numRings = *(( int* )( feature + 1 + sizeof( int ) ) );
@@ -348,8 +408,11 @@ unsigned char* QgsDistanceArea::measurePolygon( unsigned char* feature, double*
 
   QList<QgsPoint> points;
   QgsPoint pnt;
-  double x, y, areaTmp;
-  *area = 0;
+  double x, y;
+  if ( area )
+    *area = 0;
+  if ( perimeter )
+    *perimeter = 0;
 
   try
   {
@@ -383,20 +446,38 @@ unsigned char* QgsDistanceArea::measurePolygon( unsigned char* feature, double*
 
       if ( points.size() > 2 )
       {
-        areaTmp = computePolygonArea( points );
-        if ( idx == 0 )
-          *area += areaTmp; // exterior ring
-        else
-          *area -= areaTmp; // interior rings
+        if ( area )
+        {
+          double areaTmp = computePolygonArea( points );
+          if ( idx == 0 )
+          {
+            // exterior ring
+            *area += areaTmp;
+          }
+          else
+          {
+            *area -= areaTmp; // interior rings
+          }
+        }
+
+        if ( perimeter )
+        {
+          *perimeter += measureLine( points );
+        }
       }
 
       points.clear();
+
+      if ( !area )
+      {
+        break;
+      }
     }
   }
   catch ( QgsCsException &cse )
   {
     Q_UNUSED( cse );
-    QgsLogger::warning( QObject::tr( "Caught a coordinate system exception while trying to transform a point. Unable to calculate polygon area." ) );
+    QgsLogger::warning( QObject::tr( "Caught a coordinate system exception while trying to transform a point. Unable to calculate polygon area or perimeter." ) );
   }
 
   return ptr;
@@ -434,14 +515,21 @@ double QgsDistanceArea::measurePolygon( const QList<QgsPoint>& points )
 double QgsDistanceArea::bearing( const QgsPoint& p1, const QgsPoint& p2 )
 {
   QgsPoint pp1 = p1, pp2 = p2;
+  double bearing;
+
   if ( mProjectionsEnabled && ( mEllipsoid != "NONE" ) )
   {
     pp1 = mCoordTransform->transform( p1 );
     pp2 = mCoordTransform->transform( p2 );
+    computeDistanceBearing( pp1, pp2, &bearing );
+  }
+  else //compute simple planar azimuth
+  {
+    double dx = p2.x() - p1.x();
+    double dy = p2.y() - p1.y();
+    bearing = atan2( dx, dy );
   }
 
-  double bearing;
-  computeDistanceBearing( pp1, pp2, &bearing );
   return bearing;
 }
 
@@ -485,7 +573,7 @@ double QgsDistanceArea::computeDistanceBearing(
   double tu2 = 0;
 
   int iterLimit = 20;
-  while ( fabs( lambda - lambdaP ) > 1e-12 && --iterLimit > 0 )
+  while ( qAbs( lambda - lambdaP ) > 1e-12 && --iterLimit > 0 )
   {
     sinLambda = sin( lambda );
     cosLambda = cos( lambda );
@@ -656,25 +744,28 @@ double QgsDistanceArea::computePolygonFlatArea( const QList<QgsPoint>& points )
   }
   // QgsDebugMsg("Area from point: " + (points[i % size]).toString(2));
   area = area / 2.0;
-  return fabs( area ); // All areas are positive!
+  return qAbs( area ); // All areas are positive!
 }
 
-QString QgsDistanceArea::textUnit( double value, int decimals, QGis::UnitType u, bool isArea )
+QString QgsDistanceArea::textUnit( double value, int decimals, QGis::UnitType u, bool isArea, bool keepBaseUnit )
 {
   QString unitLabel;
 
-
   switch ( u )
   {
     case QGis::Meters:
       if ( isArea )
       {
-        if ( fabs( value ) > 1000000.0 )
+        if ( keepBaseUnit )
+        {
+          unitLabel = QObject::tr( " m2" );
+        }
+        else if ( qAbs( value ) > 1000000.0 )
         {
           unitLabel = QObject::tr( " km2" );
           value = value / 1000000.0;
         }
-        else if ( fabs( value ) > 10000.0 )
+        else if ( qAbs( value ) > 10000.0 )
         {
           unitLabel = QObject::tr( " ha" );
           value = value / 10000.0;
@@ -686,23 +777,21 @@ QString QgsDistanceArea::textUnit( double value, int decimals, QGis::UnitType u,
       }
       else
       {
-        if ( fabs( value ) == 0.0 )
+        if ( keepBaseUnit || qAbs( value ) == 0.0 )
         {
-          // Special case for pretty printing.
           unitLabel = QObject::tr( " m" );
-
         }
-        else if ( fabs( value ) > 1000.0 )
+        else if ( qAbs( value ) > 1000.0 )
         {
           unitLabel = QObject::tr( " km" );
           value = value / 1000;
         }
-        else if ( fabs( value ) < 0.01 )
+        else if ( qAbs( value ) < 0.01 )
         {
           unitLabel = QObject::tr( " mm" );
           value = value * 1000;
         }
-        else if ( fabs( value ) < 0.1 )
+        else if ( qAbs( value ) < 0.1 )
         {
           unitLabel = QObject::tr( " cm" );
           value = value * 100;
@@ -716,29 +805,33 @@ QString QgsDistanceArea::textUnit( double value, int decimals, QGis::UnitType u,
     case QGis::Feet:
       if ( isArea )
       {
-        if ( fabs( value ) > ( 528.0*528.0 ) )
+        if ( keepBaseUnit  || qAbs( value ) <= ( 528.0*528.0 ) )
+        {
+          unitLabel = QObject::tr( " sq ft" );
+        }
+        else
         {
           unitLabel = QObject::tr( " sq mile" );
           value = value / ( 5280.0 * 5280.0 );
         }
-        else
-        {
-          unitLabel = QObject::tr( " sq ft" );
-        }
       }
       else
       {
-        if ( fabs( value ) > 528.0 )
+        if ( qAbs( value ) <= 528.0 || keepBaseUnit )
         {
-          unitLabel = QObject::tr( " mile" );
-          value = value / 5280.0;
+          if ( qAbs( value ) == 1.0 )
+          {
+            unitLabel = QObject::tr( " foot" );
+          }
+          else
+          {
+            unitLabel = QObject::tr( " feet" );
+          }
         }
         else
         {
-          if ( fabs( value ) == 1.0 )
-            unitLabel = QObject::tr( " foot" );
-          else
-            unitLabel = QObject::tr( " feet" );
+          unitLabel = QObject::tr( " mile" );
+          value = value / 5280.0;
         }
       }
       break;
@@ -749,7 +842,7 @@ QString QgsDistanceArea::textUnit( double value, int decimals, QGis::UnitType u,
       }
       else
       {
-        if ( fabs( value ) == 1.0 )
+        if ( qAbs( value ) == 1.0 )
           unitLabel = QObject::tr( " degree" );
         else
           unitLabel = QObject::tr( " degrees" );
@@ -763,5 +856,4 @@ QString QgsDistanceArea::textUnit( double value, int decimals, QGis::UnitType u,
 
 
   return QLocale::system().toString( value, 'f', decimals ) + unitLabel;
-
 }
