@@ -90,6 +90,9 @@ void QgsLabel::renderLabel( QgsRenderContext &renderContext,
                             QgsFeature &feature, bool selected,
                             QgsLabelAttributes *classAttributes )
 {
+  if ( mLabelAttributes->selectedOnly() && !selected )
+    return;
+
   QPen pen;
   QFont font;
   QString value;
@@ -207,6 +210,16 @@ void QgsLabel::renderLabel( QgsRenderContext &renderContext,
     font.setUnderline(( bool ) value.toInt() );
   }
 
+  value = fieldValue( StrikeOut, feature );
+  if ( value.isEmpty() )
+  {
+    font.setStrikeOut( mLabelAttributes->strikeOut() );
+  }
+  else
+  {
+    font.setStrikeOut(( bool ) value.toInt() );
+  }
+
   //
   QgsPoint overridePoint;
   bool useOverridePoint = false;
@@ -404,7 +417,6 @@ void QgsLabel::renderLabel( QgsRenderContext &renderContext,
   x = x + xoffset * cos( rad ) - yoffset * sin( rad );
   y = y - xoffset * sin( rad ) - yoffset * cos( rad );
 
-
   painter->save();
   painter->setFont( font );
   painter->translate( x, y );
@@ -832,6 +844,20 @@ void QgsLabel::readXML( const QDomNode& node )
     readLabelField( el, Underline );
   }
 
+  /* Strikeout */
+  scratchNode = node.namedItem( "strikeout" );
+
+  if ( scratchNode.isNull() )
+  {
+    QgsDebugMsg( "couldn't find QgsLabel ``strikeout'' attribute" );
+  }
+  else
+  {
+    el = scratchNode.toElement();
+    mLabelAttributes->setStrikeOut(( bool )el.attribute( "on", "0" ).toInt() );
+    readLabelField( el, StrikeOut );
+  }
+
   /* Color */
   scratchNode = node.namedItem( "color" );
 
@@ -993,6 +1019,18 @@ void QgsLabel::readXML( const QDomNode& node )
     readLabelField( el, MultilineEnabled );
   }
 
+  scratchNode = node.namedItem( "selectedonly" );
+
+  if ( scratchNode.isNull() )
+  {
+    QgsDebugMsg( "couldn't find QgsLabel ``selectedonly'' attribute" );
+  }
+  else
+  {
+    el = scratchNode.toElement();
+    mLabelAttributes->setSelectedOnly(( bool )el.attribute( "on", "0" ).toInt() );
+  }
+
 } // QgsLabel::readXML()
 
 
@@ -1130,6 +1168,27 @@ void QgsLabel::writeXML( QDomNode & layer_node, QDomDocument & document ) const
   }
   labelattributes.appendChild( underline );
 
+  // strikeout
+  QDomElement strikeOut = document.createElement( "strikeout" );
+  if ( mLabelAttributes->strikeOutIsSet() )
+  {
+    strikeOut.setAttribute( "on", mLabelAttributes->strikeOut() );
+    if ( mLabelFieldIdx[StrikeOut] != -1 )
+    {
+      strikeOut.setAttribute( "fieldname", labelField( StrikeOut ) );
+    }
+    else
+    {
+      strikeOut.setAttribute( "fieldname", "" );
+    }
+  }
+  else
+  {
+    strikeOut.setAttribute( "on", 0 );
+    strikeOut.setAttribute( "fieldname", "" );
+  }
+  labelattributes.appendChild( strikeOut );
+
   // color
   QDomElement color = document.createElement( "color" );
   if ( mLabelAttributes->colorIsSet() )
@@ -1312,6 +1371,17 @@ void QgsLabel::writeXML( QDomNode & layer_node, QDomDocument & document ) const
   }
   labelattributes.appendChild( multilineenabled );
 
+  QDomElement selectedonly = document.createElement( "selectedonly" );
+  if ( mLabelAttributes->selectedOnly() )
+  {
+    selectedonly.setAttribute( "on", mLabelAttributes->selectedOnly() );
+  }
+  else
+  {
+    selectedonly.setAttribute( "on", "" );
+  }
+  labelattributes.appendChild( selectedonly );
+
   layer_node.appendChild( labelattributes );
 }
 
