@@ -23,11 +23,11 @@
 #include <QSettings> // TODO: get rid of it [MD]
 #include <QDir>
 #include <QFile>
-#include <QFileInfo>
 #include <QDomDocument>
 #include <QDomElement>
 #include <QDomImplementation>
 #include <QTextStream>
+#include <QUrl>
 
 #include <sqlite3.h>
 
@@ -38,7 +38,6 @@
 #include "qgscoordinatereferencesystem.h"
 #include "qgsapplication.h"
 #include "qgsproject.h"
-#include "qgslogger.h"
 #include "qgsdatasourceuri.h"
 #include "qgsvectorlayer.h"
 
@@ -46,7 +45,7 @@ QgsMapLayer::QgsMapLayer( QgsMapLayer::LayerType type,
                           QString lyrname,
                           QString source ) :
     mTransparencyLevel( 255 ), // 0 is completely transparent
-    mValid( FALSE ), // assume the layer is invalid
+    mValid( false ), // assume the layer is invalid
     mDataSource( source ),
     mID( "" ),
     mLayerType( type )
@@ -95,16 +94,16 @@ QgsMapLayer::LayerType QgsMapLayer::type() const
 }
 
 /** Get this layer's unique ID */
-QString QgsMapLayer::getLayerID() const
+QString QgsMapLayer::id() const
 {
   return mID;
 }
 
 /** Write property of QString layerName. */
-void QgsMapLayer::setLayerName( const QString & _newVal )
+void QgsMapLayer::setLayerName( const QString & name )
 {
-  QgsDebugMsg( "new name is '" + _newVal + "'" );
-  mLayerName = capitaliseLayerName( _newVal );
+  QgsDebugMsg( "new name is '" + name + "'" );
+  mLayerName = capitaliseLayerName( name );
   emit layerNameChanged();
 }
 
@@ -177,6 +176,21 @@ bool QgsMapLayer::readXML( QDomNode & layer_node )
     theURIParts[0] = QgsProject::instance()->readPath( theURIParts[0] );
     mDataSource = theURIParts.join( "|" );
   }
+  else if ( provider == "delimitedtext" )
+  {
+    QUrl urlSource = QUrl::fromEncoded( mDataSource.toAscii() );
+
+    if ( !mDataSource.startsWith( "file:" ) )
+    {
+      QUrl file = QUrl::fromLocalFile( mDataSource.left( mDataSource.indexOf( "?" ) ) );
+      urlSource.setScheme( "file" );
+      urlSource.setPath( file.path() );
+    }
+
+    QUrl urlDest = QUrl::fromLocalFile( QgsProject::instance()->readPath( urlSource.toLocalFile() ) );
+    urlDest.setQueryItems( urlSource.queryItems() );
+    mDataSource = QString::fromAscii( urlDest.toEncoded() );
+  }
   else
   {
     mDataSource = QgsProject::instance()->readPath( mDataSource );
@@ -186,8 +200,12 @@ bool QgsMapLayer::readXML( QDomNode & layer_node )
   // Make it the saved CRS to have WMS layer projected correctly.
   // We will still overwrite whatever GDAL etc picks up anyway
   // further down this function.
+  mnl = layer_node.namedItem( "layername" );
+  mne = mnl.toElement();
+
   QDomNode srsNode = layer_node.namedItem( "srs" );
   mCRS->readXML( srsNode );
+  mCRS->setValidationHint( tr( "Specify CRS for layer %1" ).arg( mne.text() ) );
   mCRS->validate();
   savedCRS = *mCRS;
 
@@ -272,11 +290,11 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
   maplayer.setAttribute( "maximumScale", maximumScale() );
 
   // ID
-  QDomElement id = document.createElement( "id" );
-  QDomText idText = document.createTextNode( getLayerID() );
-  id.appendChild( idText );
+  QDomElement layerId = document.createElement( "id" );
+  QDomText layerIdText = document.createTextNode( id() );
+  layerId.appendChild( layerIdText );
 
-  maplayer.appendChild( id );
+  maplayer.appendChild( layerId );
 
   // data source
   QDomElement dataSource = document.createElement( "datasource" );
@@ -297,6 +315,13 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
     theURIParts[0] = QgsProject::instance()->writePath( theURIParts[0] );
     src = theURIParts.join( "|" );
   }
+  else if ( vlayer && vlayer->providerType() == "delimitedtext" )
+  {
+    QUrl urlSource = QUrl::fromEncoded( src.toAscii() );
+    QUrl urlDest = QUrl::fromLocalFile( QgsProject::instance()->writePath( urlSource.toLocalFile() ) );
+    urlDest.setQueryItems( urlSource.queryItems() );
+    src = QString::fromAscii( urlDest.toEncoded() );
+  }
   else
   {
     src = QgsProject::instance()->writePath( src );
@@ -315,6 +340,17 @@ bool QgsMapLayer::writeXML( QDomNode & layer_node, QDomDocument & document )
 
   maplayer.appendChild( layerName );
 
+  // timestamp if supported
+  if ( timestamp() > QDateTime() )
+  {
+    QDomElement stamp = document.createElement( "timestamp" );
+    QDomText stampText = document.createTextNode( timestamp().toString( Qt::ISODate ) );
+    stamp.appendChild( stampText );
+    maplayer.appendChild( stamp );
+  }
+
+  maplayer.appendChild( layerName );
+
   // zorder
   // This is no longer stored in the project file. It is superfluous since the layers
   // are written and read in the proper order.
@@ -582,7 +618,7 @@ QString QgsMapLayer::loadNamedStyle( const QString theURI, bool &theResultFlag )
   QDomElement myRoot = myDocument.firstChildElement( "qgis" );
   if ( myRoot.isNull() )
   {
-    myErrorMessage = "Error: qgis element could not be found in " + theURI;
+    myErrorMessage = tr( "Error: qgis element could not be found in %1" ).arg( theURI );
     theResultFlag = false;
     return myErrorMessage;
   }
@@ -660,6 +696,10 @@ QString QgsMapLayer::saveNamedStyle( const QString theURI, bool & theResultFlag
     QStringList theURIParts = theURI.split( "|" );
     filename = theURIParts[0];
   }
+  else if ( vlayer && vlayer->providerType() == "delimitedtext" )
+  {
+    filename = QUrl::fromEncoded( theURI.toAscii() ).toLocalFile();
+  }
   else
   {
     filename = theURI;
@@ -763,7 +803,6 @@ QString QgsMapLayer::saveNamedStyle( const QString theURI, bool & theResultFlag
       }
       else
       {
-        // try an update
         theResultFlag = false;
         myErrorMessage = tr( "The style %1 could not be inserted into database." ).arg( theURI );
       }
@@ -801,13 +840,35 @@ void QgsMapLayer::removeCustomProperty( const QString& key )
   mCustomProperties.remove( key );
 }
 
-void QgsMapLayer::readCustomProperties( QDomNode & layerNode )
+void QgsMapLayer::readCustomProperties( const QDomNode& layerNode, const QString& keyStartsWith )
 {
   QDomNode propsNode = layerNode.namedItem( "customproperties" );
   if ( propsNode.isNull() ) // no properties stored...
     return;
 
-  mCustomProperties.clear();
+  if ( !keyStartsWith.isEmpty() )
+  {
+    //remove old keys
+    QStringList keysToRemove;
+    QMap<QString, QVariant>::const_iterator pIt = mCustomProperties.constBegin();
+    for ( ; pIt != mCustomProperties.constEnd(); ++pIt )
+    {
+      if ( pIt.key().startsWith( keyStartsWith ) )
+      {
+        keysToRemove.push_back( pIt.key() );
+      }
+    }
+
+    QStringList::const_iterator sIt = keysToRemove.constBegin();
+    for ( ; sIt != keysToRemove.constEnd(); ++sIt )
+    {
+      mCustomProperties.remove( *sIt );
+    }
+  }
+  else
+  {
+    mCustomProperties.clear();
+  }
 
   QDomNodeList nodes = propsNode.childNodes();
 
@@ -819,17 +880,27 @@ void QgsMapLayer::readCustomProperties( QDomNode & layerNode )
     QDomElement propElement = propNode.toElement();
 
     QString key = propElement.attribute( "key" );
-    QString value = propElement.attribute( "value" );
-    mCustomProperties[key] = QVariant( value );
+    if ( key.isEmpty() || key.startsWith( keyStartsWith ) )
+    {
+      QString value = propElement.attribute( "value" );
+      mCustomProperties[key] = QVariant( value );
+    }
   }
 
 }
 
-void QgsMapLayer::writeCustomProperties( QDomNode & layerNode, QDomDocument & doc )
+void QgsMapLayer::writeCustomProperties( QDomNode & layerNode, QDomDocument & doc ) const
 {
+  //remove already existing <customproperties> tags
+  QDomNodeList propertyList = layerNode.toElement().elementsByTagName( "customproperties" );
+  for ( int i = 0; i < propertyList.size(); ++i )
+  {
+    layerNode.removeChild( propertyList.at( i ) );
+  }
+
   QDomElement propsElement = doc.createElement( "customproperties" );
 
-  for ( QMap<QString, QVariant>::const_iterator it = mCustomProperties.begin(); it != mCustomProperties.end(); ++it )
+  for ( QMap<QString, QVariant>::const_iterator it = mCustomProperties.constBegin(); it != mCustomProperties.constEnd(); ++it )
   {
     QDomElement propElement = doc.createElement( "property" );
     propElement.setAttribute( "key", it.key() );
@@ -843,6 +914,9 @@ void QgsMapLayer::writeCustomProperties( QDomNode & layerNode, QDomDocument & do
 void QgsMapLayer::setCacheImage( QImage * thepImage )
 {
   QgsDebugMsg( "cache Image set!" );
+  if ( mpCacheImage == thepImage )
+    return;
+
   if ( mpCacheImage )
   {
     delete mpCacheImage;
@@ -850,3 +924,17 @@ void QgsMapLayer::setCacheImage( QImage * thepImage )
   mpCacheImage = thepImage;
 }
 
+bool QgsMapLayer::isEditable() const
+{
+  return false;
+}
+
+void QgsMapLayer::setValid( bool valid )
+{
+  mValid = valid;
+}
+
+void QgsMapLayer::clearCacheImage()
+{
+  setCacheImage( 0 );
+}
