@@ -22,7 +22,6 @@
 #include "qgsmaplayer.h"
 #include "qgslogger.h"
 
-
 //
 // Static calls to enforce singleton behaviour
 //
@@ -73,16 +72,16 @@ QgsMapLayerRegistry::addMapLayer( QgsMapLayer * theMapLayer, bool theEmitSignal
   }
 
   //check the layer is not already registered!
-  QMap<QString, QgsMapLayer*>::iterator myIterator = mMapLayers.find( theMapLayer->getLayerID() );
+  QMap<QString, QgsMapLayer*>::iterator myIterator = mMapLayers.find( theMapLayer->id() );
   //if myIterator returns mMapLayers.end() then it does not exist in registry and its safe to add it
   if ( myIterator == mMapLayers.end() )
   {
-    mMapLayers[theMapLayer->getLayerID()] = theMapLayer;
+    mMapLayers[theMapLayer->id()] = theMapLayer;
 
     if ( theEmitSignal )
       emit layerWasAdded( theMapLayer );
 
-    return mMapLayers[theMapLayer->getLayerID()];
+    return mMapLayers[theMapLayer->id()];
   }
   else
   {
@@ -102,20 +101,20 @@ void QgsMapLayerRegistry::removeMapLayer( QString theLayerId, bool theEmitSignal
 
 void QgsMapLayerRegistry::removeAllMapLayers()
 {
-
   // moved before physically removing the layers
-  emit removedAll();            // now let all canvas Observers know to clear
-  // themselves, and then consequently any of
-  // their map legends
+  emit removedAll();
 
-  QMap<QString, QgsMapLayer *>::iterator it;
-  for ( it = mMapLayers.begin(); it != mMapLayers.end() ; ++it )
+  // now let all canvas observers know to clear themselves,
+  // and then consequently any of their map legends
+  while ( mMapLayers.size() > 0 )
   {
-    emit layerWillBeRemoved( it.key() );
-    delete it.value(); // delete the map layer
+    QString id = mMapLayers.begin().key();
+    emit layerWillBeRemoved( id );
+    delete mMapLayers[ id ]; // delete the map layer
+    mMapLayers.remove( id );
   }
-  mMapLayers.clear();
 
+  mMapLayers.clear();
 } // QgsMapLayerRegistry::removeAllMapLayers()
 
 //Added in QGIS 1.4
@@ -129,6 +128,19 @@ void QgsMapLayerRegistry::clearAllLayerCaches()
   }
 } // QgsMapLayerRegistry::clearAllLayerCaches()
 
+void QgsMapLayerRegistry::reloadAllLayers()
+{
+  QMap<QString, QgsMapLayer *>::iterator it;
+  for ( it = mMapLayers.begin(); it != mMapLayers.end() ; ++it )
+  {
+    QgsMapLayer* layer = it.value();
+    if ( layer )
+    {
+      layer->reload();
+    }
+  }
+}
+
 QMap<QString, QgsMapLayer*> & QgsMapLayerRegistry::mapLayers()
 {
   return mMapLayers;
