@@ -27,9 +27,10 @@
 #include "qgsmaplayerregistry.h"
 #include "qgsexception.h"
 #include "qgsprojectproperty.h"
-#include "qgslogger.h"
 #include "qgsprojectfiletransform.h"
 #include "qgsprojectversion.h"
+#include "qgspluginlayer.h"
+#include "qgspluginlayerregistry.h"
 
 #include <QApplication>
 #include <QFileInfo>
@@ -40,19 +41,9 @@
 
 static const char *const ident_ = "$Id$";
 
-
-
-
-
-// / canonical project instance
+// canonical project instance
 QgsProject * QgsProject::theProject_;
 
-
-
-
-
-
-
 /**
     Take the given scope and key and convert them to a string list of key
     tokens that will be used to navigate through a Property hierarchy
@@ -83,8 +74,9 @@ QStringList makeKeyTokens_( QString const &scope, QString const &key )
 /**
    return the property that matches the given key sequence, if any
 
-   @param sequence is a tokenized key sequence
-   @param p is likely to be the top level QgsPropertyKey in QgsProject:e:Imp.
+   @param scope scope of key
+   @param key keyname
+   @param rootProperty is likely to be the top level QgsPropertyKey in QgsProject:e:Imp.
 
    @return null if not found, otherwise located Property
 */
@@ -157,7 +149,8 @@ QgsProperty * findKey_( QString const & scope,
 
 /** add the given key and value
 
-@param sequence is list of keys
+@param scope scope of key
+@param key key name
 @param rootProperty is the property from which to start adding
 @param value the value associated with the key
 */
@@ -333,6 +326,7 @@ struct QgsProject::Imp
     // XXX THESE SHOULD BE MOVED TO STATUSBAR RELATED SOURCE
     QgsProject::instance()->writeEntry( "PositionPrecision", "/Automatic", true );
     QgsProject::instance()->writeEntry( "PositionPrecision", "/DecimalPlaces", 2 );
+    QgsProject::instance()->writeEntry( "Paths", "/Absolute", false );
   }
 
 };  // struct QgsProject::Imp
@@ -346,6 +340,7 @@ QgsProject::QgsProject()
   // XXX THESE SHOULD BE MOVED TO STATUSBAR RELATED SOURCE
   writeEntry( "PositionPrecision", "/Automatic", true );
   writeEntry( "PositionPrecision", "/DecimalPlaces", 2 );
+  writeEntry( "Paths", "/Absolute", false );
   // XXX writeEntry() makes the project dirty, but it doesn't make sense
   // for a new project to be dirty, so let's clean it up
   dirty( false );
@@ -433,10 +428,11 @@ static void dump_( QgsPropertyKey const & topQgsPropertyKey )
 
 Restore any optional properties found in "doc" to "properties".
 
-<properties> tags for all optional properties.  Within that there will be
-scope tags.  In the following example there exist one property in the
-"fsplugin" scope.  "layers" is a list containing three string values.
+properties tags for all optional properties.  Within that there will be scope
+tags.  In the following example there exist one property in the "fsplugin"
+scope.  "layers" is a list containing three string values.
 
+\verbatim
 <properties>
   <fsplugin>
     <foo type="int" >42</foo>
@@ -452,7 +448,9 @@ scope tags.  In the following example there exist one property in the
     </feature_types>
   </fsplugin>
 </properties>
+\endverbatim
 
+@param doc xml document
 @param project_properties should be the top QgsPropertyKey node.
 
 */
@@ -489,43 +487,44 @@ _getProperties( QDomDocument const &doc, QgsPropertyKey & project_properties )
     QgsDebugMsg( "Project_properties.readXML() failed" );
   }
 
+#if 0
 // DEPRECATED as functionality has been shoved down to QgsProperyKey::readXML()
+  size_t i = 0;
+  while ( i < scopes.count() )
+  {
+    QDomNode curr_scope_node = scopes.item( i );
 
-//     size_t i = 0;
-//     while (i < scopes.count())
-//     {
-//         QDomNode curr_scope_node = scopes.item(i);
+    qDebug( "found %d property node(s) for scope %s",
+            curr_scope_node.childNodes().count(),
+            curr_scope_node.nodeName().utf8().constData() );
 
-//         qDebug("found %d property node(s) for scope %s",
-//                curr_scope_node.childNodes().count(),
-//                curr_scope_node.nodeName().utf8().constData());
+    QString key( curr_scope_node.nodeName() );
 
-//         QString key(curr_scope_node.nodeName());
+    QgsPropertyKey * currentKey =
+      dynamic_cast<QgsPropertyKey*>( project_properties.find( key ) );
 
-//         QgsPropertyKey * currentKey =
-//             dynamic_cast<QgsPropertyKey*>(project_properties.find( key ));
+    if ( ! currentKey )
+    {
+      // if the property key doesn't yet exist, create an empty instance
+      // of that key
 
-//         if ( ! currentKey )
-//         {
-//             // if the property key doesn't yet exist, create an empty instance
-//             // of that key
+      currentKey = project_properties.addKey( key );
 
-//             currentKey = project_properties.addKey( key );
+      if ( ! currentKey )
+      {
+        qDebug( "%s:%d unable to add key", __FILE__, __LINE__ );
+      }
+    }
 
-//             if ( ! currentKey )
-//             {
-//                 qDebug( "%s:%d unable to add key", __FILE__, __LINE__ );
-//             }
-//         }
+    if ( ! currentKey->readXML( curr_scope_node ) )
+    {
+      qDebug( "%s:%d unable to read XML for property %s", __FILE__, __LINE__,
+              curr_scope_node.nodeName().utf8().constData() );
+    }
 
-//         if (! currentKey->readXML(curr_scope_node))
-//         {
-//             qDebug("%s:%d unable to read XML for property %s", __FILE__, __LINE__,
-//                    curr_scope_node.nodeName().utf8().constData());
-//         }
-
-//         ++i;
-//     }
+    ++i;
+  }
+#endif
 } // _getProperties
 
 
@@ -535,10 +534,12 @@ _getProperties( QDomDocument const &doc, QgsPropertyKey & project_properties )
    Get the project title
 
    XML in file has this form:
+\verbatim
    <qgis projectname="default project">
    <title>a project title</title>
+\endverbatim
 
-   @todo XXX we should go with the attribute xor <title>, not both.
+   @todo XXX we should go with the attribute xor title, not both.
 */
 static void _getTitle( QDomDocument const &doc, QString & title )
 {
@@ -602,12 +603,13 @@ static QgsProjectVersion _getVersion( QDomDocument const &doc )
    Read map layers from project file
 
 
-   @returns pair of < bool, list<QDomNode> >; bool is true if function worked;
-            else is false.  list contains nodes corresponding to layers that couldn't
-            be loaded
+   @returns \code QPair< bool, QList<QDomNode> > \endcode
+        bool is true if function worked; else is false.
+     list contains nodes corresponding to layers that couldn't be loaded
 
    @note XML of form:
 
+\verbatim
    <maplayer type="vector">
       <layername>Hydrop</layername>
       <datasource>/data/usgs/city_shp/hydrop.shp</datasource>
@@ -642,12 +644,12 @@ static QgsProjectVersion _getVersion( QDomDocument const &doc )
          <alignment value="center" field="" />
       </labelattributes>
    </maplayer>
-
+\endverbatim
 */
 QPair< bool, QList<QDomNode> > QgsProject::_getMapLayers( QDomDocument const &doc )
 {
   // Layer order is set by the restoring the legend settings from project file.
-  // This is done on the 'readProject( ... ) signal
+  // This is done on the 'readProject( ... )' signal
 
   QDomNodeList nl = doc.elementsByTagName( "maplayer" );
 
@@ -676,6 +678,10 @@ QPair< bool, QList<QDomNode> > QgsProject::_getMapLayers( QDomDocument const &do
 
   emit layerLoaded( 0, nl.count() );
 
+  //Collect vector layers with joins.
+  //They need to refresh join caches and symbology infos after all layers are loaded
+  QList< QPair< QgsVectorLayer*, QDomElement > > vLayerList;
+
   for ( int i = 0; i < nl.count(); i++ )
   {
     QDomNode node = nl.item( i );
@@ -695,6 +701,11 @@ QPair< bool, QList<QDomNode> > QgsProject::_getMapLayers( QDomDocument const &do
     {
       mapLayer = new QgsRasterLayer;
     }
+    else if ( type == "plugin" )
+    {
+      QString typeName = element.attribute( "name" );
+      mapLayer = QgsPluginLayerRegistry::instance()->createLayer( typeName );
+    }
 
     Q_CHECK_PTR( mapLayer );
 
@@ -706,9 +717,14 @@ QPair< bool, QList<QDomNode> > QgsProject::_getMapLayers( QDomDocument const &do
     }
 
     // have the layer restore state that is stored in Dom node
-    if ( mapLayer->readXML( node ) )
+    if ( mapLayer->readXML( node ) && mapLayer->isValid() )
     {
       mapLayer = QgsMapLayerRegistry::instance()->addMapLayer( mapLayer );
+      QgsVectorLayer* vLayer = qobject_cast<QgsVectorLayer*>( mapLayer );
+      if ( vLayer && vLayer->vectorJoins().size() > 0 )
+      {
+        vLayerList.push_back( qMakePair( vLayer, element ) );
+      }
     }
     else
     {
@@ -720,10 +736,24 @@ QPair< bool, QList<QDomNode> > QgsProject::_getMapLayers( QDomDocument const &do
 
       brokenNodes.push_back( node );
     }
-
     emit layerLoaded( i + 1, nl.count() );
   }
 
+  //Update field map of layers with joins and create join caches if necessary
+  //Needs to be done here once all dependent layers are loaded
+  QString errorMessage;
+  QList< QPair< QgsVectorLayer*, QDomElement > >::iterator vIt = vLayerList.begin();
+  for ( ; vIt != vLayerList.end(); ++vIt )
+  {
+    vIt->first->createJoinCaches();
+    vIt->first->updateFieldMap();
+    //for old symbology, it is necessary to read the symbology again after having the complete field map
+    if ( !vIt->first->isUsingRendererV2() )
+    {
+      vIt->first->readSymbology( vIt->second, errorMessage );
+    }
+  }
+
   return qMakePair( returnStatus, brokenNodes );
 
 } // _getMapLayers
@@ -739,14 +769,13 @@ bool QgsProject::read( QFileInfo const &file )
   imp_->file.setFileName( file.filePath() );
 
   return read();
-} // QgsProject::read( QFile & file )
+} // QgsProject::read
 
 
 
 /**
-   @note it's presumed that the caller has already reset the map canvas, map
-   registry, and legend
-*/
+   presuming that the caller has already reset the map canvas, map registry, and legend
+ */
 bool QgsProject::read()
 {
   clearError();
@@ -756,10 +785,10 @@ bool QgsProject::read()
 
   if ( !imp_->file.open( QIODevice::ReadOnly ) )
   {
-    imp_->file.close();     // even though we got an error, let's make
-    // sure it's closed anyway
+    imp_->file.close();
 
     setError( tr( "Unable to open %1" ).arg( imp_->file.fileName() ) );
+
     return false;
   }
 
@@ -770,9 +799,10 @@ bool QgsProject::read()
   if ( !doc->setContent( &imp_->file, &errorMsg, &line, &column ) )
   {
     // want to make this class as GUI independent as possible; so commented out
-    // QMessageBox::critical( 0x0, "Project File Read Error",
-    // errorMsg + " at line " + QString::number( line ) +
-    // " column " + QString::number( column ) );
+#if 0
+    QMessageBox::critical( 0, tr( "Project File Read Error" ),
+                           tr( "%1 at line %2 column %3" ).arg( errorMsg ).arg( line ).arg( column ) );
+#endif
 
     QString errorString = tr( "Project file read error: %1 at line %2 column %3" )
                           .arg( errorMsg ).arg( line ).arg( column );
@@ -782,6 +812,7 @@ bool QgsProject::read()
     imp_->file.close();
 
     setError( tr( "%1 for file %2" ).arg( errorString ).arg( imp_->file.fileName() ) );
+
     return false;
   }
 
@@ -840,8 +871,9 @@ bool QgsProject::read()
   QPair< bool, QList<QDomNode> > getMapLayersResults =  _getMapLayers( *doc );
 
   // review the integrity of the retrieved map layers
+  bool clean = getMapLayersResults.first;
 
-  if ( ! getMapLayersResults.first )
+  if ( !clean )
   {
     QgsDebugMsg( "Unable to get map layers from project file." );
 
@@ -858,8 +890,9 @@ bool QgsProject::read()
   // read the project: used by map canvas and legend
   emit readProject( *doc );
 
-  // can't be dirty since we're allegedly in pristine state
-  dirty( false );
+  // if all went well, we're allegedly in pristine state
+  if ( clean )
+    dirty( false );
 
   return true;
 
@@ -873,7 +906,7 @@ bool QgsProject::read( QDomNode & layerNode )
 {
   QString type = layerNode.toElement().attribute( "type" );
 
-  QgsMapLayer *mapLayer;
+  QgsMapLayer *mapLayer = NULL;
 
   if ( type == "vector" )
   {
@@ -883,6 +916,11 @@ bool QgsProject::read( QDomNode & layerNode )
   {
     mapLayer = new QgsRasterLayer;
   }
+  else if ( type == "plugin" )
+  {
+    QString typeName = layerNode.toElement().attribute( "name" );
+    mapLayer = QgsPluginLayerRegistry::instance()->createLayer( typeName );
+  }
   else
   {
     QgsDebugMsg( "bad layer type" );
@@ -1312,7 +1350,7 @@ void QgsProject::dumpProperties() const
 // return the absolute path from a filename read from project file
 QString QgsProject::readPath( QString src ) const
 {
-  if ( readBoolEntry( "Paths", "/Absolute", true ) )
+  if ( readBoolEntry( "Paths", "/Absolute", false ) )
   {
     return src;
   }
@@ -1322,6 +1360,7 @@ QString QgsProject::readPath( QString src ) const
   {
 #if defined(Q_OS_WIN)
     if ( src.startsWith( "\\\\" ) ||
+         src.startsWith( "//" ) ||
          ( src[0].isLetter() && src[1] == ':' ) )
     {
       // UNC or absolute path
@@ -1340,7 +1379,9 @@ QString QgsProject::readPath( QString src ) const
     // where the source file had to exist and only the project directory was stripped
     // from the filename.
     QFileInfo pfi( fileName() );
-    Q_ASSERT( pfi.exists() );
+    if ( !pfi.exists() )
+      return src;
+
     QFileInfo fi( pfi.canonicalPath() + "/" + src );
 
     if ( !fi.exists() )
@@ -1356,14 +1397,29 @@ QString QgsProject::readPath( QString src ) const
   QString srcPath = src;
   QString projPath = fileName();
 
+  if ( projPath.isEmpty() )
+  {
+    return src;
+  }
+
 #if defined(Q_OS_WIN)
   srcPath.replace( "\\", "/" );
   projPath.replace( "\\", "/" );
+
+  bool uncPath = projPath.startsWith( "//" );
 #endif
 
   QStringList srcElems = srcPath.split( "/", QString::SkipEmptyParts );
   QStringList projElems = projPath.split( "/", QString::SkipEmptyParts );
 
+#if defined(Q_OS_WIN)
+  if ( uncPath )
+  {
+    projElems.insert( 0, "" );
+    projElems.insert( 0, "" );
+  }
+#endif
+
   // remove project file element
   projElems.removeLast();
 
@@ -1391,7 +1447,7 @@ QString QgsProject::readPath( QString src ) const
 // return the absolute or relative path to write it to the project file
 QString QgsProject::writePath( QString src ) const
 {
-  if ( readBoolEntry( "Paths", "/Absolute", true ) )
+  if ( readBoolEntry( "Paths", "/Absolute", false ) || src.isEmpty() )
   {
     return src;
   }
@@ -1399,6 +1455,11 @@ QString QgsProject::writePath( QString src ) const
   QString srcPath = src;
   QString projPath = fileName();
 
+  if ( projPath.isEmpty() )
+  {
+    return src;
+  }
+
 #if defined( Q_OS_WIN )
   const Qt::CaseSensitivity cs = Qt::CaseInsensitive;
 
