@@ -21,6 +21,7 @@
 #include <limits>
 #include <QString>
 #include <QTextStream>
+#include <qnumeric.h>
 
 #include "qgspoint.h"
 #include "qgsrectangle.h"
@@ -149,11 +150,11 @@ bool QgsRectangle::intersects( const QgsRectangle& rect ) const
 {
   double x1 = ( xmin > rect.xmin ? xmin : rect.xmin );
   double x2 = ( xmax < rect.xmax ? xmax : rect.xmax );
-  if ( x1 > x2 ) return FALSE;
+  if ( x1 > x2 ) return false;
   double y1 = ( ymin > rect.ymin ? ymin : rect.ymin );
   double y2 = ( ymax < rect.ymax ? ymax : rect.ymax );
-  if ( y1 > y2 ) return FALSE;
-  return TRUE;
+  if ( y1 > y2 ) return false;
+  return true;
 }
 
 bool QgsRectangle::contains( const QgsRectangle& rect ) const
@@ -191,14 +192,7 @@ void QgsRectangle::combineExtentWith( double x, double y )
 
 bool QgsRectangle::isEmpty() const
 {
-  if ( xmax <= xmin || ymax <= ymin )
-  {
-    return TRUE;
-  }
-  else
-  {
-    return FALSE;
-  }
+  return xmax <= xmin || ymax <= ymin;
 }
 
 QString QgsRectangle::asWktCoordinates() const
@@ -220,7 +214,7 @@ QString QgsRectangle::toString( bool automaticPrecision ) const
     int precision = 0;
     if (( width() < 1 || height() < 1 ) && ( width() > 0 && height() > 0 ) )
     {
-      precision = static_cast<int>( ceil( -1.0 * log10( std::min( width(), height() ) ) ) ) + 1;
+      precision = static_cast<int>( ceil( -1.0 * log10( qMin( width(), height() ) ) ) ) + 1;
       // sanity check
       if ( precision > 20 )
         precision = 20;
@@ -235,17 +229,18 @@ QString QgsRectangle::toString( bool automaticPrecision ) const
 // Return a string representation of the rectangle with high precision
 QString QgsRectangle::toString( int thePrecision ) const
 {
+  QString rep;
+  if ( isEmpty() )
+    rep = "Empty";
+  else
+    rep = QString( "%1,%2 : %3,%4" )
+          .arg( xmin, 0, 'f', thePrecision )
+          .arg( ymin, 0, 'f', thePrecision )
+          .arg( xmax, 0, 'f', thePrecision )
+          .arg( ymax, 0, 'f', thePrecision );
+
+  QgsDebugMsgLevel( QString( "Extents : %1" ).arg( rep ), 4 );
 
-  QString rep = QString::number( xmin, 'f', thePrecision ) +
-                QString( "," ) +
-                QString::number( ymin, 'f', thePrecision ) +
-                QString( " : " ) +
-                QString::number( xmax, 'f', thePrecision ) +
-                QString( "," ) +
-                QString::number( ymax, 'f', thePrecision ) ;
-#ifdef QGISDEBUG
-// QgsDebugMsg(QString("Extents : %1").arg(rep));
-#endif
   return rep;
 }
 
@@ -313,18 +308,13 @@ void QgsRectangle::unionRect( const QgsRectangle& r )
 
 bool QgsRectangle::isFinite() const
 {
-  if ( std::numeric_limits<double>::has_infinity )
+  if ( qIsInf( xmin ) || qIsInf( ymin ) || qIsInf( xmax ) || qIsInf( ymax ) )
   {
-    if ( xmin == std::numeric_limits<double>::infinity() ||
-         xmax == std::numeric_limits<double>::infinity() ||
-         ymin == std::numeric_limits<double>::infinity() ||
-         ymax == std::numeric_limits<double>::infinity() )
-      return false;
-  }
-  // By design, if a variable is nan, it won't equal itself, so that's
-  // how we test for nan
-  if ( xmin != xmin || xmax != xmax || ymin != ymin || ymax != ymax )
     return false;
-
+  }
+  if ( qIsNaN( xmin ) || qIsNaN( ymin ) || qIsNaN( xmax ) || qIsNaN( ymax ) )
+  {
+    return false;
+  }
   return true;
 }
