@@ -21,8 +21,8 @@
 QgsRenderContext::QgsRenderContext()
     : mPainter( 0 ),
     mCoordTransform( 0 ),
-    mDrawEditingInformation( false ),
-    mForceVectorOutput( true ),
+    mDrawEditingInformation( true ),
+    mForceVectorOutput( false ),
     mRenderingStopped( false ),
     mScaleFactor( 1.0 ),
     mRasterScaleFactor( 1.0 ),
