@@ -23,7 +23,7 @@ originally part of the larger QgsRasterLayer class
 
 #include "qgscolorrampshader.h"
 
-#include <math.h>
+#include <cmath>
 
 QgsColorRampShader::QgsColorRampShader( double theMinimumValue, double theMaximumValue ) : QgsRasterShaderFunction( theMinimumValue, theMaximumValue )
 {
@@ -63,9 +63,10 @@ bool QgsColorRampShader::discreteColor( double theValue, int* theReturnRedValue,
   {
     //Start searching from the last index - assumtion is that neighboring pixels tend to be similar values
     myColorRampItem = mColorRampItemList.value( mCurrentColorRampItemIndex );
-    myTinyDiff = fabs( theValue - myColorRampItem.value );
+    myTinyDiff = qAbs( theValue - myColorRampItem.value );
     //If the previous entry is less, then search closer to the top of the list (assumes mColorRampItemList is sorted)
-    if ( mCurrentColorRampItemIndex != 0 && theValue <= mColorRampItemList.at( mCurrentColorRampItemIndex - 1 ).value )
+    if ( mCurrentColorRampItemIndex != 0 && 
+         theValue <= mColorRampItemList.at( mCurrentColorRampItemIndex - 1 ).value )
     {
       mCurrentColorRampItemIndex--;
     }
@@ -105,7 +106,7 @@ bool QgsColorRampShader::exactColor( double theValue, int* theReturnRedValue, in
   {
     //Start searching from the last index - assumtion is that neighboring pixels tend to be similar values
     myColorRampItem = mColorRampItemList.value( mCurrentColorRampItemIndex );
-    myTinyDiff = fabs( theValue - myColorRampItem.value );
+    myTinyDiff = qAbs( theValue - myColorRampItem.value );
     if ( theValue == myColorRampItem.value || myTinyDiff <= DOUBLE_DIFF_THRESHOLD )
     {
       *theReturnRedValue = myColorRampItem.color.red();
@@ -119,7 +120,9 @@ bool QgsColorRampShader::exactColor( double theValue, int* theReturnRedValue, in
       return true;
     }
     //pixel value sits between ramp entries so bail
-    else if ( mCurrentColorRampItemIndex != myColorRampItemCount - 1 && theValue > myColorRampItem.value && theValue < mColorRampItemList.at( mCurrentColorRampItemIndex + 1 ).value )
+    else if ( mCurrentColorRampItemIndex != myColorRampItemCount - 1 &&
+        theValue > myColorRampItem.value && theValue < mColorRampItemList.at(
+          mCurrentColorRampItemIndex + 1 ).value )
     {
       return false;
     }
@@ -138,7 +141,8 @@ bool QgsColorRampShader::exactColor( double theValue, int* theReturnRedValue, in
   return false; // value not found
 }
 
-bool QgsColorRampShader::interpolatedColor( double theValue, int* theReturnRedValue, int* theReturnGreenValue, int* theReturnBlueValue )
+bool QgsColorRampShader::interpolatedColor( double theValue, int*
+    theReturnRedValue, int* theReturnGreenValue, int* theReturnBlueValue )
 {
   int myColorRampItemCount = mColorRampItemList.count();
   if ( myColorRampItemCount <= 0 )
@@ -154,7 +158,7 @@ bool QgsColorRampShader::interpolatedColor( double theValue, int* theReturnRedVa
   {
     //Start searching from the last index - assumtion is that neighboring pixels tend to be similar values
     myColorRampItem = mColorRampItemList.value( mCurrentColorRampItemIndex );
-    myTinyDiff = fabs( theValue - myColorRampItem.value );
+    myTinyDiff = qAbs( theValue - myColorRampItem.value );
     //If the previous entry is less, then search closer to the top of the list (assumes mColorRampItemList is sorted)
     if ( mCurrentColorRampItemIndex != 0 && theValue <= mColorRampItemList.at( mCurrentColorRampItemIndex - 1 ).value )
     {
@@ -206,6 +210,13 @@ bool QgsColorRampShader::interpolatedColor( double theValue, int* theReturnRedVa
   return false;
 }
 
+void QgsColorRampShader::setColorRampItemList( const QList<QgsColorRampShader::ColorRampItem>& theList )
+{
+  mColorRampItemList = theList;
+  //Clear the cache
+  mColorCache.clear();
+}
+
 void QgsColorRampShader::setColorRampType( QgsColorRampShader::ColorRamp_TYPE theColorRampType )
 {
   //When the ramp type changes we need to clear out the cache
@@ -268,7 +279,9 @@ bool QgsColorRampShader::shade( double theValue, int* theReturnRedValue, int* th
   return discreteColor( theValue, theReturnRedValue, theReturnGreenValue, theReturnBlueValue );
 }
 
-bool QgsColorRampShader::shade( double theRedValue, double theGreenValue, double theBlueValue, int* theReturnRedValue, int* theReturnGreenValue, int* theReturnBlueValue )
+bool QgsColorRampShader::shade( double theRedValue, double theGreenValue,
+    double theBlueValue, int* theReturnRedValue, int* theReturnGreenValue, int*
+    theReturnBlueValue )
 {
   *theReturnRedValue = 0;
   *theReturnGreenValue = 0;
