@@ -25,7 +25,7 @@
 #include "qgsvectordataprovider.h"
 #include "qgsvectorlayer.h"
 #include "qgsrendercontext.h"
-#include <math.h>
+#include <cmath>
 #include <QDomNode>
 #include <QDomElement>
 #include <QImage>
@@ -145,7 +145,7 @@ void QgsGraduatedSymbolRenderer::renderFeature( QgsRenderContext &renderContext,
     {
       //first find out the value for the scale classification attribute
       const QgsAttributeMap& attrs = f.attributeMap();
-      fieldScale = sqrt( fabs( attrs[theSymbol->scaleClassificationField()].toDouble() ) );
+      fieldScale = sqrt( qAbs( attrs[theSymbol->scaleClassificationField()].toDouble() ) );
       QgsDebugMsgLevel( QString( "Feature has field scale factor %1" ).arg( fieldScale ), 3 );
     }
     if ( theSymbol->rotationClassificationField() >= 0 )
@@ -170,14 +170,7 @@ void QgsGraduatedSymbolRenderer::renderFeature( QgsRenderContext &renderContext,
 
     if ( theSymbol->pointSizeUnits() )
     {
-      /* Calc scale (still not nice) */
-      QgsPoint point;
-      point = renderContext.mapToPixel().transform( 0, 0 );
-      double x1 = point.x();
-      point = renderContext.mapToPixel().transform( 1000, 0 );
-      double x2 = point.x();
-
-      scale *= ( x2 - x1 ) * 0.001;
+      scale = 1.0 / renderContext.mapToPixel().mapUnitsPerPixel();
     }
 
     *img = theSymbol->getPointSymbolAsImage( scale, selected, mSelectionColor, fieldScale,
@@ -275,10 +268,10 @@ int QgsGraduatedSymbolRenderer::readXML( const QDomNode& rnode, QgsVectorLayer&
     mMode = QgsGraduatedSymbolRenderer::EqualInterval;
   }
 
-  int classificationId = theProvider->fieldNameIndex( classificationField );
+  int classificationId = vl.fieldNameIndex( classificationField );
   if ( classificationId == -1 )
   {
-    return 2; //@todo: handle gracefully in gui situation where user needs to nominate field
+    //go on. Because with joins, it might be the joined layer is not loaded yet
   }
   setClassificationField( classificationId );
 
