@@ -24,7 +24,7 @@
 #include "qgssymbol.h"
 #include "qgssymbologyutils.h"
 #include "qgslogger.h"
-#include <math.h>
+#include <cmath>
 #include <QDomNode>
 #include <QPainter>
 #include <QImage>
@@ -123,7 +123,7 @@ void QgsUniqueValueRenderer::renderFeature( QgsRenderContext &renderContext, Qgs
     {
       //first find out the value for the scale classification attribute
       const QgsAttributeMap& attrs = f.attributeMap();
-      fieldScale = sqrt( fabs( attrs[symbol->scaleClassificationField()].toDouble() ) );
+      fieldScale = sqrt( qAbs( attrs[symbol->scaleClassificationField()].toDouble() ) );
     }
     if ( symbol->rotationClassificationField() >= 0 )
     {
@@ -145,14 +145,7 @@ void QgsUniqueValueRenderer::renderFeature( QgsRenderContext &renderContext, Qgs
 
     if ( symbol->pointSizeUnits() )
     {
-      /* Calc scale (still not nice) */
-      QgsPoint point;
-      point = renderContext.mapToPixel().transform( 0, 0 );
-      double x1 = point.x();
-      point = renderContext.mapToPixel().transform( 1000, 0 );
-      double x2 = point.x();
-
-      scale *= ( x2 - x1 ) * 0.001;
+      scale = 1.0 / renderContext.mapToPixel().mapUnitsPerPixel();
     }
 
     *img = symbol->getPointSymbolAsImage( scale, selected, mSelectionColor,
@@ -235,7 +228,7 @@ int QgsUniqueValueRenderer::readXML( const QDomNode& rnode, QgsVectorLayer& vl )
   int classificationId = theProvider->fieldNameIndex( classificationField );
   if ( classificationId == -1 )
   {
-    return 2; //@todo: handle gracefully in gui situation where user needs to nominate field
+    //go on. Because with joins, it might be the joined layer is not loaded yet
   }
   setClassificationField( classificationId );
 
