@@ -7,6 +7,7 @@
 #include "qgsvectorcolorrampv2.h"
 
 #include "qgslogger.h"
+#include "qgsrendercontext.h"
 
 #include <QColor>
 #include <QDomNode>
@@ -16,15 +17,26 @@
 
 QString QgsSymbolLayerV2Utils::encodeColor( QColor color )
 {
-  return QString( "%1,%2,%3" ).arg( color.red() ).arg( color.green() ).arg( color.blue() );
+  return QString( "%1,%2,%3,%4" ).arg( color.red() ).arg( color.green() ).arg( color.blue() ).arg( color.alpha() );
 }
 
 QColor QgsSymbolLayerV2Utils::decodeColor( QString str )
 {
   QStringList lst = str.split( "," );
-  if ( lst.count() != 3 )
+  if ( lst.count() < 3 )
+  {
     return QColor();
-  return QColor( lst[0].toInt(), lst[1].toInt(), lst[2].toInt() );
+  }
+  int red, green, blue, alpha;
+  red = lst[0].toInt();
+  green = lst[1].toInt();
+  blue = lst[2].toInt();
+  alpha = 255;
+  if ( lst.count() > 3 )
+  {
+    alpha = lst[3].toInt();
+  }
+  return QColor( red, green, blue, alpha );
 }
 
 QString QgsSymbolLayerV2Utils::encodePenStyle( Qt::PenStyle style )
@@ -48,7 +60,7 @@ Qt::PenStyle QgsSymbolLayerV2Utils::decodePenStyle( QString str )
   if ( str == "dash" ) return Qt::DashLine;
   if ( str == "dot" ) return Qt::DotLine;
   if ( str == "dash dot" ) return Qt::DashDotLine;
-  if ( str == "dast dot dot" ) return Qt::DashDotDotLine;
+  if ( str == "dash dot dot" ) return Qt::DashDotDotLine;
   return Qt::SolidLine;
 }
 
@@ -147,6 +159,63 @@ QPointF QgsSymbolLayerV2Utils::decodePoint( QString str )
   return QPointF( lst[0].toDouble(), lst[1].toDouble() );
 }
 
+QString QgsSymbolLayerV2Utils::encodeOutputUnit( QgsSymbolV2::OutputUnit unit )
+{
+  switch ( unit )
+  {
+    case QgsSymbolV2::MM:
+      return "MM";
+    case QgsSymbolV2::MapUnit:
+      return "MapUnit";
+    default:
+      return "MM";
+  }
+}
+
+QgsSymbolV2::OutputUnit QgsSymbolLayerV2Utils::decodeOutputUnit( QString str )
+{
+  if ( str == "MM" )
+  {
+    return QgsSymbolV2::MM;
+  }
+  else if ( str == "MapUnit" )
+  {
+    return QgsSymbolV2::MapUnit;
+  }
+
+  // milimeters are default
+  return QgsSymbolV2::MM;
+}
+
+QString QgsSymbolLayerV2Utils::encodeRealVector( const QVector<qreal>& v )
+{
+  QString vectorString;
+  QVector<qreal>::const_iterator it = v.constBegin();
+  for ( ; it != v.constEnd(); ++it )
+  {
+    if ( it != v.constBegin() )
+    {
+      vectorString.append( ";" );
+    }
+    vectorString.append( QString::number( *it ) );
+  }
+  return vectorString;
+}
+
+QVector<qreal> QgsSymbolLayerV2Utils::decodeRealVector( const QString& s )
+{
+  QVector<qreal> resultVector;
+
+  QStringList realList = s.split( ";" );
+  QStringList::const_iterator it = realList.constBegin();
+  for ( ; it != realList.constEnd(); ++it )
+  {
+    resultVector.append( it->toDouble() );
+  }
+
+  return resultVector;
+}
+
 QIcon QgsSymbolLayerV2Utils::symbolPreviewIcon( QgsSymbolV2* symbol, QSize size )
 {
   return QIcon( symbolPreviewPixmap( symbol, size ) );
@@ -154,25 +223,29 @@ QIcon QgsSymbolLayerV2Utils::symbolPreviewIcon( QgsSymbolV2* symbol, QSize size
 
 QPixmap QgsSymbolLayerV2Utils::symbolPreviewPixmap( QgsSymbolV2* symbol, QSize size )
 {
+  Q_ASSERT( symbol );
+
   QPixmap pixmap( size );
+  pixmap.fill( Qt::transparent );
   QPainter painter;
   painter.begin( &pixmap );
   painter.setRenderHint( QPainter::Antialiasing );
-  painter.eraseRect( QRect( QPoint( 0, 0 ), size ) );
   symbol->drawPreviewIcon( &painter, size );
   painter.end();
   return pixmap;
 }
 
 
-QIcon QgsSymbolLayerV2Utils::symbolLayerPreviewIcon( QgsSymbolLayerV2* layer, QSize size )
+QIcon QgsSymbolLayerV2Utils::symbolLayerPreviewIcon( QgsSymbolLayerV2* layer, QgsSymbolV2::OutputUnit u, QSize size )
 {
   QPixmap pixmap( size );
+  pixmap.fill( Qt::transparent );
   QPainter painter;
   painter.begin( &pixmap );
   painter.setRenderHint( QPainter::Antialiasing );
-  painter.eraseRect( QRect( QPoint( 0, 0 ), size ) );
-  layer->drawPreviewIcon( &painter, size );
+  QgsRenderContext renderContext = createRenderContext( &painter );
+  QgsSymbolV2RenderContext symbolContext( renderContext, u );
+  layer->drawPreviewIcon( symbolContext, size );
   painter.end();
   return QIcon( pixmap );
 }
@@ -215,15 +288,17 @@ static bool lineInfo( QPointF p1, QPointF p2, double& angle, double& t )
     return false;
 
   // tangent
-  t = ( x1 == x2 ? t = DBL_MAX : ( y2 - y1 ) / ( x2 - x1 ) );
+  t = ( x1 == x2 ? DBL_MAX : ( y2 - y1 ) / ( x2 - x1 ) );
 
   // angle
   if ( t == DBL_MAX )
-    angle = ( y2 >= y1 ? M_PI / 2 : M_PI * 3 / 2 );  // angle is 90 or 270
+    angle = ( y2 > y1 ? M_PI / 2 : M_PI * 3 / 2 );  // angle is 90 or 270
+  else if ( t == 0 )
+    angle = ( x2 > x1 ? 0 : M_PI ); // angle is 0 or 180
   else if ( t >= 0 )
-    angle = ( y2 >= y1 ? atan( t ) : M_PI + atan( t ) );
+    angle = ( y2 > y1 ? atan( t ) : M_PI + atan( t ) );
   else // t < 0
-    angle = ( y2 >= y1 ? M_PI + atan( t ) : M_PI * 2 + atan( t ) );
+    angle = ( y2 > y1 ? M_PI + atan( t ) : atan( t ) );
 
   return true;
 }
@@ -237,8 +312,8 @@ static QPointF offsetPoint( QPointF pt, double angle, double dist )
 // calc intersection of two (infinite) lines defined by one point and tangent
 static QPointF linesIntersection( QPointF p1, double t1, QPointF p2, double t2 )
 {
-  // parallel lines?
-  if (( t1 == DBL_MAX && t2 == DBL_MAX ) || t1 == t2 )
+  // parallel lines? (or the difference between angles is less than cca 0.1 degree)
+  if (( t1 == DBL_MAX && t2 == DBL_MAX ) || qAbs( t1 - t2 ) < 0.001 )
     return QPointF();
 
   double x, y;
@@ -275,6 +350,7 @@ QPolygonF offsetLine( QPolygonF polyline, double dist )
   double angle = 0.0, t_new, t_old = 0;
   QPointF pt_old, pt_new;
   QPointF p1 = polyline[0], p2;
+  bool first_point = true;
 
   for ( int i = 1; i < polyline.count(); i++ )
   {
@@ -285,11 +361,12 @@ QPolygonF offsetLine( QPolygonF polyline, double dist )
 
     pt_new = offsetPoint( p1, angle + M_PI / 2, dist );
 
-    if ( i != 1 )
+    if ( ! first_point )
     {
       // if it's not the first line segment
       // calc intersection with last line (with offset)
-      pt_new = linesIntersection( pt_old, t_old, pt_new, t_new );
+      QPointF pt_tmp = linesIntersection( pt_old, t_old, pt_new, t_new );
+      if ( !pt_tmp.isNull() ) pt_new = pt_tmp;
     }
 
     newLine.append( pt_new );
@@ -297,6 +374,7 @@ QPolygonF offsetLine( QPolygonF polyline, double dist )
     pt_old = pt_new;
     t_old = t_new;
     p1 = p2;
+    first_point = false;
   }
 
   // last line segment:
@@ -339,17 +417,24 @@ QgsSymbolV2* QgsSymbolLayerV2Utils::loadSymbol( QDomElement& element )
   }
 
   QString symbolType = element.attribute( "type" );
+
+  QgsSymbolV2* symbol = 0;
   if ( symbolType == "line" )
-    return new QgsLineSymbolV2( layers );
+    symbol = new QgsLineSymbolV2( layers );
   else if ( symbolType == "fill" )
-    return new QgsFillSymbolV2( layers );
+    symbol = new QgsFillSymbolV2( layers );
   else if ( symbolType == "marker" )
-    return new QgsMarkerSymbolV2( layers );
+    symbol = new QgsMarkerSymbolV2( layers );
   else
   {
     QgsDebugMsg( "unknown symbol type " + symbolType );
     return NULL;
   }
+
+  symbol->setOutputUnit( decodeOutputUnit( element.attribute( "outputUnit" ) ) );
+  symbol->setAlpha( element.attribute( "alpha", "1.0" ).toDouble() );
+
+  return symbol;
 }
 
 QgsSymbolLayerV2* QgsSymbolLayerV2Utils::loadSymbolLayer( QDomElement& element )
@@ -389,10 +474,13 @@ static QString _nameForSymbolType( QgsSymbolV2::SymbolType type )
 
 QDomElement QgsSymbolLayerV2Utils::saveSymbol( QString name, QgsSymbolV2* symbol, QDomDocument& doc, QgsSymbolV2Map* subSymbols )
 {
+  Q_ASSERT( symbol );
+
   QDomElement symEl = doc.createElement( "symbol" );
   symEl.setAttribute( "type", _nameForSymbolType( symbol->type() ) );
   symEl.setAttribute( "name", name );
-
+  symEl.setAttribute( "outputUnit", encodeOutputUnit( symbol->outputUnit() ) );
+  symEl.setAttribute( "alpha", symbol->alpha() );
   QgsDebugMsg( "num layers " + QString::number( symbol->symbolLayerCount() ) );
   for ( int i = 0; i < symbol->symbolLayerCount(); i++ )
   {
@@ -589,3 +677,134 @@ QDomElement QgsSymbolLayerV2Utils::saveColorRamp( QString name, QgsVectorColorRa
   QgsSymbolLayerV2Utils::saveProperties( ramp->properties(), doc, rampEl );
   return rampEl;
 }
+
+double QgsSymbolLayerV2Utils::lineWidthScaleFactor( QgsRenderContext& c, QgsSymbolV2::OutputUnit u )
+{
+
+  if ( u == QgsSymbolV2::MM )
+  {
+    return c.scaleFactor();
+  }
+  else //QgsSymbol::MapUnit
+  {
+    double mup = c.mapToPixel().mapUnitsPerPixel();
+    if ( mup > 0 )
+    {
+      return 1.0 / mup;
+    }
+    else
+    {
+      return 1.0;
+    }
+  }
+}
+
+double QgsSymbolLayerV2Utils::pixelSizeScaleFactor( QgsRenderContext& c, QgsSymbolV2::OutputUnit u )
+{
+  if ( u == QgsSymbolV2::MM )
+  {
+    return ( c.scaleFactor() * c.rasterScaleFactor() );
+  }
+  else //QgsSymbol::MapUnit
+  {
+    double mup = c.mapToPixel().mapUnitsPerPixel();
+    if ( mup > 0 )
+    {
+      return c.rasterScaleFactor() / c.mapToPixel().mapUnitsPerPixel();
+    }
+    else
+    {
+      return 1.0;
+    }
+  }
+}
+
+QgsRenderContext QgsSymbolLayerV2Utils::createRenderContext( QPainter* p )
+{
+  QgsRenderContext context;
+  context.setPainter( p );
+  context.setRasterScaleFactor( 1.0 );
+  if ( p && p->device() )
+  {
+    context.setScaleFactor( p->device()->logicalDpiX() / 25.4 );
+  }
+  else
+  {
+    context.setScaleFactor( 3.465 ); //assume 88 dpi as standard value
+  }
+  return context;
+}
+
+void QgsSymbolLayerV2Utils::multiplyImageOpacity( QImage* image, qreal alpha )
+{
+  if ( !image )
+  {
+    return;
+  }
+
+  QRgb myRgb;
+  QImage::Format format = image->format();
+  if ( format != QImage::Format_ARGB32_Premultiplied && format != QImage::Format_ARGB32 )
+  {
+    QgsDebugMsg( "no alpha channel." );
+    return;
+  }
+
+  //change the alpha component of every pixel
+  for ( int heightIndex = 0; heightIndex < image->height(); ++heightIndex )
+  {
+    QRgb* scanLine = ( QRgb* )image->scanLine( heightIndex );
+    for ( int widthIndex = 0; widthIndex < image->width(); ++widthIndex )
+    {
+      myRgb = scanLine[widthIndex];
+      if ( format == QImage::Format_ARGB32_Premultiplied )
+        scanLine[widthIndex] = qRgba( alpha * qRed( myRgb ), alpha * qGreen( myRgb ), alpha * qBlue( myRgb ), alpha * qAlpha( myRgb ) );
+      else
+        scanLine[widthIndex] = qRgba( qRed( myRgb ), qGreen( myRgb ), qBlue( myRgb ), alpha * qAlpha( myRgb ) );
+    }
+  }
+}
+
+static bool _QVariantLessThan( const QVariant& lhs, const QVariant& rhs )
+{
+  switch( lhs.type() )
+  {
+  case QVariant::Int:
+    return lhs.toInt() < rhs.toInt();
+  case QVariant::UInt:
+    return lhs.toUInt() < rhs.toUInt();
+  case QVariant::LongLong:
+    return lhs.toLongLong() < rhs.toLongLong();
+  case QVariant::ULongLong:
+    return lhs.toULongLong() < rhs.toULongLong();
+  case QVariant::Double:
+    return lhs.toDouble() < rhs.toDouble();
+  case QVariant::Char:
+    return lhs.toChar() < rhs.toChar();
+  case QVariant::Date:
+    return lhs.toDate() < rhs.toDate();
+  case QVariant::Time:
+    return lhs.toTime() < rhs.toTime();
+  case QVariant::DateTime:
+    return lhs.toDateTime() < rhs.toDateTime();
+  default:
+    return QString::localeAwareCompare( lhs.toString(), rhs.toString() ) < 0;
+  }
+}
+
+static bool _QVariantGreaterThan( const QVariant& lhs, const QVariant& rhs )
+{
+  return ! _QVariantLessThan( lhs, rhs);
+}
+
+void QgsSymbolLayerV2Utils::sortVariantList( QList<QVariant>& list, Qt::SortOrder order )
+{
+  if (order == Qt::AscendingOrder)
+  {
+    qSort( list.begin(), list.end(), _QVariantLessThan );
+  }
+  else // Qt::DescendingOrder
+  {
+    qSort( list.begin(), list.end(), _QVariantGreaterThan );
+  }
+}
