@@ -12,26 +12,45 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-/* $Id: qgscolorbutton.h 6254 2006-12-15 07:54:15Z mhugent $ */
+/* $Id: qgscolorbutton.h 6251 2006-12-13 23:23:50Z telwertowski $ */
 #ifndef QGSCOLORBUTTON_H
 #define QGSCOLORBUTTON_H
 
 #include <QToolButton>
+#include <QPushButton>
 
-class QgsColorButton: public QToolButton
+/** \ingroup gui
+ * A cross platform button subclass for selecting colors.
+ */
+class GUI_EXPORT QgsColorButton: public QToolButton
 {
   public:
-    QgsColorButton(QWidget *parent = 0);
+    QgsColorButton( QWidget *parent = 0 );
     ~QgsColorButton();
 
-    void setColor(const QColor &color); 
+    void setColor( const QColor &color );
     QColor color() const { return mColor; }
 
   protected:
-    void paintEvent(QPaintEvent *e);
+    void paintEvent( QPaintEvent *e );
 
   private:
     QColor mColor;
 };
 
+
+class GUI_EXPORT QgsColorButtonV2 : public QPushButton
+{
+  public:
+    QgsColorButtonV2( QWidget* parent = 0 );
+    QgsColorButtonV2( QString text, QWidget* parent = 0 );
+
+    void setColor( const QColor &color );
+    QColor color() const { return mColor; }
+
+  private:
+    QColor mColor;
+};
+
+
 #endif
