@@ -28,9 +28,10 @@
 #include "qgscomposerpicture.h"
 #include "qgscomposerscalebar.h"
 #include "qgscomposershape.h"
+#include "qgscomposerattributetable.h"
 
 QgsComposerView::QgsComposerView( QWidget* parent, const char* name, Qt::WFlags f ) :
-    QGraphicsView( parent ), mShiftKeyPressed( false ), mRubberBandItem( 0 ), mRubberBandLineItem( 0 ), mMoveContentItem( 0 )
+    QGraphicsView( parent ), mShiftKeyPressed( false ), mRubberBandItem( 0 ), mRubberBandLineItem( 0 ), mMoveContentItem( 0 ), mPaintingEnabled( true )
 {
   setResizeAnchor( QGraphicsView::AnchorViewCenter );
   setMouseTracking( true );
@@ -105,6 +106,7 @@ void QgsComposerView::mousePressEvent( QMouseEvent* e )
       mRubberBandLineItem->setZValue( 100 );
       scene()->addItem( mRubberBandLineItem );
       scene()->update();
+      break;
     }
 
     //create rubber band for map and ellipse items
@@ -125,7 +127,7 @@ void QgsComposerView::mousePressEvent( QMouseEvent* e )
     case AddLabel:
     {
       QgsComposerLabel* newLabelItem = new QgsComposerLabel( composition() );
-      newLabelItem->setText( "Quantum GIS" );
+      newLabelItem->setText( tr( "Quantum GIS" ) );
       newLabelItem->adjustSizeToText();
       newLabelItem->setSceneRect( QRectF( snappedScenePoint.x(), snappedScenePoint.y(), newLabelItem->rect().width(), newLabelItem->rect().height() ) );
       addComposerLabel( newLabelItem );
@@ -136,8 +138,8 @@ void QgsComposerView::mousePressEvent( QMouseEvent* e )
     case AddScalebar:
     {
       QgsComposerScaleBar* newScaleBar = new QgsComposerScaleBar( composition() );
-      addComposerScaleBar( newScaleBar );
       newScaleBar->setSceneRect( QRectF( snappedScenePoint.x(), snappedScenePoint.y(), 20, 20 ) );
+      addComposerScaleBar( newScaleBar );
       emit actionFinished();
     }
     break;
@@ -145,16 +147,26 @@ void QgsComposerView::mousePressEvent( QMouseEvent* e )
     case AddLegend:
     {
       QgsComposerLegend* newLegend = new QgsComposerLegend( composition() );
-      addComposerLegend( newLegend );
       newLegend->setSceneRect( QRectF( snappedScenePoint.x(), snappedScenePoint.y(), newLegend->rect().width(), newLegend->rect().height() ) );
+      addComposerLegend( newLegend );
+      emit actionFinished();
       break;
     }
     case AddPicture:
     {
       QgsComposerPicture* newPicture = new QgsComposerPicture( composition() );
-      addComposerPicture( newPicture );
       newPicture->setSceneRect( QRectF( snappedScenePoint.x(), snappedScenePoint.y(), 30, 30 ) );
+      addComposerPicture( newPicture );
       emit actionFinished();
+      break;
+    }
+    case AddTable:
+    {
+      QgsComposerAttributeTable* newTable = new QgsComposerAttributeTable( composition() );
+      newTable->setSceneRect( QRectF( snappedScenePoint.x(), snappedScenePoint.y(), 50, 50 ) );
+      addComposerTable( newTable );
+      emit actionFinished();
+      break;
     }
 
     default:
@@ -192,7 +204,10 @@ void QgsComposerView::mouseReleaseEvent( QMouseEvent* e )
 
         double moveX = scenePoint.x() - mMoveContentStartPos.x();
         double moveY = scenePoint.y() - mMoveContentStartPos.y();
+
+        composition()->beginCommand( mMoveContentItem, tr( "Move item content" ) );
         mMoveContentItem->moveContent( -moveX, -moveY );
+        composition()->endCommand();
         mMoveContentItem = 0;
       }
       break;
@@ -224,6 +239,7 @@ void QgsComposerView::mouseReleaseEvent( QMouseEvent* e )
       addComposerShape( composerShape );
       scene()->removeItem( mRubberBandItem );
       delete mRubberBandItem;
+      mRubberBandItem = 0;
       emit actionFinished();
       break;
     }
@@ -232,8 +248,12 @@ void QgsComposerView::mouseReleaseEvent( QMouseEvent* e )
     {
       if ( !mRubberBandItem || mRubberBandItem->rect().width() < 0.1 || mRubberBandItem->rect().width() < 0.1 )
       {
-        scene()->removeItem( mRubberBandItem );
-        delete mRubberBandItem;
+        if ( mRubberBandItem )
+        {
+          scene()->removeItem( mRubberBandItem );
+          delete mRubberBandItem;
+          mRubberBandItem = 0;
+        }
         return;
       }
 
@@ -345,6 +365,11 @@ void QgsComposerView::mouseMoveEvent( QMouseEvent* e )
   }
 }
 
+void QgsComposerView::mouseDoubleClickEvent( QMouseEvent* e )
+{
+  e->ignore();
+}
+
 void QgsComposerView::keyPressEvent( QKeyEvent * e )
 {
   if ( e->key() == Qt::Key_Shift )
@@ -361,7 +386,7 @@ void QgsComposerView::keyPressEvent( QKeyEvent * e )
   QList<QgsComposerItem*>::iterator itemIt = composerItemList.begin();
 
   //delete selected items
-  if ( e->key() == Qt::Key_Delete )
+  if ( e->key() == Qt::Key_Delete || e->key() == Qt::Key_Backspace )
   {
     for ( ; itemIt != composerItemList.end(); ++itemIt )
     {
@@ -369,7 +394,30 @@ void QgsComposerView::keyPressEvent( QKeyEvent * e )
       if ( !map || !map->isDrawing() ) //don't delete a composer map while it draws
       {
         composition()->removeItem( *itemIt );
-        emit itemRemoved( *itemIt );
+        QgsComposerItemGroup* itemGroup = dynamic_cast<QgsComposerItemGroup*>( *itemIt );
+        if ( itemGroup && composition() )
+        {
+          //add add/remove item command for every item in the group
+          QUndoCommand* parentCommand = new QUndoCommand( tr( "Remove item group" ) );
+
+          QSet<QgsComposerItem*> groupedItems = itemGroup->items();
+          QSet<QgsComposerItem*>::iterator it = groupedItems.begin();
+          for ( ; it != groupedItems.end(); ++it )
+          {
+            QgsAddRemoveItemCommand* subcommand = new QgsAddRemoveItemCommand( QgsAddRemoveItemCommand::Removed, *it, composition(), "", parentCommand );
+            connectAddRemoveCommandSignals( subcommand );
+            emit itemRemoved( *it );
+          }
+
+          composition()->undoStack()->push( parentCommand );
+          delete itemGroup;
+          emit itemRemoved( itemGroup );
+        }
+        else
+        {
+          emit itemRemoved( *itemIt );
+          pushAddRemoveCommand( *itemIt, tr( "Item deleted" ), QgsAddRemoveItemCommand::Removed );
+        }
       }
     }
   }
@@ -423,11 +471,26 @@ void QgsComposerView::wheelEvent( QWheelEvent* event )
     if ( theItem->isSelected() )
     {
       QPointF itemPoint = theItem->mapFromScene( scenePoint );
+      theItem->beginCommand( tr( "Zoom item content" ) );
       theItem->zoomContent( event->delta(), itemPoint.x(), itemPoint.y() );
+      theItem->endCommand();
     }
   }
 }
 
+void QgsComposerView::paintEvent( QPaintEvent* event )
+{
+  if ( mPaintingEnabled )
+  {
+    QGraphicsView::paintEvent( event );
+    event->accept();
+  }
+  else
+  {
+    event->ignore();
+  }
+}
+
 void QgsComposerView::setComposition( QgsComposition* c )
 {
   setScene( c );
@@ -453,6 +516,7 @@ void QgsComposerView::addComposerArrow( QgsComposerArrow* arrow )
   scene()->clearSelection();
   arrow->setSelected( true );
   emit selectedItemChanged( arrow );
+  pushAddRemoveCommand( arrow, tr( "Arrow added" ) );
 }
 
 void QgsComposerView::addComposerLabel( QgsComposerLabel* label )
@@ -462,6 +526,7 @@ void QgsComposerView::addComposerLabel( QgsComposerLabel* label )
   scene()->clearSelection();
   label->setSelected( true );
   emit selectedItemChanged( label );
+  pushAddRemoveCommand( label, tr( "Label added" ) );
 }
 
 void QgsComposerView::addComposerMap( QgsComposerMap* map )
@@ -474,32 +539,40 @@ void QgsComposerView::addComposerMap( QgsComposerMap* map )
   scene()->clearSelection();
   map->setSelected( true );
   emit selectedItemChanged( map );
+  pushAddRemoveCommand( map, tr( "Map added" ) );
 }
 
 void QgsComposerView::addComposerScaleBar( QgsComposerScaleBar* scaleBar )
 {
-  //take first available map...
+  //take first available map
   QList<const QgsComposerMap*> mapItemList = composition()->composerMapItems();
   if ( mapItemList.size() > 0 )
   {
     scaleBar->setComposerMap( mapItemList.at( 0 ) );
   }
-  scaleBar->applyDefaultSettings(); //4 segments, 1/5 of composer map width
+  scaleBar->applyDefaultSize(); //4 segments, 1/5 of composer map width
   scene()->addItem( scaleBar );
   emit composerScaleBarAdded( scaleBar );
   scene()->clearSelection();
   scaleBar->setSelected( true );
   emit selectedItemChanged( scaleBar );
+  pushAddRemoveCommand( scaleBar, tr( "Scale bar added" ) );
 }
 
 void QgsComposerView::addComposerLegend( QgsComposerLegend* legend )
 {
+  //take first available map
+  QList<const QgsComposerMap*> mapItemList = composition()->composerMapItems();
+  if ( mapItemList.size() > 0 )
+  {
+    legend->setComposerMap( mapItemList.at( 0 ) );
+  }
   scene()->addItem( legend );
   emit composerLegendAdded( legend );
   scene()->clearSelection();
   legend->setSelected( true );
   emit selectedItemChanged( legend );
-  emit actionFinished();
+  pushAddRemoveCommand( legend, tr( "Legend added" ) );
 }
 
 void QgsComposerView::addComposerPicture( QgsComposerPicture* picture )
@@ -509,6 +582,7 @@ void QgsComposerView::addComposerPicture( QgsComposerPicture* picture )
   scene()->clearSelection();
   picture->setSelected( true );
   emit selectedItemChanged( picture );
+  pushAddRemoveCommand( picture, tr( "Picture added" ) );
 }
 
 void QgsComposerView::addComposerShape( QgsComposerShape* shape )
@@ -518,6 +592,17 @@ void QgsComposerView::addComposerShape( QgsComposerShape* shape )
   scene()->clearSelection();
   shape->setSelected( true );
   emit selectedItemChanged( shape );
+  pushAddRemoveCommand( shape, tr( "Shape added" ) );
+}
+
+void QgsComposerView::addComposerTable( QgsComposerAttributeTable* table )
+{
+  scene()->addItem( table );
+  emit composerTableAdded( table );
+  scene()->clearSelection();
+  table->setSelected( true );
+  emit selectedItemChanged( table );
+  pushAddRemoveCommand( table, tr( "Table added" ) );
 }
 
 void QgsComposerView::groupItems()
@@ -532,16 +617,14 @@ void QgsComposerView::groupItems()
   {
     return; //not enough items for a group
   }
-
   QgsComposerItemGroup* itemGroup = new QgsComposerItemGroup( composition() );
-  //connect signal/slot to let item group tell if child items get removed
-  connect( itemGroup, SIGNAL( childItemDeleted( QgsComposerItem* ) ), this, SLOT( sendItemRemovedSignal( QgsComposerItem* ) ) );
 
   QList<QgsComposerItem*>::iterator itemIter = selectionList.begin();
   for ( ; itemIter != selectionList.end(); ++itemIter )
   {
     itemGroup->addItem( *itemIter );
   }
+
   composition()->addItem( itemGroup );
   itemGroup->setSelected( true );
   emit selectedItemChanged( itemGroup );
@@ -567,17 +650,73 @@ void QgsComposerView::ungroupItems()
       emit itemRemoved( *itemIter );
     }
   }
-
 }
 
-void QgsComposerView::sendItemRemovedSignal( QgsComposerItem* item )
+void QgsComposerView::sendItemAddedSignal( QgsComposerItem* item )
 {
-  emit itemRemoved( item );
+  //cast and send proper signal
+  item->setSelected( true );
+  QgsComposerArrow* arrow = dynamic_cast<QgsComposerArrow*>( item );
+  if ( arrow )
+  {
+    emit composerArrowAdded( arrow );
+    emit selectedItemChanged( arrow );
+    return;
+  }
+  QgsComposerLabel* label = dynamic_cast<QgsComposerLabel*>( item );
+  if ( label )
+  {
+    emit composerLabelAdded( label );
+    emit selectedItemChanged( label );
+    return;
+  }
+  QgsComposerMap* map = dynamic_cast<QgsComposerMap*>( item );
+  if ( map )
+  {
+    emit composerMapAdded( map );
+    emit selectedItemChanged( map );
+    return;
+  }
+  QgsComposerScaleBar* scalebar = dynamic_cast<QgsComposerScaleBar*>( item );
+  if ( scalebar )
+  {
+    emit composerScaleBarAdded( scalebar );
+    emit selectedItemChanged( scalebar );
+    return;
+  }
+  QgsComposerLegend* legend = dynamic_cast<QgsComposerLegend*>( item );
+  if ( legend )
+  {
+    emit composerLegendAdded( legend );
+    emit selectedItemChanged( legend );
+    return;
+  }
+  QgsComposerPicture* picture = dynamic_cast<QgsComposerPicture*>( item );
+  if ( picture )
+  {
+    emit composerPictureAdded( picture );
+    emit selectedItemChanged( picture );
+    return;
+  }
+  QgsComposerShape* shape = dynamic_cast<QgsComposerShape*>( item );
+  if ( shape )
+  {
+    emit composerShapeAdded( shape );
+    emit selectedItemChanged( shape );
+    return;
+  }
+  QgsComposerAttributeTable* table = dynamic_cast<QgsComposerAttributeTable*>( item );
+  if ( table )
+  {
+    emit composerTableAdded( table );
+    emit selectedItemChanged( table );
+    return;
+  }
 }
 
 QMainWindow* QgsComposerView::composerWindow()
 {
-  QObject* composerObject = 0;
+  QMainWindow* composerObject = 0;
   QObject* currentObject = parent();
   if ( !currentObject )
   {
@@ -586,14 +725,35 @@ QMainWindow* QgsComposerView::composerWindow()
 
   while ( true )
   {
-    if ( currentObject->parent() == 0 )
+    composerObject = qobject_cast<QMainWindow*>( currentObject );
+    if ( composerObject || currentObject->parent() == 0 )
     {
-      composerObject = currentObject;
-      break;
+      return composerObject;
     }
     currentObject = currentObject->parent();
   }
 
-  return qobject_cast<QMainWindow *>( composerObject );
+  return 0;
 }
 
+void QgsComposerView::connectAddRemoveCommandSignals( QgsAddRemoveItemCommand* c )
+{
+  if ( !c )
+  {
+    return;
+  }
+  QObject::connect( c, SIGNAL( itemRemoved( QgsComposerItem* ) ), this, SIGNAL( itemRemoved( QgsComposerItem* ) ) );
+  QObject::connect( c, SIGNAL( itemAdded( QgsComposerItem* ) ), this, SLOT( sendItemAddedSignal( QgsComposerItem* ) ) );
+}
+
+void QgsComposerView::pushAddRemoveCommand( QgsComposerItem* item, const QString& text, QgsAddRemoveItemCommand::State state )
+{
+  if ( !composition() )
+  {
+    return;
+  }
+
+  QgsAddRemoveItemCommand* c = new QgsAddRemoveItemCommand( state, item, composition(), text );
+  connectAddRemoveCommandSignals( c );
+  composition()->undoStack()->push( c );
+}
