@@ -16,6 +16,8 @@
 #include "qgsencodingfiledialog.h"
 #include "qgsproject.h"
 #include "qgslogger.h"
+#include "qgsvectordataprovider.h"
+
 #include <QSettings>
 #include <QComboBox>
 #include <QPushButton>
@@ -23,7 +25,6 @@
 #include <QLayout>
 #include <QTextCodec>
 
-
 QgsEncodingFileDialog::QgsEncodingFileDialog( QWidget * parent,
     const QString & caption, const QString & directory,
     const QString & filter, const QString & encoding )
@@ -35,48 +36,8 @@ QgsEncodingFileDialog::QgsEncodingFileDialog( QWidget * parent,
   QLabel* l = new QLabel( tr( "Encoding:" ), this );
   layout()->addWidget( l );
   layout()->addWidget( mEncodingComboBox );
-  mEncodingComboBox->addItem( "BIG5" );
-  mEncodingComboBox->addItem( "BIG5-HKSCS" );
-  mEncodingComboBox->addItem( "EUCJP" );
-  mEncodingComboBox->addItem( "EUCKR" );
-  mEncodingComboBox->addItem( "GB2312" );
-  mEncodingComboBox->addItem( "GBK" );
-  mEncodingComboBox->addItem( "GB18030" );
-  mEncodingComboBox->addItem( "JIS7" );
-  mEncodingComboBox->addItem( "SHIFT-JIS" );
-  mEncodingComboBox->addItem( "TSCII" );
-  mEncodingComboBox->addItem( "UTF-8" );
-  mEncodingComboBox->addItem( "UTF-16" );
-  mEncodingComboBox->addItem( "KOI8-R" );
-  mEncodingComboBox->addItem( "KOI8-U" );
-  mEncodingComboBox->addItem( "ISO8859-1" );
-  mEncodingComboBox->addItem( "ISO8859-2" );
-  mEncodingComboBox->addItem( "ISO8859-3" );
-  mEncodingComboBox->addItem( "ISO8859-4" );
-  mEncodingComboBox->addItem( "ISO8859-5" );
-  mEncodingComboBox->addItem( "ISO8859-6" );
-  mEncodingComboBox->addItem( "ISO8859-7" );
-  mEncodingComboBox->addItem( "ISO8859-8" );
-  mEncodingComboBox->addItem( "ISO8859-8-I" );
-  mEncodingComboBox->addItem( "ISO8859-9" );
-  mEncodingComboBox->addItem( "ISO8859-10" );
-  mEncodingComboBox->addItem( "ISO8859-13" );
-  mEncodingComboBox->addItem( "ISO8859-14" );
-  mEncodingComboBox->addItem( "ISO8859-15" );
-  mEncodingComboBox->addItem( "IBM 850" );
-  mEncodingComboBox->addItem( "IBM 866" );
-  mEncodingComboBox->addItem( "CP874" );
-  mEncodingComboBox->addItem( "CP1250" );
-  mEncodingComboBox->addItem( "CP1251" );
-  mEncodingComboBox->addItem( "CP1252" );
-  mEncodingComboBox->addItem( "CP1253" );
-  mEncodingComboBox->addItem( "CP1254" );
-  mEncodingComboBox->addItem( "CP1255" );
-  mEncodingComboBox->addItem( "CP1256" );
-  mEncodingComboBox->addItem( "CP1257" );
-  mEncodingComboBox->addItem( "CP1258" );
-  mEncodingComboBox->addItem( "Apple Roman" );
-  mEncodingComboBox->addItem( "TIS-620" );
+
+  mEncodingComboBox->addItems( QgsVectorDataProvider::availableEncodings() );
 
   // Use default encoding if none supplied
   QString enc = encoding;
@@ -129,7 +90,7 @@ void QgsEncodingFileDialog::addCancelAll()
 {
   if ( ! mCancelAllButton )
   {
-    mCancelAllButton = new QPushButton( "Cancel &All", NULL );
+    mCancelAllButton = new QPushButton( tr( "Cancel &All" ), NULL );
     layout()->addWidget( mCancelAllButton ); // Ownership transfered, no need to delete later on
     connect( mCancelAllButton, SIGNAL( clicked() ), this, SLOT( pbnCancelAll_clicked() ) );
   }
