@@ -58,6 +58,11 @@ void QgsGenericProjectionSelector::setMessage( QString theMessage )
 QgsGenericProjectionSelector::~QgsGenericProjectionSelector()
 {}
 
+void QgsGenericProjectionSelector::setSelectedEpsg( long theId )
+{
+  projectionSelector->setSelectedAuthId( QString( "EPSG:%1" ).arg( theId ) );
+}
+
 void QgsGenericProjectionSelector::setSelectedCrsName( QString theName )
 {
   projectionSelector->setSelectedCrsName( theName );
@@ -68,30 +73,38 @@ void QgsGenericProjectionSelector::setSelectedCrsId( long theID )
   projectionSelector->setSelectedCrsId( theID );
 }
 
-void QgsGenericProjectionSelector::setSelectedEpsg( long theID )
+void QgsGenericProjectionSelector::setSelectedAuthId( QString theID )
 {
-  projectionSelector->setSelectedEpsg( theID );
+  projectionSelector->setSelectedAuthId( theID );
 }
 
 QString QgsGenericProjectionSelector::selectedProj4String()
 {
-  //@NOTE don't use getSelectedWkt as that just returns the name part!
+  //@note don't use getSelectedWkt as that just returns the name part!
   return projectionSelector->selectedProj4String();
 }
 
 long QgsGenericProjectionSelector::selectedCrsId()
 {
-  //@NOTE don't use getSelectedWkt as that just returns the name part!
+  //@note don't use getSelectedWkt as that just returns the name part!
   return projectionSelector->selectedCrsId();
 }
 
 long QgsGenericProjectionSelector::selectedEpsg()
 {
-  return projectionSelector->selectedEpsg();
+  QString authid = projectionSelector->selectedAuthId();
+  if ( authid.startsWith( "EPSG:", Qt::CaseInsensitive ) )
+    return authid.mid( 5 ).toLong();
+  else
+    return 0;
+}
+
+QString QgsGenericProjectionSelector::selectedAuthId()
+{
+  return projectionSelector->selectedAuthId();
 }
 
 void QgsGenericProjectionSelector::setOgcWmsCrsFilter( QSet<QString> crsFilter )
 {
   projectionSelector->setOgcWmsCrsFilter( crsFilter );
 }
-
