@@ -54,12 +54,12 @@ email                : sherman at mrcc.com
 #include "qgsvectorlayer.h"
 #include <math.h>
 
-/**  @DEPRECATED: to be deleted, stuff from here should be moved elsewhere */
+/**  @deprecated to be deleted, stuff from here should be moved elsewhere */
 class QgsMapCanvas::CanvasProperties
 {
   public:
 
-    CanvasProperties() : panSelectorDown( false ) { }
+    CanvasProperties() : mouseButtonDown( false ), panSelectorDown( false ) { }
 
     //!Flag to indicate status of mouse button
     bool mouseButtonDown;
@@ -78,9 +78,18 @@ class QgsMapCanvas::CanvasProperties
 
 
 QgsMapCanvas::QgsMapCanvas( QWidget * parent, const char *name )
-    : QGraphicsView( parent ),
-    mCanvasProperties( new CanvasProperties )
+    : QGraphicsView( parent )
+    , mCanvasProperties( new CanvasProperties )
+    , mNewSize( QSize() )
+    , mPainting( false )
+    , mAntiAliasing( false )
 {
+  //disable the update that leads to the resize crash
+  if ( viewport() )
+  {
+    viewport()->setAttribute( Qt::WA_PaintOnScreen, true );
+  }
+
   mScene = new QGraphicsScene();
   setScene( mScene );
   setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
@@ -110,7 +119,7 @@ QgsMapCanvas::QgsMapCanvas( QWidget * parent, const char *name )
   mScene->addItem( mMap );
   mScene->update(); // porting??
 
-  moveCanvasContents( TRUE );
+  moveCanvasContents( true );
 
   //connect(mMapRenderer, SIGNAL(updateMap()), this, SLOT(updateMap()));
   connect( mMapRenderer, SIGNAL( drawError( QgsMapLayer* ) ), this, SLOT( showError( QgsMapLayer* ) ) );
@@ -120,7 +129,7 @@ QgsMapCanvas::QgsMapCanvas( QWidget * parent, const char *name )
            this, SLOT( readProject( const QDomDocument & ) ) );
   connect( QgsProject::instance(), SIGNAL( writeProject( QDomDocument & ) ),
            this, SLOT( writeProject( QDomDocument & ) ) );
-
+  mMap->resize( size() );
 } // QgsMapCanvas ctor
 
 
@@ -155,6 +164,7 @@ QgsMapCanvas::~QgsMapCanvas()
 
 void QgsMapCanvas::enableAntiAliasing( bool theFlag )
 {
+  mAntiAliasing = theFlag;
   mMap->enableAntiAliasing( theFlag );
   if ( mMapOverview )
     mMapOverview->enableAntiAliasing( theFlag );
@@ -222,33 +232,38 @@ const QgsMapToPixel * QgsMapCanvas::getCoordinateTransform()
   return mMapRenderer->coordinateTransform();
 }
 
-void QgsMapCanvas::setLayerSet( QList<QgsMapCanvasLayer>& layers )
+void QgsMapCanvas::setLayerSet( QList<QgsMapCanvasLayer> &layers )
 {
   if ( mDrawing )
   {
     return;
   }
-  int i;
 
   // create layer set
   QStringList layerSet, layerSetOverview;
 
+  int i;
   for ( i = 0; i < layers.size(); i++ )
   {
-    QgsMapCanvasLayer& lyr = layers[i];
+    QgsMapCanvasLayer &lyr = layers[i];
+    if ( !lyr.layer() )
+    {
+      continue;
+    }
+
     if ( lyr.isVisible() )
     {
-      layerSet.push_back( lyr.layer()->getLayerID() );
+      layerSet.push_back( lyr.layer()->id() );
     }
     if ( lyr.isInOverview() )
     {
-      layerSetOverview.push_back( lyr.layer()->getLayerID() );
+      layerSetOverview.push_back( lyr.layer()->id() );
     }
   }
 
   QStringList& layerSetOld = mMapRenderer->layerSet();
 
-  bool layerSetChanged = ( layerSetOld != layerSet );
+  bool layerSetChanged = layerSetOld != layerSet;
 
   // update only if needed
   if ( layerSetChanged )
@@ -308,7 +323,7 @@ void QgsMapCanvas::setLayerSet( QList<QgsMapCanvasLayer>& layers )
     refresh();
   }
 
-} // addLayer
+} // setLayerSet
 
 void QgsMapCanvas::enableOverviewMode( QgsMapOverviewCanvas* overview )
 {
@@ -366,6 +381,9 @@ void QgsMapCanvas::refresh()
 
     // Tell the user we're going to be a while
     QApplication::setOverrideCursor( Qt::WaitCursor );
+
+    emit renderStarting();
+
     mMap->render();
 
     mDirty = false;
@@ -414,7 +432,11 @@ void QgsMapCanvas::saveAsImage( QString theFileName, QPixmap * theQPixmap, QStri
   }
   else //use the map view
   {
-    mMap->pixmap().save( theFileName, theFormat.toLocal8Bit().data() );
+    QPixmap *pixmap = dynamic_cast<QPixmap *>( &mMap->paintDevice() );
+    if ( !pixmap )
+      return;
+
+    pixmap->save( theFileName, theFormat.toLocal8Bit().data() );
   }
   //create a world file to go with the image...
   QgsRectangle myRect = mMapRenderer->extent();
@@ -435,7 +457,7 @@ void QgsMapCanvas::saveAsImage( QString theFileName, QPixmap * theQPixmap, QStri
   myHeader += QString::number( myRect.yMaximum() - ( mapUnitsPerPixel() / 2 ), 'g', 17 ) + "\r\n";
   QFileInfo myInfo  = QFileInfo( theFileName );
   // allow dotted names
-  QString myWorldFileName = myInfo.absolutePath() + QDir::separator() + myInfo.completeBaseName() + "." + theFormat + "w";
+  QString myWorldFileName = myInfo.absolutePath() + "/" + myInfo.completeBaseName() + "." + theFormat + "w";
   QFile myWorldFile( myWorldFileName );
   if ( !myWorldFile.open( QIODevice::WriteOnly ) ) //don't use QIODevice::Text
   {
@@ -535,7 +557,7 @@ void QgsMapCanvas::updateScale()
 void QgsMapCanvas::clear()
 {
   // Indicate to the next paint event that we need to rebuild the canvas contents
-  setDirty( TRUE );
+  setDirty( true );
 
 } // clear
 
@@ -581,6 +603,8 @@ void QgsMapCanvas::zoomToPreviousExtent()
     // update controls' enabled state
     emit zoomLastStatusChanged( mLastExtentIndex > 0 );
     emit zoomNextStatusChanged( mLastExtentIndex < mLastExtent.size() - 1 );
+    // notify canvas items of change
+    updateCanvasItemPositions();
   }
 
 } // zoomToPreviousExtent
@@ -603,6 +627,8 @@ void QgsMapCanvas::zoomToNextExtent()
     // update controls' enabled state
     emit zoomLastStatusChanged( mLastExtentIndex > 0 );
     emit zoomNextStatusChanged( mLastExtentIndex < mLastExtent.size() - 1 );
+    // notify canvas items of change
+    updateCanvasItemPositions();
   }
 }// zoomToNextExtent
 
@@ -709,8 +735,8 @@ void QgsMapCanvas::keyPressEvent( QKeyEvent * e )
     // Don't want to interfer with mouse events
 
     QgsRectangle currentExtent = mMapRenderer->extent();
-    double dx = fabs(( currentExtent.xMaximum() - currentExtent.xMinimum() ) / 4 );
-    double dy = fabs(( currentExtent.yMaximum() - currentExtent.yMinimum() ) / 4 );
+    double dx = qAbs(( currentExtent.xMaximum() - currentExtent.xMinimum() ) / 4 );
+    double dy = qAbs(( currentExtent.yMaximum() - currentExtent.yMinimum() ) / 4 );
 
     switch ( e->key() )
     {
@@ -894,8 +920,11 @@ void QgsMapCanvas::mouseReleaseEvent( QMouseEvent * e )
         QgsDebugMsg( "Right click in map tool zoom or pan, last tool is " +
                      QString( mLastNonZoomMapTool ? "not null." : "null." ) );
 
+        QgsVectorLayer *vlayer = qobject_cast<QgsVectorLayer *>( mCurrentLayer );
+
         // change to older non-zoom tool
-        if ( mLastNonZoomMapTool )
+        if ( mLastNonZoomMapTool
+             && ( !mLastNonZoomMapTool->isEditTool() || ( vlayer && vlayer->isEditable() ) ) )
         {
           QgsMapTool* t = mLastNonZoomMapTool;
           mLastNonZoomMapTool = NULL;
@@ -917,47 +946,54 @@ void QgsMapCanvas::mouseReleaseEvent( QMouseEvent * e )
 
 void QgsMapCanvas::resizeEvent( QResizeEvent * e )
 {
-  static bool isAlreadyIn = false;
-  static QSize lastSize = QSize( -1, -1 );
+  mNewSize = e->size();
+}
 
-  lastSize = e->size();
-
-  if ( isAlreadyIn || mDrawing )
+void QgsMapCanvas::paintEvent( QPaintEvent *e )
+{
+  if ( mNewSize.isValid() )
   {
-    //cancel current render progress
-    if ( mMapRenderer )
+    if ( mPainting || mDrawing )
     {
-      QgsRenderContext* theRenderContext = mMapRenderer->rendererContext();
-      if ( theRenderContext )
+      //cancel current render progress
+      if ( mMapRenderer )
       {
-        theRenderContext->setRenderingStopped( true );
+        QgsRenderContext* theRenderContext = mMapRenderer->rendererContext();
+        if ( theRenderContext )
+        {
+          theRenderContext->setRenderingStopped( true );
+        }
       }
+      return;
     }
-    return;
+
+    mPainting = true;
+
+    while ( mNewSize.isValid() )
+    {
+      QSize lastSize = mNewSize;
+      mNewSize = QSize();
+
+      //set map size before scene size helps keep scene indexes updated properly
+      // this was the cause of rubberband artifacts
+      mMap->resize( lastSize );
+      mScene->setSceneRect( QRectF( 0, 0, lastSize.width(), lastSize.height() ) );
+
+      // notify canvas items of change
+      updateCanvasItemPositions();
+
+      updateScale();
+
+      refresh();
+
+      emit extentsChanged();
+    }
+
+    mPainting = false;
   }
-  isAlreadyIn = true;
-
-  while ( lastSize != QSize( -1, -1 ) )
-  {
-    int width = lastSize.width();
-    int height = lastSize.height();
-    lastSize = QSize( -1, -1 );
-
-    //set map size before scene size helps keep scene indexes updated properly
-    // this was the cause of rubberband artifacts
-    mMap->resize( QSize( width, height ) );
-    mScene->setSceneRect( QRectF( 0, 0, width, height ) );
-
-    // notify canvas items of change
-    updateCanvasItemPositions();
-
-    updateScale();
-    refresh();
-    emit extentsChanged();
-  }
-  isAlreadyIn = false;
-} // resizeEvent
 
+  QGraphicsView::paintEvent( e );
+} // paintEvent
 
 void QgsMapCanvas::updateCanvasItemPositions()
 {
@@ -1044,6 +1080,11 @@ void QgsMapCanvas::zoomOut()
   zoomByFactor( mWheelZoomFactor );
 }
 
+void QgsMapCanvas::zoomScale( double newScale )
+{
+  zoomByFactor( newScale / scale() );
+}
+
 void QgsMapCanvas::zoomWithCenter( int x, int y, bool zoomIn )
 {
   if ( mDrawing )
@@ -1164,6 +1205,18 @@ int QgsMapCanvas::layerCount() const
 } // layerCount
 
 
+QList<QgsMapLayer*> QgsMapCanvas::layers() const
+{
+  QList<QgsMapLayer*> lst;
+  foreach( QString layerID, mMapRenderer->layerSet() )
+  {
+    QgsMapLayer* layer = QgsMapLayerRegistry::instance()->mapLayer( layerID );
+    if ( layer )
+      lst.append( layer );
+  }
+  return lst;
+}
+
 
 void QgsMapCanvas::layerStateChange()
 {
@@ -1188,10 +1241,33 @@ bool QgsMapCanvas::isFrozen()
 
 QPixmap& QgsMapCanvas::canvasPixmap()
 {
-  return mMap->pixmap();
+  QPixmap *pixmap = dynamic_cast<QPixmap *>( &canvasPaintDevice() );
+  if ( pixmap )
+  {
+    return *pixmap;
+  }
+
+  qWarning( "QgsMapCanvas::canvasPixmap() deprecated - returning static pixmap instance - use QgsMapCanvas::paintDevice()" );
+
+  static QPixmap staticPixmap;
+
+  QImage *image = dynamic_cast<QImage *>( &mMap->paintDevice() );
+  if ( image )
+  {
+    staticPixmap = QPixmap::fromImage( *image );
+  }
+  else
+  {
+    staticPixmap = QPixmap( canvasPaintDevice().width(), canvasPaintDevice().height() );
+  }
+
+  return staticPixmap;
 } // canvasPixmap
 
-
+QPaintDevice &QgsMapCanvas::canvasPaintDevice()
+{
+  return mMap->paintDevice();
+}
 
 double QgsMapCanvas::mapUnitsPerPixel() const
 {
@@ -1250,14 +1326,14 @@ void QgsMapCanvas::panActionEnd( QPoint releasePoint )
   }
 
   // move map image and other items to standard position
-  moveCanvasContents( TRUE ); // TRUE means reset
+  moveCanvasContents( true ); // true means reset
 
   // use start and end box points to calculate the extent
   QgsPoint start = getCoordinateTransform()->toMapCoordinates( mCanvasProperties->rubberStartPoint );
   QgsPoint end = getCoordinateTransform()->toMapCoordinates( releasePoint );
 
-  double dx = fabs( end.x() - start.x() );
-  double dy = fabs( end.y() - start.y() );
+  double dx = qAbs( end.x() - start.x() );
+  double dy = qAbs( end.y() - start.y() );
 
   // modify the extent
   QgsRectangle r = mMapRenderer->extent();
@@ -1415,8 +1491,7 @@ void QgsMapCanvas::zoomByFactor( double scaleFactor )
 void QgsMapCanvas::selectionChangedSlot()
 {
   // Find out which layer it was that sent the signal.
-  QgsMapLayer * layer = ( QgsMapLayer * )QObject::sender();
-
+  QgsMapLayer *layer = qobject_cast<QgsMapLayer *>( sender() );
   emit selectionChanged( layer );
   refresh();
 }
