@@ -17,7 +17,9 @@
 
 #include "qgsmapcanvasitem.h"
 #include "qgsmapcanvas.h"
+#include "qgsmaprenderer.h"
 #include "qgsmaptopixel.h"
+#include "qgsrendercontext.h"
 #include <QGraphicsScene>
 #include <QRect>
 #include <QPen>
@@ -40,6 +42,10 @@ void QgsMapCanvasItem::paint( QPainter * painter,
                               const QStyleOptionGraphicsItem * option,
                               QWidget * widget )
 {
+  if ( mMapCanvas->antiAliasingEnabled() )
+  {
+    painter->setRenderHint( QPainter::Antialiasing );
+  }
   paint( painter ); // call the derived item's drawing routines
 }
 
@@ -76,10 +82,8 @@ void QgsMapCanvasItem::setRect( const QgsRectangle& rect )
     r = r.normalized();
   }
 
-  // update the point prior to changing its position
-  update();
-
   // set position in canvas where the item will have coordinate (0,0)
+  prepareGeometryChange();
   setPos( r.topLeft() );
   mItemSize = QSizeF( r.width() + 2, r.height() + 2 );
 
@@ -101,6 +105,44 @@ void QgsMapCanvasItem::updateCanvas()
   //mMapCanvas->scene()->update(); //Contents();
 }
 
+bool QgsMapCanvasItem::setRenderContextVariables( QPainter* p, QgsRenderContext& context ) const
+{
+  if ( !mMapCanvas || !p )
+  {
+    return false;
+  }
+  QgsMapRenderer* mapRenderer = mMapCanvas->mapRenderer();
+  if ( !mapRenderer )
+  {
+    return false;
+  }
+
+  context.setPainter( p );
+  context.setRendererScale( mMapCanvas->scale() );
+
+  int dpi = mapRenderer->outputDpi();
+  int painterDpi = p->device()->logicalDpiX();
+  double scaleFactor = 1.0;
+  double rasterScaleFactor = 1.0;
+
+  //little trick to find out if painting origines from composer or main map canvas
+  if ( data( 0 ).toString() == "composer" )
+  {
+    rasterScaleFactor = painterDpi / 25.4;
+    scaleFactor = dpi / 25.4;
+  }
+  else
+  {
+    if ( mapRenderer->outputUnits() == QgsMapRenderer::Millimeters )
+    {
+      scaleFactor = dpi / 25.4;
+    }
+  }
+  context.setScaleFactor( scaleFactor );
+  context.setRasterScaleFactor( rasterScaleFactor );
+  return true;
+}
+
 void QgsMapCanvasItem::updatePosition()
 {
   // default implementation: recalculate position of the item
