@@ -44,9 +44,11 @@ QRectF QgsMapCanvasMap::boundingRect() const
 
 void QgsMapCanvasMap::resize( QSize size )
 {
+  QgsDebugMsg( QString( "resizing to %1x%2" ).arg( size.width() ).arg( size.height() ) );
   prepareGeometryChange(); // to keep QGraphicsScene indexes up to date on size change
 
   mPixmap = QPixmap( size );
+  mPixmap.fill( mBgColor.rgb() );
   mImage = QImage( size, QImage::Format_RGB32 ); // temporary image - build it here so it is available when switching from QPixmap to QImage rendering
   mCanvas->mapRenderer()->setOutputSize( size, mPixmap.logicalDpiX() );
 }
@@ -59,11 +61,6 @@ void QgsMapCanvasMap::setPanningOffset( const QPoint& point )
 
 void QgsMapCanvasMap::render()
 {
-  // Rendering to a QImage gives incorrectly filled polygons in some
-  // cases (as at Qt4.1.4), but it is the only renderer that supports
-  // anti-aliasing, so we provide the means to swap between QImage and
-  // QPixmap.
-
   if ( mUseQImageToRender )
   {
     // use temporary image for rendering
@@ -97,6 +94,11 @@ void QgsMapCanvasMap::render()
     paint.begin( &mPixmap );
     // Clip our drawing to the QPixmap
     paint.setClipRect( mPixmap.rect() );
+
+    // antialiasing
+    if ( mAntiAliasing )
+      paint.setRenderHint( QPainter::Antialiasing );
+
     mCanvas->mapRenderer()->render( &paint );
     paint.end();
   }
