@@ -1,5 +1,5 @@
 /***************************************************************************
- *   qgsprojectionselector.cpp
+ *   qgsprojectionselector.cpp                                             *
  *   Copyright (C) 2005 by Tim Sutton                                      *
  *   tim@linfiniti.com                                                     *
  *                                                                         *
@@ -16,7 +16,7 @@
 #include <sqlite3.h>
 
 //qgis includes
-#include "qgis.h" //magick numbers here
+#include "qgis.h" //magic numbers here
 #include "qgsapplication.h"
 #include "qgslogger.h"
 #include <qgscoordinatereferencesystem.h>
@@ -29,52 +29,57 @@
 #include <QResizeEvent>
 #include <QMessageBox>
 #include <QSettings>
-#include "qgslogger.h"
 
 const int NAME_COLUMN = 0;
-const int EPSG_COLUMN = 1;
+const int AUTHID_COLUMN = 1;
 const int QGIS_CRS_ID_COLUMN = 2;
-const int POPULAR_CRSES = 4;
 
-QgsProjectionSelector::QgsProjectionSelector( QWidget* parent,
-    const char * name,
-    Qt::WFlags fl )
-    : QWidget( parent, fl ),
-    mProjListDone( FALSE ),
-    mUserProjListDone( FALSE ),
-    mCRSNameSelectionPending( FALSE ),
-    mCRSIDSelectionPending( FALSE ),
-    mEPSGIDSelectionPending( FALSE )
+QgsProjectionSelector::QgsProjectionSelector( QWidget* parent, const char *name, Qt::WFlags fl )
+    : QWidget( parent, fl )
+    , mProjListDone( false )
+    , mUserProjListDone( false )
+    , mRecentProjListDone( false )
+    , mCRSNameSelectionPending( false )
+    , mCRSIDSelectionPending( false )
+    , mAuthIDSelectionPending( false )
 {
+  Q_UNUSED( name );
   setupUi( this );
   connect( lstCoordinateSystems, SIGNAL( currentItemChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ),
            this, SLOT( coordinateSystemSelected( QTreeWidgetItem* ) ) );
-  connect( leSearch, SIGNAL( returnPressed() ), pbnFind, SLOT( animateClick() ) );
 
   // Get the full path name to the sqlite3 spatial reference database.
   mSrsDatabaseFileName = QgsApplication::srsDbFilePath();
-  lstCoordinateSystems->header()->setResizeMode( EPSG_COLUMN, QHeaderView::Stretch );
+  lstCoordinateSystems->header()->setResizeMode( AUTHID_COLUMN, QHeaderView::Stretch );
   lstCoordinateSystems->header()->resizeSection( QGIS_CRS_ID_COLUMN, 0 );
   lstCoordinateSystems->header()->setResizeMode( QGIS_CRS_ID_COLUMN, QHeaderView::Fixed );
 
+  lstRecent->header()->setResizeMode( AUTHID_COLUMN, QHeaderView::Stretch );
+  lstRecent->header()->resizeSection( QGIS_CRS_ID_COLUMN, 0 );
+  lstRecent->header()->setResizeMode( QGIS_CRS_ID_COLUMN, QHeaderView::Fixed );
+
+  cbxAuthority->addItem( tr( "All" ) );
+  cbxAuthority->addItems( authorities() );
+
   // Read settings from persistent storage
   QSettings settings;
   mRecentProjections = settings.value( "/UI/recentProjections" ).toStringList();
-  /*** The reading (above) of internal id from persistent storage should be removed sometims in the future */
+  /*** The reading (above) of internal id from persistent storage should be removed sometime in the future */
   /*** This is kept now for backwards compatibility */
 
-  QStringList projectionsEpsg  = settings.value( "/UI/recentProjectionsEpsg" ).toStringList();
-  QStringList projectionsProj4 = settings.value( "/UI/recentProjectionsProj4" ).toStringList();
-  if ( projectionsEpsg.size() >= mRecentProjections.size() )
+  QStringList projectionsProj4  = settings.value( "/UI/recentProjectionsProj4" ).toStringList();
+  QStringList projectionsAuthId = settings.value( "/UI/recentProjectionsAuthId" ).toStringList();
+  if ( projectionsAuthId.size() >= mRecentProjections.size() )
   {
-    // We had saved state with EPSG and Proj4. Use that instead
-    // to find out the csr id
-    QgsDebugMsg( "Use popular projection list from EPSG/Proj4 saved state" );
+    // We had saved state with AuthId and Proj4. Use that instead
+    // to find out the crs id
+    QgsDebugMsg( "Use popular projection list from AuthId/Proj4 saved state" );
     mRecentProjections.clear();
-    for ( int i = 0; i <  projectionsEpsg.size(); i++ )
+    for ( int i = 0; i <  projectionsAuthId.size(); i++ )
     {
       // Create a crs from the EPSG
-      QgsCoordinateReferenceSystem crs( projectionsEpsg.at( i ).toLong(), QgsCoordinateReferenceSystem::EpsgCrsId );
+      QgsCoordinateReferenceSystem crs;
+      crs.createFromOgcWmsCrs( projectionsAuthId.at( i ) );
       if ( ! crs.isValid() )
       {
         // Couldn't create from EPSG, try the Proj4 string instead
@@ -110,10 +115,10 @@ QgsProjectionSelector::~QgsProjectionSelector()
     // Save to file *** Should be removed sometims in the future ***
     settings.setValue( "/UI/recentProjections", mRecentProjections );
 
-    // Convert to EPGS and proj4, and save those values also
+    // Convert to EPSG and proj4, and save those values also
 
-    QStringList projectionsEpsg;
     QStringList projectionsProj4;
+    QStringList projectionsAuthId;
     for ( int i = 0; i <  mRecentProjections.size(); i++ )
     {
       // Create a crs from the crsId
@@ -123,21 +128,24 @@ QgsProjectionSelector::~QgsProjectionSelector()
         // No? Skip this entry
         continue;
       }
-      projectionsEpsg  << QString::number( crs.epsg() );
       projectionsProj4 << crs.toProj4();
+      projectionsAuthId << crs.authid();
     }
-    settings.setValue( "/UI/recentProjectionsEpsg", projectionsEpsg );
     settings.setValue( "/UI/recentProjectionsProj4", projectionsProj4 );
+    settings.setValue( "/UI/recentProjectionsAuthId", projectionsAuthId );
   }
 }
 
 
 void QgsProjectionSelector::resizeEvent( QResizeEvent * theEvent )
 {
-
-  lstCoordinateSystems->header()->resizeSection( NAME_COLUMN, ( theEvent->size().width() - 120 ) );
-  lstCoordinateSystems->header()->resizeSection( EPSG_COLUMN, 120 );
+  lstCoordinateSystems->header()->resizeSection( NAME_COLUMN, theEvent->size().width() - 240 );
+  lstCoordinateSystems->header()->resizeSection( AUTHID_COLUMN, 240 );
   lstCoordinateSystems->header()->resizeSection( QGIS_CRS_ID_COLUMN, 0 );
+
+  lstRecent->header()->resizeSection( NAME_COLUMN, theEvent->size().width() - 240 );
+  lstRecent->header()->resizeSection( AUTHID_COLUMN, 240 );
+  lstRecent->header()->resizeSection( QGIS_CRS_ID_COLUMN, 0 );
 }
 
 void QgsProjectionSelector::showEvent( QShowEvent * theEvent )
@@ -157,65 +165,26 @@ void QgsProjectionSelector::showEvent( QShowEvent * theEvent )
 
   // check if a paricular projection is waiting
   // to be pre-selected, and if so, to select it now.
-  if ( mCRSNameSelectionPending )
+  if ( mCRSNameSelectionPending || mCRSIDSelectionPending || mAuthIDSelectionPending )
   {
-    applyCRSNameSelection();
-  }
-  if ( mCRSIDSelectionPending )
-  {
-    applyCRSIDSelection();
-  }
-  if ( mEPSGIDSelectionPending )
-  {
-    applyEPSGIDSelection();
+    applySelection();
   }
 
-  // Update buttons
-  pbnPopular1->setDisabled( true );
-  pbnPopular2->setDisabled( true );
-  pbnPopular3->setDisabled( true );
-  pbnPopular4->setDisabled( true );
-  pbnPopular1->hide();
-  pbnPopular2->hide();
-  pbnPopular3->hide();
-  pbnPopular4->hide();
-
-  if ( mRecentProjections.size() > 0 )
+  if ( !mRecentProjListDone )
   {
-    pbnPopular1->setText( getCrsIdName( mRecentProjections.at( 0 ).toLong() ) );
-    pbnPopular1->setDisabled( false );
-    pbnPopular1->show();
+    for ( int i = mRecentProjections.size() - 1; i >= 0; i-- )
+      insertRecent( mRecentProjections.at( i ).toLong() );
+    mRecentProjListDone = true;
   }
 
-  if ( mRecentProjections.size() > 1 )
-  {
-    pbnPopular2->setText( getCrsIdName( mRecentProjections.at( 1 ).toLong() ) );
-    pbnPopular2->setDisabled( false );
-    pbnPopular2->show();
-  }
-
-  if ( mRecentProjections.size() > 2 )
-  {
-    pbnPopular3->setText( getCrsIdName( mRecentProjections.at( 2 ).toLong() ) );
-    pbnPopular3->setDisabled( false );
-    pbnPopular3->show();
-  }
-
-  if ( mRecentProjections.size() > 3 )
-  {
-    pbnPopular4->setText( getCrsIdName( mRecentProjections.at( 3 ).toLong() ) );
-    pbnPopular4->setDisabled( false );
-    pbnPopular4->show();
-  }
-
-  // Pass up the inheritance heirarchy
+  // Pass up the inheritance hierarchy
   QWidget::showEvent( theEvent );
 }
 
 QString QgsProjectionSelector::ogcWmsCrsFilterAsSqlExpression( QSet<QString> * crsFilter )
 {
   QString sqlExpression = "1";             // it's "SQL" for "true"
-  QStringList epsgParts = QStringList();
+  QMap<QString, QStringList> authParts;
 
   if ( !crsFilter )
   {
@@ -240,39 +209,30 @@ QString QgsProjectionSelector::ogcWmsCrsFilterAsSqlExpression( QSet<QString> * c
 
   // iterate through all incoming CRSs
 
-  QSet<QString>::const_iterator i = crsFilter->begin();
-  QRegExp rx( "[^0-9]" );
-  while ( i != crsFilter->end() )
+  foreach( QString auth_id, crsFilter->values() )
   {
-    QStringList parts = i->split( ":" );
+    QStringList parts = auth_id.split( ":" );
 
-    if ( parts.at( 0 ) == "EPSG" && parts.size() >= 2 )
-    {
-      //this line is necessary to make change projection work
-      //with geoserver because for some reason geoserver returns
-      //EPSG:WGS84(DD) as the first srs and is invalid because the
-      //epsg database expect an integer string
-      if ( rx.indexIn( parts.at( 1 ) ) == -1 )
-        epsgParts.push_back( parts.at( 1 ) );
-    }
+    if ( parts.size() < 2 )
+      continue;
 
-    ++i;
+    authParts[ parts.at( 0 ).toUpper()].append( parts.at( 1 ).toUpper() );
   }
 
-//   for ( int i = 0; i < crsFilter->size(); i++ )
-//   {
-//     QStringList parts = crsFilter->at(i).split(":");
-//
-//     if (parts.at(0) == "EPSG")
-//     {
-//       epsgParts.push_back( parts.at(1) );
-//     }
-//   }
+  if ( authParts.isEmpty() )
+    return sqlExpression;
 
-  if ( epsgParts.size() )
+  if ( authParts.size() > 0 )
   {
-    sqlExpression = "epsg in (";
-    sqlExpression += epsgParts.join( "," );
+    QString prefix = " AND (";
+    foreach( QString auth_name, authParts.keys() )
+    {
+      sqlExpression += QString( "%1(upper(auth_name)='%2' AND upper(auth_id) IN ('%3'))" )
+                       .arg( prefix )
+                       .arg( auth_name )
+                       .arg( authParts[auth_name].join( "','" ) );
+      prefix = " OR ";
+    }
     sqlExpression += ")";
   }
 
@@ -285,13 +245,13 @@ QString QgsProjectionSelector::ogcWmsCrsFilterAsSqlExpression( QSet<QString> * c
 void QgsProjectionSelector::setSelectedCrsName( QString theCRSName )
 {
   mCRSNameSelection = theCRSName;
-  mCRSNameSelectionPending = TRUE;
-  mCRSIDSelectionPending = FALSE;  // only one type can be pending at a time
-  mEPSGIDSelectionPending = TRUE;
+  mCRSNameSelectionPending = true;
+  mCRSIDSelectionPending = false;  // only one type can be pending at a time
+  mAuthIDSelectionPending = true;
 
   if ( isVisible() )
   {
-    applyCRSNameSelection();
+    applySelection();
   }
   // else we will wait for the projection selector to
   // become visible (with the showEvent()) and set the
@@ -302,133 +262,92 @@ void QgsProjectionSelector::setSelectedCrsName( QString theCRSName )
 void QgsProjectionSelector::setSelectedCrsId( long theCRSID )
 {
   mCRSIDSelection = theCRSID;
-  mCRSIDSelectionPending = TRUE;
-  mCRSNameSelectionPending = FALSE;  // only one type can be pending at a time
-  mEPSGIDSelectionPending = FALSE;
+  mCRSIDSelectionPending = true;
+  mCRSNameSelectionPending = false;  // only one type can be pending at a time
+  mAuthIDSelectionPending = false;
 
   if ( isVisible() )
   {
-    applyCRSIDSelection();
+    applySelection();
   }
   // else we will wait for the projection selector to
   // become visible (with the showEvent()) and set the
   // selection there
 }
 
-void QgsProjectionSelector::setSelectedEpsg( long epsg )
+void QgsProjectionSelector::setSelectedEpsg( long id )
 {
-  mEPSGIDSelection = epsg;
-  mCRSIDSelectionPending = FALSE;
-  mEPSGIDSelectionPending = TRUE;
-  mCRSNameSelectionPending = FALSE;  // only one type can be pending at a time
+  setSelectedAuthId( QString( "EPSG:%1" ).arg( id ) );
 }
 
-void QgsProjectionSelector::applyCRSNameSelection()
+void QgsProjectionSelector::setSelectedAuthId( QString id )
 {
-  if (
-    ( mCRSNameSelectionPending ) &&
-    ( mProjListDone ) &&
-    ( mUserProjListDone )
-  )
+  mAuthIDSelection = id;
+  mCRSIDSelectionPending = false;
+  mAuthIDSelectionPending = true;
+  mCRSNameSelectionPending = false;  // only one type can be pending at a time
+}
+
+void QgsProjectionSelector::applySelection()
+{
+  if ( !mProjListDone || !mUserProjListDone )
+    return;
+
+  QList<QTreeWidgetItem*> nodes;
+  if ( mCRSNameSelectionPending )
   {
     //get the srid given the wkt so we can pick the correct list item
     QgsDebugMsg( "called with " + mCRSNameSelection );
-    QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( mCRSNameSelection, Qt::MatchExactly | Qt::MatchRecursive, 0 );
+    nodes = lstCoordinateSystems->findItems( mCRSNameSelection, Qt::MatchExactly | Qt::MatchRecursive, 0 );
 
-    if ( nodes.count() > 0 )
-    {
-      lstCoordinateSystems->setCurrentItem( nodes.first() );
-      lstCoordinateSystems->scrollToItem( nodes.first() );
-    }
-    else // unselect the selected item to avoid confusing the user
-    {
-      lstCoordinateSystems->clearSelection();
-      teProjection->setText( "" );
-    }
-
-    mCRSNameSelectionPending = FALSE;
+    mCRSNameSelectionPending = false;
   }
-}
 
-QString QgsProjectionSelector::getCrsIdName( long theCrsId )
-{
-  QString retvalue( "" );
-  if (
-    ( mProjListDone ) &&
-    ( mUserProjListDone )
-  )
-  {
-    QString myCRSIDString = QString::number( theCrsId );
-
-    QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( myCRSIDString, Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
-
-    if ( nodes.count() > 0 )
-    {
-      retvalue = nodes.first()->text( NAME_COLUMN );
-      if ( nodes.first()->text( EPSG_COLUMN ) != "" )
-      {
-        retvalue += QString( " (EPSG : %1)" ).arg( nodes.first()->text( EPSG_COLUMN ) );
-      }
-    }
-  }
-  return retvalue;
-
-}
-
-void QgsProjectionSelector::applyEPSGIDSelection()
-{
-  if (
-    ( mEPSGIDSelectionPending ) &&
-    ( mProjListDone ) &&
-    ( mUserProjListDone )
-  )
+  if ( mAuthIDSelectionPending )
   {
     //get the srid given the wkt so we can pick the correct list item
-    QgsDebugMsg( "called with " + QString::number( mEPSGIDSelection ) );
-    QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( QString::number( mEPSGIDSelection ), Qt::MatchExactly | Qt::MatchRecursive, EPSG_COLUMN );
+    QgsDebugMsg( "called with " + mAuthIDSelection );
+    nodes = lstCoordinateSystems->findItems( mAuthIDSelection, Qt::MatchExactly | Qt::MatchRecursive, AUTHID_COLUMN );
 
-    if ( nodes.count() > 0 )
-    {
-      lstCoordinateSystems->setCurrentItem( nodes.first() );
-      lstCoordinateSystems->scrollToItem( nodes.first() );
-    }
-    else // unselect the selected item to avoid confusing the user
-    {
-      lstCoordinateSystems->clearSelection();
-      teProjection->setText( "" );
-    }
-
-    mEPSGIDSelectionPending = FALSE;
+    mAuthIDSelectionPending = false;
   }
-}
 
-void QgsProjectionSelector::applyCRSIDSelection()
-{
-  if (
-    ( mCRSIDSelectionPending ) &&
-    ( mProjListDone ) &&
-    ( mUserProjListDone )
-  )
+  if ( mCRSIDSelectionPending )
   {
     QString myCRSIDString = QString::number( mCRSIDSelection );
 
-    QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( myCRSIDString, Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
+    nodes = lstCoordinateSystems->findItems( myCRSIDString, Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
 
-    if ( nodes.count() > 0 )
-    {
-      lstCoordinateSystems->setCurrentItem( nodes.first() );
-      lstCoordinateSystems->scrollToItem( nodes.first() );
-    }
-    else // unselect the selected item to avoid confusing the user
-    {
-      lstCoordinateSystems->clearSelection();
-      teProjection->setText( "" );
-    }
+    mCRSIDSelectionPending = false;
+  }
 
-    mCRSIDSelectionPending = FALSE;
+  if ( nodes.count() > 0 )
+  {
+    lstCoordinateSystems->setCurrentItem( nodes.first() );
+    lstCoordinateSystems->scrollToItem( lstCoordinateSystems->currentItem(), QAbstractItemView::PositionAtCenter );
+  }
+  else // unselect the selected item to avoid confusing the user
+  {
+    lstCoordinateSystems->clearSelection();
+    teProjection->setText( "" );
   }
 }
 
+void QgsProjectionSelector::insertRecent( long theCrsId )
+{
+  if ( !mProjListDone || !mUserProjListDone )
+    return;
+
+  QList<QTreeWidgetItem*> nodes = lstCoordinateSystems->findItems( QString::number( theCrsId ), Qt::MatchExactly | Qt::MatchRecursive, QGIS_CRS_ID_COLUMN );
+
+  if ( nodes.count() == 0 )
+    return;
+
+  lstRecent->insertTopLevelItem( 0, new QTreeWidgetItem( lstRecent, QStringList()
+                                 << nodes.first()->text( NAME_COLUMN )
+                                 << nodes.first()->text( AUTHID_COLUMN )
+                                 << nodes.first()->text( QGIS_CRS_ID_COLUMN ) ) );
+}
 
 //note this line just returns the projection name!
 QString QgsProjectionSelector::selectedName()
@@ -501,8 +420,7 @@ QString QgsProjectionSelector::selectedProj4String()
       // prepare the sql statement
       const char *pzTail;
       sqlite3_stmt *ppStmt;
-      QString sql = "select parameters from tbl_srs where srs_id = ";
-      sql += mySrsId;
+      QString sql = QString( "select parameters from tbl_srs where srs_id = %1" ).arg( mySrsId );
 
       QgsDebugMsg( "Selection sql: " + sql );
 
@@ -537,7 +455,7 @@ QString QgsProjectionSelector::selectedProj4String()
 
 }
 
-long QgsProjectionSelector::getSelectedLongAttribute( QString attributeName )
+QString QgsProjectionSelector::getSelectedExpression( QString expression )
 {
   // Only return the attribute if there is a node in the tree
   // selected that has an srs_id.  This prevents error if the user
@@ -587,10 +505,9 @@ long QgsProjectionSelector::getSelectedLongAttribute( QString attributeName )
       // prepare the sql statement
       const char *pzTail;
       sqlite3_stmt *ppStmt;
-      QString sql = "select ";
-      sql += attributeName;
-      sql += " from tbl_srs where srs_id = ";
-      sql += lvi->text( QGIS_CRS_ID_COLUMN );
+      QString sql = QString( "select %1 from tbl_srs where srs_id=%2" )
+                    .arg( expression )
+                    .arg( lvi->text( QGIS_CRS_ID_COLUMN ) );
 
       QgsDebugMsg( QString( "Finding selected attribute using : %1" ).arg( sql ) );
       rc = sqlite3_prepare( db, sql.toUtf8(), sql.toUtf8().length(), &ppStmt, &pzTail );
@@ -609,8 +526,8 @@ long QgsProjectionSelector::getSelectedLongAttribute( QString attributeName )
       sqlite3_finalize( ppStmt );
       // close the database
       sqlite3_close( db );
-      // return the srs wkt
-      return myAttributeValue.toLong();
+      // return the srs
+      return myAttributeValue;
     }
   }
 
@@ -618,16 +535,32 @@ long QgsProjectionSelector::getSelectedLongAttribute( QString attributeName )
   return 0;
 }
 
+long QgsProjectionSelector::selectedEpsg()
+{
+  if ( getSelectedExpression( "auth_name" ).compare( "EPSG", Qt::CaseInsensitive ) == 0 )
+  {
+    return getSelectedExpression( "auth_id" ).toLong();
+  }
+  else
+  {
+    QgsDebugMsg( "selected projection is NOT EPSG" );
+    return 0;
+  }
+}
 
 long QgsProjectionSelector::selectedPostgresSrId()
 {
-  return getSelectedLongAttribute( "srid" );
+  return getSelectedExpression( "srid" ).toLong();
 }
 
 
-long QgsProjectionSelector::selectedEpsg()
+QString QgsProjectionSelector::selectedAuthId()
 {
-  return getSelectedLongAttribute( "epsg" );
+  int srid = getSelectedExpression( "srs_id" ).toLong();
+  if ( srid >= USER_CRS_START_ID )
+    return QString( "USER:%1" ).arg( srid );
+  else
+    return getSelectedExpression( "upper(auth_name||':'||auth_id)" );
 }
 
 
@@ -667,8 +600,8 @@ void QgsProjectionSelector::loadUserCrsList( QSet<QString> * crsFilter )
   mUserProjList = new QTreeWidgetItem( lstCoordinateSystems, QStringList( tr( "User Defined Coordinate Systems" ) ) );
 
   QFont fontTemp = mUserProjList->font( 0 );
-  fontTemp.setItalic( TRUE );
-  fontTemp.setBold( TRUE );
+  fontTemp.setItalic( true );
+  fontTemp.setBold( true );
   mUserProjList->setFont( 0, fontTemp );
   mUserProjList->setIcon( 0, QIcon( QgsApplication::activeThemePath() + "user.png" ) );
 
@@ -684,7 +617,7 @@ void QgsProjectionSelector::loadUserCrsList( QSet<QString> * crsFilter )
   {
     QgsDebugMsg( "Users qgis.db not found...skipping" );
 
-    mUserProjListDone = TRUE;
+    mUserProjListDone = true;
     return;
   }
 
@@ -705,9 +638,7 @@ void QgsProjectionSelector::loadUserCrsList( QSet<QString> * crsFilter )
   }
 
   // Set up the query to retrieve the projection information needed to populate the list
-  QString mySql = "select description, srs_id from vw_srs ";
-  mySql += "where ";
-  mySql += sqlFilter;
+  QString mySql = QString( "select description, srs_id from vw_srs where %1" ).arg( sqlFilter );
 
   myResult = sqlite3_prepare( myDatabase, mySql.toUtf8(), mySql.toUtf8().length(), &myPreparedStatement, &myTail );
   // XXX Need to free memory from the error msg if one is set
@@ -729,10 +660,10 @@ void QgsProjectionSelector::loadUserCrsList( QSet<QString> * crsFilter )
   sqlite3_finalize( myPreparedStatement );
   sqlite3_close( myDatabase );
 
-  mUserProjListDone = TRUE;
+  mUserProjListDone = true;
 }
 
-void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
+void QgsProjectionSelector::loadCrsList( QSet<QString> *crsFilter )
 {
   // convert our Coordinate Reference System filter into the SQL expression
   QString sqlFilter = ogcWmsCrsFilterAsSqlExpression( crsFilter );
@@ -744,8 +675,8 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
   mGeoList = new QTreeWidgetItem( lstCoordinateSystems, QStringList( tr( "Geographic Coordinate Systems" ) ) );
 
   QFont fontTemp = mGeoList->font( 0 );
-  fontTemp.setItalic( TRUE );
-  fontTemp.setBold( TRUE );
+  fontTemp.setItalic( true );
+  fontTemp.setBold( true );
   mGeoList->setFont( 0, fontTemp );
   mGeoList->setIcon( 0, QIcon( QgsApplication::activeThemePath() + "geographic.png" ) );
 
@@ -753,8 +684,8 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
   mProjList = new QTreeWidgetItem( lstCoordinateSystems, QStringList( tr( "Projected Coordinate Systems" ) ) );
 
   fontTemp = mProjList->font( 0 );
-  fontTemp.setItalic( TRUE );
-  fontTemp.setBold( TRUE );
+  fontTemp.setItalic( true );
+  fontTemp.setBold( true );
   mProjList->setFont( 0, fontTemp );
   mProjList->setIcon( 0, QIcon( QgsApplication::activeThemePath() + "transformed.png" ) );
 
@@ -767,7 +698,7 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
   myFileInfo.setFile( mSrsDatabaseFileName );
   if ( !myFileInfo.exists( ) )
   {
-    mProjListDone = TRUE;
+    mProjListDone = true;
     return;
   }
 
@@ -795,12 +726,10 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
   sqlite3_finalize( ppStmt );
 
   // Set up the query to retrieve the projection information needed to populate the list
-  //note I am giving the full field names for clarity here and in case someown
+  //note I am giving the full field names for clarity here and in case someone
   //changes the underlying view TS
-  sql = "select description, srs_id, epsg, is_geo, name, parameters from vw_srs ";
-  sql += "where ";
-  sql += sqlFilter;
-  sql += " order by name, description";
+  sql = QString( "select description, srs_id, upper(auth_name||':'||auth_id), is_geo, name, parameters, deprecated from vw_srs where %1 order by name,description" )
+        .arg( sqlFilter );
 
   rc = sqlite3_prepare( db, sql.toUtf8(), sql.toUtf8().length(), &ppStmt, &pzTail );
   // XXX Need to free memory from the error msg if one is set
@@ -822,8 +751,8 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
         // Add it to the tree (field 0)
         newItem = new QTreeWidgetItem( mGeoList, QStringList( QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 0 ) ) ) );
 
-        // display the epsg (field 2) in the second column of the list view
-        newItem->setText( EPSG_COLUMN, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 2 ) ) );
+        // display the authority name (field 2) in the second column of the list view
+        newItem->setText( AUTHID_COLUMN, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 2 ) ) );
 
         // display the qgis srs_id (field 1) in the third column of the list view
         newItem->setText( QGIS_CRS_ID_COLUMN, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 1 ) ) );
@@ -849,7 +778,7 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
             node = new QTreeWidgetItem( mProjList, QStringList( srsType ) );
 
             QFont fontTemp = node->font( 0 );
-            fontTemp.setItalic( TRUE );
+            fontTemp.setItalic( true );
             node->setFont( 0, fontTemp );
           }
           else
@@ -862,11 +791,16 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
         }
         // add the item, setting the projection name in the first column of the list view
         newItem = new QTreeWidgetItem( node, QStringList( QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 0 ) ) ) );
-        // display the epsg (field 2) in the second column of the list view
-        newItem->setText( EPSG_COLUMN, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 2 ) ) );
+        // display the authority id (field 2) in the second column of the list view
+        newItem->setText( AUTHID_COLUMN, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 2 ) ) );
         // display the qgis srs_id (field 1) in the third column of the list view
         newItem->setText( QGIS_CRS_ID_COLUMN, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 1 ) ) );
+
       }
+
+      // display the qgis deprecated in the user data of the item
+      newItem->setData( 0, Qt::UserRole, QString::fromUtf8(( char * )sqlite3_column_text( ppStmt, 6 ) ) );
+      newItem->setHidden( cbxHideDeprecated->isChecked() );
     }
     mProjList->setExpanded( true );
   }
@@ -875,7 +809,7 @@ void QgsProjectionSelector::loadCrsList( QSet<QString> * crsFilter )
   // close the database
   sqlite3_close( db );
 
-  mProjListDone = TRUE;
+  mProjListDone = true;
 }
 
 
@@ -884,7 +818,7 @@ void QgsProjectionSelector::coordinateSystemSelected( QTreeWidgetItem * theItem
 {
   // If the item has children, it's not an end node in the tree, and
   // hence is just a grouping thingy, not an actual CRS.
-  if ( theItem != NULL && theItem->childCount() == 0 )
+  if ( theItem && theItem->childCount() == 0 )
   {
     // Found a real CRS
     QString myDescription;
@@ -892,66 +826,86 @@ void QgsProjectionSelector::coordinateSystemSelected( QTreeWidgetItem * theItem
     QString myProjString = selectedProj4String();
     lstCoordinateSystems->scrollToItem( theItem );
     teProjection->setText( myProjString );
+
+    lstRecent->clearSelection();
   }
   else
   {
     // Not an CRS - remove the highlight so the user doesn't get too confused
-    lstCoordinateSystems->setItemSelected( theItem, FALSE );  // TODO - make this work.
+    if ( theItem )
+      theItem->setSelected( false );
     teProjection->setText( "" );
   }
 }
 
-void QgsProjectionSelector::on_pbnPopular1_clicked()
+void QgsProjectionSelector::hideDeprecated( QTreeWidgetItem *item )
 {
-  setSelectedCrsId( mRecentProjections.at( 0 ).toLong() );
+  if ( item->data( 0, Qt::UserRole ).toBool() )
+  {
+    item->setHidden( cbxHideDeprecated->isChecked() );
+    if ( item->isSelected() && item->isHidden() )
+    {
+      item->setSelected( false );
+      teProjection->setText( "" );
+    }
+  }
+
+  for ( int i = 0; i < item->childCount(); i++ )
+    hideDeprecated( item->child( i ) );
 }
 
-void QgsProjectionSelector::on_pbnPopular2_clicked()
+void QgsProjectionSelector::on_cbxHideDeprecated_stateChanged()
 {
-  setSelectedCrsId( mRecentProjections.at( 1 ).toLong() );
+  for ( int i = 0; i < lstCoordinateSystems->topLevelItemCount(); i++ )
+    hideDeprecated( lstCoordinateSystems->topLevelItem( i ) );
 }
-void QgsProjectionSelector::on_pbnPopular3_clicked()
+
+void QgsProjectionSelector::on_lstRecent_currentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous )
 {
-  setSelectedCrsId( mRecentProjections.at( 2 ).toLong() );
-}
-void QgsProjectionSelector::on_pbnPopular4_clicked()
-{
-  setSelectedCrsId( mRecentProjections.at( 3 ).toLong() );
+  Q_UNUSED( previous );
+  if ( current )
+    setSelectedCrsId( current->text( QGIS_CRS_ID_COLUMN ).toLong() );
 }
 
 void QgsProjectionSelector::on_pbnFind_clicked()
 {
-
   QgsDebugMsg( "pbnFind..." );
 
   QString mySearchString( sqlSafeString( leSearch->text() ) );
+
   // Set up the query to retrieve the projection information needed to populate the list
-  QString mySql;
-  if ( radEpsgCrsId->isChecked() )
+  QString mySql = "select srs_id from tbl_srs where ";
+  if ( cbxAuthority->currentIndex() > 0 )
   {
-    mySql = "select srs_id from tbl_srs where epsg=" + mySearchString;
+    mySql += QString( "auth_name='%1' AND " ).arg( cbxAuthority->currentText() );
   }
-  else if ( radName->isChecked() ) //name search
+
+  if ( cbxHideDeprecated->isChecked() )
   {
-    //we need to find what the largest srsid matching our query so we know whether to
-    //loop backto the beginning
-    mySql = "select srs_id from tbl_srs where description like '%" + mySearchString + "%'" +
-            " order by srs_id desc limit 1";
-    long myLargestSrsId = getLargestCRSIDMatch( mySql );
+    mySql += "not deprecated AND ";
+  }
+
+  if ( cbxMode->currentIndex() == 0 )
+  {
+    mySql += QString( "auth_id='%1'" ).arg( mySearchString );
+  }
+  else
+  {
+    mySql += "upper(description) like '%" + mySearchString.toUpper() + "%' ";
+
+    long myLargestSrsId = getLargestCRSIDMatch( QString( "%1 order by srs_id desc limit 1" ).arg( mySql ) );
     QgsDebugMsg( QString( "Largest CRSID%1" ).arg( myLargestSrsId ) );
-    //a name search is ambiguous, so we find the first srsid after the current seelcted srsid
+
+    //a name search is ambiguous, so we find the first srsid after the current selected srsid
     // each time the find button is pressed. This means we can loop through all matches.
     if ( myLargestSrsId <= selectedCrsId() )
     {
-      //roll search around to the beginning
-      mySql = "select srs_id from tbl_srs where description like '%" + mySearchString + "%'" +
-              " order by srs_id limit 1";
+      mySql = QString( "%1 order by srs_id limit 1" ).arg( mySql );
     }
     else
     {
       // search ahead of the current position
-      mySql = "select srs_id from tbl_srs where description like '%" + mySearchString + "%'" +
-              " and srs_id > " + QString::number( selectedCrsId() ) + " order by srs_id limit 1";
+      mySql = QString( "%1 and srs_id > %2 order by srs_id limit 1" ).arg( mySql ).arg( selectedCrsId() );
     }
   }
   QgsDebugMsg( QString( " Search sql: %1" ).arg( mySql ) );
@@ -1020,8 +974,13 @@ void QgsProjectionSelector::on_pbnFind_clicked()
       // close the sqlite3 statement
       sqlite3_finalize( myPreparedStatement );
       sqlite3_close( myDatabase );
+      return;
     }
   }
+
+  QMessageBox::information( this, tr( "Find projection" ), tr( "No matching projection found." ) );
+  lstCoordinateSystems->clearSelection();
+  teProjection->setText( "" );
 }
 
 long QgsProjectionSelector::getLargestCRSIDMatch( QString theSql )
@@ -1100,6 +1059,42 @@ long QgsProjectionSelector::getLargestCRSIDMatch( QString theSql )
   }
   return mySrsId;
 }
+
+QStringList QgsProjectionSelector::authorities()
+{
+  sqlite3      *myDatabase;
+  const char   *myTail;
+  sqlite3_stmt *myPreparedStatement;
+  int           myResult;
+
+  myResult = sqlite3_open( mSrsDatabaseFileName.toUtf8().data(), &myDatabase );
+  if ( myResult )
+  {
+    QgsDebugMsg( QString( "Can't open * user * database: %1" ).arg( sqlite3_errmsg( myDatabase ) ) );
+    //no need for assert because user db may not have been created yet
+    return QStringList();
+  }
+
+  QString theSql = "select distinct auth_name from tbl_srs";
+  myResult = sqlite3_prepare( myDatabase, theSql.toUtf8(), theSql.toUtf8().length(), &myPreparedStatement, &myTail );
+
+  QStringList authorities;
+
+  if ( myResult == SQLITE_OK )
+  {
+    while ( sqlite3_step( myPreparedStatement ) == SQLITE_ROW )
+    {
+      authorities << QString::fromUtf8(( char * )sqlite3_column_text( myPreparedStatement, 0 ) );
+    }
+
+    // close the sqlite3 statement
+    sqlite3_finalize( myPreparedStatement );
+    sqlite3_close( myDatabase );
+  }
+
+  return authorities;
+}
+
 /*!
 * \brief Make the string safe for use in SQL statements.
 *  This involves escaping single quotes, double quotes, backslashes,
