@@ -247,11 +247,6 @@ void QgsQuickPrint::printMap()
 #endif
   QString myFontFamily = "Arial";
 
-  // Background color for pixmaps
-  QColor myLegendBackgroundColor = Qt::white;
-  //QColor myMapBackgroundColor = "#98dbf9"; // nice blue color
-
-
   //
   // Draw the PageBorder
   //
@@ -798,7 +793,7 @@ void QgsQuickPrint::renderPrintScaleBar( QPainter * thepPainter,
   //Get map units per pixel. This can be negative at times (to do with
   //projections) and that just confuses the rest of the code in this
   //function, so force to a positive number.
-  double myMapUnitsPerPixelDouble = std::abs( thepMapRenderer->mapUnitsPerPixel() );
+  double myMapUnitsPerPixelDouble = qAbs( thepMapRenderer->mapUnitsPerPixel() );
   //
   // Exit if the canvas width is 0 or layercount is 0 or QGIS will freeze
   int myLayerCount = thepMapRenderer->layerSet().count();
