@@ -6,6 +6,8 @@
 class QgsSingleSymbolRendererV2;
 class QgsSymbolV2SelectorDialog;
 
+class QMenu;
+
 class GUI_EXPORT QgsSingleSymbolRendererV2Widget : public QgsRendererV2Widget
 {
     Q_OBJECT
@@ -22,10 +24,16 @@ class GUI_EXPORT QgsSingleSymbolRendererV2Widget : public QgsRendererV2Widget
   public slots:
     void changeSingleSymbol();
 
+    void rotationFieldChanged( QString fldName );
+    void sizeScaleFieldChanged( QString fldName );
+
   protected:
+
     QgsSingleSymbolRendererV2* mRenderer;
     QgsSymbolV2SelectorDialog* mSelector;
     QgsSymbolV2* mSingleSymbol;
+
+    QgsRendererV2DataDefinedMenus* mDataDefinedMenus;
 };
 
 
