@@ -53,6 +53,7 @@ static const char * const sIdent = "$Id: plugin.cpp 8053 2008-01-26 13:59:53Z ti
 static const QString sName = QObject::tr( "Coordinate Capture" );
 static const QString sDescription = QObject::tr( "Capture mouse coordinates in different CRS" );
 static const QString sPluginVersion = QObject::tr( "Version 0.1" );
+static const QString sPluginIcon = ":/coordinate_capture/coordinate_capture.png";
 static const QgisPlugin::PLUGINTYPE sPluginType = QgisPlugin::UI;
 
 //////////////////////////////////////////////////////////////////////
@@ -187,8 +188,8 @@ void CoordinateCapture::setCRS()
 
 void CoordinateCapture::setSourceCrs()
 {
-  mTransform.setSourceCrs( mQGisIface->mapCanvas()->mapRenderer()->destinationSrs() );
-  mCanvasDisplayPrecision = ( mQGisIface->mapCanvas()->mapRenderer()->destinationSrs().mapUnits() == QGis::Degrees ) ? 5 : 3; // for the map canvas coordinate display
+  mTransform.setSourceCrs( mQGisIface->mapCanvas()->mapRenderer()->destinationCrs() );
+  mCanvasDisplayPrecision = ( mQGisIface->mapCanvas()->mapRenderer()->destinationCrs().mapUnits() == QGis::Degrees ) ? 5 : 3; // for the map canvas coordinate display
 }
 
 void CoordinateCapture::mouseClicked( QgsPoint thePoint )
@@ -343,6 +344,11 @@ QGISEXTERN QString version()
   return sPluginVersion;
 }
 
+QGISEXTERN QString icon()
+{
+  return sPluginIcon;
+}
+
 // Delete ourself
 QGISEXTERN void unload( QgisPlugin * thePluginPointer )
 {
