@@ -24,7 +24,7 @@
 ** National Oceanic and Atmospheric Administration or the Department of Commerce.
 **
 **/
-/*  $Id: $ */
+/*  $Id$ */
 #include "evisgenericeventbrowsergui.h"
 
 #include "qgsapplication.h"
@@ -258,8 +258,7 @@ bool eVisGenericEventBrowserGui::initBrowser( )
    */
   if ( 0 == mVectorLayer->selectedFeatureCount( ) ) //if nothing is selected select everything
   {
-    QgsRectangle myRect = mDataProvider->extent( );
-    mVectorLayer->select( myRect, true );
+    mVectorLayer->invertSelection();
     mFeatureIds = mVectorLayer->selectedFeaturesIds( ).toList( );
   }
   else //use selected features
@@ -267,6 +266,8 @@ bool eVisGenericEventBrowserGui::initBrowser( )
     mFeatureIds = mVectorLayer->selectedFeaturesIds( ).toList( );
   }
 
+  if ( 0 == mFeatureIds.size( ) ) { return false; }
+
   //get the first feature in the list so we can set the field in the pulldown menues
   QgsFeature* myFeature = featureAtId( mFeatureIds.at( mCurrentFeatureIndex ) );
   if ( !myFeature )
