@@ -100,7 +100,7 @@ QgsGrassBrowser::QgsGrassBrowser( QgisInterface *iface,
   mTree->setContextMenuPolicy( Qt::CustomContextMenu );
 
   mTextBrowser = new QTextBrowser( 0 );
-  mTextBrowser->setReadOnly( TRUE );
+  mTextBrowser->setReadOnly( true );
 
   mSplitter = new QSplitter( 0 );
   mSplitter->addWidget( mTree );
@@ -137,7 +137,6 @@ void QgsGrassBrowser::addMap()
   QgsDebugMsg( "entered." );
 
   QModelIndexList indexes = mTree->selectionModel()->selectedIndexes();
-  bool mapSelected = false;
 
   QList<QModelIndex>::const_iterator it = indexes.begin();
   for ( ; it != indexes.end(); ++it )
@@ -149,8 +148,9 @@ void QgsGrassBrowser::addMap()
     if ( type == QgsGrassModel::Raster )
     {
       QgsDebugMsg( QString( "add raster: %1" ).arg( uri ) );
-      mIface->addRasterLayer( uri, map );
-      mapSelected = true;
+      //mIface->addRasterLayer( uri, map );
+      mIface->addRasterLayer( uri, map, "grassraster", QStringList(), QStringList(),
+                              QString(), QString() );
     }
     else if ( type == QgsGrassModel::Vector )
     {
@@ -175,7 +175,6 @@ void QgsGrassBrowser::addMap()
                        map, layer, list.size() );
 
       mIface->addVectorLayer( uri, name, "grass" );
-      mapSelected = true;
     }
     else if ( type == QgsGrassModel::Region )
     {
