@@ -43,6 +43,7 @@
 #include <QMessageBox>
 #include <QSettings>
 #include <QToolBar>
+#include <QDebug>
 
 extern "C"
 {
@@ -116,7 +117,6 @@ QWidget *QgsGrassEditAttributeTableItemDelegate::createEditor( QWidget *parent,
   return editor;
 }
 
-#include<QDebug>
 void QgsGrassEditAttributeTableItemDelegate::setEditorData( QWidget *editor,
     const QModelIndex &index ) const
 {
@@ -854,7 +854,7 @@ int QgsGrassEdit::nodeSymbFromMap( int node )
 
   for ( int i = 0; i < nlines; i++ )
   {
-    int line = abs( mProvider->nodeLine( node, i ) );
+    int line = qAbs( mProvider->nodeLine( node, i ) );
     int type = mProvider->readLine( NULL, NULL, line );
 
     if ( type & GV_LINES )
@@ -892,10 +892,10 @@ int QgsGrassEdit::lineSymbFromMap( int line )
 
       if ( !( mProvider->lineAreas( line, &left, &right ) ) ) return 0;
 
-      /* Count areas/isles on both sides */
+      /* Count areas on both sides */
       nareas = 0;
-      if ( left != 0 ) nareas++;
-      if ( right != 0 ) nareas++;
+      if ( left > 0 || ( left < 0 && mProvider->isleArea( -left ) > 0 ) ) nareas++;
+      if ( right > 0 || ( right < 0 && mProvider->isleArea( -right ) > 0 ) ) nareas++;
       if ( nareas == 0 ) return SYMB_BOUNDARY_0;
       else if ( nareas == 1 ) return SYMB_BOUNDARY_1;
       else return SYMB_BOUNDARY_2;
@@ -1463,12 +1463,6 @@ void QgsGrassEdit::addAttributes( int field, int cat )
       }
       else
       {
-        int size;
-        if ( atts->size() < ( int ) cols->size() )
-          size = atts->size();
-        else
-          size = cols->size();
-
         for ( unsigned int j = 0; j < cols->size(); j++ )
         {
           QgsField col = ( *cols )[j];
@@ -1786,9 +1780,8 @@ void QgsGrassEdit::displayDynamic( struct line_pnts *Points, double x, double y,
 
 void QgsGrassEdit::displayNode( int node, const QPen & pen, int size, QPainter *painter )
 {
-  QgsDebugMsg( QString( "node = %1" ).arg( node ) );
-
-  if ( !mSymbDisplay[mNodeSymb[node]] ) return;
+  if ( !mSymbDisplay[mNodeSymb[node]] )
+    return;
 
   double x, y;
 
@@ -1811,8 +1804,6 @@ QgsPoint QgsGrassEdit::transformLayerToMap( QgsPoint point )
 void QgsGrassEdit::displayIcon( double x, double y, const QPen & pen,
                                 int type, int size, QPainter *painter )
 {
-  QgsDebugMsg( "entered." );
-
   QgsPoint point;
   QPolygon pointArray( 2 );
 
