@@ -434,15 +434,13 @@ void QgsGrassMapcalc::mouseReleaseEvent( QMouseEvent* e )
 QStringList QgsGrassMapcalc::arguments()
 {
   QString cmd = "";
-  //cmd.append("'");
+  // Attention with quotes and spaces!
+  //cmd.append("\"");
 
   cmd.append( mOutputLineEdit->text() );
-  cmd.append( "=" );
+  cmd.append( " = " );
   cmd.append( mOutput->expression() );
-  //cmd.append("'");
-
-  //cmd = "\"pok=1\"";
-  //cmd = mOutputLineEdit->text();
+  //cmd.append("\"");
 
   return QStringList( cmd );
 }
@@ -1314,7 +1312,7 @@ void QgsGrassMapcalc::load()
     {
       case QgsGrassMapcalcObject::Map:
       {
-        QString label = e.attribute( "label", "???" );
+        QString label = QApplication::translate( "grasslabel", e.attribute( "label", "???" ).toUtf8() );
         obj->setValue( value, label );
         break;
       }
@@ -1989,12 +1987,8 @@ bool QgsGrassMapcalcConnector::tryConnectEnd( int end )
     }
   }
 
-  if ( !object ) return false;
-
   // try to connect
-  if ( !object->tryConnect( this, end ) ) return false;
-
-  return true;
+  return object && object->tryConnect( this, end );
 }
 
 void QgsGrassMapcalcConnector::setSocket( int end,
