@@ -21,6 +21,7 @@
 #include "qgsgrassprovider.h"
 #include "qgsgrassutils.h"
 #include "qgsgrass.h"
+#include "qgsconfig.h"
 
 #include "qgisinterface.h"
 #include "qgsapplication.h"
@@ -43,6 +44,7 @@
 extern "C"
 {
 #include <grass/Vect.h>
+#include <grass/glocale.h>
 }
 
 #include <gdal.h>         // to collect version information
@@ -115,8 +117,7 @@ QString QgsGrassModule::findExec( QString file )
 
 bool QgsGrassModule::inExecPath( QString file )
 {
-  if ( findExec( file ).isNull() ) return false;
-  return true;
+  return !findExec( file ).isNull();
 }
 
 QStringList QgsGrassModule::execArguments( QString module )
@@ -130,6 +131,13 @@ QStringList QgsGrassModule::execArguments( QString module )
     return arguments;
   }
 
+#if defined(WIN32)
+  if ( exe.endsWith( ".py" ) )
+  {
+    arguments.append( "python" );
+  }
+#endif
+
   arguments.append( exe );
 
   return arguments;
@@ -228,8 +236,8 @@ QgsGrassModule::QgsGrassModule( QgsGrassTools *tools, QString moduleName, QgisIn
   }
 
   // Hide display if there is no output
-  if ( mOptions->output( QgsGrassModuleOption::Vector ).size() == 0
-       && mOptions->output( QgsGrassModuleOption::Raster ).size() == 0 )
+  if ( !mOptions->hasOutput( QgsGrassModuleOption::Vector )
+       && !mOptions->hasOutput( QgsGrassModuleOption::Raster ) )
   {
     mViewButton->hide();
   }
@@ -241,6 +249,7 @@ QgsGrassModule::QgsGrassModule( QgsGrassTools *tools, QString moduleName, QgisIn
   QFile manFile( manPath );
   if ( manFile.exists() )
   {
+    mManualTextBrowser->setOpenExternalLinks( true );
     mManualTextBrowser->setSource( QUrl::fromLocalFile( manPath ) );
   }
   else
@@ -260,7 +269,7 @@ QgsGrassModule::QgsGrassModule( QgsGrassTools *tools, QString moduleName, QgisIn
   strcpy( envstr, env );
   putenv( envstr );
 
-  mOutputTextBrowser->setReadOnly( TRUE );
+  mOutputTextBrowser->setReadOnly( true );
 }
 
 /******************* QgsGrassModuleOptions *******************/
@@ -293,10 +302,10 @@ QgsGrassModuleStandardOptions::QgsGrassModuleStandardOptions(
   QgisInterface *iface,
   QString xname, QDomElement qDocElem,
   QWidget * parent, Qt::WFlags f )
-    : QgsGrassModuleOptions( tools, module, iface ),
-    QWidget( parent, f )
+    : QWidget( parent, f ),
+    QgsGrassModuleOptions( tools, module, iface )
 {
-  QgsDebugMsg( "called." );
+  //QgsDebugMsg( "called." );
   QgsDebugMsg( QString( "PATH = %1" ).arg( getenv( "PATH" ) ) );
 
   // Attention!: sh.exe (MSYS) sets $0 in scripts to file name
@@ -325,7 +334,9 @@ QgsGrassModuleStandardOptions::QgsGrassModuleStandardOptions(
 
   // ? Does binary on Win need .exe extension ?
   // Return code 255 (-1) was correct in GRASS < 6.1.0
-  if ( !process.waitForFinished()
+  if ( !process.waitForStarted()
+       || !process.waitForReadyRead()
+       || !process.waitForFinished()
        || ( process.exitCode() != 0 && process.exitCode() != 255 ) )
   {
     QgsDebugMsg( "process.exitCode() = " + QString::number( process.exitCode() ) );
@@ -337,19 +348,15 @@ QgsGrassModuleStandardOptions::QgsGrassModuleStandardOptions(
                           .arg( process.readAllStandardError().constData() ) );
     return;
   }
-  QByteArray gDescArray = process.readAllStandardOutput();
-  QByteArray errArray = process.readAllStandardError();
 
   QDomDocument gDoc( "task" );
   QString err;
   int line, column;
-  if ( !gDoc.setContent(( QByteArray )gDescArray, &err, &line, &column ) )
+  if ( !gDoc.setContent( &process, false, &err, &line, &column ) )
   {
     QString errmsg = tr( "Cannot read module description (%1):" ).arg( mXName )
                      + tr( "\n%1\nat line %2 column %3" ).arg( err ).arg( line ).arg( column );
     QgsDebugMsg( errmsg );
-    QgsDebugMsg( QString( gDescArray ) );
-    QgsDebugMsg( QString( errArray ) );
     QMessageBox::warning( 0, tr( "Warning" ), errmsg );
     return;
   }
@@ -371,15 +378,58 @@ QgsGrassModuleStandardOptions::QgsGrassModuleStandardOptions(
   //transfers frame ownership so no need to call delete
   mypScrollArea->setWidget( mypInnerFrame );
   mypScrollArea->setWidgetResizable( true );
-  QVBoxLayout *layout = new QVBoxLayout( mypInnerFrame );
+  QVBoxLayout *mypInnerFrameLayout = new QVBoxLayout( mypInnerFrame );
+  // Add frames for simple/advanced options
+  QFrame * mypSimpleFrame = new QFrame();
+  mypSimpleFrame->setFrameShape( QFrame::NoFrame );
+  mypSimpleFrame->setFrameShadow( QFrame::Plain );
+  mAdvancedFrame.setFrameShape( QFrame::NoFrame );
+  mAdvancedFrame.setFrameShadow( QFrame::Plain );
+
+  QFrame * mypAdvancedPushButtonFrame = new QFrame();
+  QHBoxLayout *mypAdvancedPushButtonFrameLayout = new QHBoxLayout( mypAdvancedPushButtonFrame );
+  connect( &mAdvancedPushButton, SIGNAL( clicked() ), this, SLOT( switchAdvanced() ) );
+  mypAdvancedPushButtonFrameLayout->addWidget( &mAdvancedPushButton );
+  mypAdvancedPushButtonFrameLayout->addStretch( 1 );
+
+  mypInnerFrameLayout->addWidget( mypSimpleFrame );
+  mypInnerFrameLayout->addWidget( mypAdvancedPushButtonFrame );
+  mypInnerFrameLayout->addWidget( &mAdvancedFrame );
+  mypInnerFrameLayout->addStretch( 1 );
+
+  // Hide advanced and set butto next
+  switchAdvanced();
+
+  QVBoxLayout *mypSimpleLayout = new QVBoxLayout( mypSimpleFrame );
+  QVBoxLayout *mypAdvancedLayout = new QVBoxLayout( &mAdvancedFrame );
+  QVBoxLayout *layout = 0;
   while ( !n.isNull() )
   {
     QDomElement e = n.toElement();
     if ( !e.isNull() )
     {
+      // Check GRASS version
+      QString version_min = e.attribute( "version_min" );
+      QString version_max = e.attribute( "version_max" );
+
+      if ( !QgsGrassModuleOption::checkVersion( e.attribute( "version_min" ), e.attribute( "version_max" ) ) )
+      {
+        n = n.nextSibling();
+        continue;
+      }
+
       QString optionType = e.tagName();
       QgsDebugMsg( "optionType = " + optionType );
 
+      if ( e.attribute( "advanced", "no" ) == "yes" )
+      {
+        layout = mypAdvancedLayout;
+      }
+      else
+      {
+        layout = mypSimpleLayout;
+      }
+
       QString key = e.attribute( "key" );
       QgsDebugMsg( "key = " + key );
 
@@ -474,6 +524,11 @@ QgsGrassModuleStandardOptions::QgsGrassModuleStandardOptions(
     n = n.nextSibling();
   }
 
+  if ( mypAdvancedLayout->count() == 0 )
+  {
+    mypAdvancedPushButtonFrame->hide();
+  }
+
   // Create list of flags
   n = gDocElem.firstChild();
   while ( !n.isNull() )
@@ -494,7 +549,22 @@ QgsGrassModuleStandardOptions::QgsGrassModuleStandardOptions(
     n = n.nextSibling();
   }
 
-  layout->addStretch();
+  if ( layout )
+    layout->addStretch();
+}
+
+void QgsGrassModuleStandardOptions::switchAdvanced()
+{
+  if ( mAdvancedFrame.isHidden() )
+  {
+    mAdvancedFrame.show();
+    mAdvancedPushButton.setText( tr( "<< Hide advanced options" ) );
+  }
+  else
+  {
+    mAdvancedFrame.hide();
+    mAdvancedPushButton.setText( tr( "Show advanced options >>" ) );
+  }
 }
 
 QStringList QgsGrassModuleStandardOptions::arguments()
@@ -514,6 +584,23 @@ QStringList QgsGrassModuleStandardOptions::arguments()
   return arg;
 }
 
+// id is not used in fact, was intended for field, but key is used instead
+QgsGrassModuleItem *QgsGrassModuleStandardOptions::itemByKey( QString key )
+{
+  QgsDebugMsg( "key = " + key );
+
+  for ( unsigned int i = 0; i < mItems.size(); i++ )
+  {
+    if ( mItems[i]->key() == key )
+    {
+      return mItems[i];
+    }
+  }
+
+  QMessageBox::warning( 0, tr( "Warning" ), tr( "Item with key %1 not found" ).arg( key ) );
+  return 0;
+}
+
 QgsGrassModuleItem *QgsGrassModuleStandardOptions::item( QString id )
 {
   QgsDebugMsg( "id = " + id );
@@ -694,7 +781,6 @@ void QgsGrassModuleStandardOptions::thawOutput()
 #endif
 }
 
-
 QStringList QgsGrassModuleStandardOptions::output( int type )
 {
   QgsDebugMsg( "called." );
@@ -713,7 +799,10 @@ QStringList QgsGrassModuleStandardOptions::output( int type )
       if ( opt->outputType() == type )
       {
         QString out = opt->value();
-        list.append( out );
+        if ( !out.isEmpty() )
+        {
+          list.append( out );
+        }
       }
     }
   }
@@ -721,6 +810,28 @@ QStringList QgsGrassModuleStandardOptions::output( int type )
   return list;
 }
 
+bool QgsGrassModuleStandardOptions::hasOutput( int type )
+{
+  QgsDebugMsg( "called." );
+  QStringList list;
+
+  for ( unsigned int i = 0; i < mItems.size(); i++ )
+  {
+    QgsGrassModuleOption *opt = dynamic_cast<QgsGrassModuleOption *>( mItems[i] );
+    if ( !opt )
+      continue;
+
+    QgsDebugMsg( "opt->key() = " + opt->key() );
+
+    if ( opt->isOutput() )
+    {
+      if ( opt->outputType() == type ) return true;
+    }
+  }
+
+  return false;
+}
+
 QStringList QgsGrassModuleStandardOptions::ready()
 {
   QgsDebugMsg( "entered." );
@@ -941,12 +1052,12 @@ QString QgsGrassModule::label( QString path )
   qFile.close();
   QDomElement qDocElem = qDoc.documentElement();
 
-  return ( qDocElem.attribute( "label" ) );
+  return QApplication::translate( "grasslabel", qDocElem.attribute( "label" ).trimmed().toUtf8() );
 }
 
 QPixmap QgsGrassModule::pixmap( QString path, int height )
 {
-  QgsDebugMsg( "called." );
+  //QgsDebugMsg( QString( "path = %1" ).arg( path ) );
 
   std::vector<QPixmap> pixmaps;
 
@@ -969,7 +1080,8 @@ QPixmap QgsGrassModule::pixmap( QString path, int height )
       int width = ( int )( scale * br.width() );
       if ( width <= 0 ) width = height; // should not happen
       QPixmap pixmap( width, height );
-      pixmap.fill( QColor( 255, 255, 255 ) );
+      pixmap.fill( Qt::transparent );
+      //pixmap.fill( QColor( 255, 255, 255 ) );
       QPainter painter( &pixmap );
       painter.setRenderHint( QPainter::Antialiasing );
 
@@ -1010,18 +1122,88 @@ QPixmap QgsGrassModule::pixmap( QString path, int height )
 
   if ( width <= 0 ) width = height; //should not happen
 
-  int plusWidth = 8; // +
-  int arrowWidth = 9; // ->
-  int buffer = 10; // buffer around a sign
+  QString iconsPath = QgsApplication::pkgDataPath() + "/grass/modules/";
+  QFileInfo iconsfi( iconsPath );
+
+  int plusWidth = 8;
+  int arrowWidth = 9;
+
+  QString arrowPath = iconsPath + "grass_arrow.png";
+  QPixmap arrowPixmap;
+  iconsfi.setFile( arrowPath );
+  if ( iconsfi.exists() && arrowPixmap.load( arrowPath, "PNG" ) )
+  {
+    double scale = 1. * height / arrowPixmap.height();
+    arrowWidth = ( int )( scale * arrowPixmap.width() );
+
+    QImage img = arrowPixmap.toImage();
+    img = img.scaled( arrowWidth, height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
+    arrowPixmap = QPixmap::fromImage( img );
+  }
+  /*if ( iconsfi.exists() )
+  {
+    QSvgRenderer pic;
+    if ( pic.load( arrowPath ) )
+    {
+      QRect br( QPoint( 0, 0 ), pic.defaultSize() );
+
+      double scale = 1. * height / br.height();
+
+      arrowWidth = ( int )( scale * br.width() );
+      if ( arrowWidth <= 0 ) arrowWidth = height; // should not happen
+      arrowPixmap = QPixmap( arrowWidth, height );
+      arrowPixmap.fill( Qt::transparent );
+      QPainter painter( &arrowPixmap );
+      painter.setRenderHint( QPainter::Antialiasing );
+
+      pic.render( &painter );
+      painter.end();
+    }
+  }*/
+
+  QString plusPath = iconsPath + "grass_plus.svg";
+  QPixmap plusPixmap;
+  iconsfi.setFile( plusPath );
+  /*if ( iconsfi.exists() && plusPixmap.load( plusPath, "PNG" ) )
+  {
+    double scale = 1. * height / plusPixmap.height();
+    plusWidth = ( int )( scale * plusPixmap.width() );
+
+    QImage img = plusPixmap.toImage();
+    img = img.scaled( plusWidth, height, Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
+    plusPixmap = QPixmap::fromImage( img );
+  }*/
+  if ( iconsfi.exists() )
+  {
+    QSvgRenderer pic;
+    if ( pic.load( plusPath ) )
+    {
+      QRect br( QPoint( 0, 0 ), pic.defaultSize() );
+
+      double scale = 1. * height / br.height();
+
+      plusWidth = ( int )( scale * br.width() );
+      if ( plusWidth <= 0 ) plusWidth = height; // should not happen
+      plusPixmap = QPixmap( plusWidth, height );
+      plusPixmap.fill( Qt::transparent );
+      QPainter painter( &plusPixmap );
+      painter.setRenderHint( QPainter::Antialiasing );
+
+      pic.render( &painter );
+      painter.end();
+    }
+  }
+  int buffer = height / 3; // buffer around a sign
   if ( pixmaps.size() > 1 ) width += arrowWidth + 2 * buffer; // ->
   if ( pixmaps.size() > 2 ) width += plusWidth + 2 * buffer; // +
 
   QPixmap pixmap( width, height );
-  pixmap.fill( QColor( 255, 255, 255 ) );
+  pixmap.fill( Qt::transparent );
+  //pixmap.fill( QColor( 255, 255, 255 ) );
   QPainter painter( &pixmap );
 
-  QColor color( 200, 200, 200 );
-  painter.setBrush( QBrush( color ) );
+  //QColor color( 255, 255, 255 );
+  //painter.setBrush( QBrush( color ) );
 
   painter.setRenderHint( QPainter::Antialiasing );
 
@@ -1031,25 +1213,13 @@ QPixmap QgsGrassModule::pixmap( QString path, int height )
     if ( i == 1 && pixmaps.size() == 3 )   // +
     {
       pos += buffer;
-
-      painter.setPen( QPen( color, 3 ) );
-      painter.drawLine( pos, height / 2, pos + plusWidth, height / 2 );
-      painter.drawLine( pos + plusWidth / 2, height / 2 - plusWidth / 2, pos + plusWidth / 2, height / 2 + plusWidth / 2 );
+      painter.drawPixmap( pos, 0, plusPixmap );
       pos += buffer + plusWidth;
     }
     if (( i == 1 && pixmaps.size() == 2 ) || ( i == 2 && pixmaps.size() == 3 ) ) // ->
     {
       pos += buffer;
-      painter.setPen( QPen( color, 3 ) );
-      painter.drawLine( pos, height / 2, pos + arrowWidth - arrowWidth / 2, height / 2 );
-
-      QPolygon pa( 3 );
-      pa.setPoint( 0, pos + arrowWidth / 2 + 1, height / 2 - arrowWidth / 2 );
-      pa.setPoint( 1, pos + arrowWidth, height / 2 );
-      pa.setPoint( 2, pos + arrowWidth / 2 + 1, height / 2 + arrowWidth / 2 );
-      painter.setPen( QPen( color, 1 ) );
-      painter.drawPolygon( pa );
-
+      painter.drawPixmap( pos, 0, arrowPixmap );
       pos += buffer + arrowWidth;
     }
     painter.drawPixmap( pos, 0, pixmaps[i] );
@@ -1201,13 +1371,14 @@ void QgsGrassModule::run()
     commandHtml.replace( ">", "&gt;" );
     mOutputTextBrowser->append( "<B>" +  commandHtml + "</B>" );
 
+    QStringList environment = QProcess::systemEnvironment();
+    environment.append( "GRASS_HTML_BROWSER=" + QgsApplication::prefixPath() + "/" QGIS_LIBEXEC_SUBDIR "/grass/bin/qgis.g.browser" );
+
     // Warning: it is not useful to write requested region to WIND file and
     //          reset then to original beacuse it is reset before
     //          the region is read by a module even if waitForStarted() is used
     //          -> necessary to pass region as environment variable
     //             but the feature is available in GRASS 6.1 only since 23.3.2006
-
-    QStringList environment = QProcess::systemEnvironment();
     if ( resetRegion )
     {
       QString reg = QgsGrass::regionString( &tempWindow );
@@ -1347,7 +1518,8 @@ void QgsGrassModule::readStdout()
   {
     //line = QString::fromLocal8Bit( mProcess.readLineStdout().ascii() );
     QByteArray ba = mProcess.readLine();
-    line = QString::fromUtf8( ba );
+    line = QString::fromUtf8( ba ).replace( '\n', "" );
+    //QgsDebugMsg(QString("line: '%1'").arg(line));
 
     // GRASS_INFO_PERCENT is catched here only because of bugs in GRASS,
     // normaly it should be printed to stderr
@@ -1358,7 +1530,7 @@ void QgsGrassModule::readStdout()
     }
     else
     {
-      mOutputTextBrowser->append( line );
+      mOutputTextBrowser->append( "<pre>" + line + "</pre>" );
     }
   }
 }
@@ -1381,7 +1553,7 @@ void QgsGrassModule::readStderr()
   {
     //line = QString::fromLocal8Bit( mProcess.readLineStderr().ascii() );
     QByteArray ba = mProcess.readLine();
-    line = QString::fromUtf8( ba );
+    line = QString::fromUtf8( ba ).replace( '\n', "" );
     //QgsDebugMsg(QString("line: '%1'").arg(line));
 
     if ( rxpercent.indexIn( line ) != -1 )
@@ -1391,7 +1563,7 @@ void QgsGrassModule::readStderr()
     }
     else if ( rxmessage.indexIn( line ) != -1 )
     {
-      mOutputTextBrowser->append( rxmessage.cap( 1 ) );
+      mOutputTextBrowser->append( "<pre>" + rxmessage.cap( 1 ) + "</pre>" );
     }
     else if ( rxwarning.indexIn( line ) != -1 )
     {
@@ -1411,7 +1583,7 @@ void QgsGrassModule::readStderr()
     }
     else
     {
-      mOutputTextBrowser->append( line + "\n" );
+      mOutputTextBrowser->append( "<pre>" + line + "</pre>" );
     }
   }
 }
@@ -1476,7 +1648,9 @@ void QgsGrassModule::viewOutput()
                   + QgsGrass::getDefaultMapset()
                   + "/cellhd/" + map;
 
-    mIface->addRasterLayer( uri, map );
+    //mIface->addRasterLayer( uri, map );
+    mIface->addRasterLayer( uri, map, "grassraster", QStringList(), QStringList(),
+                            QString(), QString() );
   }
 }
 
@@ -1491,6 +1665,11 @@ QgsGrassModule::~QgsGrassModule()
   }
 }
 
+QString QgsGrassModule::translate( QString msg )
+{
+  return QString::fromUtf8( G_gettext( "grassmods", msg.trimmed().toUtf8() ) );
+}
+
 QDomNode QgsGrassModule::nodeByKey( QDomElement elem, QString key )
 {
   QgsDebugMsg( "called with key=" + key );
@@ -1867,6 +2046,51 @@ QString QgsGrassModuleOption::value()
   return value;
 }
 
+bool QgsGrassModuleOption::checkVersion( QString version_min, QString version_max )
+{
+  QgsDebugMsg( "version_min = " + version_min );
+  QgsDebugMsg( "version_max = " + version_max );
+
+  QRegExp rxVersion( "(\\d+)\\.(\\d+)" );
+  if ( !version_min.isEmpty() )
+  {
+    if ( !rxVersion.exactMatch( version_min ) )
+    {
+      QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot parse version_min %1" ).arg( version_min ) );
+      return false;
+    }
+    else
+    {
+      int versionMajor = rxVersion.cap( 1 ).toInt();
+      int versionMinor = rxVersion.cap( 2 ).toInt();
+      if ( QgsGrass::versionMajor() < versionMajor || ( QgsGrass::versionMajor() == versionMajor && QgsGrass::versionMinor() < versionMinor ) )
+      {
+        return false;
+      }
+    }
+  }
+
+  if ( !version_max.isEmpty() )
+  {
+    if ( !rxVersion.exactMatch( version_max ) )
+    {
+      QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot parse version_max %1" ).arg( version_max ) );
+      return false;
+    }
+    else
+    {
+      int versionMajor = rxVersion.cap( 1 ).toInt();
+      int versionMinor = rxVersion.cap( 2 ).toInt();
+      if ( QgsGrass::versionMajor() > versionMajor || ( QgsGrass::versionMajor() == versionMajor && QgsGrass::versionMinor() > versionMinor ) )
+      {
+        return false;
+      }
+    }
+  }
+
+  return true;
+}
+
 QStringList QgsGrassModuleOption::options()
 {
   QStringList list;
@@ -1944,10 +2168,13 @@ QgsGrassModuleInput::QgsGrassModuleInput( QgsGrassModule *module,
     QgsGrassModuleStandardOptions *options, QString key,
     QDomElement &qdesc, QDomElement &gdesc, QDomNode &gnode,
     QWidget * parent )
-    : QgsGrassModuleGroupBoxItem( module, key, qdesc, gdesc, gnode, parent ),
-    mModuleStandardOptions( options ),
-    mGeometryTypeOption( 0 ), mVectorLayerOption( 0 ),
-    mRegionButton( 0 ), mUpdate( false ), mRequired( false )
+    : QgsGrassModuleGroupBoxItem( module, key, qdesc, gdesc, gnode, parent )
+    , mModuleStandardOptions( options )
+    , mGeometryTypeOption( "" )
+    , mVectorLayerOption( "" )
+    , mRegionButton( 0 )
+    , mUpdate( false )
+    , mRequired( false )
 {
   QgsDebugMsg( "called." );
   mGeometryTypeMask = GV_POINT | GV_LINE | GV_AREA;
@@ -2148,13 +2375,7 @@ bool QgsGrassModuleInput::useRegion()
 {
   QgsDebugMsg( "called." );
 
-  if ( mUsesRegion && mType == Raster && mRegionButton &&
-       mRegionButton->isChecked() )
-  {
-    return true;
-  }
-
-  return false;
+  return mUsesRegion && mType == Raster && mRegionButton && mRegionButton->isChecked();
 }
 
 void QgsGrassModuleInput::updateQgisLayers()
@@ -2163,11 +2384,11 @@ void QgsGrassModuleInput::updateQgisLayers()
 
   QString current = mLayerComboBox->currentText();
   mLayerComboBox->clear();
-  mMaps.resize( 0 );
-  mGeometryTypes.resize( 0 );
-  mVectorLayerNames.resize( 0 );
-  mMapLayers.resize( 0 );
-  mVectorFields.resize( 0 );
+  mMaps.clear();
+  mGeometryTypes.clear();
+  mVectorLayerNames.clear();
+  mMapLayers.clear();
+  mVectorFields.clear();
 
   // If not required, add an empty item to combobox and a padding item into
   // layer containers.
@@ -2352,31 +2573,29 @@ QStringList QgsGrassModuleInput::options()
   QStringList list;
   QString opt;
 
-  int c = mLayerComboBox->currentIndex();
-  if ( c < 0 ) // not found
+  int current = mLayerComboBox->currentIndex();
+  if ( current < 0 ) // not found
     return list;
 
-  unsigned current = c;
-
   // TODO: this is hack for network nodes, do it somehow better
   if ( mMapId.isEmpty() )
   {
-    opt = mKey + "=";
-
     if ( current <  mMaps.size() )
     {
-      opt.append( mMaps[current] );
+      if ( ! mMaps[current].isEmpty() )
+      {
+        list.push_back( mKey + "=" + mMaps[current] );
+      }
     }
-    list.push_back( opt );
   }
 
-  if ( !mGeometryTypeOption.isNull() && current < mGeometryTypes.size() )
+  if ( !mGeometryTypeOption.isEmpty() && current < mGeometryTypes.size() )
   {
     opt = mGeometryTypeOption + "=" + mGeometryTypes[current] ;
     list.push_back( opt );
   }
 
-  if ( !mVectorLayerOption.isNull() && current < mVectorLayerNames.size() )
+  if ( !mVectorLayerOption.isEmpty() && current < mVectorLayerNames.size() )
   {
     opt = mVectorLayerOption + "=" + mVectorLayerNames[current] ;
     list.push_back( opt );
@@ -2431,15 +2650,15 @@ QString QgsGrassModuleInput::currentMap()
 {
   QgsDebugMsg( "called." );
 
-  unsigned int limit = 0;
+  int limit = 0;
   if ( !mRequired )
     limit = 1;
 
-  unsigned int current = mLayerComboBox->currentIndex();
+  int current = mLayerComboBox->currentIndex();
   if ( current < limit )
     return QString();
 
-  if ( current >= limit && current <  mMaps.size() )
+  if ( current >= limit && current < mMaps.size() )
   {
     return mMaps[current];
   }
@@ -2503,7 +2722,7 @@ QgsGrassModuleItem::QgsGrassModuleItem( QgsGrassModule *module, QString key,
   QString label, description;
   if ( !qdesc.attribute( "label" ).isEmpty() )
   {
-    label = qdesc.attribute( "label" );
+    label = QApplication::translate( "grasslabel", qdesc.attribute( "label" ).trimmed().toUtf8() );
   }
   if ( label.isEmpty() )
   {
@@ -2511,16 +2730,14 @@ QgsGrassModuleItem::QgsGrassModuleItem( QgsGrassModule *module, QString key,
     if ( !n.isNull() )
     {
       QDomElement e = n.toElement();
-      label = e.text().trimmed();
-      label.replace( 0, 1, label.left( 1 ).toUpper() );
+      label = module->translate( e.text() );
     }
   }
   QDomNode n = gnode.namedItem( "description" );
   if ( !n.isNull() )
   {
     QDomElement e = n.toElement();
-    description = e.text().trimmed();
-    description.replace( 0, 1, description.left( 1 ).toUpper() );
+    description = module->translate( e.text() );
   }
 
   if ( !label.isEmpty() )
@@ -2580,12 +2797,14 @@ void QgsGrassModuleGroupBoxItem::adjustTitle()
 QgsGrassModuleGdalInput::QgsGrassModuleGdalInput(
   QgsGrassModule *module, int type, QString key, QDomElement &qdesc,
   QDomElement &gdesc, QDomNode &gnode, QWidget * parent )
-    : QgsGrassModuleGroupBoxItem( module, key, qdesc, gdesc, gnode, parent ),
-    mType( type ), mOgrLayerOption( 0 ), mOgrWhereOption( 0 )
+    : QgsGrassModuleGroupBoxItem( module, key, qdesc, gdesc, gnode, parent )
+    , mType( type )
+    , mOgrLayerOption( "" )
+    , mOgrWhereOption( "" )
 {
   if ( mTitle.isEmpty() )
   {
-    mTitle = "OGR/PostGIS/GDAL Input";
+    mTitle = tr( "OGR/PostGIS/GDAL Input" );
   }
   adjustTitle();
 
@@ -2656,8 +2875,8 @@ void QgsGrassModuleGdalInput::updateQgisLayers()
 
   QString current = mLayerComboBox->currentText();
   mLayerComboBox->clear();
-  mUri.resize( 0 );
-  mOgrLayers.resize( 0 );
+  mUri.clear();
+  mOgrLayers.clear();
 
   // If not required, add an empty item to combobox and a padding item into
   // layer containers.
@@ -2751,7 +2970,6 @@ void QgsGrassModuleGdalInput::updateQgisLayers()
       if ( layer->name() == current ) mLayerComboBox->setItemText( mLayerComboBox->currentIndex(), current );
 
       mUri.push_back( uri );
-
       mOgrLayers.push_back( ogrLayer );
       mOgrWheres.push_back( ogrWhere );
     }
@@ -2761,6 +2979,8 @@ void QgsGrassModuleGdalInput::updateQgisLayers()
       mLayerComboBox->addItem( layer->name() );
       if ( layer->name() == current ) mLayerComboBox->setItemText( mLayerComboBox->currentIndex(), current );
       mUri.push_back( uri );
+      mOgrLayers.push_back( "" );
+      mOgrWheres.push_back( "" );
     }
   }
 }
@@ -2769,21 +2989,19 @@ QStringList QgsGrassModuleGdalInput::options()
 {
   QStringList list;
 
-  int c = mLayerComboBox->currentIndex();
-  if ( c < 0 )
+  int current = mLayerComboBox->currentIndex();
+  if ( current < 0 )
     return list;
 
-  unsigned int current = c;
-
   QString opt( mKey + "=" );
 
-  if ( current >= 0 && current <  mUri.size() )
+  if ( current >= 0 && current < mUri.size() )
   {
     opt.append( mUri[current] );
   }
   list.push_back( opt );
 
-  if ( !mOgrLayerOption.isNull() && mOgrLayers[current].length() > 0 )
+  if ( !mOgrLayerOption.isEmpty() && mOgrLayers[current].size() > 0 )
   {
     opt = mOgrLayerOption + "=";
     // GDAL 1.4.0 supports schemas (r9998)
@@ -2812,7 +3030,7 @@ QStringList QgsGrassModuleGdalInput::options()
     list.push_back( opt );
   }
 
-  if ( !mOgrWhereOption.isNull() && mOgrWheres[current].length() > 0 )
+  if ( !mOgrWhereOption.isEmpty() && mOgrWheres[current].length() > 0 )
   {
     list.push_back( mOgrWhereOption + "=" + mOgrWheres[current] );
   }
@@ -2857,16 +3075,22 @@ QgsGrassModuleField::QgsGrassModuleField(
   QDomElement promptElem = promptNode.toElement();
   QString element = promptElem.attribute( "element" );
 
-  mLayerId = qdesc.attribute( "layerid" );
-
   mType = qdesc.attribute( "type" );
 
-  QgsGrassModuleItem *item = mModuleStandardOptions->item( mLayerId );
-  // TODO check type
-  if ( item )
+  mLayerKey = qdesc.attribute( "layer" );
+  if ( mLayerKey.isNull() || mLayerKey.length() == 0 )
   {
-    mLayerInput = dynamic_cast<QgsGrassModuleInput *>( item );
-    connect( mLayerInput, SIGNAL( valueChanged() ), this, SLOT( updateFields() ) );
+    QMessageBox::warning( 0, tr( "Warning" ), tr( "'layer' attribute in field tag with key= %1 is missing." ).arg( mKey ) );
+  }
+  else
+  {
+    QgsGrassModuleItem *item = mModuleStandardOptions->itemByKey( mLayerKey );
+    // TODO check type
+    if ( item )
+    {
+      mLayerInput = dynamic_cast<QgsGrassModuleInput *>( item );
+      connect( mLayerInput, SIGNAL( valueChanged() ), this, SLOT( updateFields() ) );
+    }
   }
 
   QHBoxLayout *l = new QHBoxLayout( this );
@@ -3108,46 +3332,39 @@ QStringList QgsGrassModuleFile::options()
 
 void QgsGrassModuleFile::browse()
 {
-  // TODO: unfortunately QFileDialog does not support 'new' directory
-  QFileDialog *fd = new QFileDialog( this, NULL, mLineEdit->text() );
+  static QString lastDir = QDir::currentPath();
 
-  static QDir currentDir = QDir::current();
-  fd->setDirectory( currentDir );
-
-  switch ( mType )
+  if ( mType == Multiple )
   {
-    case New:
-      fd->setFileMode( QFileDialog::AnyFile );
-      fd->setAcceptMode( QFileDialog::AcceptSave );
-      break;
-    case Multiple:
-      fd->setFileMode( QFileDialog::ExistingFiles );
-      fd->setAcceptMode( QFileDialog::AcceptOpen );
-      break;
-    case Directory:
-      fd->setFileMode( QFileDialog::Directory );
-      fd->setAcceptMode( QFileDialog::AcceptOpen );
-      break;
-    default:
-      fd->setFileMode( QFileDialog::ExistingFile );
-      fd->setAcceptMode( QFileDialog::AcceptOpen );
+    QString path = mLineEdit->text().split( "," ).first();
+    if ( path.isEmpty() )
+      path = lastDir;
+    else
+      path = QFileInfo( path ).absolutePath();
+
+    QStringList files = QFileDialog::getOpenFileNames( this, 0, path );
+    if ( files.isEmpty() )
+      return;
+
+    lastDir = QFileInfo( files[0] ).absolutePath();
+
+    mLineEdit->setText( files.join( "," ) );
   }
-
-  if ( mFilters.size() > 0 )
+  else
   {
-    fd->setFilters( mFilters );
-  }
-  fd->setDefaultSuffix( mSuffix );
+    QString selectedFile = mLineEdit->text();
+    if ( selectedFile.isEmpty() )
+      selectedFile = lastDir;
+
+    if ( mType == New )
+      selectedFile = QFileDialog::getSaveFileName( this, 0, selectedFile );
+    else if ( mType == Directory )
+      selectedFile = QFileDialog::getExistingDirectory( this, 0, selectedFile );
+    else
+      selectedFile = QFileDialog::getOpenFileName( this, 0, selectedFile );
+
+    lastDir = QFileInfo( selectedFile ).absolutePath();
 
-  if ( fd->exec() == QDialog::Accepted )
-  {
-    QString selectedFile = fd->selectedFiles().last();
-    QFileInfo fi = QFileInfo( selectedFile );
-    currentDir = fi.absoluteDir();
-    if ( mType == Multiple )
-    {
-      selectedFile = fd->selectedFiles().join( "," );
-    }
     mLineEdit->setText( selectedFile );
   }
 }
