@@ -57,6 +57,8 @@ class QgsGrassModule: public QDialog, private  Ui::QgsGrassModuleBase
     //! Destructor
     ~QgsGrassModule();
 
+    QString translate( QString string );
+
     //! Returns module label for module description path
     static QString label( QString path );
 
@@ -190,6 +192,9 @@ class QgsGrassModuleOptions
     //! Get list of current output maps
     virtual QStringList output( int type ) { return QStringList() ; }
 
+    //! Has any output
+    virtual bool hasOutput( int type ) { return true; }
+
     //! Has raster input or output
     virtual bool usesRegion() { return false; }
 
@@ -233,8 +238,10 @@ class QgsGrassModuleOptions
  *  \brief Widget with GRASS standard options.
  *
  */
-class QgsGrassModuleStandardOptions: public QgsGrassModuleOptions, QWidget
+class QgsGrassModuleStandardOptions: QWidget, public QgsGrassModuleOptions
 {
+    Q_OBJECT
+
   public:
     //! Constructor
     QgsGrassModuleStandardOptions(
@@ -252,18 +259,26 @@ class QgsGrassModuleStandardOptions: public QgsGrassModuleOptions, QWidget
     // ! Get item by ID
     QgsGrassModuleItem *item( QString id );
 
+    // ! Get item by key
+    QgsGrassModuleItem *itemByKey( QString key );
+
     // Reimplemented methods from QgsGrassModuleOptions
     QStringList checkOutput();
     void freezeOutput();
     void thawOutput();
     QStringList ready() ;
     QStringList output( int type );
+    bool hasOutput( int type );
     QStringList checkRegion();
     bool usesRegion();
     bool requestsRegion();
     bool inputRegion( struct Cell_head *window, bool all );
     QStringList flagNames() { return mFlagNames; }
 
+  public slots:
+    // ! Show/hide advanced options
+    void switchAdvanced();
+
   private:
     //! Name of module executable
     QString mXName;
@@ -279,6 +294,12 @@ class QgsGrassModuleStandardOptions: public QgsGrassModuleOptions, QWidget
 
     //! Use of region defined in qgm
     bool mUsesRegion;
+
+    // ! Advanced options switch button
+    QPushButton mAdvancedPushButton;
+
+    // ! Advanced options frame
+    QFrame mAdvancedFrame;
 };
 
 /****************** QgsGrassModuleGroupBoxItem ************************/
@@ -460,6 +481,9 @@ class QgsGrassModuleOption: public QgsGrassModuleGroupBoxItem
     //  Use of region can be forced by 'region' attribute in qgm
     bool usesRegion() { return mUsesRegion; }
 
+    //! Check min/max version
+    static bool checkVersion( QString version_min, QString version_max );
+
   public slots:
     // Add new line edit for multiple options
     void addLineEdit();
@@ -619,13 +643,13 @@ class QgsGrassModuleInput: public QgsGrassModuleGroupBoxItem
     QString mMapId;
 
     //! Vector of map@mapset in the combobox
-    std::vector<QString> mMaps;
+    QStringList mMaps;
 
     //! Type of vector in the combobox
-    std::vector<QString> mGeometryTypes;
+    QStringList mGeometryTypes;
 
     //! Layer names in the combobox
-    std::vector<QString> mVectorLayerNames;
+    QStringList mVectorLayerNames;
 
     //! Pointers to vector layers in combobox
     std::vector<QgsMapLayer*> mMapLayers;
@@ -688,13 +712,13 @@ class QgsGrassModuleGdalInput: public QgsGrassModuleGroupBoxItem
     QComboBox *mLayerComboBox;
 
     //! Vector of URI in the combobox
-    std::vector<QString> mUri;
+    QStringList mUri;
 
     //! Ogr layer options
-    std::vector<QString> mOgrLayers;
+    QStringList mOgrLayers;
 
     //! Ogr where clauses
-    std::vector<QString> mOgrWheres;
+    QStringList mOgrWheres;
 
     //! Required field
     bool mRequired;
@@ -735,7 +759,7 @@ class QgsGrassModuleField: public QgsGrassModuleGroupBoxItem
     QgsGrassModuleStandardOptions *mModuleStandardOptions;
 
     //! Layer key
-    QString mLayerId;
+    QString mLayerKey;
 
     //! Pointer to layer input
     QgsGrassModuleInput *mLayerInput;
