@@ -60,15 +60,6 @@ QgsGrassNewMapset::QgsGrassNewMapset( QgisInterface *iface,
   QgsDebugMsg( "QgsGrassNewMapset()" );
 
   setupUi( this );
-#if QT_VERSION < 0x040400
-  // this is not done by Qt4.3 uic
-  addPage( WizardPage1 );
-  addPage( WizardPage2 );
-  addPage( WizardPage3 );
-  addPage( WizardPage4 );
-  addPage( WizardPage5 );
-  addPage( WizardPage6 );
-#endif
 #ifdef Q_WS_MAC
   setWizardStyle( QWizard::ClassicStyle );
 #endif
@@ -164,14 +155,12 @@ QgsGrassNewMapset::~QgsGrassNewMapset()
 void QgsGrassNewMapset::browseDatabase()
 {
   // TODO: unfortunately QFileDialog does not support 'new' directory
-  QFileDialog *fd = new QFileDialog( this, NULL, mDatabaseLineEdit->text() );
-  fd->setFileMode( QFileDialog::DirectoryOnly );
+  QString selectedDir = QFileDialog::getExistingDirectory( this, NULL, mDatabaseLineEdit->text() );
+  if ( selectedDir.isEmpty() )
+    return;
 
-  if ( fd->exec() == QDialog::Accepted )
-  {
-    mDatabaseLineEdit->setText( fd->selectedFiles().first() );
-    databaseChanged();
-  }
+  mDatabaseLineEdit->setText( selectedDir );
+  databaseChanged();
 }
 
 void QgsGrassNewMapset::databaseChanged()
@@ -442,11 +431,11 @@ void QgsGrassNewMapset::setGrassProjection()
     }
     else
     {
-#ifdef QGISDEBUG
+      char *wkt = NULL;
+
       QgsDebugMsg( QString( "OSRIsGeographic = %1" ).arg( OSRIsGeographic( hCRS ) ) );
       QgsDebugMsg( QString( "OSRIsProjected = %1" ).arg( OSRIsProjected( hCRS ) ) );
 
-      char *wkt = NULL;
       if (( errcode = OSRExportToWkt( hCRS, &wkt ) ) != OGRERR_NONE )
       {
         QgsDebugMsg( QString( "OGR can't get Wkt-style parameter string\nOGR Error code was %1" ).arg( errcode ) );
@@ -455,27 +444,21 @@ void QgsGrassNewMapset::setGrassProjection()
       {
         QgsDebugMsg( QString( "wkt = %1" ).arg( wkt ) );
       }
-#endif
 
       int ret;
       // Note: GPJ_osr_to_grass() defaults in PROJECTION_XY if projection
       //       cannot be set
 
       // There was a bug in GRASS, it is present in 6.0.x line
-#if GRASS_VERSION_MAJOR == 6 && GRASS_VERSION_MINOR >= 1
-      ret = GPJ_osr_to_grass( &mCellHead, &mProjInfo,
-                              &mProjUnits, hCRS, 0 );
-#else
-      // Buggy version:
-      ret = GPJ_osr_to_grass( &mCellHead, &mProjInfo,
-                              &mProjUnits, ( void ** )hCRS, 0 );
-#endif
+      ret = GPJ_wkt_to_grass( &mCellHead, &mProjInfo, &mProjUnits, wkt, 0 );
 
       // Note: It seems that GPJ_osr_to_grass()returns always 1,
       //   -> test if mProjInfo was set
 
       QgsDebugMsg( QString( "ret = %1" ).arg( ret ) );
       QgsDebugMsg( QString( "mProjInfo = %1" ).arg( QString::number(( qulonglong )mProjInfo, 16 ).toLocal8Bit().constData() ) );
+
+      OGRFree( wkt );
     }
 
     if ( !mProjInfo || !mProjUnits )
@@ -1120,7 +1103,7 @@ void QgsGrassNewMapset::drawRegion()
       double x1 = points[i].x();
       double x2 = points[i+1].x();
 
-      if ( fabs( x2 - x1 ) > 150 )
+      if ( qAbs( x2 - x1 ) > 150 )
       {
         if ( x2 < x1 )
         {
@@ -1162,7 +1145,6 @@ void QgsGrassNewMapset::setMapsets()
   QDir d( locationPath );
 
   // Add all subdirs containing WIND
-  QTreeWidgetItem *lvi;
   for ( unsigned int i = 0; i < d.count(); i++ )
   {
     if ( d[i] == "." || d[i] == ".." ) continue;
@@ -1173,7 +1155,7 @@ void QgsGrassNewMapset::setMapsets()
 
     if ( QFile::exists( windPath ) )
     {
-      lvi = new QTreeWidgetItem( mMapsetsListView, QStringList() << d[i] << mapsetInfo.owner() );
+      new QTreeWidgetItem( mMapsetsListView, QStringList() << d[i] << mapsetInfo.owner() );
     }
   }
 }
