@@ -50,6 +50,7 @@ extern "C"
 }
 
 static const QString pluginVersion = QObject::tr( "Version 0.1" );
+static const QString pluginIcon = ":/images/themes/default/grass/grass_tools.png";
 
 /**
  * Constructor for the plugin. The plugin is passed a pointer to the main app
@@ -110,12 +111,12 @@ void QgsGrassPlugin::initGui()
   mRegion = 0;
 
   QSettings settings;
-
   QgsGrass::init();
-
   mCanvas = qGisInterface->mapCanvas();
   QWidget* qgis = qGisInterface->mainWindow();
 
+  connect( mCanvas->mapRenderer(), SIGNAL( destinationSrsChanged() ), this, SLOT( setTransform() ) );
+
   // Connect project
   connect( qgis, SIGNAL( projectRead() ), this, SLOT( projectRead() ) );
   connect( qgis, SIGNAL( newProject() ), this, SLOT( newProject() ) );
@@ -245,6 +246,20 @@ void QgsGrassPlugin::mapsetChanged()
     {
       mTools->mapsetChanged();
     }
+    QString gisdbase = QgsGrass::getDefaultGisdbase();
+    QString location = QgsGrass::getDefaultLocation();
+    try
+    {
+      mCrs = QgsGrass::crsDirect( gisdbase, location );
+    }
+    catch ( QgsGrass::Exception &e )
+    {
+      QgsDebugMsg( "Cannot read GRASS CRS : " + QString( e.what() ) );
+      mCrs = QgsCoordinateReferenceSystem();
+    }
+    QgsDebugMsg( "mCrs: " + mCrs.toWkt() );
+    setTransform();
+    redrawRegion();
   }
 }
 
@@ -303,11 +318,25 @@ void QgsGrassPlugin::addVector()
     /* Open vector */
     try
     {
-      Vect_set_open_level( 2 );
+      //Vect_set_open_level( 2 );
       struct Map_info map;
       int level = Vect_open_old_head( &map, sel->map.toUtf8().data(),
                                       sel->mapset.toUtf8().data() );
 
+      if ( level == 1 )
+      {
+        QgsDebugMsg( "Cannot open vector on level 2" );
+        QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open vector %1 in mapset %2 on level 2 (topology not available, try to rebuild topology using v.build module)." ).arg( sel->map ).arg( sel->mapset ) );
+        Vect_close( &map );
+        return;
+      }
+      else if ( level < 1 )
+      {
+        QgsDebugMsg( "Cannot open vector" );
+        QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open vector %1 in mapset %2" ).arg( sel->map ).arg( sel->mapset ) );
+        return;
+      }
+
       if ( level >= 2 )
       {
         // Count layers
@@ -337,7 +366,7 @@ void QgsGrassPlugin::addVector()
     }
     catch ( QgsGrass::Exception &e )
     {
-      QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open GRASS vector:\n %1" ).arg( QString::fromUtf8( e.what() ) ) );
+      QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open GRASS vector:\n %1" ).arg( e.what() ) );
     }
 
     qGisInterface->addVectorLayer( uri, name, "grass" );
@@ -384,7 +413,9 @@ void QgsGrassPlugin::addRaster()
     QString name = uri.right( uri.length() - pos - 1 );
     name.replace( '/', ' ' );
 
-    qGisInterface->addRasterLayer( uri, sel->map );
+    //qGisInterface->addRasterLayer( uri, sel->map );
+    qGisInterface->addRasterLayer( uri, sel->map, "grassraster", QStringList(), QStringList(),
+                                   QString(), QString() );
   }
 }
 
@@ -421,6 +452,7 @@ void QgsGrassPlugin::edit()
     mCanvas->refresh();
     connect( mEdit, SIGNAL( finished() ), this, SLOT( setEditAction() ) );
     connect( mEdit, SIGNAL( finished() ), this, SLOT( cleanUp() ) );
+    connect( mEdit, SIGNAL( destroyed() ), this, SLOT( editClosed() ) );
     connect( QgsMapLayerRegistry::instance(), SIGNAL( layerWillBeRemoved( QString ) ), this, SLOT( closeEdit( QString ) ) );
   }
   else
@@ -449,12 +481,18 @@ void QgsGrassPlugin::setEditAction()
 
 void QgsGrassPlugin::closeEdit( QString layerId )
 {
-  if ( mEdit->layer()->getLayerID() == layerId )
+  if ( mEdit->layer()->id() == layerId )
   {
     mEdit->closeEdit();
   }
 }
 
+void QgsGrassPlugin::editClosed()
+{
+  if( mEdit == sender() )
+    mEdit = 0;
+}
+
 void QgsGrassPlugin::cleanUp()
 {
   disconnect( QgsMapLayerRegistry::instance(), SIGNAL( layerWillBeRemoved( QString ) ), this, SLOT( closeEdit( QString ) ) );
@@ -502,7 +540,7 @@ void QgsGrassPlugin::newVector()
   catch ( QgsGrass::Exception &e )
   {
     QMessageBox::warning( 0, tr( "Warning" ),
-                          tr( "Cannot create new vector: %1" ).arg( QString::fromUtf8( e.what() ) ) );
+                          tr( "Cannot create new vector: %1" ).arg( e.what() ) );
     return;
   }
 
@@ -548,6 +586,8 @@ void QgsGrassPlugin::newVector()
 
 void QgsGrassPlugin::postRender( QPainter *painter )
 {
+  // We have to redraw rectangle, because canvas->mapRenderer()->destinationCrs is set after GRASS plugin constructor! This way it is redrawn also if canvas CRS has changed.
+  displayRegion();
 // QgsDebugMsg("entered.");
 }
 
@@ -556,6 +596,7 @@ void QgsGrassPlugin::displayRegion()
 // QgsDebugMsg("entered.");
 
   mRegionBand->reset();
+  if ( !mRegionAction->isChecked() ) { return; }
 
   // Display region of current mapset if in active mode
   if ( !QgsGrass::activeMode() ) return;
@@ -582,19 +623,9 @@ void QgsGrassPlugin::displayRegion()
     return;
   }
 
-  std::vector<QgsPoint> points;
-  points.resize( 5 );
+  QgsRectangle rect( QgsPoint( window.west, window.north ), QgsPoint( window.east, window.south ) );
 
-  points[0].setX( window.west ); points[0].setY( window.south );
-  points[1].setX( window.east ); points[1].setY( window.south );
-  points[2].setX( window.east ); points[2].setY( window.north );
-  points[3].setX( window.west ); points[3].setY( window.north );
-  points[4].setX( window.west ); points[4].setY( window.south );
-
-  for ( int i = 0; i < 5; i++ )
-  {
-    mRegionBand->addPoint( points[i] );
-  }
+  QgsGrassRegionEdit::drawRegion( mCanvas, mRegionBand, rect, &mCoordinateTransform );
 }
 
 void QgsGrassPlugin::switchRegion( bool on )
@@ -618,10 +649,7 @@ void QgsGrassPlugin::redrawRegion()
 {
 // QgsDebugMsg("entered.");
 
-  if ( mRegionAction->isChecked() )
-  {
-    displayRegion();
-  }
+  displayRegion();
 }
 
 void QgsGrassPlugin::changeRegion( void )
@@ -635,7 +663,7 @@ void QgsGrassPlugin::changeRegion( void )
   }
 
   // Warning: don't use Qt::WType_Dialog, it would ignore restorePosition
-  mRegion = new QgsGrassRegion( this, qGisInterface, qGisInterface->mainWindow(), Qt::Window );
+  mRegion = new QgsGrassRegion( this, qGisInterface, qGisInterface->mainWindow() );
 
   connect( mRegion, SIGNAL( destroyed( QObject * ) ), this, SLOT( regionClosed() ) );
 
@@ -857,6 +885,17 @@ QIcon QgsGrassPlugin::getThemeIcon( const QString theName )
   }
 }
 
+void QgsGrassPlugin::setTransform()
+{
+  if ( mCrs.isValid() && mCanvas->mapRenderer()->destinationCrs().isValid() )
+  {
+    QgsDebugMsg( "srcCrs: " + mCrs.toWkt() );
+    QgsDebugMsg( "destCrs " + mCanvas->mapRenderer()->destinationCrs().toWkt() );
+    mCoordinateTransform.setSourceCrs( mCrs );
+    mCoordinateTransform.setDestCRS( mCanvas->mapRenderer()->destinationCrs() );
+  }
+}
+
 /**
  * Required extern functions needed  for every plugin
  * These functions can be called prior to creating an instance
@@ -893,6 +932,11 @@ QGISEXTERN QString version()
   return pluginVersion;
 }
 
+QGISEXTERN QString icon()
+{
+  return pluginIcon;
+}
+
 // Delete ourself
 QGISEXTERN void unload( QgisPlugin * thePluginPointer )
 {
