@@ -34,6 +34,8 @@ QgsGrassSelect::QgsGrassSelect( int type ): QgsGrassSelectBase()
   QgsDebugMsg( QString( "QgsGrassSelect() type = %1" ).arg( type ) );
 
   setupUi( this );
+  connect( buttonBox, SIGNAL( accepted() ), SLOT( on_ok_clicked() ) );
+  connect( buttonBox, SIGNAL( rejected() ), this, SLOT( on_cancel_clicked() ) );
 
   if ( first )
   {
@@ -90,28 +92,11 @@ QgsGrassSelect::QgsGrassSelect( int type ): QgsGrassSelectBase()
   egisdbase->setText( lastGisdbase );
 
   setLocations();
-
-  restorePosition();
+  adjustSize();
 }
 
 QgsGrassSelect::~QgsGrassSelect()
 {
-  saveWindowLocation();
-}
-
-void QgsGrassSelect::restorePosition()
-{
-  optionsFrame->adjustSize();
-  adjustSize();
-
-  QSettings settings;
-  restoreGeometry( settings.value( "/GRASS/windows/select/geometry" ).toByteArray() );
-}
-
-void QgsGrassSelect::saveWindowLocation()
-{
-  QSettings settings;
-  settings.setValue( "/GRASS/windows/select/geometry", saveGeometry() );
 }
 
 bool QgsGrassSelect::first = true;
@@ -141,21 +126,15 @@ void QgsGrassSelect::setLocations()
 
     QString ldpath = egisdbase->text() + "/" + d[i];
 
-    /* TODO: G_is_location() was added to GRASS 6.1 06-05-24,
-    enable its use after some period (others do update) */
-#if 0
     if ( QgsGrass::versionMajor() > 6 || QgsGrass::versionMinor() > 0 )
     {
       if ( !G_is_location( ldpath.toLocal8Bit().constData() ) ) continue;
     }
     else
     {
-#endif
       QString chf = egisdbase->text() + "/" + d[i] + "/PERMANENT/DEFAULT_WIND";
       if ( !QFile::exists( chf ) ) continue;
-#if 0
     }
-#endif
 
     // if type is MAPSET check also if at least one mapset owned by user exists
     if ( QgsGrassSelect::type == QgsGrassSelect::MAPSET )
@@ -191,7 +170,7 @@ void QgsGrassSelect::setLocations()
   {
     elocation->setCurrentIndex( sel );
   }
-  ok->setDefault( true );
+  buttonBox->button( QDialogButtonBox::Ok )->setDefault( true );
   GisdbaseBrowse->setDefault( elocation->count() == 0 );
 
   setMapsets();
@@ -234,7 +213,7 @@ void QgsGrassSelect::setMapsets()
   }
   if ( emap->isHidden() )
   {
-    ok->setDefault( emapset->count() > 0 );
+    buttonBox->button( QDialogButtonBox::Ok )->setDefault( emapset->count() > 0 );
   }
 
   setMaps();
@@ -329,7 +308,7 @@ void QgsGrassSelect::setMaps()
   */
   if ( !emap->isHidden() )
   {
-    ok->setDefault( emap->count() > 0 );
+    buttonBox->button( QDialogButtonBox::Ok )->setDefault( emap->count() > 0 );
   }
 
   setLayers();
@@ -400,17 +379,10 @@ QStringList QgsGrassSelect::vectorLayers( QString gisdbase,
 
   /* Open vector */
   QgsGrass::resetError();
-  Vect_set_open_level( 2 );
+  //Vect_set_open_level( 2 );
   struct Map_info map;
   int level = -1;
 
-  // Mechanism to recover from fatal errors in GRASS
-  // Since fatal error routine in GRASS >= 6.3 terminates the process,
-  // we use setjmp() to set recovery place in case of a fatal error.
-  // Call to setjmp() returns 0 first time. In case of fatal error,
-  // our error routine uses longjmp() to come back to this context,
-  // this time setjmp() will return non-zero value and we can continue...
-
   try
   {
     level = Vect_open_old_head( &map, ( char * ) mapName.toUtf8().data(), ( char * ) mapset.toUtf8().data() );
@@ -422,10 +394,22 @@ QStringList QgsGrassSelect::vectorLayers( QString gisdbase,
     return list;
   }
 
-  if ( level < 2 )
+  if ( level == 1 )
   {
     QgsDebugMsg( "Cannot open vector on level 2" );
-    QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open vector on level 2 (topology not available)." ) );
+    QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open vector %1 in mapset %2 on level 2 (topology not available, try to rebuild topology using v.build module)." ).arg( mapName ).arg( mapset ) );
+    // Vect_close here is correct, it should work, but it seems to cause
+    // crash on win http://trac.osgeo.org/qgis/ticket/2003
+    // disabled on win test it
+#if !defined(WIN32)
+    Vect_close( &map );
+#endif
+    return list;
+  }
+  else if ( level < 1 )
+  {
+    QgsDebugMsg( "Cannot open vector" );
+    QMessageBox::warning( 0, tr( "Warning" ), tr( "Cannot open vector %1 in mapset %2" ).arg( mapName ).arg( mapset ) );
     return list;
   }
 
@@ -492,8 +476,6 @@ void QgsGrassSelect::on_GisdbaseBrowse_clicked()
 
 void QgsGrassSelect::on_ok_clicked()
 {
-  saveWindowLocation();
-
   gisdbase = egisdbase->text();
   lastGisdbase = QString( gisdbase );
 
@@ -557,6 +539,5 @@ void QgsGrassSelect::on_ok_clicked()
 
 void QgsGrassSelect::on_cancel_clicked()
 {
-  saveWindowLocation();
   QDialog::reject();
 }
