@@ -743,16 +743,16 @@ void TerminalDisplay::scrollImage( int lines , const QRect& screenWindowRegion )
   if ( lines == 0
        || _image == 0
        || !region.isValid()
-       || ( region.top() + abs( lines ) ) >= region.bottom()
+       || ( region.top() + qAbs( lines ) ) >= region.bottom()
        || this->_lines <= region.height() ) return;
 
   QRect scrollRect;
 
   void* firstCharPos = &_image[ region.top() * this->_columns ];
-  void* lastCharPos = &_image[( region.top() + abs( lines ) ) * this->_columns ];
+  void* lastCharPos = &_image[( region.top() + qAbs( lines ) ) * this->_columns ];
 
   int top = _topMargin + ( region.top() * _fontHeight );
-  int linesToMove = region.height() - abs( lines );
+  int linesToMove = region.height() - qAbs( lines );
   int bytesToMove = linesToMove *
                     this->_columns *
                     sizeof( Character );
@@ -789,7 +789,7 @@ void TerminalDisplay::scrollImage( int lines , const QRect& screenWindowRegion )
 
     //set region of the display to scroll, making sure that
     //the region aligns correctly to the character grid
-    QPoint topPoint( _leftMargin , top + abs( lines )*_fontHeight );
+    QPoint topPoint( _leftMargin , top + qAbs( lines )*_fontHeight );
 
     scrollRect = QRect( topPoint ,
                         QSize( this->_usedColumns * _fontWidth ,
@@ -1747,7 +1747,6 @@ void TerminalDisplay::extendSelection( const QPoint& position )
   // this widget.
 
   // Adjust position within text area bounds. See FIXME above.
-  QPoint oldpos = pos;
   if ( pos.x() < tLx + _leftMargin )
     pos.setX( tLx + _leftMargin );
   if ( pos.x() > tLx + _leftMargin + _usedColumns*_fontWidth - 1 )
@@ -2423,6 +2422,8 @@ QVariant TerminalDisplay::inputMethodQuery( Qt::InputMethodQuery query ) const
     case Qt::ImCurrentSelection:
       return QString();
       break;
+    default:
+      break;
   }
 
   return QVariant();
