@@ -26,6 +26,12 @@
 
 #include "qgsvectorlayer.h"
 
+#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1800
+#define TO8(x) (x).toUtf8().constData()
+#else
+#define TO8(x) (x).toLocal8Bit().constData()
+#endif
+
 QgsOracleSelectGeoraster::QgsOracleSelectGeoraster( QWidget* parent,
     QgisInterface* iface,
     Qt::WFlags fl ) : QDialog( parent, fl ), mIface( iface )
@@ -92,7 +98,7 @@ void QgsOracleSelectGeoraster::populateConnectionList()
   }
 }
 
-void QgsOracleSelectGeoraster::addNewConnection()
+void QgsOracleSelectGeoraster::on_btnNew_clicked()
 {
   QgsOracleConnect *oc = new QgsOracleConnect( this, "New Connection" );
   if ( oc->exec() )
@@ -101,7 +107,7 @@ void QgsOracleSelectGeoraster::addNewConnection()
   }
 }
 
-void QgsOracleSelectGeoraster::editConnection()
+void QgsOracleSelectGeoraster::on_btnEdit_clicked()
 {
   QgsOracleConnect *oc = new QgsOracleConnect( this, cmbConnections->currentText() );
   if ( oc->exec() )
@@ -110,7 +116,7 @@ void QgsOracleSelectGeoraster::editConnection()
   }
 }
 
-void QgsOracleSelectGeoraster::deleteConnection()
+void QgsOracleSelectGeoraster::on_btnDelete_clicked()
 {
   QSettings settings;
   QString key = "/Oracle/connections/" + cmbConnections->currentText();
@@ -137,6 +143,9 @@ void QgsOracleSelectGeoraster::deleteConnection()
 
 void QgsOracleSelectGeoraster::connectToServer()
 {
+  if ( cmbConnections->currentText().isEmpty() )
+    return;
+
   QSettings settings;
   QString key = "/Oracle/connections/" + cmbConnections->currentText();
   QString username = settings.value( key + "/username" ).toString();
@@ -184,7 +193,7 @@ void QgsOracleSelectGeoraster::showSelection( const QString & line )
    *  Try to open georaster dataset
    */
 
-  hDS = GDALOpenShared( identification.toAscii(), eAccess );
+  hDS = GDALOpenShared( TO8( identification ), eAccess );
 
   buttonBox->button( QDialogButtonBox::Ok )->setEnabled( false );
   if ( hDS == NULL )
@@ -281,32 +290,30 @@ void QgsOracleSelectGeoraster::showSelection( const QString & line )
   GDALClose( hDS );
 }
 
-void QgsOracleSelectGeoraster::showHelp()
+void QgsOracleSelectGeoraster::on_listWidget_clicked( QModelIndex Index )
 {
-  // implement me
+  if ( lineEdit->text() == listWidget->currentItem()->text() )
+  {
+    showSelection( lineEdit->text() );
+  }
+  else
+  {
+    lineEdit->setText( listWidget->currentItem()->text() );
+  }
 }
 
 void QgsOracleSelectGeoraster::setConnectionListPosition()
 {
-  QSettings settings;
   // If possible, set the item currently displayed database
+  QSettings settings;
   QString toSelect = settings.value( "/Oracle/connections/selected" ).toString();
-  // Does toSelect exist in cmbConnections?
-  bool set = false;
-  for ( int i = 0; i < cmbConnections->count(); ++ i )
-  {
-    if ( cmbConnections->itemText( i ) == toSelect )
-    {
-      cmbConnections->setCurrentIndex( i );
-      set = true;
-      break;
-    }
-  }
+  cmbConnections->setCurrentIndex( cmbConnections->findText( toSelect ) );
+
   // If we couldn't find the stored item, but there are some,
   // default to the last item (this makes some sense when deleting
   // items as it allows the user to repeatidly click on delete to
   // remove a whole lot of items).
-  if ( ! set && cmbConnections->count() > 0 )
+  if ( cmbConnections->currentIndex() == -1 && cmbConnections->count() > 0 )
   {
     // If toSelect is null, then the selected connection wasn't found
     // by QSettings, which probably means that this is the first time
