@@ -43,7 +43,7 @@ QgsRasterTerrainAnalysisDialog::QgsRasterTerrainAnalysisDialog( QgisInterface* i
     QgsRasterLayer* rl = qobject_cast<QgsRasterLayer *>( layer_it.value() );
     if ( rl )
     {
-      mInputLayerComboBox->addItem( rl->name(), QVariant( rl->getLayerID() ) );
+      mInputLayerComboBox->addItem( rl->name(), QVariant( rl->id() ) );
     }
   }
 
@@ -57,7 +57,7 @@ QgsRasterTerrainAnalysisDialog::QgsRasterTerrainAnalysisDialog( QgisInterface* i
     if ( driver != NULL )
     {
       char** driverMetadata = GDALGetMetadata( driver, NULL );
-      if ( CSLFetchBoolean( driverMetadata, GDAL_DCAP_CREATE, FALSE ) )
+      if ( CSLFetchBoolean( driverMetadata, GDAL_DCAP_CREATE, false ) )
       {
         mOutputFormatComboBox->addItem( GDALGetDriverLongName( driver ), QVariant( GDALGetDriverShortName( driver ) ) );
 
@@ -159,19 +159,14 @@ QString QgsRasterTerrainAnalysisDialog::selectedOuputFilePath() const
 
 bool QgsRasterTerrainAnalysisDialog::addLayerToProject() const
 {
-  if ( mAddResultToProjectCheckBox->checkState() == Qt::Checked )
-  {
-    return true;
-  }
-  else
-  {
-    return false;
-  }
+  return mAddResultToProjectCheckBox->checkState() == Qt::Checked;
 }
 
 void QgsRasterTerrainAnalysisDialog::on_mOutputLayerPushButton_clicked()
 {
-  QString saveFileName = QFileDialog::getSaveFileName( 0, tr( "Enter result file" ) );
+  QSettings s;
+  QString lastDir = s.value( "/RasterTerrainAnalysis/lastOutputDir" ).toString();
+  QString saveFileName = QFileDialog::getSaveFileName( 0, tr( "Enter result file" ), lastDir );
   if ( !saveFileName.isNull() )
   {
     mOutputLayerLineEdit->setText( saveFileName );
@@ -188,6 +183,10 @@ void QgsRasterTerrainAnalysisDialog::on_mButtonBox_accepted()
   //save last output format
   QSettings s;
   s.setValue( "/RasterTerrainAnalysis/lastOutputFormat", QVariant( mOutputFormatComboBox->currentText() ) );
+
+  //save last output directory
+  QFileInfo outputFileInfo( mOutputLayerLineEdit->text() );
+  s.setValue( "/RasterTerrainAnalysis/lastOutputDir", QVariant( outputFileInfo.absolutePath() ) );
 }
 
 void QgsRasterTerrainAnalysisDialog::on_mOutputLayerLineEdit_textChanged( const QString& text )
@@ -198,8 +197,8 @@ void QgsRasterTerrainAnalysisDialog::on_mOutputLayerLineEdit_textChanged( const
     return;
   }
 
-  QFileInfo fileInfo( text );
-  if ( mInputLayerComboBox->count() > 0 && fileInfo.dir().exists() )
+  QString outputPath = QFileInfo( text ).absolutePath();
+  if ( mInputLayerComboBox->count() > 0 && QFileInfo( outputPath ).isWritable() )
   {
     okButton->setEnabled( true );
   }
