@@ -64,6 +64,7 @@ static const QString name_ = QObject::tr( "ScaleBar" );
 static const QString description_ = QObject::tr( "Draws a scale bar" );
 static const QString version_ = QObject::tr( "Version 0.1" );
 static const QgisPlugin::PLUGINTYPE type_ = QgisPlugin::UI;
+static const QString icon_ = ":/scale_bar.png";
 
 
 /**
@@ -191,7 +192,7 @@ void QgsScaleBarPlugin::renderScaleBar( QPainter * theQPainter )
   //Get map units per pixel. This can be negative at times (to do with
   //projections) and that just confuses the rest of the code in this
   //function, so force to a positive number.
-  double myMapUnitsPerPixelDouble = std::abs( qGisInterface->mapCanvas()->mapUnitsPerPixel() );
+  double myMapUnitsPerPixelDouble = qAbs( qGisInterface->mapCanvas()->mapUnitsPerPixel() );
 
   // Exit if the canvas width is 0 or layercount is 0 or QGIS will freeze
   int myLayerCount = qGisInterface->mapCanvas()->layerCount();
@@ -660,6 +661,11 @@ QGISEXTERN QString version()
   return version_;
 }
 
+QGISEXTERN QString icon()
+{
+  return icon_;
+}
+
 // Delete ourself
 QGISEXTERN void unload( QgisPlugin * thePluginPointer )
 {
