@@ -36,25 +36,11 @@
 
 #include "qgspgutil.h"
 #include "qgsspit.h"
-#include "qgsnewconnection.h"
+#include "qgspgnewconnection.h"
 #include "qgsdatasourceuri.h"
 #include "qgsmessageviewer.h"
 #include "qgslogger.h"
 
-// Qt implementation of alignment() + changed the numeric types to be shown on the left as well
-/* Is this still needed? Numbers in Qt4 table seem to be left justified by default.
-int Q3TableItem::alignment() const
-{
-  bool num;
-  bool ok1 = FALSE, ok2 = FALSE;
-  ( void ) txt.toInt( &ok1 );
-  if ( !ok1 )
-    ( void ) txt.toDouble( &ok2 );
-  num = ok1 || ok2;
-
-  return ( num ? Qt::AlignLeft : Qt::AlignLeft ) | Qt::AlignVCenter;
-}
-*/
 
 QgsSpit::QgsSpit( QWidget *parent, Qt::WFlags fl ) : QDialog( parent, fl )
 {
@@ -125,7 +111,7 @@ void QgsSpit::populateConnectionList()
 
 void QgsSpit::newConnection()
 {
-  QgsNewConnection *nc = new QgsNewConnection( this );
+  QgsPgNewConnection *nc = new QgsPgNewConnection( this );
   nc->exec();
   delete nc;
 
@@ -134,7 +120,7 @@ void QgsSpit::newConnection()
 
 void QgsSpit::editConnection()
 {
-  QgsNewConnection *nc = new QgsNewConnection( this, cmbConnections->currentText() );
+  QgsPgNewConnection *nc = new QgsPgNewConnection( this, cmbConnections->currentText() );
   nc->exec();
   delete nc;
 
@@ -175,7 +161,7 @@ void QgsSpit::addFile()
   QgsEncodingFileDialog dlg( this,
                              tr( "Add Shapefiles" ),
                              settings.value( "/Plugin-Spit/last_directory" ).toString(),
-                             tr( "Shapefiles (*.shp);;All files (*.*)" ),
+                             tr( "Shapefiles (*.shp);;All files (*)" ),
                              settings.value( "/Plugin-Spit/last_encoding" ).toString() );
   dlg.setFileMode( QFileDialog::ExistingFiles );
 
@@ -437,6 +423,17 @@ void QgsSpit::dbConnect()
     }
   }
 
+  if ( conn )
+  {
+    PGresult *res = PQexec( conn, "SET application_name='Quantum GIS'" );
+    if ( PQresultStatus( res ) != PGRES_COMMAND_OK )
+    {
+      PQclear( res );
+      res = PQexec( conn, "ROLLBACK" );
+    }
+    PQclear( res );
+  }
+
   schema_list.clear();
   schema_list << "public";
 
@@ -805,7 +802,7 @@ void QgsSpit::import()
       else if ( !canceled )
       { // if problem importing file occured
         pro.setValue( temp_progress + tblShapefiles->item( i, ColFEATURECOUNT )->text().toInt() );
-        QString errTxt = error + "\n" + errorText;
+        QString errTxt = "<pre>" + error + "\n\n" + errorText + "</pre>";
         QMessageBox::warning( this, tr( "Import Shapefiles" ), errTxt );
         query = "ROLLBACK";
         res = PQexec( conn, query.toUtf8() );
