@@ -18,7 +18,6 @@
 
 #include "qgsdelimitedtextprovider.h"
 
-
 #include <QtGlobal>
 #include <QFile>
 #include <QDataStream>
@@ -29,24 +28,48 @@
 #include <QRegExp>
 #include <QUrl>
 
-
 #include "qgsapplication.h"
 #include "qgsdataprovider.h"
 #include "qgsfeature.h"
 #include "qgsfield.h"
+#include "qgsgeometry.h"
 #include "qgslogger.h"
 #include "qgsmessageoutput.h"
 #include "qgsrectangle.h"
-#include "qgscoordinatereferencesystem.h"
 #include "qgis.h"
 
 static const QString TEXT_PROVIDER_KEY = "delimitedtext";
 static const QString TEXT_PROVIDER_DESCRIPTION = "Delimited text data provider";
 
 
+QString QgsDelimitedTextProvider::readLine( QTextStream *stream )
+{
+  QString buffer;
+
+  while ( !stream->atEnd() )
+  {
+    QChar c = stream->read( 1 ).at( 0 );
+
+    if ( c == '\r' || c == '\n' )
+    {
+      if ( buffer.isEmpty() )
+      {
+        // skip leading CR / LF
+        continue;
+      }
+
+      break;
+    }
+
+    buffer.append( c );
+  }
+
+  return buffer;
+}
+
 QStringList QgsDelimitedTextProvider::splitLine( QString line )
 {
-  QgsDebugMsg( "Attempting to split the input line: " + line + " using delimiter " + mDelimiter );
+  QgsDebugMsgLevel( "Attempting to split the input line: " + line + " using delimiter " + mDelimiter, 3 );
 
   QStringList parts;
   if ( mDelimiterType == "regexp" )
@@ -54,7 +77,7 @@ QStringList QgsDelimitedTextProvider::splitLine( QString line )
   else
     parts = line.split( mDelimiter );
 
-  QgsDebugMsg( "Split line into " + QString::number( parts.size() ) + " parts" );
+  QgsDebugMsgLevel( "Split line into " + QString::number( parts.size() ) + " parts", 3 );
 
   if ( mDelimiterType == "plain" )
   {
@@ -106,39 +129,48 @@ QStringList QgsDelimitedTextProvider::splitLine( QString line )
 }
 
 QgsDelimitedTextProvider::QgsDelimitedTextProvider( QString uri )
-    : QgsVectorDataProvider( uri ),
-    mXFieldIndex( -1 ), mYFieldIndex( -1 ),
-    mShowInvalidLines( true )
+    : QgsVectorDataProvider( uri )
+    , mDelimiter( "," )
+    , mDelimiterRegexp()
+    , mDelimiterType( "plain" )
+    , mFieldCount( 0 )
+    , mXFieldIndex( -1 )
+    , mYFieldIndex( -1 )
+    , mWktFieldIndex( -1 )
+    , mWktHasZM( false )
+    , mWktZMRegexp( "\\s+(?:z|m|zm)(?=\\s*\\()", Qt::CaseInsensitive )
+    , mWktCrdRegexp( "(\\-?\\d+(?:\\.\\d*)?\\s+\\-?\\d+(?:\\.\\d*)?)\\s[\\s\\d\\.\\-]+" )
+    , mSkipLines( 0 )
+    , mFirstDataLine( 0 )
+    , mShowInvalidLines( false )
+    , mCrs()
+    , mWkbType( QGis::WKBUnknown )
 {
-  // Get the file name and mDelimiter out of the uri
-  mFileName = uri.left( uri.indexOf( "?" ) );
-  // split the string up on & to get the individual parameters
-  QStringList parameters = uri.mid( uri.indexOf( "?" ) ).split( "&", QString::SkipEmptyParts );
+  QUrl url = QUrl::fromEncoded( uri.toAscii() );
 
-  QgsDebugMsg( "Parameter count after split on &" + QString::number( parameters.size() ) );
+  // Extract the provider definition from the url
+  mFileName = url.toLocalFile();
 
-  // get the individual parameters and assign values
-  QStringList temp = parameters.filter( "delimiter=" );
-  mDelimiter = temp.size() ? temp[0].mid( temp[0].indexOf( "=" ) + 1 ) : "";
-  temp = parameters.filter( "delimiterType=" );
-  mDelimiterType = temp.size() ? temp[0].mid( temp[0].indexOf( "=" ) + 1 ) : "";
-  temp = parameters.filter( "xField=" );
-  QString xField = temp.size() ? temp[0].mid( temp[0].indexOf( "=" ) + 1 ) : "";
-  temp = parameters.filter( "yField=" );
-  QString yField = temp.size() ? temp[0].mid( temp[0].indexOf( "=" ) + 1 ) : "";
-  // Decode the parts of the uri. Good if someone entered '=' as a delimiter, for instance.
-  mFileName  = QUrl::fromPercentEncoding( mFileName.toUtf8() );
-  mDelimiter = QUrl::fromPercentEncoding( mDelimiter.toUtf8() );
-  mDelimiterType = QUrl::fromPercentEncoding( mDelimiterType.toUtf8() );
-  xField    = QUrl::fromPercentEncoding( xField.toUtf8() );
-  yField    = QUrl::fromPercentEncoding( yField.toUtf8() );
+  QString wktField;
+  QString xField;
+  QString yField;
+
+  if ( url.hasQueryItem( "delimiter" ) ) mDelimiter = url.queryItemValue( "delimiter" );
+  if ( url.hasQueryItem( "delimiterType" ) ) mDelimiterType = url.queryItemValue( "delimiterType" );
+  if ( url.hasQueryItem( "wktField" ) ) wktField = url.queryItemValue( "wktField" );
+  if ( url.hasQueryItem( "xField" ) ) xField = url.queryItemValue( "xField" );
+  if ( url.hasQueryItem( "yField" ) ) yField = url.queryItemValue( "yField" );
+  if ( url.hasQueryItem( "skipLines" ) ) mSkipLines = url.queryItemValue( "skipLines" ).toInt();
+  if ( url.hasQueryItem( "crs" ) ) mCrs.createFromString( url.queryItemValue( "crs" ) );
 
   QgsDebugMsg( "Data source uri is " + uri );
   QgsDebugMsg( "Delimited text file is: " + mFileName );
   QgsDebugMsg( "Delimiter is: " + mDelimiter );
   QgsDebugMsg( "Delimiter type is: " + mDelimiterType );
+  QgsDebugMsg( "wktField is: " + wktField );
   QgsDebugMsg( "xField is: " + xField );
   QgsDebugMsg( "yField is: " + yField );
+  QgsDebugMsg( "skipLines is: " + QString::number( mSkipLines ) );
 
   // if delimiter contains some special characters, convert them
   if ( mDelimiterType == "regexp" )
@@ -150,10 +182,10 @@ QgsDelimitedTextProvider::QgsDelimitedTextProvider( QString uri )
   mSelectionRectangle = QgsRectangle();
   // assume the layer is invalid until proven otherwise
   mValid = false;
-  if ( mFileName.isEmpty() || mDelimiter.isEmpty() || xField.isEmpty() || yField.isEmpty() )
+  if ( mFileName.isEmpty() || mDelimiter.isEmpty() )
   {
     // uri is invalid so the layer must be too...
-    QString( "Data source is invalid" );
+    QgsDebugMsg( "Data source is invalid" );
     return;
   }
 
@@ -187,104 +219,199 @@ QgsDelimitedTextProvider::QgsDelimitedTextProvider( QString uri )
   QString line;
   mNumberFeatures = 0;
   int lineNumber = 0;
-  bool firstPoint = true;
   bool hasFields = false;
   while ( !mStream->atEnd() )
   {
     lineNumber++;
-    line = mStream->readLine(); // line of text excluding '\n', default local 8 bit encoding.
+    line = readLine( mStream ); // line of text excluding '\n', default local 8 bit encoding.
+
+    if ( lineNumber < mSkipLines + 1 )
+      continue;
+
+    if ( line.isEmpty() )
+      continue;
+
     if ( !hasFields )
     {
       // Get the fields from the header row and store them in the
       // fields vector
       QStringList fieldList = splitLine( line );
 
+      mFieldCount = fieldList.count();
+
       // We don't know anything about a text based field other
       // than its name. All fields are assumed to be text
       int fieldPos = 0;
-      for ( QStringList::Iterator it = fieldList.begin();
-            it != fieldList.end(); ++it )
+      for ( int column = 0; column < mFieldCount; column++ )
       {
-        QString field = *it;
-        if ( field.length() > 0 )
+        QString field = fieldList[column];
+
+        if (( field.left( 1 ) == "'" || field.left( 1 ) == "\"" ) &&
+            field.left( 1 ) == field.right( 1 ) )
+          // eat quotes
+          field = field.mid( 1, field.length() - 2 );
+
+        if ( field.length() == 0 )
+          // skip empty field names
+          continue;
+
+        // check to see if this field matches either the x or y field
+        if ( !wktField.isEmpty() && wktField == field )
         {
-          // for now, let's set field type as text
-          attributeFields[fieldPos] = QgsField( *it, QVariant::String, "Text" );
-
-          // check to see if this field matches either the x or y field
-          if ( xField == *it )
-          {
-            QgsDebugMsg( "Found x field: " + ( *it ) );
-            mXFieldIndex = fieldPos;
-          }
-          else if ( yField == *it )
-          {
-            QgsDebugMsg( "Found y field: " + ( *it ) );
-            mYFieldIndex = fieldPos;
-          }
-
-          QgsDebugMsg( "Adding field: " + ( *it ) );
-          // assume that the field could be integer or double
-          couldBeInt.insert( fieldPos, true );
-          couldBeDouble.insert( fieldPos, true );
-          fieldPos++;
+          QgsDebugMsg( "Found wkt field: " + ( field ) );
+          mWktFieldIndex = column;
         }
+        else if ( !xField.isEmpty() && xField == field )
+        {
+          QgsDebugMsg( "Found x field: " + ( field ) );
+          mXFieldIndex = column;
+        }
+        else if ( !yField.isEmpty() && yField == field )
+        {
+          QgsDebugMsg( "Found y field: " + ( field ) );
+          mYFieldIndex = column;
+        }
+
+        // WKT geometry field won't be displayed in attribute tables
+        if ( column == mWktFieldIndex )
+          continue;
+
+        QgsDebugMsg( "Adding field: " + ( field ) );
+        // assume that the field could be integer or double
+        // for now, let's set field type as text
+        attributeColumns.append( column );
+        attributeFields[fieldPos] = QgsField( field, QVariant::String, "Text" );
+        couldBeInt.insert( fieldPos, true );
+        couldBeDouble.insert( fieldPos, true );
+        fieldPos++;
       }
+      if ( mWktFieldIndex >= 0 )
+      {
+        mXFieldIndex = -1;
+        mYFieldIndex = -1;
+      }
+      QgsDebugMsg( "wktfield index: " + QString::number( mWktFieldIndex ) );
+      QgsDebugMsg( "xfield index: " + QString::number( mXFieldIndex ) );
+      QgsDebugMsg( "yfield index: " + QString::number( mYFieldIndex ) );
       QgsDebugMsg( "Field count for the delimited text file is " + QString::number( attributeFields.size() ) );
       hasFields = true;
     }
-    else if ( mXFieldIndex != -1 && mYFieldIndex != -1 )
+    else // hasFields == true - field names already read
     {
-      mNumberFeatures++;
+      if ( mFirstDataLine == 0 )
+        mFirstDataLine = lineNumber;
 
       // split the line on the delimiter
       QStringList parts = splitLine( line );
 
-      // Skip malformed lines silently. Report line number with nextFeature()
-      if ( attributeFields.size() != parts.size() )
+      // Ensure that the input has at least the required number of fields (mainly to tolerate
+      // missed blank strings at end of row)
+      while ( parts.size() < mFieldCount )
+        parts.append( QString::null );
+
+      if ( mWktFieldIndex >= 0 )
       {
-        continue;
-      }
+        // Get the wkt - confirm it is valid, get the type, and
+        // if compatible with the rest of file, add to the extents
 
-      // Get the x and y values, first checking to make sure they
-      // aren't null.
-      QString sX = parts[mXFieldIndex];
-      QString sY = parts[mYFieldIndex];
-
-      bool xOk = true;
-      bool yOk = true;
-      double x = sX.toDouble( &xOk );
-      double y = sY.toDouble( &yOk );
-
-      if ( xOk && yOk )
-      {
-        if ( !firstPoint )
+        QString sWkt = parts[mWktFieldIndex];
+        QgsGeometry *geom = 0;
+        try
         {
-          mExtent.combineExtentWith( x, y );
+          if ( !mWktHasZM && sWkt.indexOf( mWktZMRegexp ) >= 0 )
+            mWktHasZM = true;
+          if ( mWktHasZM )
+          {
+            sWkt.remove( mWktZMRegexp ).replace( mWktCrdRegexp, "\\1" );
+          }
+          geom = QgsGeometry::fromWkt( sWkt );
+        }
+        catch ( ... )
+        {
+          mInvalidLines << line;
+          geom = 0;
+        }
+
+        if ( geom )
+        {
+          QGis::WkbType type = geom->wkbType();
+          if ( type != QGis::WKBNoGeometry )
+          {
+            if ( mNumberFeatures == 0 )
+            {
+              mNumberFeatures++;
+              mWkbType = type;
+              mExtent = geom->boundingBox();
+            }
+            else if ( type == mWkbType )
+            {
+              mNumberFeatures++;
+              QgsRectangle bbox( geom->boundingBox() );
+              mExtent.combineExtentWith( &bbox );
+            }
+          }
+          delete geom;
+        }
+      }
+      else if ( mWktFieldIndex == -1 && mXFieldIndex >= 0 && mYFieldIndex >= 0 )
+      {
+        // Get the x and y values, first checking to make sure they
+        // aren't null.
+
+        QString sX = parts[mXFieldIndex];
+        QString sY = parts[mYFieldIndex];
+
+        bool xOk = false;
+        bool yOk = false;
+        double x = sX.toDouble( &xOk );
+        double y = sY.toDouble( &yOk );
+
+        if ( xOk && yOk )
+        {
+          if ( mNumberFeatures > 0 )
+          {
+            mExtent.combineExtentWith( x, y );
+          }
+          else
+          {
+            // Extent for the first point is just the first point
+            mExtent.set( x, y, x, y );
+            mWkbType = QGis::WKBPoint;
+          }
+          mNumberFeatures++;
         }
         else
-        { // Extent for the first point is just the first point
-          mExtent.set( x, y, x, y );
-          firstPoint = false;
+        {
+          mInvalidLines << line;
         }
       }
-
-      int i = 0;
-      for ( QStringList::iterator it = parts.begin(); it != parts.end(); ++it, ++i )
+      else
       {
+        mWkbType = QGis::WKBNoGeometry;
+        mNumberFeatures++;
+      }
+
+      for ( int i = 0; i < attributeFields.size(); i++ )
+      {
+        QString &value = parts[attributeColumns[i]];
+        if ( value.isEmpty() )
+          continue;
         // try to convert attribute values to integer and double
         if ( couldBeInt[i] )
         {
-          it->toInt( &couldBeInt[i] );
+          value.toInt( &couldBeInt[i] );
         }
         if ( couldBeDouble[i] )
         {
-          it->toDouble( &couldBeDouble[i] );
+          value.toDouble( &couldBeDouble[i] );
         }
       }
     }
   }
 
+  QgsDebugMsg( "geometry type is: " + QString::number( mWkbType ) );
+  QgsDebugMsg( "feature count is: " + QString::number( mNumberFeatures ) );
+
   // now it's time to decide the types for the fields
   for ( QgsFieldMap::iterator it = attributeFields.begin(); it != attributeFields.end(); ++it )
   {
@@ -300,20 +427,7 @@ QgsDelimitedTextProvider::QgsDelimitedTextProvider( QString uri )
     }
   }
 
-  if ( mXFieldIndex != -1 && mYFieldIndex != -1 )
-  {
-    QgsDebugMsg( "Data store is valid" );
-    QgsDebugMsg( "Number of features " + QString::number( mNumberFeatures ) );
-    QgsDebugMsg( "Extents " + mExtent.toString() );
-
-    mValid = true;
-  }
-  else
-  {
-    QgsDebugMsg( "Data store is invalid. Specified x,y fields do not match those in the database" );
-  }
-  QgsDebugMsg( "Done checking validity" );
-
+  mValid = mWkbType != QGis::WKBUnknown;
 }
 
 QgsDelimitedTextProvider::~QgsDelimitedTextProvider()
@@ -335,109 +449,119 @@ bool QgsDelimitedTextProvider::nextFeature( QgsFeature& feature )
   // before we do anything else, assume that there's something wrong with
   // the feature
   feature.setValid( false );
-  while ( ! mStream->atEnd() )
+  while ( !mStream->atEnd() )
   {
-    double x = 0.0;
-    double y = 0.0;
-    QString line = mStream->readLine(); // Default local 8 bit encoding
+    QString line = readLine( mStream ); // Default local 8 bit encoding
+    if ( line.isEmpty() )
+      continue;
 
     // lex the tokens from the current data line
     QStringList tokens = splitLine( line );
 
-    bool xOk = false;
-    bool yOk = false;
+    while ( tokens.size() < mFieldCount )
+      tokens.append( QString::null );
 
-    // Skip indexing malformed lines.
-    if ( attributeFields.size() == tokens.size() )
+    QgsGeometry *geom = 0;
+
+    if ( mWktFieldIndex >= 0 )
     {
-      x = tokens[mXFieldIndex].toDouble( &xOk );
-      y = tokens[mYFieldIndex].toDouble( &yOk );
+      try
+      {
+        QString &sWkt = tokens[mWktFieldIndex];
+        // Remove Z and M coordinates if present, as currently fromWkt doesn't
+        // support these.
+        if ( mWktHasZM )
+        {
+          sWkt.remove( mWktZMRegexp ).replace( mWktCrdRegexp, "\\1" );
+        }
+
+        geom = QgsGeometry::fromWkt( sWkt );
+      }
+      catch ( ... )
+      {
+        geom = 0;
+      }
+
+      if ( geom && geom->wkbType() != mWkbType )
+      {
+        delete geom;
+        geom = 0;
+      }
+      mFid++;
+      if ( geom && !boundsCheck( geom ) )
+      {
+        delete geom;
+        geom = 0;
+      }
+    }
+    else if ( mXFieldIndex >= 0 && mYFieldIndex >= 0 )
+    {
+      bool xOk, yOk;
+      double x = tokens[mXFieldIndex].toDouble( &xOk );
+      double y = tokens[mYFieldIndex].toDouble( &yOk );
+      if ( xOk && yOk )
+      {
+        mFid++;
+        if ( boundsCheck( x, y ) )
+        {
+          geom = QgsGeometry::fromPoint( QgsPoint( x, y ) );
+        }
+      }
     }
 
-    if ( !( xOk && yOk ) )
+    if ( !geom && mWkbType != QGis::WKBNoGeometry )
     {
-      // Accumulate any lines that weren't ok, to report on them
-      // later, and look at the next line in the file, but only if
-      // we need to.
-      QgsDebugMsg( "Malformed line : " + line );
-      if ( mShowInvalidLines )
-        mInvalidLines << line;
-
+      mInvalidLines << line;
       continue;
     }
 
-    // Give every valid line in the file an id, even if it's not
-    // in the current extent or bounds.
-    ++mFid;             // increment to next feature ID
+    // At this point the current feature values are valid
 
-    // skip the feature if it's out of current bounds
-    if ( ! boundsCheck( x, y ) )
-      continue;
-
-    // at this point, one way or another, the current feature values
-    // are valid
     feature.setValid( true );
 
     feature.setFeatureId( mFid );
 
-    QByteArray  buffer;
-    QDataStream s( &buffer, static_cast<QIODevice::OpenMode>( QIODevice::WriteOnly ) ); // open on buffers's data
-
-    switch ( QgsApplication::endian() )
-    {
-      case QgsApplication::NDR :
-        // we're on a little-endian platform, so tell the data
-        // stream to use that
-        s.setByteOrder( QDataStream::LittleEndian );
-        s << ( quint8 )1; // 1 is for little-endian
-        break;
-      case QgsApplication::XDR :
-        // don't change byte order since QDataStream is big endian by default
-        s << ( quint8 )0; // 0 is for big-endian
-        break;
-      default :
-        QgsDebugMsg( "unknown endian" );
-        //delete [] geometry;
-        return false;
-    }
-
-    s << ( quint32 )QGis::WKBPoint;
-    s << x;
-    s << y;
-
-    unsigned char* geometry = new unsigned char[buffer.size()];
-    memcpy( geometry, buffer.data(), buffer.size() );
-
-    feature.setGeometryAndOwnership( geometry, sizeof( wkbPoint ) );
+    if ( geom )
+      feature.setGeometry( geom );
 
     for ( QgsAttributeList::const_iterator i = mAttributesToFetch.begin();
           i != mAttributesToFetch.end();
           ++i )
     {
+      int fieldIdx = *i;
+      if ( fieldIdx < 0 || fieldIdx >= attributeColumns.count() )
+        continue; // ignore non-existant fields
+
+      QString &value = tokens[attributeColumns[fieldIdx]];
       QVariant val;
-      switch ( attributeFields[*i].type() )
+      switch ( attributeFields[fieldIdx].type() )
       {
         case QVariant::Int:
-          val = QVariant( tokens[*i].toInt() );
+          if ( !value.isEmpty() )
+            val = QVariant( value );
+          else
+            val = QVariant( attributeFields[fieldIdx].type() );
           break;
         case QVariant::Double:
-          val = QVariant( tokens[*i].toDouble() );
+          if ( !value.isEmpty() )
+            val = QVariant( value.toDouble() );
+          else
+            val = QVariant( attributeFields[fieldIdx].type() );
           break;
         default:
-          val = QVariant( tokens[*i] );
+          val = QVariant( value );
           break;
       }
-      feature.addAttribute( *i, val );
+      feature.addAttribute( fieldIdx, val );
     }
 
     // We have a good line, so return
     return true;
 
-  } // ! textStream EOF
+  } // !mStream->atEnd()
 
   // End of the file. If there are any lines that couldn't be
   // loaded, display them now.
-
   if ( mShowInvalidLines && !mInvalidLines.isEmpty() )
   {
     mShowInvalidLines = false;
@@ -455,7 +579,7 @@ bool QgsDelimitedTextProvider::nextFeature( QgsFeature& feature )
     output->showMessage();
 
     // We no longer need these lines.
-    mInvalidLines.empty();
+    mInvalidLines.clear();
   }
 
   return false;
@@ -471,6 +595,7 @@ void QgsDelimitedTextProvider::select( QgsAttributeList fetchAttributes,
   mSelectionRectangle = rect;
   mAttributesToFetch = fetchAttributes;
   mFetchGeom = fetchGeometry;
+  mUseIntersect = useIntersect;
   if ( rect.isEmpty() )
   {
     mSelectionRectangle = mExtent;
@@ -496,7 +621,7 @@ QgsRectangle QgsDelimitedTextProvider::extent()
  */
 QGis::WkbType QgsDelimitedTextProvider::geometryType() const
 {
-  return QGis::WKBPoint;
+  return mWkbType;
 }
 
 /**
@@ -525,10 +650,11 @@ void QgsDelimitedTextProvider::rewind()
 {
   // Reset feature id to 0
   mFid = 0;
-  // Skip ahead one line since first record is always assumed to be
-  // the header record
+  // Skip to first data record
   mStream->seek( 0 );
-  mStream->readLine();
+  int n = mFirstDataLine - 1;
+  while ( n-- > 0 )
+    readLine( mStream );
 }
 
 bool QgsDelimitedTextProvider::isValid()
@@ -541,12 +667,23 @@ bool QgsDelimitedTextProvider::isValid()
  */
 bool QgsDelimitedTextProvider::boundsCheck( double x, double y )
 {
-  // no selection rectangle => always in the bounds
-  if ( mSelectionRectangle.isEmpty() )
+  // no selection rectangle or geometry => always in the bounds
+  if ( mSelectionRectangle.isEmpty() || !mFetchGeom )
     return true;
 
-  return ( x <= mSelectionRectangle.xMaximum() ) && ( x >= mSelectionRectangle.xMinimum() ) &&
-         ( y <= mSelectionRectangle.yMaximum() ) && ( y >= mSelectionRectangle.yMinimum() );
+  return mSelectionRectangle.contains( QgsPoint( x, y ) );
+}
+/**
+ * Check to see if the geometry is within the selection rectangle
+ */
+bool QgsDelimitedTextProvider::boundsCheck( QgsGeometry *geom )
+{
+  // no selection rectangle or geometry => always in the bounds
+  if ( mSelectionRectangle.isEmpty() || !mFetchGeom )
+    return true;
+
+  return geom->boundingBox().intersects( mSelectionRectangle ) &&
+         ( !mUseIntersect || geom->intersects( mSelectionRectangle ) );
 }
 
 int QgsDelimitedTextProvider::capabilities() const
@@ -557,13 +694,10 @@ int QgsDelimitedTextProvider::capabilities() const
 
 QgsCoordinateReferenceSystem QgsDelimitedTextProvider::crs()
 {
-  // TODO: make provider projection-aware
-  return QgsCoordinateReferenceSystem(); // return default CRS
+  return mCrs;
 }
 
 
-
-
 QString  QgsDelimitedTextProvider::name() const
 {
   return TEXT_PROVIDER_KEY;
