@@ -19,12 +19,17 @@
 
 #include "qgslogger.h"
 #include "qgsapplication.h"
+#include "qgscoordinatereferencesystem.h"
+#include "qgsrectangle.h"
+#include "qgsconfig.h"
 
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QProcess>
 #include <QSettings>
 #include <QTextStream>
+#include <QTemporaryFile>
+#include <QHash>
 
 #include <QTextCodec>
 
@@ -33,10 +38,18 @@ extern "C"
 #ifndef _MSC_VER
 #include <unistd.h>
 #endif
+#include <grass/gprojects.h>
 #include <grass/Vect.h>
 #include <grass/version.h>
 }
 
+#if !defined(GRASS_VERSION_MAJOR) || \
+    !defined(GRASS_VERSION_MINOR) || \
+    GRASS_VERSION_MAJOR<6 || \
+    (GRASS_VERSION_MAJOR == 6 && GRASS_VERSION_MINOR <= 2)
+#define G__setenv(name,value) G__setenv( ( char * ) (name), (char *) (value) )
+#endif
+
 #if defined(WIN32)
 #include <windows.h>
 QString GRASS_EXPORT QgsGrass::shortPath( const QString &path )
@@ -52,6 +65,8 @@ QString GRASS_EXPORT QgsGrass::shortPath( const QString &path )
   }
 
   QString res = QString::fromUtf8( buf );
+  // GRASS wxpyton GUI fails on paths with backslash, Windows do support forward slashesin paths
+  res.replace( "\\", "/" );
   return res;
 }
 #endif
@@ -90,6 +105,10 @@ void GRASS_EXPORT QgsGrass::init( void )
   // Init GRASS libraries (required)
   G_no_gisinit();  // Doesn't check write permissions for mapset compare to G_gisinit("libgrass++");
 
+  // I think that mask should not be used in QGIS as it can only confuses people,
+  // anyway, I don't think anybody is using MASK
+  G_suppress_masking();
+
   // Set program name
   G_set_program_name( "QGIS" );
 
@@ -179,11 +198,7 @@ void GRASS_EXPORT QgsGrass::init( void )
   }
 
   QgsDebugMsg( QString( "Valid GRASS gisBase is: %1" ).arg( gisBase ) );
-  QString gisBaseEnv = "GISBASE=" + gisBase;
-  /* _Correct_ putenv() implementation is not making copy! */
-  char *gisBaseEnvChar = new char[gisBaseEnv.toUtf8().length()+1];
-  strcpy( gisBaseEnvChar, gisBaseEnv.toUtf8().constData() );
-  putenv( gisBaseEnvChar );
+  putEnv( "GISBASE", gisBase );
 
   // Add path to GRASS modules
 #ifdef WIN32
@@ -191,7 +206,7 @@ void GRASS_EXPORT QgsGrass::init( void )
 #else
   QString sep = ":";
 #endif
-  QString path = "PATH=" + gisBase + "/bin";
+  QString path = gisBase + "/bin";
   path.append( sep + gisBase + "/scripts" );
   path.append( sep +  QgsApplication::pkgDataPath() + "/grass/scripts/" );
 
@@ -214,18 +229,14 @@ void GRASS_EXPORT QgsGrass::init( void )
   path.append( sep + p );
 
   QgsDebugMsg( QString( "set PATH: %1" ).arg( path ) );
-  char *pathEnvChar = new char[path.toUtf8().length()+1];
-  strcpy( pathEnvChar, path.toUtf8().constData() );
-  putenv( pathEnvChar );
+  putEnv( "PATH", path );
 
   // Set PYTHONPATH
-  QString pythonpath = "PYTHONPATH=" + gisBase + "/etc/python";
-  QString pp = getenv( "PATH" );
+  QString pythonpath = gisBase + "/etc/python";
+  QString pp = getenv( "PYTHONPATH" );
   pythonpath.append( sep + pp );
   QgsDebugMsg( QString( "set PYTHONPATH: %1" ).arg( pythonpath ) );
-  char *pythonpathEnvChar = new char[pythonpath.toUtf8().length()+1];
-  strcpy( pythonpathEnvChar, pythonpath.toUtf8().constData() );
-  putenv( pythonpathEnvChar );
+  putEnv( "PYTHONPATH", pythonpath );
 
   // Set GRASS_PAGER if not set, it is necessary for some
   // modules printing to terminal, e.g. g.list
@@ -264,10 +275,7 @@ void GRASS_EXPORT QgsGrass::init( void )
 
     if ( pager.length() > 0 )
     {
-      pager.prepend( "GRASS_PAGER=" );
-      char *pagerEnvChar = new char[pager.length()+1];
-      strcpy( pagerEnvChar, pager.toUtf8().constData() );
-      putenv( pagerEnvChar );
+      putEnv( "GRASS_PAGER", pager );
     }
   }
 
@@ -283,23 +291,25 @@ bool QgsGrass::isValidGrassBaseDir( QString const gisBase )
   // GRASS currently doesn't handle paths with blanks
   if ( gisBase.isEmpty() || gisBase.contains( " " ) )
   {
-    return FALSE;
+    return false;
   }
 
   /* TODO: G_is_gisbase() was added to GRASS 6.1 06-05-24,
            enable its use after some period (others do update) */
-  /*
+#if 0
   if ( QgsGrass::versionMajor() > 6 || QgsGrass::versionMinor() > 0 )
   {
-  if ( G_is_gisbase( gisBase.toUtf8().constData() ) ) return TRUE;
+    if ( G_is_gisbase( gisBase.toUtf8().constData() ) ) return true;
   }
   else
   {
-  */
-  QFileInfo gbi( gisBase + "/etc/element_list" );
-  if ( gbi.exists() ) return TRUE;
-  //}
-  return FALSE;
+#endif
+    QFileInfo gbi( gisBase + "/etc/element_list" );
+    if ( gbi.exists() ) return true;
+#if 0
+  }
+#endif
+  return false;
 }
 
 bool QgsGrass::activeMode( void )
@@ -332,13 +342,15 @@ void QgsGrass::setLocation( QString gisdbase, QString location )
   init();
 
   // Set principal GRASS variables (in memory)
-// #if defined(WIN32)
-//  G__setenv(( char * ) "GISDBASE", shortPath( gisdbase ).toLocal8Bit().data() );
-//#else
-  G__setenv(( char * ) "GISDBASE", gisdbase.toUtf8().constData() );
-//#endif
-  G__setenv(( char * ) "LOCATION_NAME", location.toUtf8().constData() );
-  G__setenv(( char * ) "MAPSET", ( char * ) "PERMANENT" ); // PERMANENT must always exist
+#if defined(WIN32)
+  G__setenv( "GISDBASE", shortPath( gisdbase ).toLocal8Bit().data() );
+#else
+  // This does not work for GISBASE with non ascii chars on Windows XP,
+  // gives error 'LOCATION ... not available':
+  G__setenv( "GISDBASE", gisdbase.toUtf8().constData() );
+#endif
+  G__setenv( "LOCATION_NAME", location.toUtf8().constData() );
+  G__setenv( "MAPSET", "PERMANENT" ); // PERMANENT must always exist
 
   // Add all available mapsets to search path
   char **ms = G_available_mapsets();
@@ -351,13 +363,13 @@ void QgsGrass::setMapset( QString gisdbase, QString location, QString mapset )
   init();
 
   // Set principal GRASS variables (in memory)
-// #if defined(WIN32)
-//  G__setenv(( char * ) "GISDBASE", shortPath( gisdbase ).toUtf8().data() );
-// #else
-  G__setenv(( char * ) "GISDBASE", gisdbase.toUtf8().data() );
-// #endif
-  G__setenv(( char * ) "LOCATION_NAME", location.toUtf8().data() );
-  G__setenv(( char * ) "MAPSET", mapset.toUtf8().data() );
+#if defined(WIN32)
+  G__setenv( "GISDBASE", shortPath( gisdbase ).toUtf8().data() );
+#else
+  G__setenv( "GISDBASE", gisdbase.toUtf8().data() );
+#endif
+  G__setenv( "LOCATION_NAME", location.toUtf8().data() );
+  G__setenv( "MAPSET", mapset.toUtf8().data() );
 
   // Add all available mapsets to search path
   char **ms = G_available_mapsets();
@@ -387,14 +399,18 @@ int QgsGrass::error_routine( char *msg, int fatal )
 
 int QgsGrass::error_routine( const char *msg, int fatal )
 {
+  // Unfortunately the exceptions thrown here can only be caught if GRASS libraries are compiled
+  // with -fexception option on Linux (works on Windows)
+  // GRASS developers are reluctant to add -fexception by default
+  // https://trac.osgeo.org/grass/ticket/869
   QgsDebugMsg( QString( "error_routine (fatal = %1): %2" ).arg( fatal ).arg( msg ) );
 
   error_message = msg;
 
   if ( fatal )
   {
-    // we have to do a long jump here, otherwise GRASS >= 6.3 will kill our process
-    throw QgsGrass::Exception( msg );
+    // we have to throw an exception here, otherwise GRASS >= 6.3 will kill our process
+    throw QgsGrass::Exception( QString::fromUtf8( msg ) );
   }
   else
     lastError = WARNING;
@@ -427,39 +443,28 @@ QString GRASS_EXPORT QgsGrass::openMapset( QString gisdbase, QString location, Q
 
   // Check if the mapset is in use
   QString gisBase = getenv( "GISBASE" );
-  if ( gisBase.isNull() ) return QObject::tr( "GISBASE is not set." );
+  if ( gisBase.isEmpty() ) return QObject::tr( "GISBASE is not set." );
 
   QFileInfo fi( mapsetPath + "/WIND" );
   if ( !fi.exists() )
   {
     return QObject::tr( "%1 is not a GRASS mapset." ).arg( mapsetPath );
   }
-
   QString lock = mapsetPath + "/.gislock";
-  QFile lockFile( lock );
-#if WIN32
-  // lock on Windows doesn't support locking (see #808)
-  if ( lockFile.exists() )
-    return QObject::tr( "Mapset is already in use." );
 
-  lockFile.open( QIODevice::WriteOnly );
 #ifndef _MSC_VER
   int pid = getpid();
 #else
   int pid = GetCurrentProcessId();
 #endif
-  lockFile.write( QString( "%1" ).arg( pid ).toUtf8() );
-  lockFile.close();
-#else
+
+  QgsDebugMsg( QString( "pid = %1" ).arg( pid ) );
+
+#ifndef Q_OS_WIN
+  QFile lockFile( lock );
   QProcess *process = new QProcess();
   QString lockProgram( gisBase + "/etc/lock" );
 
-  // TODO: getpid() probably is not portable
-#ifndef _MSC_VER
-  int pid = getpid();
-#else
-  int pid = GetCurrentProcessId();
-#endif
   QgsDebugMsg( QString( "pid = %1" ).arg( pid ) );
 
   process->start( lockProgram, QStringList() << lock << QString::number( pid ) );
@@ -490,13 +495,17 @@ QString GRASS_EXPORT QgsGrass::openMapset( QString gisdbase, QString location, Q
     QFileInfo dirInfo( mTmp );
     if ( !dirInfo.isWritable() )
     {
+#ifndef Q_OS_WIN
       lockFile.remove();
+#endif
       return QObject::tr( "Temporary directory %1 exists but is not writable" ).arg( mTmp );
     }
   }
   else if ( !dir.mkdir( mTmp ) )
   {
+#ifndef Q_OS_WIN
     lockFile.remove();
+#endif
     return QObject::tr( "Cannot create temporary directory %1" ).arg( mTmp );
   }
 
@@ -510,13 +519,16 @@ QString GRASS_EXPORT QgsGrass::openMapset( QString gisdbase, QString location, Q
   QFile out( mGisrc );
   if ( !out.open( QIODevice::WriteOnly ) )
   {
+#ifndef Q_OS_WIN
     lockFile.remove();
+#endif
     return QObject::tr( "Cannot create %1" ).arg( mGisrc );
   }
   QTextStream stream( &out );
 
   QFile in( globalGisrc );
   QString line;
+  bool guiSet = false;
   char buf[1000];
   if ( in.open( QIODevice::ReadOnly ) )
   {
@@ -529,6 +541,7 @@ QString GRASS_EXPORT QgsGrass::openMapset( QString gisdbase, QString location, Q
       {
         continue;
       }
+      if ( line.contains( "GRASS_GUI:" ) ) guiSet = true;
       stream << line;
     }
     in.close();
@@ -539,38 +552,41 @@ QString GRASS_EXPORT QgsGrass::openMapset( QString gisdbase, QString location, Q
   stream << line;
   line = "MAPSET: " + mapset + "\n";
   stream << line;
+  if ( !guiSet )
+  {
+    stream << "GRASS_GUI: wxpython\n";
+  }
 
   out.close();
 
   // Set GISRC environment variable
 
   /* _Correct_ putenv() implementation is not making copy! */
-  QString gisrcEnv = "GISRC=" + mGisrc;
-  char *gisrcEnvChar = new char[gisrcEnv.toUtf8().length()+1];
-  strcpy( gisrcEnvChar, gisrcEnv.toLocal8Bit().constData() );
-  putenv( gisrcEnvChar );
+  putEnv( "GISRC", mGisrc );
 
   // Reinitialize GRASS
-  G__setenv(( char * ) "GISRC", gisrcEnv.toUtf8().data() );
+  G__setenv( "GISRC", mGisrc.toUtf8().data() );
 #if defined(WIN32)
-  G__setenv(( char * ) "GISDBASE", shortPath( gisdbase ).toLocal8Bit().data() );
+  G__setenv( "GISDBASE", shortPath( gisdbase ).toLocal8Bit().data() );
 #else
-  G__setenv(( char * ) "GISDBASE", gisdbase.toUtf8().data() );
+  G__setenv( "GISDBASE", gisdbase.toUtf8().data() );
 #endif
-  G__setenv(( char * ) "LOCATION_NAME", location.toLocal8Bit().data() );
-  G__setenv(( char * ) "MAPSET", mapset.toLocal8Bit().data() );
+  G__setenv( "LOCATION_NAME", location.toLocal8Bit().data() );
+  G__setenv( "MAPSET", mapset.toLocal8Bit().data() );
   defaultGisdbase = gisdbase;
   defaultLocation = location;
   defaultMapset = mapset;
 
   active = true;
 
+#ifndef Q_OS_WIN
   // Close old mapset
   if ( mMapsetLock.length() > 0 )
   {
     QFile file( mMapsetLock );
     file.remove();
   }
+#endif
 
   mMapsetLock = lock;
 
@@ -593,10 +609,16 @@ QString QgsGrass::closeMapset( )
     putenv(( char * ) "GISRC" );
 
     // Reinitialize GRASS
-    G__setenv(( char * ) "GISRC", ( char * ) "" );
-    G__setenv(( char * ) "GISDBASE", ( char * ) "" );
-    G__setenv(( char * ) "LOCATION_NAME", ( char * ) "" );
-    G__setenv(( char * ) "MAPSET", ( char * ) "" );
+    G__setenv( "GISRC", ( char * ) "" );
+
+    // Temporarily commented because of
+    //   http://trac.osgeo.org/qgis/ticket/1900
+    //   http://trac.osgeo.org/gdal/ticket/3313
+    // it can be uncommented once GDAL with patch gets deployed (probably GDAL 1.8)
+    //G__setenv( "GISDBASE", ( char * ) "" );
+    //G__setenv( "LOCATION_NAME", ( char * ) "" );
+    //G__setenv( "MAPSET", ( char * ) "" );
+
     defaultGisdbase = "";
     defaultLocation = "";
     defaultMapset = "";
@@ -1015,6 +1037,297 @@ bool GRASS_EXPORT QgsGrass::mapRegion( int type, QString gisbase,
   return true;
 }
 
+QProcess GRASS_EXPORT *QgsGrass::startModule( QString gisdbase, QString location,
+    QString module, QStringList arguments, QTemporaryFile &gisrcFile )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+  QProcess *process = new QProcess();
+
+#ifdef WIN32
+  module += ".exe";
+#endif
+
+  // We have to set GISRC file, uff
+  if ( !gisrcFile.open() )
+  {
+    throw QgsGrass::Exception( QObject::tr( "Cannot open GISRC file" ) );
+  }
+
+  QTextStream out( &gisrcFile );
+  out << "GISDBASE: " << gisdbase << "\n";
+  out << "LOCATION_NAME: " << location << "\n";
+  out << "MAPSET: PERMANENT\n";
+  out.flush();
+  QgsDebugMsg( gisrcFile.fileName() );
+  gisrcFile.close();
+
+  QStringList environment = QProcess::systemEnvironment();
+  environment.append( "GISRC=" + gisrcFile.fileName() );
+
+  process->setEnvironment( environment );
+
+  QgsDebugMsg( module + " " + arguments.join( " " ) );
+  process->start( module, arguments );
+  if ( !process->waitForStarted() )
+  {
+    throw QgsGrass::Exception( QObject::tr( "Cannot start module" ) + "\n"
+                               + QObject::tr( "command: %1 %2" )
+                               .arg( module ).arg( arguments.join( " " ) ) );
+  }
+  return process;
+}
+
+QByteArray GRASS_EXPORT QgsGrass::runModule( QString gisdbase, QString location,
+    QString module, QStringList arguments )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+
+  QTemporaryFile gisrcFile;
+  QProcess *process = QgsGrass::startModule( gisdbase, location, module, arguments, gisrcFile );
+
+  if ( !process->waitForFinished()
+       || ( process->exitCode() != 0 && process->exitCode() != 255 ) )
+  {
+    QgsDebugMsg( "process->exitCode() = " + QString::number( process->exitCode() ) );
+
+    throw QgsGrass::Exception( QObject::tr( "Cannot run module" ) + "\n"
+                               + QObject::tr( "command: %1 %2<br>%3<br>%4" )
+                               .arg( module ).arg( arguments.join( " " ) )
+                               .arg( process->readAllStandardOutput().constData() )
+                               .arg( process->readAllStandardError().constData() ) );
+  }
+  QByteArray data = process->readAllStandardOutput();
+  delete process;
+  return data;
+}
+
+QString GRASS_EXPORT QgsGrass::getInfo( QString info, QString gisdbase, QString location,
+                                        QString mapset, QString map, MapType type, double x, double y )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+
+  QStringList arguments;
+
+  QString cmd = QgsApplication::prefixPath() + "/" QGIS_LIBEXEC_SUBDIR "/grass/modules/qgis.g.info";
+
+  arguments.append( "info=" + info );
+  if ( !map.isEmpty() )
+  {
+    QString opt;
+    switch ( type )
+    {
+      case Raster:
+        opt = "rast";
+        break;
+      case Vector:
+        opt = "vect";
+        break;
+      default:
+        QgsDebugMsg( "unexpected type:" + type );
+        return "";
+    }
+    arguments.append( opt + "=" +  map + "@" + mapset );
+  }
+  if ( info == "query" )
+  {
+    arguments.append( QString( "coor=%1,%2" ).arg( x ).arg( y ) );
+  }
+
+  QByteArray data =  QgsGrass::runModule( gisdbase, location, cmd, arguments );
+  QgsDebugMsg( data );
+  return QString( data );
+}
+
+QgsCoordinateReferenceSystem GRASS_EXPORT QgsGrass::crs( QString gisdbase, QString location )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+  QgsCoordinateReferenceSystem crs = QgsCoordinateReferenceSystem();
+  try
+  {
+    QString wkt = QgsGrass::getInfo( "proj", gisdbase, location );
+    QgsDebugMsg( "wkt: " + wkt );
+    crs.createFromWkt( wkt );
+    QgsDebugMsg( "crs.toWkt: " + crs.toWkt() );
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QMessageBox::warning( 0, QObject::tr( "Warning" ),
+                          QObject::tr( "Cannot get projection " ) + "\n" + e.what() );
+  }
+
+  return crs;
+}
+
+QgsCoordinateReferenceSystem GRASS_EXPORT QgsGrass::crsDirect( QString gisdbase, QString location )
+{
+  QString Wkt;
+
+  struct Cell_head cellhd;
+
+  QgsGrass::resetError();
+  QgsGrass::setLocation( gisdbase, location );
+
+  const char *oldlocale = setlocale( LC_NUMERIC, NULL );
+  setlocale( LC_NUMERIC, "C" );
+
+  try
+  {
+    G_get_default_window( &cellhd );
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    Q_UNUSED( e );
+    setlocale( LC_NUMERIC, oldlocale );
+    QgsDebugMsg( QString( "Cannot get default window: %1" ).arg( e.what() ) );
+    return QgsCoordinateReferenceSystem();
+  }
+
+  if ( cellhd.proj != PROJECTION_XY )
+  {
+    struct Key_Value *projinfo = G_get_projinfo();
+    struct Key_Value *projunits = G_get_projunits();
+    char *wkt = GPJ_grass_to_wkt( projinfo, projunits,  0, 0 );
+    Wkt = QString( wkt );
+    G_free( wkt );
+  }
+
+  setlocale( LC_NUMERIC, oldlocale );
+
+  QgsCoordinateReferenceSystem srs;
+  srs.createFromWkt( Wkt );
+
+  return srs;
+}
+
+QgsRectangle GRASS_EXPORT QgsGrass::extent( QString gisdbase, QString location, QString mapset, QString map, MapType type )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+
+  try
+  {
+    QString str = QgsGrass::getInfo( "window", gisdbase, location, mapset, map, type );
+    QStringList list = str.split( "," );
+    if ( list.size() != 4 )
+    {
+      throw QgsGrass::Exception( "Cannot parse GRASS map extent: " + str );
+    }
+    return QgsRectangle( list[0].toDouble(), list[1].toDouble(), list[2].toDouble(), list[3].toDouble() ) ;
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QMessageBox::warning( 0, QObject::tr( "Warning" ),
+                          QObject::tr( "Cannot get raster extent" ) + "\n" + e.what() );
+  }
+  return QgsRectangle( 0, 0, 0, 0 );
+}
+
+void GRASS_EXPORT QgsGrass::size( QString gisdbase, QString location, QString mapset, QString map, int *cols, int *rows )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+
+  *cols = 0;
+  *rows = 0;
+  try
+  {
+    QString str = QgsGrass::getInfo( "size", gisdbase, location, mapset, map, QgsGrass::Raster );
+    QStringList list = str.split( "," );
+    if ( list.size() != 2 )
+    {
+      throw QgsGrass::Exception( "Cannot parse GRASS map size: " + str );
+    }
+    *cols = list[0].toInt();
+    *rows = list[1].toInt();
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QMessageBox::warning( 0, QObject::tr( "Warning" ),
+                          QObject::tr( "Cannot get raster extent" ) + "\n" + e.what() );
+  }
+
+  QgsDebugMsg( QString( "raster size = %1 %2" ).arg( *cols ).arg( *rows ) );
+}
+
+QHash<QString, QString> GRASS_EXPORT QgsGrass::info( QString gisdbase, QString location, QString mapset, QString map, MapType type )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+  QHash<QString, QString> inf;
+
+  try
+  {
+    QString str = QgsGrass::getInfo( "info", gisdbase, location, mapset, map, type );
+    QgsDebugMsg( str );
+    QStringList list = str.split( "\n" );
+    for ( int i = 0; i < list.size(); i++ )
+    {
+      QStringList keyVal = list[i].split( ':' );
+      if ( list[i].isEmpty() ) { continue; }
+      if ( keyVal.size() != 2 )
+      {
+        throw QgsGrass::Exception( "Cannot parse GRASS map info key value : " + list[i] + " (" + str + " ) " );
+      }
+      inf[keyVal[0]] = keyVal[1];
+    }
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QMessageBox::warning( 0, QObject::tr( "Warning" ),
+                          QObject::tr( "Cannot get map info" ) + "\n" + e.what() );
+  }
+  return inf;
+}
+
+QList<QgsGrass::Color> GRASS_EXPORT QgsGrass::colors( QString gisdbase, QString location, QString mapset, QString map )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+  QList<QgsGrass::Color> ct;
+
+  try
+  {
+    QString str = QgsGrass::getInfo( "colors", gisdbase, location, mapset, map, QgsGrass::Raster );
+    QgsDebugMsg( str );
+    QStringList list = str.split( "\n" );
+    for ( int i = 0; i < list.size(); i++ )
+    {
+      QgsGrass::Color c;
+      if ( list[i].isEmpty() ) { continue; }
+      if ( sscanf( list[i].toUtf8().data(), "%lf %lf %d %d %d %d %d %d", &( c.value1 ), &( c.value2 ), &( c.red1 ), &( c.green1 ), &( c.blue1 ), &( c.red2 ), &( c.green2 ), &( c.blue2 ) ) != 8 )
+      {
+        throw QgsGrass::Exception( "Cannot parse GRASS colors" + list[i] + " (" + str + " ) " );
+      }
+      ct.append( c );
+    }
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QMessageBox::warning( 0, QObject::tr( "Warning" ),
+                          QObject::tr( "Cannot get colors" ) + "\n" + e.what() );
+  }
+  return ct;
+}
+
+QMap<QString, QString> GRASS_EXPORT QgsGrass::query( QString gisdbase, QString location, QString mapset, QString map, MapType type, double x, double y )
+{
+  QgsDebugMsg( QString( "gisdbase = %1 location = %2" ).arg( gisdbase ).arg( location ) );
+
+  QMap<QString, QString> result;
+  // TODO: multiple values (more rows)
+  try
+  {
+    QString str = QgsGrass::getInfo( "query", gisdbase, location, mapset, map, type, x, y );
+    QStringList list = str.trimmed().split( ":" );
+    if ( list.size() == 2 )
+    {
+      result[list[0]] = list[1];
+    }
+  }
+  catch ( QgsGrass::Exception &e )
+  {
+    QMessageBox::warning( 0, QObject::tr( "Warning" ),
+                          QObject::tr( "Cannot query raster " ) + "\n" + e.what() );
+  }
+  return result;
+}
+
 // GRASS version constants have been changed on 26.4.2007
 // http://freegis.org/cgi-bin/viewcvs.cgi/grass6/include/version.h.in.diff?r1=1.4&r2=1.5
 // The following lines workaround this change
@@ -1074,3 +1387,25 @@ QString GRASS_EXPORT QgsGrass::lockFilePath()
 {
   return mMapsetLock;
 }
+
+QString GRASS_EXPORT QgsGrass::gisrcFilePath()
+{
+  if ( mGisrc.isEmpty() )
+  {
+    // Started from GRASS shell
+    if ( getenv( "GISRC" ) )
+    {
+      return QString( getenv( "GISRC" ) );
+    }
+  }
+  return mGisrc;
+}
+
+void GRASS_EXPORT QgsGrass::putEnv( QString name, QString value )
+{
+  QString env = name + "=" + value;
+  /* _Correct_ putenv() implementation is not making copy! */
+  char *envChar = new char[env.toUtf8().length()+1];
+  strcpy( envChar, env.toUtf8().constData() );
+  putenv( envChar );
+}
