@@ -24,7 +24,14 @@ extern "C"
 }
 
 #include <stdexcept>
+#include "qgsexception.h"
+#include <QProcess>
 #include <QString>
+#include <QMap>
+#include <QHash>
+#include <QTemporaryFile>
+class QgsCoordinateReferenceSystem;
+class QgsRectangle;
 
 /*!
    Methods for C library initialization and error handling.
@@ -32,9 +39,25 @@ extern "C"
 class QgsGrass
 {
   public:
+    // This does not work (gcc/Linux), such exception cannot be caught
+    // so I have enabled the old version, if you are able to fix it, please
+    // check first if it realy works, i.e. can be caught!
+    /*
+    struct Exception : public QgsException
+    {
+      Exception( const QString &msg ) : QgsException( msg ) {}
+    };
+    */
     struct Exception : public std::runtime_error
     {
-      Exception( const std::string &msg ) : std::runtime_error( msg ) {}
+      //Exception( const std::string &msg ) : std::runtime_error( msg ) {}
+      Exception( const QString &msg ) : std::runtime_error( msg.toUtf8().constData() ) {}
+    };
+
+    struct Color
+    {
+      double value1, value2;
+      int red1, red2, green1, green2, blue1, blue2;
     };
 
     //! Get info about the mode
@@ -74,7 +97,7 @@ class QgsGrass
               };
 
     //! Map type
-    enum MapType { Raster, Vector, Region };
+    enum MapType { None, Raster, Vector, Region };
 
     //! Reset error code (to OK). Call this before a piece of code where an error is expected
     static GRASS_EXPORT void resetError( void );  // reset error status
@@ -157,12 +180,54 @@ class QgsGrass
     // ! Get the lock file
     static GRASS_EXPORT QString lockFilePath();
 
+    // ! Get current gisrc path
+    static GRASS_EXPORT QString gisrcFilePath();
+
+    // ! Start a GRASS module in any gisdbase/location
+    static GRASS_EXPORT QProcess *startModule( QString gisdbase, QString location, QString module, QStringList arguments, QTemporaryFile &gisrcFile );
+
+    // ! Run a GRASS module in any gisdbase/location
+    static GRASS_EXPORT QByteArray runModule( QString gisdbase, QString location, QString module, QStringList arguments );
+
+    // ! Get info string from qgis.g.info module
+    static GRASS_EXPORT QString getInfo( QString info, QString gisdbase,
+                                         QString location, QString mapset = "", QString map = "", MapType type = None, double x = 0.0, double y = 0.0 );
+
+    // ! Get location projection
+    static GRASS_EXPORT QgsCoordinateReferenceSystem crs( QString gisdbase, QString location );
+
+    // ! Get location projection calling directly GRASS library
+    static GRASS_EXPORT QgsCoordinateReferenceSystem crsDirect( QString gisdbase, QString location );
+
+    // ! Get map extent
+    static GRASS_EXPORT QgsRectangle extent( QString gisdbase, QString location,
+        QString mapset, QString map, MapType type = None );
+
+    // ! Get raster map size
+    static GRASS_EXPORT void size( QString gisdbase, QString location,
+                                   QString mapset, QString map, int *cols, int *rows );
+
+    // ! Get raster info
+    static GRASS_EXPORT QHash<QString, QString> info( QString gisdbase, QString location,
+        QString mapset, QString map, MapType type );
+
+    // ! List of Color
+    static GRASS_EXPORT QList<QgsGrass::Color> colors( QString gisdbase, QString location,
+        QString mapset, QString map );
+
+    // ! Get map value / feautre info
+    static GRASS_EXPORT QMap<QString, QString> query( QString gisdbase, QString location,
+        QString mapset, QString map, MapType type, double x, double y );
+
     //! Library version
     static GRASS_EXPORT int versionMajor();
     static GRASS_EXPORT int versionMinor();
     static GRASS_EXPORT int versionRelease();
     static GRASS_EXPORT QString versionString();
 
+    // set environment variable
+    static GRASS_EXPORT void putEnv( QString name, QString value );
+
 #if defined(WIN32)
     static GRASS_EXPORT QString shortPath( const QString &path );
 #endif
