@@ -21,6 +21,7 @@ class QgsField;
 
 #include <QDateTime>
 #include "qgsvectordataprovider.h"
+#include <vector>
 
 /* Update.
  * Vectors are updated (reloaded) if:
@@ -110,6 +111,8 @@ struct GMAP
 */
 class GRASS_EXPORT QgsGrassProvider : public QgsVectorDataProvider
 {
+    Q_OBJECT
+
   public:
 
     QgsGrassProvider( QString uri = QString() );
@@ -280,6 +283,12 @@ class GRASS_EXPORT QgsGrassProvider : public QgsVectorDataProvider
      */
     bool lineAreas( int line, int *left, int *right );
 
+    /** Get isle area
+     *   @param isle number
+     *   @return area number
+     */
+    int isleArea( int isle );
+
     /** Get centroid area
      *   @param centroid line number
      *   @return area number (negative for island)
