@@ -26,6 +26,8 @@ email                : sherman at mrcc.com
 #include <cpl_error.h>
 #include <cpl_conv.h>
 
+#include <limits>
+
 #include <QtDebug>
 #include <QFile>
 #include <QDir>
@@ -39,8 +41,8 @@ email                : sherman at mrcc.com
 #include "qgsfeature.h"
 #include "qgsfield.h"
 #include "qgsgeometry.h"
-#include "qgslogger.h"
 #include "qgscoordinatereferencesystem.h"
+#include "qgsvectorfilewriter.h"
 
 static const QString TEXT_PROVIDER_KEY = "ogr";
 static const QString TEXT_PROVIDER_DESCRIPTION =
@@ -51,6 +53,35 @@ static const QString TEXT_PROVIDER_DESCRIPTION =
   + GDALVersionInfo( "RELEASE_NAME" )
   + ")";
 
+#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1800
+#define TO8(x)   (x).toUtf8().constData()
+#define TO8F(x)  (x).toUtf8().constData()
+#define FROM8(x) QString::fromUtf8(x)
+#else
+#define TO8(x)   (x).toLocal8Bit().constData()
+#define TO8F(x)  QFile::encodeName( x ).constData()
+#define FROM8(x) QString::fromLocal8Bit(x)
+#endif
+
+class QgsCPLErrorHandler
+{
+    static void CPL_STDCALL showError( CPLErr errClass, int errNo, const char *msg )
+    {
+      QgsLogger::warning( QString( "OGR[%1] error %2: %3" ).arg( errClass ).arg( errNo ).arg( msg ) );
+    }
+
+  public:
+    QgsCPLErrorHandler()
+    {
+      CPLPushErrorHandler( showError );
+    }
+
+    ~QgsCPLErrorHandler()
+    {
+      CPLPopErrorHandler();
+    }
+};
+
 QgsOgrProvider::QgsOgrProvider( QString const & uri )
     : QgsVectorDataProvider( uri ),
     ogrDataSource( 0 ),
@@ -60,6 +91,8 @@ QgsOgrProvider::QgsOgrProvider( QString const & uri )
     ogrDriver( 0 ),
     featuresCounted( -1 )
 {
+  QgsCPLErrorHandler handler;
+
   QgsApplication::registerOgrDrivers();
 
   // set the selection rectangle pointer to 0
@@ -122,14 +155,12 @@ QgsOgrProvider::QgsOgrProvider( QString const & uri )
   QgsDebugMsg( "mLayerName: " + mLayerName );
   QgsDebugMsg( "mSubsetString: " + mSubsetString );
   CPLSetConfigOption( "OGR_ORGANIZE_POLYGONS", "ONLY_CCW" );  // "SKIP" returns MULTIPOLYGONs for multiringed POLYGONs
-  CPLPushErrorHandler( CPLQuietErrorHandler );
-  ogrDataSource = OGROpen( QFile::encodeName( mFilePath ).constData(), TRUE, &ogrDriver );
-  CPLPopErrorHandler();
 
+  ogrDataSource = OGROpen( TO8F( mFilePath ), true, &ogrDriver );
   if ( ogrDataSource == NULL )
   {
     // try to open read-only
-    ogrDataSource = OGROpen( QFile::encodeName( mFilePath ).constData(), FALSE, &ogrDriver );
+    ogrDataSource = OGROpen( TO8F( mFilePath ), false, &ogrDriver );
 
     //TODO Need to set a flag or something to indicate that the layer
     //TODO is in read-only mode, otherwise edit ops will fail
@@ -153,13 +184,18 @@ QgsOgrProvider::QgsOgrProvider( QString const & uri )
     }
     else
     {
-      ogrOrigLayer = OGR_DS_GetLayerByName( ogrDataSource, mLayerName.toLocal8Bit().data() );
+      ogrOrigLayer = OGR_DS_GetLayerByName( ogrDataSource, TO8( mLayerName ) );
     }
 
-    extent_ = calloc( sizeof( OGREnvelope ), 1 );
-
     ogrLayer = ogrOrigLayer;
-    setSubsetString( mSubsetString );
+    if (ogrLayer != NULL )
+    {
+      setSubsetString( mSubsetString );
+    }
+    else
+    {
+      valid = false;
+    }
   }
   else
   {
@@ -185,8 +221,13 @@ QgsOgrProvider::~QgsOgrProvider()
 
   OGR_DS_Destroy( ogrDataSource );
   ogrDataSource = 0;
-  free( extent_ );
-  extent_ = 0;
+
+  if ( extent_ )
+  {
+    free( extent_ );
+    extent_ = 0;
+  }
+
   if ( mSelectionRectangle )
   {
     OGR_G_DestroyGeometry( mSelectionRectangle );
@@ -194,8 +235,10 @@ QgsOgrProvider::~QgsOgrProvider()
   }
 }
 
-bool QgsOgrProvider::setSubsetString( QString theSQL )
+bool QgsOgrProvider::setSubsetString( QString theSQL, bool updateFeatureCount )
 {
+  QgsCPLErrorHandler handler;
+
   if ( theSQL == mSubsetString && featuresCounted >= 0 )
     return true;
 
@@ -206,12 +249,14 @@ bool QgsOgrProvider::setSubsetString( QString theSQL )
   if ( !mSubsetString.isEmpty() )
   {
     QString sql = QString( "SELECT * FROM %1 WHERE %2" )
-                  .arg( quotedIdentifier( OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) ) ) )
+                  .arg( quotedIdentifier( FROM8( OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) ) ) ) )
                   .arg( mSubsetString );
-    ogrLayer = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).constData(), NULL, NULL );
+    QgsDebugMsg( QString( "SQL: %1" ).arg( sql ) );
+    ogrLayer = OGR_DS_ExecuteSQL( ogrDataSource, TO8( sql ), NULL, NULL );
 
     if ( !ogrLayer )
     {
+      pushError( QString( "OGR[%1] error %2: %3" ).arg( CPLGetLastErrorType() ).arg( CPLGetLastErrorNo() ).arg( CPLGetLastErrorMsg() ) );
       ogrLayer = prevLayer;
       mSubsetString = prevSubsetString;
       return false;
@@ -248,21 +293,22 @@ bool QgsOgrProvider::setSubsetString( QString theSQL )
 
   // getting the total number of features in the layer
   // TODO: This can be expensive, do we really need it!
-  featuresCounted = OGR_L_GetFeatureCount( ogrLayer, TRUE );
-
-  // get the extent_ (envelope) of the layer
-  QgsDebugMsg( "Starting get extent" );
-
-  // TODO: This can be expensive, do we really need it!
-  OGR_L_GetExtent( ogrLayer, ( OGREnvelope * ) extent_, TRUE );
-
-  QgsDebugMsg( "Finished get extent" );
+  if ( updateFeatureCount )
+  {
+    recalculateFeatureCount();
+  }
 
   // check the validity of the layer
   QgsDebugMsg( "checking validity" );
   loadFields();
   QgsDebugMsg( "Done checking validity" );
 
+  if ( extent_ )
+  {
+    free( extent_ );
+    extent_ = 0;
+  }
+
   return true;
 }
 
@@ -280,7 +326,7 @@ QStringList QgsOgrProvider::subLayers() const
   }
   for ( unsigned int i = 0; i < layerCount() ; i++ )
   {
-    QString theLayerName = QString::fromLocal8Bit( OGR_FD_GetName( OGR_L_GetLayerDefn( OGR_DS_GetLayer( ogrDataSource, i ) ) ) );
+    QString theLayerName = FROM8( OGR_FD_GetName( OGR_L_GetLayerDefn( OGR_DS_GetLayer( ogrDataSource, i ) ) ) );
     OGRwkbGeometryType layerGeomType = OGR_FD_GetGeomType( OGR_L_GetLayerDefn( OGR_DS_GetLayer( ogrDataSource, i ) ) );
 
     int theLayerFeatureCount = OGR_L_GetFeatureCount( OGR_DS_GetLayer( ogrDataSource, i ), 1 ) ;
@@ -311,7 +357,19 @@ QStringList QgsOgrProvider::subLayers() const
 
 void QgsOgrProvider::setEncoding( const QString& e )
 {
+#if defined(OLCStringsAsUTF8)
+  if ( !OGR_L_TestCapability( ogrLayer, OLCStringsAsUTF8 ) )
+  {
+    QgsVectorDataProvider::setEncoding( e );
+  }
+  else
+  {
+    QgsVectorDataProvider::setEncoding( "UTF-8" );
+  }
+#else
   QgsVectorDataProvider::setEncoding( e );
+#endif
+
   loadFields();
 }
 
@@ -376,17 +434,45 @@ QString QgsOgrProvider::storageType() const
   return ogrDriverName;
 }
 
+void QgsOgrProvider::setRelevantFields( bool fetchGeometry, const QgsAttributeList &fetchAttributes )
+{
+#if defined(GDAL_VERSION_NUM) && GDAL_VERSION_NUM >= 1800
+  if ( OGR_L_TestCapability( ogrLayer, OLCIgnoreFields ) )
+  {
+    QVector<const char*> ignoredFields;
+    OGRFeatureDefnH featDefn = OGR_L_GetLayerDefn( ogrLayer );
+    for ( int i = 0; i < mAttributeFields.size(); i++ )
+    {
+      if ( !fetchAttributes.contains( i ) )
+      {
+        // add to ignored fields
+        ignoredFields.append( OGR_Fld_GetNameRef( OGR_FD_GetFieldDefn( featDefn, i ) ) );
+      }
+    }
+
+    if ( !fetchGeometry )
+      ignoredFields.append( "OGR_GEOMETRY" );
+    ignoredFields.append( "OGR_STYLE" ); // not used by QGIS
+    ignoredFields.append( NULL );
+
+    OGR_L_SetIgnoredFields( ogrLayer, ignoredFields.data() );
+  }
+#endif
+}
 
 bool QgsOgrProvider::featureAtId( int featureId,
                                   QgsFeature& feature,
                                   bool fetchGeometry,
                                   QgsAttributeList fetchAttributes )
 {
+  setRelevantFields( fetchGeometry, fetchAttributes );
+
   OGRFeatureH fet = OGR_L_GetFeature( ogrLayer, featureId );
   if ( fet == NULL )
     return false;
 
   feature.setFeatureId( OGR_F_GetFID( fet ) );
+  feature.clearAttributeMap();
   // skip features without geometry
   if ( OGR_F_GetGeometryRef( fet ) == NULL && !mFetchFeaturesWithoutGeom )
   {
@@ -394,7 +480,6 @@ bool QgsOgrProvider::featureAtId( int featureId,
     return false;
   }
 
-
   /* fetch geometry */
   if ( fetchGeometry )
   {
@@ -439,6 +524,8 @@ bool QgsOgrProvider::nextFeature( QgsFeature& feature )
   OGRFeatureH fet;
   QgsRectangle selectionRect;
 
+  setRelevantFields( mFetchGeom || mUseIntersect, mAttributesToFetch );
+
   while (( fet = OGR_L_GetNextFeature( ogrLayer ) ) != NULL )
   {
     // skip features without geometry
@@ -451,6 +538,7 @@ bool QgsOgrProvider::nextFeature( QgsFeature& feature )
     OGRFeatureDefnH featureDefinition = OGR_F_GetDefnRef( fet );
     QString featureTypeName = featureDefinition ? QString( OGR_FD_GetName( featureDefinition ) ) : QString( "" );
     feature.setFeatureId( OGR_F_GetFID( fet ) );
+    feature.clearAttributeMap();
     feature.setTypeName( featureTypeName );
 
     /* fetch geometry */
@@ -526,6 +614,11 @@ bool QgsOgrProvider::nextFeature( QgsFeature& feature )
 
 void QgsOgrProvider::select( QgsAttributeList fetchAttributes, QgsRectangle rect, bool fetchGeometry, bool useIntersect )
 {
+  if ( geometryType() == QGis::WKBNoGeometry )
+  {
+    fetchGeometry = false;
+  }
+
   mUseIntersect = useIntersect;
   mAttributesToFetch = fetchAttributes;
   mFetchGeom = fetchGeometry;
@@ -582,7 +675,53 @@ unsigned char * QgsOgrProvider::getGeometryPointer( OGRFeatureH fet )
 
 QgsRectangle QgsOgrProvider::extent()
 {
-  OGREnvelope *ext = ( OGREnvelope * ) extent_;
+  if ( !extent_ )
+  {
+    extent_ = calloc( sizeof( OGREnvelope ), 1 );
+
+    // get the extent_ (envelope) of the layer
+    QgsDebugMsg( "Starting get extent" );
+
+    // TODO: This can be expensive, do we really need it!
+    if ( ogrLayer == ogrOrigLayer )
+    {
+      OGR_L_GetExtent( ogrLayer, ( OGREnvelope * ) extent_, true );
+    }
+    else
+    {
+      OGREnvelope *bb = static_cast<OGREnvelope*>( extent_ );
+
+      bb->MinX = std::numeric_limits<double>::max();
+      bb->MinY = std::numeric_limits<double>::max();
+      bb->MaxX = -std::numeric_limits<double>::max();
+      bb->MaxY = -std::numeric_limits<double>::max();
+
+      OGRFeatureH f;
+
+      OGR_L_ResetReading( ogrLayer );
+      while (( f = OGR_L_GetNextFeature( ogrLayer ) ) )
+      {
+        OGRGeometryH g = OGR_F_GetGeometryRef( f );
+        if ( g )
+        {
+          OGREnvelope env;
+          OGR_G_GetEnvelope( g, &env );
+
+          if ( env.MinX < bb->MinX ) bb->MinX = env.MinX;
+          if ( env.MinY < bb->MinY ) bb->MinY = env.MinY;
+          if ( env.MaxX > bb->MaxX ) bb->MaxX = env.MaxX;
+          if ( env.MaxY > bb->MaxY ) bb->MaxY = env.MaxY;
+        }
+
+        OGR_F_Destroy( f );
+      }
+      OGR_L_ResetReading( ogrLayer );
+    }
+
+    QgsDebugMsg( "Finished get extent" );
+  }
+
+  OGREnvelope *ext = static_cast<OGREnvelope *>( extent_ );
   mExtentRect.set( ext->MinX, ext->MinY, ext->MaxX, ext->MaxY );
   return mExtentRect;
 }
@@ -674,10 +813,10 @@ bool QgsOgrProvider::addFeature( QgsFeature& f )
   bool returnValue = true;
   OGRFeatureDefnH fdef = OGR_L_GetLayerDefn( ogrLayer );
   OGRFeatureH feature = OGR_F_Create( fdef );
-  unsigned char* wkb = f.geometry()->asWkb();
 
-  if ( f.geometry()->wkbSize() > 0 )
+  if ( f.geometry() && f.geometry()->wkbSize() > 0 )
   {
+    unsigned char* wkb = f.geometry()->asWkb();
     OGRGeometryH geom = NULL;
 
     if ( OGR_G_CreateFromWkb( wkb, NULL, &geom, f.geometry()->wkbSize() )
@@ -746,7 +885,6 @@ bool QgsOgrProvider::addFeature( QgsFeature& f )
   {
     f.setFeatureId( OGR_F_GetFID( feature ) );
   }
-  ++featuresCounted;
   OGR_F_Destroy( feature );
   return returnValue;
 }
@@ -754,6 +892,8 @@ bool QgsOgrProvider::addFeature( QgsFeature& f )
 
 bool QgsOgrProvider::addFeatures( QgsFeatureList & flist )
 {
+  setRelevantFields( true, mAttributeFields.keys() );
+
   bool returnvalue = true;
   for ( QgsFeatureList::iterator it = flist.begin(); it != flist.end(); ++it )
   {
@@ -767,7 +907,10 @@ bool QgsOgrProvider::addFeatures( QgsFeatureList & flist )
   {
     returnvalue = false;
   }
-  featuresCounted = OGR_L_GetFeatureCount( ogrLayer, TRUE ); //new feature count
+  recalculateFeatureCount();
+
+  if ( returnvalue )
+    clearMinMaxCache();
 
   return returnvalue;
 }
@@ -797,11 +940,11 @@ bool QgsOgrProvider::addAttributes( const QList<QgsField> &attributes )
         continue;
     }
 
-    OGRFieldDefnH fielddefn = OGR_Fld_Create( mEncoding->fromUnicode( iter->name() ).data(), type );
+    OGRFieldDefnH fielddefn = OGR_Fld_Create( TO8( iter->name() ), type );
     OGR_Fld_SetWidth( fielddefn, iter->length() );
     OGR_Fld_SetPrecision( fielddefn, iter->precision() );
 
-    if ( OGR_L_CreateField( ogrLayer, fielddefn, TRUE ) != OGRERR_NONE )
+    if ( OGR_L_CreateField( ogrLayer, fielddefn, true ) != OGRERR_NONE )
     {
       QgsLogger::warning( "QgsOgrProvider.cpp: writing of field failed" );
       returnvalue = false;
@@ -814,6 +957,13 @@ bool QgsOgrProvider::addAttributes( const QList<QgsField> &attributes )
 
 bool QgsOgrProvider::changeAttributeValues( const QgsChangedAttributesMap & attr_map )
 {
+  if ( attr_map.isEmpty() )
+    return true;
+
+  clearMinMaxCache();
+
+  setRelevantFields( true, mAttributeFields.keys() );
+
   for ( QgsChangedAttributesMap::const_iterator it = attr_map.begin(); it != attr_map.end(); ++it )
   {
     long fid = ( long ) it.key();
@@ -883,6 +1033,8 @@ bool QgsOgrProvider::changeGeometryValues( QgsGeometryMap & geometry_map )
   OGRFeatureH theOGRFeature = 0;
   OGRGeometryH theNewGeometry = 0;
 
+  setRelevantFields( true, mAttributeFields.keys() );
+
   for ( QgsGeometryMap::iterator it = geometry_map.begin(); it != geometry_map.end(); ++it )
   {
     theOGRFeature = OGR_L_GetFeature( ogrLayer, it.key() );
@@ -935,10 +1087,13 @@ bool QgsOgrProvider::changeGeometryValues( QgsGeometryMap & geometry_map )
 
 bool QgsOgrProvider::createSpatialIndex()
 {
+  QgsCPLErrorHandler handler;
+
   QString layerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) );
 
   QString sql = QString( "CREATE SPATIAL INDEX ON %1" ).arg( quotedIdentifier( layerName ) );  // quote the layer name so spaces are handled
-  OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), OGR_L_GetSpatialFilter( ogrOrigLayer ), "" );
+  QgsDebugMsg( QString( "SQL: %1" ).arg( sql ) );
+  OGR_DS_ExecuteSQL( ogrDataSource, TO8( sql ), OGR_L_GetSpatialFilter( ogrOrigLayer ), "" );
 
   QFileInfo fi( mFilePath );     // to get the base name
   //find out, if the .qix file is there
@@ -946,8 +1101,24 @@ bool QgsOgrProvider::createSpatialIndex()
   return indexfile.exists();
 }
 
+bool QgsOgrProvider::createAttributeIndex( int field )
+{
+  QString layerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) );
+  QString dropSql = QString( "DROP INDEX ON %1" ).arg( quotedIdentifier( layerName ) );
+  QString createSql = QString( "CREATE INDEX ON %1 USING %2" ).arg( quotedIdentifier( layerName ) ).arg( fields()[field].name() );
+  OGR_DS_ExecuteSQL( ogrDataSource, TO8( dropSql ), OGR_L_GetSpatialFilter( ogrOrigLayer ), "SQL" );
+  OGR_DS_ExecuteSQL( ogrDataSource, TO8( createSql ), OGR_L_GetSpatialFilter( ogrOrigLayer ), "SQL" );
+
+  QFileInfo fi( mFilePath );     // to get the base name
+  //find out, if the .idm file is there
+  QFile indexfile( fi.path().append( "/" ).append( fi.completeBaseName() ).append( ".idm" ) );
+  return indexfile.exists();
+}
+
 bool QgsOgrProvider::deleteFeatures( const QgsFeatureIds & id )
 {
+  QgsCPLErrorHandler handler;
+
   bool returnvalue = true;
   for ( QgsFeatureIds::const_iterator it = id.begin(); it != id.end(); ++it )
   {
@@ -964,22 +1135,31 @@ bool QgsOgrProvider::deleteFeatures( const QgsFeatureIds & id )
 
   QString layerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrOrigLayer ) );
 
-  QFileInfo fi( dataSourceUri() );     // to get the base name
   QString sql = QString( "REPACK %1" ).arg( layerName );   // don't quote the layer name as it works with spaces in the name and won't work if the name is quoted
-  OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, NULL );
-  featuresCounted = OGR_L_GetFeatureCount( ogrLayer, TRUE ); //new feature count
+  QgsDebugMsg( QString( "SQL: %1" ).arg( sql ) );
+  OGR_DS_ExecuteSQL( ogrDataSource, TO8( sql ), NULL, NULL );
+
+  recalculateFeatureCount();
+
+  clearMinMaxCache();
+
+  if ( extent_ )
+  {
+    free( extent_ );
+    extent_ = 0;
+  }
+
   return returnvalue;
 }
 
 bool QgsOgrProvider::deleteFeature( int id )
 {
-  OGRErr res = OGR_L_DeleteFeature( ogrLayer, id );
-  return ( res == OGRERR_NONE );
+  return OGR_L_DeleteFeature( ogrLayer, id ) == OGRERR_NONE;
 }
 
 int QgsOgrProvider::capabilities() const
 {
-  int ability = NoCapabilities;
+  int ability = SetEncoding;
 
   // collect abilities reported by OGR
   if ( ogrLayer )
@@ -992,7 +1172,7 @@ int QgsOgrProvider::capabilities() const
     // the #defines we want to test for here.
 
     if ( OGR_L_TestCapability( ogrLayer, "RandomRead" ) )
-      // TRUE if the GetFeature() method works *efficiently* for this layer.
+      // true if the GetFeature() method works *efficiently* for this layer.
       // TODO: Perhaps influence if QGIS caches into memory
       //       (vs read from disk every time) based on this setting.
     {
@@ -1001,19 +1181,19 @@ int QgsOgrProvider::capabilities() const
     }
 
     if ( OGR_L_TestCapability( ogrLayer, "SequentialWrite" ) )
-      // TRUE if the CreateFeature() method works for this layer.
+      // true if the CreateFeature() method works for this layer.
     {
       ability |= QgsVectorDataProvider::AddFeatures;
     }
 
     if ( OGR_L_TestCapability( ogrLayer, "DeleteFeature" ) )
-      // TRUE if this layer can delete its features
+      // true if this layer can delete its features
     {
       ability |= DeleteFeatures;
     }
 
     if ( OGR_L_TestCapability( ogrLayer, "RandomWrite" ) )
-      // TRUE if the SetFeature() method is operational on this layer.
+      // true if the SetFeature() method is operational on this layer.
     {
       // TODO According to http://shapelib.maptools.org/ (Shapefile C Library V1.2)
       // TODO "You can't modify the vertices of existing structures".
@@ -1026,9 +1206,9 @@ int QgsOgrProvider::capabilities() const
 
 #if 0
     if ( OGR_L_TestCapability( ogrLayer, "FastSpatialFilter" ) )
-      // TRUE if this layer implements spatial filtering efficiently.
+      // true if this layer implements spatial filtering efficiently.
       // Layers that effectively read all features, and test them with the
-      // OGRFeature intersection methods should return FALSE.
+      // OGRFeature intersection methods should return false.
       // This can be used as a clue by the application whether it should build
       // and maintain it's own spatial index for features in this layer.
     {
@@ -1036,25 +1216,25 @@ int QgsOgrProvider::capabilities() const
     }
 
     if ( OGR_L_TestCapability( ogrLayer, "FastFeatureCount" ) )
-      // TRUE if this layer can return a feature count
+      // true if this layer can return a feature count
       // (via OGRLayer::GetFeatureCount()) efficiently ... ie. without counting
-      // the features. In some cases this will return TRUE until a spatial
-      // filter is installed after which it will return FALSE.
+      // the features. In some cases this will return true until a spatial
+      // filter is installed after which it will return false.
     {
       // TODO: Perhaps use as a clue by QGIS whether it should spawn a thread to count features.
     }
 
     if ( OGR_L_TestCapability( ogrLayer, "FastGetExtent" ) )
-      // TRUE if this layer can return its data extent
+      // true if this layer can return its data extent
       // (via OGRLayer::GetExtent()) efficiently ... ie. without scanning
-      // all the features. In some cases this will return TRUE until a
-      // spatial filter is installed after which it will return FALSE.
+      // all the features. In some cases this will return true until a
+      // spatial filter is installed after which it will return false.
     {
       // TODO: Perhaps use as a clue by QGIS whether it should spawn a thread to calculate extent.
     }
 
     if ( OGR_L_TestCapability( ogrLayer, "FastSetNextByIndex" ) )
-      // TRUE if this layer can perform the SetNextByIndex() call efficiently.
+      // true if this layer can perform the SetNextByIndex() call efficiently.
     {
       // No use required for this QGIS release.
     }
@@ -1071,6 +1251,7 @@ int QgsOgrProvider::capabilities() const
       // adding attributes was added in GDAL 1.6
       ability |= AddAttributes;
 #endif
+      ability |= CreateAttributeIndex;
 
       if ( mAttributeFields.size() == 0 )
       {
@@ -1121,8 +1302,7 @@ QString  QgsOgrProvider::description() const
 */
 static QString createFileFilter_( QString const &longName, QString const &glob )
 {
-  return "[OGR] " +
-         longName + " (" + glob.toLower() + " " + glob.toUpper() + ");;";
+  return longName + " [OGR] (" + glob.toLower() + " " + glob.toUpper() + ");;";
 } // createFileFilter_
 
 
@@ -1143,16 +1323,11 @@ QString createFilters( QString type )
 
   if ( myFileFilters.isEmpty() || myFileFilters.isNull() )
   {
-
-
-
     // register ogr plugins
     QgsApplication::registerOgrDrivers();
 
     // first get the GDAL driver manager
-
     OGRSFDriverH driver;          // current driver
-
     QString driverName;           // current driver name
 
     // Grind through all the drivers and their respective metadata.
@@ -1179,149 +1354,161 @@ QString createFilters( QString type )
 
       if ( driverName.startsWith( "AVCBin" ) )
       {
-        myDirectoryDrivers += "Arc/Info Binary Coverage,AVCBin;";
+        myDirectoryDrivers += QObject::tr( "Arc/Info Binary Coverage" ) + ",AVCBin;";
       }
       else if ( driverName.startsWith( "AVCE00" ) )
       {
-        myFileFilters += createFileFilter_( "Arc/Info ASCII Coverage", "*.e00" );
+        myFileFilters += createFileFilter_( QObject::tr( "Arc/Info ASCII Coverage" ), "*.e00" );
       }
       else if ( driverName.startsWith( "BNA" ) )
       {
-        myFileFilters += createFileFilter_( "Atlas BNA", "*.bna" );
+        myFileFilters += createFileFilter_( QObject::tr( "Atlas BNA" ), "*.bna" );
       }
       else if ( driverName.startsWith( "CSV" ) )
       {
-        myFileFilters += createFileFilter_( "Comma Separated Value", "*.csv" );
+        myFileFilters += createFileFilter_( QObject::tr( "Comma Separated Value" ), "*.csv" );
       }
-      else if ( driverName.startsWith( "DODS" ) )
+      else if ( driverName.startsWith( QObject::tr( "DODS" ) ) )
       {
         myProtocolDrivers += "DODS/OPeNDAP,DODS;";
       }
       else if ( driverName.startsWith( "PGeo" ) )
       {
-        myDatabaseDrivers += "ESRI Personal GeoDatabase,PGeo;";
+        myDatabaseDrivers += QObject::tr( "ESRI Personal GeoDatabase" ) + ",PGeo;";
 #ifdef WIN32
-        myFileFilters += createFileFilter_( "ESRI Personal GeoDatabase", "*.mdb" );
+        myFileFilters += createFileFilter_( QObject::tr( "ESRI Personal GeoDatabase" ), "*.mdb" );
 #endif
       }
       else if ( driverName.startsWith( "SDE" ) )
       {
-        myDatabaseDrivers += "ESRI ArcSDE,SDE;";
+        myDatabaseDrivers += QObject::tr( "ESRI ArcSDE" ) + ",SDE;";
       }
       else if ( driverName.startsWith( "ESRI" ) )
       {
-        myFileFilters += createFileFilter_( "ESRI Shapefiles", "*.shp" );
+        myFileFilters += createFileFilter_( QObject::tr( "ESRI Shapefiles" ), "*.shp" );
       }
-      else if ( driverName.startsWith( "FMEObjects Gateway" ) )
+      else if ( driverName.startsWith( QObject::tr( "FMEObjects Gateway" ) ) )
       {
-        myFileFilters += createFileFilter_( "FMEObjects Gateway", "*.fdd" );
+        myFileFilters += createFileFilter_( QObject::tr( "FMEObjects Gateway" ), "*.fdd" );
       }
       else if ( driverName.startsWith( "GeoJSON" ) )
       {
         myProtocolDrivers += "GeoJSON,GeoJSON;";
-        myFileFilters += createFileFilter_( "GeoJSON", "*.geojson" );
+        myFileFilters += createFileFilter_( QObject::tr( "GeoJSON" ), "*.geojson" );
       }
       else if ( driverName.startsWith( "GeoRSS" ) )
       {
-        myFileFilters += createFileFilter_( "GeoRSS", "*.xml" );
+        myFileFilters += createFileFilter_( QObject::tr( "GeoRSS" ), "*.xml" );
       }
       else if ( driverName.startsWith( "GML" ) )
       {
-        myFileFilters += createFileFilter_( "Geography Markup Language", "*.gml" );
+        myFileFilters += createFileFilter_( QObject::tr( "Geography Markup Language [GML]" ), "*.gml" );
       }
       else if ( driverName.startsWith( "GMT" ) )
       {
-        myFileFilters += createFileFilter_( "GMT", "*.gmt" );
+        myFileFilters += createFileFilter_( QObject::tr( "Generic Mapping Tools [GMT]" ), "*.gmt" );
       }
       else if ( driverName.startsWith( "GPX" ) )
       {
-        myFileFilters += createFileFilter_( "GPX", "*.gpx" );
+        myFileFilters += createFileFilter_( QObject::tr( "GPS eXchange Format [GPX]" ), "*.gpx" );
       }
       else if ( driverName.startsWith( "GRASS" ) )
       {
-        myDirectoryDrivers += "Grass Vector,GRASS;";
+        myDirectoryDrivers += QObject::tr( "Grass Vector" ) + ",GRASS;";
       }
       else if ( driverName.startsWith( "IDB" ) )
       {
-        myDatabaseDrivers += "Informix DataBlade,IDB;";
+        myDatabaseDrivers += QObject::tr( "Informix DataBlade" ) + ",IDB;";
       }
       else if ( driverName.startsWith( "Interlis 1" ) )
       {
-        myFileFilters += createFileFilter_( "INTERLIS 1", "*.itf *.xml *.ili" );
+        myFileFilters += createFileFilter_( QObject::tr( "INTERLIS 1" ), "*.itf *.xml *.ili" );
       }
       else if ( driverName.startsWith( "Interlis 2" ) )
       {
-        myFileFilters += createFileFilter_( "INTERLIS 2", "*.itf *.xml *.ili" );
+        myFileFilters += createFileFilter_( QObject::tr( "INTERLIS 2" ), "*.itf *.xml *.ili" );
       }
       else if ( driverName.startsWith( "INGRES" ) )
       {
-        myDatabaseDrivers += "INGRES,INGRES;";
+        myDatabaseDrivers += QObject::tr( "INGRES" ) + ",INGRES;";
       }
       else if ( driverName.startsWith( "KML" ) )
       {
-        myFileFilters += createFileFilter_( "KML", "*.kml" );
+        myFileFilters += createFileFilter_( QObject::tr( "Keyhole Markup Language [KML]" ), "*.kml" );
       }
       else if ( driverName.startsWith( "MapInfo File" ) )
       {
-        myFileFilters += createFileFilter_( "Mapinfo File", "*.mif *.tab" );
+        myFileFilters += createFileFilter_( QObject::tr( "Mapinfo File" ), "*.mif *.tab" );
       }
       else if ( driverName.startsWith( "DGN" ) )
       {
-        myFileFilters += createFileFilter_( "Microstation DGN", "*.dgn" );
+        myFileFilters += createFileFilter_( QObject::tr( "Microstation DGN" ), "*.dgn" );
       }
       else if ( driverName.startsWith( "MySQL" ) )
       {
-        myDatabaseDrivers += "MySQL,MySQL;";
+        myDatabaseDrivers += QObject::tr( "MySQL" ) + ",MySQL;";
+      }
+      else if ( driverName.startsWith( "MSSQL" ) )
+      {
+        myDatabaseDrivers += QObject::tr( "MSSQL" ) + ",MSSQL;";
       }
       else if ( driverName.startsWith( "OCI" ) )
       {
-        myDatabaseDrivers += "Oracle Spatial,OCI;";
+        myDatabaseDrivers += QObject::tr( "Oracle Spatial" ) + ",OCI;";
       }
       else if ( driverName.startsWith( "ODBC" ) )
       {
-        myDatabaseDrivers += "ODBC,ODBC;";
+        myDatabaseDrivers += QObject::tr( "ODBC" ) + ",ODBC;";
       }
       else if ( driverName.startsWith( "OGDI" ) )
       {
-        myDatabaseDrivers += "OGDI Vectors,OGDI;";
+        myDatabaseDrivers += QObject::tr( "OGDI Vectors" ) + ",OGDI;";
       }
       else if ( driverName.startsWith( "PostgreSQL" ) )
       {
-        myDatabaseDrivers += "PostgreSQL,PostgreSQL;";
+        myDatabaseDrivers += QObject::tr( "PostgreSQL" ) + ",PostgreSQL;";
       }
       else if ( driverName.startsWith( "S57" ) )
       {
-        myFileFilters += createFileFilter_( "S-57 Base file",
+        myFileFilters += createFileFilter_( QObject::tr( "S-57 Base file" ),
                                             "*.000" );
       }
       else if ( driverName.startsWith( "SDTS" ) )
       {
-        myFileFilters += createFileFilter_( "Spatial Data Transfer Standard",
+        myFileFilters += createFileFilter_( QObject::tr( "Spatial Data Transfer Standard [SDTS]" ),
                                             "*catd.ddf" );
       }
       else if ( driverName.startsWith( "SQLite" ) )
       {
-        myFileFilters += createFileFilter_( "SQLite",
+        myFileFilters += createFileFilter_( QObject::tr( "SQLite" ),
                                             "*.sqlite" );
       }
       else if ( driverName.startsWith( "UK .NTF" ) )
       {
-        myDirectoryDrivers += "UK. NTF,UK. NTF;";
+        myDirectoryDrivers += QObject::tr( "UK. NTF2" ) + ",UK. NTF;";
       }
       else if ( driverName.startsWith( "TIGER" ) )
       {
-        myDirectoryDrivers += "U.S. Census TIGER/Line,TIGER;";
+        myDirectoryDrivers += QObject::tr( "U.S. Census TIGER/Line" ) + ",TIGER;";
       }
       else if ( driverName.startsWith( "VRT" ) )
       {
-        myFileFilters += createFileFilter_( "VRT - Virtual Datasource ",
+        myFileFilters += createFileFilter_( QObject::tr( "VRT - Virtual Datasource" ),
                                             "*.vrt" );
       }
       else if ( driverName.startsWith( "XPlane" ) )
       {
-        myFileFilters += createFileFilter_( "X-Plane/Flighgear",
+        myFileFilters += createFileFilter_( QObject::tr( "X-Plane/Flightgear" ),
                                             "apt.dat nav.dat fix.dat awy.dat" );
       }
+      else if ( driverName.startsWith( "Geoconcept" ) )
+      {
+        myFileFilters += createFileFilter_( QObject::tr( "Geoconcept" ), "*.gxt *.txt" );
+      }
+      else if ( driverName.startsWith( "DXF" ) )
+      {
+        myFileFilters += createFileFilter_( QObject::tr( "AutoCAD DXF" ), "*.dxf" );
+      }
       else
       {
         // NOP, we don't know anything about the current driver
@@ -1333,7 +1520,7 @@ QString createFilters( QString type )
 
     // can't forget the default case
 
-    myFileFilters += "All files (*.*)";
+    myFileFilters += QObject::tr( "All files" ) + " (*)";
   }
 
   if ( type == "file" )
@@ -1482,10 +1669,16 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
       }
       fieldNames << name;
     }
+
+    QgsVectorFileWriter::deleteShapeFile( uri );
+  }
+  else
+  {
+    QFile::remove( uri );
   }
 
   OGRDataSourceH dataSource;
-  dataSource = OGR_Dr_CreateDataSource( driver, QFile::encodeName( uri ).constData(), NULL );
+  dataSource = OGR_Dr_CreateDataSource( driver, TO8F( uri ), NULL );
   if ( dataSource == NULL )
   {
     return false;
@@ -1542,7 +1735,7 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
   }
 
   OGRLayerH layer;
-  layer = OGR_DS_CreateLayer( dataSource, QFile::encodeName( QFileInfo( uri ).completeBaseName() ).constData(), reference, OGRvectortype, NULL );
+  layer = OGR_DS_CreateLayer( dataSource, TO8F( QFileInfo( uri ).completeBaseName() ), reference, OGRvectortype, NULL );
   if ( layer == NULL )
   {
     return false;
@@ -1551,6 +1744,13 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
   //create the attribute fields
 
   QTextCodec* codec = QTextCodec::codecForName( encoding.toLocal8Bit().data() );
+  if ( !codec )
+  {
+    // fall back to "System" codec
+    codec = QTextCodec::codecForLocale();
+    Q_ASSERT( codec );
+  }
+
 
   for ( std::list<std::pair<QString, QString> >::const_iterator it = attributes.begin(); it != attributes.end(); ++it )
   {
@@ -1570,7 +1770,7 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
       if ( precision < 0 )
         precision = 3;
 
-      field = OGR_Fld_Create( codec->fromUnicode( it->first ).data(), OFTReal );
+      field = OGR_Fld_Create( TO8( it->first ), OFTReal );
       OGR_Fld_SetWidth( field, width );
       OGR_Fld_SetPrecision( field, precision );
     }
@@ -1579,7 +1779,7 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
       if ( width < 0 || width > 10 )
         width = 10;
 
-      field = OGR_Fld_Create( codec->fromUnicode( it->first ).data(), OFTInteger );
+      field = OGR_Fld_Create( TO8( it->first ), OFTInteger );
       // limit to 10.  otherwise OGR sets it to 11 and recognizes as OFTDouble later
       OGR_Fld_SetWidth( field, width );
     }
@@ -1588,7 +1788,7 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
       if ( width < 0 || width > 255 )
         width = 255;
 
-      field = OGR_Fld_Create( codec->fromUnicode( it->first ).data(), OFTString );
+      field = OGR_Fld_Create( TO8( it->first ), OFTString );
       OGR_Fld_SetWidth( field, width );
     }
     else
@@ -1596,7 +1796,7 @@ QGISEXTERN bool createEmptyDataSource( const QString &uri,
       continue;
     }
 
-    if ( OGR_L_CreateField( layer, field, TRUE ) != OGRERR_NONE )
+    if ( OGR_L_CreateField( layer, field, true ) != OGRERR_NONE )
     {
       QgsLogger::warning( "creation of field failed" );
     }
@@ -1650,33 +1850,32 @@ QgsCoordinateReferenceSystem QgsOgrProvider::crs()
         QString myWktString = prjStream.readLine();
         prjFile.close();
 
-        // create CRS from Wkt
-        srs.createFromWkt( myWktString );
-
-        if ( srs.isValid() )
+        if ( srs.createFromWkt( myWktString.toUtf8().constData() ) )
           return srs;
       }
     }
   }
 
+  CPLSetConfigOption( "GDAL_FIX_ESRI_WKT", "TOWGS84" ); // add towgs84 parameter
+
   OGRSpatialReferenceH mySpatialRefSys = OGR_L_GetSpatialRef( ogrLayer );
-  if ( mySpatialRefSys == NULL )
+  if ( mySpatialRefSys )
   {
-    QgsDebugMsg( "no spatial reference found" );
+    // get the proj4 text
+    char *pszProj4;
+    OSRExportToProj4( mySpatialRefSys, &pszProj4 );
+    QgsDebugMsg( pszProj4 );
+    OGRFree( pszProj4 );
+
+    char *pszWkt = NULL;
+    OSRExportToWkt( mySpatialRefSys, &pszWkt );
+
+    srs.createFromWkt( pszWkt );
+    OGRFree( pszWkt );
   }
   else
   {
-    // get the proj4 text
-    char *ppszProj4;
-    OSRExportToProj4( mySpatialRefSys, &ppszProj4 );
-    QgsDebugMsg( ppszProj4 );
-    char *pszWkt = NULL;
-    OSRExportToWkt( mySpatialRefSys, &pszWkt );
-    QString myWktString = QString( pszWkt );
-    OGRFree( pszWkt );
-
-    // create CRS from Wkt
-    srs.createFromWkt( myWktString );
+    QgsDebugMsg( "no spatial reference found" );
   }
 
   return srs;
@@ -1684,8 +1883,15 @@ QgsCoordinateReferenceSystem QgsOgrProvider::crs()
 
 void QgsOgrProvider::uniqueValues( int index, QList<QVariant> &uniqueValues, int limit )
 {
-  QgsField fld = mAttributeFields[index];
-  QString theLayerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrLayer ) );
+  uniqueValues.clear();
+
+  QgsField fld = mAttributeFields.value( index );
+  if ( fld.name().isNull() )
+  {
+    return; //not a provider field
+  }
+
+  QString theLayerName = FROM8( OGR_FD_GetName( OGR_L_GetLayerDefn( ogrLayer ) ) );
 
   QString sql = QString( "SELECT DISTINCT %1 FROM %2" )
                 .arg( quotedIdentifier( fld.name() ) )
@@ -1696,19 +1902,17 @@ void QgsOgrProvider::uniqueValues( int index, QList<QVariant> &uniqueValues, int
     sql += QString( " WHERE %1" ).arg( mSubsetString );
   }
 
-  sql += QString( " ORDER BY %1" ).arg( fld.name() );
-
-  uniqueValues.clear();
+  sql += QString( " ORDER BY %1 ASC" ).arg( fld.name() ); // quoting of fieldname produces a syntax error
 
   QgsDebugMsg( QString( "SQL: %1" ).arg( sql ) );
-  OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, "SQL" );
+  OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, TO8( sql ), NULL, "SQL" );
   if ( l == 0 )
     return QgsVectorDataProvider::uniqueValues( index, uniqueValues, limit );
 
   OGRFeatureH f;
   while ( 0 != ( f = OGR_L_GetNextFeature( l ) ) )
   {
-    uniqueValues << convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) );
+    uniqueValues << ( OGR_F_IsFieldSet( f, 0 ) ? convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) ) : QVariant( fld.type() ) );
     OGR_F_Destroy( f );
 
     if ( limit >= 0 && uniqueValues.size() >= limit )
@@ -1720,7 +1924,13 @@ void QgsOgrProvider::uniqueValues( int index, QList<QVariant> &uniqueValues, int
 
 QVariant QgsOgrProvider::minimumValue( int index )
 {
-  QgsField fld = mAttributeFields[index];
+  QgsFieldMap::const_iterator attIt = mAttributeFields.find( index );
+  if ( attIt == mAttributeFields.constEnd() )
+  {
+    return QVariant();
+  }
+  const QgsField& fld = attIt.value();
+
   QString theLayerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrLayer ) );
 
   QString sql = QString( "SELECT MIN(%1) FROM %2" )
@@ -1732,7 +1942,7 @@ QVariant QgsOgrProvider::minimumValue( int index )
     sql += QString( " WHERE %1" ).arg( mSubsetString );
   }
 
-  OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, "SQL" );
+  OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, TO8( sql ), NULL, "SQL" );
 
   if ( l == 0 )
     return QgsVectorDataProvider::minimumValue( index );
@@ -1744,7 +1954,7 @@ QVariant QgsOgrProvider::minimumValue( int index )
     return QVariant();
   }
 
-  QVariant value = convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) );
+  QVariant value = OGR_F_IsFieldSet( f, 0 ) ? convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) ) : QVariant( fld.type() );
   OGR_F_Destroy( f );
 
   OGR_DS_ReleaseResultSet( ogrDataSource, l );
@@ -1754,7 +1964,13 @@ QVariant QgsOgrProvider::minimumValue( int index )
 
 QVariant QgsOgrProvider::maximumValue( int index )
 {
-  QgsField fld = mAttributeFields[index];
+  QgsFieldMap::const_iterator attIt = mAttributeFields.find( index );
+  if ( attIt == mAttributeFields.constEnd() )
+  {
+    return QVariant();
+  }
+  const QgsField& fld = mAttributeFields[index];
+
   QString theLayerName = OGR_FD_GetName( OGR_L_GetLayerDefn( ogrLayer ) );
 
   QString sql = QString( "SELECT MAX(%1) FROM %2" )
@@ -1766,7 +1982,7 @@ QVariant QgsOgrProvider::maximumValue( int index )
     sql += QString( " WHERE %1" ).arg( mSubsetString );
   }
 
-  OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, mEncoding->fromUnicode( sql ).data(), NULL, "SQL" );
+  OGRLayerH l = OGR_DS_ExecuteSQL( ogrDataSource, TO8( sql ), NULL, "SQL" );
   if ( l == 0 )
     return QgsVectorDataProvider::maximumValue( index );
 
@@ -1777,7 +1993,7 @@ QVariant QgsOgrProvider::maximumValue( int index )
     return QVariant();
   }
 
-  QVariant value = convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) );
+  QVariant value = OGR_F_IsFieldSet( f, 0 ) ? convertValue( fld.type(), mEncoding->toUnicode( OGR_F_GetFieldAsString( f, 0 ) ) ) : QVariant( fld.type() );
   OGR_F_Destroy( f );
 
   OGR_DS_ReleaseResultSet( ogrDataSource, l );
@@ -1820,3 +2036,22 @@ bool QgsOgrProvider::syncToDisc()
 
   return true;
 }
+
+void QgsOgrProvider::recalculateFeatureCount()
+{
+  OGRGeometryH filter = OGR_L_GetSpatialFilter( ogrLayer );
+  if ( filter )
+  {
+    filter = OGR_G_Clone( filter );
+    OGR_L_SetSpatialFilter( ogrLayer, 0 );
+  }
+
+  // feature count returns number of features within current spatial filter
+  // so we remove it if there's any and then put it back
+  featuresCounted = OGR_L_GetFeatureCount( ogrLayer, true );
+
+  if ( filter )
+  {
+    OGR_L_SetSpatialFilter( ogrLayer, filter );
+  }
+}
