@@ -36,9 +36,6 @@ class QgsGeometry;
 
 #include "qgsdatasourceuri.h"
 
-#include "qgspostgrescountthread.h"
-#include "qgspostgresextentthread.h"
-
 /**
   \class QgsPostgresProvider
   \brief Data provider for PostgreSQL/PostGIS layers.
@@ -253,7 +250,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     QString subsetString();
 
     /** mutator for sql where clause used to limit dataset size */
-    bool setSubsetString( QString theSQL );
+    bool setSubsetString( QString theSQL, bool updateFeatureCount = true );
 
     virtual bool supportsSubsetString() { return true; }
 
@@ -333,6 +330,8 @@ class QgsPostgresProvider : public QgsVectorDataProvider
 
     QString whereClause( int featureId ) const;
 
+    bool hasSufficientPermsAndCapabilities();
+
     const QgsField &field( int index ) const;
 
     /** Double quote a PostgreSQL identifier for placement in a SQL string.
@@ -349,7 +348,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
 
     /** Load the field list
     */
-    void loadFields();
+    bool loadFields();
 
     /**Parses the enum_range of an attribute and inserts the possible values into a stringlist
     @param enumValues the stringlist where the values are appended
@@ -364,7 +363,8 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     */
     bool parseDomainCheckConstraint( QStringList& enumValues, const QString& attributeName ) const;
 
-    bool mFetching;   // true if a cursor was declared
+    bool mFetching; // true if a cursor was declared
+    int mFetched; // number of retrieved features
     std::vector < QgsFeature > features;
     QgsFieldMap attributeFields;
     QString mDataComment;
@@ -372,25 +372,29 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     //! Data source URI struct for this layer
     QgsDataSourceURI mUri;
 
-
-    //! Child thread for calculating extents.
-    QgsPostgresExtentThread mExtentThread;
-
-    //! Child thread for calculating count.
-    QgsPostgresCountThread mCountThread;
-
     /**
      * Flag indicating if the layer data source is a valid PostgreSQL layer
      */
     bool valid;
+
+    /**
+     * provider references query (instead of a table)
+     */
+    bool isQuery;
+
+    /**
+     * geometry is geography
+     */
+    bool isGeography;
+
     /**
      * Name of the table with no schema
      */
     QString mTableName;
     /**
-     * Name of the table with schema included
+     * Name of the table or subquery
      */
-    QString mSchemaTableName;
+    QString mQuery;
     /**
      * Name of the schema
      */
@@ -403,11 +407,16 @@ class QgsPostgresProvider : public QgsVectorDataProvider
      * SQL statement used to limit the features retrieved
      */
     QString sqlWhereClause;
+
     /**
      * Primary key column for fetching features. If there is no primary key
      * the oid is used to fetch features.
      */
     QString primaryKey;
+    /**
+     * Primary key column is "real" primary key
+     */
+    bool mIsDbPrimaryKey;
     /**
      * Data type for the primary key
      */
@@ -433,7 +442,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     /**
      * Number of features in the layer
      */
-    long featuresCounted;
+    mutable long featuresCounted;
 
     /**
      * Feature queue that GetNextFeature will retrieve from
@@ -461,10 +470,13 @@ class QgsPostgresProvider : public QgsVectorDataProvider
     bool deduceEndian();
     bool getGeometryDetails();
 
+    /* Use estimated metadata. Uses fast table counts, geometry type and extent determination */
+    bool mUseEstimatedMetadata;
+
     // Produces a QMessageBox with the given title and text. Doesn't
     // return until the user has dismissed the dialog box.
-    static void showMessageBox( const QString& title, const QString& text );
-    static void showMessageBox( const QString& title, const QStringList& text );
+    static void showMessageBox( const QString& title, const QString &text );
+    static void showMessageBox( const QString& title, const QStringList &text );
 
     // A simple class to store the rows of the sql executed in the
     // findColumns() function.
@@ -539,7 +551,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
 
     // A function that determines if the given schema.table.column
     // contains unqiue entries
-    bool uniqueData( QString schemaName, QString tableName, QString colName );
+    bool uniqueData( QString query, QString colName );
 
     // Function that populates the given cols structure.
     void findColumns( tableCols& cols );
@@ -560,28 +572,17 @@ class QgsPostgresProvider : public QgsVectorDataProvider
 
     int enabledCapabilities;
 
-    //! Get the feature count based on the where clause
-    long getFeatureCount();
-
-    //! Calculate the extents of the layer
-    void calculateExtents();
-
-    /**
-     * Event sink for events from threads
-     */
-    void customEvent( QEvent *e );
-
     void appendGeomString( QgsGeometry *geom, QString &geomParam ) const;
-    QByteArray paramValue( QString fieldvalue, const QString &defaultValue ) const;
+    QString paramValue( QString fieldvalue, const QString &defaultValue ) const;
 
     class Conn
     {
       public:
-        Conn( PGconn *connection ) :
-            ref( 1 ),
-            openCursors( 0 ),
-            conn( connection ),
-            gotPostgisVersion( false )
+        Conn( PGconn *connection )
+            : ref( 1 )
+            , openCursors( 0 )
+            , conn( connection )
+            , gotPostgisVersion( false )
         {
         }
 
@@ -603,10 +604,13 @@ class QgsPostgresProvider : public QgsVectorDataProvider
         //! major PostgreSQL version
         int majorVersion() { return postgisVersionMajor; }
 
-        // run a query and free result buffer
+        //! PostgreSQL version
+        int pgVersion() { return postgresqlVersion; }
+
+        //! run a query and free result buffer
         bool PQexecNR( QString query );
 
-        // cursor handling
+        //! cursor handling
         bool openCursor( QString cursorName, QString declare );
         bool closeCursor( QString cursorName );
 
@@ -643,6 +647,9 @@ class QgsPostgresProvider : public QgsVectorDataProvider
         //! Are postgisVersionMajor, postgisVersionMinor, geosAvailable, gistAvailable, projAvailable valid?
         bool gotPostgisVersion;
 
+        //! PostgreSQL version
+        int postgresqlVersion;
+
         //! PostGIS major version
         int postgisVersionMajor;
 
@@ -660,6 +667,7 @@ class QgsPostgresProvider : public QgsVectorDataProvider
 
         static QMap<QString, Conn *> connectionsRW;
         static QMap<QString, Conn *> connectionsRO;
+        static QMap<QString, QString> passwordCache;
     };
 
     class Result
