@@ -16,7 +16,8 @@ email                : a.furieri@lqt.it
 
 extern "C"
 {
-#include <spatialite/sqlite3.h>
+#include <sys/types.h>
+#include <sqlite3.h>
 #include <spatialite/gaiageo.h>
 #include <spatialite.h>
 }
@@ -79,7 +80,7 @@ class QgsSpatiaLiteProvider: public QgsVectorDataProvider
     virtual QString subsetString();
 
     /** mutator for sql where clause used to limit dataset size */
-    virtual bool setSubsetString( QString theSQL );
+    virtual bool setSubsetString( QString theSQL, bool updateFeatureCount = true );
 
     virtual bool supportsSubsetString() { return true; }
 
@@ -257,6 +258,7 @@ class QgsSpatiaLiteProvider: public QgsVectorDataProvider
     void repaintRequested();
 
   private:
+
     /** loads fields from input file to member attributeFields */
     void loadFields();
 
@@ -265,6 +267,10 @@ class QgsSpatiaLiteProvider: public QgsVectorDataProvider
        * Flag indicating if the layer data source is a valid SpatiaLite layer
        */
     bool valid;
+    /**
+       * Flag indicating if the layer data source is based on a query
+       */
+    bool isQuery;
     /**
        * Flag indicating if the layer data source is based on a plain Table
        */
@@ -289,6 +295,10 @@ class QgsSpatiaLiteProvider: public QgsVectorDataProvider
      * Name of the table with no schema
      */
     QString mTableName;
+    /**
+       * Name of the table or subquery
+       */
+    QString mQuery;
     /**
        * Name of the primary key column in the table
        */
@@ -321,6 +331,10 @@ class QgsSpatiaLiteProvider: public QgsVectorDataProvider
      * String used to define a subset of the layer
      */
     QString mSubsetString;
+    /**
+       * CoodDimensions of the layer
+       */
+    int nDims;
     /**
        * Spatial reference id of the layer
        */
@@ -356,14 +370,52 @@ class QgsSpatiaLiteProvider: public QgsVectorDataProvider
     */
     //void sqliteOpen();
     void closeDb();
+    QString quotedIdentifier( QString id ) const;
     QString quotedValue( QString value ) const;
     bool checkLayerType();
     bool getGeometryDetails();
     bool getTableGeometryDetails();
     bool getViewGeometryDetails();
     bool getVShapeGeometryDetails();
+    bool getQueryGeometryDetails();
     bool getSridDetails();
     bool getTableSummary();
+    void convertToGeosWKB ( const unsigned char *blob, size_t blob_size, 
+                            unsigned char **wkb, size_t *geom_size );
+    int computeSizeFromMultiWKB2D( const unsigned char *p_in, int nDims,
+                                   int little_endian, 
+                                   int endian_arch );
+    int computeSizeFromMultiWKB3D( const unsigned char *p_in, int nDims, 
+                                   int little_endian,
+                                   int endian_arch );
+    void convertFromGeosWKB2D( const unsigned char *blob, size_t blob_size, 
+                              unsigned char *wkb, size_t geom_size,  
+                              int nDims, int little_endian, int endian_arch );
+    void convertFromGeosWKB3D( const unsigned char *blob, size_t blob_size, 
+                              unsigned char *wkb, size_t geom_size, 
+                              int nDims, int little_endian, int endian_arch );
+    int computeMultiWKB3Dsize( const unsigned char *p_in, int little_endian, 
+                               int endian_arch );
+    void convertFromGeosWKB ( const unsigned char *blob, size_t blob_size, 
+                              unsigned char **wkb, size_t *geom_size, 
+                              int dims );
+    int computeSizeFromGeosWKB3D( const unsigned char *blob, size_t size, 
+                                  int type, int nDims, int little_endian, 
+                                  int endian_arch );
+    int computeSizeFromGeosWKB2D( const unsigned char *blob, size_t size,    
+                                  int type, int nDims, int little_endian, 
+                                  int endian_arch );
+	
+    enum GEOS_3D
+    {
+        GEOS_3D_POINT              = -2147483647,
+        GEOS_3D_LINESTRING         = -2147483646,
+        GEOS_3D_POLYGON            = -2147483645,
+        GEOS_3D_MULTIPOINT         = -2147483644,
+        GEOS_3D_MULTILINESTRING    = -2147483643,
+        GEOS_3D_MULTIPOLYGON       = -2147483642,
+        GEOS_3D_GEOMETRYCOLLECTION = -2147483641,
+    };
 
   public:
     class SqliteHandles
