@@ -28,9 +28,13 @@
 #include <QStringList>
 #include <QDomElement>
 #include <QMap>
+#include <QUrl>
 #include <QVector>
 
 class QgsCoordinateTransform;
+class QNetworkAccessManager;
+class QNetworkReply;
+class QNetworkRequest;
 
 /*
  * The following structs reflect the WMS XML schema,
@@ -277,7 +281,7 @@ struct QgsWmsLayerProperty
   QString                                 title;
   QString                                 abstract;
   QStringList                             keywordList;
-  QVector<QString>                        crs;        // coord ref sys
+  QStringList                             crs;        // coord ref sys
   QgsRectangle                            ex_GeographicBoundingBox;
   QVector<QgsWmsBoundingBoxProperty>      boundingBox;
   QVector<QgsWmsDimensionProperty>        dimension;
@@ -301,22 +305,35 @@ struct QgsWmsLayerProperty
   int                fixedHeight;
 };
 
+struct QgsWmsTileSetProfile
+{
+  QString crs;
+  QgsWmsBoundingBoxProperty boundingBox;
+  QStringList resolutions;
+  int tileWidth;
+  int tileHeight;
+  QString format;
+  QStringList styles;
+  QStringList layers;
+};
+
 /** Capability Property structure */
 // TODO: Fill to WMS specifications
 struct QgsWmsCapabilityProperty
 {
-  QgsWmsRequestProperty           request;
-  QgsWmsExceptionProperty         exception;
-  QgsWmsLayerProperty             layer;
+  QgsWmsRequestProperty         request;
+  QgsWmsExceptionProperty       exception;
+  QgsWmsLayerProperty           layer;
+  QVector<QgsWmsTileSetProfile> tileSetProfiles;
 };
 
 /** Capabilities Property structure */
 // TODO: Fill to WMS specifications
 struct QgsWmsCapabilitiesProperty
 {
-  QgsWmsServiceProperty           service;
-  QgsWmsCapabilityProperty        capability;
-  QString                         version;
+  QgsWmsServiceProperty         service;
+  QgsWmsCapabilityProperty      capability;
+  QString                       version;
 };
 
 
@@ -351,26 +368,28 @@ class QgsWmsProvider : public QgsRasterDataProvider
      *
      * \param[out] layers   The list of layers will be placed here.
      *
-     * \retval FALSE if the layers could not be retrieved or parsed -
+     * \retval false if the layers could not be retrieved or parsed -
      *         see lastError() for more info
      *
      * \todo Document this better, make static
      */
-    virtual bool supportedLayers( QVector<QgsWmsLayerProperty> & layers );
+    virtual bool supportedLayers( QVector<QgsWmsLayerProperty> &layers );
+
+    /**
+     * \brief   Returns a list of the supported tile sets of the WMS server
+     *
+     * \param[out] tileset  The list of tile sets will be placed here.
+     *
+     * \retval false if the tile sets could not be retrieved or parsed -
+     *         see lastError() for more info
+     */
+    virtual bool supportedTileSets( QVector<QgsWmsTileSetProfile> &tilesets );
 
     /**
      * \brief   Returns a map for the hierarchy of layers
      */
     virtual void layerParents( QMap<int, int> &parents, QMap<int, QStringList> &parentNames ) const;
 
-    // TODO: Document this better
-    /** \brief   Returns a list of the supported CRSs of the given layers
-     *  \note    Since WMS can specify CRSs per layer, this may change depending
-                 on which layers are specified!  The "lowest common denominator" between
-                 all specified layers is returned.
-     */
-    virtual QSet<QString> supportedCrsForLayers( QStringList const & layers );
-
     /*! Get the QgsCoordinateReferenceSystem for this layer
      * @note Must be reimplemented by each provider.
      * If the provider isn't capable of returning
@@ -381,9 +400,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     /**
      * Add the list of WMS layer names to be rendered by this server
      */
-    void addLayers( QStringList const &  layers,
-                    QStringList const &  styles = QStringList() );
-
+    void addLayers( QStringList const &layers, QStringList const &styles = QStringList() );
 
     /** return the number of layers for the current data source
 
@@ -442,27 +459,40 @@ class QgsWmsProvider : public QgsRasterDataProvider
      *  \warning A pointer to an QImage is used, as a plain QImage seems to have difficulty being
      *           shared across library boundaries
      */
-    QImage * draw( QgsRectangle const &  viewExtent, int pixelWidth, int pixelHeight );
+    QImage *draw( QgsRectangle const &  viewExtent, int pixelWidth, int pixelHeight );
 
+    void readBlock( int bandNo, QgsRectangle  const & viewExtent, int width, int height, void *data );
+    //void readBlock( int bandNo, QgsRectangle  const & viewExtent, int width, int height, QgsCoordinateReferenceSystem theSrcCRS, QgsCoordinateReferenceSystem theDestCRS, void *data );
 
-//  /** Experimental function only **/
-//  void getServerCapabilities();
-
-    /* Example URI: http://ims.cr.usgs.gov:80/servlet/com.esri.wms.Esrimap/USGS_EDC_Trans_BTS_Roads?SERVICE=WMS&REQUEST=GetCapabilities */
 
     /** Return the extent for this data layer
     */
     virtual QgsRectangle extent();
 
-    /** Reset the layer - for a PostgreSQL layer, this means clearing the PQresult
-     * pointer and setting it to 0
-     */
-    void begin();
-
     /**Returns true if layer is valid
-    */
+     */
     bool isValid();
 
+    /**Returns true if layer has tile set profiles
+     * @added in 1.5
+     */
+    virtual bool hasTiles() const;
+
+    /**Returns the base url
+     * @added in 1.5
+     */
+    virtual QString baseUrl() const;
+
+    /**Returns the GetMap url
+     * @added in 1.5
+     */
+    virtual QString getMapUrl() const;
+
+    /**Returns the GetFeatureInfo url
+     * @added in 1.5
+     */
+    virtual QString getFeatureInfoUrl() const;
+
     //! get WMS Server version string
     QString wmsVersion();
 
@@ -500,6 +530,11 @@ class QgsWmsProvider : public QgsRasterDataProvider
       */
     int capabilities() const;
 
+    int dataType( int bandNo ) const;
+    int srcDataType( int bandNo ) const;
+    int bandCount() const;
+
+
     /**
      * Get metadata in a format suitable for feeding directly
      * into a subset of the GUI raster properties "Metadata" tab.
@@ -507,6 +542,21 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString metadata();
 
 
+    /**
+     * \brief Identify details from a WMS Server from the last screen update
+     *
+     * \param point[in]  The pixel coordinate (as it was displayed locally on screen)
+     *
+     * \return  A html document containing the return from the WMS server
+     *
+     * \note WMS Servers prefer to receive coordinates in image space, therefore
+     *       this function expects coordinates in that format.
+     *
+     * \note  The arbitraryness of the returned document is enforced by WMS standards
+     *        up to at least v1.3.0
+     */
+    QString identifyAsHtml( const QgsPoint& point );
+
     /**
      * \brief Identify details from a WMS Server from the last screen update
      *
@@ -540,9 +590,13 @@ class QgsWmsProvider : public QgsRasterDataProvider
      * Interactive users of this provider can then, for example,
      * call a QMessageBox to display the contents.
      */
-
     QString lastError();
 
+    /**
+     * \brief   Returns the format of the error message (text or html)
+     */
+    QString lastErrorFormat();
+
     /** return a provider name
 
     Essentially just returns the provider key.  Should be used to build file
@@ -573,6 +627,10 @@ class QgsWmsProvider : public QgsRasterDataProvider
     */
     QString description() const;
 
+    /**Reloads the data from the the source. Needs to be implemented by providers with data caches to
+      synchronize with changes in the data source*/
+    virtual void reloadData();
+
 
   signals:
 
@@ -582,133 +640,135 @@ class QgsWmsProvider : public QgsRasterDataProvider
     /** \brief emit a signal to be caught by qgisapp and display a msg on status bar */
     void statusChanged( QString const &  theStatusQString );
 
+    void dataChanged();
 
-  public slots:
-
-    void showStatusMessage( QString const &  theMessage );
-
+  private slots:
+    void cacheReplyFinished();
+    void cacheReplyProgress( qint64, qint64 );
+    void capabilitiesReplyFinished();
+    void capabilitiesReplyProgress( qint64, qint64 );
+    void identifyReplyFinished();
+    void tileReplyFinished();
 
   private:
+    void showMessageBox( const QString& title, const QString& text );
 
     /**
      * \brief Retrieve and parse the (cached) Capabilities document from the server
      *
      * \param forceRefresh  if true, ignores any previous response cached in memory
      *                      and always contact the server for a new copy.
-     * \retval FALSE if the capabilities document could not be retrieved or parsed -
+     * \retval false if the capabilities document could not be retrieved or parsed -
      *         see lastError() for more info
      *
      * When this returns, "layers" will make sense.
      *
      * TODO: Make network-timeout tolerant
      */
-    bool retrieveServerCapabilities( bool forceRefresh = FALSE );
+    bool retrieveServerCapabilities( bool forceRefresh = false );
 
-    /**
-     * \brief Common URL retreival code for the differing WMS request types
-     *
-     * \retval 0 if an error occured - use lastError() and lastErrorTitle() for details
-     *
-     */
-    QByteArray retrieveUrl( QString url );
-
-    /*
-      //! Test function: see if we can download a WMS' capabilities
-      //! \retval FALSE if the download failed in some way
-      bool downloadCapabilitiesURI(QString const &  uri);
-    */
-
-    //! \return FALSE if the capabilities document could not be parsed - see lastError() for more info
-    bool parseCapabilitiesDom( QByteArray const & xml, QgsWmsCapabilitiesProperty& capabilitiesProperty );
+    //! \return false if the capabilities document could not be parsed - see lastError() for more info
+    bool parseCapabilitiesDom( QByteArray const &xml, QgsWmsCapabilitiesProperty &capabilitiesProperty );
 
     //! parse the WMS Service XML element
-    void parseService( QDomElement const & e, QgsWmsServiceProperty& serviceProperty );
+    void parseService( QDomElement const &e, QgsWmsServiceProperty &serviceProperty );
 
     //! parse the WMS Capability XML element
-    void parseCapability( QDomElement const & e, QgsWmsCapabilityProperty& capabilityProperty );
+    void parseCapability( QDomElement const &e, QgsWmsCapabilityProperty &capabilityProperty );
 
     //! parse the WMS ContactPersonPrimary XML element
-    void parseContactPersonPrimary( QDomElement const & e, QgsWmsContactPersonPrimaryProperty&
-                                    contactPersonPrimaryProperty );
+    void parseContactPersonPrimary( QDomElement const &e, QgsWmsContactPersonPrimaryProperty &contactPersonPrimaryProperty );
 
     //! parse the WMS ContactAddress XML element
-    void parseContactAddress( QDomElement const & e, QgsWmsContactAddressProperty& contactAddressProperty );
+    void parseContactAddress( QDomElement const &e, QgsWmsContactAddressProperty &contactAddressProperty );
 
     //! parse the WMS ContactInformation XML element
-    void parseContactInformation( QDomElement const & e, QgsWmsContactInformationProperty&
-                                  contactInformationProperty );
+    void parseContactInformation( QDomElement const &e, QgsWmsContactInformationProperty &contactInformationProperty );
 
     //! parse the WMS OnlineResource XML element
-    void parseOnlineResource( QDomElement const & e, QgsWmsOnlineResourceAttribute& onlineResourceAttribute );
+    void parseOnlineResource( QDomElement const &e, QgsWmsOnlineResourceAttribute &onlineResourceAttribute );
 
     //! parse the WMS KeywordList XML element
-    void parseKeywordList( QDomElement const & e, QStringList& keywordListProperty );
+    void parseKeywordList( QDomElement const &e, QStringList &keywordListProperty );
 
     //! parse the WMS Get XML element
-    void parseGet( QDomElement const & e, QgsWmsGetProperty& getProperty );
+    void parseGet( QDomElement const &e, QgsWmsGetProperty &getProperty );
 
     //! parse the WMS Post XML element
-    void parsePost( QDomElement const & e, QgsWmsPostProperty& postProperty );
+    void parsePost( QDomElement const &e, QgsWmsPostProperty &postProperty );
 
     //! parse the WMS HTTP XML element
-    void parseHttp( QDomElement const & e, QgsWmsHttpProperty& httpProperty );
+    void parseHttp( QDomElement const &e, QgsWmsHttpProperty &httpProperty );
 
     //! parse the WMS DCPType XML element
-    void parseDcpType( QDomElement const & e, QgsWmsDcpTypeProperty& dcpType );
+    void parseDcpType( QDomElement const &e, QgsWmsDcpTypeProperty &dcpType );
 
     //! parse the WMS GetCapabilities, GetMap, or GetFeatureInfo XML element, each of type "OperationType".
     void parseOperationType( QDomElement const & e, QgsWmsOperationType& operationType );
 
     //! parse the WMS Request XML element
-    void parseRequest( QDomElement const & e, QgsWmsRequestProperty& requestProperty );
+    void parseRequest( QDomElement const &e, QgsWmsRequestProperty &requestProperty );
 
     //! parse the WMS Legend URL XML element
-    void parseLegendUrl( QDomElement const & e, QgsWmsLegendUrlProperty& legendUrlProperty );
+    void parseLegendUrl( QDomElement const &e, QgsWmsLegendUrlProperty &legendUrlProperty );
 
     //! parse the WMS Style XML element
-    void parseStyle( QDomElement const & e, QgsWmsStyleProperty& styleProperty );
+    void parseStyle( QDomElement const &e, QgsWmsStyleProperty &styleProperty );
 
     //! parse the WMS Layer XML element
     // TODO: Make recursable
-    void parseLayer( QDomElement const & e, QgsWmsLayerProperty& layerProperty,
+    void parseLayer( QDomElement const &e, QgsWmsLayerProperty &layerProperty,
                      QgsWmsLayerProperty *parentProperty = 0 );
 
+    //! extract tile information from VendorSpecificCapabilities
+    void parseTileSetProfile( QDomElement const &e, QVector<QgsWmsTileSetProfile> &tileSetProfiles );
+
     /**
      * \brief parse the full WMS ServiceExceptionReport XML document
      *
      * \note mErrorCaption and mError are updated to suit the results of this function.
      */
-    bool parseServiceExceptionReportDom( QByteArray const & xml );
+    bool parseServiceExceptionReportDom( QByteArray const &xml );
 
     //! parse the WMS ServiceException XML element
-    void parseServiceException( QDomElement const & e );
+    void parseServiceException( QDomElement const &e );
 
 
     /**
      * \brief Calculates the combined extent of the layers selected by layersDrawn
      *
-     * \retval FALSE if the capabilities document could not be retrieved or parsed -
+     * \retval false if the capabilities document could not be retrieved or parsed -
      *         see lastError() for more info
      */
     bool calculateExtent();
 
     /**
-     * \brief Check for authentication information contained in the uri,
-     * stripping and saving the username and password if present.
+     * \brief Check for parameters in the uri,
+     * stripping and saving them if present.
      *
      * \param uri uri to check
      *
      * \note added in 1.1
      */
 
-    void setAuthentication( QString uri );
+    void parseUri( QString uri );
 
     /**
      * \brief Prepare the URI so that we can later simply append param=value
      * \param uri uri to prepare
      * \retval prepared uri
      */
-    QString prepareUri( QString uri );
+    QString prepareUri( QString uri ) const;
+
+    QStringList identifyAs( const QgsPoint &point, QString format );
+
+    QString layerMetadata( QgsWmsLayerProperty &layer );
+
+    //! remove query item and replace it with a new value
+    void setQueryItem( QUrl &url, QString key, QString value );
+
+    //! set authorization header
+    void setAuthorization( QNetworkRequest &request ) const;
 
     //! Data source URI of the WMS for this layer
     QString httpuri;
@@ -717,7 +777,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     QString connectionName;
 
     //! URL part of URI (httpuri)
-    QString baseUrl;
+    QString mBaseUrl;
 
     /**
      * Flag indicating if the layer data source is a valid WMS layer
@@ -759,6 +819,11 @@ class QgsWmsProvider : public QgsRasterDataProvider
      */
     QVector<QgsWmsLayerProperty> layersSupported;
 
+    /**
+     * tilesets hosted by the WMS Server
+     */
+    QVector<QgsWmsTileSetProfile> tilesetsSupported;
+
     /**
      * extents per layer (in WMS CRS:84 datum)
      */
@@ -767,7 +832,7 @@ class QgsWmsProvider : public QgsRasterDataProvider
     /**
      * available CRSs per layer
      */
-    QMap<QString, QVector<QString> > crsForLayer;
+    QMap<QString, QStringList > crsForLayer;
 
     /**
      * WMS "queryable" per layer
@@ -780,7 +845,6 @@ class QgsWmsProvider : public QgsRasterDataProvider
      * (some may not be visible in a draw function, cf. activeSubLayerVisibility)
      */
     QStringList activeSubLayers;
-
     QStringList activeSubStyles;
 
     /**
@@ -803,22 +867,39 @@ class QgsWmsProvider : public QgsRasterDataProvider
      * This can be reused if draw() is called consecutively
      * with the same parameters.
      */
-    QImage* cachedImage;
+    QImage *cachedImage;
 
     /**
-     * The previous parameter to draw().
+     * The reply to the on going request to fill the cache
+     */
+    QNetworkReply *cacheReply;
+
+    /**
+     * Running tile requests
+     */
+    QList<QNetworkReply*> tileReplies;
+
+    /**
+     * The reply to the capabilities request
+     */
+    QNetworkReply *mCapabilitiesReply;
+
+    /**
+     * The reply to the capabilities request
+     */
+    QNetworkReply *mIdentifyReply;
+
+    /**
+     * The result of the identify reply
+     */
+    QString mIdentifyResult;
+
+    /**
+     * The previous parameters to draw().
      */
     QgsRectangle cachedViewExtent;
-
-    /**
-     * The previous parameter to draw().
-     */
-    int cachedPixelWidth;
-
-    /**
-     * The previous parameter to draw().
-     */
-    int cachedPixelHeight;
+    int cachedViewWidth;
+    int cachedViewHeight;
 
     /**
      * The error caption associated with the last WMS error.
@@ -830,8 +911,13 @@ class QgsWmsProvider : public QgsRasterDataProvider
      */
     QString mError;
 
+
+    /** The mime type of the message
+     */
+    QString mErrorFormat;
+
     //! A QgsCoordinateTransform is used for transformation of WMS layer extents
-    QgsCoordinateTransform * mCoordinateTransform;
+    QgsCoordinateTransform *mCoordinateTransform;
 
     //! See if calculateExtents() needs to be called before extent() returns useful data
     bool extentDirty;
@@ -839,16 +925,39 @@ class QgsWmsProvider : public QgsRasterDataProvider
     //! Base URL for WMS GetFeatureInfo requests
     QString mGetFeatureInfoUrlBase;
 
+    //! number of layers and parents
     int mLayerCount;
     QMap<int, int> mLayerParents;
     QMap<int, QStringList> mLayerParentNames;
 
+    //! flag set while provider is fetching tiles synchronously
+    bool mWaiting;
+
+    //! tile request number, cache hits and misses
+    int mTileReqNo;
+    int mCacheHits;
+    int mCacheMisses;
+    int mErrors;
+
     //! Username for basic http authentication
     QString mUserName;
 
     //! Password for basic http authentication
     QString mPassword;
 
+    //! layer is tiled, tile size and available resolutions
+    bool mTiled;
+    int mTileWidth;
+    int mTileHeight;
+    QVector<double> mResolutions;
+
+    //! whether to use hrefs from GetCapabilities (default) or
+    // the given base urls for GetMap and GetFeatureInfo
+    bool mIgnoreGetMapUrl;
+    bool mIgnoreGetFeatureInfoUrl;
+
+    //! supported formats for GetFeatureInfo in order of preference
+    QStringList mSupportedGetFeatureFormats;
 };
 
 #endif
