@@ -50,23 +50,16 @@ class PYTHON_EXPORT QgsPythonUtils
 
     /* console */
 
+    //! run a statement, show an error message on error
+    //! @return true if no error occured
+    virtual bool runString( const QString& command, QString msgOnError = QString() ) = 0;
+
     //! run a statement, error reporting is not done
     //! @return true if no error occured
     virtual bool runStringUnsafe( const QString& command, bool single = true ) = 0;
 
     virtual bool evalString( const QString& command, QString& result ) = 0;
 
-    //! change displayhook and excepthook
-    //! our hooks will just save the result to special variables
-    //! and those can be used in the program
-    virtual void installConsoleHooks() = 0;
-
-    //! get back to the original settings (i.e. write output to stdout)
-    virtual void uninstallConsoleHooks() = 0;
-
-    //! get result from the last statement as a string
-    virtual QString getResult() = 0;
-
     //! get information about error to the supplied arguments
     //! @return false if there was no python error
     virtual bool getError( QString& errorClassName, QString& errorText ) = 0;
@@ -92,6 +85,9 @@ class PYTHON_EXPORT QgsPythonUtils
     //! @param function one of these strings: name, tpye, version, description
     virtual QString getPluginMetadata( QString pluginName, QString function ) = 0;
 
+    //! confirm that the plugin can be uninstalled
+    virtual bool canUninstallPlugin( QString packageName ) = 0;
+
     //! unload plugin
     virtual bool unloadPlugin( QString packageName ) = 0;
 };
