@@ -23,6 +23,7 @@
 #endif
 #include <Python.h>
 
+#include "qgis.h"
 #include "qgspythonutilsimpl.h"
 
 #include "qgsapplication.h"
@@ -56,10 +57,20 @@ void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
   mMainDict = PyModule_GetDict( mMainModule ); // borrowed reference
 
   runString( "import sys" ); // import sys module (for display / exception hooks)
+  runString( "import os" ); // import os module (for user paths)
+
+  // construct a list of plugin paths
+  // plugin dirs passed in QGIS_PLUGINPATH env. variable have highest priority (usually empty)
+  // locally installed plugins have priority over the system plugins
+  QStringList pluginpaths = extraPluginsPaths();
+  pluginpaths << homePluginsPath() << pluginsPath();
 
   // expect that bindings are installed locally, so add the path to modules
   // also add path to plugins
-  runString( "sys.path = [\"" + pythonPath() + "\", \"" + homePythonPath()  + "\", \"" + homePluginsPath()  + "\", \"" + pluginsPath() + "\"] + sys.path" );
+  QStringList newpaths;
+  newpaths << pythonPath() << homePythonPath();
+  newpaths += pluginpaths;
+  runString( "sys.path = [\"" + newpaths.join( "\", \"" ) + "\"] + sys.path" );
 
   // import SIP
   if ( !runString( "from sip import wrapinstance, unwrapinstance",
@@ -93,6 +104,9 @@ void QgsPythonUtilsImpl::initPython( QgisInterface* interface )
     return;
   }
 
+  // tell the utils script where to look for the plugins
+  runString( "qgis.utils.plugin_paths = [\"" + pluginpaths.join( "\",\"" ) + "\"]" );
+
   // initialize 'iface' object
   runString( "qgis.utils.initInterface(" + QString::number(( unsigned long ) interface ) + ")" );
 }
@@ -121,24 +135,25 @@ void QgsPythonUtilsImpl::uninstallErrorHook()
   runString( "qgis.utils.uninstallErrorHook()" );
 }
 
-void QgsPythonUtilsImpl::installConsoleHooks()
-{
-  runString( "qgis.utils.installConsoleHooks()" );
-}
-
-void QgsPythonUtilsImpl::uninstallConsoleHooks()
-{
-  runString( "qgis.utils.uninstallConsoleHooks()" );
-}
 
 
 bool QgsPythonUtilsImpl::runStringUnsafe( const QString& command, bool single )
 {
+  // acquire global interpreter lock to ensure we are in a consistent state
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
+
   // TODO: convert special characters from unicode strings u"..." to \uXXXX
   // so that they're not mangled to utf-8
   // (non-unicode strings can be mangled)
   PyRun_String( command.toUtf8().data(), single ? Py_single_input : Py_file_input, mMainDict, mMainDict );
-  return ( PyErr_Occurred() == 0 );
+
+  bool res = ( PyErr_Occurred() == 0 );
+
+  // we are done calling python API, release global interpreter lock
+  PyGILState_Release( gstate );
+
+  return res;
 }
 
 bool QgsPythonUtilsImpl::runString( const QString& command, QString msgOnError )
@@ -160,9 +175,10 @@ bool QgsPythonUtilsImpl::runString( const QString& command, QString msgOnError )
   evalString( "str(sys.path)", path );
   evalString( "sys.version", version );
 
-  QString str = "<font color=\"red\">" + msgOnError + "</font><br><br>" + traceback + "<br>" +
-                QObject::tr( "Python version:" ) + "<br>" + version + "<br><br>" +
-                QObject::tr( "Python path:" ) + "<br>" + path;
+  QString str = "<font color=\"red\">" + msgOnError + "</font><br><pre>\n" + traceback + "\n</pre>"
+                + QObject::tr( "Python version:" ) + "<br>" + version + "<br><br>"
+                + QObject::tr( "QGIS version:" ) + "<br>" + QString( "%1 '%2', %3" ).arg( QGis::QGIS_VERSION ).arg( QGis::QGIS_RELEASE_NAME ).arg( QGis::QGIS_DEV_VERSION ) + "<br><br>"
+                + QObject::tr( "Python path:" ) + "<br>" + path;
   str.replace( "\n", "<br>" ).replace( "  ", "&nbsp; " );
 
   QgsMessageOutput* msg = QgsMessageOutput::createMessageOutput();
@@ -178,6 +194,9 @@ QString QgsPythonUtilsImpl::getTraceback()
 {
 #define TRACEBACK_FETCH_ERROR(what) {errMsg = what; goto done;}
 
+  // acquire global interpreter lock to ensure we are in a consistent state
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
 
   QString errMsg;
   QString result;
@@ -243,6 +262,9 @@ done:
   Py_XDECREF( traceback );
   Py_XDECREF( type );
 
+  // we are done calling python API, release global interpreter lock
+  PyGILState_Release( gstate );
+
   return result;
 }
 
@@ -270,8 +292,15 @@ QString QgsPythonUtilsImpl::getTypeAsString( PyObject* obj )
 
 bool QgsPythonUtilsImpl::getError( QString& errorClassName, QString& errorText )
 {
+  // acquire global interpreter lock to ensure we are in a consistent state
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
+
   if ( !PyErr_Occurred() )
+  {
+    PyGILState_Release( gstate );
     return false;
+  }
 
   PyObject* err_type;
   PyObject* err_value;
@@ -296,17 +325,12 @@ bool QgsPythonUtilsImpl::getError( QString& errorClassName, QString& errorText )
   Py_XDECREF( err_value );
   Py_XDECREF( err_tb );
 
+  // we are done calling python API, release global interpreter lock
+  PyGILState_Release( gstate );
+
   return true;
 }
 
-QString QgsPythonUtilsImpl::getResult()
-{
-  QString res;
-  evalString( "qgis.utils.console_output", res );
-  // clear output
-  runString( "qgis.utils.console_output = None" );
-  return res;
-}
 
 QString QgsPythonUtilsImpl::PyObjectToQString( PyObject* obj )
 {
@@ -368,37 +392,25 @@ QString QgsPythonUtilsImpl::PyObjectToQString( PyObject* obj )
   return "(qgis error)";
 }
 
-QString QgsPythonUtilsImpl::getVariableFromMain( QString name )
-{
-  PyObject* obj;
-  QString output;
-
-  // get the result
-  obj = PyDict_GetItemString( mMainDict, name.toUtf8() ); // obj is borrowed reference
-
-  if ( obj != NULL && obj != Py_None )
-  {
-    output = PyObjectToQString( obj );
-  }
-
-  // erase result
-  PyDict_SetItemString( mMainDict, name.toUtf8(), Py_None );
-
-  return output;
-}
 
 bool QgsPythonUtilsImpl::evalString( const QString& command, QString& result )
 {
+  // acquire global interpreter lock to ensure we are in a consistent state
+  PyGILState_STATE gstate;
+  gstate = PyGILState_Ensure();
+
   PyObject* res = PyRun_String( command.toUtf8().data(), Py_eval_input, mMainDict, mMainDict );
+  bool success = ( res != NULL );
 
   // TODO: error handling
 
-  if ( res != NULL )
-  {
+  if ( success )
     result = PyObjectToQString( res );
-    return true;
-  }
-  return false;
+
+  // we are done calling python API, release global interpreter lock
+  PyGILState_Release( gstate );
+
+  return success;
 }
 
 
@@ -414,7 +426,7 @@ QString QgsPythonUtilsImpl::pluginsPath()
 
 QString QgsPythonUtilsImpl::homePythonPath()
 {
-  return QgsApplication::qgisSettingsDirPath() + "/python";
+  return QgsApplication::qgisSettingsDirPath() + "python";
 }
 
 QString QgsPythonUtilsImpl::homePluginsPath()
@@ -422,25 +434,31 @@ QString QgsPythonUtilsImpl::homePluginsPath()
   return homePythonPath() + "/plugins";
 }
 
+QStringList QgsPythonUtilsImpl::extraPluginsPaths()
+{
+  const char* cpaths = getenv( "QGIS_PLUGINPATH" );
+  if ( cpaths == NULL )
+    return QStringList();
+
+  QString paths = QString::fromLocal8Bit( cpaths );
+#ifndef Q_OS_WIN
+  if ( paths.contains( ':' ) )
+    return paths.split( ':', QString::SkipEmptyParts );
+#endif
+  if ( paths.contains( ';' ) )
+    return paths.split( ';', QString::SkipEmptyParts );
+  else
+    return QStringList( paths );
+}
+
+
 QStringList QgsPythonUtilsImpl::pluginList()
 {
-  QDir pluginDir( QgsPythonUtilsImpl::pluginsPath(), "*",
-                  QDir::Name | QDir::IgnoreCase, QDir::Dirs | QDir::NoDotAndDotDot );
+  runString( "qgis.utils.updateAvailablePlugins()" );
 
-  QDir homePluginDir( QgsPythonUtilsImpl::homePluginsPath(), "*",
-                      QDir::Name | QDir::IgnoreCase, QDir::Dirs | QDir::NoDotAndDotDot );
-
-  QStringList pluginList = pluginDir.entryList();
-
-  for ( uint i = 0; i < homePluginDir.count(); i++ )
-  {
-    QString packageName = homePluginDir[i];
-    if ( !pluginList.contains( packageName ) )
-      pluginList.append( packageName );
-
-  }
-
-  return pluginList;
+  QString output;
+  evalString( "'\\n'.join(qgis.utils.available_plugins)", output );
+  return output.split( QChar( '\n' ), QString::SkipEmptyParts );
 }
 
 QString QgsPythonUtilsImpl::getPluginMetadata( QString pluginName, QString function )
@@ -452,7 +470,6 @@ QString QgsPythonUtilsImpl::getPluginMetadata( QString pluginName, QString funct
   return res;
 }
 
-
 bool QgsPythonUtilsImpl::loadPlugin( QString packageName )
 {
   QString output;
@@ -460,7 +477,6 @@ bool QgsPythonUtilsImpl::loadPlugin( QString packageName )
   return ( output == "True" );
 }
 
-
 bool QgsPythonUtilsImpl::startPlugin( QString packageName )
 {
   QString output;
@@ -468,6 +484,12 @@ bool QgsPythonUtilsImpl::startPlugin( QString packageName )
   return ( output == "True" );
 }
 
+bool QgsPythonUtilsImpl::canUninstallPlugin( QString packageName )
+{
+  QString output;
+  evalString( "qgis.utils.canUninstallPlugin('" + packageName + "')", output );
+  return ( output == "True" );
+}
 
 bool QgsPythonUtilsImpl::unloadPlugin( QString packageName )
 {
@@ -487,5 +509,5 @@ QStringList QgsPythonUtilsImpl::listActivePlugins()
 {
   QString output;
   evalString( "'\\n'.join(qgis.utils.active_plugins)", output );
-  return output.split( QChar( '\n' ) );
+  return output.split( QChar( '\n' ), QString::SkipEmptyParts );
 }
